var PurchasesManager =
{
    ready: false,
    products: [],
    
    readyCallback: null,
    purchaseCompleteCallback: null,
    errorCallback: null,
    refreshCallback: null,
    isAndroid: false,

    cache: []
};

if(navigator.userAgent.toLowerCase().indexOf('android') >= 0)
{
    PurchasesManager.isAndroid = true;

    PurchasesManager.init = function(readyCallback, purchaseCompleteCallback)
    {
        if(!window.cordova || !window.inAppPurchase) return;

        PurchasesManager.ready = true;
        PurchasesManager.purchaseCompleteCallback = purchaseCompleteCallback;

        if (Session.get('inappbilling_init')) return;
        Session.set('inappbilling_init', true);

        console.log('Billing: ready');

        PurchasesManager.readyCallback = readyCallback;

        if(readyCallback) readyCallback();
    };

    PurchasesManager.onError = function(err)
    {
        if(PurchasesManager.errorCallback) PurchasesManager.errorCallback();

        console.log("Billing error: " + JSON.stringify(err));
    };
    
    PurchasesManager.refresh = function(productIds, callback)
    {
        if(!PurchasesManager.ready) return console.log('Billing: not ready...');;
        console.log('Billing: refresh = ' + productIds.join(","));

        inAppPurchase
          .getProducts(productIds)
          .then(function (products) {
            PurchasesManager.onRefresh(products, callback);
          })
          .catch(PurchasesManager.onError);
    };
    
    PurchasesManager.onRefresh = function(products, callback)
    {
    	console.log("Billing refresh result: " + JSON.stringify(products));

        PurchasesManager.products = products || [];

        if(callback) callback(PurchasesManager.products);
    };

    PurchasesManager.prepareItems = function(products)
    {
    	var ret = [];
    	
    	for(var i=0; i<products.length; i++)
    	{
    		if(products[i].productId && products[i].price)
    		{
	    		ret.push({"id": products[i].productId,
	    				  "title": products[i].title,
	    				  "price": products[i].price // + products[i].price_currency_code
	    				 });
    		}
    	}
    	
    	return ret;
	};

	PurchasesManager.purchase = function(productId, payload)
    {
        if(!PurchasesManager.ready) return;
        payload = (payload+'') || '';

        console.log('Billing buy: ' + productId + ' (payload: '+payload+')');

        inAppPurchase
          .buy(productId)
          .then(function (data) {
            PurchasesManager.onPurchase(data);
          })
          .catch(PurchasesManager.onError);
    };
   
    PurchasesManager.onPurchase = function(data)
    {
        console.log("Billing buy response: " + JSON.stringify(data));

        if(PurchasesManager.purchaseCompleteCallback)
        {
            PurchasesManager.purchaseCompleteCallback(data);
        }
    };
    
    PurchasesManager.sendToServer = function(receiver, server, productId, receipt, okCallback, errCallback, transactionId)
    {
        var rcp = {
            purchaseTime: (new Date()).getTime(),
            purchaseToken: receipt
        };

        var data = {
        	"orderId": null,
        	"productId": productId,
        	"receipt": JSON.stringify(rcp)
        };
        return PurchasesManager.sendToServerFull(receiver, server, data, okCallback, errCallback);
    };
    
    PurchasesManager.sendToServerFull = function(receiver, server, data, okCallback, errCallback)
    {
        var orderId = data.orderId || null;
        var receipt = JSON.parse(data.receipt);
        var purchaseTime = (receipt.purchaseTime || (new Date()).getTime()) + "";
        var purchaseToken = receipt.purchaseToken;

        var params = [receiver, server, data.productId, orderId, purchaseTime, purchaseToken];

        console.log("SEND TO SERVER");
        console.log(JSON.stringify(params));

        PortalService.account.callMethod("onGooglePaymentFull", params, function(){

            inAppPurchase.consume(data.type, data.receipt, data.signature);

            if (typeof okCallback == 'function')
            {
                okCallback(data);
            }

        }, errCallback);
    };
    
    PurchasesManager.onUserLogin = function(username, server)
    {
		console.log("RESTORE PURCHASES");
		inAppPurchase
            .restorePurchases()
            .then(function (purchases) {
                for (var i = 0; i < purchases.length; i++) {
                    var data = purchases[i];
                    var receipt = JSON.parse(data.receipt);
                    if(receipt && receipt.purchaseState == 0) {
                        console.log("TRY TO CONSUME");
                        console.log(JSON.stringify(data));
                        PurchasesManager.sendToServerFull(username, server, data,
                            function(rsp){ console.log("Billing ITEM CONSUMED " + JSON.stringify(rsp)); },
                            function(err){ console.log("Billing ERROR CONSUMING ITEM " + JSON.stringify(err)); }
                        );
                    }
                }
            })
            .catch(function (err) {
                console.log("RESTORE ERROR");
                console.log(err);
            });

		/*
		server = server || null;
		inappbilling.getPurchases(function(data){
			
			console.log('Billing owned items: ' + JSON.stringify(data));
			
			for (var i = 0; i < data.length; i++)
			{
				if (data[i].purchaseState == 0)
				{
					PurchasesManager.sendToServerFull(username, server, data[i], 
						function(rsp){ console.log("Billing ITEM CONSUMED " + JSON.stringify(rsp)); }, 
						function(err){ console.log("Billing ERROR CONSUMING ITEM " + JSON.stringify(err)); }
					);
				}
			}
			
		}, PurchasesManager.onError);
		*/
    };
    
    PurchasesManager.onRestore = function(originalTransactionId, productId)
    {
    };
    
    PurchasesManager.onRestoreFailed = function(errCode)
    {
    };
    
    PurchasesManager.onRestoreCompleted = function()
    {
    };
    
}
else
{
    PurchasesManager.cache = Storage.get("applePurchases") || [];

    PurchasesManager.init = function(readyCallback, purchaseCompleteCallback)
    {
        if(!window.cordova || !window.storekit) return;
        
        PurchasesManager.readyCallback = readyCallback;
        PurchasesManager.purchaseCompleteCallback = purchaseCompleteCallback;
        
        storekit.init({
            debug: false,
            ready:    PurchasesManager.onReady,
            error:    PurchasesManager.onError,
            purchase: PurchasesManager.onPurchase,
            restore:  PurchasesManager.onRestore,
            restoreFailed:    PurchasesManager.onRestoreFailed,
            restoreCompleted: PurchasesManager.onRestoreCompleted
        });
    };
    
    PurchasesManager.refresh = function(productIds, callback)
    {
        if(!PurchasesManager.ready) return;
        
        storekit.load(productIds, function(validProducts, invalidProductIds)
        {
            PurchasesManager.products = [];
            var val;
            for(var i in validProducts)
            {
                PurchasesManager.products.push(validProducts[i]);
            }
                      
            callback(PurchasesManager.products);
        });
    };
    
    PurchasesManager.purchase = function(productId, payload)
    {
        if(!PurchasesManager.ready) return;
        payload = (payload+'') || ''; // not used for apple
        
        storekit.purchase(productId);
    };
    
    PurchasesManager.prepareItems = function(products)
    {
    	var ret = [];
    	
    	for(var i=0; i<products.length; i++)
    	{
    		if(products[i].id && products[i].price)
    		{
	    		ret.push({"id": products[i].id,
	    				  "title": products[i].description,
	    				  "price": products[i].price
	    				 });
    		}
    	}
    	
    	return ret;
	};
   
	PurchasesManager.onReady = function()
    {
        PurchasesManager.ready = true;
        if(PurchasesManager.readyCallback) PurchasesManager.readyCallback();
    };
    
    PurchasesManager.onError = function(errorCode, errorText)
    {
        if(window.Alert) Alert.create(errorCode + ": " + errorText);
        if(PurchasesManager.errorCallback) PurchasesManager.errorCallback(errorCode, errorText);
    };
    
    PurchasesManager.onPurchase = function(transactionId, productId, receipt)
    {
        PurchasesManager.putToCache(transactionId, productId, receipt);

        if(PurchasesManager.purchaseCompleteCallback)
        {
        	PurchasesManager.purchaseCompleteCallback(transactionId, productId, receipt);
        }
    };
    
    PurchasesManager.sendToServer = function(receiver, server, productId, receipt, okCallback, errCallback, transactionId)
    {
        PortalService.account.callMethod("onApplePayment", [receiver, server, receipt], function()
        {
            PurchasesManager.removeFromCache(transactionId);
            if(okCallback) okCallback();
        }, errCallback);
    };
    
    PurchasesManager.sendToServerFull = function(receiver, server, data, okCallback, errCallback)
    {
		return PurchasesManager.sendToServer(receiver, server, data.productId, data.receipt, okCallback, errCallback, data.transactionId);
    };

    PurchasesManager.getCacheIx = function(transactionId)
    {
        try
        {
            for(var i=0; i<PurchasesManager.cache.length; i++)
            {
                if(PurchasesManager.cache[i].transactionId == transactionId) return i;
            }
        }
        catch(e){}

        return -1;
    };

    PurchasesManager.putToCache = function(transactionId, productId, receipt)
    {
        try
        {
            if(PurchasesManager.getCacheIx(transactionId) >= 0) return;

            var data = {
                transactionId: transactionId,
                productId: productId,
                receipt: receipt
            };

            PurchasesManager.cache.push(data);
            Storage.set("applePurchases", PurchasesManager.cache);
        }
        catch(e){}
    };

    PurchasesManager.removeFromCache = function(transactionId)
    {
        try
        {
            var ix = PurchasesManager.getCacheIx(transactionId);
            if(ix >= 0)
            {
                PurchasesManager.cache.splice(ix, 1);
                Storage.set("applePurchases", PurchasesManager.cache);
            }
        }
        catch(e){}
    };
    
    PurchasesManager.onUserLogin = function(username, server)
    {
    	server = server || null;
    	// process failed transactions here...

        try
        {
            var cache = PurchasesManager.cache.slice();

            if(cache.length)
            {
                for(var i=0; i<cache.length; i++)
                {
                    var data = cache[i];
                    PurchasesManager.sendToServer(username, server, data.productId, data.receipt, null, null, data.transactionId);
                }
            }
        }
        catch(e){}
    };
    
    PurchasesManager.onRestore = function(originalTransactionId, productId)
    {
    };
    
    PurchasesManager.onRestoreFailed = function(errCode)
    {
    };
    
    PurchasesManager.onRestoreCompleted = function()
    {
    };
}