function ComboBox()
{
	callParentConstructor(ComboBox.prototype, this);
	
	this.showOptions = $.proxy(this.showOptions, this);
	this.hideOptions = $.proxy(this.hideOptions, this);
	this.optionChanged = $.proxy(this.optionChanged, this);
	
	this.__defineGetter__('dataProvider', this.getDataProvider);
	this.__defineSetter__('dataProvider', this.setDataProvider);
	
	this.__defineGetter__('selectedIndex', this.getSelectedIndex);
	this.__defineSetter__('selectedIndex', this.setSelectedIndex);
	
	this.__defineGetter__('selectedOption', this.getSelectedOption);
	this.__defineSetter__('selectedOption', this.setSelectedOption);
	
	this.__defineGetter__('selectedItem', this.getSelectedOption);
	this.__defineSetter__('selectedItem', this.setSelectedOption);
	
	this.__defineGetter__('visibleItems', this.getVisibleItems);
	this.__defineSetter__('visibleItems', this.setVisibleItems);
	
	this._defaultLabel = I18.f('SELECT');
	this.__defineGetter__('placeholder', this.getLabel);
	this.__defineSetter__('placeholder', this.setLabel);
}

extendClass(ComboBox.prototype, Input.prototype);

ComboBox.prototype._defaultLabel = I18.f('SELECT');

ComboBox.prototype.__construct = function()
{
	superMethod(ComboBox.prototype, '__construct', this)();
	
	// init default text
	if (this.DOM.hasAttribute('label'))
	{
		this._defaultLabel = this.DOM.getAttribute('label');
	}
	
	if (this.DOM.hasAttribute('min-width'))
	{
		this.optionsBox.style.minWidth = this.DOM.getAttribute('min-width');
	}
	
	this.visibleItems = this.DOM.getAttribute('visibleItems')*1 || ScrollableList.prototype.visibleItems*1;
	
	// init options
	if (this.__inlineData)
	{
		var dp = [], data = $('<span>'+this.__inlineData+'</span>'), self = this;
		$('option', data).each(function(n, obj)
		{
			var label = $(obj).html();
			var value = obj.hasAttribute('value') ? obj.getAttribute('value') : label;
			var disabled = obj.hasAttribute('disabled');
			
			var opt = self.addOption(value, label, disabled);
						
			if (obj.hasAttribute('selected'))
			{
				COM(self.list).selectedIndex = n;
			}
		});
	}
	
	COM(this.list).unselectable = false;
	$(this.list).on('select', this.optionChanged);
	
	$(this.button).on(CLICK_EVENT, this.showOptions);
	$(this.shader).on(CLICK_EVENT, this.hideOptions);
	
	this.hideOptions();
	this.refresh();
}

ComboBox.prototype.createOptionRenderer = function(value, label, disabled)
{
	var opt = COM.create('ComboBoxOption');
	
	if (typeof label == 'undefined') label = String(value);
	opt.label = String(label);
	opt.value = value;
	opt.disabled = Boolean(disabled);
	
	return opt;
}

ComboBox.prototype.addOption = function(value, label, disabled)
{
	var opt = this.createOptionRenderer(value, label, disabled);
	COM(this.list).addItem(opt);
	return opt;
}

ComboBox.prototype.clear = function()
{
	COM(this.list).dataProvider = null;
	this.refresh();
}

ComboBox.prototype.setLabel = function(val)
{
	superMethod(ComboBox.prototype, 'setLabel', this)(val);
	this.refresh();
}

ComboBox.prototype.getDataProvider = function()
{
	return COM(this.list).dataProvider;
}

ComboBox.prototype.setDataProvider = function(val)
{
	var dp = COM(this.list).dataProvider, refresh = (Boolean(dp)^Boolean(val)) || (dp && val && dp.length != val.length);
	if (!refresh) for (var i = 0, len = val.length; i < len; i++)
	{
		if (dp[i] !== val[i])
		{
			refresh = true;
			break;
		}
	}
	if (!refresh) return;
	
	this.clear();
	val = val || [];
	for (var i = 0, len = val.length; i < len; i++)
	{
		if ($.isArray(val[i]))
		{
			this.addOption.apply(this, val[i]);
		}
		else if (val[i] && typeof val[i] == 'object' && val[i]['label'])
		{
			this.addOption(val[i]['value'], val[i]['label'], val[i]['disabled']);
		}
		else
		{
			this.addOption(val[i]);
		}
	}
	this.refresh();
}

ComboBox.prototype.getSelectedIndex = function()
{
	return COM(this.list).selectedIndex;
}

ComboBox.prototype.setSelectedIndex = function(val)
{
	COM(this.list).selectedIndex = val*1;
	this.refresh();
}

ComboBox.prototype.getSelectedOption = function()
{
	return this.selectedIndex < 0 
		? null 
		: COM(COM(this.list).items[this.selectedIndex])
	;
}

ComboBox.prototype.setSelectedOption = function(item)
{
	item = item ? (item.toString ? item.toString() : item) : "";
	var dp = COM(this.list).dataProvider;
	
	var op;
	for(var i = 0; i < dp.length; i++)
	{
		op = dp[i].toString ? dp[i].toString() : dp[i];
		if(item == op)
		{
			this.selectedIndex = i;
			return;
		}
	}
}

ComboBox.prototype.getVisibleItems = function()
{
	return COM(this.list).visibleItems;
}

ComboBox.prototype.setVisibleItems = function(val)
{
	COM(this.list).visibleItems = val;
	COM(this.list).refreshHeight();
}

ComboBox.prototype.showOptions = function(e)
{
	if (e) e.stopImmediatePropagation();
	if (this.disabled)
	{
		this.hideOptions();
		return false;
	}
	if (!this.dataProvider || this.dataProvider.length < 2)
	{
		this.hideOptions();
		return false;
	}
	
	var list = COM(this.list);
	
	document.body.appendChild(this.shader);
	list.refresh();
	
	this.optionsBox.style.width = this.DOM.offsetWidth + 'px';
	var h = Math.min(list.content.offsetHeight, list.wrapper.offsetHeight);
	this.optionsBox.style.height = this.list.style.height = h + 'px';
	list.refresh();
	
	var offset = $(this.button).offset();
	
	var pos = {
		left: offset.left + (this.button.offsetWidth - this.optionsBox.offsetWidth) / 2,
		top: offset.top + this.button.offsetHeight - 8
	}
	pos.left = Math.max(0, Math.min(this.shader.offsetWidth - this.optionsBox.offsetWidth, pos.left));
	if (pos.top > this.shader.offsetHeight - this.optionsBox.offsetHeight)
	{
		pos.top = offset.top - this.optionsBox.offsetHeight;
	}
	pos.top = Math.max(0, Math.min(this.shader.offsetHeight - this.optionsBox.offsetHeight, pos.top));
	
	this.optionsBox.style.left = pos.left + 'px';
	this.optionsBox.style.top = pos.top + 'px';
	
	$(COM(this.button).wrapper).addClass('checked');
	return false;
}

ComboBox.prototype.hideOptions = function(e)
{
	if (e) e.stopImmediatePropagation();
	
	if (this.shader.parentNode)
	{
		this.shader.parentNode.removeChild(this.shader);
	}
	
	$(COM(this.button).wrapper).removeClass('checked');
	return false;
}

ComboBox.prototype.getValue = function()
{
	var opt = this.selectedOption;
	return opt ? opt.value : null;
}

ComboBox.prototype.setValue = function(val)
{
	var options = COM(this.list).items;
	for (var i = 0, len = options.length; i < len; i++)
	{
		if (COM(options[i]).value == val)
		{
			this.selectedIndex = i;
			return;
		}
	}
	this.selectedIndex = -1;
}

ComboBox.prototype.optionChanged = function()
{
	var opt = this.selectedOption;
	this.value = opt ? opt.value : null;
	
	this.refresh();
	this.hideOptions();
	
	$(this.DOM).triggerHandler('change');
}

ComboBox.prototype.refresh = function()
{
	var dp = this.dataProvider || [];
	COM(this.button).enabled = Boolean(dp.length) && !this.disabled;
	
	if (dp.length == 1) COM(this.list).selectedIndex = 0;
	
	var opt = this.selectedOption;
	COM(this.button).caption = opt ? opt.label : this._defaultLabel;
	COM(this.button).icon = (dp.length > 1) 
		? [null, STATIC_INTERFACE_URL + 'common/input/dropdown.png']
		: null
	;
}
