function NumberInput()
{
	this._min = -Number.MAX_VALUE;
	this.__defineGetter__('min', this.getMin);
	this.__defineSetter__('min', this.setMin);
	
	this._max = Number.MAX_VALUE;
	this.__defineGetter__('max', this.getMax);
	this.__defineSetter__('max', this.setMax);
	
	this._step = 1;
	this.__defineGetter__('step', this.getStep);
	this.__defineSetter__('step', this.setStep);
	
	this._value = 0;
	
	callParentConstructor(NumberInput.prototype, this);
}

extendClass(NumberInput.prototype, TextInput.prototype);

NumberInput.prototype._min = 0;
NumberInput.prototype._max = Number.MAX_VALUE;
NumberInput.prototype._step = 1;

NumberInput.prototype.__construct = function()
{
	if (this.DOM.hasAttribute('min')) this._min = this.DOM.getAttribute('min') * 1;
	if (this.DOM.hasAttribute('max')) this._max = this.DOM.getAttribute('max') * 1;
	if (this.DOM.hasAttribute('step')) this._step = this.DOM.getAttribute('step') * 1;
	
	this.keyboard = 'NumericKeyboard';
	
	superMethod(NumberInput.prototype, '__construct', this)();
}

NumberInput.prototype.validate = function(val)
{
	// cast to number (Boolean(NaN) === false)
	val = val * 1 || 0;
	
	// fit step
	if (this.min != Number.MIN_VALUE)
	{
		var stepPoint =  this.min;
		val = stepPoint + (this.step * Math.ceil((val - stepPoint) / (this.step || 1)));
	}
	
	// fit min-max range
	val = Math.max(this._min, Math.min(this._max, val));
	
	return val;
}

NumberInput.prototype.getValue = function()
{
	return this.validate(this._value);
}

NumberInput.prototype.setValue = function(val)
{
	val = this.validate(val);
	return superMethod(NumberInput.prototype, 'setValue', this)(val);
}

NumberInput.prototype.focus = function(e)
{
	superMethod(NumberInput.prototype, 'focus', this)(e);
	this._value = '';
	this.inputValue.value = this._value;
	this.inputReplacer.innerHTML = this._value;
}

NumberInput.prototype.getMin = function()
{
	var min = isNaN(this._min * 1) ? -Number.MAX_VALUE : this._min * 1;
	return min;
}

NumberInput.prototype.setMin = function(val)
{
	val = isNaN(val * 1) || val === null ? -Number.MAX_VALUE : val * 1;
	this._min = Math.min(val, this._max);
	this.value = this.validate(this.value);
}

NumberInput.prototype.getMax = function()
{
	var max = isNaN(this._max * 1) ? Number.MAX_VALUE : this._max * 1;
	return max;
}
NumberInput.prototype.setMax = function(val)
{
	val = isNaN(val * 1) || val === null ? Number.MAX_VALUE : val * 1;
	this._max = Math.max(val, this._min);
	this.value = this.validate(this.value);
}

NumberInput.prototype.getStep = function()
{
	return this._step > 0 ? this._step * 1 : 1;
}

NumberInput.prototype.setStep = function(val)
{
	val = val > 0 ? val*1 : 1;
	this._step = Math.min(Math.abs(val*1), Math.abs(this._max - this._min));
	this.value = this.validate(this.value);
}

NumberInput.prototype.doStep = function(n)
{
	this.value += n * (this._step || 1);
}
