function TextInput()
{
	this._size = 0;
	this.__defineGetter__('size', this.getSize);
	this.__defineSetter__('size', this.setSize);
	
	this.__defineGetter__('icon', this.getIcon);
	this.__defineSetter__('icon', this.setIcon);
	
	this._keyboard = null;
	this.__defineGetter__('keyboard', this.getKeyboard);
	this.__defineSetter__('keyboard', this.setKeyboard);
	
	this.__defineGetter__('placeholder', this.getPlaceholder);
	this.__defineSetter__('placeholder', this.setPlaceholder);
	
	this.__defineGetter__('valid', this.getValid);
	this.__defineSetter__('valid', this.setValid);
	
	this.__defineGetter__('datalist', this.getDatalist);
	this.__defineSetter__('datalist', this.setDatalist);
	
	this.__defineGetter__('nativeType', this.getNativeType);
	this.__defineSetter__('nativeType', this.setNativeType);
	
	callParentConstructor(TextInput.prototype, this);
}

extendClass(TextInput.prototype, Input.prototype);

TextInput.prototype._size = 0;

TextInput.prototype.__construct = function()
{
	if (this.DOM.hasAttribute('size')) this.size = this.DOM.getAttribute('size') * 1;
	if (this.DOM.hasAttribute('keyboard')) this.keyboard = this.DOM.getAttribute('keyboard');
	if (this.DOM.hasAttribute('placeholder')) this.placeholder = this.DOM.getAttribute('placeholder');
	if (this.DOM.hasAttribute('nativeType')) this.nativeType = this.DOM.getAttribute('nativeType');
	if (this.DOM.hasAttribute('disabled')) this.disabled = true;
	this.icon = this.DOM.getAttribute('icon');
	
	if (this['autocomleteData'])
	{
		var dataId = this.ID + '-datalist';
		this.autocomleteData.setAttribute('id', dataId);
		this.inputValue.setAttribute('list', dataId);
	}
		
	superMethod(TextInput.prototype, '__construct', this)();
}

TextInput.prototype.setDisabled = function(val)
{
	superMethod(TextInput.prototype, "setDisabled", this)(val);
	if (this._disabled)
	{
		this.inputValue.setAttribute('disabled', 'disabled');
		this.inputReplacer.setAttribute('disabled', 'disabled');
	}
	else
	{
		this.inputValue.removeAttribute('disabled');
		this.inputReplacer.removeAttribute('disabled');
	}
	$(this.DOM).toggleClass('disabled', !!this._disabled);
}

TextInput.prototype.getValue = function()
{
	this._value = this.inputValue.value;
	return superMethod(TextInput.prototype, 'getValue', this)();
}

TextInput.prototype.setValue = function(val)
{
	var oldVal = this._value;

	val = this.validate(val);
	this.inputValue.value = val;
	this.inputReplacer.innerHTML = val;
	
	superMethod(TextInput.prototype, 'setValue', this)(val);
}

TextInput.prototype.getIcon = function()
{
	return this.iconImage.src;
}
TextInput.prototype.setIcon = function(val)
{
	this.iconImage.src = val || '';
	$('icon', this.DOM).toggleClass('no-icon', !Boolean(val));
}

TextInput.prototype.getValid = function()
{
	if ($(this.DOM).hasClass('input-valid')) return true;
	if ($(this.DOM).hasClass('input-invalid')) return false;
	return undefined;
}
TextInput.prototype.setValid = function(val)
{
	$(this.DOM)
		.toggleClass('input-valid', (val === true))
		.toggleClass('input-invalid', (val === false))
	;
}

TextInput.prototype.getPlaceholder = function()
{
	return this.inputValue.placeholder;
}
TextInput.prototype.setPlaceholder = function(val)
{
	this.inputValue.placeholder = val;
}

TextInput.prototype.getSize = function()
{
	return this._size;
}
TextInput.prototype.setSize = function(val)
{
	this._size = val*1 || null;
	this.inputValue.size = this._size;
	this.inputValue.maxLength = this._size;
}

TextInput.prototype.getKeyboard = function()
{
	return this._keyboard;
}

TextInput.prototype.setKeyboard = function(cls)
{
	$(this.clickArea).off(CLICK_EVENT);
	if (cls)
	{
		$(this.clickArea).on(CLICK_EVENT, $.proxy(this.focus, this));
	}
	this._keyboard = cls || null;
	if (this._keyboard && ('ontouchstart' in window))
	{
		this.inputValue.setAttribute('readonly', 'readonly');
		UI.show(false, [this.inputValue]);
		UI.show(true, [this.inputReplacer]);
	}
	else
	{
		this.inputValue.removeAttribute('readonly');
		UI.show(true, [this.inputValue]);
		UI.show(false, [this.inputReplacer]);
	}
}

TextInput.prototype.getNativeType = function()
{
	return this.inputValue.getAttribute('type') || 'text';
}

TextInput.prototype.setNativeType = function(val)
{
	this.inputValue.getAttribute('type', val);
}

TextInput.prototype.getDatalist = function()
{
	var res = [];
	if (this['autocomleteData']) for (var i = 0; i < this.autocomleteData.children.length; i++)
	{
		res.push(this.autocomleteData.children[i].value);
	}
	return res;
}
TextInput.prototype.setDatalist = function(val)
{
	if (!this['autocomleteData']) return;
	if (!val) val = [];
	if (!$.isArray(val)) val = [val];
	
	this.autocomleteData.innerHTML = '';
	for (var i = 0; i < val.length; i++)
	{
		var option = document.createElement('option');
		option.value = val[i];
		this.autocomleteData.appendChild(option);
	}
}

TextInput.prototype.focus = function(e)
{
	if(this._disabled)
	{
		return;
	}
	
	if (e)
	{
		e.preventDefault();
		e.stopImmediatePropagation();
	}
	if (VirtualKeyboard.instance)
	{
		VirtualKeyboard.instance.hide();
	}
	if (this.keyboard)
	{
		// create virtual keyboard
		var kbd = VirtualKeyboard.create(this.keyboard, this.inputValue, this);
		kbd.onChange = $.proxy(function(val)
		{
			this.inputReplacer.innerHTML = val;
			this.inputValue.value = val;
			
		}, this);
		kbd.onCommit = $.proxy(function(val)
		{
			this.value = val;
			$(this.DOM).triggerHandler('change');
			
		}, this);
		kbd.onRevert = $.proxy(function(val)
		{
			this.value = val;
			
		}, this);
		kbd.show();
	}
	// use native keyboard if not readonly
	if (!this.inputValue.getAttribute('readonly')) 
	{
		$(this.inputValue).off('change').on('change', $.proxy(function(){
			$(this.DOM).triggerHandler('change');
		}, this));
		this.inputValue.focus();
	}
	return false;
}
