function VirtualKeyboard()
{
	if (this.__className__ == 'VirtualKeyboard') throw new Error('VirtualKeyboard is abstract class and cannot be instantiated.');
	
	this.input = null;
	this.target = null;
	this.originalValue = '';
	this.edited = false;
	
	this.onChange = null;
	this.onCommit = null;
	this.onRevert = null;
	
	this.__defineGetter__('keys', this.getKeys);
}

VirtualKeyboard.prototype.__className__ = 'VirtualKeyboard';

VirtualKeyboard.prototype.input = null; // input node
VirtualKeyboard.prototype.target = null; // input component
VirtualKeyboard.prototype.originalValue = '';
VirtualKeyboard.prototype.edited = false;

VirtualKeyboard.prototype.onChange = null;
VirtualKeyboard.prototype.onCommit = null;
VirtualKeyboard.prototype.onRevert = null;

VirtualKeyboard.instance = null;
VirtualKeyboard.create = function(cls, input, target)
{
	if (VirtualKeyboard.instance)
	{
		VirtualKeyboard.instance.hide();
	}
	VirtualKeyboard.instance = COM.create(cls, null, input, target);
	return VirtualKeyboard.instance;
}

VirtualKeyboard.prototype.__construct = function(input, target)
{
	this.input = input;
	this.target = target;
	
	if (!this.input) return;
	this.originalValue = this.input.value + '';
	 
	Mousetrap.bindGlobal(this.keys, $.proxy(this.processKey, this));
	
	$(this.DOM).on(CLICK_EVENT, $.proxy(function(){ return false; }, this));
	var triggerKey = $.proxy(this.triggerKey, this);
	$('key[k]', this.DOM).each(function(){ COM.bindExclusiveClick(this, triggerKey); });

	//this.input.setAttribute('readonly', 'readonly');
	var preventNativeKeyboard = Boolean('ontouchstart' in window);
	if (preventNativeKeyboard)
	{
		this.input.setAttribute('readonly', 'readonly');
	}
	else
	{
		this.input.removeAttribute('readonly');
	}
	
	$(this.target.DOM).addClass('focus');
}

VirtualKeyboard.prototype.__destruct = function()
{
	Mousetrap.pause();
	Mousetrap.reset();
	$(this.target.DOM).removeClass('focus');
	this.input.blur();
	VirtualKeyboard.instance = null;
	APP.keyboard(null);
}

VirtualKeyboard.prototype.getKeys = function()
{
	var nodes = this.DOM.querySelectorAll('key[k]'), len = nodes.length, keys = ['esc'];
	for (var i = 0; i < len; i++) 
	{
		keys.push(nodes[i].getAttribute('k'));
	}
	return keys;
}

VirtualKeyboard.prototype.processKey = function(e, alias)
{
	var readonly = this.input.getAttribute('readonly');
	var currentValue = this.input.value + '';
	var newValue = currentValue + '';
	switch (alias)
	{
		case 'enter':
			this.commit();
			this.hide();
			return false;

		case 'esc':
			this.revert();
			this.hide();
			return false;

		case 'backspace':
			newValue = '';
			this.input.value = newValue;//currentValue.length ? currentValue.substr(0, currentValue.length - 1) : '';
			this.edited = false;
			if (typeof this.onChange == 'function')
			{
				this.onChange(newValue, currentValue);
			}
			break;
			
		/*
		case 'tab': break;
		case 'shift': break;
		case 'ctrl': break;
		case 'alt': break;
		case 'capslock': break;
		case 'pageup': break;
		case 'pagedown': break;
		case 'end': break;
		case 'home': break;
		case 'left': break;
		case 'up': break;
		case 'right': break;
		case 'down': break;
		case 'ins': break;
		case 'del': break;
		case 'meta': break;
		*/
		
		default: 
			// Must be symbolic key and be present on virtual keyboard
			if (alias.length > 1 || this.keys.indexOf(alias) < 0)
			{
				return false;
			}
			
			this.edited = true;
			newValue += alias;
			this.input.value = newValue;
			if (typeof this.onChange == 'function')
			{
				this.onChange(newValue, currentValue);
			}
			break;
	}
	if (!readonly) this.input.focus();
	return false;
}

VirtualKeyboard.prototype.triggerKey = function(e)
{
	var k = e['currentTarget'] ? e.currentTarget.getAttribute('k') : String(e);
	Mousetrap.trigger(k);
	return false;
}

VirtualKeyboard.prototype.show = function()
{
	var readonly = this.input.getAttribute('readonly');
	APP.keyboard(this);
	Mousetrap.unpause();
	this.input.focus();
	return false;
}

VirtualKeyboard.prototype.hide = function(e)
{
	if (VirtualKeyboard.instance)
	{
		VirtualKeyboard.instance.destroy();
	}
	if (e)
	{
		e.preventDefault();
		e.stopImmediatePropagation();
	}
	return false;
}

VirtualKeyboard.prototype.commit = function()
{
	var val = this.edited ? this.input.value : this.originalValue;
	this.target.value = val;
	if (typeof this.onCommit == 'function')
	{
		this.onCommit(this.target.value, this.originalValue);
	}
}

VirtualKeyboard.prototype.revert = function()
{
	var newValue = this.input.value;
	this.input.value = this.target.value = this.originalValue;
	if (typeof this.onRevert == 'function')
	{
		this.onRevert(this.originalValue, newValue);
	}
}

VirtualKeyboard.prototype.fitLayout = function(fit)
{
	// window bounds
	var w = {w: window.innerWidth, h: window.innerHeight};
	
	function intersects(r1, r2)
	{
		if (r1.x < r2.x && r1.x + r1.w < r2.x) return false;
		if (r1.y < r2.y && r1.y + r1.h < r2.y) return false;
		
		if (r1.x > r2.x + r2.w && r1.x + r1.w > r2.x + r2.w) return false;
		if (r1.y > r2.y + r2.h && r1.y + r1.h > r2.y + r2.h) return false;
		
		return true;
	}
	
	function fitK(obj, extend)
	{
		var k = $.extend(obj, extend || {});
		k.x = Math.max(0, Math.min(w.w - k.w, k.x*1));
		k.y = Math.max(0, Math.min(w.h - k.h, k.y*1));
		return k;
	}
	
	// input rectangle
	var offset = $(this.target.DOM).offset();
	var i = {
		x: offset.left,
		y: offset.top,
		w: this.target.DOM.offsetWidth,
		h: this.target.DOM.offsetHeight
	};
	
	// keyboard rectangle
	var k = {
		w: this.DOM.offsetWidth, 
		h: this.DOM.offsetHeight,
	}
	
	// default position and check window bounds
	k = fitK(k, {x: i.x - (k.w - i.w) / 2, y: i.y + i.h + 4});
	
	// check keyboard/input intersection
	if (intersects(k, i))
	{
		// 1. try to move up
		k = fitK(k, {y: i.y - k.h - 4});
	}
	if (intersects(k, i))
	{
		// 2. try to move right
		k = fitK(k, {x: i.x + i.w + 4, y: i.y});
	}
	if (intersects(k, i))
	{
		// 3. try to move left
		k = fitK(k, {x: i.x - k.w - 4, y: i.y});
	}
	
	if (intersects(k, i))
	{
		// finally don't care ))
		k = fitK(k, {x: i.x - (k.w - i.w) / 2, y: i.y + i.h + 4});
	}
	
	this.DOM.style.left = k.x + 'px';
	this.DOM.style.top = k.y + 'px';
}
