function ScrollPager()
{
	this._total = 1;
	this.__defineGetter__('total', this.getTotal);
	this.__defineSetter__('total', this.setTotal);
	
	this._page = 0;
	this.__defineGetter__('page', this.getPage);
	this.__defineSetter__('page', this.setPage);

	this._enabled = true;
	this.__defineGetter__('enabled', this.getEnabled);
	this.__defineSetter__('enabled', this.setEnabled);

	this.onChange = null;
	this.onSelect = null;

	this._needRefreshScale = true;
	
	this.__defineGetter__('hintPosition', function(){
		return $(this.hint).hasClass('arrow-left') ? 'right' : 'left';
	});
	this.__defineSetter__('hintPosition', function(val)
	{
		flag = val != 'right';
		$(this.hint)
			.toggleClass('arrow-right', flag)
			.toggleClass('arrow-left', !flag)
		;

	});
}

ScrollPager.prototype.onChange = null;
ScrollPager.prototype.onSelect = null;
ScrollPager.prototype._total = 0;
ScrollPager.prototype._page = 0;
ScrollPager.prototype._enabled = true;

ScrollPager.prototype._needRefreshScale = true;

ScrollPager.prototype.__construct = function()
{
	//UI.show(false, [this.DOM]); // should be visible only if total pages > 1
	UI.show(false, [this.hint]); // should be visible only on dragging marker
	
	if (this.DOM.hasAttribute('hintPosition') && (this.DOM.getAttribute('hintPosition') == 'right'))
	{
		this.hintPosition = 'right';
	}
	
	var scale = $(this.scale);
	
	document.body.appendChild(this.hint);
	
	var cancelEvent = function(e)
	{
		if (e)
		{
			e.preventDefault();
			e.stopImmediatePropagation();
		}
		return false;
	}
	
	var syncMarker = $.proxy(function(e)
	{
		var y = (e['originalEvent']['changedTouches'] ? e['originalEvent']['changedTouches'][0] : e).clientY - scale.offset().top,
			h = this.scale.offsetHeight,
			p = Math.round(this.total * y / h)
		;
		if (this.page != p)
		{
			if (typeof this.onChange == 'function')
			{
				this.onChange(this.page, this.total);
			}
			this.page = p;
		}
		
	}, this);
	
	var scrollFlag = !!iScroll.enabled, pagerFlag = false, ticksFlag = !!APP['ticksIsPaused'];
	var addListeners = $.proxy(function(e)
	{
		if (pagerFlag) return cancelEvent(e);

		pagerFlag = true;
		scrollFlag = !!iScroll.enabled;
		ticksFlag = !!APP['ticksIsPaused'];
		APP.pauseTicks();
		iScroll.enabled = false;
		$(document.body)
			.on(MOUSE_MOVE_EVENT, onMouseMove)
			.on(MOUSE_UP_EVENT, onMouseUp)
		;
	}, this);
	
	var removeListeners = $.proxy(function()
	{
		pagerFlag = false;
		iScroll.enabled = scrollFlag;
		setTimeout(function(){APP.ticksIsPaused = ticksFlag; }, 500);
		$(document.body)
			.off(MOUSE_MOVE_EVENT, onMouseMove)
			.off(MOUSE_UP_EVENT, onMouseUp)
		;
		
	}, this);
	
	var onMouseDown = $.proxy(function(e)
	{
		if (this.enabled && this.total > 1)
		{
			syncMarker(e);
			this.showHint();
			addListeners();
		}
		else
		{
			removeListeners(e);
		}
		return cancelEvent(e);
	}, this);
	
	var onMouseMove = $.proxy(function(e)
	{
		syncMarker(e);
		return cancelEvent(e);
	}, this);
	
	var onMouseUp = $.proxy(function(e)
	{
		removeListeners(e);
		this.hideHint();
		if (typeof this.onSelect == 'function')
		{
			this.onSelect(this.page, this.total);
		}
		return cancelEvent(e);
	}, this);
	
	this.hideHintTimer = null;
	
	scale.on(MOUSE_DOWN_EVENT, onMouseDown);
}

ScrollPager.prototype.__destruct = function()
{
	iScroll.enabled = true;
	document.body.removeChild(this.hint);
	this.hint = null;
}

ScrollPager.prototype.showHint = function(autoHide)
{
	UI.show(true, [this.hint]);
	clearTimeout(this.hideHintTimer);
	
	if(autoHide) this.hideHintTimer = setTimeout($.proxy(this.hideHint, this), 500);
}

ScrollPager.prototype.hideHint = function()
{
	UI.show(false, [this.hint]);
}

ScrollPager.prototype.getEnabled = function()
{
	return Boolean(this._enabled); 
}
ScrollPager.prototype.setEnabled = function(val)
{
	val = Boolean(val);
	if (this.enabled ^ val)
	{
		this._enabled = val;
		$(this.DOM).toggleClass('disabled grayscale', !val);
	}
	
}

ScrollPager.prototype.getTotal = function()
{
	return this._total; 
}
ScrollPager.prototype.setTotal = function(val)
{
	this._total = Math.max(0, val*1);
	
	if (!UI.show(this.total > 1, [this.min, this.max, this.marker])) return;
	
	this._page = Math.min(this._total-1, this._page);
	this._needRefreshScale = true;
	this.min.innerHTML = 1;
	this.max.innerHTML = this.total;
	this.refreshMarker();
}

ScrollPager.prototype.getPage = function()
{
	return this._page; 
}

ScrollPager.prototype.setPage = function(val)
{
	this._page = Math.max(0, Math.min(this._total-1, val*1));
	this.refreshMarker();
}

ScrollPager.prototype.setIndexAndCount = function(page, total)
{
	this.total = total;
	this.page = page;
}

ScrollPager.prototype.refreshScale = function(force)
{
}

ScrollPager.prototype.refreshMarker = function(force)
{
	if (!this.total) return;
	
	this.refreshScale(force);
	
	var h = (this.points.offsetHeight - this.marker.offsetHeight);
	var pos = Math.max(0, Math.min(1, this.page / (this.total-1)));
	
	this.marker.style.top = (h * pos) + 'px';
	
	var hintPos = $(this.DOM).offset();
	hintPos.top += (this.marker.offsetHeight / 2) + (h * pos);
	hintPos.left += (this.hintPosition == 'right' ? this.DOM.offsetWidth + 2 : -2);
	
	this.hint.style.top = hintPos.top + 'px';
	this.hint.style.left = hintPos.left + 'px';
	
	this.hintContent.innerHTML = (this.page+1) + '/' + this.total;
	this.marker.innerHTML = (this.page+1);
}
