function ScrollableList()
{
	this.scroll = null;
	
	this._data = null;
	this._skin = 'default';
	this._selectable = false;
	this.visibleItems = 5.5;
	
	this.__defineGetter__('dataProvider', this.getDataProvider);
	this.__defineSetter__('dataProvider', this.setDataProvider);
	
	this.__defineGetter__('skin', this.getSkin);
	this.__defineSetter__('skin', this.setSkin);
	
	this.__defineGetter__('selectable', this.getSelectable);
	this.__defineSetter__('selectable', this.setSelectable);
	
	this.__defineGetter__('selectedIndex', this.getSelectedIndex);
	this.__defineSetter__('selectedIndex', this.setSelectedIndex);
	
	this.__defineGetter__('enabled', this.getEnabled);
	this.__defineSetter__('enabled', this.setEnabled);
	
	this.__defineGetter__('selectedItem', this.getSelectedItem);
	this.__defineGetter__('value', this.getSelectedValue);
	
	this.__defineGetter__('items', this.getItems);
	
	this.onItemClick = null;
	this.onItemSelect = null;

	setTimeout($.proxy(this.refresh, this), 500);
	setTimeout($.proxy(this.refreshHeader, this), 500);
}

ScrollableList.prototype._data = null;
ScrollableList.prototype._skin = 'default';
ScrollableList.prototype._selectable = false;
ScrollableList.prototype.unselectable = true;
ScrollableList.prototype.visibleItems = 5.5;

ScrollableList.prototype.onItemClick = null;
ScrollableList.prototype.onItemSelect = null;

ScrollableList.prototype.rendererClass = 'ScrollableListItem';
ScrollableList.prototype.scrollOptions = {};

ScrollableList.prototype.__construct = function()
{
	this.DOM.style.display = 'block';
	if (this.DOM.hasAttribute('renderer'))
	{
		this.rendererClass = this.DOM.getAttribute('renderer');
	}
	if (this.DOM.hasAttribute('selectable'))
	{
		this.selectable = true;
	}
	if (this.DOM.hasAttribute('visibleItems'))
	{
		var n = this.DOM.getAttribute('visibleItems')*1;
		if (!isNaN(n) && n > 0) this.visibleItems = n;
	}
	
	this.skin = this.DOM.hasAttribute('skin') ? this.DOM.getAttribute('skin') : 'default';
}

ScrollableList.prototype.getItems = function()
{
	return this.content.children;
}

ScrollableList.prototype.getEnabled = function()
{
	return this.scroll ? this.scroll.enabled : false;
}

ScrollableList.prototype.setEnabled = function(val)
{
	if (this.scroll) 
	{
		this.scroll.enabled = Boolean(val);
		if (val)
		{
			$(this.DOM).off(MOUSE_DOWN_EVENT);
		}
		else
		{
			$(this.DOM).on(MOUSE_DOWN_EVENT, function(e){ 
				e.preventDefault(); 
				e.stopImmediatePropagation(); 
				return false; 
			});
		}
	}
}

ScrollableList.prototype.getDataProvider = function()
{
	return this._data;
}

ScrollableList.prototype.setDataProvider = function(val)
{
	this._data = (val && typeof val == 'object' && val['length']) ? val : null;
	
	this.updateChildren();
	this.refreshHeader();
	this.refresh();
}

ScrollableList.prototype.getSkin = function()
{
	return this._skin;
}

ScrollableList.prototype.setSkin = function(val)
{
	if (this._skin) 
	{
		$(this.list).removeClass(this._skin);
	}
	this._skin = val;
	$(this.list).addClass(this._skin);
	
	this.refresh();
}

ScrollableList.prototype.getSelectable = function()
{
	return this._selectable;
}

ScrollableList.prototype.setSelectable = function(val)
{
	val = Boolean(val);
	if (!val)
	{
		this.selectedIndex = -1;
	}
	if (val == this._selectable) return;
	
	this._selectable = Boolean(val);
	$(this.list).toggleClass('selectable', this._selectable);
}

ScrollableList.prototype.getSelectedItem = function()
{
	var idx = this.selectedIndex, d = this.items;
	return (idx >= 0 && d && d.length > idx) ? COM(d[idx]) : null;
}

ScrollableList.prototype.getSelectedValue = function()
{
	var idx = this.selectedIndex, d = this.dataProvider;
	return (idx >= 0 && d && d.length > idx) ? d[idx] : null;
}

ScrollableList.prototype.getSelectedIndex = function()
{
	for (var i = 0, len = this.items.length; i < len; i++)
	{
		if (COM(this.items[i]).selected) return i;
	}
	return -1;
}

ScrollableList.prototype.setSelectedIndex = function(val)
{
	for (var i = 0, len = this.items.length; i < len; i++)
	{
		COM(this.items[i]).selected = false;
	}
	
	val = Math.min(val, this.items.length - 1);
	if (val < 0) return;
	 
	COM(this.items[val]).selected = true;
}

ScrollableList.prototype.createRenderer = function(data)
{
	var renderer = COM.create(this.rendererClass);
	if (data)
	{
		renderer.data = data;
	}
	return renderer;
}

ScrollableList.prototype.setHeader = function(data)
{
	var ok = data && data['length'];
	
	UI.show(ok, [this.header]);
	this.headerRow.innerHTML = '';
	
	if (ok)
	{
		for (var i = 0, len = data.length; i < len; i++)
		{
			var cell = document.createElement('cell');
			var wrapper = document.createElement('div');
			$(wrapper).html(data[i]);
			cell.appendChild(wrapper);
			this.headerRow.appendChild(cell);
		}
	}
	
	this.refreshHeader();
	this.refresh();
}

ScrollableList.prototype.refreshHeader = function(data)
{
	var 
		h = this.headerRow.children.length, 
		items = this.content.children[0] ? this.content.children[0].children.length : 0
	;
	if (h < 1 || items < 1) return;
	
	var sum = 0, w = 0, hh = [];
	for (var i = 0, len = Math.max(h, items); i < len; i++)
	{
		//w = (i < items) ? this.content.children[0].children[i].clientWidth : 0;
		w = (i < items) ? parseFloat(window.getComputedStyle(this.content.children[0].children[i]).width) : 0;
		sum += w;
		if (i < h) 
		{
			hh.push(w);
		}
		else 
		{
			hh[h-1] += w;
		}
		
		if (i < h-1) 
		{
			this.headerRow.children[i].clientWidth = w + 'px';
			w = 0;
		}
	}
	$('>*', this.headerRow).each(function(n, obj){
		var prc = Math.round((hh[n] / sum) * 100);
		
		$(obj).css('width', prc + '%');
		$(">div", obj).css({'width': hh[n] + 'px', 'overflow': 'hidden'});
	});
}

ScrollableList.prototype.clearChildren = function(len)
{
	len = Math.max(0, parseInt(len) || 0);
	while (this.content.children.length > len)
	{
		var node = this.content.children[0], obj = COM.get(node);
		if (obj) 
		{
			obj.destroy();
		}
		else
		{
			this.content.removeChild(node);
		}
	}
}

ScrollableList.prototype.itemClick = function(e)
{
	if (this.scroll && !this.scroll.isReady())
	{
		return;
	}
	
	var item = COM(e.currentTarget);
	$(this.DOM).triggerHandler('itemclick', item, this);
	if (typeof this.onItemClick == 'function')
	{
		if (this.onItemClick(item, this, e) === false) return;
	}
	
	if (this.selectable)
	{
		return this.itemSelect(e);
	}
}

ScrollableList.prototype.itemSelect = function(e)
{
	if (!this.selectable)
	{
		return;
	}
	 
	var 
		target = e.currentTarget, 
		items = this.items,
		oldIdx = this.selectedIndex, idx = -1
	;
	if (COM(target).disabled && oldIdx >= 0)
	{
		if (!this.unselectable) return;
		
		COM(this.items[oldIdx]).selected = false;
		idx = -1;
	}
	else for (var i = 0, len = items.length; i < len; i++)
	{
		if (target.isSameNode(items[i]))
		{
			var sel = (i != oldIdx || !this.unselectable);
			COM(items[i]).selected = sel;
			idx = sel ? i : -1;
		}
		else
		{
			COM(items[i]).selected = false;
		}
	}
	if (idx != oldIdx)
	{
		if (idx >= 0)
		{
			$(this.DOM).triggerHandler('select');
			$(target).triggerHandler('select');
			if (typeof this.onItemSelect == 'function')
			{
				this.onItemSelect(target, idx, this);
			}
		}
		else
		{
			$(this.DOM).triggerHandler('unselect');
			$(target).triggerHandler('unselect');
		}
	}
}

ScrollableList.prototype.updateChildren = function()
{
	if (!this.dataProvider)
	{
		return this.clearChildren(0);
	}
	
	this.clearChildren(this.dataProvider.length);
	while(this.content.children.length < this.dataProvider.length)
	{
		var item = this.createRenderer(this.dataProvider[this.content.children.length] || null);
		this.content.appendChild(item.DOM);
		COM.bindExclusiveClick(item.DOM, $.proxy(this.itemClick, this));
	}
	
	for (var i = 0, len = this.dataProvider.length; i < len; i++)
	{
		COM(this.content.children[i]).data = this.dataProvider[i];
	}
	
	this.refreshHeight();
	
	var c = this.content;
	$('header cell', this.list).each(function(n, obj){
		$(obj).css({ 'width': $('cell', c)[n].offsetWidth });
	});
	
	this.refreshHeader();
}

ScrollableList.prototype.addItem = function(item, idx)
{
	var len = this.items.length;
	if (typeof idx == 'undefined') idx = len;
	idx = Math.min(idx, len);
	
	try
	{
		if (idx == 0)
		{
			$(this.content).prepend(item.DOM);
		}
		else
		{
			$(this.items[idx-1]).after(item.DOM);
		}
		
		if (!this._data) 
		{
			this._data = [];
		}
		this._data.splice(idx, 0, item.data);
		
		COM.bindExclusiveClick(item.DOM, $.proxy(this.itemClick, this));
		
		this.refreshHeader();
		this.refresh();
	}
	catch (e)
	{
		console.error("Can't add list item:", item, e);
	}
}

ScrollableList.prototype.removeItem = function(idx)
{
	// find node index 
	if (typeof idx == 'object')
	{
		var node = idx['DOM'] || idx;
		idx = -1;
		if (node instanceof HTMLElement) for (var i = 0, len = this.items.length; i < len; i++)
		{
			if (this.items[i].isSameNode(node))
			{
				idx = i;
				break;
			}
		}
	}
	
	if (idx < 0 || idx >= this.items.length) return;
	
	/*
	var dy = this.items[idx].offsetHeight;
	var pos = this.scroll._offset(this.items[idx]);
	pos.left += this.scroll.wrapperOffsetLeft;
	pos.top += this.scroll.wrapperOffsetTop;
	// if before current position - move scroll position by this value
	if (pos.top - dy > this.scroll.y)
	{
		this.scroll.scrollTo(0, -dy, 0, true);
	}
	*/
	
	COM(this.items[idx]).destroy();
	this._data.splice(idx, 1);
	
	this.refreshHeader();
	this.refresh();
}

ScrollableList.prototype.refreshHeight = function()
{
	var h = $(this.DOM).height();
	
	var itemsCnt = 0;
	if(this.items)
	{
		for(var i=0, len = this.items.length; i < len; i++)
		{
			if(this.items[i].style.display != "none")
			{
				itemsCnt++;
			}
		}
	}
	
	if (this.visibleItems > 0)
	{
		if (h == 0 && this.items && this.items.length)
		{
			h = Math.min(this.visibleItems, itemsCnt) * this.items[0].offsetHeight;
		}
	}
	$(this.list).css({ 'height': h });
	$(this.wrapper).css({ 'height': h - $(this.header).height() });
}

ScrollableList.prototype.refresh = function()
{
	this.refreshHeight();
	if (!this.scroll)
	{
		var options = $.extend({
			scrollbarColor: '#784f0b',
			hScroll: false,
			hScrollbar: false
		}, this.scrollOptions);
		this.scroll = new iScroll(this.content.parentNode, options);
	}
	
	setTimeout($.proxy(this.scroll.refresh, this.scroll), 300);
}
