function Funds(f)
{
	if (f)
	{
		this.add(f);
	}
}

extendClass(Funds.prototype, Arithmetical.prototype);

Funds.prototype.__CLS__ = 'game.model.Funds';

Funds.prototype.gold = 0;
Funds.prototype.wood = 0;
Funds.prototype.stone = 0;
Funds.prototype.grain = 0;
Funds.prototype.iron = 0;
Funds.prototype.people = 0;

Funds.prototype.supportedProps = ["grain", "wood", "stone", "iron", "gold", "people"];

Funds.prototype.cacheProps = function()
{
	this.supportedProps = ["grain", "wood", "stone", "iron", "gold", "people"]
	return this.supportedProps;
}

Funds.prototype.prepareForRequest = function()
{
	var ret = {__CLS__: this.__CLS__};
	
	for(var i=0; i<this.supportedProps.length; i++)
	{
		ret[this.supportedProps[i]] = this[this.supportedProps[i]];
	}
	
	return ret;
}

Funds.prototype.getValue = function(r)
{
	switch(r.name)
	{
		case Resource.WOOD.name:     return this.wood;
		case Resource.GRAIN.name:    return this.grain;
		case Resource.STONE.name:    return this.stone;
		case Resource.IRON.name:     return this.iron;
		case Resource.GOLD.name:     return this.gold;
		case Resource.PEOPLE.name:   return this.people;
		default: throw new Error("Can't get fund value for resource: " + r);
	}
}

Funds.prototype.setValue = function(r, value)
{
	switch(r.name)
	{
		case Resource.WOOD.name:     	this.wood = value; break;
		case Resource.GRAIN.name:    	this.grain = value; break;
		case Resource.STONE.name:    	this.stone = value; break;
		case Resource.IRON.name:     	this.iron = value; break;
		case Resource.GOLD.name:     	this.gold = value; break;
		case Resource.PEOPLE.name:		this.people = value; break;
		default: 
			console.log("Can't set fund value for resource: ", r);
			throw new Error("Can't set fund value for resource: " + r);
	}
	return this;
}

Funds.prototype.calcCountByCost = function(cost)
{
	var minCount = Number.MAX_VALUE;
	for (var i = 0; i < this.supportedProps.length; i++)
	{
		var prop = this.supportedProps[i];
		if (cost[prop] > 0)
		{
			minCount = Math.min(minCount, Math.floor(this[prop] / cost[prop]));
		}
	}
	return minCount;
}

Funds.prototype.addResource = function(r, amount)
{
	this.setValue(r, this.getValue(r) + amount);
}

Funds.prototype.subtractResource = function(r, amount)
{
	this.setValue(r, this.getValue(r) - amount);
}

Funds.prototype.extractAndReturnOverflow = function(r, amount)
{
	var diff = this.getValue(r) - amount;
	this.setValue(r, Math.max(diff, 0));
	return diff < 0 ? -diff : 0;
}

Funds.prototype.getNonZeroResources = function()
{
	var result = [];
	for (var i = 0; i < Resource.FUNDS_RESOURCES.length; i++)
	{
		var r = Resource.FUNDS_RESOURCES[i];
		if (this.getValue(r) > 0)
		{
			result.push(r);
		}
	}
	return result;
}

Funds.prototype.isEmpty = function ()
{
	for (var i = 0; i < Resource.FUNDS_RESOURCES.length; i++)
	{
		var r = Resource.FUNDS_RESOURCES[i];
		if (this.getValue(r) != 0)
		{
			return false;
		}
	}
	return true;
}

Funds.prototype.clearAllExcept = function(r)
{
	var f = new Funds();
	f.setValue(r, 1);

	for (var i = 0; i < this.supportedProps.length; i++)
	{
		var prop = this.supportedProps[i];
		this[prop] *= f[prop];
	}
	return this;
}

Funds.prototype.propertyToString = function (prop)
{
	return Strings.formatNumber((this[prop]), 3);
}

Funds.prototype.equals = function(value)
{
	if ( typeof value == 'object')
	{
		for (var i = 0; i < this.supportedProps.length; i++)
		{
			var prop = this.supportedProps[i];
			if (!NumberUtils.equals(this[prop], value[prop]))
			{
				return false;
			}
		}
	}
	return true;
}

Funds.prototype.toCost = function()
{
	var self = this;
	var convert = function(r)
	{
		return self.getValue(r) * FConfig.SUPPLY_PRICE_LIMITS.getValue(r).average;
	};
	return this.gold + convert(Resource.WOOD) + convert(Resource.STONE) + convert(Resource.GRAIN) + convert(Resource.IRON);
} 

Funds.prototype.multiplyCopyResourcesPeople = function(resourcesRatio, peoplesRatio)
{
	var ret = new Funds();
	ret.gold = this.gold * resourcesRatio;
	ret.wood = this.wood * resourcesRatio;
	ret.stone = this.stone * resourcesRatio;
	ret.grain = this.grain * resourcesRatio;
	ret.iron = this.iron * resourcesRatio;
	ret.people = this.people * peoplesRatio;
	return ret;
}

Funds.createWithGold = function(amount)
{
	var funds = new Funds();
	funds.gold = amount;
	return funds;
}

