function Science(v, dependsOnRace, dependsOnLevel, separate1stLevel)
{
	callParentConstructor(Science.prototype, this);
	
	this.name = v;
	
	if(typeof dependsOnRace == "undefined") dependsOnRace = true;
	if(typeof dependsOnLevel == "undefined") dependsOnLevel = true;
	if(typeof separate1stLevel == "undefined") separate1stLevel = false;

	this.dependsOnRace = dependsOnRace;
	this.dependsOnLevel = dependsOnLevel;
	this.separate1stLevel = separate1stLevel;
}

extendClass(Science.prototype, Enum.prototype);

Science.__isEnum__ = true;

Science.prototype.__CLS__ = "game.model.Science";

///////// Static methods and constants /////////

Science.prototype.dependsOnRace = false;   // transient (text translation depends on race)
Science.prototype.dependsOnLevel = false;   // transient (text translation depends on level)
Science.prototype.separate1stLevel = false;   // transient (separate text translation for 1st level)

Science.prototype.__construct = function()
{
	var s = Science[this.name];
	
	this.dependsOnRace = s.dependsOnRace;
	this.dependsOnLevel = s.dependsOnLevel;
	this.separate1stLevel = s.separate1stLevel;
}

Science.getUTypeFor = function(s)
{
    var unitType = UnitType[s.name];
	return UnitType.MILITARY.indexOf(unitType) >= 0 ? unitType : null;
}

Science.getMaxConcurrentBuildings = function(scienceValue)
{
	var maxConcurrentBuildings = FConfig.MAX_CONCURRENT_BUILDINGS;
	if (scienceValue > 0)
	{
		maxConcurrentBuildings += scienceValue;
	}
	return maxConcurrentBuildings;
}

Science.getMaxConcurrentRepairings = function(scienceValue)
{
    var maxConcurrentRepairings = 0;
    if (scienceValue > 0) 
    {
        maxConcurrentRepairings += scienceValue;
    }
    return maxConcurrentRepairings;
}

Science.getMaxMissionsCount = function(level)
{
    var maxMissionsCount = FConfig.MAX_MISSIONS;
    if (level > 0) 
    {
        maxMissionsCount += Science.ARMY_MANAGEMENT.getLevel(level - 1).value;
    }
    return maxMissionsCount;
}

Science.getMarketDiscount = function(level)
{
    var marketDiscount = 0;
    if (level > 0) 
    {
        marketDiscount = Science.SPEC_TRADER.getLevel(level - 1).marketTaxDiscount;
    }
    return marketDiscount;
}

Science.getShopRatioBonus = function(level)
{
    var shopRatioBonus = 0;
    if (level > 0) 
    {
        shopRatioBonus = Science.SPEC_TRADER.getLevel(level - 1).shopRatioBonus;
    }
    return shopRatioBonus;
}

Science.getHousesCapacityBonus = function(level)
{
    if (level > 0) 
    {
        return Science.SPEC_BENEFACTOR.getLevel(level - 1).capacityBonus;
    }
    return 0;
}

Science.getWarehouseCapacityBonus = function(level)
{
    if (level > 0) 
    {
        return Science.WAREHOUSE.getLevel(level - 1).value / 100;
    }
    return 0;
}

Science.getHousesIncreaseBonus = function(level)
{
    if (level > 0) 
    {
        return Science.SPEC_BENEFACTOR.getLevel(level - 1).increaseBonus;
    }
    return 0;
}

Science.getFortificationKillsBonus = function(level)
{
    var killsBonus = 0;
    if (level > 0) 
    {
        killsBonus = Science.SPEC_DEFENDER.getLevel(level - 1).fortificationBonus;
    }
    return killsBonus;
}

Science.getFortificationDefenceBonus = function(level)
{
    var defenceBonus = 0;
    if (level > 0) 
    {
        defenceBonus = Science.SPEC_DEFENDER.getLevel(level - 1).fortificationDefenceBonus;
    }
    return defenceBonus;
}

// todo: move to FConfig?
Science.prototype.getLevel = function(i)
{
    return FConfig.science.getValue(this)[i];
}

Science.prototype.getMaxLevel = function()
{
    var levels = FConfig.science.getValueOrNull(this);
    return levels ? levels.length : 0;
}

Science.prototype.isSpecialization = function()
{
    for(var i = 0; i < Science.SPECIALIZATION.length; i++)
    {
    	if (Science.SPECIALIZATION[i].name == this.name) return true;
    } 
    return false;
}

Science.prototype.isClannish = function()
{
    for(var i = 0; i < Science.CLANNISH.length; i++)
    {
    	if (Science.CLANNISH[i].name == this.name) return true;
    } 
    return false;
}


Science.CAPITAL = new Science("CAPITAL", false, false);
Science.MARKET = new Science("MARKET", false, false, true);
Science.BUILDING = new Science("BUILDING", false, false);
Science.COLONIZATION = new Science("COLONIZATION", false, false, true);
Science.TREASURY = new Science("TREASURY", false, false, false);
Science.OUTER_ESTATES = new Science("OUTER_ESTATES", false, false);
Science.ESPIONAGE = new Science("ESPIONAGE", false, false, true);
Science.CARTOGRAPHY = new Science("CARTOGRAPHY", false, false, true);
Science.ARMY_MANAGEMENT = new Science("ARMY_MANAGEMENT", false, false);
Science.OBSERVATION = new Science("OBSERVATION", false, false, true);
Science.REPAIRING = new Science("REPAIRING", false, false);
Science.DIPLOMACY = new Science("DIPLOMACY", false, false, true);
Science.CLANS = new Science("CLANS", false, false, false);
Science.CASTLES = new Science("CASTLES", false, false, false);
Science.SECTORS = new Science("SECTORS", false, false);
Science.HIDEOUT = new Science("HIDEOUT", false, false);
Science.HIDDEN_RESOURCES = new Science("HIDDEN_RESOURCES", false, false);
Science.MERCHANT = new Science("MERCHANT", false, false, true);
Science.TERRAFORMING = new Science("TERRAFORMING", false, false);
Science.FORTIFICATION = new Science("FORTIFICATION", false, false);
Science.TOWER = new Science("TOWER", false, false);
Science.MAGIC_TOWER = new Science("MAGIC_TOWER", false, false);
Science.TEMPLE = new Science("TEMPLE", false, false);
Science.RESURRECTOR = new Science("RESURRECTOR", false, false);
Science.ANTI_ESPIONAGE = new Science("ANTI_ESPIONAGE", false, false);
Science.ANTI_OBSERVATION = new Science("ANTI_OBSERVATION", false, false);
Science.GATE = new Science("GATE", false, false);
Science.GEOLOGY = new Science("GEOLOGY", false, false);
Science.WAREHOUSE = new Science("WAREHOUSE", false, false);
Science.ADD_CELL = new Science("ADD_CELL", false, false);
Science.LABOUR_SERVICE = new Science("LABOUR_SERVICE", false, false);
Science.MILITARY_SERVICE = new Science("MILITARY_SERVICE", false, false);

Science.SPEC_TRADER = new Science("SPEC_TRADER", false, false);
Science.SPEC_CLERIC = new Science("SPEC_CLERIC", false, false);
Science.SPEC_MINER = new Science("SPEC_MINER", false, false);
Science.SPEC_BENEFACTOR = new Science("SPEC_BENEFACTOR", false, false);
Science.SPEC_ARCHITECT = new Science("SPEC_ARCHITECT", false, false);
Science.SPEC_DEFENDER = new Science("SPEC_DEFENDER", false, false);

Science.HEROES = new Science("HEROES", false, false, true);
Science.HEROES_SKILL = new Science("HEROES_SKILL", false, false);
Science.PRISON = new Science("PRISON", false, false);
Science.CAMPS = new Science("CAMPS", false, false);
Science.ALCHEMY = new Science("ALCHEMY", false, true);

Science.MELEE = new Science("MELEE");
Science.SCAVENGER = new Science("SCAVENGER");
Science.RANGER = new Science("RANGER");
Science.CAVALRY = new Science("CAVALRY");
Science.FLYER = new Science("FLYER");
Science.HEALER = new Science("HEALER");
Science.MINER = new Science("MINER");
Science.MAGE = new Science("MAGE");

Science.SPECIALIZATION = [Science.SPEC_CLERIC, Science.SPEC_BENEFACTOR, Science.SPEC_DEFENDER, Science.SPEC_MINER, Science.SPEC_TRADER, Science.SPEC_ARCHITECT];
Science.CLANNISH = [Science.CASTLES];
