//***********************************************************************************************************************
// Base classes
//***********************************************************************************************************************

function Exception(e)
{
	this.__className__ = this.__CLS__.split(".").pop();
}
Exception.prototype.IS_EXCEPTION = true;
Exception.prototype.__CLS__ = 'Exception';

Exception.prototype.getMessage = function()
{
	return this.message;
}

Exception.prototype.toString = function()
{
	return this.getMessage();
}

Exception.check = function(e)
{
	if (!e) return e;
	if ((e['rootCause'] && e['rootCause']['__CLS__']) || (e['__CLS__']))
	{
		//var check = /java.*\.[a-zA-Z]Exception[a-zA-Z]*/;
		var check = /[a-zA-Z]*Exception[a-zA-Z]*/.exec(e['__CLS__'] || e['rootCause']['__CLS__']);
		if (check && (typeof window[check[0]] != 'function'))
		{
			return $.extend(new Exception(), e);
		}
	}
	return e;
}

//---------------------------------------------------------------------------
// PortalException
//---------------------------------------------------------------------------

function PortalException()
{
	callParentConstructor(PortalException.prototype, this);
}
extendClass(PortalException.prototype, Exception.prototype);
PortalException.prototype.__CLS__ = 'game.service.exception.PortalException';

//---------------------------------------------------------------------------
// ClientKnowsException
//---------------------------------------------------------------------------

function ClientKnowsException()
{
	callParentConstructor(ClientKnowsException.prototype, this);
}

extendClass(ClientKnowsException.prototype, PortalException.prototype);
ClientKnowsException.prototype.__CLS__ = 'game.service.exception.ClientKnowsException';

ClientKnowsException.prototype.type = null; // String
ClientKnowsException.prototype.gameException = null; // GameException
ClientKnowsException.prototype.getMessage = function()
{
	if (this.type) 
	{
		return I18.s("CLIENT_KNOWS_ERROR", this.type, this.message);
	}
	return this.message;
}

//---------------------------------------------------------------------------
// GameException
//---------------------------------------------------------------------------

function GameException()
{
	callParentConstructor(GameException.prototype, this);
}
extendClass(GameException.prototype, Exception.prototype);
GameException.prototype.__CLS__ = 'game.service.exception.GameException';
GameException.prototype.IS_GAME_EXCEPTION = true;
GameException.prototype.type = null; // String
GameException.isDisplayable = function(e)
{
	//game exception
	var ret = (e.__CLS__ ==  "game.service.exception.GameException" || (e.superClass && e.superClass.__CLS__ == "game.service.exception.GameException"));
	//hidden exceptions
	ret = (ret && e.__className__ != "SaveInProgressException" && e.__className__ != "SessionExpiredException");
	
	return ret;
}

//***********************************************************************************************************************
// Custom classes
//***********************************************************************************************************************

//---------------------------------------------------------------------------
// AccessDeniedException
//---------------------------------------------------------------------------

function AccessDeniedException()
{
	callParentConstructor(AccessDeniedException.prototype, this);
}

extendClass(AccessDeniedException.prototype, GameException.prototype);
AccessDeniedException.prototype.__CLS__ = 'game.service.exception.AccessDeniedException';

AccessDeniedException.prototype.estateName = null; // String
AccessDeniedException.prototype.location = null; // XYZ
AccessDeniedException.prototype.getMessage = function()
{
	return I18.accessDeniedError(this.type, I18.formatEstateName(this.estateName, this.location));
}

//---------------------------------------------------------------------------
// AllianceException
//---------------------------------------------------------------------------

function AllianceException()
{
	callParentConstructor(AllianceException.prototype, this);
}

extendClass(AllianceException.prototype, GameException.prototype);
AllianceException.prototype.__CLS__ = 'game.service.exception.AllianceException';

AllianceException.prototype.limit = null; // int
AllianceException.prototype.getMessage = function()
{
        return I18.allianceError(this.type, this.limit);
}

//---------------------------------------------------------------------------
// AssertException
//---------------------------------------------------------------------------

function AssertException()
{
}
AssertException.prototype.__CLS__ = 'game.service.exception.AssertException';

AssertException.prototype.gameException = null; // GameException

//---------------------------------------------------------------------------
// AuctionException
//---------------------------------------------------------------------------

function AuctionException()
{
	callParentConstructor(AuctionException.prototype, this);
}

extendClass(AuctionException.prototype, GameException.prototype);
AuctionException.prototype.__CLS__ = 'game.service.exception.AuctionException';

AuctionException.prototype.bannedTill = null; // int
AuctionException.prototype.getMessage = function()
{
	return I18.auctionError(this.type, Tick.toTimeFuture(this.bannedTill));
}

//---------------------------------------------------------------------------
// AuthorizationException
//---------------------------------------------------------------------------

function AuthorizationException()
{
	callParentConstructor(AuthorizationException.prototype, this);
}

extendClass(AuthorizationException.prototype, GameException.prototype);
AuthorizationException.prototype.__CLS__ = 'game.service.exception.AuthorizationException';

//---------------------------------------------------------------------------
// AutoExpeditionNowhereToSend
//---------------------------------------------------------------------------

function AutoExpeditionNowhereToSend()
{
	callParentConstructor(AutoExpeditionNowhereToSend.prototype, this);
}

extendClass(AutoExpeditionNowhereToSend.prototype, GameException.prototype);
AutoExpeditionNowhereToSend.prototype.__CLS__ = 'game.service.exception.AutoExpeditionNowhereToSend';

AutoExpeditionNowhereToSend.prototype.hasMissions = null; // Boolean
AutoExpeditionNowhereToSend.prototype.getMessage = function()
{
	return I18.f(this.hasMissions ? 'NO_FIELD_FOR_AUTOEXPEDITION' : 'AUTOEXPEDITION_TOO_EARLY');
}

//---------------------------------------------------------------------------
// AvatarNotFoundException
//---------------------------------------------------------------------------

function AvatarNotFoundException()
{
	callParentConstructor(AvatarNotFoundException.prototype, this);
}

extendClass(AvatarNotFoundException.prototype, GameException.prototype);
AvatarNotFoundException.prototype.__CLS__ = 'game.service.exception.AvatarNotFoundException';

AvatarNotFoundException.prototype.getMessage = function()
{
	return I18.f('NONEXISTENT_CONTACT_ERROR');
}

//---------------------------------------------------------------------------
// BrotherhoodException
//---------------------------------------------------------------------------

function BrotherhoodException()
{
	callParentConstructor(BrotherhoodException.prototype, this);
}

extendClass(BrotherhoodException.prototype, GameException.prototype);
BrotherhoodException.prototype.__CLS__ = 'game.model.user.brotherhood.BrotherhoodException';

BrotherhoodException.prototype.getMessage = function()
{
	return I18.getBrotherhoodErrorMessage(this.type);
}
BrotherhoodException.errorHandler = function(e)
{
	if (e.__className__ == 'BrotherhoodException') 
	{
		Alert.create(/*BrotherhoodException*/ e.getMessage());
		return true;
	}
	return false;
}

//---------------------------------------------------------------------------
// BuildQueueException
//---------------------------------------------------------------------------

function BuildQueueException()
{
	callParentConstructor(BuildQueueException.prototype, this);
}

extendClass(BuildQueueException.prototype, GameException.prototype);
BuildQueueException.prototype.__CLS__ = 'game.service.exception.BuildQueueException';

//---------------------------------------------------------------------------
// BuildingException
//---------------------------------------------------------------------------

function BuildingException()
{
	callParentConstructor(BuildingException.prototype, this);
}

extendClass(BuildingException.prototype, GameException.prototype);
BuildingException.prototype.__CLS__ = 'game.service.exception.BuildingException';

BuildingException.prototype.getMessage = function()
{
	return I18.buildingError(this.type);
}

//---------------------------------------------------------------------------
// CannotCastException
//---------------------------------------------------------------------------

function CannotCastException()
{
	callParentConstructor(CannotCastException.prototype, this);
}

extendClass(CannotCastException.prototype, GameException.prototype);
CannotCastException.prototype.__CLS__ = 'game.service.exception.CannotCastException';

CannotCastException.prototype.getMessage = function()
{
	return I18.castError(this.type);
}

//---------------------------------------------------------------------------
// CannotShowOuterEstates
//---------------------------------------------------------------------------

function CannotShowOuterEstates()
{
	callParentConstructor(CannotShowOuterEstates.prototype, this);
}

extendClass(CannotShowOuterEstates.prototype, GameException.prototype);
CannotShowOuterEstates.prototype.__CLS__ = 'game.service.exception.CannotShowOuterEstates';

CannotShowOuterEstates.prototype.template = null; // String
CannotShowOuterEstates.prototype.getMessage = function()
{
	return I18.f("DONT_HAVE_SO_MANY_OUTER_ESTATES", this.template);
}

//---------------------------------------------------------------------------
// CantStartMissionException
//---------------------------------------------------------------------------

function CantStartMissionException()
{
	callParentConstructor(CantStartMissionException.prototype, this);
}

extendClass(CantStartMissionException.prototype, GameException.prototype);
CantStartMissionException.prototype.__CLS__ = 'game.service.exception.CantStartMissionException';

CantStartMissionException.prototype.nextAttackTick = null; // Number
CantStartMissionException.prototype.attacksLimit = null; // Number
CantStartMissionException.prototype.getMessage = function()
{
	return I18.missionStartError(this.type, this.attacksLimit, Tick.toTimeFuture(this.nextAttackTick));
}

//---------------------------------------------------------------------------
// CityDeleteException
//---------------------------------------------------------------------------

function CityDeleteException()
{
	callParentConstructor(CityDeleteException.prototype, this);
}

extendClass(CityDeleteException.prototype, GameException.prototype);
CityDeleteException.prototype.__CLS__ = 'game.service.exception.CityDeleteException';

CityDeleteException.prototype.getMessage = function()
{
	return I18.cityDeleteError(this.type);
}

//---------------------------------------------------------------------------
// ClanException
//---------------------------------------------------------------------------

function ClanException()
{
	callParentConstructor(ClanException.prototype, this);
}

extendClass(ClanException.prototype, GameException.prototype);
ClanException.prototype.__CLS__ = 'game.service.exception.ClanException';

ClanException.prototype.username = null; // String
ClanException.prototype.getMessage = function()
{
	return I18.getClanErrorMessage(this.type, this.username, FConfig.CLAN_DONATE_RATING);
}
ClanException.errorHandler = function(e)
{
	if (e.__className__ == 'ClanException') 
	{
		Alert.create(/*ClanException*/ e.getMessage());
		return true;
	}
	return false;
}

//---------------------------------------------------------------------------
// CreateAvatarException
//---------------------------------------------------------------------------

function CreateAvatarException()
{
	callParentConstructor(CreateAvatarException.prototype, this);
}

extendClass(CreateAvatarException.prototype, GameException.prototype);
CreateAvatarException.prototype.__CLS__ = 'game.service.exception.CreateAvatarException';

CreateAvatarException.prototype.getMessage = function()
{
	return I18.s("CREATE_AVATAR_ERROR", this.type);
}

//---------------------------------------------------------------------------
// EspionageException
//---------------------------------------------------------------------------

function EspionageException()
{
	callParentConstructor(EspionageException.prototype, this);
}

extendClass(EspionageException.prototype, GameException.prototype);
EspionageException.prototype.__CLS__ = 'game.service.exception.EspionageException';

EspionageException.prototype.getMessage = function()
{
	return I18.s("ESPIONAGE_ERROR", this.type);
}

//---------------------------------------------------------------------------
// EstateNotFoundException
//---------------------------------------------------------------------------

function EstateNotFoundException()
{
	callParentConstructor(EstateNotFoundException.prototype, this);
}

extendClass(EstateNotFoundException.prototype, GameException.prototype);
EstateNotFoundException.prototype.__CLS__ = 'game.service.exception.EstateNotFoundException';

EstateNotFoundException.prototype.pwnedBy = null; // String
EstateNotFoundException.prototype.getMessage = function()
{
	return I18.f('ESTATE_NOT_FOUND');
}
EstateNotFoundException.prototype.showMessageAndGo = function()
{
    FUser.pwnedDialogShown = true;
    if (this.pwnedBy) 
    {
        PwnedDialog.createAndShow(this.pwnedBy);
    } 
    else 
    {
		//Alert.create(this.getMessage()).onClose = function() {
			Events.onShowKingdom.fire(null);
		//};
    }
}
EstateNotFoundException.prototype.showMessageAndStay = function()
{
	Alert.create(this.getMessage());
}

//---------------------------------------------------------------------------
// FriendException
//---------------------------------------------------------------------------

function FriendException()
{
	callParentConstructor(FriendException.prototype, this);
}

extendClass(FriendException.prototype, GameException.prototype);
FriendException.prototype.__CLS__ = 'game.service.exception.FriendException';

FriendException.prototype.limit = null; // int
FriendException.prototype.getMessage = function()
{
	return I18.friendError(this.type);
}

//---------------------------------------------------------------------------
// GarrisonIsBlockedException
//---------------------------------------------------------------------------

function GarrisonIsBlockedException()
{
	callParentConstructor(GarrisonIsBlockedException.prototype, this);
}

extendClass(GarrisonIsBlockedException.prototype, GameException.prototype);
GarrisonIsBlockedException.prototype.__CLS__ = 'game.service.exception.GarrisonIsBlockedException';

//---------------------------------------------------------------------------
// GiftException
//---------------------------------------------------------------------------

function GiftException()
{
	callParentConstructor(GiftException.prototype, this);
}

extendClass(GiftException.prototype, GameException.prototype);
GiftException.prototype.__CLS__ = 'game.service.exception.GiftException';

GiftException.prototype.getMessage = function()
{
	return I18.giftError(this.type);
}

//---------------------------------------------------------------------------
// GodProtectionException
//---------------------------------------------------------------------------

function GodProtectionException()
{
	callParentConstructor(GodProtectionException.prototype, this);
}

extendClass(GodProtectionException.prototype, GameException.prototype);
GodProtectionException.prototype.__CLS__ = 'game.service.exception.GodProtectionException';

GodProtectionException.prototype.getMessage = function()
{
	if (this.type == "VICTIM") 
	{
		return I18.f('GODPROTECT_ERROR_VICTIM', NumberUtils.floor(FConfig.GOD_PROTECTION_VICTIM_TIMEOUT / 60 / 60 / 24));
	}
	return I18.godProtectionError(this.type);
}

//---------------------------------------------------------------------------
// HeroException
//---------------------------------------------------------------------------

function HeroException()
{
	callParentConstructor(HeroException.prototype, this);
}

extendClass(HeroException.prototype, GameException.prototype);
HeroException.prototype.__CLS__ = 'game.service.exception.HeroException';

HeroException.prototype.getMessage = function()
{
	return I18.heroError(this.type);
}

//---------------------------------------------------------------------------
// IllegalGateStateException
//---------------------------------------------------------------------------

function IllegalGateStateException()
{
	callParentConstructor(IllegalGateStateException.prototype, this);
}

extendClass(IllegalGateStateException.prototype, GameException.prototype);
IllegalGateStateException.prototype.__CLS__ = 'game.service.exception.IllegalGateStateException';

//---------------------------------------------------------------------------
// IllegalScienceStateException
//---------------------------------------------------------------------------

function IllegalScienceStateException()
{
	callParentConstructor(IllegalScienceStateException.prototype, this);
}

extendClass(IllegalScienceStateException.prototype, GameException.prototype);
IllegalScienceStateException.prototype.__CLS__ = 'game.service.exception.IllegalScienceStateException';

//---------------------------------------------------------------------------
// InvalidSMSCodeException
//---------------------------------------------------------------------------

function InvalidSMSCodeException()
{
	callParentConstructor(InvalidSMSCodeException.prototype, this);
}

extendClass(InvalidSMSCodeException.prototype, ClientKnowsException.prototype);
InvalidSMSCodeException.prototype.__CLS__ = 'game.service.exception.InvalidSMSCodeException';

//---------------------------------------------------------------------------
// KingdomNotFoundException
//---------------------------------------------------------------------------

function KingdomNotFoundException()
{
	callParentConstructor(KingdomNotFoundException.prototype, this);
}

extendClass(KingdomNotFoundException.prototype, GameException.prototype);
KingdomNotFoundException.prototype.__CLS__ = 'game.service.exception.KingdomNotFoundException';

KingdomNotFoundException.prototype.getMessage = function()
{
	return I18.f('KINGDOM_NOT_FOUND');
}

//---------------------------------------------------------------------------
// MailException
//---------------------------------------------------------------------------

function MailException()
{
	callParentConstructor(MailException.prototype, this);
}

extendClass(MailException.prototype, GameException.prototype);
MailException.prototype.__CLS__ = 'game.service.exception.MailException';

MailException.prototype.getMessage = function()
{
	return I18.mailError(this.type);
}

//---------------------------------------------------------------------------
// MarketException
//---------------------------------------------------------------------------

function MarketException()
{
	callParentConstructor(MarketException.prototype, this);
}

extendClass(MarketException.prototype, GameException.prototype);
MarketException.prototype.__CLS__ = 'game.service.exception.MarketException';

MarketException.prototype.getMessage = function()
{
	return I18.marketError(this.type);
}

//---------------------------------------------------------------------------
// MentorshipException
//---------------------------------------------------------------------------

function MentorshipException()
{
	callParentConstructor(MentorshipException.prototype, this);
}

extendClass(MentorshipException.prototype, GameException.prototype);
MentorshipException.prototype.__CLS__ = 'game.service.exception.MentorshipException';

MentorshipException.prototype.username = null; // String
MentorshipException.prototype.getMessage = function()
{
	return I18.mentorshipError(this.type, this.username);
}

//---------------------------------------------------------------------------
// MissionCancelException
//---------------------------------------------------------------------------

function MissionCancelException()
{
	callParentConstructor(MissionCancelException.prototype, this);
}

extendClass(MissionCancelException.prototype, GameException.prototype);
MissionCancelException.prototype.__CLS__ = 'game.service.exception.MissionCancelException';

//---------------------------------------------------------------------------
// MoveCapitalException
//---------------------------------------------------------------------------

function MoveCapitalException()
{
	callParentConstructor(MoveCapitalException.prototype, this);
}

extendClass(MoveCapitalException.prototype, GameException.prototype);
MoveCapitalException.prototype.__CLS__ = 'game.service.exception.MoveCapitalException';

MoveCapitalException.prototype.getMessage = function()
{
	return I18.moveCapitalError(this.type);
}

//---------------------------------------------------------------------------
// NoSuchUserException
//---------------------------------------------------------------------------

function NoSuchUserException()
{
	callParentConstructor(NoSuchUserException.prototype, this);
}

extendClass(NoSuchUserException.prototype, GameException.prototype);
NoSuchUserException.prototype.__CLS__ = 'game.service.exception.NoSuchUserException';

//---------------------------------------------------------------------------
// NotEnoughArmyException
//---------------------------------------------------------------------------

function NotEnoughArmyException()
{
	callParentConstructor(NotEnoughArmyException.prototype, this);
}

extendClass(NotEnoughArmyException.prototype, GameException.prototype);
NotEnoughArmyException.prototype.__CLS__ = 'game.service.exception.NotEnoughArmyException';

NotEnoughArmyException.prototype.getMessage = function()
{
	return I18.f('ARMY_DESYNC_MESSAGE');
}

//---------------------------------------------------------------------------
// NotEnoughResourcesException
//---------------------------------------------------------------------------

function NotEnoughResourcesException()
{
	callParentConstructor(NotEnoughResourcesException.prototype, this);
}

extendClass(NotEnoughResourcesException.prototype, GameException.prototype);
NotEnoughResourcesException.prototype.__CLS__ = 'game.service.exception.NotEnoughResourcesException';

NotEnoughResourcesException.prototype.getMessage = function()
{
	return I18.f('NOT_ENOUGH_RESOURCES');
}

//---------------------------------------------------------------------------
// NotEnoughUnitsCapacityException
//---------------------------------------------------------------------------

function NotEnoughUnitsCapacityException()
{
	callParentConstructor(NotEnoughUnitsCapacityException.prototype, this);
}

extendClass(NotEnoughUnitsCapacityException.prototype, GameException.prototype);
NotEnoughUnitsCapacityException.prototype.__CLS__ = 'game.service.exception.NotEnoughUnitsCapacityException';

//---------------------------------------------------------------------------
// ObsoleteClientException
//---------------------------------------------------------------------------

function ObsoleteClientException()
{
	callParentConstructor(ObsoleteClientException.prototype, this);
}

extendClass(ObsoleteClientException.prototype, GameException.prototype);
ObsoleteClientException.prototype.__CLS__ = 'game.service.exception.ObsoleteClientException';

//---------------------------------------------------------------------------
// PearlBoostException
//---------------------------------------------------------------------------

function PearlBoostException()
{
	callParentConstructor(PearlBoostException.prototype, this);
}

extendClass(PearlBoostException.prototype, GameException.prototype);
PearlBoostException.prototype.__CLS__ = 'game.service.exception.PearlBoostException';

PearlBoostException.prototype.buff = null; // FBuff
PearlBoostException.prototype.getMessage = function()
{
	return I18.pearlBoostError(this.type);
}

//---------------------------------------------------------------------------
// PromoExpiredException
//---------------------------------------------------------------------------

function PromoExpiredException()
{
	callParentConstructor(PromoExpiredException.prototype, this);
}

extendClass(PromoExpiredException.prototype, GameException.prototype);
PromoExpiredException.prototype.__CLS__ = 'game.service.exception.PromoExpiredException';

//---------------------------------------------------------------------------
// QuestRewardException
//---------------------------------------------------------------------------

function QuestRewardException()
{
	callParentConstructor(QuestRewardException.prototype, this);
}

extendClass(QuestRewardException.prototype, GameException.prototype);
QuestRewardException.prototype.__CLS__ = 'game.service.exception.QuestRewardException';

QuestRewardException.prototype.getMessage = function()
{
	return I18.questError(this.type);
}

//---------------------------------------------------------------------------
// RegistrationException
//---------------------------------------------------------------------------

function RegistrationException()
{
	callParentConstructor(RegistrationException.prototype, this);
}

extendClass(RegistrationException.prototype, ClientKnowsException.prototype);
RegistrationException.prototype.__CLS__ = 'game.service.exception.RegistrationException';

RegistrationException.prototype.availableUsernames = null;
RegistrationException.prototype.getMessage = function()
{
	return I18.s("REGISTRATION_ERROR", this.type);
}
RegistrationException.prototype.isUsernameWrong = function()
{
	return this.isUsernameAlreadyExists() || this.type == "WRONG_USERNAME";
}
RegistrationException.prototype.isUsernameAlreadyExists = function()
{
	return this.type == "ALREADY_EXISTS";
}
RegistrationException.prototype.isPasswordWrong = function()
{
	return this.type == "SIMPLE_PASSWORD" || this.type == "WRONG_PASSWORD";
}
RegistrationException.prototype.isCaptchaWrong = function()
{
	return this.type == "WRONG_CAPTCHA";
}
RegistrationException.prototype.isEmailWrong = function()
{
	return this.type == "ALREADY_EXISTS_EMAIL" || this.type == "WRONG_EMAIL";
}
RegistrationException.prototype.isReferalInfolWrong = function()
{
	return this.type == "WRONG_REF_INFO";
}

//---------------------------------------------------------------------------
// RegistrationRequiredException
//---------------------------------------------------------------------------

function RegistrationRequiredException()
{
	callParentConstructor(RegistrationRequiredException.prototype, this);
}

extendClass(RegistrationRequiredException.prototype, GameException.prototype);
RegistrationRequiredException.prototype.__CLS__ = 'game.service.exception.RegistrationRequiredException';

//---------------------------------------------------------------------------
// SaveInProgressException
//---------------------------------------------------------------------------

function SaveInProgressException()
{
	callParentConstructor(SaveInProgressException.prototype, this);
}

extendClass(SaveInProgressException.prototype, GameException.prototype);
SaveInProgressException.prototype.__CLS__ = 'game.service.exception.SaveInProgressException';

SaveInProgressException.prototype.timeout = null; // Number

//---------------------------------------------------------------------------
// SessionExpiredException
//---------------------------------------------------------------------------

function SessionExpiredException()
{
	callParentConstructor(SessionExpiredException.prototype, this);
}

extendClass(SessionExpiredException.prototype, GameException.prototype);
SessionExpiredException.prototype.__CLS__ = 'game.service.exception.SessionExpiredException';

//---------------------------------------------------------------------------
// SubscriptionExpiredException
//---------------------------------------------------------------------------

function SubscriptionExpiredException()
{
	callParentConstructor(SubscriptionExpiredException.prototype, this);
}

extendClass(SubscriptionExpiredException.prototype, GameException.prototype);
SubscriptionExpiredException.prototype.__CLS__ = 'game.service.exception.SubscriptionExpiredException';

//---------------------------------------------------------------------------
// this.templateException
//---------------------------------------------------------------------------

function TemplateException()
{
	callParentConstructor(TemplateException.prototype, this);
}

extendClass(TemplateException.prototype, GameException.prototype);
TemplateException.prototype.__CLS__ = 'game.service.exception.TemplateException';

TemplateException.prototype.params = null; // FMap
TemplateException.prototype.getBox = function()
{
	return I18.getMegaBox(this.type, this.params);
}
TemplateException.prototype.getMessage = function()
{
	return I18.getMegaString(this.type, this.params);
}

//---------------------------------------------------------------------------
// TooManyOrderedUnitsException
//---------------------------------------------------------------------------

function TooManyOrderedUnitsException()
{
	callParentConstructor(TooManyOrderedUnitsException.prototype, this);
}

extendClass(TooManyOrderedUnitsException.prototype, GameException.prototype);
TooManyOrderedUnitsException.prototype.__CLS__ = 'game.service.exception.TooManyOrderedUnitsException';

//---------------------------------------------------------------------------
// TournamentException
//---------------------------------------------------------------------------

function TournamentException()
{
	callParentConstructor(TournamentException.prototype, this);
}

extendClass(TournamentException.prototype, GameException.prototype);
TournamentException.prototype.__CLS__ = 'game.service.exception.TournamentException';

TournamentException.prototype.getMessage = function()
{
	return I18.tournamentError(this.type);
}

//---------------------------------------------------------------------------
// UserBannedException
//---------------------------------------------------------------------------

function UserBannedException()
{
	callParentConstructor(UserBannedException.prototype, this);
}

extendClass(UserBannedException.prototype, GameException.prototype);
UserBannedException.prototype.__CLS__ = 'game.service.exception.UserBannedException';

UserBannedException.prototype.till = null; // Number
UserBannedException.prototype.getMessage = function()
{
	return I18.getBanReason(this.type);
}

//---------------------------------------------------------------------------
// UsernameNotFoundException
//---------------------------------------------------------------------------

function UsernameNotFoundException()
{
	callParentConstructor(UsernameNotFoundException.prototype, this);
}

extendClass(UsernameNotFoundException.prototype, ClientKnowsException.prototype);
UsernameNotFoundException.prototype.__CLS__ = 'game.service.exception.UsernameNotFoundException';

//---------------------------------------------------------------------------
// VacationException
//---------------------------------------------------------------------------

function VacationException()
{
	callParentConstructor(VacationException.prototype, this);
}

extendClass(VacationException.prototype, GameException.prototype);
VacationException.prototype.__CLS__ = 'game.service.exception.VacationException';

VacationException.prototype.getMessage = function()
{
	return I18.vacationError(this.type);
}

//---------------------------------------------------------------------------
// CatapultException
//---------------------------------------------------------------------------

function CatapultException()
{
	callParentConstructor(CatapultException.prototype, this);
	this.username = '';
}

extendClass(CatapultException.prototype, GameException.prototype);
CatapultException.prototype.__CLS__ = 'game.service.exception.CatapultException';
CatapultException.prototype.username = '';

CatapultException.prototype.getMessage = function()
{
	return I18.getCatapultErrorMessage(this.type, this.username);
}

CatapultException.errorHandler = function(e)
{
	if (e instanceof CatapultException) 
	{
		Alert.create(e.getMessage(), I18.f('ERROR'));
		return true;
	}
	return false;
}

//---------------------------------------------------------------------------
// MoveCityException
//---------------------------------------------------------------------------

function MoveCityException()
{
    callParentConstructor(MoveCityException.prototype, this);
}

extendClass(MoveCityException.prototype, GameException.prototype);
MoveCityException.prototype.__CLS__ = 'game.service.exception.MoveCityException';

MoveCityException.prototype.getMessage = function()
{
    return I18.moveCityError(this.type);
}

//---------------------------------------------------------------------------
// MandatoryPaymentNotPayedException
//---------------------------------------------------------------------------

function MandatoryPaymentNotPayedException()
{
    callParentConstructor(MandatoryPaymentNotPayedException.prototype, this);
}

extendClass(MandatoryPaymentNotPayedException.prototype, GameException.prototype);
MandatoryPaymentNotPayedException.prototype.__CLS__ = 'game.service.exception.MandatoryPaymentNotPayedException';

//---------------------------------------------------------------------------
// MandatoryOffersException
//---------------------------------------------------------------------------

function MandatoryOffersException()
{
    callParentConstructor(MandatoryOffersException.prototype, this);
}

extendClass(MandatoryOffersException.prototype, GameException.prototype);
MandatoryOffersException.prototype.__CLS__ = 'game.service.exception.MandatoryOffersException';

