var Arrays = {}

Arrays.strContains = function(array, value)  
{
	return Arrays.contains(array, value);
}

Arrays.strSubtract = function(arr1, arr2) 
{
	return Arrays.subtract(arr1, arr2);
}

Arrays.subtract = function(arr1, arr2)   
{
    var res = [];
    for (var i = 0; i < arr1.length; i++)
    {
    	var element = arr1[i];
        if (!Arrays.contains(arr2, element)) 
        {
            res.push(element);
        }
    }
    return res;
}

// produces a new Array which contains values that present given two arrays
// result elements have the same order as "a1"
Arrays.intersectionOf = function(arr1, arr2) 
{
    var res = [];
    for (var i = 0; i < arr1.length; i++)
    {
    	var element = arr1[i];
        if (Arrays.contains(arr2, element)) 
        {
            res.push(element);
        }
    }
    return res;
}

Arrays.unionOf = function(arrays) 
{
    var arrays = Array.prototype.slice.call(arguments);
    var res = [];
    for (var a = 0; a < arrays.length; a++) 
    {
        for(var i = 0; i < arrays[a].length; i++) 
        {
            var obj = arrays[a][i];
            if (res.indexOf(obj) < 0) 
            {
                res.push(obj);
            }
        }
    }
    return res;
}

Arrays.clone = function(source) 
{
    return source.concat();
}

Arrays.antidot = function(values) 
{
    if (!values) 
    {
        return;
    }
    if (values.length > 0 && $.isArray(values[0])) 
    {
        return values[0];
    }
    return values;
}

Arrays.sort = function(source) 
{
    source.sort(function(item1, item2) 
    {
        if (typeof item1['compareTo'] == 'function')
        {
        	return item1.compareTo(item2);
        }
        else
        {
        	return (item1==item2) ? 0 : (item1>item2 ? 1 : -1);
        }
    });
}

Arrays.sorted = function(source) 
{
    var cloned = source.concat();
    Arrays.sort(cloned);
    return cloned;
}

Arrays.join = function(source, delimiter, f) 
{
	if (typeof delimiter == 'undefined') delimiter = "";
    var res = "";
    if (source) 
    {
        for (var i = 0; i < source.length; i++) 
        {
            res += (typeof f == 'undefined') ? source[i] : f(source[i]);
            if (i < source.length - 1) 
            {
                res += delimiter;
            }
        }
    }
    return res;
}

Arrays.map = function(source, f) 
{
    var res = [];
    for (var i = 0; i < source.length; i++) 
    {
        res.push(f(source[i]));
    }
    return res;
}

Arrays.foreach = function(source, f) 
{
    for (var i = 0; i < source.length; i++) 
    {
        f(source[i]);
    }
}

Arrays.firstWith = function(source, predicate) 
{
    for (var i = 0; i < source.length; i++)
    {
        if (predicate(source[i])) 
        {
            return source[i];
        }
    }
}

Arrays.count = function(source, predicate) 
{
    var res = 0;
    for (var i = 0; i < source.length; i++)
    {
        if (predicate(source[i])) 
        {
            res++;
        }
    }
    return res;
}

Arrays.filter = function(source, predicate) 
{
    var res = [];
    for (var i = 0; i < source.length; i++)
    {
        if (predicate(source[i]))
        { 
            res.push(source[i]);
        }
    }
    return res;
}

Arrays.filterCollection = function(source, predicate)
{
    return new ArrayCollection(Arrays.filter(source.toArray(), predicate));
}

Arrays.equals = function(arr1, arr2) 
{
    if (arr1.length != arr2.length) 
    {
        return false;
    }
    
    for (var i = 0; i < arr1.length; i++) 
    {
        if (arr1[i] !== arr2[i]) 
        {
            if ((typeof arr1[i]['compareTo'] != 'function') || (typeof arr2[i]['compareTo'] != 'function'))
            {
            	return false;
            }
            if (arr1[i].compareTo(arr2[i]) != 0)
            {
            	return false;
            }
        }
    }
    return true;
}

Arrays.zip = function(arr1, arr2) 
{
    if (arr1.length != arr2.length) 
    {
        throw new Error("Arrays has inconsistent length");
    }
    var res = [];
    for (var i = 0; i < arr1.length; i++) 
    {
        res.push([arr1[i], arr2[i]]);
    }
    return res;
}
Arrays.unzip = function(arr) 
{
    var arr1 = [];
    var arr2 = [];
    for (var i = 0; i < arr.length; i++) 
    {
        arr1.push(arr[i][0]);
        arr2.push(arr[i][1]);
    }
    return [arr1, arr2];
}

Arrays.range = function(max) 
{
    var res = [];
    for (var i = 0; i < max; i++) 
    {
        res.push(i);
    }
    return res;
}


Arrays.contains = function(array, value) 
{
    return Arrays.indexOf(array, value) != -1;
}

Arrays.indexOf = function(array, value) 
{
    for (var i = 0; i < array.length; i++) 
    {
        if (array[i] == value) 
        {
            return i;
        }
    }
    return -1;
}

Arrays.remove = function(array, obj) 
{
    return Arrays.filter(array, function(element) 
    {
        return element != obj;
    });
}

Arrays.fill = function(array, value) 
{
    for (var i = 0; i < array.length; i++) 
    {
        array[i] = value;
    }
}

Arrays.removeByIndex = function(array, index) 
{
    if (index < 0 || index >= array.length) 
    {
        throw new Error("index is out of bounds, must be in [0,"+(array.length - 1)+"}]");
    }
    array.splice(index, 1);
}
