function CombatBonusUtil()
{
	
}

CombatBonusUtil.calcBetrayerAttackBonus = function(attacker, defender)
{
    var raceBetrayingBonus = 0;
    if (attacker.isSameRace(defender)) 
    {
        raceBetrayingBonus += FConfig.combatConfig.RACE_ATTACK_BONUS;
    } 
    else if (attacker.isSameSide(defender)) 
    {
        raceBetrayingBonus += FConfig.combatConfig.SIDE_ATTACK_BONUS;
    }
    return raceBetrayingBonus;
}

CombatBonusUtil.calcAttackBonus = function(race, betrayer, terrain,  ut)
{
    var tc = FConfig.getTerrainConfig(terrain);
    return (betrayer ? 0 : CombatBonusUtil.getRaceBonus(tc, race).damage) + CombatBonusUtil.getUnitBonus(tc, ut).damage;
}

CombatBonusUtil.calcDefenceBonus = function(race, betrayer, terrain, ut)
{
    var tc = FConfig.getTerrainConfig(terrain);
    return (betrayer ? 0 : CombatBonusUtil.getRaceBonus(tc, race).defence) + CombatBonusUtil.getUnitBonus(tc, ut).defence;
}

CombatBonusUtil.getUnitBonus = function(tc, type)
{
    return tc.unitBonuses.containsKey(type) ? tc.unitBonuses.getValue(type) : new AuraBonus();
}

CombatBonusUtil.getRaceBonus = function(tc, race)
{
    return tc.raceBonuses.containsKey(race) ? tc.raceBonuses.getValue(race) : new AuraBonus();
}


CombatBonusUtil.getBonuses = function(race, isBetrayer, otherRace, isOtherBetrayer, isAttacker, terrain, fortificationLevels, isTargetNeutral, maxDefence)
{
	var result = [];
    var tc = FConfig.getTerrainConfig(terrain);

    var calcAsBetrayer = isAttacker && race.isSameSide(otherRace) && !isOtherBetrayer && !isTargetNeutral;

    var raceBonus = tc.raceBonuses.getValueOrNull(race);
    if (!raceBonus || calcAsBetrayer || isBetrayer) 
    {
        raceBonus = new AuraBonus();
    }

    var betrayerAttackBonus = 0;
    if (isAttacker && calcAsBetrayer) 
    {
        betrayerAttackBonus += CombatBonusUtil.calcBetrayerAttackBonus(race, otherRace);
    }

    var fortBonus = CombatBonusUtil.getFortificationBonuses(race, fortificationLevels);

    for (var i = 0; i < UnitType.MILITARY.length; i++) 
    {
    	var ut = UnitType[i];
        var bonus = tc.unitBonuses.getValueOrNull(ut);
        bonus = !bonus ? new AuraBonus() : bonus.copy();
        bonus.damage += betrayerAttackBonus;
        bonus.add(fortBonus);
        bonus.add(raceBonus);
        if (maxDefence) 
        {
            bonus.defence = NumberUtils.floor(Number.MAX_VALUE / 2);
        }
        result.push(bonus);
    }

    return result;
}

CombatBonusUtil.getFortificationBonuses = function(race, fortificationLevels)
{
	var fortBonus = new AuraBonus();
    if (fortificationLevels) 
    {
        var fortLevels = fortificationLevels.getValue(BType.FORTIFICATION);
        if (fortLevels.length > 0) 
        {
            var configLevels = BType.FORTIFICATION.getLevels(race);
            for (var i = 0; i < fortLevels.length; i++) 
            {
                var level = fortLevels[i]
                fortBonus.add(configLevels[level].getAuraBonus());
            }
        }
    }
    return fortBonus;
}

CombatBonusUtil.getTowersBonus = function(estate)
{
    return CombatBonusUtil.getTowersBonusForReport(estate.getScienceLevel(Science.SPEC_DEFENDER), estate.sumOfBuildingBuffs(BuffType.BOOST_BUILDING, BType.TOWER));
}

CombatBonusUtil.getTowersBonusForReport = function(defenderSpecLevel, buffBonusPercent)
{
    var result = 0;
    if (defenderSpecLevel > 0) 
    {
        result += DefenderLevelPrice(Science.SPEC_DEFENDER.getLevel(defenderSpecLevel - 1)).towersBonus / 100;
    }
    result += buffBonusPercent;
    return result;
}

CombatBonusUtil.getMagicTowersBonus = function(estate)
{
    return estate.sumOfBuildingBuffs(BuffType.BOOST_BUILDING, BType.MAGIC_TOWER);
}

CombatBonusUtil.getTowerAttack = function(bt, level, estate, shouldAddBonus)
{
	if (typeof shouldAddBonus == 'undefined') shouldAddBonus = true;
    if (shouldAddBonus) 
    {
        return CombatBonusUtil.getTowerAttackWithBonuses(bt, level, estate.race, 
        	CombatBonusUtil.getTowersBonus(estate),
        	CombatBonusUtil.getMagicTowersBonus(estate),
        	CombatBonusUtil.getGarrisonAttackForMagicTower(estate)
        );
    }
    var blt = bt.getLevels(estate.race)[level];
    if (BType.MAGIC_TOWER.equals(bt)) 
    {
        return Range.single(blt.garrisonAttackBonus * CombatBonusUtil.getGarrisonAttackForMagicTower(estate)).floor();
    }
    return blt.attack.copy();
}

CombatBonusUtil.getTowerAttackWithBonuses = function(bt, level, r, towerBonus, magicTowerBonus, garrisonAttack)
{
    var blt = bt.getLevels(r)[level];
    var attack = blt.attack.copy();
    if (BType.TOWER.equals(bt)) 
    {
        attack.multiply(1. + towerBonus);
    } 
    else 
    {
        attack.add(Range.single(blt.garrisonAttackBonus * garrisonAttack));
        attack.floor();
        attack.multiply(1. + magicTowerBonus);
    }
    attack.floor();
    return attack;
}


CombatBonusUtil.getGarrisonAttackForMagicTower = function(estate)
{
    var garrisonAttack = 0;
    
    for (var i = 0; i < UnitType.MILITARY.length; i++) 
    {
    	var mut = UnitType.MILITARY[i];
        var count = estate.garrison.getValue(mut);
        var stat = FConfig.getStat(estate.owner.race, mut, estate.science.levelOf(mut));
        var bonus = CombatBonusUtil.getAffectingAura(mut, estate.science.levelOf(mut), estate.owner.race, estate);
        var bs = new BuffSummator(estate.buffs, mut, FEstate.tickNowForEstate(estate), true);
        var damageBonus = Math.max(FConfig.MIN_DAMAGE_PERCENT - 1, CombatBonusUtil.getDamageBonus(bonus, bs));
        garrisonAttack += ((stat.damage.max + stat.damage.min) * (1.0 + damageBonus) / 2) * count;
    }
    return garrisonAttack;
}

CombatBonusUtil.getDamageBonus = function(bonus, bs, round)
{
    if (typeof round == 'undefined') round = 1;
    return bonus.damage / 100 + bs.sum(BuffType.ATTACK_BONUS) + Math.max(0, round - 1) * bs.sum(BuffType.ENRAGE);
}

CombatBonusUtil.getAffectingAura = function(type, level, race, estate)
{
    var bonus = estate.getBonus(type);
    var stat = FConfig.getStat(race, type, level);
    var maxDefence = Math.min(FConfig.MAX_MAX_DEFENCE, FConfig.MAX_DEFENCE + NumberUtils.floor(estate.sumOfBuffs(BuffType.INCREASE_MAX_DEFENCE)));
    if (bonus.defence + stat.defence > maxDefence) 
    {
        bonus.defence -= bonus.defence + stat.defence - maxDefence;
    }
    return bonus;
}

CombatBonusUtil.getGarrisonAttackForCombatReport = function(r, levels, bonuses, garrison, buffs, time)
{
    var garrisonAttack = 0;
    for (var i = 0; i < UnitType.MILITARY.length; i++) 
    {
        var mut = UnitType.MILITARY[i];
        var count = garrison[i];
        if (count > 0) 
        {
            var stat = FConfig.getStat(r, mut, levels[i]);
            garrisonAttack += ((stat.damage.max + stat.damage.min) * (1.0 + bonuses[i].damage / 100 + FBuff.sumOfBuffs(buffs, BuffType.ATTACK_BONUS, mut, time)) / 2) * count;
        }
    }
    return garrisonAttack;
}
