function DateUtils()
{
}

DateUtils.TODAY_DATE_FORMAT = "HH:MM";
DateUtils.ANOTHER_DAY_DATE_FORMAT = "DD MMM HH:MM";
DateUtils.TODAY_DATE_FORMAT_WITH_SECONDS = "HH:MM:SS";
DateUtils.ANOTHER_DAY_DATE_FORMAT_WITH_SECONDS = "DD MM HH:MM:SS";
DateUtils.ANOTHER_YEAR_DATE_FORMAT = "DD MMM YYYY HH:MM";
DateUtils.SERVER_DATE_FORMAT = "YYYY-MM-DD HH:MM:SS";
DateUtils.PORTALKA_DATE_SHORT = "DD.MM.YYYY";
DateUtils.PORTALKA_DATE = "DD.MM.YY HH:MM";
DateUtils.PORTALKA_DATE_LONG = "DD.MM.YYYY HH:MM:SS";

DateUtils.TODAY_DATE_FORMAT = "JJ:NN";
DateUtils.ANOTHER_DAY_DATE_FORMAT = "DD MMM JJ:NN";
DateUtils.TODAY_DATE_FORMAT_WITH_SECONDS = "JJ:NN:SS";
DateUtils.ANOTHER_DAY_DATE_FORMAT_WITH_SECONDS = "DD MMM JJ:NN:SS";
DateUtils.ANOTHER_YEAR_DATE_FORMAT = "DD MMM YYYY JJ:NN";
DateUtils.SERVER_DATE_FORMAT = "YYYY-MM-DD JJ:NN:SS";
DateUtils.PORTALKA_DATE_SHORT = "DD.MM.YYYY";
DateUtils.PORTALKA_DATE = "DD.MM.YY JJ:NN";
DateUtils.PORTALKA_DATE_LONG = "DD.MM.YYYY JJ:NN:SS";

DateUtils.TIMEZONE_OFFSET = 4 * 60 * 60 * 1000;

DateUtils.compare = function(d1/*:Date*/, d2/*:Date*/)
{
    if (d1.getTime() == d2.getTime()) 
    {
        return 0;
    }
    return d1.getTime() > d2.getTime() ? 1 : -1;
}

DateUtils.formatWithSeconds = function(ms/*:Number*/, ignoreDate/*:Boolean = false*/)
{
    var d = DateUtils.getDisplayServerDate(ms);
    var formatString = (ignoreDate || DateUtils.isToday(d)) ? DateUtils.TODAY_DATE_FORMAT_WITH_SECONDS : DateUtils.ANOTHER_DAY_DATE_FORMAT_WITH_SECONDS;
    return DateUtils.format(ms, formatString);
}

DateUtils.happensAt = function(ms/*:Number*/)
{
	var d = DateUtils.getDisplayServerDate(ms);
    var formatString = DateUtils.isToday(d) ? I18.f("HAPPENS_TODAY_FORMAT") : I18.f("HAPPENS_ANOTHER_DAY_FORMAT");
    return DateUtils.format(ms, formatString);
}

DateUtils.getDisplayServerDate = function(serverTickMs/*:Number*/)
{
    var res = new Date();
    var offset = res.getTimezoneOffset();
    var serverMs = serverTickMs + DateUtils.TIMEZONE_OFFSET + offset * 60 * 1000;
    res.setTime(serverMs);
    return res;
}

DateUtils.flexDateFormatter = function(date, format)
{
	// http://help.adobe.com/en_US/flex/using/WS8b1c39bd7e9fc364-70b5c6d212c98ca85e2-7fff.html#WS2db454920e96a9e51e63e3d11c0bf648c2-7ff9
	
	var d = date;//, res = format + '', reg;
	
	if (!(date instanceof Date))
	{
		d = new Date();
		d.setTime(date*1);
	}

	var tokens = ['Y','M','D','E','A','J','H','K','L','N','S','Q'];
	var token = '', tokenLength = 0;
	var shortMonths = I18.f("SHORT_MONTHS_NAMES").split('|');
	
	var padZero = function(str, len)
	{
		for (var i = (str+'').length; i < len; i++) str = '0' + str + '';
		return str; 
	}
	
	var processToken = function(token, length, d)
	{
		var res = '', len;
		if (length < 1) return res;
		
		while (length > 0)
		{
			if (token == 'Y')
			{
				if (length >= 4) { res += padZero(d.getFullYear(), length); length = 0; }
				if (length >= 2) { res += (d.getFullYear()+'').substr(2,2); length -= 2; }
				if (length > 0) { res += token; length--; }
			}
			
			if (token == 'M')
			{
				if (length >= 4) { res += shortMonths[d.getMonth()]; length -= 4; }
				else if (length == 3) { res += shortMonths[d.getMonth()]; length -= 3; }
				else if (length >= 1) { res += padZero(d.getMonth()+1, length); length = 0; }
			}
			
			if (token == 'D')
			{
				if (length >= 2) { res += padZero(d.getDate(), 2); length -= 2; }
				else if (length == 1) { res += d.getDate(); length -= 1; }
			}

			if (token == 'D')
			{
				if (length >= 2) { res += padZero(d.getDate(), 2); length -= 2; }
				else if (length == 1) { res += d.getDate(); length -= 1; }
			}
			
			if (token == 'E')
			{
				if (length >= 4) { res += d.getDay(); length -= 4; }
				else if (length == 3) { res += d.getDay(); length -= 3; }
				else if (length == 2) { res += padZero(d.getDay(), 2); length -= 2; }
				else if (length == 1) { res += d.getDay(); length -= 1; }
			}
			
			if (token == 'A')
			{
				length = 0; // not implemented
			}
			
			if (token == 'J' || token == 'K')
			{
				res += padZero(d.getHours(), length);
				length = 0;
			}

			if (token == 'H' || token == 'L')
			{
				res += padZero(d.getHours()+1, length);
				length = 0;
			}

			if (token == 'N')
			{
				if (length >= 2) { res += padZero(d.getMinutes(), 2); length -= 2; }
				else if (length == 1) { res += d.getMinutes(); length -= 1; }
			}

			if (token == 'S')
			{
				if (length >= 2) { res += padZero(d.getSeconds(), 2); length -= 2; }
				else if (length == 1) { res += d.getSeconds(); length -= 1; }
			}

			if (token == 'Q')
			{
				if (length >= 2) { res += padZero(d.getSeconds(), length); length = 0; }
				else if (length == 1) { res += d.getSeconds(); length -= 1; }
			}
		}
		
		return res;
	}
	
	var str = '';
	for (var i = 0; i < format.length; i++)
	{
		var ch = format.charAt(i);
		if (tokens.indexOf(ch) < 0)
		{
			str += processToken(token, tokenLength, d);
			token = ''; tokenLength = 0;
			str += ch;
			continue;
		}
		else
		{
			if (token == ch)
			{
				tokenLength++;
			}
			else
			{
				str += processToken(token, tokenLength, d);
				token = ch; tokenLength = 1;
			}
			if (i == format.length - 1)
			{
				str += processToken(token, tokenLength, d);
				token = ''; tokenLength = 0;
			}
		}
	}
	
	return str;
	
	/*
	reg = new RegExp('[Y]{4,}', 'g');	
	while (reg.test(res)) res = res.replace(reg, function(m){ return padZero(d.getFullYear(), m.length); });
	reg = new RegExp('[Y]{2}', 'g');	
	while (reg.test(res)) res = res.replace(reg, function(m){ return (d.getFullYear()+'').substr(2,2); });

	reg = new RegExp('[M]{4}', 'g');	
	while (reg.test(res)) res = res.replace(reg, function(m){ return shortMonths[d.getMonth()]; });
	reg = new RegExp('[M]{3}', 'g');	
	while (reg.test(res)) res = res.replace(reg, function(m){ return shortMonths[d.getMonth()]; });
	reg = new RegExp('[M]{2}', 'g');	
	while (reg.test(res)) res = res.replace(reg, function(m){ return padZero(d.getMonth()+1, 2); });
	reg = new RegExp('[M]{1}', 'g');	
	while (reg.test(res)) res = res.replace(reg, function(m){ return d.getMonth()+1; });

	reg = new RegExp('[D]{2}', 'g');	
	while (reg.test(res)) res = res.replace(reg, function(m){ return padZero(d.getDate(), 2); });
	reg = new RegExp('[D]{1}', 'g');	
	while (reg.test(res)) res = res.replace(reg, function(m){ return d.getDate(); });

	reg = new RegExp('[E]{4}', 'g');	
	while (reg.test(res)) res = res.replace(reg, function(m){ return d.getDay(); });
	reg = new RegExp('[E]{3}', 'g');	
	while (reg.test(res)) res = res.replace(reg, function(m){ return d.getDay(); });
	reg = new RegExp('[E]{2}', 'g');	
	while (reg.test(res)) res = res.replace(reg, function(m){ return padZero(d.getDay(), 2); });
	reg = new RegExp('[E]{1}', 'g');	
	while (reg.test(res)) res = res.replace(reg, function(m){ return d.getDay(); });

	reg = new RegExp('[A]{1}', 'g');	
	while (reg.test(res)) res = res.replace(reg, function(m){ return ''; });

	reg = new RegExp('[J|K]{1,}', 'g');	
	while (reg.test(res)) res = res.replace(reg, function(m){ return padZero(d.getHours(), m.length); });
	reg = new RegExp('[H|L]{1,}', 'g');	
	while (reg.test(res)) res = res.replace(reg, function(m){ return padZero(d.getHours()+1, m.length); });

	reg = new RegExp('[N]{2}', 'g');
	while (reg.test(res)) res = res.replace(reg, function(m){ return padZero(d.getMinutes(), 2); });
	reg = new RegExp('[N]{1}', 'g');
	while (reg.test(res)) res = res.replace(reg, function(m){ return d.getMinutes(); });

	reg = new RegExp('[S]{2}', 'g');
	while (reg.test(res)) res = res.replace(reg, function(m){ return padZero(d.getSeconds(), 2); });
	reg = new RegExp('[S]{1}', 'g');
	while (reg.test(res)) res = res.replace(reg, function(m){ return d.getSeconds(); });

	reg = new RegExp('[Q]{2,}', 'g');
	while (reg.test(res)) res = res.replace(reg, function(m){ return padZero((d.getTime() % 1000), m.length); });
	reg = new RegExp('[Q]{1}', 'g');
	while (reg.test(res)) res = res.replace(reg, function(m){ return (d.getTime() % 1000); });

	return res;
	*/
}

DateUtils.format = function(ms/*:Number*/, formatString/*:String*/)
{
	var res = DateUtils.getDisplayServerDate(ms);
	//console.log(res, formatString);
	//return res.format(formatString);
	return DateUtils.flexDateFormatter(res, formatString);
}

DateUtils.isToday = function(date/*:Date*/)
{
    return DateUtils.isSameDay(date, DateUtils.getNow());
}

DateUtils.getNow = function()
{
    return DateUtils.getDisplayServerDate(Tick.toMs(Tick.now()));
}

DateUtils.isSameDay = function(date/*:Date*/, curDate/*:Date*/)
{
    var copy = new Date();
    copy.setTime(curDate.getTime());
    copy.setHours(0, 0, 0, 0);
    var msInDay = 1000 * 60 * 60 * 24;
    return (date.getTime() - copy.getTime() < msInDay &&
            date.getTime() >= copy.getTime());
}

DateUtils.formatWithDate = function(ms/*:Number*/)
{
    if (DateUtils.getDisplayServerDate(ms).getFullYear() == DateUtils.getNow().getFullYear()) 
    {
        return DateUtils.format(ms, DateUtils.ANOTHER_DAY_DATE_FORMAT);
    }
    return DateUtils.format(ms, DateUtils.ANOTHER_YEAR_DATE_FORMAT);
}

DateUtils.formatWithDateAndSeconds = function(ms/*:Number*/)
{
    return DateUtils.format(ms, DateUtils.ANOTHER_DAY_DATE_FORMAT_WITH_SECONDS);
}

DateUtils.formatWithYear = function(ms/*:Number*/)
{
    return DateUtils.format(ms, DateUtils.ANOTHER_YEAR_DATE_FORMAT);
}

DateUtils.formatSmart = function(ms/*:Number*/)
{
    var d = DateUtils.getDisplayServerDate(ms);
    var now = DateUtils.getNow();
    if (d.getFullYear() == now.getFullYear()) 
    {
        if (d.getMonth() == now.getMonth()) 
        {
            if (d.getDate() == now.getDate()) 
            {//day of month (1..31)
                return DateUtils.format(ms, DateUtils.TODAY_DATE_FORMAT);
            }
        }
        return DateUtils.format(ms, DateUtils.ANOTHER_DAY_DATE_FORMAT);
    }
    return DateUtils.format(ms, DateUtils.ANOTHER_YEAR_DATE_FORMAT);
}

DateUtils.monthsFrom = function(c/*:Date*/, months/*:int*/)
{
    var d = new Date();
    d.month += months;
    return d;
}

DateUtils.daysFrom = function(c/*:Date*/, days/*:int*/)
{
    var d = new Date(c);
    d.date += days;
    return d;
}

DateUtils.endOfDay = function(d/*:Date*/)
{
    var result = new Date(d);
    result.setHours(23);
    result.setMinutes(59);
    result.setSeconds(59);
    result.setMilliseconds(999);
    return result;
}

DateUtils.startOfDay = function(d/*:Date*/)
{
    var result = new Date(d);
    result.setHours(0);
    result.setMinutes(0);
    result.setSeconds(0);
    result.setMilliseconds(0);
    return result;
}

DateUtils.formatForServer = function(d/*:Date*/)
{
    return DateUtils.flexDateFormatter(d, DateUtils.SERVER_DATE_FORMAT);
    //return d.format(DateUtils.SERVER_DATE_FORMAT);
}

DateUtils.formatForPortalka = function(d/*:Date*/)
{
    return DateUtils.flexDateFormatter(d, DateUtils.PORTALKA_DATE);
    //return d.format(DateUtils.PORTALKA_DATE);
}

DateUtils.formatForPortalkaMs = function(ms/*:Number*/)
{
    return DateUtils.formatForPortalka(DateUtils.getDisplayServerDate(ms));
}

DateUtils.formatForPortalkaLong = function(d/*:Date*/)
{
    return DateUtils.flexDateFormatter(d, DateUtils.PORTALKA_DATE_LONG);
    //return d.format(DateUtils.PORTALKA_DATE_LONG);
}

DateUtils.formatForPortalkaLongMs = function(ms/*:Number*/)
{
    return DateUtils.formatForPortalkaLong(DateUtils.getDisplayServerDate(ms));
}

DateUtils.formatForPortalkaShort = function(d/*:Date*/)
{
    return DateUtils.flexDateFormatter(d, DateUtils.PORTALKA_DATE_SHORT);
    //return d.format(DateUtils.PORTALKA_DATE_SHORT);
}

DateUtils.formatForPortalkaShortMs = function(ms/*:Number*/)
{
    return DateUtils.formatForPortalkaShort(DateUtils.getDisplayServerDate(ms));
}