function FMap()
{
	this._entries = [];
	
	this.__defineGetter__('raw', this.getRaw);
	
	this.__defineGetter__('keys', function()
	{
        var res = [];
        var entries = this.raw;
        for (var i in entries) 
        {
            if (typeof entries[i]['key'] != 'undefined' && entries[i]['key'] !== null) 
            {
                res.push(entries[i]['key']);
            }
        }
        return res;
	});

	this.__defineGetter__('values', function()
	{
        var res = [];
        var entries = this.raw;
        for (var i in entries) 
        {
            if (typeof entries[i]['key'] != 'undefined' && entries[i]['key'] !== null) 
            {
                res.push(entries[i].value);
            }
        }
        return res;
	});
	
	this.__defineGetter__('entries', function()
	{
        var res = [];
        var entries = this.raw;
        for (var i in entries) 
        {
            if (typeof entries[i]['key'] != 'undefined' && entries[i]['key'] !== null) 
            {
                res.push([entries[i].key, entries[i].value]);
            }
        }
        return res;
	});
	
	this.__defineGetter__('size', function()
	{
        var res = 0;
        var entries = this.raw;
        for (var i in entries) 
        {
            if (typeof entries[i]['key'] != 'undefined' && entries[i]['key'] !== null) 
            {
                res++;
            }
        }
        return res;
	});
};

FMap.prototype.__CLS__ = "game.util.FMap";

FMap.prototype.getEntry = function(key)
{
	var entries = this.raw;
	for (var i in entries)
	{
		var e = entries[i];
		if ((typeof e != 'object') || (typeof e['key'] == 'undefined'))
		{
			continue;
		}
		
		var k1 = (typeof key == 'object' && key['name']) ? key['name'] : key;
		var k2 = (typeof e['key'] == 'object' && e['key']['name']) ? e['key']['name'] : e['key'];
		
		if (k1 == k2)
		{
			return e;
		}
	}
	return null;
}

FMap.prototype.getValue = function(key)
{
	var e = this.getEntry(key);
	return e ? e.value : null;
}

FMap.prototype.getValueOrNull = function(key)
{
	var entry = this.getEntry(key);
	return entry ? entry.value : null;
}

FMap.prototype.getValueOrDefault = function(key, defaultValue)
{
	var entry = this.getEntry(key);
	return entry ? entry.value : defaultValue;
}

FMap.prototype.put = function(key, value)
{
	var item = this.getEntry(key);
	if(item)
	{
		item.value = value;
	}
	else
	{
		this.raw.push(
		{
			"key" : key,
			"value" : value
		});
	}
}

FMap.prototype.remove = function(key)
{
	for (var i = 0; i < this._entries.length; i++)
	{
		if (this._entries[i] && this._entries[i]['key'] === key)
		{
			this._entries.splice(i, 1);
			i--;
		}
	}
}

FMap.prototype.getRaw = function()
{
	return this._entries.source ? this._entries.source : this._entries;
}

FMap.prototype.containsKey = function(key)
{
	return this.getValue(key) !== null;
}
	
FMap.prototype.isEmpty = function()
{
	return this.raw.length == 0;
}

FMap.groupBy = function (source, keyFactory)
{
    var map = new FMap();
    for (var i=0; i<source.length; i++)
    {
    	var key = keyFactory(source[i]);
        var group = map.getValueOrNull(key);
        if (group == null) 
        {
            group = [];
            map.put(key, group);
        }
        group.push(source[i]);
    }
    return map;
}

