 var StringUtil =
{
	whitespaceChars : "\s\t\n\r\0\x0B ",

	trim : function(str)
	{
		return StringUtil.ltrim(StringUtil.rtrim(str));
	},

	ltrim : function(str)
	{
		return str.replace(new RegExp("^[" + StringUtil.whitespaceChars + "]+", "g"), "");
	},

	rtrim : function(str)
	{
		return str.replace(new RegExp("[" + StringUtil.whitespaceChars + "]+$", "g"), "");
	},

	trimArrayElements : function(value, delimiter)
	{
		if (value != "" && value != null)
		{
			var items = value.split(delimiter);

			var len = items.length;
			for (var i = 0; i < len; i++)
			{
				items[i] = StringUtil.trim(items[i]);
			}

			if (len > 0)
			{
				value = items.join(delimiter);
			}
		}

		return value;
	},

	isWhitespace : function(character)
	{
		return StringUtil.whitespaceChars.indexOf(character) >= 0;
	},

	substitute : function(str)
	{
		var rest = Array.prototype.slice.call(arguments, 1);
		var len = rest.length;
		var args = [];
		if (len == 1 && typeof rest[0] == "object")
		{
			args = rest[0];
			len = args.length;
		}
		else
		{
			args = rest;
		}
		
		for (var i = 0; i < len; i++)
		{
			str = str.replace(new RegExp("\\{" + i + "\\}", "g"), args[i]);
		}

		return str;
	}
};

var I18 =
{
	strings : {"OK": "OK"},
	loaded: false,

	NOWHERE: "[--- --- ---]",

	applyPatch: function(patch)
	{
		I18.strings = $.extend(I18.strings, patch);
	},

	load : function(data)
	{
		//I18.strings = {};
		var en = data['_entries'];
		if (en)
		{
			for (var i = 0; i < en.length; i++) I18.strings[en[i].key] = en[i].value;
		} 
		else
		{
			for (var i in data) I18.strings[i] = data[i];
		}
		if (DEBUG_MODE.OVERRIDE_I18) I18.applyPatch(LOCALE_STRINGS_MOBILE);
		I18.loaded = true;
	},

	//////////////////// Utils /////////////////////

	getString : function(key, values)
	{
		if ( typeof values == "undefined")
		{
			values = null;
		}

		var str = I18.getStringOrNull(key, values);
		if (str == null)
		{
			return "{" + key + "}";
		}
		
		return str;
	},

	getStringOrNull : function(key, args)
	{
		
		if ( typeof args == "undefined")
		{
			args = null;
		}

		var value = I18.strings[key];
		if ( typeof value == "undefined")
		{
			value = null;
		}

		return (args == null || value == null) ? value : StringUtil.substitute(value, Arrays.antidot(args));
	},

	f : function(key)
	{
		var values = Arrays.antidot(Array.prototype.slice.call(arguments, 1));
		
		if (!$.isArray(values))
		{
			values = [values];
		}
		return I18.getString(key, values);
	},

	s : function(prefix, key, values)
	{
		if (!$.isArray(values))
		{
			values = [values];
		}
		return I18.getString(prefix + "_" + key, Arrays.antidot(values));
	},

	sf : function(key, suffix, values)
	{
		return I18.getString(key + "_" + suffix, Arrays.antidot(values));
	},

	psf : function(prefix, key, suffix, values, temp)
	{
		return I18.getString(prefix + "_" + key + "_" + suffix, Arrays.antidot(values));
	},

	htmlFlow : function(html)
	{
		return html;
	},

	flow : function()
	{
		var args = Array.prototype.slice.call(arguments, 0);
		return I18.htmlFlow(I18.f.apply(null, args));
	},

	//////////////////// Game strings /////////////////////

	getFractionName : function(race)
	{
		return I18.s("FRACTION", race.isSameSide(Race.HUMAN) ? "LIGHT" : "DARK");
	},

	fixEstateName : function(name)
	{
		switch (name)
		{
			case "%CITY_DEFAULT_NAME%":
				return I18.f("CITY_DEFAULT_NAME");
			case "%PIT_NAME%":
				return I18.f("PIT_NAME");
			case "%RUIN_NAME%":
				return I18.f("RUIN_NAME");
			case "%LAKE_NAME%":
				return I18.f("LAKE_NAME");
			default:
				return name;
		}
	},

	getBuildingName : function(bt, race, res)
	{
		var bt = bt.name ? bt.name : bt;
		
		switch (bt)
		{
			case "NULL":
				return I18.f("UNKNOWN_BUILDING");
			case "PITHALL":
				return I18.f("BUILDING_PITHALL");
			case "RUINHALL":
				return I18.f("BUILDING_RUINHALL");
			case "LAKEHALL":
				return I18.f("BUILDING_LAKEHALL");
			case "GRAILHALL":
                return I18.f("BUILDING_GRAILHALL");
			case "CAMPHALL":
				return I18.f("BUILDING_CAMPHALL");
			case "GROTTOHALL":
				return I18.f("BUILDING_GROTTOHALL");
			case "ABANDONED_MERCHANT":
				return I18.f("BUILDING_ABANDONED_MERCHANT");
			case "MINEHALL":
				return I18.getMineName(res);
			case "STONEHENGEHALL":
			    return I18.f("BUILDING_STONEHENGEHALL");
			case "CITYHALL":
				return I18.s("BUILDING", bt + "_" + race.name);
			default:
				var special = I18.getStringOrNull("BUILDING_" + bt + "_" + race.name);
				return special != null ? special : I18.s("BUILDING", bt + "_COMMON");
		}
	},

	getMineName : function(res)
	{
		if (!res)
		{
			res = "IRON";
		}
		
		return I18.s("MINE_NAME", res.name ? res.name : res);
	},

	getBWhereIn : function(name, params)
	{
		return I18.s("BUILDING_CONSTRAINT_WHERE_IN", name.toUpperCase(), params);
	},

	getBWhereOn : function(name, params)
	{
		return I18.s("BUILDING_CONSTRAINT_WHERE_ON", name.toUpperCase(), params);
	},

	getConcurrentLimitText : function(limit)
	{
		return I18.f("BUILDING_CONSTRAINT_CONCURRENT", limit);
	},
	
	getScience: function(science)
	{
        return I18.s("SCIENCE", science.name);

    },
	
	allianceError: function(type, limit)
	{
        return I18.getMegaString(I18.s("ALLIANCE_ERROR", type, limit));
    },
		
	getUnitClassName: function (ut)
	{
        return I18.s("UNIT", ut.name);
    },
	
	getSpellTargetTypeName: function (name)
	{
        return I18.s("SPELL_TARGET_TYPE", name);
    },
	
	getRaiserBoostPhrase: function (type)
	{
        return I18.s("BOOST_RAISER", type.name);
    },
	
	getEstateTypeName: function (t)
	{
        return I18.s("ESTATE_TYPE", t ? t.name : "");
   	},
    
    getSpyLevelDescription: function(level)
	{
		return I18.s("SPY_LEVEL", level.toString());
	},

    getSpecialUnitDescription: function(ut)
	{
		return I18.s("SPECIAL_UNIT", ut.name);
	},
	
	getScience: function (science)
	{
        return I18.s("SCIENCE", science.name);
    },					

	noFreeUnitsMessage: function(type/*:UnitType*/)
	{
        return I18.s("MISSION_NO_FREE", type.name);
	},

    specialUnitsInUnavailableSectorMessage: function(type/*:UnitType*/)
    {
        return I18.s("MISSION_SECTOR_UNAVAILABLE", type.name);
    },

    selectedNoneUnitsMessage: function (type/*:UnitType*/)
    {
        return I18.s("MISSION_NOT_SELECTED", type.name);
    },

    actionGoal: function (type/*:UnitType*/)
    {
        return I18.s("GOAL", type.name);
    },

    targetLocatedTooFar: function(type/*:UnitType*/)
    {
        return I18.s("TARGET_LOCATED_TOO_FAR", type.name);
    },
	
	getChangeSkinText: function(race/*:Race*/)
    {
    	return I18.s("CHANGE_SKIN_TEXT", race.name);
    },

    getInstantlyFinishUnitText:function(utype/*:UnitType*/)
    {
        return I18.s("INSTANTLY_FINISH", utype.name);
    },
	
	getHexViewTip: function(name/*:String*/) 
	{
        return I18.s("HEX_VIEW_TIP", name);
   	},
   	
   	getOverfilledAlertTitle: function (bType/*:BType*/)
	{
        if (bType != BType.HOUSES) 
        {
            bType = BType.WAREHOUSE;
        }
        return I18.s("OVERFILLED_ALERT_TITLE", bType.toString())
    },
   	
   	getResourceName: function(r/*:Resource*/)
	{
        return I18.s("RESOURCE", r.name);
	},
   
	getWorkers: function(bt/*:BType*/)
	{
        var s = I18.getStringOrNull("WORKERS_" + bt.name);
        return s != null ? s : I18.getString("WORKERS");
	},

    getBWhereIn: function(name)
	{
        var params = Array.prototype.slice.call(arguments, 1);
        return I18.s("BUILDING_CONSTRAINT_WHERE_IN", name.toUpperCase(), params);
    },

    getBWhereOn: function(name) 
    {
        var params = Array.prototype.slice.call(arguments, 1);
        return I18.s("BUILDING_CONSTRAINT_WHERE_ON", name.toUpperCase(), params);
    },
    
    
    getGiftName: function(giftName/*:String*/) 
    {
	    var result = I18.getStringOrNull("GIFT_" + giftName.toUpperCase());
	    if (result != null) 
	    {
	        return result;
	    }
	    return I18.getString("GIFT_MEDAL");
    },

	getTitleName: function(titleIndex/*:int*/) 
	{
        return I18.s("TITLE_NAME", "" + titleIndex);
	},
///////////////////
	
	
    getOuterEstateName: function(type/*:EstateType*/, resources/*:Array*/)
	{
        switch (type.name) 
        {
            case EstateType.MINE.name:
            	return I18.getMineName(resources[0]);
            default:
            	return I18.getEstateTypeName(type);
        }
	},

    getEntranceStateText: function(isOuterEstate/*:Boolean*/, state/*:String*/)
	{
        if (isOuterEstate) 
        {
            return I18.s("TUNNEL", state);
        } 
        else 
        {
            return I18.s("GATE", state);
        }
    },

	getExtractorExtractsText: function(r/*:Resource*/)
	{
        switch (r) 
        {
            case Resource.STONE:            return I18.f("C_B_TIP_EXTRACTOR_EXTRACTS_STONE");
            case Resource.GOLD:             return I18.f("C_B_TIP_EXTRACTOR_EXTRACTS_GOLD");
            case Resource.GRAIN:            return I18.f("C_B_TIP_EXTRACTOR_EXTRACTS_GRAIN");
            case Resource.IRON:             return I18.f("C_B_TIP_EXTRACTOR_EXTRACTS_IRON");
            case Resource.WOOD:             return I18.f("C_B_TIP_EXTRACTOR_EXTRACTS_WOOD");
            default: console.log("Unknown resource: " + r);
        }
	},
   
	getKingdomOuterEstateTip: function(r/*:Race*/, type/*:EstateType*/, resources/*:Array*/)
	{
        switch (type) 
        {
            case EstateType.PIT:        return I18.s("KINGDOM_PIT_TIP", r.name);
            case EstateType.MINE:       return I18.s("KINGDOM_MINE", resources[0].name + "_TIP_" + r.name);
            case EstateType.LAKE:       return I18.s("KINGDOM_LAKE_TIP", r.name);
            case EstateType.GRAIL:      return I18.s("KINGDOM_GRAIL_TIP", r.name);
            case EstateType.CAMP:       return I18.s("KINGDOM_CAMP_TIP", r.name);
            case EstateType.GROTTO:     return I18.s("KINGDOM_GROTTO_TIP", r.name);
            case EstateType.RUIN:       return I18.f("RUIN_NAME");
            case EstateType.EVIL:       return I18.f("EVIL_NAME");
            case EstateType.CASTLE:     return I18.f("CASTLE_NAME");
            case EstateType.STONEHENGE: return I18.f("STONEHENGE_NAME");
            case EstateType.CATAPULT:	return I18.f("CATAPULT_NAME");
            default:                    return getEstateTypeName(type);
        }
   },
   
	getUnitName: function(ut/*:UnitType*/, level/*:int*/, race/*:Race*/)
	{
		var special = I18.getStringOrNull("UNIT_" + ut.name + "_" + race.name + "_L" + UnitType.normalize(level));
        return special != null ? special : I18.getUnitClassName(ut);
	},
   
	getAttackFailSubject: function(failReason/*:String*/, toFieldXYZ/*:XYZ*/, toEstateNameAndLocation/*:String*/)
	{
	    return I18.s("SYSTEM_MESSAGE_SUBJECT_ATTACK_FAIL", (failReason ? failReason : "NO_ESTATE"), toFieldXYZ, toEstateNameAndLocation);
	},

	getBanReason: function(reason/*:String*/)
	{
		var r = I18.getStringOrNull("BAN_REASON_" + (reason != null ? reason.toUpperCase() : ""));
		if (r != null) 
		{
			return r;
		}
		return reason;
	},
	
	getMissionEntranceClosedMessage: function(isOuterEstate/*:Boolean*/)
	{
        return isOuterEstate ? I18.f("MISSION_TUNNEL_IS_CLOSED") : I18.f("MISSION_GATE_IS_CLOSED");
	},
   
   	heroError: function(type/*:String*/)
	{
		
        return I18.getMegaString(I18.s("HERO_ERROR", type), FConfig);
   	},
   
   	getMegaString: function(key/*:**/, values/*:Object = null*/)
	{
		var template = I18.getStringOrNull(key);
        if (!template)
        {
            template = key;
        }
        return I18.megaTemplate(template, values);
    },
   
   megaTemplate: function(template/*:String*/, values/*:Object*/)
   {
   		if(typeof values != "object")
   		{
   			values = [values];
   		}
   		
        var obj = values || {};
        //var obj/*:Object*/ = Reflector.toObject(values);
        var tokens/*:Array*/ = I18.parseMegaTemplate(template);
        var res/*:String*/ = "";
        
        for (var i=0; i<tokens.length; i++) 
        {
            if (typeof tokens[i] == 'string') 
            {
                res += tokens[i];
            } 
            else if ($.isArray(tokens[i])) 
            {
            	var tmp = tokens[i];
                if (tmp[1] == "text") 
                {
                    res += I18.getString(tmp[0]);
                } 
                else 
                {
                	var p = obj[tmp[0]*1];
                	if(typeof p == "undefined")
                	{
                		p = obj[tmp[0]];
                	}
                	if(typeof p == "undefined" && obj.getValue)
                	{
                		p = obj.getValue(tmp[0]);
                	}
                	if(typeof p == "undefined")
					{
						p = '';
					}
                	
                	res += I18.typedValueToString(tmp[1], p);
                }
            }
        }
        
        return res;
	},
   	
   	typedValueToString: function(type/*:String*/, value/*:**/)
	{
		if (value === undefined) 
		{
            return "{undefined:" + type + "}";
        } 
        else if (value === null) 
        {
            return "";
        }
        switch (type) 
        {
            case "list":
        	{
        		return Arrays.join(value, ", ");
        	}
                
            case "real":
                return "" + value;
            case "on-duration":
                return Tick.toTimeLeftFull(value, true);
            case "on-on-duration":
                return I18.getMegaString("ON_DURATION", {duration:value});
            case "duration":
                return Tick.toTimeLeftFull(value, false);
            case "signed-int":
            {
            	if (value*1 > 0) 
                {
                    return "+" + Math.round(value);
                }
                return Math.round(value).toString();
            }
            case "percent":
            case "percent-reversed":
            case "percent-green":
                var n = value;
                return Strings.formatPercent(Math.round(n * 100), true);
            case "percent-unsigned":
            case "percent-unsigned-reversed":
                return Strings.formatPercent(Math.round(Math.abs(value) * 100));
            case "tick-date-seconds":
                return DateUtils.formatWithDateAndSeconds(Tick.toMs(value));
            case "times":
                return value + " " + I18.getString("T" + Tick.getNaklon(value, true)) + "";
            case "outer-estate-type":
                return I18.getString(value.name + "_NAME");
            case "city-name":
            {
            	if (value == null) 
                {
                    return "?";
                }
                if (value == FCity.CITY_DEFAULT_NAME) 
                {
                    return I18.f("CITY_DEFAULT_NAME");
                }
                return value;
            }
            case "terrain":
            {
            	if (value == null) 
                {
                    return "TERRAIN_NULL";
                }
                return value.displayName;
            }
            case "terrain-m":
            {
            	if (value == null) 
                {
                    return "TERRAIN_NULL";
                }
                return I18.getString("TERRAIN_" + value.name + "_M");
            }
            case "building":
            {
            	if (value == null) 
                {
                    return I18.f("RACE_R_ALL");
                }
                return I18.getBuildingName(value, Race.HUMAN);
            }
            case "science":
                return I18.getScience(value);
            case "races-r":
            {
            	if (!value) 
                {
                    return I18.f("RACE_R_ALL");
                }
                var rr = value;
                var racenames= [];
                for (var i=0; i<rr.length; i++)
                {
                	if (rr[i].name)
                	{
                		racenames.push(r[i].name);
                	}
                }
                return I18.racesR(racenames);
            }
            case "race":
                return I18.race(value);
            case "resource":
                return I18.getResourceName(value);
            case "unit-type-r":
            {
            	if (!value) 
                {
                    return I18.f("RACE_R_ALL");
                }
                
                return I18.getString("UNIT_R_" + (value.name ? value.name : value));
            }
            case "spellname":
                return Texts.spellName(value);
            case "spell":
            	return Texts.spellName(SpellNameAndLevel(value).spellName) + I18.f("LVL_X",SpellNameAndLevel(value).spellLevel);
            case "skill":
                return Texts.skillName(value);
            case "artifact":
                return Texts.t("ARTIFACT_" + String(value).toUpperCase());
            case "artifact-decl":
                return Texts.artifactName(value);
            case "set":
                return Texts.artifactSetName(value);
            case "name_and_location":
                return I18.formatEstateName(value.name, value.location);
            case "username":
                return FUser.getUserNameForMessages(value);
            case "to-time-future":
                return Tick.toTimeFuture(value);
            case "per-hour":
            case "real-per-hour":
            {
            	var s1 = (type == "per-hour" ? Math.round(value) : value) + "";
                if (value > 0) 
                {
                    s1 = "+" + s1;
                }
                return I18.getMegaString("PER_TIME", [s1]);
            }
            case "ref":
                return I18.getString(value);
            default:
                return "" + value;
        }
    },
   	
   	racesR: function(r/*:Array*/)
   	{
        if (r.length == Race.ALL.length) 
        {
            return I18.toString("RACE_R_ALL");
        }
		if (r.length == Race.ALL.length - 1) 
        {
            for (var i=0; i<r.length; i++) 
            {
                if (r.indexOf(r1.name) == -1) 
                {
                    return I18.RACE_R_EXCEPT + " " + I18.getString("RACE_R_" + r1.name);
                }
            }
        }

        if (r.length == 0) 
        {
            return "RACE_R_NOBODY";
        }

        var result = "";
        for (var i=0; i<Race.ALL.length; i++) 
        {
            if (r.indexOf(Race.ALL[i].name) != -1) 
            {
                if (result.length > 0) 
                {
                    if (r.length == 2)
                    {
                    	result += " " + I18.f("AND") + " ";
                    }
                    else
                    {
                    	result += ", ";
                    }
                }
                result += I18.getString("RACE_R_" + Race.ALL[i].name);
            }
        }
        return result;
    },
   	   	
   	race: function(r/*:Array*/)
   	{
        var result = "";
        for (var i=0; i<r.length; i++) 
        {
            if (result.length > 0) 
            {
                if (r.length == 2)
                {
                	result += " " + I18.f("AND") + " ";
                }
                else
                {
                	result += ", ";
                }
            }
            result += I18.getString("RACE_" + r[i].name);
        }
        return result;
    },
    
   	getBanReason: function(reason)
	{
		var r = I18.getStringOrNull("BAN_REASON_" + (reason != null ? reason.toUpperCase() : ""));
        if (r != null)
        {
            return r;
        }
        return reason;
	},
   	
   	cityDeleteError: function(type/*:String*/)
   	{
        return I18.getMegaString(I18.s("CITY_DELETE_ERROR", type), FConfig);
    },

    parseMegaTemplate: function(template/*:String*/)
	{
		var acc = "";
        var r = [];
        for (var i = 0; i < template.length; i++) 
        {
            switch (template.charAt(i)) 
            {
                case "^":
                {
                	i++;
                    acc += template.charAt(i);
                    break;
                }
                case "{":
                {
                	r.push(acc);
                    acc = "";
                    var type = "";
                    while (i < template.length) 
                    {
                        i++;
                        if (template.charAt(i) == "}") 
                        {
                            break;
                        }
                        type += template.charAt(i);
                    }
                    var typeValueLabel = type.split(":");
                    if (typeValueLabel.length == 1) {
                        typeValueLabel = typeValueLabel.concat([""]);
                    }
                    r.push(typeValueLabel);
                    break;
                }
                default:
                {
                	acc += template.charAt(i);
                    break;
                }
            }
        }
        if (acc != "")
        {
            r.push(acc);
        }
        return r;
   },
   
   godProtectionError: function(type/*:String*/)
	{
		
        return I18.getMegaString(I18.s("GODPROTECT_ERROR", type), FConfig);
   },
   
   	friendError: function(type/*:String*/)
	{
        return I18.getMegaString(I18.s("FRIEND_ERROR", type), FConfig);
   },
   
   getHeroClassName: function(heroClass/*:String*/)
	{
        return heroClass ? Texts.t("HERO_CLASS_" + heroClass) : "";
   },

    getBranchName: function(branch/*:String*/)
	{
        return Texts.t("BRANCH_" + branch + "_NAME");
    },
   
   	auctionError: function(type/*:String*/, banDate/*:String*/)
	{
        return I18.getMegaString(I18.s("AUCTION_ERROR", type, banDate), FConfig);
   	},
   
   	accessDeniedError: function(type)
	{
        var values = Array.prototype.slice.call(arguments, 1);
        return I18.getMegaString(I18.s("ACCESS_DENIED_ERROR", type, Arrays.antidot(values)), FConfig);
	},
   
  	missionStartError: function(type)
	{
        var values = Array.prototype.slice.call(arguments, 1);
        return I18.getMegaString(I18.s("MISSION_START_ERROR", type, Arrays.antidot(values)), FConfig);
   	},

   	monsterAttackError: function(type/*:String*/)
	{
        return I18.getMegaString(I18.getStringOrNull("MONSTER_ATTACK_ERROR" + "_" + type) || I18.missionStartError(type), FConfig);
   	},
   
   	marketError: function(type)
	{
        var values = Array.prototype.slice.call(arguments, 1);
        return I18.getMegaString(I18.s("MARKET_ERROR", type, Arrays.antidot(values)), FConfig);
   	},
   
	giftError: function(type)
	{
        var values = Array.prototype.slice.call(arguments, 1);
        return I18.getMegaString(I18.s("GIFT_ERROR", type, Arrays.antidot(values)), FConfig);
   	},
    
    vacationError: function(type)
	{
        var values = Array.prototype.slice.call(arguments, 1);
        return I18.getMegaString(I18.s("VACATION_ERROR", type, Arrays.antidot(values)), FConfig);
   	},
   
   	mentorshipError: function(type)
	{
        var values = Array.prototype.slice.call(arguments, 1);
        return I18.getMegaString(I18.s("MENTORSHIP_ERROR", type, Arrays.antidot(values)), FConfig);
   	},
   
	assistanceFailMessageSubject: function(reason)
	{
        var values = Array.prototype.slice.call(arguments, 1);
        return I18.s("SYSTEM_MESSAGE_SUBJECT_ASSISTANCE_FAIL", reason, Arrays.antidot(values));
   	},
   
	mailError: function(type/*:String*/)
	{
        return I18.getMegaString(I18.s("MAIL_ERROR", type), FConfig);
   	},

    questError: function(type/*:String*/)
	{
        return I18.getMegaString(I18.s("QUEST_ERROR", type), FConfig);
   	},

    tournamentError: function(type/*:String*/)
	{
        return I18.getMegaString(I18.s("TOURNAMENT_ERROR", type), FConfig);
   	},

    pearlBoostError: function(type/*:String*/)
	{
        return I18.getMegaString(I18.s("PEARL_BOOST_ERROR", type), FConfig);
   	},
   
   	castError: function(type)
	{
        var values = Array.prototype.slice.call(arguments, 1);
        return I18.getMegaString(I18.s("CAST_ERROR", type, Arrays.antidot(values)), FConfig);
   	},

	buildingError: function(type/*:String*/)
	{
        return I18.getMegaString(I18.s("BUILDING_ERROR", type), FConfig);
    },

    moveCapitalError: function(type/*:String*/)
	{
        return I18.getMegaString(I18.s("MOVE_CAPITAL_ERROR", type), FConfig);
    },

    getClanErrorMessage: function(type)
	{
        var values = Array.prototype.slice.call(arguments, 1);
        return I18.getMegaString(I18.s("CLAN_ERROR", type, Arrays.antidot(values)), FConfig);
   	},

    getCatapultErrorMessage: function(type)
	{
        var values = Array.prototype.slice.call(arguments, 1);
        return I18.getMegaString(I18.s("CATAPULT_ERROR", type, Arrays.antidot(values)), FConfig);
   	},

    getBrotherhoodErrorMessage: function(type)
	{
        var values = Array.prototype.slice.call(arguments, 1);
        return I18.getMegaString(I18.s("BROTHERHOOD_ERROR", type, Arrays.antidot(values)), FConfig);
    },

	getBuildingAndLevelFrom: function(race/*:Race*/, type/*:BType*/, level/*:int*/)
	{
        return I18.getBuildingName(type, race) + " " + I18.f("LVL_X",level);
   	},
    
    getConcurrentLimitText: function(limit/*:int*/)
	{
        return I18.f('BUILDING_CONSTRAINT_CONCURRENT', limit);
   	},

    getConcurrentRepairingLimitText: function(limit/*:int*/)
	{
        if (limit > 0) 
        {
            return I18.f("REPAIRING_CONSTRAINT_CONCURRENT",limit);
        } 
        else 
        {
            return I18.f("REPAIRING_CONSTRAINT_SCIENCE",I18.getScience(Science.REPAIRING));
        }
    },
    
    getFlagName: function(flag/*:int*/)
	{
        return I18.s("FLAG_NAME", flag.toString());
   	},
    
    getDefaultScienceDesc: function(science/*:Science*/, level/*:Number*/, race/*:Race*/)
	{
        var values = Array.prototype.slice.call(arguments, 3);
		if (level == 0) 
        {
            return "";
        }
        
        var tipName = "SCIENCE_" + (science.dependsOnRace ? race.name : "TIP") + "_" + science.name;
        if (science.dependsOnLevel || science.separate1stLevel && level == 1) 
        {
            tipName += "_" + level;
        }
        
        return I18.getMegaString(tipName, Arrays.antidot(values));
   	},
    
    assistanceFailMessageBody: function(reason/*:String*/, username/*:String*/, location/*:String*/)
	{
        var key = username && FUser.isClanUser(username) ? reason + "_CLAN" : reason;
        var owner = username && FUser.isClanUser(username) ? FUser.parseClanName(username) : username;
        return I18.s("SYSTEM_MESSAGE_BODY_ASSISTANCE_FAIL", key, [owner, location]);
   	},
     
    fixName: function(name/*:String*/)
	{
        name = htmlEscape(name);
        if (name.indexOf("%") < 0) 
        {
            return name;
        }
        return I18.getString(name.substr(1, name.length - 2));
   	},

   	activeMagicTip: function(tick/*:Number*/)
	{
        return I18.f("MAGIC_IS_ACTIVE_UNTIL", Tick.toTimeFuture(tick));
   	},
 
	getClanResourcesReceivedSubject: function(bt/*:BType*/)
	{
	    if (bt.equals(BType.CLAN_RESERVATION) || bt.equals(BType.CLAN_WAREHOUSE)) 
	    {
	        return I18.s("CLAN_RESOURCES_RECEIVED_SUBJECT", bt.name);
		} 
		else 
		{
		    throw new Error("Invalid BType {0}", bt);
		}
	},
   
   
	getClanResourcesReceivedBody: function(bt/*:BType*/, self/*:Boolean*/, username/*:String*/)
	{
        if (bt.equals(BType.CLAN_RESERVATION) || bt.equals(BType.CLAN_WAREHOUSE)) 
        {
            var isUser = /^[a-zA-Z0-9_]{1,15}$/.test(username);
            return I18.s("CLAN_RESOURCES_RECEIVED_BODY_" + (self ? "SELF" : "OTHER") + (isUser ? "" : "_CASTLE"), bt.name, username);
        } 
        else 
        {
            throw new Error("Invalid BType {0}", bt);
        }
	},

	getScienceDesc: function(science/*:Science*/, level/*:Number*/, race/*:Race*/)
	{
	    level = Math.max(1, level*1) || 1;
	    
	    var slevel = science.getLevel(level - 1);
	    var value = slevel ? slevel.value : 1;
	    
	    switch (science) 
	    {
	        case Science.BUILDING:
	            return I18.getDefaultScienceDesc(science, level, race, Science.getMaxConcurrentBuildings(value));
	        case Science.ARMY_MANAGEMENT:
	            return I18.getDefaultScienceDesc(science, level, race, Science.getMaxMissionsCount(level));
	        case Science.LABOUR_SERVICE:
	        case Science.MILITARY_SERVICE:
	        {
	        	return I18.getDefaultScienceDesc(science, level, race,
	                    science.getLevel(level - 1).buff);
	        }
	        case Science.SPEC_TRADER:
	            return I18.getDefaultScienceDesc(science, level, race,
	                    [science.getLevel(level - 1).marketTaxDiscount,
	                    science.getLevel(level - 1).shopRatioBonus]);
	        case Science.SPEC_MINER:
	            return I18.getDefaultScienceDesc(science, level, race,
	                    [science.getLevel(level - 1).ironBonus,
	                    science.getLevel(level - 1).stoneBonus,
	                    science.getLevel(level - 1).goldBonus]);
	        case Science.SPEC_BENEFACTOR:
	            return I18.getDefaultScienceDesc(science, level, race,
	                    [science.getLevel(level - 1).capacityBonus,
	                    science.getLevel(level - 1).increaseBonus]);
	        case Science.SPEC_DEFENDER:
	            return I18.getDefaultScienceDesc(science, level, race,
	                    [science.getLevel(level - 1).towersBonus,
	                    science.getLevel(level - 1).fortificationBonus]);
	        case Science.SPEC_CLERIC:
	            return I18.getDefaultScienceDesc(science, level, race,
	                    [science.getLevel(level - 1).resurrectionBonus * 100,
	                    science.getLevel(level - 1).healerTrainBonus * 100]);
	        case Science.MARKET:
	            if (value == 0) 
	            {
	                return I18.f("SCIENCE_MARKET_OWN_KINGDOM");
	            } 
	            else 
	            {
	                return I18.getDefaultScienceDesc(science, level, race, value);
	            }
	        case Science.HEROES_SKILL:
	            return I18.getDefaultScienceDesc(science, level, race, value + FConfig.MAX_HERO_LEVEL_BASE);
	        default:
	            return I18.getDefaultScienceDesc(science, level, race, value);
	    }
	},
	
	percentText: function(value)
	{
        return I18.typedValue("percent", value);
	},
   
	percentReversedText: function(value)
	{
        return I18.typedValue("percent-reversed", value);
	},
		
    m: function(item/*:I18*/,  value/*:Object*/)
	{
        return I18.getMegaString(item.key, value);
	},

    mega: function(item/*:I18*/, values)
	{
        var values = Array.prototype.splice.call(arguments, 1);
        return I18.getMegaString(item, Arrays.antidot(values));
    },
	
	formatEstateName: function(name/*:String*/, xyz/*:XYZ*/)
	{
		//console.log(I18.NOWHERE);
	    //if (!name || !xyz)
	    if (!xyz)
	    {
	        return I18.NOWHERE;
	    } 
	    else if (!name || name.length == 0) 
	    {
	    	//console.log(xyz);
	        return I18.f("FIELD_LOCATION_FORMAT", xyz.x + 1, xyz.y + 1, xyz.z + 1);
	    } 
	    else 
	    {
	        return I18.f("ESTATE_NAME_AND_LOCATION_FORMAT",I18.fixName(name), xyz.x + 1, xyz.y + 1, xyz.z + 1);
	    }
	},
		
	getFriendEffectTooltip: function(effect/*:String*/, durationKey/*:String*/)
	{
        return I18.f("FRIEND_HELP_EFFECT",I18.getString("FRIEND_HELP_EFFECT_" + effect)) + "\n" +
                I18.getMegaString(durationKey, [FConfig.HELP_INTERVAL.asNumber()]);
   	},
      
    box: function(item/*:I18*/)
	{
       	var values = Arrays.antidot(Array.prototype.slice.call(arguments, 1));
		if (!$.isArray(values))
		{
			values = [values];
		}
       	
       	
        return I18.getMegaBox(item, values);
    },
    
    getServerTypeName: function(type/*:GameServerType*/)
	{
        return I18.s("GAME_SERVER_TYPE", type.name);
   	},
     	
 	getForBodyPart: function(bodyPart/*:BodyPart*/)
	{
        return I18.s("FOR", bodyPart.name);
   	},
      
    getAttackMissionGoal: function(goal/*:MissionAttackGoal*/)
	{
        return goal != null ? I18.s("MISSION_GOAL", goal.name.replace("_", "")) : "";
	},
	
	getMissionStateWithResult: function(m/*:FMission*/)
	{
	    var stateName = '';
	    if (MissionState.EXECUTING.equals(m.state))
	    {
	        stateName = I18.s("MISSION_STATE_EXECUTING", m.isOwner ? "OWNED" : "NON_OWNED");
	    } 
	    else 
	    {
	        stateName = I18.s("MISSION_STATE", m.state.name);
	    }
	    if (MissionResult.UNKNOWN.equals(m.result)) 
	    {
	        return stateName;
	    }
	    var resultName = I18.s("MISSION_RESULT", m.result.name);
	    return Strings.format("{0} ({1})", stateName, resultName);
	},

	getMissionGoal: function(m/*:FMission*/)
	{
        var res = m.className.replace("Mission", "").toUpperCase();
        if (res.indexOf("TRADE") != -1) 
        {
            if (!m.isOwner) 
            {
                res += "_YOUR";
            }
            res += '_' + m.resource.name;
        }
        return I18.s("MISSION_GOAL", res);
  	},
	
	getCastEffectMessage: function(r/*:FCastEffect*/)
	{
	    return I18.getMegaString("CAST_EFFECT_" + r.type.name, r.params);
	},
	  
	getBuffCause: function(buff/*:FBuff*/, gCase/*:String = ""*/, forUser/*:String=""*/)
	{
	    if (typeof gCase == 'undefined') gCase = '';
	    if (typeof forUser == 'undefined') forUser = '';
        return I18.getCastCause(buff.cast, gCase, true, forUser);
    }, 
	
    getCastCause: function(cast/*:FCast*/, gCase/*:String = ""*/, suppressNames/*:Boolean = false*/, forUser/*:String=""*/)
    {
	    if (typeof gCase == 'undefined') gCase = '';
	    if (typeof suppressNames == 'undefined') suppressNames = false;
	    if (typeof forUser == 'undefined') forUser = '';
	    var key = "BUFF_CAUSE" + gCase + "_" + cast.cause.name;
	    switch (cast.cause.name) 
	    {
	        case CastCause.PERSONAL_EVENT.name:
	            return I18.getMegaString(key + "_" + cast.spell);
	        case CastCause.ARTIFACT.name:
	        case CastCause.SET.name:
	            return I18.getMegaString(key, [cast.artifact]);
	        case CastCause.SPELL.name:
	            return I18.getMegaString(key, [cast.spell, cast.level]);
	        case CastCause.PASSIVE.name:
	        {
	            if (!suppressNames) 
	            {
	                return I18.getMegaString(key, [cast.spell])
	            }
	            return I18.getMegaString(key + ((forUser.length > 0 && cast.caster != forUser) ? "_OTHER" : "_OWN"));
			}
			case CastCause.HALL_PASSIVE.name:
	            return I18.getMegaString(key + "_" + cast.spell, [cast.level]);
	        default:
        	{
        		return I18.getMegaString(key);
        	}
	    }
    },
		
	parseAndCreateMegaObject: function(key/*:String*/, values/*:Object*/, makeLabelIfString/*:Boolean*/)
	{
        var obj/*:Object*/ = values;
        var tokens/*:Array*/ = I18.parseMegaTemplate(key);
        var items/*:Array*/ = [];
        var stringsOnly/*:Boolean*/ = true;
        for (var i=0; i<tokens.length; i++) 
        {
            if (typeof tokens[i] == 'string') 
            {
                items.push(tokens[i]);
            } 
            else if ($.isArray(tokens[i])) 
            {
                if (tokens[i][1] == "text") 
                {
                    items.push(I18.getString(tokens[i][0]));
                } 
                else 
                {
                    var val = obj[tokens[i][0]];
                    // FMap
                    if ((typeof val == 'undefined') && (typeof obj['_entries'] != 'undefined'))
                    {
                    	val = obj.getValue(tokens[i][0]);
                    }
                    // ArrayCollection
                    if ((typeof val == 'undefined') && (typeof obj['data'] != 'undefined'))
                    {
                    	val = obj.data[tokens[i][0]];
                    }
                    
                    var value = I18.typedValue(tokens[i][1], val, tokens[i].length == 3 ? tokens[i][2] : null);
                    items.push(value);
                    if (!(typeof value == 'string')) 
                    {
                        stringsOnly = false;
                    }
                }
            }
        }
		
		var res = document.createElement('I18-mega-object');
        if (stringsOnly) 
        {
            var text = items.join(' ');
            if (!makeLabelIfString) 
            {
                return text;
            }
            $(res).append(text);
        } 
        else 
        {
            for (var i = 0, len = items.length; i < len; i++)
            {
            	// UIh.addItem
            	var item = items[i];
            	if (!item) continue;
            	
            	if ((typeof item == 'object') && !(item instanceof HTMLElement))
            	{
            		var cls = item['__className__'] || item['__CLS__'].split('.').pop() || '';
            		if (cls == 'Funds')
            		{
            			var f = COM.create('FundsBox');
            			f.update(item, FUser.CURRENT_USER.race);
            			item = f.DOM;
            		}
            		// TODO: else if (cls == 'Asset')
            		else if (cls == 'ArtifactDeclaration')
            		{
			            var art = new FArtifact();
			            art.declaration = item;
			            var artPic = ArtifactPic.create(art, []);
			            item = artPic.DOM;
            		}
            		else
            		{
            			item = String(item);
            		}
            	}
            	if (undefined != I18.strings[item]) item = I18.f(item);
            	$(res).append(item);
            }
        }
        return res;
   	},
	  
    getFriendRewardMessage: function(r/*:FCastEffect*/)
	{
        return I18.getMegaBox("FRIEND_REWARD_" + r.type.name, r.params);
   	},
	  
    getMegaBox: function(key/*:String*/, values/*:Object = null*/)
	{
        var template = I18.getStringOrNull(key);
        if (template == null) 
        {
            template = key;
        }
        var lines = template.split("\n");
        if (lines.length == 1) 
        {
            return I18.parseAndCreateMegaObject(template, values, true);
        }

        var items = [];
        var stringsOnly = true;
        for (var i = 0; i < lines.length; i++) 
        {
            var box = I18.parseAndCreateMegaObject(lines[i], values, false);
            items.push(box);
            if (typeof box != "string") 
            {
                stringsOnly = false;
            }
        }
		
        var res = document.createElement('I18-mega-box');
        if (stringsOnly) 
        {
            $(res).append(items.join(' '));
        } 
        else 
        {
            for (var i = 0; i < items.length; i++) 
            {
                $(res).append(items[i]);
                $(res).append(" ");
            }
        }
        return res;
	},
	  	  
	typedValue: function(type/*:String*/, value/*:**/, arg/*:String = null*/)
	{
		//return value;
				
	    if (typeof value == 'undefined') 
	    {
	        return "{undefined:" + type + "}";
	    } 
	    else if (value === null) 
	    {
	        return "";
	    }
	
	    //Style,UI,Label,FundsBox,Canvas,Image,ToolTipEvent,TimeView,FSpell,FlexEvent,
	    //,PearlPrice,//ArmyViewDense,EffectsView,GodpointsView,
	    //KnowledgeView,ServiceHelper,HBox
	    switch (type) 
	    {
	    	case "outer-estate-type":
	           return I18.getString(value.name + "_NAME"); // value - EstateType
			case "terrain":
	            return value.displayName;
	        case "terrain-m":
	            return I18.getString("TERRAIN_" + value.name + "_M");
	    	case "science":
	            return I18.getScience(value);
	    	case "unit-type-r":
	    	{
	    		if (value == null) 
	            {
	                return I18.f("UNIT_R_ALL");
	            }
	            return I18.getString("UNIT_R_" + String(value)); 
	    	}
	        case "ref":
	            return I18.getString(String(value));
	    	case "real":
	            return String(value);    
	        case "username":
	            return FUser.getUserNameForMessages(value);    
	        case "resource":
	            return I18.getResourceName(value);    
	    	case "building":
	    	{
	    		if (value == null) 
	            {
	                return I18.f("BUILDING_R_ALL");
	            }
	            return I18.getBuildingName(value, Race.HUMAN);
	    	}    
	        case "name_and_location":
	            return I18.formatEstateName(value.name, value.location);    //value - NameAndLocation
            case "spellname":
	            return Texts.spellName(value);
            case "on-duration":
	            return Tick.toTimeLeftFull(value, true);
	        case "duration":
	            return Tick.toTimeLeftFull(value, false);
	            
            case "city-name": // FCity
	    	{
	    		if (value == null) 
	            {
	                return "?";
	            }
	            if (value == FCity.CITY_DEFAULT_NAME) 
	            {
	                return I18.f("CITY_DEFAULT_NAME");
	            }
	            return value;
	    	}
	    	case "on-on-duration":
	            return I18.getMegaString("ON_DURATION", {duration:value});
	    	case "times":
	            return value + " " + I18.getString("T" + Tick.getNaklon(value, true)) + "";

	    	case "tick-date-seconds":
	            return DateUtils.formatWithDateAndSeconds(Tick.toMs(value));
			case "to-time-future":
	            return Tick.toTimeFuture(value);
	        case "wrapped":
	    	{
	    		return String(value);
	    	}
	    	case "percent":
	        case "percent-reversed":
	        case "percent-unsigned":
	        case "percent-green":
	        case "percent-unsigned-reversed":///////////?????????????
	        {
	        	var n = value;
	            var t = document.createElement(type); 
	            var reversed = type == "percent-reversed" || type == "percent-unsigned-reversed";
	            var unsigned = type == "percent-unsigned" || type == "percent-unsigned-reversed";
	            var green = type == "percent-green";
	            
	            $(t).html(' ' + Strings.formatPercent(Math.round(n * 100), !unsigned));
	            if (green || !reversed && n > 0 || reversed && n < 0) 
	            {
	                Style.POSITIVE_BONUS.applyTo(t);
	            } 
	            else if (n != 0) 
	            {
	                Style.NEGATIVE_BONUS.applyTo(t);
	            }
	            return t;
	        }    
        	case "army"://///////??????????????
	        {
	        	if (arg == null) 
	            {
	                arg = I18.f("ARMY_R");
	            } 
	            else 
	            {
	                arg = I18.getString(arg);//ArmyViewDense,EffectsView,Assets,Army,GodpointsView,KnowledgeView,ServiceHelper,HBox
	            }

                if(value instanceof MissionInfo)
                {
                    return MissionDetailsTip.createTipText(arg, value);
                }
                else
                {
                    var armyRace = FUser.CURRENT_USER.race ? FUser.CURRENT_USER.race : Race.MONSTER;
                    return ArmyViewDense.createTipText(arg, (value.army ? value.army : value), FUser.CURRENT_USER.science.unitLevels, armyRace);
                }
	        }
	    	case "monster-army":
			{
				return ArmyViewDense.createTipText(I18.f("ANTIENT_ARMY_R"), Army.create(value[0]), value[1], Race.MONSTER);
	        	//return '<span style="color:red;">ArmyViewDense:</span> '+I18.f("ANTIENT_ARMY_R");
	        }
	        
	        case "spell":
	        {
            	var l = document.createElement("span");
            	l.innerHTML = Texts.spellName(value.spellName) + I18.f("LVL_X", value.spellLevel);
            	l.style.textDecoration = "underline";
            	l.style.fontWeight = "bold";
            	COM.bindExclusiveClick(l, function()
            	{
            		SpellTip.alert(new FSpell(value.spellName, value.spellLevel, null, true), [], [], []);
            	});
                    
                return l;
	        }
	        case "races-r":
	        {
	        	if (value == null) 
	            {
	                return I18.f("RACE_R_ALL");
	            }
	            var rr = []; 
	            rr = value;
	            var racenames = [];
	            for(var i=0;i<rr.length; i++) 
	            {
	                racenames.push(rr[i].name);
	            }
	            return I18.racesR(racenames);
	        }
            
	    	case "list":
	        if (arg) 
            {
            	return Strings.joinColumns(value ? value : value, arg*1); // value - ArrayCollection
            } 
            else 
            {
                return Arrays.join(value ? value : value, ", ");
            }
	        
	        ///////////////??????????????????
	        case "funds":
	        case "funds-per-hour":
	        case "funds-per-hour-red":
	        {
	            var funds = value;
	            var peopleRace = FUser.CURRENT_USER.race ? FUser.CURRENT_USER.race : Race.MONSTER;
	            var fundsBox = COM.create('FundsBox');
	            fundsBox.allPeople = true;
	            
	            if (type == "funds-per-hour" || type == "funds-per-hour-red") 
	            {
	                fundsBox.megaStringKey = "PER_TIME";
	            }
	            
                if (type == "funds-per-hour-red" || type == "funds-red") 
                {
                    fundsBox.updateAll(funds, new Funds(), peopleRace);
                } 
                else 
                {
                    fundsBox.update(funds, peopleRace);
                }
	            
	            return fundsBox.DOM;
	        }

	        case "per-hour":
	        case "real-per-hour":
	        {
	            var style1 = value > 0 ? Style.POSITIVE_BONUS : Style.NEGATIVE_BONUS;
	            var s1 = (type == "per-hour" ? Math.round(value) : value) + "";
	            
	            if (value > 0) 
	            {
	                s1 = "+" + s1;
	            } 
	            s1 = I18.getMegaString("PER_TIME", [s1]);
	            
	            var res = document.createElement(type);
	            style1.applyTo(res);
	            $(res).html(' ' + s1);
	            return res;
	        }

			case "int":
			case "signed-int":
	        {
	        	var style;
	            var s = Math.round(value) + "";
	            var res = document.createElement(type);
	            if (value > 0) 
	            {
					Style.POSITIVE_BONUS.applyTo(res);
	                if (type == "signed-int") s = "+" + s;
	            } 
	            else 
	            {
					Style.NEGATIVE_BONUS.applyTo(res);
	            }
	            $(res).html(' ' + s);
	            return res;
	        }
	        
	        case "time-left":
	 		{
	 			return TimeView.create(value).DOM;
	 		}
	            
	        ////////////////////????????????????
	        case "unit-type-pic":
	        	var res = document.createElement('img');
	        	res.src = STATIC_INTERFACE_URL + Assets.getUnitTypeIcon(value);
	            return res;
	        
	        case "race":
	            return I18.race(value);
	        
	        case "pearl":
	            var pp = COM.create('PearlPrice');
				pp.showText = false;
				pp.update(value, false);
	            return pp.DOM;
	            
	        case "hero-face":
	            var c = document.createElement('I18-hero-face');
	            
	            var b = new Image();
	            b.src = STATIC_INTERFACE_URL + Assets.getHeroBack(value[1]);
	            c.appendChild(b);
	            
	            var f = new Image();
	            f.src = STATIC_INTERFACE_URL + Assets.getHeroFace(value[0]);
	            c.appendChild(f);
	            
	            return c;
        
	        case "godpoints":
	            var gpview = COM.create('GodpointsView');
                gpview.asSpeed = false;
                gpview.value = Math.round(value), Number.MAX_VALUE;
	            return gpview.DOM;
	            
	        case "knowledge":
	            var knowledgeView = COM.create('KnowledgeView');
                knowledgeView.asSpeed = false;
                knowledgeView.value = Math.round(value), Number.MAX_VALUE;
	            return knowledgeView;
	            
	        case "service":
	            var decl/*:ServiceItemDecl*/ = ServiceHelper.getGloabalServiceDecl(value.toString(), 0);
	            if (!decl) 
	            {
	                decl = ServiceHelper.getLocalServiceDecl(value.toString(), 0);
	            }
	            if (!decl) 
	            {
	                return "{" + value.toString() + "}";
	            }
	            
	            var box = document.createElement('I18-service');
	            EffectsView.createDescriptiveView(box, decl.dtoReward, false, false, false, false);
	            return box;
	       	
            case "standard":
                var standard = FConfig.WAR_STANDARDS.getValueOrNull(value) || FConfig.PEACE_STANDARDS.getValueOrNull(value);
                var standardBox = document.createElement('I18-standard');
                EffectsView.createDescriptiveView(standardBox, standard.reward.dtoActions, false, false, false, false, "", false);
                return standardBox.outerHTML;
                
            case "wrapped":
                var box = document.createElement('div');
                box.className = 'I18-wrapped';
                box.innerHTML = value;
                return box.outerHTML;
                
            case "ref":
                return I18.getString(value);
                
            case "unit-type-pic":
                return '<img src="' + STATIC_INTERFACE_URL + Assets.getUnitTypeIcon(value) +'" />';
	       	
	        default:

	            if(undefined === I18.strings[value])
                {
                    if(typeof value == "string") return Strings.safeHtml(value);
                    else return value;
                }
                else
                {
                    return I18.f(value);
                }
		}
	},
	
	getKingdomCityTip: function(r)
	{
        return I18.s("KINGDOM_CITY_TIP", r.name);
	},

    getServiceDescription: function(s)
    {
        return I18.getStringOrNull("SERVICE_DESC_" + s.service);
    },

    moveCityError: function(type)
    {
        return I18.getMegaString(I18.s("MOVE_CITY_ERROR", type), FConfig);
    },

	////////////////////////////

	formatNumber: function(value, precision)
	{
        if(typeof precision == "undefined") precision = 2;
        return NumberUtils.format(value, precision, I18.f("DECIMAL_SEPARATOR"), I18.f("THOUSAND_SEPARATOR"));
   },
   
   _s: function(str)
   {
		var reg = new RegExp(str, 'img'), res = {};
		for (var prop in I18.strings)
		{
			if (reg.test(I18.strings[prop])) res[prop] = I18.strings[prop];
		}
		return res;
   },
   
   _k: function(str)
   {
		var reg = new RegExp(str, 'img'), res = {};
		for (var prop in I18.strings)
		{
			if (reg.test(prop)) res[prop] = I18.strings[prop];
		}
		return res;
   }
};
