function NumberUtils()
{
}

NumberUtils.EPSILON = 1e-6;

NumberUtils.equals = function(d1, d2)
{
    return Math.abs(d1 - d2) < NumberUtils.EPSILON;
}

NumberUtils.compareTo = function(d1, d2) {
    var diff = d1 - d2;
    if (Math.abs(diff) < NumberUtils.EPSILON) 
    {
        return 0;
    }
    return diff < 0 ? -1 : 1;
}

NumberUtils.strictCompare = function(d1, d2) 
{
    var diff = d1 - d2;
    if (diff == 0) 
    {
        return 0;
    }
    return diff < 0 ? -1 : 1;
}

NumberUtils.isPositive = function(value) 
{
    return this.compareTo(value, 0) > 0;
}

NumberUtils.greaterThan = function(value, other) 
{
    return this.compareTo(value, other) > 0;
}

NumberUtils.greaterThanOrEqualsTo = function(value, other) 
{
    return this.compareTo(value, other) >= 0;
}

NumberUtils.floor = function(value) 
{
    return Math.floor(value + NumberUtils.EPSILON);
}

NumberUtils.ceil = function(value) 
{
    return Math.ceil(value - NumberUtils.EPSILON);
}

NumberUtils.roundDownToDivide = function(value, _multiplier) 
{
    if (value < 0) 
    {
        return 0;
    }
    if (value % _multiplier == 0) 
    {
        return value;
    }
    var cnt = value / _multiplier;
    return cnt * _multiplier;
}

NumberUtils.roundUpToDivide = function(value, _multiplier) 
{
    if (value < 0) 
    {
        return 0;
    }
    if (value % _multiplier == 0) 
    {
        return value;
    }
    var cnt = value / _multiplier;
    return (cnt + 1) * _multiplier;
}

NumberUtils.format = function(value, precision, decSep, thouSep) 
{
    if (typeof precision == 'undefined') precision = -1;
    if (typeof decSep == 'undefined') decSep = ".";
    if (typeof thouSep == 'undefined') thouSep = "";
    if (thouSep == decSep) thouSep = "";
    
    value = value*1 || 0;
    var ret = value + '';
    
    // round to precicion
    if (precision >= 0)
    {
    	// do round
    	var p = Math.pow(10, precision); 
    	value = Math.round(value * p) / p;
		
		ret = value + '';
		if (precision > 0)
		{
			// add zeros if precicion is more than decimal signs count
			if (ret.indexOf(".") < 0) ret = ret + ".";
			ret = ret.split('.');
			while (ret[1].length < precision) ret[1] = ret[1] + '0';
			ret = ret.join('.');
		}
    }
    
    // prepare output
    if (thouSep)
    {
    	// thousand separators
    	ret = ret.split('.');
    	for (var i = ret[0].length - 3; i > 0; i -= 3)
    	{
    		ret[0] = ret[0].substring(0, i) + thouSep + ret[0].substring(i);
    	}
    	ret = ret.join('.');
    }
    
   	// decimal separator
   	var d = ret.lastIndexOf(".");
    if (decSep && decSep != "." && (d > 0))
    {
   		ret = ret.substring(0, d) + decSep + ret.substring(d + 1);
    }

	return ret;
}

NumberUtils.floorTo = function(value, precision) 
{
    if (precision == 0) 
    {
        return this.floor(value);
    }

    var base = 1;
    for (var i = 0; i < precision; i++) 
    {
        base *= 10;
    }
    return this.floor(base * value) / base;
}

NumberUtils.clamp01 = function(value) 
{
    if (value > 1)
    {
        return 1;
    }
    if (value < 0)
    {
        return 0;
    }
    return value;
}

NumberUtils.clamp = function(value, min, max) 
{
    if (value < min)
    {
        return min;
    }
    if (value > max)
    {
        return max;
    }
    return value;
}

NumberUtils.clampAbs = function(value, limit) 
{
    limit = Math.abs(limit);
    return NumberUtils.clamp(value, -limit, limit);
}

NumberUtils.getFadedIndex = function(x, bounds) 
{
    var index = bounds.length - 2;
    while (index >= 0) 
    {
        if (x > bounds[index]) 
        {
            return index + 1;
        }
        index--;
    }
    return 0;
}

NumberUtils.limit = function(value, min, max) 
{
    return Math.max(min, Math.min(value, max));
}

NumberUtils.sing = function(value, signed) 
{
    if (value < 0) 
    {
        return "-";
    } 
    else 
    {
        return signed ? "+" : "";
    }
}

NumberUtils.sign = function(value)
{
	if(value == 0)
	{
		return 0;
	}
	
	return value > 0 ? 1 : -1;
}

NumberUtils.toNumberOrZero = function(s) 
{
    var r = parseFloat(s);
    return isNaN(r) ? 0 : r;
}
