var Strings = {}

Strings.EMAIL_REGEXP = /^[0-9a-zA-Z]([-\.\w]*[0-9a-zA-Z])*@([0-9a-zA-Z]([-\w]*[0-9a-zA-Z])*\.)+[a-zA-Z]{2,9}$/;

Strings.format = function(template) 
{
	var values = Array.prototype.slice.call(arguments, 1);
	for (var i = 0; i < values.length; i++)
	{
		template = template.replace('{'+i+'}', values[i]+'');
	}
	return template;
}

Strings.ellipsis = function(s, maxLength) 
{
    var dottes = "...";
    if (s.length <= maxLength) 
    {
        return s;
    }
    return s.substr(0, maxLength - dottes.length) + dottes;
}

Strings.removeLast = function(s) 
{
    return s.substr(0, s.length - 1);
}

Strings.formatNumber = function(n, floorOrCeilPrecision, doFloor) 
{
    if (typeof floorOrCeilPrecision == 'undefined') floorOrCeilPrecision = 2;
    if (typeof doFloor == 'undefined') doFloor = true;
    
    var res = (doFloor ? NumberUtils.floorTo(n, floorOrCeilPrecision) : n);
    
    //res = res.toFixed(floorOrCeilPrecision);
    
    res = String(res);
    
    if (res.indexOf(".") < 0)
    {
    	res += ".";
    }
    
    if (floorOrCeilPrecision > 0) 
    {
        while (res.substr(res.length - 1, 1) == '0') 
        {
            res = Strings.removeLast(res);
        }
    }
    if (res.substr(res.length - 1, 1) == '.') 
    {
        res = Strings.removeLast(res);
    }
    return res;
}

Strings.formatNumberPercent = function(n, signed, precision, doFloor) 
{
    if (typeof signed == 'undefined') signed = false;
    if (typeof precision == 'undefined') precision = 2;
    if (typeof doFloor == 'undefined') doFloor = true;
    
    return I18.f('PERCENT_NUMBER_PATTERN', [NumberUtils.sing(n, signed), Strings.formatNumber(Math.abs(n), precision, doFloor)]);
}

Strings.formatPercent = function(n, signed) 
{
    if (typeof signed == 'undefined') signed = false;
    return I18.f('PERCENT_NUMBER_PATTERN', [NumberUtils.sing(n, signed), String(Math.abs(n))]);
}

Strings.safeHtml = function(txt)
{
    txt = txt + ''; // cast it to string implicitly
    txt = txt.replace(new RegExp('<', 'g'), '&lt;');
    txt = txt.replace(new RegExp('>', 'g'), '&gt;');
    return txt;
};

/**
 * Boyer–Moore–Horspool algorithm
 * Modified for Unicode search
 * @see http://en.wikipedia.org/wiki/Boyer-Moore-Horspool_algorithm
 * @see http://ru.wikipedia.org/wiki/%D0%90%D0%BB%D0%B3%D0%BE%D1%80%D0%B8%D1%82%D0%BC_%D0%91%D0%BE%D0%B9%D0%B5%D1%80%D0%B0-%D0%9C%D1%83%D1%80%D0%B0-%D0%A5%D0%BE%D1%80%D1%81%D0%BF%D1%83%D0%BB%D0%B0
 * @return Array of indices found in the haystack
 */
Strings.findAll = function(haystack, needle, ignoreCase) 
{
    if (typeof ignoreCase == 'undefined') ignoreCase = false;
    
    var haystackLen = haystack.length;
    var needleLen = needle.length;

    if (needleLen > haystackLen) 
    {
        return [];
    }

    var last = needleLen - 1;
    var scan = 0;
    var code;
    var badCharSkip = {};

    for (scan = 0; scan < last; scan++) 
    {
        code = charCode(needle, scan, ignoreCase);
        if (badCharSkip[code] != null) 
        {
            badCharSkip[code] = Math.min(badCharSkip[code], last - scan);
        } 
        else 
        {
            badCharSkip[code] = last - scan;
        }
    }

    var res = [];
    var index = 0;
    var found = false;
    while (haystackLen - index >= needleLen) 
    {
        for (scan = last; Strings.charCode(haystack, index + scan, ignoreCase) == Strings.charCode(needle, scan, ignoreCase); scan--) 
        {
            if (scan == 0) 
            {
                res.push(index);
                found = true;
                index += needleLen;
                break;
            }
        }

        if (found) 
        {
            found = false;
        } 
        else 
        {
            code = Strings.charCode(haystack, index + last, ignoreCase);
            if (badCharSkip[code] == null) 
            {
                index += needleLen;
            } 
            else 
            {
                index += badCharSkip[code];
            }
        }
    }

    return res;
}

Strings.charCode = function(s, index, ignoreCase) 
{
    if (!ignoreCase) 
    {
        return s.charCodeAt(index);
    }
    return s.charAt(index).toLowerCase().charCodeAt(0);
}


Strings.isEmpty = function(str) 
{
    return !(str && str.length > 0);
}

Strings.formatBonusPerTime = function(value) 
{
    if (value > 0) 
    {
        return Strings.format("+{0}", I18.f('PER_TIME', value));
    }
    if (value < 0) 
    {
        return Strings.format("-{0}", I18.f('PER_TIME', -value));
    }
    return "";
}

Strings.formatBonusConst = function(value, original) 
{
    if (typeof original == 'undefined') original = NaN;
    return Strings.formatBonus(value, false, original);
}

Strings.formatBonusPercent = function(value) 
{
    value = value || 0;
    return Strings.formatBonus(NumberUtils.floor(value * 100), true);
}

Strings.formatBonus = function(value, isPercent, original, precision) 
{
    if (typeof original == 'undefined') original = NaN;
    if (typeof precicion == 'undefined') precision = 2;
    if (value == 0) 
    {
        if (original < 0) return "-0";
        if (original > 0) return "+0";
        return "";
    }
    if (isPercent) 
    {
        return Strings.formatNumberPercent(value, true, precision);
    } 
    else 
    {
        return (value > 0 ? "+" : "") + Strings.formatNumber(value, precision);
    }
}

Strings.separator3d = " ";

Strings.format3d = function(value) 
{
    var v = Math.round(value);
    if (v < 0) 
    {
        return "-" + Strings.format3d(-v);
    }
    var s = v.toString();
    var r = "";
    for (var z = 0, i = s.length; i >= 0; --i, ++z) 
    {
        r = s.charAt(i) + r;
        if (i > 0 && z > 0 && (z % 3) == 0) 
        {
            r = Strings.separator3d + r;
        }
    }
    return r;
}

Strings.parse3d = function(value) 
{
    return parseInt(value.replace(Strings.separator3d, ""));
}

Strings.isValidEmail = function(username) 
{
    if (typeof username != 'string' || username.length < 6) return false; // min length = x@x.xx
    if (username.indexOf('@') < 1 || username.indexOf('.') < 1) return false; // @. are required
    var u = username.split('@');
    if (u.length != 2) return false; // only 1 @
    if (u[1].indexOf('.') < 1) return false; // domain contains but not starts with .
    if (!/^[0-9a-zA-Z]([-\.\w]*[0-9a-zA-Z])*$/.test(u[0])) return false;
    if (!/^[0-9a-zA-Z]([-\.\w]*[0-9a-zA-Z])*$/.test(u[1])) return false;
    return true;//Strings.EMAIL_REGEXP.test(username)
}

Strings.joinColumns = function(array, column) 
{
    column = Math.max(1, column);
    var str = "";
    var i = 0;
    while (i < array.length) 
    {
        str += (str ? "\n" : "") + Arrays.join(array.slice(i, i + column), ", ");
        i += column;
    }
    return str;
}

Strings.join = function(source, delimiter, f)
{
    if (typeof delimiter == 'undefined') delimiter = '';
    var res = "";
    if (source) 
    {
        for (var i = 0, len = source.length; i < len; i++) 
        {
            res += (typeof f == 'function') ? f(source[i]) : source[i];
            if (i < source.length - 1) 
            {
                res += delimiter;
            }
        }
    }
    return res;
}

Strings.startsWith = function(text, substring)
{
    return text && substring && text.indexOf(substring) == 0;
}

Strings.endsWith = function(text, substring)
{
    if (!text || !substring) return false;
    var i = text.lastIndexOf(substring);
    return i >= 0 && i + substring.length == text.length;
}