function Tick()
{
}

Tick.MSECS_PER_TICK = 1000;
Tick.TICKS_PER_HOUR = 1000 * 60 * 60 / Tick.MSECS_PER_TICK;
Tick.TICKS_PER_DAY = Tick.TICKS_PER_HOUR * 24;

Tick.INFINITY_SYMBOL = "\u221e";

Tick._updateTick = 0;
Tick.afterUpdateMs = 0;
Tick.lastLocalTimeMs = 0;
Tick.TICK_INFINITY = 9223372036854776000;

Tick.masterTimer = setInterval(Events.onMasterTimerTick.fire, 1000);

Tick.now = function()
{
	return Tick._updateTick + Tick.fromMs(Tick.afterUpdateMs);
};

Tick.fromMs = function(ticks)
{
	return ticks / Tick.MSECS_PER_TICK;
}

Tick.updateTick = function(tick)
{
    Tick._updateTick = tick;
    Tick.afterUpdateMs = 0;
}

Tick.updateLocalTime = function()
{
	if(Tick.now() == 0) return;
	
	var localTimeMs = Tick.timeInMs();
    // normal situation
    if (localTimeMs - Tick.lastLocalTimeMs > 0)
    {
        Tick.afterUpdateMs += Tick.lastLocalTimeMs < 1 ? 0 : (localTimeMs - Tick.lastLocalTimeMs);
    }
    Tick.lastLocalTimeMs = localTimeMs;
}

Tick.timeInMs = function()
{
	return (new Date()).getTime();
}

Tick.toTimeFuture = function(futureTick/*:Number*/)
{
    if (Tick.isInfinity(futureTick)) 
    {
        return Tick.INFINITY_SYMBOL;
    }
    return DateUtils.formatSmart(Tick.toMs(futureTick));
}

Tick.isInfinity = function(tick/*:Number*/)
{
    return !isFinite(tick) || tick == Tick.TICK_INFINITY;
}

Tick.toMs = function(ticks/*:Number*/)
{
    return ticks * Tick.MSECS_PER_TICK; 
}

Tick.ticksTill = function(futureTick)
{
	return futureTick - Tick.now();
}

Tick.hasPassed = function(futureTick)
{
	return Tick.ticksTill(futureTick) <= 0;
}

Tick.toTimeLeft = function(ticksLeft)
{
    if (Tick.isInfinity(ticksLeft)) return Tick.INFINITY_SYMBOL;
    if (ticksLeft < 0) return "...";
    
    var s = Math.ceil(ticksLeft);
    
    var d = Math.floor(s / (3600*24)); 
    s = s % (3600*24);
    
    var h = Math.floor(s / 3600); 
    s = s % 3600;
    
    var m = Math.floor(s / 60); 
    s = s % 60;
    
    var minDays = 400, // "N days" format
    	minHours = 999 // "N hours" format
    ;
    if (d >= minDays) 
    {
    	//d = d + " " + I18.getString("D" + Tick.getNaklon(d, false));
    	//h = h ? " " + h + " " + I18.getString("H" + Tick.getNaklon(h, false)) : "";
    	return d + " " + I18.getString("D" + Tick.getNaklon(d, false));
    }
	
	h += d*24;
	if (h > minHours)
	{
		return h + " " + I18.getString("H" + Tick.getNaklon(h, false));
	}
    if (h > 0)
    {
        if (String(m).length < 2) m = "0" + m;
        if (String(s).length < 2) s = "0" + s;
        return Strings.format("{0}:{1}:{2}", h, m, s);
    }
    if (m > 0)
    {
        if (String(s).length < 2) s = "0" + s;
        return Strings.format("{0}:{1}", m, s);
    }
    return s.toString();
}

Tick.timeTill = function(futureTick)
{
	return Tick.toTimeLeft(Tick.ticksTill(futureTick));
}

Tick.toTimeLeftFull = function(ticksLeft/*:Number*/, on/*:Boolean = false*/)
{
    if (Tick.isInfinity(ticksLeft)) 
    {
        return Tick.INFINITY_SYMBOL;
    }
    if (ticksLeft < 0) 
    {
        return "...";
    }
    var sTotal = Math.ceil(ticksLeft);
    var d = Math.floor(sTotal/60/60/24);
    var h = Math.floor(sTotal/60/60) - d*24;
    var m = Math.floor(sTotal/60) - h*60 - d*24*60;
    var s = sTotal - h*60*60 - m*60 - d*24*60*60;

    var result = "";
    if (d > 0) 
    {
        result += d + " " + I18.getString("D" + Tick.getNaklon(d, false));
    }
    if (h > 0) 
    {
        if (result.length > 0) result += " ";
        result += h + " " + I18.getString("H" + Tick.getNaklon(h, false));
    }
    if (m > 0) 
    {
        if (result.length > 0) result += " ";
        result += m + " " + I18.getString("M" + Tick.getNaklon(m, on));
    }
    if (s > 0 || result == "") 
    {
        if (result.length > 0) result += " ";
        result += s + " " + I18.getString("S" + Tick.getNaklon(s, on));
    }

    return result;
}

Tick.getNaklon = function(n/*:Number*/, on/*:Boolean*/)
{
    var i = n;
    if (i != n) 
    {
        return 1;
    }
    i = i % 100;
    if (i > 10 && i < 20) return 2;
    i = i % 10;
    if (i == 1) return on ? 3 : 0;
    if (i < 5 && i > 1) return 1;
    return 2;
}

Tick.toDays = function(ticks)
{
    return Math.floor(ticks / (Tick.TICKS_PER_HOUR * 24));
}

Tick.happensAt = function(tick)
{
    if (Tick.isInfinity(tick))
    {
        return Tick.INFINITY_SYMBOL;
    }
    return DateUtils.happensAt(Tick.toMs(tick));
}

/*-------------------------------------------------------------------------------------------------*/
    // the setter

Tick.inFuture = function(tick)
{
    return Tick.now() < tick;
}

Tick.toTimeFutureWithYear = function(futureTick)
{
    if (Tick.isInfinity(futureTick)) 
    {
        return Tick.INFINITY_SYMBOL;
    }
    return DateUtils.formatWithYear(Tick.toMs(futureTick));
}

Tick.toTimeFutureSeconds = function(futureTick, ignoreDate)
{
    if (Tick.isInfinity(futureTick)) 
    {
        return Tick.INFINITY_SYMBOL;
    }
    return DateUtils.formatWithSeconds(Tick.toMs(futureTick), ignoreDate);
}

Tick.truncateTime = function(d)
{
    d.setMilliseconds(0);
    d.setSeconds(0);
    d.setMinutes(0);
    d.setHours(0);
    return d;
}

Tick.schedule = function(ticks, action)
{
    /*
    var timer:Timer = new Timer(Math.floor(Math.max(ticks, 0)) * MSECS_PER_TICK, 1);
    timer.addEventListener(TimerEvent.TIMER, function(e:TimerEvent):void {
        action();
    });
    timer.start();
    return timer;
    */
   action();
}

Tick.toHours = function(ticks)
{
    return Math.floor(ticks / Tick.TICKS_PER_HOUR);
}


Tick.toTimeWithDate = function(tick)
{
    if (Tick.isInfinity(tick)) 
    {
        return Tick.INFINITY_SYMBOL;
    }
    return DateUtils.formatWithDate(Tick.toMs(tick));
}
