var UI = {};

UI.show = function(doShow, components, onShow, onHide) // :void 
{
    return UI.visible(doShow, components, onShow, onHide);
}

UI.visible = function(doShow, components, onShow, onHide) // :void 
{
    if (doShow && onShow) 
    {
        onShow();
    }
    if (!doShow && onHide) 
    {
        onHide();
    }
    for (var i = 0; i < components.length; i++)
    {
    	var obj = components[i]['DOM'] || components[i];
    	if (obj && obj instanceof HTMLElement)
    	{
    		obj.style.display = doShow ? '' : 'none';
    	}
    }
    return Boolean(doShow);
}

UI.image = function(source) // :Image 
{
    var img = new Image();
    img.src = source;
    return img;
}

UI.label = function(text) // :Label 
{
    var l = document.createElement('span');
    l.innerHTML = String(text);
    return l;
}

UI.bonusLabelConst = function(value) // :Label 
{
    return UI.styledLabel(Strings.formatBonusConst(value), (value > 0 ? 'positive' : 'negative'));
}

UI.bonusLabelPercent = function(value) // :Label 
{
    return UI.styledLabel(Strings.formatBonusPercent(value), (value > 0 ? 'positive' : 'negative'));
}

UI.stylePercentBonus = function(value, label, limit) // :void 
{
    if (typeof limit != 'undefined') return UI.stylePercentBonusWithLimit(value, label, limit);
    label.innerHTML = Strings.formatBonusPercent(value);
    $(label).addClass('bonus');
}

UI.stylePercentBonusWithLimit = function(value, label, limit) // :void 
{
    if (NumberUtils.compareTo(value, limit) <= 0) 
    {
        label.innerHTML = Strings.formatBonusPercent(limit);
        $(label).addClass('overflow');
    } 
    else 
    {
        label.innerHTML = Strings.formatBonusPercent(value);
    	$(label).addClass('bonus');
    }
}

UI.styledLabel = function(text, style) // :Label 
{
    var l = UI.label(text);
    if (style) 
    {
        $(l).addClass(style.name ? style.name : style);
    }
    return l;
}

UI.styledText = function(text, style) // :Label 
{
    return UI.styledLabel(text, style);
}

UI.hideIfEmpty = function(component) // :Boolean 
{
	var obj = component.DOM || component;
	if (!obj) return;
    var hasVisibleChild = $('>*:visible', obj).length > 0;
    if (!hasVisibleChild) 
    {
        UI.hide(obj);
    }
    return !hasVisibleChild;
}

UI.showOnlyIfHasVisibleChildren = function(component) // :Boolean 
{
	var obj = component.DOM || component;
	if (!obj) return;
    var hasVisibleChild = $('>*:visible', obj).length > 0;
    if (hasVisibleChild) 
    {
        UI.show(true, [obj]);
    } 
    else 
    {
        UI.hide(obj);
    }
    return hasVisibleChild;
}

UI.hide = function(component) // :void 
{
	var obj = component.DOM || component;
	if (!obj) return;
	obj.style.display = 'none';
}

UI.text = function(text, color, fontWeight, centered) // :UIComponent 
{
    var res = UI.label(text);
    res.style.fontWeight = fontWeight;
    res.style.background = color;
    return res;
}

UI.line = function(text, color, fontWeight, centered) // :UIComponent 
{
    var res = document.createElement('div');
    res.innerHTML = String(text);
    res.style.fontWeight = fontWeight;
    res.style.background = color;
    return res;
}

UI.button = function(label, handler) // :Button 
{
    var b = COM.create('SimpleButton');
    b.caption = I18.f(label);
    b.onClick = handler;
    return b
}

UI.labelWithTip = function(text, onToolTipCreate) // :Label 
{
    var label = UI.label(text);
    label.style.textDecoration = "underline";
    label.style.cursor = 'help';
    return label;
}

UI.makeNChilds = function(c, n, childFactory)
{
    var parent = c['DOM'] || c, node = null;
    if (!parent) return;
    
    var len = parent.children.length;
    while (len > n) 
    {
        node = parent.children[0];
        if (node.getAttribute('role') == 'component')
        {
        	COM(node).destroy();
        }
        else 
        {
        	parent.removeChild(parent.children[0]);
        }
        len--;
    }
    while (len < n) 
    {
        var child = childFactory();
        var node = child['DOM'] || child;
        if (!node) break;
        
        parent.appendChild(node);
        len++;
    }
}

UI.denseHBox = function()
{
	var res = document.createElement('div');
    //res.setStyle("horizontalGap", 0);
    //res.setStyle("verticalAlign", "middle");
    return res;
}