function Observer()
{
	var self = this;
	
	this.listeners = [];
	
	this.getIx = function(context)
	{
		for(var i = 0, len = self.listeners.length; i < len; i++)
		{
			if (self.listeners[i].context === context) 
			{
				return i;
			}
		}
		return -1;
	}
	
	this.has = function(context)
	{
		return self.getIx(context) >= 0 ? true : false;
	}
	
	this.add = function(context, l)
	{
		if(!context) throw Error("Context is required for event listener");
		if(!self.has(context))
		{
			self.listeners.push({"context": context, "listener": l});
		}
	}
	
	this.remove = function(context)
	{
		var ix;
		while ((ix = self.getIx(context)) >= 0)
		{
			self.listeners.splice(ix, 1);
		}
		
	}
	
	this.fire = function(param)
	{
		var ret;
		for(var i=0; i<self.listeners.length; i++)
		{
			ret = self.listeners[i].listener.apply(
				self.listeners[i].context, 
				Array.prototype.slice.call(arguments)
			);
			if(ret === false)
			{
				return;
			}
		}
	}
}

var Events =
{
    onShowCity: new Observer(),
    onShowEspionageResult: new Observer(), // c:City
    onShowExpeditionResult: new Observer(), // c:City
    onShowKingdom: new Observer(), // k:Kingdom

    onError: new Observer(), // error:Error
    onReloadConfig: new Observer(), // empty
    onMissionCancel: new Observer(), // m:FMission

    onFEstateUpdated: new Observer(),
    onFKingdomUpdated: new Observer(),
    onFUserUpdated: new Observer(),
    onMailboxStatusChange: new Observer(),

    onConnectionLost: new Observer(),
    onConnectionGain: new Observer(),

    onServerResultReceived: new Observer(),
    onExpandBuildQueue: new Observer(), // doExpand:Boolean

    onShowingAnimationChange: new Observer(), // Boolean

    onArtifactDragBegin: new Observer(), // ArtifactSlotView
    onArtifactDragEnd: new Observer(), // ArtifactSlotView

    onResourceChooserStepIndexChanged: new Observer(),
    onArmyChooserStepIndexChanged: new Observer(),
	
	onMasterTimerTick: new Observer(),
	
	onFinanceChanged: new Observer(),
	
	onForceCityUpdate: new Observer(),
}