function AddTowerDialog()
{
	callParentConstructor(AddTowerDialog.prototype, this);
	
	this.initialized = false;
	
	this.selected = null;
	
	this._canAddTower = true;
	this._canAddGate = true;
	this._canSeeGate = true;
	
	this.race = null;
	
	this.images = [];
	
	this.onAdd = function(btype, level, health) {};
	
	this.imageClicked = $.proxy(this.imageClicked, this);
}

AddTowerDialog.MAX_CHARS = 9;
AddTowerDialog.MAX_UNIT_COUNT = 999999999;

extendClass(AddTowerDialog.prototype, Dialog.prototype);

AddTowerDialog.prototype.__construct = function()
{
	var img;
	
	img = this.addFortification;
	img.data = BType.FORTIFICATION;
	img.src = STATIC_URL + CommonAssets.FORTIFICATION;
	COM.bindExclusiveClick(img, this.imageClicked);
	this.images.push(img);
	
	img = this.addTower;
	img.data = BType.TOWER;
	img.src = STATIC_URL + CommonAssets.TOWER;
	COM.bindExclusiveClick(img, this.imageClicked);
	this.images.push(img);
	
	img = this.addMagicTower;
	img.data = BType.MAGIC_TOWER;
	img.src = STATIC_URL + CommonAssets.MAGIC_TOWER;
	COM.bindExclusiveClick(img, this.imageClicked);
	this.images.push(img);
	
	img = this.addGate;
	img.data = BType.CASTLEGATE;
	img.src = STATIC_URL + CommonAssets.CASTLEGATE;
	COM.bindExclusiveClick(img, this.imageClicked);
	this.images.push(img);
	
	COM.bindExclusiveClick(this.addButton, $.proxy(this.addHandler, this));
	
	this.__defineSetter__("canAddTower", this.setCanAddTower);
	this.__defineSetter__("canAddGate", this.setCanAddGate);
	this.__defineSetter__("canSeeGate", this.setCanSeeGate);
	
	this.skin = "black";
	this.header.style.display = "none";
}

AddTowerDialog.prototype.popup = function(parent)
{
    superMethod(AddTowerDialog.prototype, "popup", this)(parent);
    this.initialized = true;
}

AddTowerDialog.prototype.close = function()
{
	this.DOM.style.display = "none";
}

AddTowerDialog.prototype.init = function(race)
{
	this.race = race;
	COM(this.healthInput).min = 0;
	COM(this.healthInput).value = 0;
    COM(this.levelSwitch).setLevel(1);
    COM(this.levelSwitch).onChange = $.proxy(this.onChangeLevel, this);
    this.chooseTower(BType.FORTIFICATION);
	this.updateCanAdd();
}

AddTowerDialog.prototype.imageClicked = function(e)
{
    if (!e.target.disabled)
    {
        this.chooseTower(e.target.data);
    }
}

AddTowerDialog.prototype.chooseTower = function(btype)
{
    this.selected = btype;
    for(var i=0; i<this.images.length; i++)
    {
    	//this.images[i].style.border = this.images[i].data.name == this.selected.name ? "2px solid #000" : "";
    	if(this.images[i].data.name == this.selected.name) $(this.images[i].parentNode).addClass("selected");
    	else $(this.images[i].parentNode).removeClass("selected");
    }
    
    var levels = btype.getLevels(this.race);

    var newMaxLevel = levels.length - 1;
    var newShowPerfect = levels[levels.length - 1].perfect;
    var levelSwitch = COM(this.levelSwitch);
    if (newMaxLevel != levelSwitch.maxLevel || newShowPerfect != levelSwitch.showPerfect)
    {
        var oldSelectedLevel = levelSwitch.level;
        levelSwitch.maxLevel = levels.length - 1;
        levelSwitch.showPerfect = levels[levels.length - 1].perfect;
        levelSwitch.init();
        if (oldSelectedLevel <= newMaxLevel)
        {
            levelSwitch.setLevel(oldSelectedLevel);
        }
    }
    
    if (UI.show(btype.name === BType.CASTLEGATE.name, [this.healthInput]))
    {
    	if (!COM(this.healthInput).value) this.onChangeLevel();
    }
}

AddTowerDialog.prototype.onChangeLevel = function()
{
    if (this.selected.name === BType.CASTLEGATE.name)
    {
        var levels = BType.CASTLEGATE.getParams(Race.CASTLE_MONSTER).levels;
        if (COM(this.levelSwitch).level >= 0 && COM(this.levelSwitch).level < levels.length)
        {
            COM(this.healthInput).value = levels[COM(this.levelSwitch).level].health * 1;
        }
    }
}


AddTowerDialog.prototype.addHandler = function()
{
    this.onAdd(this.selected, COM(this.levelSwitch).level, COM(this.healthInput).value, COM(this.countInput).value);
}

AddTowerDialog.prototype.setCanAddTower = function(value)
{
    this._canAddTower = value;
	this.updateCanAdd();
}

AddTowerDialog.prototype.setCanAddGate = function(value)
{
    this._canAddGate = value;
	this.updateCanAdd();
}

AddTowerDialog.prototype.setCanSeeGate = function(value)
{
    this._canSeeGate = value;
	this.updateCanAdd();
}

AddTowerDialog.prototype.updateCanAdd = function()
{
    if (!this._canAddTower)
    {
        this.addTower.disabled = true;
        this.addMagicTower.disabled = true;
        this.addTower.style.webkitFilter = "grayscale(100%)";
        this.addMagicTower.style.webkitFilter = "grayscale(100%)";
        if (this.selected && this.selected.isTower())
        {
            this.chooseTower(BType.FORTIFICATION);
        }
    }
    else
    {
        this.addTower.disabled = false;
        this.addMagicTower.disabled = false;
        this.addTower.style.webkitFilter = "grayscale(0%)";
        this.addMagicTower.style.webkitFilter = "grayscale(0%)";
    }
    if (!this._canAddGate)
    {
        this.addGate.disabled = true;
        this.addGate.style.webkitFilter = "grayscale(100%)";
        
        if (this.selected && this.selected.name === BType.CASTLEGATE.name)
        {
            this.chooseTower(BType.FORTIFICATION);
        }
    }
    else
    {
        this.addGate.disabled = false;
        this.addGate.style.webkitFilter = "grayscale(0%)";
    }
    
    this.addGateHolder.style.display = this._canSeeGate ? "" : "none";
}