function CalcDefenseView()
{
	this.onChange = function(){};

    this.gateHealth = 0;

    this.addTowerDialog = null;
    this.towersCount = 0;
    
    this.parent = null;
    
    this.__defineGetter__("defenderLevel", this.getDefenderLevel);
    this.__defineSetter__("defenderLevel", this.setDefenderLevel);
	this.__defineGetter__("maxDefense", this.getMaxDefense);
	this.__defineSetter__("maxDefense", this.setMaxDefense);
	this.__defineSetter__("canSeeGate", this.setCanSeeGate);
	this.__defineGetter__("towerLevels", this.getTowerLevels);
}

CalcDefenseView.prototype.__construct = function()
{
    COM(this.specLevelSwitch).setLevel(0);
    this.defIcon.src = STATIC_INTERFACE_URL + Assets.getScienceImage(Science.SPEC_DEFENDER);
    COM.bindExclusiveClick(this.addTowerButton, $.proxy(this.showTowerDialog, this));
    
    this.addTowerDialog = COM.create("AddTowerDialog");
    this.addTowerDialog.onAdd = $.proxy(this.addTower, this);
    
    COM(this.addTowerButton).icon = STATIC_INTERFACE_URL + "shared/icons/plus.png";
    
    $(this.maxDefenseSwitch).on("change", $.proxy(function()
    {
    	this.onChange();
    }, this))
}

CalcDefenseView.prototype.addTower = function(btype, level, health, count)
{
    if(typeof count == "undefined")
    {
    	count = 1;
    }
    
    if (level >= btype.getLevels(this.parent.race).length)
    {
        Alert.create("No config data for this level of tower");
        return;
    }
    
    var towerView = null;
    var towers = $("> calctowerview", this.towersHolder);
    for(var i=0; i<towers.length; i++)
    {
    	var c = COM(towers[i]);
    	if(c.matches(btype, level))
    	{
    		towerView = c;
    		break;
    	}
    }
    
    if (btype.isTower())
    {
    	if(this.towersCount+count > 2)
    	{
    		count = 2 - this.towersCount;
    	}
    }
    
    if (btype.name === BType.CASTLEGATE.name)
    {
    	count = 1;
    }
    
    if (towerView)
    {
        towerView.increaseCount(count);
    }
    else
    {
        var c = CalcTowerView.create(btype, level, $.proxy(this.removeTower, this), $.proxy(this.onDecreaseTowerCount, this));
        c.increaseCount(count-1);
        this.towersHolder.appendChild(c.DOM);
    }

    if (btype.isTower())
    {
        this.towersCount += count;
        this.addTowerDialog.canAddTower = this.towersCount < 2;
    }
    if (btype.name === BType.CASTLEGATE.name)
    {
        this.addTowerDialog.canAddGate = false;
        this.gateHealth = health;
    }
    
    this.addTowerDialog.DOM.style.display = "none";
    
    this.onChange();
}

CalcDefenseView.prototype.__destruct = function()
{
	this.addTowerDialog.destroy();
}

CalcDefenseView.prototype.showTowerDialog = function()
{
    if (!this.addTowerButton.disabled)
    {
        this.addTowerDialog.init(this.parent.race);
        
        if(!this.addTowerDialog.initialized)
        {
        	this.addTowerDialog.popup(this);
        }
        else
        {
        	this.addTowerDialog.DOM.style.display = "";
        }
    }
}

CalcDefenseView.prototype.onDecreaseTowerCount = function(c)
{
    if (c.btype.isTower())
    {
        this.towersCount--;
        this.addTowerDialog.canAddTower = true;
    }
    if (c.btype.name === BType.CASTLEGATE.name)
    {
        this.addTowerDialog.canAddGate = true;
    }
    this.onChange();
}

CalcDefenseView.prototype.removeTower = function(c)
{
    c.destroy();
    this.onChange();
}

CalcDefenseView.prototype.collectLevels = function(btype)
{
    var result = [];
    if (this.towersHolder.style.display != "none")
    {
        var towers = $("> calctowerview", this.towersHolder);
        var child;
	    for(var i=0; i<towers.length; i++)
	    {
	    	child = COM(towers[i]);
	    	if(child.btype.name == btype.name)
	    	{
	    		for (var n=0; n<child.count; n++)
	    		{
                    result.push(child.level);
                }
	    	}
	    }
    }
    return result;
}

CalcDefenseView.prototype.clearAll = function()
{
    COM(this.specLevelSwitch).setLevel(0);
    var towers = $("> calctowerview", this.towersHolder);
    for(var i=0; i<towers.length; i++)
    {
    	COM(towers[i]).destroy();
    }
    this.towersCount = 0;
    this.gateHealth = 0;
    this.addTowerDialog.canAddTower = true;
    this.addTowerDialog.canAddGate = true;
}
////
CalcDefenseView.prototype.getDefenderLevel = function()
{
	return COM(this.specLevelSwitch).level;
}

CalcDefenseView.prototype.setDefenderLevel = function(newLevel)
{
    COM(this.specLevelSwitch).setLevel(newLevel);
}

CalcDefenseView.prototype.getMaxDefense = function()
{
    return COM(this.maxDefenseSwitch).checked;
}

CalcDefenseView.prototype.setMaxDefense = function(on)
{
    COM(this.maxDefenseSwitch).checked = on;
    this.onChange();
}

CalcDefenseView.prototype.setCanSeeGate = function(b)
{
    this.addTowerDialog.canSeeGate = b;
}

CalcDefenseView.prototype.getTowerLevels = function()
{
    var map = new FMap();
    map.put(BType.FORTIFICATION, this.collectLevels(BType.FORTIFICATION));
    map.put(BType.TOWER, this.collectLevels(BType.TOWER));
    map.put(BType.MAGIC_TOWER, this.collectLevels(BType.MAGIC_TOWER));
    map.put(BType.CASTLEGATE, this.collectLevels(BType.CASTLEGATE));
    return map;
}