function ChooseArmyView()
{
	this.armyChanged = $.proxy(this.armyChanged, this);
	
	this.combatRole = CombatRole.DEFENDER;
	this.username = null;
	this.isAlly = false;
	this.isMyself = false;
	this.isMyselfPresent = true;
	
	this.onClose = null;
	this.onClearAll = function (){};
	this.showDefense = false;
	this.onAuraChanged = function (){};
	this.armyInBattle = new ArmyInBattle();
	
	this.buffs = [];
	this.fireRecalculateBuffs = function (){};
	
	this.__defineSetter__("aura", this.setAura);
	this.__defineGetter__("selectedHero", this.getSelectedHero);
	this.__defineGetter__("race", this.getRace);
	this.__defineGetter__("defenderLevel", this.getDefenderLevel);
	this.__defineGetter__("maxDefense", this.getMaxDefense);
	this.__defineGetter__("towerLevels", this.getTowerLevels);
	this.__defineGetter__("gateHealth", this.getGateHealth);
}

ChooseArmyView.prototype.__construct = function()
{
	COM(this.defenseView).parent = this;
	COM(this.defenseView).onChange = this.armyChanged;
	COM(this.armyInput).onAllChangeLevels = $.proxy(this.onLevelChanged, this);
	COM(this.armyInput).onChangeHero = this.armyChanged;
	
	$(this.raceCombo).on("change", this.armyChanged);
	$(this.betrayerSwitch).on("change", this.armyChanged);
	
	COM(this.allLevel).onChange = $.proxy(this.allUnitsLevelChanged, this);
	
	COM(this.clearBtn).icon = STATIC_INTERFACE_URL + "shared/icons/clear.png";
	COM(this.closeBtn).icon = STATIC_INTERFACE_URL + "shared/icons/close.png";
	
	COM.bindExclusiveClick(this.clearBtn, $.proxy(this.clearAll, this));
	COM.bindExclusiveClick(this.closeBtn, $.proxy(function()
	{
		if(this.onClose)
		{
			this.onClose(this);
		}
	}, this));
	
	COM(this.retreatLimit).min = FConfig.MIN_RETREAT_PERCENT;
	COM(this.retreatLimit).max = 100;
	COM(this.retreatLimit).size = 3;
	
	var rc = COM(this.raceCombo);
	rc.dataProvider = FConfig.CASTLES_ENABLED ? Race.ALL : Arrays.remove(Race.ALL, Race.CASTLE_MONSTER);
	rc.selectedIndex = 2;
	COM(rc.list).visibleItems = rc.dataProvider.length;
	
	COM(this.armyInput).race = this.race;
    
    this.init();
}

ChooseArmyView.prototype.init = function()
{
	//COM(this.armyInput).update(Army.create(this.armyInBattle.army), this.armyInBattle.levels);
	this.updateStatus(this.isMyself, this.isAlly, this.isMyselfPresent);
    this.clearAll();
}

ChooseArmyView.prototype.updateStatus = function(isMyself, isAlly, isMyselfPresent)
{
    if (this.isMyself != isMyself)
    {
        this.clearAll();
    }
    this.isMyself = isMyself;
    this.isAlly = isAlly;
    this.myselfMark.style.display = isMyself ? "" : "none";
    
    /*
    this.line1.style.background = isMyselfPresent ? (isAlly ? Effects.LIGHT_GREEN : Effects.LIGHT_RED) : Effects.LIGHT_ORANGE;
    this.DOM.style.background = isMyselfPresent ? (isAlly ? Effects.SUPER_LIGHT_GREEN : Effects.SUPER_LIGHT_RED) : Effects.SUPER_LIGHT_ORANGE;
    */
   
	this.armyInfo.style.background = isMyselfPresent ? (isAlly ? Effects.LIGHT_GREEN : Effects.LIGHT_RED) : Effects.LIGHT_ORANGE;
    
    this.combatRoleLabel.innerHTML = this.combatRole.toString();
    
    this.combatRoleLabelImage.src = STATIC_INTERFACE_URL + "shared/icons/" + (this.combatRole.name == CombatRole.ATTACKER.name ? "attacker.png" : "defender.png");
    
    this.closeBtn.style.display = this.onClose ? "" : "none";
}

ChooseArmyView.prototype.clearAll = function()
{
    COM(this.betrayerSwitch).checked = false;
    COM(this.retreatLimit).value = COM(this.retreatLimit).min;
    COM(this.defenseView).clearAll();
    var aib = new ArmyInBattle();
    aib.race = Race.ELF;
    aib.levels = COM(this.armyInput).getLevelsArray();
    aib.army = [0, 0, 0, 0, 0, 0, 0, 0];
    this.setArmyInBattle(aib, null);
    COM(this.allLevel).setLevel(1);
    if (this.onClearAll)
    {
        this.onClearAll();
    }
}

ChooseArmyView.prototype.setArmyInBattle = function(newArmy, heroes)
{
    this.armyInBattle = newArmy;
    
    var cb = COM(this.raceCombo);
    for(var i=0; i<cb.dataProvider.length; i++)
    {
    	if(cb.dataProvider[i].name == newArmy.race.name)
    	{
    		cb.selectedIndex = i;
    		break;
    	}
    }

	var ai = COM(this.armyInput);
    ai.race = newArmy.race;
    ai.buffs = this.buffs;
    ai.heroes = heroes;
    ai.selectedHero = newArmy.hero;
    ai.update(Army.create(newArmy.army), newArmy.levels);

    COM(this.betrayerSwitch).checked = newArmy.betrayer;
    this.username = newArmy.username;
    UI.show(this.showDefense, [this.defenseView]);
    this.updateRetreatLimit();
    this.onAuraChanged();
}

ChooseArmyView.prototype.updateRetreatLimit = function()
{
    if (this.race.name == Race.MONSTER.name)
    {
        COM(this.retreatLimit).disabled = true;
        COM(this.retreatLimit).value = COM(this.retreatLimit).max;
    }
    else
    {
        COM(this.retreatLimit).disabled = false;
        COM(this.retreatLimit).value = this.armyInBattle.retreatLimit > 0 ? this.armyInBattle.retreatLimit : FUser.CURRENT_USER.settings.getCalculatorRetreatLimit(this.combatRole);
    }
}

ChooseArmyView.prototype.onLevelChanged = function(level)
{
    COM(this.allLevel).setLevel(level);
}

ChooseArmyView.prototype.armyChanged = function()
{
    if (this.race)
    {
        this.fireRecalculateBuffs();
        this.onAuraChanged();
        var ai = COM(this.armyInput);
        ai.race = this.race;
        ai.buffs = this.buffs;
        ai.refresh();
        this.updateRetreatLimit();
    }
}

ChooseArmyView.prototype.getArmyInBattle = function()
{
    var result = new ArmyInBattle();
    result.army = COM(this.armyInput).getArmyArray();
    result.levels = COM(this.armyInput).getLevelsArray();
    result.race = this.race;
    result.betrayer = COM(this.betrayerSwitch).checked;
    result.neutral = Race.MONSTER.equals(this.race);
    result.retreatLimit = COM(this.retreatLimit).value*1;
    result.username = this.isMyself ? FUser.CURRENT_USER.username : (this.username ? this.username : this.combatRole.toString());
    result.hero = this.armyInBattle.hero;
    result.estateName = this.armyInBattle.estateName;
    result.estateXYZ = this.armyInBattle.estateXYZ;
    result.nonHeroCasts = this.armyInBattle.nonHeroCasts;
    return result;
}

ChooseArmyView.prototype.setDefender = function(info)
{
    var dv = COM(this.defenseView);
    
    dv.clearAll();
    dv.defenderLevel = info.defenderLevel;
    dv.maxDefense = info.maxDefence;
    var level;
    
    for(var i=0; i<info.magicTowers.length; i++)
    {
    	dv.addTower(BType.MAGIC_TOWER, info.magicTowers[i], 0);
    }
    
    for(var i=0; i<info.towers.length; i++)
    {
    	dv.addTower(BType.TOWER, info.towers[i], 0);
    }
    
    for(var i=0; i<info.fortifications.length; i++)
    {
    	dv.addTower(BType.FORTIFICATION, info.fortifications[i], 0);
    }
    
    if (info.gateLevel > 0)
    {
        dv.addTower(BType.CASTLEGATE, info.gateLevel, info.gateHealth);
    }
    
    dv.canSeeGate = info.estateType.name === EstateType.CASTLE.name;
}

ChooseArmyView.prototype.allUnitsLevelChanged = function()
{
    var al = COM(this.allLevel);
    if (al.level > 0 && al.level <= 4)
    {
        COM(this.armyInput).setAllLevels(al.level);
    }
}

ChooseArmyView.prototype.setAura = function(value)
{
    COM(this.armyInput).aura = value;
    COM(this.armyInput).refresh();
}

ChooseArmyView.prototype.getSelectedHero = function()
{
    return COM(this.armyInput).selectedHero;
}

ChooseArmyView.prototype.getRace = function()
{
    return this.raceCombo ? COM(this.raceCombo).selectedItem.data : null;
}

ChooseArmyView.prototype.getDefenderLevel = function()
{
    return COM(this.defenseView).defenderLevel;
}

ChooseArmyView.prototype.getMaxDefense = function()
{
    return COM(this.defenseView).maxDefense;
}

ChooseArmyView.prototype.getTowerLevels = function()
{
    return COM(this.defenseView).towerLevels;
}

ChooseArmyView.prototype.getGateHealth = function()
{
    return COM(this.defenseView).gateHealth;
}