function CombatCalculatorView()
{
	callParentConstructor(CombatCalculatorView.prototype, this);
	
    this.estateMilitaryInfos = {};
    this.currentEstateInfo = null;
    this.attackMissions = [];
    this.updating = false;
    
    this.calcsCount = 0;
    
    this.scroll = false;
    
    this.__defineGetter__("maxAttackers", this.getMaxAttackers);
    this.__defineGetter__("maxDefenders", this.getMaxDefenders);
    this.__defineGetter__("estateType", this.getEstateType);
    
    this.updateCalculationsLeft = $.proxy(this.updateCalculationsLeft, this);
}

extendClass(CombatCalculatorView.prototype, Dialog.prototype);

CombatCalculatorView.instance = null;

CombatCalculatorView.popup = function(currentEstateInfo, attackers)
{
    if (!CombatCalculatorView.instance)
    {
        CombatCalculatorView.instance = COM.create("CombatCalculatorView");
        CombatCalculatorView.instance.skin = "nodecor";
        
        CombatCalculatorView.instance.setCaption(I18.f("COMBAT_CALCULATOR"));
    }
    
    CombatCalculatorView.instance.currentEstateInfo = currentEstateInfo;
    CombatCalculatorView.instance.attackMissions = attackers;
    
    APP.showLoader();
    Service.user.callMethod("getEstateMilitaryInfos", [], function(infos)
    {
        CombatCalculatorView.instance.estateMilitaryInfos = infos;
		CombatCalculatorView.instance.update();
		CombatCalculatorView.instance.popup();
		CombatCalculatorView.instance.fullScreen();
		
		APP.hideLoader();
	});
}

CombatCalculatorView.prototype.onClose = function()
{
	Events.onFUserUpdated.remove(this, this.updateCalculationsLeft);
	CombatCalculatorView.instance = null;
}

CombatCalculatorView.prototype.__construct = function()
{
    COM(this.terrains).dataProvider = Terrain.ALL;
    COM(this.terrains).selectedIndex = 0;
    COM(this.myRole).dataProvider = CombatRole.ALL_ROLES;
    COM(this.myRole).selectedIndex = 1;
    COM(this.predictionPolicyCombo).dataProvider = PredictionPolicy.ALL;
    COM(this.predictionPolicyCombo).selectedIndex = 0;
    
    /////// from mxml migration ///////
    $(this.terrains).on("change", $.proxy(this.recalculateBuffs, this));
    $(this.myRole).on("change", $.proxy(this.myPositionChanged, this));
    $(this.citySelector).on("change", $.proxy(function()
    {
    	this.onCityChange(COM(this.citySelector).city);
    }, this));
    
    var cav = COM(this.attacker1);
    cav.isAlly = true;
    cav.combatRole = CombatRole.ATTACKER;
    cav.isMyself = true;
    cav.onAuraChanged = $.proxy(this.handleAuraChanged, this);
    cav.fireRecalculateBuffs = $.proxy(this.recalculateBuffs, this);
    cav.init();
    
    cav = COM(this.defender1);
    cav.combatRole = CombatRole.MAIN_DEFENDER;
    cav.showDefense = true;
    cav.onAuraChanged = $.proxy(this.handleAuraChanged, this);
    cav.fireRecalculateBuffs = $.proxy(this.recalculateBuffs, this);
    cav.init();
    
    COM(this.addAttacker).onAdd = $.proxy(this.onAddParticipant, this);
    COM(this.addDefender).onAdd = $.proxy(this.onAddParticipant, this);
    
    /*
    COM.bindExclusiveClick(this.buyButton, $.proxy(function()
    {
    	BuyCalculationsDialog.popup(this);
    }, this));
    */
   
    COM.bindExclusiveClick(this.calcButton, $.proxy(this.doCalculate, this));
	    
    ///////////////////////////////////
    
    var policy = FUser.CURRENT_USER.settings.calculatorPredictionPolicy;
    COM(this.predictionPolicyCombo).selectedItem = policy ? policy : PredictionPolicy.AVERAGE;
    
    Events.onFUserUpdated.add(this, this.updateCalculationsLeft);
    
    //this.update();
}

CombatCalculatorView.prototype.onShow = function()
{
	this.scroll = new iScroll(this.container.parentNode, {scrollbarColor: '#784f0b'});
	this.refreshScroll();
}

CombatCalculatorView.prototype.update = function()
{
    this.doUpdate();
    this.handleAuraChanged();
}

CombatCalculatorView.prototype.doUpdate = function()
{
    COM(this.citySelector).updateCities(FUser.LINKS);

    var i = 0;
    COM(this.attacker1).clearAll();
    var coms = $("> choosearmyview", this.attackers);
    for(var n=1; n<coms.length; n++)
    {
    	COM(coms[n]).destroy();
    }
    
    COM(this.defender1).clearAll();
    var coms = $("> choosearmyview", this.defenders);
    for(var n=1; n<coms.length; n++)
    {
    	COM(coms[n]).destroy();
    }
    
    UI.show(true, [this.addAttacker, this.addDefender]);

    this.updateCalculationsLeft(FUser.CURRENT_USER);
    COM(this.citySelector).selectedIndex = -1;
    COM(this.myRole).selectedIndex = -1;
	
    var hasAttackMissions = this.attackMissions && this.attackMissions.length > 0;
    if (hasAttackMissions)
    {
        var curIndex = this.findInfo(this.attackMissions[0].to.id);
        if (curIndex != -1)
        {
            this.currentEstateInfo = this.estateMilitaryInfos[curIndex];
        }
    }
    if (this.currentEstateInfo)
    {
        var newRole = CombatRole.ATTACKER;
        var mainDefender = this.currentEstateInfo.defenders[0].username;
        var ind;
        if (FUser.isCurrentUser(mainDefender))
        {
            newRole = CombatRole.MAIN_DEFENDER;
            ind = this.findInfo(this.currentEstateInfo.id);
            if (ind != -1)
            {
                COM(this.citySelector).selectedIndex = ind;
                this.currentEstateInfo.fundsWoHiddenAndDiscount = this.estateMilitaryInfos[ind].fundsWoHiddenAndDiscount;
            }
        }
        else if (FUser.CURRENT_USER.isClanMate(mainDefender) || FUser.CURRENT_USER.isAlly(mainDefender))
        {
            if (this.currentEstateInfo.defenders.length < 2 || Strings.isEmpty(this.currentEstateInfo.defenders[1].username))
            {
                this.addParticipant(false, FUser.CURRENT_USER.username);
                newRole = CombatRole.DEFENDER;
                ind = this.findInfo(this.currentEstateInfo.id);
                if (ind != -1)
                {
                    COM(this.citySelector).selectedIndex = ind;
                    this.currentEstateInfo.fundsWoHiddenAndDiscount = this.estateMilitaryInfos[ind].fundsWoHiddenAndDiscount;
                }
            }
            else
            {
                newRole = CombatRole.OBSERVER;
            }
        }
        else if (FUser.CURRENT_USER.diplomacy.hasFriend(mainDefender))
        {
            newRole = CombatRole.OBSERVER;
        }
        
        var myRole = COM(this.myRole);
        var oldRole = myRole.selectedItem;
        if (!oldRole || oldRole.name != newRole.name)
        {
            this.selectRole(newRole);
            this.myPositionChanged();
        }
       
        this.assignArmiesToViews(this.defenders, this.currentEstateInfo.defenders, this.currentEstateInfo.heroes);
        this.selectTerrain(this.currentEstateInfo.terrain);
        COM(this.defender1).setDefender(this.currentEstateInfo);
    }

    if (hasAttackMissions)
    {
        var owner2army = new FMap();
        var heroes = [];
        
        var m;
        for(var i=0; i<this.attackMissions.length; i++)
        {
        	m = this.attackMissions[i];
        	var a = owner2army.getValueOrNull(m.owner.username);
        	if (!a)
        	{
                a = m.armyInBattle;
            }
            else
            {
                a.army = Army.addArmies(a.army, m.army.toArray());
            }
            owner2army.put(m.owner.username, a);
            if (m.hero)
            {
                heroes.push(m.hero);
            }
        }
        this.assignArmiesToViews(this.attackers, owner2army.values, heroes);
    }
    
    var cs = COM(this.citySelector);
    if (cs.dataProvider.length > 0 && cs.selectedIndex == -1)
    {
        cs.selectedIndex = 0;
        this.onCityChange(cs.city);
    }
    this.recalculateBuffs();
   
	this.refreshScroll();
}

CombatCalculatorView.prototype.refreshScroll = function()
{
	if(this.scroll)
	{
		setTimeout($.proxy(function()
		{
			this.scroll.refresh();
		}, this), 300);
	}
}

CombatCalculatorView.prototype.handleAuraChanged = function()
{
    var i;
    var coms = $("> choosearmyview", this.attackers);
    for (i = 0; i<coms.length; i++)
    {
        this.assignAura(COM(coms[i]), true);
    }
    
    var coms = $("> choosearmyview", this.defenders);
    for (i = 0; i<coms.length; i++)
    {
        this.assignAura(COM(coms[i]), false);
    }
}

CombatCalculatorView.prototype.assignAura = function(view, isAttacker)
{
    var other = isAttacker ? COM(this.defender1) : COM(this.attacker1);
    view.aura = new UnitAura(CombatBonusUtil.getBonuses(view.race, COM(view.betrayerSwitch).checked, other.race,
            COM(other.betrayerSwitch).checked, isAttacker, COM(this.terrains).selectedItem.data, view.towerLevels, isAttacker && Race.MONSTER.equals(other.race),
            view.maxDefense));
}

CombatCalculatorView.prototype.updateCalculationsLeft = function(u)
{
    var calculations = u.servicesValues.getValueOrNull(ServiceDecl.BUY_CALCULATIONS);
    if (!calculations)
    {
        calculations = 0;
    }
    this.calcsLeft.innerHTML = I18.f("CALCULATIONS_LEFT", calculations);
    COM(this.calcButton).skin = calculations > 0 ? "green" : "red";
    
    this.calcsCount = calculations;
    
    //calcButton.toolTip = calculations > 0 ? "" : I18.NO_CALCULATIONS_LEFT.f();
}


CombatCalculatorView.prototype.recalculateBuffs = function()
{
    var terrain = COM(this.terrains).selectedItem.data;
    
    var attViews = $("> choosearmyview", this.attackers);
    for(var i=0; i<attViews.length; i++)
    {
    	COM(attViews[i]).buffs = [];
    }
    
    var defViews = $("> choosearmyview", this.defenders);
    for(var i=0; i<defViews.length; i++)
    {
    	COM(defViews[i]).buffs = [];
    }

    if (this.currentEstateInfo)
    {
        this.fill(terrain, attViews, defViews, this.defender1, this.currentEstateInfo.casts);
    }
	
	for(var i=0; i<attViews.length; i++)
	{
		var a = COM(attViews[i]);
		this.fillHeroCasts(terrain, defViews, attViews, a);
        this.fill(terrain, defViews, attViews, a, a.armyInBattle.nonHeroCasts);
	}
	
	for(var i=0; i<defViews.length; i++)
	{
		var d = COM(defViews[i]);
		this.fillHeroCasts(terrain, attViews, defViews, d);
        if (d.DOM != this.defender1)
        {
            this.fill(terrain, attViews, defViews, d, d.armyInBattle.nonHeroCasts);
        }
	}
}

CombatCalculatorView.prototype.fillHeroCasts = function(t, enemies, allies, self)
{
    if (!self.selectedHero || self.selectedHero.isUpset())
    {
        return;
    }
    this.fill(t, enemies, allies, self, self.selectedHero.casts);
}

CombatCalculatorView.prototype.fill = function(t, enemies, allies, self, casts)
{
    if (!casts)
    {
        return;
    }
    var enemyRaces = [];
    self = COM(self);
    for(var i=0; i<enemies.length; i++)
    {
    	enemyRaces.push(COM(enemies[i]).race);
    }

    var c, b;
    for(var i=0; i<casts.length; i++)
    {
    	c = casts[i]
    	for(var j=0; j<c.buffs.length; j++)
    	{
    		b = c.buffs[j];
    		if (b.affectsTerrain(t) && b.affectsEstateType(this.estateType))
    		{
                if (BuffTarget.contains(b.targets, BuffTarget.ENEMY))
                {
                    for(var n=0; n<enemies.length; n++)
                    {
                    	var enemy = COM(enemies[n]);
                    	if (b.affectsMyRace(self.race) && b.affectsAgainstRace(enemy.race))
                    	{
                            enemy.buffs.push(b);
                        }
                    }
                }

                if (BuffTarget.contains(b.targets, BuffTarget.ALLY))
                {
                    for(var n=0; n<allies.length; n++)
                    {
                    	var ally = COM(allies[n]);
                    	if (ally != self)
                    	{
                            if (b.affectsMyRace(ally.race) && b.affectsAgainstRaces(enemyRaces))
                            {
                                ally.buffs.push(b);
                            }
                        }
                    }
                }

                if (BuffTarget.contains(b.targets, BuffTarget.SELF))
                {
                    if (b.affectsMyRace(self.race) && b.affectsAgainstRaces(enemyRaces))
                    {
                        self.buffs.push(b);
                    }
                }
            }
    	}
    }
}

CombatCalculatorView.prototype.selectTerrain = function(terrain)
{
	for(var j=0; j<Terrain.ALL.length; j++)
    {
    	if(Terrain.ALL[j].name == terrain.name)
    	{
    		COM(this.terrains).selectedIndex = j;
    		break;
    	}
    }
}

CombatCalculatorView.prototype.selectRole = function(role)
{
	for(var j=0; j<CombatRole.ALL_ROLES.length; j++)
    {
    	if(CombatRole.ALL_ROLES[j].name == role.name)
    	{
    		COM(this.myRole).selectedIndex = j;
    		break;
    	}
    }
}

CombatCalculatorView.prototype.onCityChange = function(c)
{
    var myView = this.getMyArmyView();
    if (myView)
    {
        for(var i in this.estateMilitaryInfos)
        {
        	var info = this.estateMilitaryInfos[i];
        	if(info && info.__CLS__ == "game.service.mission.FEstateMilitaryInfo")
        	{
	        	if (info.id.equals(c.id))
	        	{
	                if (myView === this.defender1)
	                {
	                    this.currentEstateInfo = info;
	                    this.removeExtraDefenders();
	                    this.assignArmiesToViews(this.defenders, this.currentEstateInfo.defenders, this.currentEstateInfo.heroes);
	                    this.selectTerrain(info.terrain);
	                    COM(this.defender1).setDefender(info);
	                }
	                else
	                {
	                    COM(myView).setArmyInBattle(info.defenders[0].withoutEstateCasts(), info.heroes);
	                }
	            }
			}
        }
        
        this.recalculateBuffs();
    }
}

CombatCalculatorView.prototype.findInfo = function(id)
{
    for (var j in this.estateMilitaryInfos)
    {
        if (this.estateMilitaryInfos[j].__CLS__ == "game.service.mission.FEstateMilitaryInfo" && this.estateMilitaryInfos[j].id.equals(id))
        {
            return j;
        }
    }
    return -1;
}

CombatCalculatorView.prototype.assignArmiesToViews = function(views, armies, heroes)
{
    var coms = $("> choosearmyview", views);
    for (var i = 0; i < armies.length; i++)
    {
        var view = i < coms.length ? COM(coms[i]) : this.addParticipant(views === this.attackers, armies[i].username);
        view.setArmyInBattle(armies[i], i == 0 ? heroes : null);
    }
}

CombatCalculatorView.prototype.addParticipant = function(isAttacker, username)
{
    var box = isAttacker ? this.attackers : this.defenders;
    var c = COM.create("ChooseArmyView");
    var coms = $("> choosearmyview", box);
    c.isAlly = COM(coms[0]).isAlly;
    c.isMyself = false;
    var si = COM(this.myRole).selectedItem;
    c.isMyselfPresent = si && si.name != CombatRole.OBSERVER.name;
    c.username = username;
    c.combatRole = isAttacker ? CombatRole.ATTACKER : CombatRole.DEFENDER;
    c.onAuraChanged = $.proxy(this.handleAuraChanged, this);
    c.fireRecalculateBuffs = $.proxy(this.recalculateBuffs, this);
    c.onClose = $.proxy(this.removeParticipant, this);
    box.appendChild(c.DOM);
    c.clearAll();
    c.updateStatus(c.isMyself, c.isAlly, c.isMyselfPresent);
    
    var max = isAttacker ? this.maxAttackers : this.maxDefenders;
    var addButton = isAttacker ? this.addAttacker : this.addDefender;
    UI.show(coms.length + 1 < max, [addButton]);
    this.recalculateBuffs();
    this.refreshScroll();
    return c;
}

CombatCalculatorView.prototype.myPositionChanged = function()
{
    var myArmy = COM(this.getMyArmyView());
    var mr = COM(this.myRole);
    var attackersAreAllies = mr.selectedItem.data.name == CombatRole.ATTACKER.name;
    var defendersAreAllies = mr.selectedItem.data.name == CombatRole.MAIN_DEFENDER.name || mr.selectedItem.data.name == CombatRole.DEFENDER.name;
    
    var coms = $("> choosearmyview", this.attackers);
    for (var i = 0; i < coms.length; i++)
    {
        var a = COM(coms[i]);
        if (i > 0 && a != myArmy && a.isMyself)
        {
            a.destroy();
            UI.show(true, [this.addAttacker]);
        }
        else
        {
            a.updateStatus(a === myArmy, attackersAreAllies, Boolean(myArmy));
            if (a != myArmy && FUser.isSameName(a.username, FUser.CURRENT_USER.username))
            {
                a.username = null;
            }
        }
    }

    var coms = $("> choosearmyview", this.defenders);
    for (var j = 0; j < coms.length; j++)
    {
        var d = COM(coms[j]);
        if (j > 0 && d != myArmy && d.isMyself)
        {
            d.destroy();
            UI.show(true, [this.addDefender]);
        }
        else
        {
            d.updateStatus(d === myArmy, defendersAreAllies, Boolean(myArmy));
            if (d != myArmy && FUser.isSameName(d.username, FUser.CURRENT_USER.username))
            {
                d.username = null;
            }
        }
    }
    if (myArmy)
    {
        myArmy.username = FUser.CURRENT_USER.username;
    }
	
	var addAttacker = COM(this.addAttacker);
    addAttacker.isAlly = attackersAreAllies;
    addAttacker.alliesPresent = Boolean(myArmy);
    var addDefender = COM(this.addDefender);
    addDefender.isAlly = defendersAreAllies;
    addDefender.alliesPresent = Boolean(myArmy);
    if (COM(this.citySelector).selectedIndex != -1)
    {
        this.onCityChange(COM(this.citySelector).city);
    }

    this.recalculateBuffs();
}

CombatCalculatorView.prototype.getMyArmyView = function()
{
    var si = COM(this.myRole).selectedItem;
    if(!si) return null;
    
    switch(si.data.name)
    {
        case CombatRole.ATTACKER.name:
            var mea = this.findMyself(this.attackers, 0);
            return mea ? mea : $("> choosearmyview", this.attackers)[0];
        case CombatRole.MAIN_DEFENDER.name: return $("> choosearmyview", this.defenders)[0];
        case CombatRole.DEFENDER.name:
            if ($("> choosearmyview", this.defenders).length == 1)
            {
                return this.addParticipant(false, FUser.CURRENT_USER.username);
            }
            else
            {
                var med = this.findMyself(this.defenders, 1);
                return med ? med : $("> choosearmyview", this.defenders)[1];
            }
        default: return null;
    }
}

CombatCalculatorView.prototype.findMyself = function(box, fromPos)
{
    var coms = $("> choosearmyview", box);
    for(var i = fromPos; i < coms.length; i++)
    {
        var view = COM(coms[i]);
        if (FUser.isSameName(view.username, FUser.CURRENT_USER.username))
        {
            return view;
        }
    }
    return null;
}

CombatCalculatorView.prototype.getMaxAttackers = function()
{
    return this.estateType.getMaxJoinedAttackers();
}

CombatCalculatorView.prototype.getMaxDefenders = function()
{
    return 1 + this.estateType.getNumberOfAssistanceSlots();
}

CombatCalculatorView.prototype.getEstateType = function()
{
    return this.currentEstateInfo ? this.currentEstateInfo.estateType : (this.attackMissions.length > 0 ? this.attackMissions[0].to.estateType : EstateType.EMPTY_FIELD);
}

CombatCalculatorView.prototype.removeExtraDefenders = function()
{
    var coms = $("> choosearmyview", this.defenders);
    for(var i=1; i<coms.length; i++)
    {
    	COM(coms[i]).destroy();
    }
    this.recalculateBuffs();
    this.refreshScroll();
}

CombatCalculatorView.prototype.isParticipantExists = function(participantName)
{
    var coms = $("> choosearmyview", this.attackers);
    for(var i=0; i<coms.length; i++)
    {
    	if (FUser.isSameName(COM(coms[i]).username, participantName))
    	{
            return true;
        }
    }
    
    var coms = $("> choosearmyview", this.defenders);
    for(var i=0; i<coms.length; i++)
    {
    	if (FUser.isSameName(COM(coms[i]).username, participantName))
    	{
            return true;
        }
    }
    
    return false;
}

CombatCalculatorView.prototype.onAddParticipant = function(p, participantName)
{
    this.onAddParticipantX(p.DOM == this.addAttacker, participantName);
}

CombatCalculatorView.prototype.onAddParticipantX = function(isAttacker, participantName)
{
    if (Quarantine.contains(this))
    {
        return;
    }

    if (participantName == "")
    {
        this.addParticipant(isAttacker, participantName);
    }
    else
    {
        if (this.isParticipantExists(participantName))
        {
            Alert.create(I18.f("PARTICIPANT_ALREADY_ADDED"));
            return;
        }
        Quarantine.put(this);
        Service.user.callMethod("getAggregatedMilitaryInfo", participantName,
        $.proxy(function(info)
        {
            info.aggregatedArmy.username = participantName;
            this.addArmyToView(isAttacker, info.aggregatedArmy, info.allHeroes);
        }, this),
        $.proxy(function (e)
        {
            if (e.__className__ == "AvatarNotFoundException")
            {
                Alert.create(e.getMessage());
                return true;
            }
            if (e.__className__ == "AccessDeniedException")
            {
                var diplomacy = FUser.CURRENT_USER.diplomacy;
                if (diplomacy.clan && diplomacy.clan.getMember(participantName) || diplomacy.hasAlly(participantName))
                {
                    Alert.create(I18.f("NO_ACCESS_TO_ARMY_NOT_GRANTED"));
                }
                else
                {
                    Alert.create(I18.f("NO_ACCESS_TO_ARMY_NOT_ALLY"));
                }
                return true;
            }
            return false;
        }, this));
    }
}

CombatCalculatorView.prototype.addArmyToView = function(isAttacker, army, heroes)
{
    this.addParticipant(isAttacker, army.username).setArmyInBattle(army, heroes);
    if (FUser.isSameName(army.username, FUser.CURRENT_USER.username))
    {
        COM(this.myRole).selectedIndex = isAttacker ? this.selectRole(CombatRole.ATTACKER) : this.selectRole(CombatRole.DEFENDER);
        this.myPositionChanged();
    }
}

CombatCalculatorView.prototype.removeParticipant = function(c)
{
    if (c.combatRole.name == CombatRole.ATTACKER.name)
    {
        UI.show(true, [this.addAttacker]);
    }
    else
    {
        UI.show(true, [this.addDefender]);
    }
    
    c.destroy();
    
    if (FUser.isSameName(c.username, FUser.CURRENT_USER.username))
    {
        this.selectRole(CombatRole.OBSERVER)
        this.myPositionChanged();
    }
    else
    {
        this.recalculateBuffs();
    }
   
	this.refreshScroll();
}

CombatCalculatorView.prototype.doCalculate = function()
{
    if(this.calcsCount <= 0)
    {
    	BuyCalculationsDialog.popup(this);
    	return;
    }
    
    APP.showLoader();
    
    var collectForces = function(armies, parent)
    {
        var forces = [];
        var c;
        for(var i=0; i<armies.length; i++)
        {
        	c = COM(armies[i]);
    		var armyInBattle = c.getArmyInBattle();
            armyInBattle.hero = c.selectedHero;
            
            if (c.DOM == parent.defender1 && parent.currentEstateInfo)
            {
                armyInBattle.estateName = parent.currentEstateInfo.defenders[0].estateName;
                armyInBattle.estateXYZ = parent.currentEstateInfo.defenders[0].estateXYZ;
                armyInBattle.nonHeroCasts = parent.currentEstateInfo.defenders[0].nonHeroCasts;
                
                armyInBattle.nonHeroCasts = [];
                var nhc = parent.currentEstateInfo.defenders[0].nonHeroCasts;
                if(nhc)
                {
                	for(var ci=0; ci<nhc.length; ci++)
                	{
                		armyInBattle.nonHeroCasts.push(nhc[ci].prepareForRequest());
                	}
                }
            }
            
            if(armyInBattle.hero)
            {
            	armyInBattle.hero = armyInBattle.hero.prepareForRequest();
			}
            
            forces.push(armyInBattle);
        }
        
        return forces;
    };
    
    var towerLevels = COM(this.defender1).towerLevels;
    var fortifications = towerLevels.getValue(BType.FORTIFICATION);
    var towers = towerLevels.getValue(BType.TOWER);
    var magicTowers = towerLevels.getValue(BType.MAGIC_TOWER);
    var gates = towerLevels.getValue(BType.CASTLEGATE);
    
    var gateLevel = gates.length <= 0 ? 0 : gates[0];
    var gateHealth = gateLevel <= 0 ? 0 : COM(this.defender1).gateHealth;
    var gateShield = gateLevel <= 0 ? 0 : BType.CASTLEGATE.getLevels(Race.CASTLE_MONSTER)[gateLevel].shield;
	
    var fundsWoHiddenAndDiscount = this.currentEstateInfo ? this.currentEstateInfo.fundsWoHiddenAndDiscount : new Funds();
    fundsWoHiddenAndDiscount = fundsWoHiddenAndDiscount.prepareForRequest();
    var pearlsInRuin = this.currentEstateInfo ? Number(this.currentEstateInfo.pearlsInRuin) : 0;
    var prisonRatio = 0;
    if (this.currentEstateInfo)
    {
        prisonRatio = Number(this.currentEstateInfo.prisonRatio);
    }

    FUser.CURRENT_USER.servicesValues.put(ServiceDecl.BUY_CALCULATIONS, FUser.CURRENT_USER.servicesValues.getValue(ServiceDecl.BUY_CALCULATIONS) - 1);
    this.updateCalculationsLeft(FUser.CURRENT_USER);

    var predictionPolicy = COM(this.predictionPolicyCombo).selectedItem.data;
	
	/*
	fortifications = new ArrayCollection(fortifications);
	fortifications.__dataType__ = "int";
	
	towers = new ArrayCollection(towers);
	towers.__dataType__ = "int";
	
	magicTowers = new ArrayCollection(magicTowers);
	magicTowers.__dataType__ = "int";
	*/

	//return;
	
	/*
	console.log(COM(this.attacker1).isMyself);
	console.log(collectForces($("> choosearmyview", this.attackers), this));
	console.log(collectForces($("> choosearmyview", this.defenders), this));
	console.log(this.estateType);
	console.log(Terrain.ALL[COM(this.terrains).selectedIndex].prepareForRequest());
	console.log(COM(this.defender1).defenderLevel);
	console.log(fortifications);
	console.log(towers);
	console.log(magicTowers);
	console.log(gateLevel);
	console.log(gateHealth);
	console.log(gateShield);
	console.log(fundsWoHiddenAndDiscount);
	console.log(pearlsInRuin);
	console.log(COM(this.defender1).maxDefense);
	console.log(prisonRatio);
	console.log(predictionPolicy);

	return;
	*/
	
	var params = [
		COM(this.attacker1).isMyself, collectForces($("> choosearmyview", this.attackers), this), collectForces($("> choosearmyview", this.defenders), this),
		this.estateType, Terrain.ALL[COM(this.terrains).selectedIndex].prepareForRequest(),
		COM(this.defender1).defenderLevel, fortifications, towers, magicTowers,
		gateLevel, gateHealth, gateShield,
		fundsWoHiddenAndDiscount, pearlsInRuin, COM(this.defender1).maxDefense, prisonRatio, predictionPolicy
	];
	Service.user.callMethod("calculateCombat", params, $.proxy(this.handleResult, this), null);
}

CombatCalculatorView.prototype.handleResult = function(message)
{
	APP.hideLoader();
    
    //console.log(message);
    CombatResultView.popup(this, message, true);
}