var BConstraints =
{
	CHECK_ROUTINES :
	{
		template : function(cellIndex, estate, bt)
		{
			return CheckResult.success();
			// do nothing
		},

		isBlockedSuccess : function(cellIndex, estate, bt)
		{
			if (estate.getCellByIndex(cellIndex).blocked && bt.name != BType.TERRAFORMER.name)
			{
				return CheckResult.fail(I18.f("BUILDING_CONSTRAINT_UNBLOCKED_CELL", estate.terrain.blocker.tooltip));
			}
			return CheckResult.success();
		},

		isRecoverySuccess : function(cellIndex, estate, bt)
		{
			if (BType.RECOVERY.equals(bt))
			{
				if (estate.getCellByIndex(cellIndex).resource.name == Resource.NULL.name)
				{
					return CheckResult.fail(I18.f("BUILDING_CONSTRAINT_RECOVERY"));
				}
				else
				if (estate.getCellByIndex(cellIndex).hasFullRemainder())
				{
					return CheckResult.fail(I18.f("BUILDING_CONSTRAINT_RECOVERY_RESOURCE_FULL"));
				}
			}
			return CheckResult.success();
		},

		isTowerSuccess : function(cellIndex, estate, bt)
		{
			if (bt.equals(BType.TOWER) || bt.equals(BType.MAGIC_TOWER))
			{
				var towerCount = estate.countBuildings(BType.TOWER) + estate.countBuildings(BType.MAGIC_TOWER, true);
				if (towerCount >= FConfig.MAX_TOWER_COUNT)
				{
					return CheckResult.fail(I18.f("BUILDING_CONSTRAINT_TOWER", 2));
				}
			}
			return CheckResult.success();
		},

		isBuildingCountSuccess : function(cellIndex, estate, bt)
		{
			var count = estate.countBuildings(bt, true);
			if (count >= bt.getMaxCountInCity(estate.owner.race))
			{
				return CheckResult.fail(I18.f("BUILDING_CONSTRAINT_COUNT_LIMIT", bt.getMaxCountInCity(estate.owner.race)));
			}
			return CheckResult.success();
		},

		checkSectorAndGateClosed : function(cellIndex, estate, bt)
		{
			return estate.getCellByIndex(cellIndex).onSector && !estate.gateState.isOpen() ? CheckResult.fail(I18.f("BUILDING_CONSTRAINT_GATE_CLOSED")) : CheckResult.success();
		},

		checkGeology : function(cellIndex, estate, bt)
		{
			var whereOn = bt.getWhereOn(estate.owner.race);
			if (!bt.equals(BType.RECOVERY) && whereOn.equals(BWhereCell.ANYWHERE) && !estate.getCellByIndex(cellIndex).resource.equals(Resource.NULL) && estate.science.getLevel(Science.GEOLOGY) == 0)
			{
				return CheckResult.fail(I18.f("BUILDING_CONSTRAINT_GEOLOGY"));
			}
			return CheckResult.success();
		},

		checkDeputyCannotBuildForPearl : function(cellIndex, estate, bt)
		{
			var level = bt.getFirstLevel(estate.race);
			if (!estate.accessManageble && level.pearl > 0)
			{
				return CheckResult.fail(I18.f("DEPUTY_CAN_NOT_BUILD_FOR_PEARL"));
			}
			return CheckResult.success();
		}
	},

	// exclusive checker for clans
	checkClanBuildings : function(cellIndex, estate, bt)
	{
		// clan leader check
		if ((bt.equals(BType.CLAN_RESERVATION) || bt.equals(BType.CLAN_WAREHOUSE)) && (!estate.owner.isClanLeader() || !estate.owner.isInRegisteredClan()) && !(estate.__className__ == "FCastle" && estate.accessManageble))
		{
			return CheckResult.fail(I18.f("BUILDING_CONSTRAINT_BY_REGISTERED_CLAN_LEADER_ONLY"));
		}
		// pit check
		if (!bt.getWhereIn(estate.owner.race).isSuccess(cellIndex, estate))
		{
			return CheckResult.fail(bt.getWhereIn(estate.owner.race));
		}
		// one per clan check
		if (bt.equals(BType.CLAN_RESERVATION) && estate.owner.diplomacy.clan.hasReservation || bt.equals(BType.CLAN_WAREHOUSE) && estate.owner.diplomacy.clan.hasWarehouse || bt.equals(BType.CLAN_DIPLOMAT) && estate.owner.diplomacy.clan.hasDiplomatHall)
		{
			return CheckResult.fail(I18.f("BUILDING_CONSTRAINT_CAN_HAVE_ONLY_ONE"));
		}
		return CheckResult.success();
	},

	from : function(bt, cellIndex, estate)
	{
		if (cellIndex == FCastle.GATE_FAKE_CELL_INDEX)
		{
			return bt.name == BType.CASTLEGATE.name ? CheckResult.success() : CheckResult.fail(I18.f("BUILDING_CONSTRAINT_NOT_ON_GATE"));
		}

		if (bt == BType.CASTLEGATE)
		{
			return CheckResult.fail(I18.f("BUILDING_CONSTRAINT_ON_GATE"));
		}

		// clan buildingcheck hack
		var clanCR = this.checkClanBuildings(cellIndex, estate, bt);

		if (!clanCR.isSuccess)
		{
			return clanCR;
		}

		if (!bt.getWhereIn(estate.owner.race).isSuccess(cellIndex, estate))
		{
			return CheckResult.fail(bt.getWhereIn(estate.owner.race));
		}

		var whereOn = bt.getWhereOn(estate.owner.race);
		if (!whereOn.isSuccess(cellIndex, estate))
		{
			return CheckResult.fail(whereOn.message(cellIndex, estate, whereOn));
		}

		for (var k in BConstraints.CHECK_ROUTINES)
		{
			var result = BConstraints.CHECK_ROUTINES[k](cellIndex, estate, bt);
			if (!result.isSuccess)
			{
				return result;
			}
		}

		return CheckResult.success();
	}
}