function BuildDialog()
{
	callParentConstructor(BuildDialog.prototype, this);
	
	this.estate = null;
	this.cellIndex = null;
	this.selectedIndex = 0;
	this.scroll = null;
	
	this.tip = null;
}

extendClass(BuildDialog.prototype, Dialog.prototype);

BuildDialog.prototype.fitLayout = function(container)
{
	superMethod(BuildDialog.prototype, "fitLayout", this)(container);
	UI.show(false, [this.headerLeftBox, this.headerCaptionBox]);
};

BuildDialog.prototype.init = function(estate, cellIndex)
{
	this.fullScreen();
	
	this.cellIndex = cellIndex;
    this.estate = estate;
    
    var tabs = COM(this.tabs);
    
    tabs.onChange = $.proxy(this.changeTab, this);
    
    if(this.estate.race.name != Race.CASTLE_MONSTER.name)
    {
    	tabs.addItem(I18.f("BUILDINGS_PRODUCTION"), 0, true);
    }
    tabs.addItem(I18.f("BUILDINGS_MUNICIPAL"), 1, this.estate.race.name == Race.CASTLE_MONSTER.name);
    tabs.addItem(I18.f("BUILDINGS_MILITARY"), 2);
    tabs.addItem(I18.f("BUILDINGS_SPECIAL"), 3);
    tabs.addItem(I18.f("BUILDINGS_DEFENCE"), 4);
	
	this.scroll = new iScroll(this.buildsContainer);
	
	if (UI.show(this.estate.race.name != Race.CASTLE_MONSTER.name, [this.tab0]))
	{
		this.initTabByIX(0);
	}
	else
	{
		this.initTabByIX(1);
	}
	UI.show(this.estate.race.name == Race.CASTLE_MONSTER.name, [this.arena, this.clanDiplomat]);
	UI.show((this.estate.race.name == Race.CASTLE_MONSTER.name) && FConfig.CATAPULTS_ENABLED, [this.workshop]);
	
	COM(this.resourceCellTip).update(estate, estate.getCellByIndex(cellIndex));
	
	this.updateArrows(false);
}

BuildDialog.prototype.changeTab = function()
{
	this.initTabByIX(COM(this.tabs).value);
	this.updateArrows();
}

BuildDialog.prototype.initTabByIX = function(ix)
{
    UI.show(false, [this.tab0, this.tab1, this.tab2, this.tab3, this.tab4]);
    
    var tab = this['tab' + ix];
    UI.show(true, [tab]);
    
    var elements = this.getTabChildren(tab);
    for(var i=0; i<elements.length; i++)
    {
    	var ti = COM.get(elements.get(i));
    	ti.init(this.estate.owner.race, $.proxy(this.onSelect, this));
	}
	
	this.applyConstraints(tab);
	
	setTimeout($.proxy(function()
	{
    	this.scroll.refresh();
    	this.scroll.scrollTo(0, 0, 0);
	}, this), 300);
	
	this.selectedIndex = ix;
};

BuildDialog.prototype.getTabChildren = function(tab)
{
	return $("BuildDialogElement", tab);
}

BuildDialog.prototype.applyConstraints = function(tab)
{
	var elements = this.getTabChildren(tab);
   	
   	var ti;
   	for(var i=0; i<elements.length; i++)
   	{
   		var ti = COM.get(elements.get(i));
   		var bt = BType[ti.btype];
   		
   		var can = this.canBuild(bt);
        ti.setActive(can && this.hasFundsAndLimit(bt) && !this.isInChaos());
	}
};

BuildDialog.prototype.canBuild = function(bt, bc)
{
    var check = (bc != null ? bc : BConstraints.from(bt, this.cellIndex, this.estate));
    return this.hasRequiredScience(bt) && check.isSuccess;
    
};

BuildDialog.prototype.hasRequiredScience = function(bt)
{
    if (bt.name == BType.ARTIFACT_STORE.name)
    {
        if (this.estate.owner.tournamentVictoriesCount > 0 || this.estate.owner.tournamentGrottoVictoriesCount > 0)
        {
            return true;
        }
    }
    
    return !bt.requiredScience || this.estate.science.getLevel(bt.requiredScience) > 0;
    
};

BuildDialog.prototype.hasFundsAndLimit = function(bt, bc)
{
    if (!this.estate.canPerformOneMoreTask())
    {
        if (bc && bc.isSuccess)
        {
            bc.isSuccess = false;
            bc.text = I18.getConcurrentLimitText(this.estate.getMaxConcurrentBuildings());
        }
        return false;
    }

    var level = bt.getFirstLevel(this.estate.owner.race);
    return this.estate.bank.greaterOrEqualsTo(level.cost) && this.estate.owner.pearl >= level.pearl;
    
};

BuildDialog.prototype.isInChaos = function()
{
    return this.estate.hasBuff(BuffType.CONSTRUCTION_CHAOS);
    
};

BuildDialog.prototype.onSelect = function(e)
{
    var btype = $(e.currentTarget).attr("btype");
    
    this.updateArrows(btype);
    
    var bd = COM.create("DoBuildDialog");
    bd.popup(this);
    bd.update(btype);
};

BuildDialog.prototype.update = function(estate)
{
    this.estate = estate;
    this.tick();
}

BuildDialog.prototype.findBuildElement = function(tabIx, btype)
{
	var el = $("builddialogelement[btype="+btype.toString()+"]", this["tab"+tabIx])[0];
	return COM(el);
}

BuildDialog.prototype.updateArrows = function(selectedBType)
{
    var hint = FUser.CURRENT_USER.findConstructHint(this.estate);
    
    if (!hint || !this.canBuild(hint.btype))
    {
        return;
    }

	if(selectedBType && selectedBType.toString() == hint.btype.toString())
	{
		HintArrow.SELECT_BUILDING.understood();
	}

	var building = this.findBuildElement(COM(this.tabs).value, hint.btype);
    if (building)
    {
        HintArrow.CLICK_BUILDING_TAB.understood();
        HintArrow.SELECT_BUILDING.show(building.image, hint, this.DOM);
    }
    else
    {
        HintArrow.SELECT_BUILDING.hide();
        for(var i=0; i<5; i++)
        {
        	building = this.findBuildElement(i, hint.btype);
        	if(building)
        	{
        		if(COM(this.tabs).value != i)
        		{
        			HintArrow.CLICK_BUILDING_TAB.show(COM(this.tabs).items[i], hint, this.DOM);
        		}
        		else
        		{
        			HintArrow.CLICK_BUILDING_TAB.hide();
        		}
        		break;
        	}
        }
    }
}

BuildDialog.prototype.tick = function()
{
}

BuildDialog.prototype.onDoubleClick = function(e)
{
}

BuildDialog.prototype.onBuildingTip = function(e)
{
}

BuildDialog.prototype.onBuildingTimeTip = function(e)
{
}

BuildDialog.prototype.onScienceTip = function(e)
{
}

BuildDialog.prototype.onScienceClick = function()
{
}

BuildDialog.prototype.onBuildButtonTip = function(event)
{
}
