function BuildingDescriptionTip()
{
}

BuildingDescriptionTip.create = function(bt, estate, cellIndex)
{
    var tip = COM.create("BuildingDescriptionTip");
    tip.update(bt, estate, cellIndex);
    return tip;
}

BuildingDescriptionTip.setBonus = function(label, bonus, prec)
{
    if(typeof prec == "undefined") prec = -1;
    
    var bonusString = prec >= 0 ? Strings.formatNumber(bonus, prec) : bonus.toString();
    if (bonus > 0)
    {
        Style.POSITIVE_BONUS.applyTo(label);
        label.innerHTML = "+" + bonusString;
    }
    else if (bonus < 0)
    {
        Style.NEGATIVE_BONUS.applyTo(label);
        label.innerHTML = bonusString;
    }
    else
    {
        label.innerHTML = "";
    }
}

BuildingDescriptionTip.applyFormat = function(label, format)
{
    if (label.innerHTML.length > 0)
    {
        label.innerHTML = I18.f(format, label.innerHTML);
    }
}

BuildingDescriptionTip.prototype.update = function(bt, estate, cellIndex)
{
    var race = estate.race;
    var firstLevel = bt.getFirstLevel(race);
    var cell = estate.getCellByIndex(cellIndex);

    COM(this.populationIncreaseBase).source = STATIC_INTERFACE_URL + Assets.getFreePeopleIcon(race);

    //this.title.innerHTML = I18.getBuildingName(bt, estate.race, cell.resource);

    UI.show(bt.isHouses(), [this.housesBox], $.proxy(function()
    {
        // house increase
        var capacityBase = firstLevel.capacity;
        var capacityBonus = estate.getPeopleCapacityAbsoluteBonus(BType.HOUSES);
        this.houseIncreaseBase.innerHTML = capacityBase.toString();
        BuildingDescriptionTip.setBonus(this.houseIncreaseBonus, capacityBonus, 0);
        Style.TIP_LABEL.applyTo(this.houseIncreaseBonus);

        // population increase
        var increaseBase = firstLevel.populationIncrease;
        var increaseBonus = estate.getPeopleProductionBonus(BType.HOUSES);
        Style.POSITIVE_BONUS.applyTo(this.populationIncreaseBase);
        COM(this.populationIncreaseBase).value = I18.f("PER_TIME", NumberUtils.floor(increaseBase));
        BuildingDescriptionTip.setBonus(this.populationIncreaseBonus, increaseBonus, 0);
    }, this));

    UI.show(bt.isWarehouse(), [this.warehousesBox], $.proxy(function()
    {
        var warehouseCapacity = firstLevel.capacity;
        var bonus = estate.getCapacityBonus(BType.WAREHOUSE);
        this.warehosueIncrease.innerHTML = I18.f("C_B_TIP_WAREHOUSE_INCREASES_RESOURCES_CAPACITY", warehouseCapacity);
        UI.show(bonus > 0, [this.warehouseBonus], $.proxy(function()
        {
            this.warehouseBonus.innerHTML = Strings.formatPercent(NumberUtils.floor(bonus * 100), true);
            Style.POSITIVE_BONUS.applyTo(this.warehouseBonus);
        }, this));
    }, this));

    UI.show(bt.isMarket(), [this.marketBox], $.proxy(function()
    {
        this.marketCapacity.text = I18.f("C_B_TIP_MARKET_CAPACITY", firstLevel.capacity);
        UI.show(estate.getEstateType().name != EstateType.CASTLE.name, [this.marketCapacity, this.marketAllowSellLabel, this.marketAuctionAccessLabel]);
    }, this));

    UI.show(bt.isDiplomatHall(), [this.diplomaticHallBox]);

    UI.show(bt.isTerraformer(), [this.terraformerBox], $.proxy(function()
    {
        UI.show(!estate.terrain.blocker.isNull(), [this.terraformerInfo], $.proxy(function()
        {
            this.terraformerInfo.innerHTML = estate.terrain.blocker.terraformerText;
        }, this));
        this.terraformerTime.innerHTML = I18.f("TERRAFORMING_DURATION", Tick.toTimeLeft(firstLevel.workTime));
    }, this));

    UI.show(bt.isTemple(), [this.templeBox], $.proxy(function()
    {
        COM(this.godpointSpeed).value = firstLevel.godpointProduction;
        COM(this.godpointLimit).value = firstLevel.godpointLimit;
    }, this));

    UI.show(bt.isResurrector(), [this.resurrectorBox], $.proxy(function()
    {
        this.resurrectorPercentBase.innerHTML = I18.f("C_B_TIP_RESURRECTOR_ALLOWS_TO_RESURRECT", Strings.formatNumber(100.0 * firstLevel.resurrection, 0));
        if (estate.science.getLevel(Science.SPEC_CLERIC) > 0)
        {
            BuildingDescriptionTip.setBonus(this.resurrectorPercentBonus, 100.0 * firstLevel.getBonus(estate));
            BuildingDescriptionTip.applyFormat(this.resurrectorPercentBonus, "PERCENTS");
            Style.TIP_LABEL.applyTo(this.resurrectorPercentBonus);
            UI.show(true, [this.resurrectorPercentBonus]);
        }
        else
        {
            UI.show(false, [this.resurrectorPercentBonus]);
        }
    }, this));

    UI.show(bt.isAlchemist(), [this.alchemistBox], $.proxy(function()
    {
        // only sector bonus
        var bonus = (estate.hasSectorBonus(cellIndex)) ? FConfig.ratio(FConfig.PRODUCTIVITY_RATIO_IN_SECTOR) : 0;
        var speed = firstLevel.knowledgeProduction;
        COM(this.alchemistSpeedBase).value = speed;
        BuildingDescriptionTip.setBonus(this.alchemistSpeedBonus, 100.0 * bonus, 0);
        BuildingDescriptionTip.applyFormat(this.alchemistSpeedBonus, "PERCENTS");
    }, this));

    UI.show(bt.isExtractor() || bt.isSawmill(), [this.extractorBox], $.proxy(function()
    {
        var r = bt.getParams(race).raisedResource;
        this.extractorText.innerHTML = I18.getExtractorExtractsText(r);
        // bonus
        var productivityBonus = estate.getProductivityBonus(cellIndex).getValue(r);
        // spec_miner bonus
        var minerBonus = ScienceBonusUtil.getMinerBonus(estate.science.getLevel(Science.SPEC_MINER), bt);
        // speed
        var speedBase = firstLevel.raises;
        //
        if (bt.isExtractor() && r.name == cell.resource.name && NumberUtils.equals(cell.remainder, 0))
        {
            speedBase = bt.getEmptyCellSpeed(race);
        }
        
        var el = COM(this.extractorSpeedBase);
        el.alwaysShow = [r];
        el.update(new Funds().setValue(r, speedBase), race);
        el.applyStyle([r], Style.POSITIVE_BONUS);

        BuildingDescriptionTip.setBonus(this.extractorSpeedBonus, 100.0 * (productivityBonus + minerBonus), 0);
        BuildingDescriptionTip.applyFormat(this.extractorSpeedBonus, "PERCENTS");
    }, this));

    UI.show(bt.isShop(), [this.shopBox], $.proxy(function()
    {
        // goldspeed
        var r = Resource.GOLD;
        var speedBase = firstLevel.raises;
        var speedBonus = estate.getProductivityBonus(cellIndex).getValue(r);
        var el = COM(this.shopGoldSpeedBase);
        el.alwaysShow = [r];
        el.update(new Funds().setValue(r, speedBase), race);
        el.applyStyle([r], Style.POSITIVE_BONUS);
        BuildingDescriptionTip.setBonus(el, 100.0 * speedBonus, 0);
        BuildingDescriptionTip.applyFormat(el, "PERCENTS");
        // tax increase
        var taxIncreaseBonus = Science.getShopRatioBonus(estate.science.getLevel(Science.SPEC_TRADER));
        Style.POSITIVE_BONUS.applyTo(this.shopTaxIncreaseBase);
        this.shopTaxIncreaseBase.innerHTML = Strings.formatNumber(firstLevel.peopleRatio, 3);
        BuildingDescriptionTip.setBonus(this.shopTaxIncreaseBonus, taxIncreaseBonus, 3);
    }, this));

    UI.show(bt.isRecovery(), [this.recoveryBox], $.proxy(function()
    {
        var cell = estate.getCellByIndex(cellIndex);
        if (!cell.resource.isNull())
        {
            this.recoveryLabel.innerHTML = I18.f("C_B_TIP_RECOVERY_REPAIRS_RESOURCE_CELL") + ": ";
            var r = cell.resource;
            var speed = firstLevel.recovers.getValue(r);
            var bonus = estate.getProductivityBonus(cellIndex).getValue(r);
            var el = COM(this.recoverySpeedBase);
            el.alwaysShow = [r];
            el.update(new Funds().setValue(r, speed), race);
            el.applyStyle([r], Style.POSITIVE_BONUS);
            BuildingDescriptionTip.setBonus(this.recoverySpeedBonus, 100.0 * bonus, 0);
            BuildingDescriptionTip.applyFormat(this.recoverySpeedBonus, "PERCENTS");
            UI.show(true, [this.recoverySpeedBase, this.recoverySpeedBonus]);
        }
        else
        {
            this.recoveryLabel.innerHTML = I18.f("C_B_TIP_RECOVERY_REPAIRS_RESOURCE_CELL");
            UI.show(false, [this.recoverySpeedBase, this.recoverySpeedBonus]);
        }
    }, this));

    UI.show(bt.isMilitaryUnitProducer() || bt.isSpecialUnitProducer(), [this.specialMilitaryBox], $.proxy(function()
    {
        var prodLevel = bt.isMilitaryUnitProducer() ? estate.science.getLevel(bt.requiredScience) : 1;
        // training cost
        var unitType = bt.getUnitType(race);
        var unitCost = FConfig.getStat(race, unitType, prodLevel).cost;
        
        this.trainsUnit.innerHTML = I18.f("C_B_TIP_ALLOWS_TO_TRAIN", I18.getUnitName(unitType, prodLevel, race));
		
		//COM(this.trainCost).showIconBacks(true);
        COM(this.trainCost).update(unitCost, race);
        COM(this.trainCost).applyStyle(Resource.FUNDS_RESOURCES, Style.TIP_LABEL);

        // training time
        COM(this.trainTime).value = firstLevel.produceTime;
        Style.TIP_LABEL.applyTo(COM(this.trainTime).time);
        
        // capacity
        if (bt.isMilitaryUnitProducer())
        {
            this.trainCapacity.innerHTML = I18.f("C_B_TIP_MILITARY_CAPACITY", firstLevel.queueSize);
        }
        else
        {
            this.trainCapacity.innerHTML = I18.f("C_B_TIP_SPECIAL_CAPACITY", firstLevel.capacity);
        }
    }, this));

    UI.show(bt.isMilitaryUnitProducer() && !FConfig.NO_ARMY_CONSUMPTION, [this.consumeBox], $.proxy(function()
    {
        var prodLevel = bt.isMilitaryUnitProducer() ? estate.science.getLevel(bt.requiredScience) : 1;
        // training cost
        var unitType = bt.getUnitType(race);
        var consume = new Funds().setValue(Resource.GOLD, FConfig.getStat(race, unitType, prodLevel).consume);
        
        //COM(this.payroll).showIconBacks(true);
        COM(this.payroll).update(consume, race);
        COM(this.payroll).applyStyle(Resource.FUNDS_RESOURCES, Style.TIP_LABEL);
    }, this));

    UI.show(bt == BType.HEALER && estate.science.getLevel(Science.SPEC_CLERIC) > 0, [this.trainTimeBonus], $.proxy(function()
    {
        var clericLevel = estate.science.getLevel(Science.SPEC_CLERIC);
        this.trainTimeBonus.innerHTML = "-" + (100.0 * FConfig.getSciencePrice(Science.SPEC_CLERIC, clericLevel).healerTrainBonus).toString();
        Style.POSITIVE_BONUS.applyTo(this.trainTimeBonus);
        BuildingDescriptionTip.applyFormat(this.trainTimeBonus, "PERCENTS");
    }, this));

    UI.show(bt.equals(BType.TOWER), [this.towerBox], $.proxy(function()
    {
        COM(this.towerAttack).update(bt, 1, estate, false);
    }, this));

    UI.show(bt.equals(BType.MAGIC_TOWER), [this.magicTowerBox], $.proxy(function()
    {
        COM(this.magicTowerAttack).update(bt, 1, estate, false);
    }, this));

    UI.show(bt.isFortification(), [this.fortificationBox], $.proxy(function()
    {
        COM(this.fortificationDefence).value = "+" + firstLevel.armorBonus.toString();
        this.fortificationKillsEnemies.innerHTML = I18.f("C_B_TIP_FORTIFICATION_KILLS", firstLevel.attackerLoss);
    }, this));

    var defenderLevel = estate.getScienceLevel(Science.SPEC_DEFENDER);
    UI.show(defenderLevel > 0 && bt.isFortification(), [this.fortificationBonus, this.fortificationBonusDefence], $.proxy(function()
    {
        this.fortificationBonus.innerHTML = "+" + Science.getFortificationKillsBonus(defenderLevel);
        BuildingDescriptionTip.applyFormat(this.fortificationBonus, "ATTACKER_LOSS_FORMAT");
        Style.POSITIVE_BONUS.applyTo(this.fortificationBonus);

        this.fortificationBonusDefence.innerHTML = "+" + Science.getFortificationDefenceBonus(defenderLevel);
        Style.POSITIVE_BONUS.applyTo(this.fortificationBonusDefence);
    }, this));

    UI.show(bt.isClanReservation(), [this.clanReservationBox], $.proxy(function()
    {
        this.clanReservationKeepsClanPeople.innerHTML = I18.f("C_B_TIP_CLAN_RESERVATION_KEEPS_CLAN_PEOPLE", firstLevel.capacity);
    }, this));
    
    UI.show(bt.isClanWarehouse(), [this.clanWarehouseBox], $.proxy(function()
    {
        this.clanWarehouseKeepsClanResources.innerHTML = I18.f("C_B_TIP_CLAN_WAREHOUSE_KEEPS_CLAN_RESOURCES", firstLevel.capacity);
    }, this));
    
    UI.show(bt.name == BType.TAVERN.name, [this.tavernBox]);
    
    UI.show(bt.name == BType.ARTIFACT_STORE.name, [this.atrifactBox], $.proxy(function()
    {
        this.artifactCapacity.innerHTML = I18.f("ARTIFACT_CAPACITY", firstLevel.capacity);
    }, this));
    
    UI.show(bt.name == BType.PRISON.name, [this.prisonBox], $.proxy(function()
    {
        this.prisonerCapacity.innerHTML = I18.f("PRISONER_CAPACITY", firstLevel.capacity);
    }, this));
    
    UI.show(bt.name == BType.ARENA.name, [this.arenaBox], $.proxy(function()
    {
        this.arenaExpProduction.innerHTML = I18.f("PER_TIME", firstLevel.expProduction);
        BuildingDescriptionTip.setBonus(this.arenaExpBonus, 100 * estate.sumOfBuildingBuffs(BuffType.BOOST_BUILDING, bt), 0)
        BuildingDescriptionTip.applyFormat(this.arenaExpBonus, "PERCENTS");
    }, this));
    
    UI.show(bt.name == BType.CLAN_DIPLOMAT.name, [this.clanDiplomatHallBox]);
    
    UI.show(bt.name == BType.CASTLEGATE.name, [this.castleGateBox]);
}