function CapacityRatioView()
{
	this.__defineGetter__('readOnly', function(){
		return this._readOnly;
	});
	
	this.__defineSetter__('readOnly', function(val){
		this._readOnly = !!val;
		$(this.DOM).toggleClass('read-only', this._readOnly);
	});
	
	this.bank = new Funds();
	this.capacity = new Funds();
	this.resources = [Resource.GOLD, Resource.WOOD, Resource.STONE, Resource.GRAIN, Resource.IRON, /*Resource.PEARL*/];
	this.estateId = null;
	this.onChange = null; // callback
	this._readOnly = null;
	this.needUpdate = true;
}

CapacityRatioView.prototype.__construct = function()
{
	this.readOnly = (this.DOM.hasAttribute('readonly')) ? this.DOM.getAttribute('readOnly') == 'true' : false;
	$(this.wrapper).find('pickers > dot').on(MOUSE_DOWN_EVENT, $.proxy(this.onTouchPicker, this));
	$(this.wrapper).find('picker').on(MOUSE_DOWN_EVENT, $.proxy(this.onTouchPicker, this));
	$(this.wrapper).find('resource').on(MOUSE_DOWN_EVENT, $.proxy(this.onTouchPicker, this));
}

CapacityRatioView.prototype.onTouchPicker = function(e)
{
	e.preventDefault();
	
	var nodeName = e.currentTarget.nodeName.toLowerCase();
	var picker = $(e.currentTarget);
	if (nodeName != 'picker')
	{
		if (nodeName == 'resource')
		{
			picker = $(e.currentTarget).prev('picker');
		}
		else if (nodeName == 'dot')
		{
			var dots = this.wrapper.querySelectorAll('pickers > dot');
			var pickers = this.wrapper.querySelectorAll('picker');
			for (var i = 0; i < dots.length; i++)
			{
				if (e.currentTarget.isSameNode(dots[i]))
				{
					picker = $(pickers[i]);
				}
			}
		}
	}
	//var picker = e.currentTarget.nodeName.toLowerCase() == 'picker' ? $(e.currentTarget) : $(e.currentTarget).prev('picker');
	
	if (picker.hasClass('waiting'))
	{
		return false;
	}
	
	var dom = $(this.DOM), wrapper = $(this.wrapper);
	var lastX = e.originalEvent['touches'] ? e.originalEvent['touches'][0].pageX : e.pageX;
	
	var onMovePicker = $.proxy(function(e)
	{
		this.needUpdate = false;
		e.preventDefault();
		
		var pageX = e.originalEvent['touches'] ? e.originalEvent['touches'][0].pageX : e.pageX;
		var delta = this.px2val(pageX - lastX);
		lastX = pageX*1;
		
		var res = Resource[picker[delta > 0 ? 'prev' : 'next']().attr('name')];
		
		this._changeCapacity(res, Math.abs(delta), (delta > 0 ? 1 : -1));
		
		this.updateUI();
	}, this);
	
	var onDropPicker = $.proxy(function(e)
	{
		this.updateUI(true);
		$(document).off(MOUSE_MOVE_EVENT, onMovePicker);
		iScroll.enabled = true;
		//this.onApply();
	}, this);
	
	$(document)
		.on(MOUSE_MOVE_EVENT, onMovePicker)
		.one(MOUSE_UP_EVENT, onDropPicker)
	;
	
	iScroll.enabled = false;
}

CapacityRatioView.prototype._changeCapacity = function(r, delta, dir)
{
	if (!r || !delta) return 0;
	delta = Math.round(delta);
	for (var idx = 0; idx < this.resources.length; idx++)
	{
		if (this.resources[idx].name == r.name) break;
	}
	var next = idx + dir;
	var next = (next >= 0 && next < this.resources.length) ? this.resources[next] : null;
	
	// increase capacity
	if (delta > 0)
	{
		// recursively find maximum decrease value for next resources
		delta = Math.min(delta, next ? -this._changeCapacity(next, -delta, dir) : 0);
		this.capacity.addResource(r, delta);
		
		return delta;
	}
	
	// decrease capacity
	var f = Math.max(0, this.capacity.getValue(r) - this.bank.getValue(r));
	
	// enough space to decrease all delta
	if (Math.abs(delta) < f)
	{
		this.capacity.addResource(r, delta);
		return delta;
	}
	// not enough space to decrease all delta, try to decrease next...
	else
	{
		return 0;
		var d = delta + f;
		delta = -f;
		// fill all free space
		this.capacity.addResource(r, delta);
		// try to decrease next resource by remaining left
		if (next)
		{
			delta += this._changeCapacity(next, d, dir);
		}
	}
	
	// summary changed delta
	return delta;
}

CapacityRatioView.prototype.tick = function()
{
	this.updateLabels();
}

CapacityRatioView.prototype.updateLabels = function()
{
    for (var i = 0, len = this.resources.length; i < len; i++) 
    {
       	var resName = this.resources[i].name.toLowerCase();
       	this[resName].innerHTML = NumberUtils.floor(this.bank.getValue(this.resources[i]));
       	this[resName+'Capacity'].innerHTML = NumberUtils.floor(this.capacity.getValue(this.resources[i]));
    }
}

CapacityRatioView.prototype.needUpdate = true;
CapacityRatioView.prototype.update = function(estate)
{
	this.readOnly = !estate.accessDeputy;
	this.estateId = estate.id;
	this.bank = estate.bank;
	if (this.needUpdate)
	{
		this.capacity = estate.capacity.copy();
		this.capacity.setValue(Resource.PEOPLE, 0);
	}
	
	this.refreshFreePlace();
	
	$(this.wrapper).find('picker').removeClass('error waiting');
	
	this.fitLayout();
	this.tick();
}

CapacityRatioView.prototype.fitLayout = function()
{
	this.w = $(this.wrapper).width();
	this.c = this.capacity.sum();
	this.k = this.w / this.c;
	setTimeout($.proxy(function(){this.updateUI(true);}, this), 300);
}

CapacityRatioView.prototype.refreshFreePlace = function()
{
	this.free = 0;
	for (var i = 0, len = this.resources.length; i < len; i++)
	{
		var r = this.resources[i], b = this.bank.getValue(r), c = this.capacity.getValue(r);
		this.free += (b < c) ? (c - b) : 0;
	}
	//return this.capacity.sum() - this.bank.sum();
}

CapacityRatioView.prototype.w = 0; // real width in px
CapacityRatioView.prototype.c = 0; // total warehouses capacity
CapacityRatioView.prototype.k = 0; // px per 1 res
CapacityRatioView.prototype.free = 0; // free capacity

CapacityRatioView.prototype.px2val = function(val)
{
	val = isNaN(parseFloat(val)) ? 0 : parseFloat(val);
	var s = val < 0 ? -1 : 1;
	return Math.max(0, Math.min(this.c, Math.abs(val) / this.k)) * s;
}

CapacityRatioView.prototype.val2px = function(val)
{
	val = isNaN(parseFloat(val)) ? 0 : parseFloat(val);
	var s = val < 0 ? -1 : 1;
	return Math.max(0, Math.min(this.w, Math.abs(val) * this.k)) * s;
}

CapacityRatioView.prototype.updateUI = function(fullUpdate)
{
	this.updateLabels();
	
	var left = 0;
	var pickers = this.wrapper.querySelectorAll('picker');
	var dots = this.wrapper.querySelectorAll('pickers > dot');
	
	this.refreshFreePlace();
	for (var i = 0; i < this.resources.length; i++)
	{
		var r = this.resources[i]
		var res = this[r.name];
		if (res)
		{
			// position and resize resource container
			var c = this.capacity.getValue(r);
			var w = this.val2px(c);
			
			if (fullUpdate)
			{
				var b = this.bank.getValue(r);
				with (res)
				{
					style.width = Math.ceil(w) + 'px';
					style.left = Math.floor(left) + 'px';
					style.minWidth = this.val2px(Math.min(b, c)) + 'px';
					style.maxWidth = this.val2px(b + this.free) + 'px';
					querySelector('bank').style.width = this.val2px(b) + 'px';
					querySelector('empty').style.width = this.val2px(c) + 'px';
				}
			}
			
			left += w;
			
			// position picker after resource
			if (pickers[i])
			{
				//var ph = pickers[i].offsetHeight;
				pickers[i].style.left = (left - pickers[i].offsetWidth / 2) + 'px';
				dots[i].style.left = (left - dots[i].offsetWidth / 2) + 'px';
			}
		}
	}
	
	
	if (typeof this.onChange == 'function')
	{
		this.onChange(this.capacity);
	}
}

CapacityRatioView.prototype.updateValues = function(bank)
{
	this.bank = bank.copy();
	this.updateUI(true);
}

CapacityRatioView.prototype.reset = function()
{
	this.estateId = null;
}

CapacityRatioView.prototype.onApply = function()
{
    if (!this.estateId) return;
    if (!Quarantine.contains('CapacityRatioView-' + this.estateId.toString())) 
    {
        Quarantine.put('CapacityRatioView-' + this.estateId.toString());
        var c = this.capacity.copy();
        delete(c.supportedProps);
        
        $(this.wrapper).find('picker').addClass('waiting')
        Service.city.callMethod('setCapacityRatio', [this.estateId, c], 
        	$.proxy(function(){
        		$(this.wrapper).find('picker').removeClass('error waiting');
        	}, this), 
        	$.proxy(function(){
        		$(this.wrapper).find('picker').removeClass('error waiting').addClass('error');
        		this.reset();
        	}, this)
        );
    }
}
