function CellularView()
{
	this.allHexes = [];
	
	this.needToUpdateWall = false;
	
	this.onHexClick = $.proxy(this.onHexClick, this);
	this.onHallClick = $.proxy(this.onHallClick, this);
	this.onHallTipCreate = $.proxy(this.onHallTipCreate, this);
	
	this.validHexesCount = 0;
	this.oldGateState = null;
	this.oldHexesState = {};
};

CellularView.CANVAS_WIDTH = 1198;
CellularView.CANVAS_HEIGHT = 815;

CellularView.prototype.SELECTED_NONE = -100;

CellularView.prototype.estate = null;
CellularView.prototype.selectedCellIndex = CellularView.prototype.SELECTED_NONE;
CellularView.prototype.menuCellIndex = CellularView.prototype.SELECTED_NONE;

CellularView.prototype.allHexes = null;
CellularView.prototype.wasInit = false;

CellularView.prototype.ownDialog = null; // Function

CellularView.prototype.cityScroll = null;

CellularView.prototype.wrapper = null;
CellularView.prototype.hexContainer = null;
CellularView.prototype.hexCanvas = null;
CellularView.prototype.hallView = null;

//CellularView.prototype.blambaView = null;

CellularView.prototype.__construct = function()
{
	$(this.DOM).addClass('cellular-view');
	
	this.hexCanvas.width = CellularView.CANVAS_WIDTH*GAME_IMAGES_PACK;
	this.hexCanvas.height = CellularView.CANVAS_HEIGHT*GAME_IMAGES_PACK;
	
	this.hexContainer.style.width = (CellularView.CANVAS_WIDTH*GAME_IMAGES_PACK) + "px";
	this.hexContainer.style.height = (CellularView.CANVAS_HEIGHT*GAME_IMAGES_PACK) + "px";
	
	this.ctx = 
	{
		main: this.hexCanvas.getContext("2d"),
		status: this.hexCanvas.getContext("2d")
	}
	
	this.hallView = COM.create('BuildingView', null, {ctx: this.ctx});
	
	this.fitLayout = $.proxy(function(container, noCorrectScroll)
	{
		//if($(this.DOM).is(':visible'))
		//{
			$(this.DOM).css("width", container.width);
			$("> .cellular-wrapper", this.DOM).css("width", container.width);
			$("> .cellular-wrapper", this.DOM).css("height", container.height);
			if(this.cityScroll) this.cityScroll.refresh();
		//}
	}, this);
	
	var options = {
		bounce: false,
		bounceLock: false,
		onScrollMove: $.proxy(this.hexesStartScroll, this),
		onScrollEnd: $.proxy(this.hexesEndScroll, this),
		sensitivity: 12,
		momentum: false,
		scrollbarColor: 'rgba(255,255,255,0.5)',
		zoom: false, zoomMin: HEXES_SCALE, zoomMax: HEXES_SCALE
	};
	
	this.cityScroll = new iScroll(this.hexContainer.parentNode, options);
}

CellularView.prototype.hexesStartScroll = function()
{
	APP.hideCityPanels();
}

CellularView.prototype.hexesEndScroll = function()
{
}

CellularView.prototype.init = function(hexState)
{
	this.initHexes();
	
	this.alignHallView();
	
	this.wasInit = true;
	
	//this.applyHexState(hexState);
}

CellularView.prototype.alignHallView = function()
{
	if (!this.estate || !this.estate.hall) 
	{
		return;
	}
	this.hallView.getImage(this.estate.hall, null, this.estate.race);
	
	this.hallView.x += APP.hexesOffset.left - APP.hexesSize.width * 0.5;
	this.hallView.y += APP.hexesOffset.top - APP.hexesSize.height * 1.5;
	
	// fix position relative to whole view
	this.hallView.buildingCenterX = this.hallView.x + this.hallView.buildingWidth / 2;
	this.hallView.buildingBottomY = this.hallView.y + this.hallView.buildingHeight;
	this.hallView.buildingCenterY = this.hallView.buildingBottomY - this.hallView.buildingHeight / 2;
}

CellularView.prototype.initHexes = function()
{
}

CellularView.prototype.clearHexesUpdateState = function()
{
	for(var i=0; i<this.allHexes.length; i++)
    {
    	this.allHexes[i].updated = false;
    }
}

CellularView.prototype.getNextHex = function()
{
	var h;
	var minIY = Number.MAX_VALUE;
	var present = false;
	for(var i=0; i<this.allHexes.length; i++)
    {
    	h = this.allHexes[i];
    	if(!h.updated && h.y < minIY)
    	{
    		minIY = h.y;
    		present = true;
    	}
    }
    
    if(present)
    {
    	var minIX = Number.MAX_VALUE;
		var hIndex = -1;
		for(var i=0; i<this.allHexes.length; i++)
	    {
	    	h = this.allHexes[i];
	    	if(!h.updated && h.y == minIY && h.x < minIX)
	    	{
	    		minIX = h.x;
	    		hIndex = i;
	    	}
	    }
    	
    	if(hIndex >= 0)
    	{
    		this.allHexes[hIndex].updated = true;
    		return this.allHexes[hIndex];
    	}
    }
    
    return false;
}

CellularView.prototype.addHexes = function(hexesX, hexesY, indices)
{
    for (var i = 0; i < indices.length; i++)
    {
        var attrs =
        {
			name: 'cell-' + indices[i], 
			cellIndex: indices[i]
		};
        
        var data = 
		{
			cellIndex: indices[i],
			x: hexesX[i],
			y: hexesY[i],
			onClick: this.onHexClick,
			ctx: this.ctx
		};
        
        hex = new Hex();
        hex.__construct(data);
        this.putHex(hex);
    }
}

CellularView.prototype.putHex = function(h)
{
    this.allHexes[h.cellIndex] = h;
}

CellularView.prototype.getHallIndex = function()
{
    return 0;
}

CellularView.prototype.applyHexState = function(state)
{
    if (this.wasInit)
    {
        this.updateHexState(state);
    }
}

CellularView.prototype.updateHexState = function(state)
{
    var h;
    for(var i=0; i<this.allHexes.length; i++)
    {
    	h = this.allHexes[i];
    	
    	h.showBuilding = state.buildings;
        h.showCellIndex = state.numbers;
        h.buildingView.showLevel = state.levels && this.estate && this.estate.getEstateType().name != EstateType.EVIL.name;
        h.buildingView.showCountdown = state.countDown;
        h.showGrid = state.grid;
    }
    
    this.hallView.showLevel = state.levels && this.estate && this.estate.getEstateType().name != EstateType.EVIL.name;
    this.hallView.showCountdown = state.countDown;
    
    this.oldHexesState = state;
    
    APP.showLoader();
    setTimeout($.proxy(function()
    {
    	this.startDraw(false);
    	
    }, this), 10);
}

CellularView.prototype.glowCell = function(index, doHighlight)
{
    if (index >= 0)
    {
        this.allHexes[index].glowBuilding(doHighlight);
    }
}

CellularView.prototype.deselectCell = function()
{
    if (this.selectedCellIndex != this.SELECTED_NONE)
    {
        this.glowCell(this.selectedCellIndex, false);
        this.selectedCellIndex = this.SELECTED_NONE;
    }
}

CellularView.prototype.selectCell = function(cellIndex)
{
    this.selectedCellIndex = cellIndex;
    this.glowCell(this.selectedCellIndex, true);
}

CellularView.prototype.onHallClick = function(e)
{
    //if (null !== getTopDialog()) return;

    if (Quarantine.contains('Quarantine.HALL')) 
    {
        return;
    }
    
    HintArrow.OPEN_CITYHALL_MENU.understood();
	
    if (this.estate instanceof FRuin || this.estate instanceof FEvil || this.estate instanceof FStonehenge)
    {
    	this.onHallTipCreate();
    }
    else if (this.canEnterHall())
    {
    	this.onEnterHall();
    }
    APP.playSound();
}

CellularView.prototype.canEnterHall = function()
{
    if (!this.estate.hall.accessInfo) 
    {
        Alert.error(I18.f('ACCESS_LEVEL_REQUIRED', FEstate.BUILDING_INFO_ACCESS_LEVEL));
        return false;
    }
    return true;
}

CellularView.prototype.updateMenuForHall = function()
{
    if (!this.manageDialog) return;
    
    $(this.manageDialog.actionUpgrade).off(CLICK_EVENT);
    COM.bindExclusiveClick(this.manageDialog.actionUpgrade, $.proxy(this.onUpgradeHall, this));

    this.manageDialog.updateActions();
}

CellularView.prototype.onEnterHall = function()
{
    var cellIndex = -1;
    
    this.manageDialog = COM.create('ManageDialog', null, {estate: this.estate, _cellIndex: -1});
   	this.manageDialog.popup(this, -1, this.estate);
   	this.manageDialog.switchDialog = $.proxy(this.ownDialog, this);
   	
   	this.ownDialog(this.manageDialog);
    
    this.updateMenuForHall();
    
    this.manageDialog.onClose = $.proxy(function()
    {
		if (ManageDialog.prototype.onClose.call(this.manageDialog) === false) return false;
    	this.ownDialog(null);
    	this.manageDialog = null;
    	
    }, this);
}


CellularView.prototype.isHexClickAvailable = function(h)
{
    if (Quarantine.contains(h))
    {
        return false;
    }
    var cell = this.estate.getCellByIndex(h.cellIndex);
    
    if (!cell) return false;
    
    if (cell.hasBuilding() && cell.building.isForeign)
    {
        return false;
    }
    if (this.estate.__className__ == "FCamp")
    {
        return false;
    }
    
    return (cell.hasBuilding() || (this.estate.accessDeputy && this.estate.isCellReachable(h.cellIndex)));
}

CellularView.prototype.manageDialog = null;

CellularView.prototype.onHexClick = function(e)
{
    //if (null !== getTopDialog()) return;
    
    var h = e.target ? COM.get(e.target) : e;
    var cell = this.estate.getCellByIndex(h.cellIndex);
    
    if (!cell) return;
    //console.log('HEX CLICK', h.cellIndex);
    
    if (!this.isHexClickAvailable(h))
    {
        if (Quarantine.contains(h))
        {
            return;
        }
        if (cell.hasBuilding() && cell.building.isForeign)
        {
            return;
        }
        
        var message = null;
        if (this.estate.accessDeputy)
        {
            if (this.estate.canManageOnSectors() && !cell.hasBuilding())
            {
                if (this.estate.isOnSector(h.cellIndex))
                {
                    if (!this.estate.isSectorsScienceLearned())
                    {
                        message = I18.f("ADVISOR_INFO_SCIENCE_SECTORS_REQUIRED");
                    }
                    else if (!this.estate.isMaxHallLevel())
                    {
                        message = I18.f("ADVISOR_INFO_MAXIMUM_HALL_LEVEL_REQUIRED", I18.getBuildingName(BType.CITYHALL, this.estate.owner.race));
                    }
                    else if (!this.estate.isGateOpen())
                    {
                        message = I18.f("ADVISOR_INFO_OPEN_GATE_REQUIRED");
                    }
                }
                else if (!this.estate.isInsideCityWall(h.cellIndex))
                {
                    message = I18.f("ADVISOR_INFO_INSIDE_WALL_REQUIRED", I18.getBuildingName(BType.CITYHALL, this.estate.owner.race));
                }
            }
            else if (this.estate.__className__ == "FCamp")
            {
                message = I18.f("ADVISOR_INFO_CANNOT_BUILD_IN_CAMP");
            }
            else if (this.estate.__className__ == "FCastle")
            {
                message = I18.f("ADVISOR_INFO_CANNOT_BUILD_IN_CASTLE");
            }
        }
        
        if (message)
        {
            AdvisorInfoDialog.create(message, this.estate.owner.race).popup(h);
        }
        else
        {
            if(cell.resource && cell.resource.name != "NULL") ResourcesTip.create(this.estate, h.cellIndex);
        }
       	
		APP.playSound();

		return;
    }
    
    this.deselectCell();
    
    if (!cell.hasBuilding())
    {
        var estate = this.estate;
        
        var bd = COM.create('BuildDialog');
        bd.init(this.estate, h.cellIndex);
		bd.popup();
		
		APP.playSound('action');
    }
    else
    {
        this.selectCell(h.cellIndex);
        this.menuCellIndex = h.cellIndex;
        
        var reason = this.canEnterBuildingReason(h.cellIndex, h.building);
        var canEnter = cell.hasBuilding();
        if (canEnter)
        {
	        if (this.manageDialog)
	        {
	        	this.manageDialog.destroy();
	        	this.manageDialog = null;
	        }
	        
	        this.manageDialog = COM.create('ManageDialog', null, {estate: this.estate, _cellIndex: h.cellIndex, reason: reason});
	        
	        this.manageDialog.switchBuilding = $.proxy(this.switchBuilding, this);
	        this.manageDialog.switchDialog = this.ownDialog;
   	        
	        this.manageDialog.popup(this, h.cellIndex, this.estate);
	        
	        this.ownDialog(this.manageDialog);
   	        
   	        this.updateMenu(h.cellIndex, h.building);
   	        this.manageDialog.onClose = $.proxy(function()
   	        {
   	        	if (ManageDialog.prototype.onClose.call(this.manageDialog) === false) return false;
   	        	this.ownDialog(null);
   	        	this.manageDialog = null;
   	        	
   	        }, this);
        }
        APP.playSound();
    }
}

CellularView.prototype.canEnterBuildingReason = function(cellIndex, building)
{
    var cell = this.estate.getCellByIndex(cellIndex);
    if (!cell.hasBuilding()) return null;
    
    if (cell.building.type.isClanCapacitor() && !this.estate.owner.isInRegisteredClan()) 
    {
        return I18.f('CLAN_IS_REGISTERING_LOCK');
    } 
    else if (this.estate.isEspionage && cell.building.type.isMarket()) 
    {
        return I18.f('NO_ACCESS_TO_MARKET');
    } 
    else if (!cell.building.accessType) 
    {
        return I18.f('ACCESS_LEVEL_REQUIRED', FEstate.BUILDING_TYPE_ACCESS_LEVEL);
    } 
    else if (!cell.building.accessInfo) 
    {
        return I18.f('ACCESS_LEVEL_REQUIRED', FEstate.BUILDING_INFO_ACCESS_LEVEL);
    } 
    else if (cell.building.isSuppressed) 
    {
       	var msg = [];
        for (var i = 0; i < this.estate.buffs.length; i++)
        {
        	var b = this.estate.buffs[i];
            if (b.buffType.name == BuffType.SUPPRESS_BUILDING.name)
            {
	            if ((!b.btype) || (b.btype.name == cell.building.type.name)) 
	            {
	                msg.push(I18.f('EFFECT_ACTIVE_TILL', I18.getBuffCause(b), Tick.toTimeFuture(b.cast.tillTick)));
	            }
            }
        }
        if (msg.length)
        {
        	return msg.join('<br />');
        }
    } 
    return null;
}

CellularView.prototype.updateMenu = function(cellIndex, building)
{
    if (!this.manageDialog) return;
    
    var cell = this.estate.getCellByIndex(cellIndex);
    
    if (!cell.hasBuilding()) 
    {
        return;
    }
    
	$(this.manageDialog.actionDestroy).off(CLICK_EVENT);
	COM.bindExclusiveClick(this.manageDialog.actionDestroy, $.proxy(function()
    {
		var d = COM.create('DeleteDialog');
		d.popup(this, this.estate, cell.building, cellIndex);
		
    }, this));

	$(this.manageDialog.actionRepair).off(CLICK_EVENT);
    COM.bindExclusiveClick(this.manageDialog.actionRepair, $.proxy(function()
    {
        var d = COM.create('RepairDialog');
        d.popup(this, this.estate, cell.building);
        this.ownDialog(d);
        
    }, this));
    
	$(this.manageDialog.actionUpgrade).off(CLICK_EVENT);
    COM.bindExclusiveClick(this.manageDialog.actionUpgrade, $.proxy(function() 
    {
        var d = COM.create('UpgradeDialog');
        d.popup(this, this.estate, cellIndex);
        this.ownDialog(d);
        
    }, this));
    
    this.manageDialog.updateActions();
}

CellularView.prototype.setHexesToCenter = function()
{
	if(this.cityScroll) this.cityScroll.refresh();
	
	var scale = (this.cityScroll && this.cityScroll.scale) ? this.cityScroll.scale : 1;
	
	var left = -(this.hexCanvas.width*scale - this.wrapper.offsetWidth)/2;
	var top = -(this.hexCanvas.height*scale - this.wrapper.offsetHeight)/2;
	
	if(this.cityScroll) this.cityScroll.scrollTo(left, top, 0);
}

CellularView.prototype.update = function(estate, forceUpdate)
{
    this.updateStartTime = Tick.timeInMs();
    
    var newEstate = !this.estate || !this.estate.id.equals(estate.id);
      
    this.estate = estate;
    
    var isFullUpdate = newEstate || forceUpdate;
    
    isFullUpdate |= (this.oldGateState !== this.estate.isGateOpen());
    
    this.oldGateState = this.estate.isGateOpen();
    
    //count of valid hexes and change hexes view
    var cnt = 0, hexesChanged = false;
    for(var i=0; i<this.allHexes.length; i++)
    {
    	this.allHexes[i].prepareForUpdate(this.estate);
    	if(!this.allHexes[i].checkInvalid(estate))
    	{
    		if(newEstate !== true)
    		{
    			hexesChanged |= this.allHexes[i].isChanged();
    		} 
    		cnt++;
    	}
    }
    
    isFullUpdate |= (cnt != this.validHexesCount);
    
    this.validHexesCount = cnt;
    
    isFullUpdate |= hexesChanged;
    
    if(this.estate.hall)
    {
    	this.hallView.building = this.estate.hall;
    	isFullUpdate |= this.hallView.isChanged(this.estate.hall, null, this.estate.race);
    }
    
	if (this.manageDialog)
	{
		this.manageDialog.update(this.estate);
	}
    
    if(!isFullUpdate)
    {
    	this.tick();
    	return;
    }
    
    console.log("Full update");
    
    this.startDraw(newEstate);
}

CellularView.prototype.addCacheImage = function(cache, src)
{
	var cnt = 0;
	for(var i in cache)
	{
		if(cache[i] == src) return cache;
		
		cnt++;
	}
	
	cache["img"+cnt] = src;
	
	return cache;
}

CellularView.prototype.clearCanvas = function()
{
	this.hexCanvas.width = this.hexCanvas.width;
}

CellularView.prototype.startDraw = function(newEstate)
{
	APP.showLoader();
	
	APP.clearImagesCache();
	
	var cache = {};
	
	cache.bg = this.estate.terrain.assetGroup;
	if(this.estate.blamba)
	{
		cache.blamba = Assets[this.estate.blamba+"_BLAMBA"];
	}
	
	//cache.race = Assets.getRaceAssets(this.estate.owner.race);
	
	var h, src;
	for(var i=0; i<this.allHexes.length; i++)
	{
		h = this.allHexes[i];
		src = h.buildingView.getImage(h.buildingView.building, h.buildingView.order, this.estate.owner.race);
		if(src)
		{
			cache = this.addCacheImage(cache, src);
		}
	}
	
	if(this.estate.hall)
	{
		cache = this.addCacheImage(cache, this.hallView.getImage(this.estate.hall));
	}
	
	cache.specific = Assets.getRaceAssets(this.estate.owner.race).specific;
	
	cache.HEX_EMPTY = CommonAssets.HEX_EMPTY;
	cache.HEX_ERROR = CommonAssets.HEX_ERROR;
	cache.CELL_GRAIN = CommonAssets.CELL_GRAIN;
	cache.CELL_STONE = CommonAssets.CELL_STONE;
	cache.CELL_IRON = CommonAssets.CELL_IRON;
	cache.CELL_GOLD = CommonAssets.CELL_GOLD;
	cache.CELL_GRAIN_UNREACHABLE = CommonAssets.CELL_GRAIN_UNREACHABLE;
	cache.CELL_STONE_UNREACHABLE = CommonAssets.CELL_STONE_UNREACHABLE;
	cache.CELL_IRON_UNREACHABLE = CommonAssets.CELL_IRON_UNREACHABLE;
	cache.CELL_NOTHING = CommonAssets.CELL_NOTHING;
	cache.CELL_GRID = CommonAssets.CELL_GRID;
	cache.BLOCKER_LAKE = CommonAssets.BLOCKER_LAKE;
	cache.BLOCKER_ROCK = CommonAssets.BLOCKER_ROCK;
	cache.BLOCKER_TREE = CommonAssets.BLOCKER_TREE;
	cache.BLOCKER_DARKTREE = CommonAssets.BLOCKER_DARKTREE;
	cache.BUILDING_LEVEL = CommonAssets.BUILDING_LEVEL;
	cache.BUILDING_LEVEL_WARNING = CommonAssets.BUILDING_LEVEL_WARNING;
	cache.COUNTDOWN_BG = CommonAssets.COUNTDOWN_BG;
	cache.CRYSTAL_SIGN = CommonAssets.CRYSTAL_SIGN;
	cache.INPROGRESS_BUILD = CommonAssets.INPROGRESS_BUILD;
	cache.INPROGRESS_UPGRADE = CommonAssets.INPROGRESS_UPGRADE;
	cache.BROKEN = CommonAssets.BROKEN;
	cache.INPROGRESS_FROZEN = CommonAssets.INPROGRESS_FROZEN;
	cache.INPROGRESS_REPAIR = CommonAssets.INPROGRESS_REPAIR;
	cache.STATUS_ON = CommonAssets.STATUS_ON;
	cache.STATUS_OFF = CommonAssets.STATUS_OFF;
	
	cache.CASTLEGATE = Assets.Monster.Buildings.CASTLEGATE;
	cache.CASTLEGATE_EMPTY = Assets.Monster.Buildings.CASTLEGATE_EMPTY;
	
	APP.addPreloadImages(cache, STATIC_URL);
	
	APP.waitPreloadImages($.proxy(function()
    {
    	this.draw(newEstate);
    }, this));	
}

CellularView.prototype.draw = function(newEstate)
{
    this.clearCanvas();
    
    var context = this.ctx.main;
    
	var back = APP.getPreloadImage(this.estate.terrain.assetGroup);
	
	var w = CellularView.CANVAS_WIDTH*GAME_IMAGES_PACK;
	var h = CellularView.CANVAS_HEIGHT*GAME_IMAGES_PACK;

    //context.drawImage(back, 0, 0, back.width, back.height, 0, 0, w, h);
    this.hexCanvas.style.background = "url("+back.src+")";

	if(this.estate.blamba)
	{
		/*
		this.blambaView.attr("src", STATIC_URL + Assets[estate.blamba+"_BLAMBA"]);
		
		if(this.blambaView.get(0).complete) this.alignBlamba();
		else this.blambaView.one("load", $.proxy(this.alignBlamba, this));
		*/
		
		var img = APP.getPreloadImage(Assets[this.estate.blamba+"_BLAMBA"]);
		
		var left = (this.hexCanvas.width - img.width)/2;
		var top = (this.hexCanvas.height - img.height)/2;
		try {
			context.drawImage(img, 0, 0, img.width, img.height, left, top, img.width, img.height);
		} catch (e) {}
	}
	
	if(this.centralCobbles && !this.estate.isExpeditionReport)
	{
		var img;
		for(var i=0; i<this.centralCobbles.length; i++)
		{
			img = this.centralCobbles[i];
			var left = css2int($(img).css("left"))*GAME_IMAGES_PACK + APP.hexesOffset.left;
			var top = css2int($(img).css("top"))*GAME_IMAGES_PACK + APP.hexesOffset.top;
			
			try {
				context.drawImage(img, 0, 0, img.width, img.height, left, top, img.width, img.height);
			} catch (e) {}
		}
	}
    
    this.clearHexesUpdateState();
    var h, lastY = 0, hallUpdated = false;
    while(h = this.getNextHex())
    {
    	if(this.needToUpdateWall)
    	{
    		h.onGateClick = $.proxy(this.onGateClick, this);
    		h.neighbours = this.getNeighboursFor(h.cellIndex);
    	}
    	
    	if(this.estate.hall && this.hallView.zIndex >= lastY && this.hallView.zIndex <= h.y)
		{
			this.hallView.getImage(this.estate.hall, this.estate.getOrderByCellIndex(-1), this.estate.race);
			
			this.alignHallView();
			this.hallView.update(this.estate.hall, this.estate.getOrderByCellIndex(-1), this.estate.race);
			hallUpdated = true;
		}
    	 
    	h.update(this.estate, this.needToUpdateWall);
    	
    	lastY = h.y;
    }
    
    if(!hallUpdated) 
    {
    	this.hallView.getImage(this.estate.hall, this.estate.getOrderByCellIndex(-1), this.estate.race);
    	
    	this.alignHallView();
    	this.hallView.update(this.estate.hall, this.estate.getOrderByCellIndex(-1), this.estate.race);
    }
    
    if(newEstate)
    {
		this.setHexesToCenter();
	}
	
	APP.clearImagesCache();
	
	if(this.updateGate)
	{
		this.updateGate();
	}
	
	this.tick(true);
	
	APP.hideLoader();
	
	if(ANDROID_MODE)
	{
		this.hexCanvas.style.display = "none";
		setTimeout($.proxy(function()
		{
			this.hexCanvas.style.display = "block";
		}, this), 10);
	}
}

CellularView.prototype.tipIsOfProperClass = function(cellIndex)
{
}

CellularView.prototype.tick = function(force)
{
    var start = Tick.timeInMs();
    
    this.hallView.tick(null, force);
    
    for(var i=0; i<this.allHexes.length; i++)
    {
    	this.allHexes[i].tick(force);
    }
}

CellularView.prototype.onHexTipCreate = function(e, cellIndex)
{
}

CellularView.prototype.switchBuilding = function(md, next)
{
    newSelection = this.nextCellWithSameBuildingByIndex(next);
    this.deselectCell();
    this.selectCell(newSelection);
    md.switchCell(this.selectedCellIndex);
    
    this.updateMenu(md._cellIndex, md.building);
}

CellularView.prototype.nextCellWithSameBuildingByIndex = function(next)
{
    if (this.selectedCellIndex < this.estate.getMaxCellIndex() && this.selectedCellIndex >= 0) 
    {
        var currentCell = this.selectedCellIndex;
        var b = this.allHexes[this.selectedCellIndex].building;
        var inc = next ? 1 : -1;
        do 
        {
            currentCell += inc;
            if (currentCell >= this.estate.getMaxCellIndex()) 
            {
                currentCell = 0;
            }
            if (currentCell < 0) 
            {
                currentCell = this.estate.getMaxCellIndex() - 1;
            }
            var newB = this.allHexes[currentCell].building;
            if (newB && !newB.isForeign && newB.typeName == b.typeName) 
            {
                return currentCell;
            }
        } while (currentCell != this.selectedCellIndex);
    }
    return this.selectedCellIndex;
}

CellularView.prototype.onHallTipCreate = function()
{
   	var tip = HallTip.create(this.estate);
   	var d = TipDialog.create(tip.DOM, tip.getTitle());
   	return d;
}

CellularView.prototype.onUpgradeHall = function()
{
    var ud = COM.create('UpgradeDialog');
    ud.onResult = $.proxy(function(builders, workers)
    {
        Quarantine.put('Quarantine.HALL');
        Service.city.callMethod('upgradeBuilding', [this.estate.id, this.estate.hall.id, builders, workers], $.proxy(function(data){
        	this.manageDialog.update(data);
        }, this));
        
    }, this);
    
    ud.popup(this, this.estate, -1);
    this.ownDialog(ud);
}

CellularView.prototype.onGateClick = function(parent)
{
    if (!this.estate.hasBuff(BuffType.GATES_CANT_BE_CLOSED))
    {
        if (!this.estate.isGateChanging() && this.estate.accessDeputy)
        {
            COM.create("GateAdvisorDialog").popup(parent, this.estate);
        }
        else if (this.estate.isCity && this.estate.isGateOpen() && this.estate.accessManageble)
        {
            this.ownDialog(COM.create("GateChestDialog").popup(parent, null, this.estate));
        }
    }
}