function ChangeCitySkinDialog()
{
	callParentConstructor(ChangeCitySkinDialog.prototype, this);
	
	this.fromCity = null;
	this.skinIndex = -1;
	this.forCity = null;
	this.race = null;
	this.btype = null;
	this.level = null;
	this.onCityUpdated = function(){};
}

extendClass(ChangeCitySkinDialog.prototype, UpdatableEstateDialog.prototype);

ChangeCitySkinDialog.prototype.__construct = function()
{
	COM.bindExclusiveClick(this.yesButton, $.proxy(this.doAskAndApply, this));
	
	COM(this.citySelector).onCityChange = $.proxy(this.onCityChange, this);
	COM(this.skinSelector).onSkinChanged = $.proxy(this.onSelectSkin, this);
}

ChangeCitySkinDialog.prototype.fromCity = null;
ChangeCitySkinDialog.prototype.skinIndex = -1;
ChangeCitySkinDialog.prototype.forCity = null;

ChangeCitySkinDialog.prototype.onCityUpdated = function(){};

ChangeCitySkinDialog.create = function(c)
{
    var dialog = COM.create('ChangeCitySkinDialog');
   	
   	dialog.animatedPopup = true;
   	dialog.shaderOpacity = 0.5;
   	
   	dialog.update(c);
    dialog.onCityChange(c.getLink());
    
    return dialog;
}

ChangeCitySkinDialog.prototype.tick = function()
{
    if (UI.show(this.skinIndex > -1, [this.priceView]))
    {
        var neededPrice = FConfig.getCitySkinPrice(this.fromCity.race, this.skinIndex);
        COM(this.priceView).update(neededPrice);
		COM(this.yesButton).reason = this.fromCity.owner.pearl >= neededPrice ? null : I18.f('NOT_ENOUGH_RESOURCES');        
    }
    else 
    {
		COM(this.yesButton).reason = I18.f('SELECT_CITY_SKIN');        
    } 
    
}

ChangeCitySkinDialog.prototype.onSelectSkin = function(newSkinIndex)
{
    this.skinIndex = newSkinIndex;
    this.updateHallTip();
    this.tick();
}

ChangeCitySkinDialog.prototype.onCityChange = function(link)
{
    this.forCity = link.estateInfo.id;
    this.selectCitySkinText.innerHTML = I18.getChangeSkinText(link.estateInfo.race);
    COM(this.skinSelector).update(link.estateInfo.race, link.estateInfo.estateType.hallType, link.estateInfo.level, link.estateInfo.skinIndex);
    
    this.race = link.estateInfo.race;
    this.btype = link.estateInfo.estateType.hallType;
    this.level = link.estateInfo.level;
    this.skinIndex = -1;
    
    this.updateHallTip();
    
    this.tick();
}

ChangeCitySkinDialog.prototype.filterCities = function(links)
{
    var result = [];
    for (var i = 0; i < links.length; i++)
    {
    	var link = links[i];
        if (link.estateInfo.estateType.name == EstateType.CITY.name || link.estateInfo.estateType.name == EstateType.CASTLE.name) 
        {
            result.push(link);
        }
    }
    return result;
}

ChangeCitySkinDialog.prototype.update = function(estate)
{
    var cls = estate.__CLS__.split('.').pop();
    if (!(cls == 'FCity' || cls == 'FCastle')) 
    {
        return;
    }
    
    this.fromCity = estate;
	
	/*
	this.setCaption(I18.f('LOCAL_CHANGE_ARCHITECTURE')); 
	this.setSubCaption((this.fromCity.__CLS__.split('.').pop() == 'FCastle') 
		? I18.f('CHANGE_CASTLE_SKIN_HEADER') 
		: I18.f('CHANGE_SKIN_HEADER')
	);
	*/
	
	COM(this.priceView).ownerUser = this.fromCity.owner;
    COM(this.citySelector).updateCities(this.filterCities(this.fromCity.links));
    COM(this.citySelector).cityId = this.fromCity.id;
    
    this.tick();
}

ChangeCitySkinDialog.prototype.doAskAndApply = function()
{
    var neededPrice = FConfig.getCitySkinPrice(this.fromCity.race, this.skinIndex);
    if (neededPrice == 0) 
    {
        Alert.create(I18.f('CHANGE_SKIN_CONFIRMATION'), null, $.proxy(this.doApply, this));
    } 
    else 
    {
        this.doApply();
    }
}

ChangeCitySkinDialog.prototype.doApply = function()
{
    APP.showLoader();
	COM(this.yesButton).disabled = true;
    Service.city.callMethod('changeSkin', [this.forCity, this.fromCity.id, this.skinIndex], 
	    $.proxy(function (e)
	    {
	        if (typeof this.onCityUpdated == 'function') this.onCityUpdated(e);
	        this.close();
	        APP.hideLoader();
	        
	    }, this),
	    $.proxy(function (e)
	    {
	        this.close();
	        APP.hideLoader();
	        
	    }, this)
	);
}

ChangeCitySkinDialog.prototype.updateHallTip = function()
{
	if (UI.show(this.skinIndex > -1, [this.hintDetails]))
	{
        this.hintTitle.innerHTML = I18.getBuildingName(this.btype, this.race, Resource.WOOD) + " " + I18.f('LVL_X', this.level);
        COM(this.populationIncrease).source = STATIC_INTERFACE_URL + Assets.getFreePeopleIcon(this.race);
		
        if (UI.show(this.btype.name == BType.CITYHALL.name, [this.producesBox]))
        {
            COM(this.produces).alwaysShow = [Resource.WOOD];
            var f = new Funds();
            f.wood = FBuilding.staticGetBaseRaise(this.race, this.btype, this.level, this.skinIndex);
            COM(this.produces).update(f, this.race);
        }
        
        this.resourceCapacityValue.innerHTML = NumberUtils.floor(FBuilding.staticGetBaseWarehouseCapacity(this.race, this.btype, null, this.level, this.skinIndex)) + "";

        COM(this.populationIncrease).value = I18.f('PER_TIME', NumberUtils.floor(FBuilding.staticGetBasePeopleProduction(this.race, this.btype, this.level, this.skinIndex)));
        this.peopleCapacityValue.innerHTML = NumberUtils.floor(FBuilding.staticGetBasePeopleCapacity(this.race, this.btype, this.level, this.skinIndex));
	}
	else
	{
		this.hintTitle.innerHTML = I18.f('SELECT_CITY_SKIN');
	}
}

