function CityGarrisonPanel()
{
	this.estate = null;
    this.garrisonLocked = false;
    this.hideoutLocked = false;
    
    this.scroll = null;
    
    this.alliesView = null;
    this.hideoutView = null;
}

CityGarrisonPanel.prototype.__construct = function()
{
	COM.bindExclusiveClick(this.showAllies, $.proxy(this.onShowAllies, this));
	COM.bindExclusiveClick(this.showHideout, $.proxy(this.onShowHideout, this));
	COM.bindExclusiveClick(this.showCalc, $.proxy(this.onCalcClick, this));
	this.scroll = new iScroll(this.wrapper);
}

CityGarrisonPanel.prototype.fitLayout = function(container)
{
	var h = container.height - $(this.bottompanel).height();
	
	this.wrapper.style.height = h+"px";
	this.garrisonUnblock.style.height = h+"px";
}

CityGarrisonPanel.prototype.reset = function()
{
    this.estate = null;
}

CityGarrisonPanel.prototype.update = function(e)
{
	var sameEstate = this.estate && this.estate.id.equals(e.id);
    this.estate = e;
    
    this.updateGarrison(!sameEstate);
    this.tickGarrison();
    
    this.updateHideoutAndAlliesButton(e);
    
    if(sameEstate)
    {
    	if(this.alliesView)
	    {
	    	this.alliesView.update(this.estate);
	    }
	    
	    if(this.hideoutView)
	    {
	    	this.hideoutView.update(this.estate);
	    }
    }
	else
    {
        this.scroll.scrollTo(0, 0, 0);
        
        if(this.alliesView)
	    {
	    	this.alliesView.close();
	    }
	    
	    if(this.hideoutView)
	    {
	    	this.hideoutView.close();
	    }
    }
    
    this.refreshScroll();
}

CityGarrisonPanel.prototype.refreshScroll = function()
{
	if($(this.DOM).is(':visible'))
    {
    	setTimeout($.proxy(this.scroll.refresh, this.scroll), 500);
    }
}

CityGarrisonPanel.prototype.updateGarrison = function()
{
    if (this.estate.accessGarrison)
    {
        COM(this.availableArmyView).setUnits(this.estate.race, this.estate.science.unitLevels, this.estate.monsterRace(), this.estate, FBuff.getBuffsForGarrison(this.estate, this.estate.buffs));
    }
    else
    {
        COM(this.availableArmyView).setAccessLevelRequired(FEstate.GARRISON_ACCESS_LEVEL);
    }
}

CityGarrisonPanel.prototype.tickGarrison = function()
{
	/*
	var oldDelta = this.oldHideout.substractCopy(this.newHideout, false);
    this.oldHideout = this.estate.hideout.copy();
    this.totalArmy = this.estate.garrison.addCopy(this.estate.hideout);
    this.newHideout = this.active ? this.oldHideout.substractOrZeroCopy(oldDelta) : this.oldHideout.copy();
    */
    
    COM(this.availableArmyView).setCount(this.estate.garrison, this.estate.garrisonFaded);
}

CityGarrisonPanel.prototype.updateHideoutAndAlliesButton = function(e)
{
    //COM(this.showHideout).source = STATIC_INTERFACE_URL + (e.accessHideout ? CommonAssets.BUTTON_HIDEOUT : CommonAssets.BUTTON_HIDEOUT_LOCKED);
    //COM(this.showAllies).source = STATIC_INTERFACE_URL + (e.accessHideout ? CommonAssets.BUTTON_ALLIES : CommonAssets.BUTTON_ALLIES_LOCKED);
    
    if(!e.accessHideout)
    {
    	this.showHideout.style.webkitFilter = "grayscale(100%)";
    	this.showAllies.style.webkitFilter = "grayscale(100%)";
    }
    else
    {
    	this.showHideout.style.webkitFilter = "grayscale(0%)";
    	this.showAllies.style.webkitFilter = "grayscale(0%)";
    }

    COM(this.showHideout).enabled = e.accessHideout;
    COM(this.showAllies).enabled = e.accessHideout;
    
    this.showAllies.style.display = (!(e.__className__ == "FRuin")) ? "" : "none";
    this.showHideout.style.display = !(e.__className__ == "FRuin" || e.__className__ == "FCastle" || e.__className__ == "FCatapult") ? "" : "none";
}

CityGarrisonPanel.prototype.tick = function()
{
    this.tickGarrison();
	
	if(this.estate.isGarrisonBlocked)
	{
		this.garrisonUnblock.style.display = "";
		this.garrisonUnblockText.innerHTML = NumberUtils.ceil(Tick.ticksTill(this.estate.garrisonUnblockTick)).toString();
		this.restingMissions.innerHTML = I18.f("MISSION_BLOCKED_GARRISON");
		this.showCalc.style.display = "none";
	}
	else
	{
		this.garrisonUnblock.style.display = "none";
		this.showCalc.style.display = "";
	}

    //this.updateCost();
    this.garrisonLocked = !this.estate.accessGarrison;
    this.hideoutLocked = !this.estate.accessHideout;

    if(this.alliesView)
    {
    	this.alliesView.tick();
    }
    
    if(this.hideoutView)
    {
    	this.hideoutView.tick();
    }
}

CityGarrisonPanel.prototype.onShowAllies = function()
{
	if(this.alliesView)
	{
		return;
	}
	
	this.alliesView = COM.create("CityGarrisonAllies");
	this.alliesView.onClose = $.proxy(this.onCloseAllies, this);
	this.alliesView.update(this.estate);
	this.alliesView.popup();
}

CityGarrisonPanel.prototype.onCloseAllies = function()
{
	this.alliesView = null;
}

CityGarrisonPanel.prototype.onShowHideout = function()
{
    if (this.estate.science.getLevel(Science.HIDEOUT) > 0 || !this.estate.accessDeputy)
    {
        this.hideoutView = COM.create("CityGarrisonHideout");
		this.hideoutView.onClose = $.proxy(this.onCloseHideout, this);
		this.hideoutView.update(this.estate);
		this.hideoutView.popup();
    }
    else
    {
        Alert.create(I18.f("ADVISOR_HIDEOUT_NO_SCIENCE"));
    }
}

CityGarrisonPanel.prototype.onCloseHideout = function()
{
	this.hideoutView = null;
}

CityGarrisonPanel.prototype.onCalcClick = function()
{
    if (!Quarantine.contains(this.showCalc))
    {
        Quarantine.put(this.showCalc);
        CombatCalculatorView.popup(this.estate.getMilitaryInfo());
    }
}