function DoBuildDialog()
{
	callParentConstructor(DoBuildDialog.prototype, this);
	
	this.parentDialog = null;
	this.btype = null;
}

extendClass(DoBuildDialog.prototype, Dialog.prototype);

DoBuildDialog.prototype.__construct = function()
{
	COM.bindExclusiveClick(this.doBuild, $.proxy(this.onBuild, this));
	COM.bindExclusiveClick(this.addToQueue, $.proxy(this.onAddToQueue, this));
	
	COM(this.builders).icon = 'builders';
	COM(this.builders).caption = I18.f('BUILDERS');
	$(this.builders).on("change", $.proxy(this.doBuildersChange, this));
	
	this.requirementsTitle.innerHTML = I18.f('RESOURCES_NEEDED');
	
	COM(this.cost).showIconBacks(true);
	
   	COM(this.available).showZero = true;
	COM(this.available).doHighlightIfEquals = false;
	COM(this.available).alwaysHide = [Resource.PEOPLE];
	COM(this.available).shortView = true;
	COM(this.available).showIconBacks(true);
	
	this.scroll = new iScroll(this.infoWrapper, {scrollbarColor: '#784f0b'});
}

DoBuildDialog.prototype.popup = function(parentDialog)
{
	superMethod(DoBuildDialog.prototype, "popup", this)();
	
	this.parentDialog = parentDialog;
}

DoBuildDialog.prototype.update = function(btype)
{
	//console.log(btype);
	
	this.btype = BType[btype];
	
	//icon
	this.icon.src = STATIC_URL + Assets.B_findBy(this.parentDialog.estate.race, this.btype);
	//$(this.icon).css("-webkit-transform", "scale(2)");
	
	this.setCaption(I18.getBuildingName(this.btype, this.parentDialog.estate.race));
	
    this.addToQueue.style.display = (this.parentDialog.estate.owner.isBuildQueueEnabled() && this.parentDialog.estate.canAddToBuildQueue()) ? "" : "none";
    //COM(this.addToQueue).enabled = this.parentDialog.canBuild(this.btype);
    
    var firstLevel = this.btype.getFirstLevel(this.parentDialog.estate.race);
    COM(this.cost).update(firstLevel.cost, this.parentDialog.estate.race, firstLevel.pearl);
    
    COM(this.builders).rollback();
    
    this.updateSelectedInfo();
    
    var hint = BuildingDescriptionTip.create(this.btype, this.parentDialog.estate, this.parentDialog.cellIndex);
    $(hint.DOM).append("<br/>");
    $(this.hintContainer).append(hint.DOM);
    
    this.updateArrows();
    
    COM(this.available).update(this.parentDialog.estate.bank, this.parentDialog.estate.race);
    
    setTimeout($.proxy(this.scroll.refresh, this.scroll), 300);
}

DoBuildDialog.prototype.updateSelectedInfo = function()
{
    COM(this.cost).setCapacity(this.parentDialog.estate.bank, this.parentDialog.estate.owner.pearl);
    
    var bc = BConstraints.from(this.btype, this.parentDialog.cellIndex, this.parentDialog.estate);
    
    //COM(this.doBuild).enabled = this.parentDialog.canBuild(this.btype, bc) && this.parentDialog.hasFundsAndLimit(this.btype, bc) && !this.parentDialog.isInChaos();
    
    var reasons = [];
    var reasonsQueue = [];
    var level = this.btype.getFirstLevel(this.parentDialog.estate.owner.race);
    
    var r;
    if(!this.parentDialog.estate.canPerformOneMoreTask())
    {
    	reasons.push(I18.getConcurrentLimitText(this.parentDialog.estate.getMaxConcurrentBuildings()));
    }
    /*
    if(this.parentDialog.isInChaos())
    {
    	reasons.push("");
    }
    */
    if(!this.parentDialog.hasRequiredScience(this.btype))
    {
    	r = I18.f("SCIENCE_IS_NOT_RESEARCHED") + ": " + I18.getScience(this.btype.requiredScience);
    	reasons.push(r);
    	//reasonsQueue.push(r);
    }
    if(!bc.isSuccess)
    {
    	r = bc.text.toString();
    	reasons.push(r);
    	reasonsQueue.push(r);
    }
    if(!this.parentDialog.estate.bank.greaterOrEqualsTo(level.cost) || this.parentDialog.estate.owner.pearl < level.pearl)
    {
    	r = I18.f("NOT_ENOUGH_RESOURCES");
    	reasons.push(r);
    	//reasonsQueue.push(r);
    }
    
    COM(this.doBuild).reason = reasons.length > 0 ? reasons.join("<br/>") : null;
    COM(this.addToQueue).reason = reasonsQueue.length > 0 ? reasonsQueue.join("<br/>") : null;
    
    this.constraints.innerHTML = bc.text.toString();
    UI.visible(!bc.isSuccess, [this.constraints]);
    
    COM(this.scienceRequiredWarning).value = this.btype.requiredScience ? I18.getScience(this.btype.requiredScience) : "";
	UI.show(!this.parentDialog.hasRequiredScience(this.btype), [this.scienceRequiredWarning]);
	
	COM(this.builders).updateMaximized(this.parentDialog.estate, (this.btype.getFirstLevel(this.parentDialog.estate.race).builders));

    this.onBuildersChange(COM(this.builders).value);
}

DoBuildDialog.prototype.onBuildersChange = function(count)
{
    if (count > 0)
    {
        COM(this.time).value = this.parentDialog.estate.timeNeededToBuild(this.btype.getFirstLevel(this.parentDialog.estate.race), count, this.parentDialog.estate.science);
    }
    else
    {
        COM(this.time).value = Tick.TICK_INFINITY;
    }
    (this.parentDialog.estate.buildersBonusOverflow ? Style.OVERFLOW : Style.FUNDS).applyTo(COM(this.time).time);
}

DoBuildDialog.prototype.doBuildersChange = function()
{
	this.onBuildersChange(COM(this.builders).value);
}

DoBuildDialog.prototype.onBuild = function()
{
    if (this.parentDialog.canBuild(this.btype) && this.parentDialog.hasFundsAndLimit(this.btype) && !this.parentDialog.isInChaos())
    {
        APP.showLoader();
        Service.city.callMethod("createBuilding", [this.parentDialog.estate.id, this.parentDialog.cellIndex, this.btype, COM(this.builders).value], $.proxy(function()
        {
        	if (HintArrow.PRESS_CONSTRUCT.isShowing())
	        {
		        HintArrow.PRESS_CONSTRUCT.understood();
				this.noClearHints = true;
				this.parentDialog.noClearHints = true;
			}
        	
        	this.close();
        	this.parentDialog.close();
        	
        	APP.hideLoader();
        }, this));
    }
}

DoBuildDialog.prototype.onAddToQueue = function()
{
	Service.city.callMethod("addToBuildQueue", [this.parentDialog.estate.id, this.parentDialog.cellIndex, this.btype, 1]);
	this.close();
	this.parentDialog.close();
}

DoBuildDialog.prototype.updateArrows = function()
{
	var hint = FUser.CURRENT_USER.findConstructHint(this.parentDialog.estate);
    
    if (!hint || !this.parentDialog.canBuild(hint.btype) || this.btype.toString() != hint.btype.toString())
    {
        return;
    }

	HintArrow.PRESS_CONSTRUCT.show(this.doBuild, hint, this.DOM);
}