function UpgradeDialog()
{
	callParentConstructor(this);
	
	this.__defineGetter__('cell', function() {
		return this.estate.getCellByIndex(this._cellIndex); 
	});

	this._b = null; // FBuilding
	this._cellIndex = -1;
	this.scroll = null;
}

extendClass(UpgradeDialog.prototype, UpdatableEstateDialog.prototype);

UpgradeDialog.prototype._b = null; // FBuilding
UpgradeDialog.prototype._cellIndex = -1;
UpgradeDialog.prototype.scroll = null;

UpgradeDialog.prototype.__construct = function()
{
	COM.bindExclusiveClick(this.doUpgrade, $.proxy(this.onUpgrade, this));
	
	COM.bindExclusiveClick(this.toQueue, $.proxy(this.onAddToQueue, this));
	
	COM(this.builders).icon = 'builders';
	COM(this.workers).icon = 'workers';
	
   	COM(this.available).showZero = true;
	COM(this.available).doHighlightIfEquals = false;
	COM(this.available).alwaysHide = [Resource.PEOPLE];
	COM(this.available).shortView = true;
	
	this.fullScreen();
}

UpgradeDialog.prototype.onShow = function()
{
	this.scroll = new iScroll(this.content, {bounce: false, bounceLock: true});
	this.refreshScroll(300);
}

UpgradeDialog.prototype.refreshScroll = function(timeout)
{
	if (timeout*1)
	{
		return setTimeout($.proxy(this.refreshScroll, this), timeout*1);
	}
	if (this.scroll)
	{
		this.scroll.refresh();
	} 
}

UpgradeDialog.prototype.onResult = function(builders, workers)
{
    Service.city.callMethod('upgradeBuilding', [this._b.estate.id, this._b.id, builders, workers],
    	$.proxy(this.close, this),
    	$.proxy(function(){}, this)
    );
};

UpgradeDialog.prototype.onResultAddToQueue = function()
{
    Service.city.callMethod('addToBuildQueue', [this._b.estate.id, this._cellIndex, this._b.type, this._b.level + 1],
    	$.proxy(function(){
    	    Events.onForceCityUpdate.fire();
    	}, this),
    	$.proxy(function(){}, this)
    );
};

UpgradeDialog.prototype.popup = function(parent, estate, cellIndex)
{
    this.estate = estate;
    this._cellIndex = cellIndex;
    this._b = this.cell.building;
    
    superMethod(this, 'popup')(parent);
    
    this.createChildren();
    //show(parent);
}

UpgradeDialog.prototype.createNowScienceTip = function(e)
{
    //e.toolTip = ManageUpgradeTip.create(this.estate, this._cellIndex, this._b.level);
}

UpgradeDialog.prototype.createAfterScienceTip = function(e)
{
    //e.toolTip = ManageUpgradeTip.create(this.estate, this._cellIndex, this._b.level + 1);
}

UpgradeDialog.prototype.createNowExtractTip = function(r)
{
    //return ManageUpgradeTip.create(this.estate, this._cellIndex, this._b.level);
}

UpgradeDialog.prototype.createAfterExtractTip = function(r)
{
    //return ManageUpgradeTip.create(this.estate, this._cellIndex, this._b.level + 1);
}

UpgradeDialog.prototype.setTimeValue = function(nextLevel, builders)
{
    $(COM(this.time).time).removeClass('overflow funds');
    if (builders > 0) 
    {
        COM(this.time).value = this.estate.timeNeededToBuild(nextLevel, Math.min(nextLevel.builders, builders), this.estate.science);
        $(COM(this.time).time).addClass(this.estate.buildersBonusOverflow ? 'overflow' : 'funds');
    } 
    else 
    {
        COM(this.time).value = Tick.TICK_INFINITY;
        $(COM(this.time).time).addClass('funds');
    }
}

UpgradeDialog.prototype.isMagicActive = function(estate, btype)
{
    return estate.isMagicActive(btype);
}

UpgradeDialog.prototype.createChildren = function()
{
    //superMethod(this, 'createChildren')();
    
    var curLevel = this._b.getLevel(this._b.level);
    var nextLevel = this._b.getLevel(this._b.level + 1);
    
    this.setCaption(I18.getBuildingName(this._b.type, this._b.race));
    this.setSubCaption(
    	I18.f('LEVEL_X', this._b.level) 
    	+ '&nbsp;&rarr;&nbsp;' 
    	+ (this._b.isNextLevelPerfect() ? I18.f('PERFECT_BUILDING') : I18.f('LEVEL_X', this._b.level + 1))
    );
    this.building.src = STATIC_URL + Assets.B_findBy(this.estate.race, this._b.type, this._b.resource, this.estate.getSkinIndex());

    this.currentLabel.innerHTML = I18.f('LEVEL_X', this._b.level);
    this.nextLabel.innerHTML = this._b.isNextLevelPerfect() ? I18.f('PERFECT_BUILDING') : I18.f('LEVEL_X', this._b.level + 1);

	this.updateSliders();
    
    COM(this.builders).setMaximum(nextLevel.builders);
    COM(this.builders).value = nextLevel.builders;

    this.setTimeValue(nextLevel, this.estate.freeCitizens);
    
    COM(this.builders).onChange = $.proxy(function(count) 
    {
        this.setTimeValue(nextLevel, count);
        this.updateSliders();
        
    }, this);

    var uiFlag;
    
    uiFlag = this._b.type.isRequiresWorkers(this._b.race);
    UI.show(uiFlag, [this.workers]);
    if (uiFlag) with(COM(this.workers))
    {
        value = this._b.workers;
        caption = I18.getWorkers(this._b.type);
        setMaximum(curLevel.workers);
        value = curLevel.workers;
        onChange = $.proxy(this.updateSliders, this);
    }
    
    uiFlag = this._b.type.isExtractor() && this._b.isPerfect();
    UI.show(uiFlag, [this.extractFromNothingNow]);
    
    uiFlag = this._b.type.isExtractor() && this._b.isPerfect(this._b.level + 1);
    UI.show(uiFlag, [this.extractFromNothingAfter]);
    
    uiFlag = this._b.type.isRaiser() || this._b.type.isRecovery();
    UI.show(uiFlag, [this.extractNowPerHour, this.extractAfterPerHour]);
    if (uiFlag)
    {
        var now, after, r;
        if (this._b.type.isRaiser()) 
        {
            r = this._b.type.getParams(this._b.race).raisedResource;
            now = this._b.getBaseRaise(this._b.level);
            after = this._b.getBaseRaise(this._b.level + 1);
        } 
        else 
        {
            r = this.cell.resource;
            now = curLevel.recovers.getValue(r);
            after = nextLevel.recovers.getValue(r);
        }
        
        var hasProblems = true;
        if (this._b.type.isExtractor() && NumberUtils.equals(this.cell.remainder, 0)) 
        {
            now = this._b.type.getEmptyCellSpeed(this._b.race);
            if (!this._b.isNextLevelPerfect()) 
            {
                after = this._b.type.getEmptyCellSpeed(this._b.race);
            }
        } 
        else 
        {
            hasProblems = false;
        }
        
        if (!this._b.type.isEstateHall()) 
        {
            var bonus = this.estate.getProductivityBonus(this._cellIndex).getValue(r);
            var magic = this.isMagicActive(this.estate, this._b.type) ? ServiceHelper.getGloabalServiceDecl(this._b.type.name, 0).getValue() : 0;
            bonus += this.estate.sumOfBuildingBuffs(BuffType.BOOST_BUILDING, this._b.type);
            // spec_miner bonus
            var minerBonus = ScienceBonusUtil.getMinerBonus(this.estate.science.getLevel(Science.SPEC_MINER), this._b.type);

            now = now * (1.0 + bonus + minerBonus) * (1 + magic);
            after = after * (1.0 + bonus + minerBonus) * (1 + magic);
        }
        
        COM(this.extractNowPerHour).alwaysShow = [r];
        COM(this.extractNowPerHour).update((new Funds()).setValue(r, now), this.estate.race);
        COM(this.extractNowPerHour).styleFunds([r], hasProblems);
        
        COM(this.extractAfterPerHour).alwaysShow = [r];
        COM(this.extractAfterPerHour).update(new Funds().setValue(r, after), this.estate.race);
        COM(this.extractAfterPerHour).styleFunds([r], hasProblems);
    }
    
    uiFlag = this._b.type.isCapacitor() || this._b.type.isMarket();
    UI.show(uiFlag, [this.capacityNow, this.capacityAfter]);
    if (uiFlag)
    {
        COM(this.capacityNow).resolveAndUpdate(this._b, this._b.level);
        COM(this.capacityAfter).resolveAndUpdate(this._b, this._b.level + 1);
    }
    
    uiFlag = this._b.type.isHouses() || this._b.type.isEstateHall();
    UI.show(uiFlag, [this.populationIncreaseBeforeBox, this.populationIncreaseAfterBox]);
    if (uiFlag)
    {
        this.freePeopleImage.src = STATIC_URL + Assets.getFreePeopleIcon(this.estate.owner.race);
        if (this._b.type.isHouses()) 
        {
            this.populationIncreaseBefore.innerHTML = I18.f('PER_TIME', this._b.getPeopleProductionSpeedPerHourFor(this._b.level));
            //Style.assignTip(populationIncreaseBefore, " ");
            this.populationIncreaseAfter.innerHTML = I18.f('PER_TIME', this._b.getPeopleProductionSpeedPerHourFor(this._b.level + 1));
            //Style.assignTip(populationIncreaseAfter, " ");
        } 
        else 
        {
            this.populationIncreaseBefore.innerHTML = I18.f('PER_TIME', this._b.getPeopleProductionSpeedPerHourFor(this._b.level));
            //Style.removeTip(populationIncreaseBefore);
            this.populationIncreaseAfter.innerHTML = I18.f('PER_TIME', this._b.getPeopleProductionSpeedPerHourFor(this._b.level + 1));
            //Style.removeTip(populationIncreaseAfter);
        }
    }
    
    uiFlag = (this._b.type.isHouses() || this._b.type.isWarehouse()) && this._b.getLevel().drainReduction != 0;
    UI.show(uiFlag, [this.drainNow]);
    if (uiFlag)
    {
        this.drainNow.innerHTML = (this._b.type.isHouses() 
        	? I18.f('PEOPLE_DRAIN') 
        	: I18.f('RESOURCES_DRAIN')) + " " + Strings.formatNumberPercent(this._b.getLevel().drainReduction * 100, true, 0)
		;
    }
    
    uiFlag = (this._b.type.isHouses() || this._b.type.isWarehouse()) && this._b.getLevel(this._b.level + 1).drainReduction != 0;
    UI.show(uiFlag, [this.drainAfter]);
    if (uiFlag)
    {
        this.drainAfter.innerHTML = (this._b.type.isHouses() 
        	? I18.f('PEOPLE_DRAIN') 
        	: I18.f('RESOURCES_DRAIN')) + " " + Strings.formatNumberPercent(this._b.getLevel(this._b.level + 1).drainReduction * 100, true, 0)
		;
    }
	
	uiFlag = this._b.type.isMilitaryUnitProducer();
	UI.show(uiFlag, [this.queueSizeBefore, this.queueSizeAfter]);
	if (uiFlag)
	{
	    this.queueSizeBefore.innerHTML = I18.f('QUEUE_SIZE_TEXT', this._b.getLevel().queueSize);
	    this.queueSizeAfter.innerHTML = I18.f('QUEUE_SIZE_TEXT', this._b.getNextLevel().queueSize);
	}
	
	uiFlag = this._b.type.isMilitaryUnitProducer() || this._b.type.isSpecialUnitProducer();
	UI.show(uiFlag, [this.currentUnit, this.nextUnit, this.unitTimeBeforeBox, this.unitTimeAfterBox]);
	if (uiFlag)
	{
        with (COM(this.currentUnit))
        {
        	race = this._b.race;
	        showCount = !this._b.type.isMilitaryUnitProducer();
	        utype = this._b.type.getUnitType(this._b.race);
	        aura = this.estate;
	        buffs = this.estate.buffs;
	        time = FEstate.tickNowForEstate(this.estate);
		}
        
        with (COM(this.nextUnit))
        {
        	race = this._b.race;
	        showCount = !this._b.type.isMilitaryUnitProducer();
	        utype = this._b.type.getUnitType(this._b.race);
	        aura = this.estate;
	        buffs = this.estate.buffs;
	        time = FEstate.tickNowForEstate(this.estate);
		}

        if (this._b.hasProduceTimeBonuses()) 
        {
            //Style.assignTip(unitTimeBefore, " ");
            //Style.assignTip(unitTimeAfter, " ");
        } 
        else 
        {
            //Style.removeTip(unitTimeBefore);
            //Style.removeTip(unitTimeAfter);
        }
        COM(this.unitTimeBefore).value = this._b.getProduceTimeForLevel(this._b.level);
        COM(this.unitTimeAfter).value = this._b.getProduceTimeForLevel(this._b.level + 1);

        if (this._b.type.isSpecialUnitProducer()) 
        {
            COM(this.currentUnit).update(1, this._b.unitsPresent, curLevel.capacity);
            COM(this.nextUnit).update(1, this._b.unitsPresent, nextLevel.capacity);
        } 
        else 
        {
            var unitLevel = this.estate.science.getLevel(this._b.type.getUnitType(this._b.race).science);
            COM(this.currentUnit).update(unitLevel, 1);
            COM(this.nextUnit).update(unitLevel, 1);
        }
    }
    
    uiFlag = this._b.type.isTerraformer() || this._b.type.isResurrector();
    UI.show(uiFlag, [this.textInfoNowBox, this.textInfoAfterBox]);
    if (uiFlag)
    {
        if (this._b.type.isTerraformer()) 
        {
            this.textInfoNow.innerHTML = (I18.f('TERRAFORMING_DURATION', Tick.toTimeLeft(curLevel.workTime)));
            this.textInfoAfter.innerHTML = (I18.f('TERRAFORMING_DURATION', Tick.toTimeLeft(nextLevel.workTime)));
            this.textInfoNowValue.innerHTML = ("");
            this.textInfoAfterValue.innerHTML = ("");
        } 
        else 
        {
            this.textInfoNow.innerHTML = (I18.f('RESURRECTOR_PERCENT'));
            this.textInfoAfter.innerHTML = (I18.f('RESURRECTOR_PERCENT'));
            this.textInfoNowValue.innerHTML = (Strings.formatPercent(curLevel.getResurrectionPercent(this.estate)));
            this.textInfoAfterValue.innerHTML = (Strings.formatPercent(nextLevel.getResurrectionPercent(this.estate)));

            if (this.estate.science.getLevel(Science.SPEC_CLERIC) > 0 || this.estate.sumOfBuildingBuffs(BuffType.BOOST_BUILDING, this._b, BType.RESURRECTOR) != 0) 
            {
                //Style.assignTip(textInfoNowValue, " ");
                //Style.assignTip(textInfoAfterValue, " ");
            } 
            else 
            {
                //Style.removeTip(textInfoNowValue);
                //Style.removeTip(textInfoAfterValue);
            }
        }
    }
    
    uiFlag = this._b.type.isResurrector() && curLevel.costChange != 0;
    UI.show(uiFlag, [this.textInfoNowSpecialBox]);
    if (uiFlag)
    {
        this.textInfoNowSpecial.innerHTML = (I18.f('RESURRECTOR_COST_CHANGE'));
        this.textInfoNowSpecialValue.innerHTML = (Strings.formatPercent(curLevel.costChange * 100, true));
    }
    
    uiFlag = this._b.type.isResurrector() && nextLevel.costChange != 0;
    UI.show(uiFlag, [this.textInfoAfterSpecialBox]);
    if (uiFlag)
    {
        this.textInfoAfterSpecial.innerHTML = (I18.f('RESURRECTOR_COST_CHANGE'));
        this.textInfoAfterSpecialValue.innerHTML = (Strings.formatPercent(nextLevel.costChange * 100, true));
    }
    
    uiFlag = this._b.type.isShop();
    UI.show(uiFlag, [this.shopBoxBefore, this.shopBoxAfter]);
    if (uiFlag)
    {
        var bonus = Science.getShopRatioBonus(this.estate.science.getLevel(Science.SPEC_TRADER));
        var magic = this.isMagicActive(this.estate, this._b.type) ? ServiceHelper.getGloabalServiceDecl(this._b.type.name, 0).getValue() : 0;
        this.shopRatioBefore.innerHTML = (Strings.formatNumber((curLevel.peopleRatio + bonus) * (1 + magic), 3));
        this.shopRatioAfter.innerHTML = (Strings.formatNumber((nextLevel.peopleRatio + bonus) * (1 + magic), 3));
    }
    
    uiFlag = this._b.type.isAlchemist();
    UI.show(uiFlag, [this.scienceBefore, this.scienceAfter]);
    if (uiFlag)
    {
        COM(this.scienceBefore).value = this._b.getScienceProductionPerHour();
        COM(this.scienceAfter).value = this._b.getScienceProductionPerHour(-1, this._b.level + 1);
    }
    
    uiFlag = this._b.type.isTemple();
    UI.show(uiFlag, [this.templeBeforeBox, this.templeAfterBox]);
    if (uiFlag)
    {
        COM(this.godpointsSpeedBefore).value = curLevel.godpointProduction;
        COM(this.godpointsSpeedAfter).value = nextLevel.godpointProduction;
        COM(this.godpointsAvailableBefore).update(this._b.godpointsAvailable, curLevel.godpointLimit);
        COM(this.godpointsAvailableAfter).update(this._b.godpointsAvailable, nextLevel.godpointLimit);
    }
    
    uiFlag = this._b.type.isTower();
    UI.show(uiFlag, [this.towerAttackAfter, this.towerAttackBefore]);
    if (uiFlag)
    {
        COM(this.towerAttackBefore).update(this._b.type, this._b.level, this._b.estate);
        COM(this.towerAttackAfter).update(this._b.type, this._b.level + 1, this._b.estate);
    }
    
    uiFlag = this._b.type.isFortification();
    UI.show(uiFlag, [this.fortificationBoxBefore, this.fortificationBoxAfter]);
    if (uiFlag)
    {
        COM(this.fortificationBoxBefore).update(this._b.estate, this._b.level, curLevel);
        COM(this.fortificationBoxAfter).update(this._b.estate, this._b.level + 1, nextLevel);
    }
    
    uiFlag = this._b.type.isEstateHall();
    UI.show(uiFlag, [this.cellsInfoBefore, this.cellsInfoAfter]);
    if (uiFlag)
    {
        this.cellsInfoBefore.innerHTML = (I18.f('CELLS_AVAILABLE', this._b.estate.getInsideWallCellCount()));
        this.cellsInfoAfter.innerHTML = (I18.f('CELLS_AVAILABLE', this._b.estate.getNextLevelInsideWallCellCount()));
        UI.show(this._b.getLevel().passive, [this.passivesBefore]);
        if (this._b.getLevel().passive)
        {
        	//EffectsView.createDescriptiveView(this.passivesBefore, this._b.getLevel().passive.dtoActions, false, false, false, false);
        }
        UI.show(this._b.getLevel(this._b.level + 1).passive, [this.passivesAfter]);
        if (this._b.getLevel(this._b.level + 1).passive)
        {
        	//EffectsView.createDescriptiveView(this.passivesAfter, this._b.getLevel(this._b.level + 1).passive.dtoActions, false, false, false, false);
        }
    }
    
    uiFlag = this._b.type.isClanCapacitor();
    UI.show(uiFlag, [this.clanCapacitorBefore, this.clanCapacitorAfter]);
    if (uiFlag)
    {
        if (this._b.type.isClanReservation()) 
        {
            this.clanCapacitorBefore.innerHTML = (I18.f('C_B_TIP_CLAN_RESERVATION_KEEPS_CLAN_PEOPLE', this._b.getBasePeopleCapacityFor(this._b.level)));
            this.clanCapacitorAfter.innerHTML = (I18.f('C_B_TIP_CLAN_RESERVATION_KEEPS_CLAN_PEOPLE', this._b.getBasePeopleCapacityFor(this._b.level + 1)));
        } 
        else 
        {
            this.clanCapacitorBefore.innerHTML = (I18.f('C_B_TIP_CLAN_WAREHOUSE_KEEPS_CLAN_RESOURCES', this._b.getBaseWarehouseCapacity(this._b.level)));
            this.clanCapacitorAfter.innerHTML = (I18.f('C_B_TIP_CLAN_WAREHOUSE_KEEPS_CLAN_RESOURCES', this._b.getBaseWarehouseCapacity(this._b.level + 1)));
        }
    }
    
    
    uiFlag = (this._b.type.name == BType.ARTIFACT_STORE.name);
    UI.show(uiFlag, [this.artifactCapacityBefore, this.artifactCapacityAfter]);
    if (uiFlag)
    {
        this.artifactCapacityBefore.innerHTML = (I18.f('ARTIFACT_CAPACITY', this._b.getBaseWarehouseCapacity(this._b.level)));
        this.artifactCapacityAfter.innerHTML = (I18.f('ARTIFACT_CAPACITY', this._b.getBaseWarehouseCapacity(this._b.level + 1)));
    }
    
    uiFlag = (this._b.type.name == BType.PRISON.name);
    UI.show(uiFlag, [this.prisonerCaptureRateBefore, this.prisonerCaptureRateAfter, this.prisonerCapacityBefore, this.prisonerCapacityAfter, this.prisonerCostBefore, this.prisonerCostAfter]);
    if (uiFlag)
    {
        this.prisonerCaptureRateBefore.innerHTML = (I18.f('CAPTURE_RATE', this._b.getLevel().capture));
        this.prisonerCaptureRateAfter.innerHTML = (I18.f('CAPTURE_RATE', this._b.getLevel(this._b.level + 1).capture));
        this.prisonerCapacityBefore.innerHTML = (I18.f('PRISONER_CAPACITY', this._b.getBaseWarehouseCapacity(this._b.level)));
        this.prisonerCapacityAfter.innerHTML = (I18.f('PRISONER_CAPACITY', this._b.getBaseWarehouseCapacity(this._b.level + 1)));
        
        UI.show((curLevel.costChange != 0), [this.prisonerCostBefore]); 
        if (curLevel.costChange != 0)
        {
            this.prisonerCostBefore.innerHTML = (I18.mega(I18.PRISONERS_COST, curLevel.costChange));
        }
        UI.show((nextLevel.costChange != 0), [this.prisonerCostAfter]); 
        if (nextLevel.costChange != 0)
        {
            this.prisonerCostAfter.innerHTML = (I18.mega(I18.PRISONERS_COST, nextLevel.costChange));
        }
    }
    
    uiFlag = (this._b.type.name == BType.ARENA.name);
    UI.show(uiFlag, [this.arenaExpProductionBefore, this.arenaExpProductionAfter]);
    if (uiFlag)
    {
        COM(this.arenaExpProductionBefore).update(this._b);
        COM(this.arenaExpProductionAfter).update(this._b, this._b.level + 1);
    }
    
    uiFlag = (this._b.type.name == BType.CASTLEHALL.name);
    UI.show(uiFlag, [this.peacePointsBefore, this.warPointsBefore, this.peacePointsAfter, this.warPointsAfter]);
    if (uiFlag)
    {
        var productionBefore = I18.f('PER_TIME', this._b.getLevel().pointsProduction);
        COM(this.peacePointsProductionBefore).value = productionBefore;
        COM(this.warPointsProductionBefore).value = productionBefore;
        var productionAfter = I18.f('PER_TIME', this._b.getLevel(this._b.level + 1).pointsProduction);
        COM(this.peacePointsProductionAfter).value = productionAfter;
        COM(this.warPointsProductionAfter).value = productionAfter;
    }
    
    uiFlag = (this._b.type.name == BType.CASTLEGATE.name);
    UI.show(uiFlag, [this.castleGateInfoAfter, this.castleGateInfoBefore]);
    if (uiFlag)
    {
        var lvl = this._b.getLevel();
        var nextLvl = this._b.getLevel(this._b.level + 1);
        var help = this.estate.calcGateGarrisonHelp();
        COM(this.castleGateInfoBefore).update(lvl.health, lvl.health, help, lvl.shield, this.estate.buffs, FEstate.tickNowForEstate(this.estate));
        COM(this.castleGateInfoAfter).update(nextLvl.health, nextLvl.health, help, nextLvl.shield, this.estate.buffs, FEstate.tickNowForEstate(this.estate));
    }
    
    COM(this.afterEffectView).update(this._b, this._b.level + 1);
    COM(this.cost).update(nextLevel.cost, this.estate.race, nextLevel.pearl);
    COM(this.cost).setCapacity(this.estate.bank, this.estate.owner.pearl);
    
    this.update(this.estate);
}

UpgradeDialog.prototype.update = function(estate)
{
    this.estate = estate;
    this._b = this.cell.building;
    if (!this._b) {
        Alert.error(I18.f('BUILDING_ERROR_NO_BUILDING'));
        this.close();
    } 
    else 
    {
        this.tick();
    }
    
    this.artBox.style.display = this._b.isNextLevelPerfect() ? "" : "none";
    $(this.artBox).find('*').remove();
    var artCost = this._b.getNextLevel().artCost;
    if (artCost) 
    {
        var currentCounts = CrystalType.getCrystals(this.estate);
        for (var i = 0; i < CrystalType.valuesCache.length; i++) 
        {
        	var ct = CrystalType.valuesCache[i];
            if (ct.getOrdinal() < artCost.length && artCost[ct.getOrdinal()] != 0) 
            {
                var c = currentCounts[ct.getOrdinal()];
                var n = artCost[ct.getOrdinal()];
                
                var art = document.createElement('art');
                $(art)
                	.append('<img src="' + STATIC_INTERFACE_URL + ArtifactAssets[ct.name] + '" />')
                	.append('<art-info><count class="'+(c < n ? 'negative' : 'positive')+'">' + c + '</count> / <required>' + n + '</required><art-info>')
                ;
                $(this.artBox).append(art);
            }
        }
    }
    
    HintArrow.CLICK_UPGRADE_MENU.understood();
    
    this.updateArrows();
    
	COM(this.available).update(this.estate.bank, this.estate.race);
	    
    this.refreshScroll(300);
}

UpgradeDialog.prototype.updateArrows = function()
{
    var hint = FUser.CURRENT_USER.findHint(HintType.UPGRADE_HALL);
    if (hint && this._b.type.name == BType.CITYHALL.name) 
    {
        HintArrow.CLICK_UPGRADE_BUTTON.show(this.doUpgrade, hint, this.DOM);
    }
}

UpgradeDialog.prototype.tick = function()
{
    this.updateSliders();
    var nextLevel = this._b.getLevel(this._b.level + 1);
    COM(this.cost).setCapacity(this.estate.bank, this.estate.owner.pearl);

    var artCost = this._b.getNextLevel().artCost;
    var currentCounts = CrystalType.getCrystals(this.estate);
    var allArts = true;
    if (artCost) 
    {
        for (var i = 0; i < artCost.length; i++) 
        {
            if (currentCounts[i] < artCost[i]) 
            {
                allArts = false;
            }
        }
    }
    
    var alchemyNeeded = this._b.isNextLevelPerfect() && (FUser.CURRENT_USER.science.getLevel(Science.ALCHEMY) == 0);

	if (UI.show(alchemyNeeded, [this.alchemyWarning]))
	{
		UI.show(false, [this.workersBox]);
	}
    
    COM(this.toQueue).reason = null;
	if (UI.show(alchemyNeeded, [this.alchemyWarning]))
	{
    	COM(this.toQueue).reason = I18.f('ALCHEMY_REQUIRED');
    }
    else if (!this.estate.accessManageble && nextLevel.pearl > 0) 
    {
        COM(this.toQueue).reason = I18.f('DEPUTY_CAN_NOT_BUILD_FOR_PEARL');
    } 
    else if (!this.estate.owner.isBuildQueueEnabled()) 
    {
        COM(this.toQueue).reason = I18.f('PREMIUM_ACCOUNT_OWNER_ONLY');
    } 
    else if (!this.estate.canAddToBuildQueue()) 
    {
        COM(this.toQueue).reason = I18.f('BUILD_QUEUE_FULL');
    } 

    COM(this.doUpgrade).reason = null;
	if (alchemyNeeded)
	{
		COM(this.doUpgrade).reason = I18.f('ALCHEMY_REQUIRED');
	} 
    else if (!allArts || !this.estate.bank.greaterOrEqualsTo(nextLevel.cost) || this.estate.owner.pearl < nextLevel.pearl)
    {
    	COM(this.doUpgrade).reason = I18.f('NOT_ENOUGH_RESOURCES');
    }
    else if (!this.estate.canPerformOneMoreTask())
    {
    	COM(this.doUpgrade).reason = I18.getConcurrentLimitText(this.estate.getMaxConcurrentBuildings());
    }
    else if(this.cell.onSector && !this.estate.gateState.isOpen())
    {
    	COM(this.doUpgrade).reason = I18.f('BUILDING_CONSTRAINT_GATE_CLOSED');
    }
}

UpgradeDialog.prototype.onBuildingTimeTip = function(e)
{
    var time = this.estate.timeNeededToBuild(this._b.getNextLevel(), COM(this.builders).value);
    //e.toolTip = SpecializationsTooltipHelper.onBuildingTimeTip(time, this._b.level + 1, this.estate);
}

UpgradeDialog.prototype.updateSliders = function()
{
    if (this._b.type.isRequiresWorkers(this._b.race)) 
    {
        var workersLimit = this._b.workers + this.estate.freeCitizens - (COM(this.builders).value - this._b.builders);
        COM(this.workers).update(this.estate, this._b.workers, workersLimit);
    }
    
    var buildersLimit = this._b.builders + this.estate.freeCitizens - (COM(this.workers).value - this._b.workers);
    COM(this.builders).update(this.estate, buildersLimit, buildersLimit);
    this.setTimeValue(this._b.getNextLevel(), COM(this.builders).value);
}

UpgradeDialog.prototype.onUpgrade = function()
{
    this.onResult(COM(this.builders).value, COM(this.workers).value);
    this.onResult = Dialog.EMPTY_FUNCTION;
    this.onResultAddToQueue = Dialog.EMPTY_FUNCTION;
    
    HintArrow.CLICK_UPGRADE_BUTTON.understood();
    this.noClearHints = true;
    
    this.close();
}

UpgradeDialog.prototype.onAddToQueue = function()
{
    this.onResultAddToQueue();
    this.onResult = Dialog.EMPTY_FUNCTION;
    this.onResultAddToQueue = Dialog.EMPTY_FUNCTION;
    
    this.close();
}

UpgradeDialog.prototype.onShopRatioBeforeTip = function(e)
{
    //e.toolTip = ShopRatioTip.createForLevel(this.estate, this._b.level);
}

UpgradeDialog.prototype.onShopRatioAfterTip = function(e)
{
    //e.toolTip = ShopRatioTip.createForLevel(this.estate, this._b.level + 1);
}

UpgradeDialog.prototype.onIncreaseBeforeTip = function(e)
{
    //e.toolTip = CapacityTipFactory.createAsRaiser(this._b, this._b.level);
}

UpgradeDialog.prototype.onIncreaseAfterTip = function(e)
{
    //e.toolTip = CapacityTipFactory.createAsRaiser(this._b, this._b.level + 1);
}

UpgradeDialog.prototype.onUnitTimeBeforeTooltip = function(e)
{
    //e.toolTip = UnitProductionSpeedTip.createForLevel(this.estate, this._b.type, this._b.level);
}

UpgradeDialog.prototype.onUnitTimeAfterTooltip = function(e)
{
    //e.toolTip = UnitProductionSpeedTip.createForLevel(this.estate, this._b.type, this._b.level + 1);
}

UpgradeDialog.prototype.onResurrectBeforeTooltip = function(e)
{
    //e.toolTip = ManageResurrectorTip.createForLevel(this.estate, this._b.level);
}

UpgradeDialog.prototype.onResurrectAfterTooltip = function(e)
{
    //e.toolTip = ManageResurrectorTip.createForLevel(this.estate, this._b.level + 1);
}

UpgradeDialog.prototype.onBoxResize = function()
{
}
