function BuildingView()
{
	this.building = null;
	this.order = null;
    this.pending = false;
    this.showLevel = false;
    this.showCountdown = true;
    this.highlightBuilding = true;
    
    this.statusIcon = null;
    this.busyCountdown = null;
    this.level = null;
    
    this.busyCountdownText = "";
    
    this.x = 0;
    this.y = 0;
    this.buildingWidth = 0;
    this.buildingHeight = 0;
    this.buildingCenterX = 0; 
	this.buildingCenterY = 0;
	
	this.lastImage = null;
	
	this.ctx = null;
	
	this.__construct = function(params)
	{
		if (params)
		{
			$.extend(this, params);
		}
		
		if (!this.ctx && this.mainCanvas) 
		{
			this.ctx = {
				main: this.mainCanvas.getContext('2d'), 
				status: this.statusCanvas ? this.statusCanvas.getContext('2d') : this.mainCanvas.getContext('2d')
			}
		}
		
		/*
		if (this.attr('showLevel') !== null)
		{
			this.showLevel = (this.attr('showLevel') != 'false' && this.attr('showLevel') != '0' && this.attr('showLevel') != 'no');
		}
		*/
	}
	
	this.countDownView = null;
}

BuildingView.prototype.building = null;
BuildingView.prototype.pending = false;
BuildingView.prototype.showLevel = true;
BuildingView.prototype.showCountdown = false;
BuildingView.prototype.highlightBuilding = true;

BuildingView.prototype.statusIcon = null;
BuildingView.prototype.busyCountdown = null;
BuildingView.prototype.level = null;

BuildingView.prototype.busyCountdownText = "";

BuildingView.prototype.x = 0;
BuildingView.prototype.y = 0;
BuildingView.prototype.buildingWidth = 0;
BuildingView.prototype.buildingHeight = 0;
BuildingView.prototype.buildingCenterX = 0; 
BuildingView.prototype.buildingCenterY = 0;

BuildingView.prototype.buildingBottomY = null;

BuildingView.prototype.lastImage = null;
BuildingView.prototype.lastIconForStatus = null;

BuildingView.prototype.debug = function(caller)
{
	console.log('BUILDING: %sx%s', this.ctx.main.canvas.width, this.ctx.main.canvas.height);
}

BuildingView.prototype.isChanged = function(b, o, race)
{
	var changed = this.getImage(b, o, race) != this.lastImage;
	var info = this.getExtraInfo();
	changed |= (this.lastIconForStatus != info.iconForStatus);
	
	return changed;
}

BuildingView.prototype.clearImage = function()
{
	for (var i in this.ctx)
	{
		this.drawImage(null, this.ctx[i]);
	}
}

BuildingView.prototype.drawImage = function(img, ctx)
{
	if (!ctx) ctx = this.ctx.main;
	
	var tmp = img;
	
	if (typeof img == 'string')
	{
		img = APP.getPreloadImage(img);
	}
	
	var left = (this.buildingCenterX - this.buildingWidth / 2) * GAME_IMAGES_PACK;
	var top = (this.buildingBottomY === null ? this.buildingCenterY - this.buildingHeight / 2 : this.buildingBottomY - this.buildingHeight) * GAME_IMAGES_PACK;
	var w = this.buildingWidth * GAME_IMAGES_PACK;
	var h = this.buildingHeight * GAME_IMAGES_PACK;
	
	
	if (img)
	{
		if (this.building && this.building.blinking)
		{
			//img = BuildingView.generateTintImage(img, 255, 255, 0);
		}
		try
		{
			ctx.drawImage(img, 0, 0, w, h, left, top, w, h);
		}
		catch (e) {}
		
	}
	else
	{
		ctx.clearRect(0, 0, w, h);
	}
}

BuildingView.generateTintImage = function(img, red, green, blue) 
{
	return img;
	
	var w = img.width;
	var h = img.height;
	var rgbks = [];
	
	var canvas = document.createElement("canvas");
	canvas.width = w;
	canvas.height = h;

	var ctx = canvas.getContext('2d');
	ctx.drawImage(img, 0, 0);
	var data = ctx.getImageData(0, 0, w, h);
	for (var i = 0, length = data.data.length; i < length; i += 4) 
	{
		if (data.data[i+3] == 255)
		{
			data.data[i] = Math.max(128, data.data[i]);
			data.data[i+1] = Math.max(128, data.data[i]);
		}
	}
	ctx.putImageData(data, 0, 0);
	
	var imgComp = new Image();
	imgComp.src = canvas.toDataURL();
	
	return imgComp;
}
    
BuildingView.prototype.fixDimensions = function(src)
{
    if (src)
    {
    	// fix image size if changed or not initialized
	    if (this.lastImage != src || this.buildingWidth == 0 || this.buildingHeight == 0)
	    {
   			var img = APP.getPreloadImage(src);
   			
   			if(!img)
   			{
   				return;
   			}
   			
			this.buildingWidth = img.width / GAME_IMAGES_PACK;
			this.buildingHeight = img.height / GAME_IMAGES_PACK;
		    // fix center
		    if (!this.buildingCenterX)
		    {
		    	this.buildingCenterX = this.buildingWidth / 2;
		    }
		    if (!this.buildingCenterY)
		    {
		    	this.buildingCenterY = (this.buildingBottomY === null) 
		    		? this.buildingHeight / 2 
		    		: this.buildingBottomY - this.buildingHeight / 2
		    	;
		    }
		    
		   	// fix canvas size if too small
			var resize = false;
			for (var i in this.ctx)
			{
				if (this.ctx[i].canvas.width < this.buildingWidth || this.ctx[i].canvas.height < this.buildingHeight)
				{
					resize = true;
					break;
				}
			}
			if (resize)
			{
				var offset = 0;
				for (var i in this.ctx)
				{
					this.ctx[i].canvas.width = this.buildingWidth * GAME_IMAGES_PACK;
					this.ctx[i].canvas.height = this.buildingHeight * GAME_IMAGES_PACK;
					if (offset)
					{
						$(this.ctx[i].canvas).css('margin-left', -offset);
					}
					offset = this.buildingWidth * GAME_IMAGES_PACK;
				}
			}
	    }
    }
    else
    {
		this.buildingWidth = 0;
		this.buildingHeight = 0;
    }
}

BuildingView.prototype.getImage = function(b, o, race)
{
	var src = null;
	if (b)
    {
    	var skinIndex = b.estate.getSkinIndex();
    	src = (b.accessType ? Assets.B_findBy(b.race, b.type, b.resource, skinIndex) : Assets.B_locked(b.race));
    }
    else if (o)
    {
    	src = Assets.B_findBy(race, o.type);
    }
    
    this.fixDimensions(src);
    return src;
}

BuildingView.prototype.update = function(b, o, race)
{
    this.building = b;
    this.pending = Boolean(o);
    this.order = o;
    
    var src = this.getImage(b, o, race);
    this.lastImage = src;
    this.drawImage(src, this.ctx.main);
    

}

BuildingView.prototype.getLevelVisibility = function()
{
	return this.showLevel && this.building && this.busyCountdownText.length == 0 && !this.building.isPerfect();
}

BuildingView.prototype.updateLevelVisibility = function()
{
    /*
    if(this.showLevel && this.building && this.busyCountdown.html().length == 0 && !this.building.isPerfect())
    {
    	if(this.level.css("display") == "none") this.level.show();
    }
    else
    {
    	if(this.level.css("display") != "none") this.level.hide();
    }
    */
}

BuildingView.prototype.getExtraInfo = function(builders)
{
    var iconForStatus = null;
    var countDownText = "";
    var busyCountdownColor = "#fff"; 
    
    if(!this.building && !this.pending)
    {
        return {};
    }
    
    if (this.pending)
    {
        countDownText = Tick.INFINITY_SYMBOL;
        iconForStatus = !this.building ? CommonAssets.INPROGRESS_BUILD : this.building.isNextLevelPerfect() ? CommonAssets.CRYSTAL_SIGN : CommonAssets.INPROGRESS_UPGRADE;
    }
    else if (!this.building.accessType)
    {
        //do nothing
    }
    else if (this.building.isBroken)
    {
        iconForStatus = CommonAssets.BROKEN;
    }
    else if (this.building.type.isClanCapacitor() && !this.building.estate.owner.isInRegisteredClan())
    {
        iconForStatus = CommonAssets.BROKEN;
    }
    else if (this.building.isBuildingOrUpgrading)
    {
        if (builders == 0 || (builders == -1 && this.building.builders == 0))
        {
            countDownText = Tick.INFINITY_SYMBOL;
            iconForStatus = CommonAssets.INPROGRESS_FROZEN;
        }
        else
        {
            countDownText = Tick.timeTill(Tick.now() + (this.building.isForeign
                    ? this.building.estimateCurrentWork()
                    : (this.building.estimateBuildDuration(builders == -1 ? this.building.builders : builders))));
            iconForStatus = this.building.isBuildPending ? CommonAssets.INPROGRESS_BUILD : this.building.isNextLevelPerfect() ? CommonAssets.CRYSTAL_SIGN : CommonAssets.INPROGRESS_UPGRADE;
        }
    }
    else if (this.building.isRepairing)
    {
        if (builders == 0 || (builders == -1 && this.building.builders == 0))
        {
            countDownText = Tick.INFINITY_SYMBOL;
            iconForStatus = CommonAssets.INPROGRESS_FROZEN;
        }
        else
        {
            countDownText = Tick.timeTill(Tick.now() + (this.building.isForeign
                    ? this.building.estimateCurrentRepair()
                    : (this.building.estimateRepairDuration(builders == -1 ? this.building.builders : builders))));
            iconForStatus = CommonAssets.INPROGRESS_REPAIR;
        }
    }
    else if (this.building.isStopped)
    {
        iconForStatus = CommonAssets.INPROGRESS_FROZEN;
    }
    else if (this.building.resource != Resource.PEOPLE && this.building.isWarehouseOverfilled() && this.building.hasSpeedProblems())
    {
        iconForStatus = CommonAssets.INPROGRESS_FROZEN;
    }
    else if (this.building.isOnSector && this.building.estate.isGateClosed())
    {
        iconForStatus = CommonAssets.INPROGRESS_FROZEN;
    }
    else if (this.building.isWarehouseDoesntWork())
    {
        iconForStatus = CommonAssets.INPROGRESS_FROZEN;
    }
    else if (this.building.type.isExtractor() && this.building.cell.remainder == 0 && !this.building.isPerfect())
    {
        iconForStatus = CommonAssets.INPROGRESS_FROZEN;
    }
    if (this.building.blinking)
    {
    	//iconForStatus = CommonAssets.INPROGRESS_FROZEN;
    	iconForStatus = CommonAssets.STATUS_ON;
    }

    if (this.building && (this.building.type.isOuterEstateHall() || BType.STONEHENGEHALL.name == this.building.type.name) && this.building.estate.shouldShowDestructionWarning())
    {
        countDownText = Tick.toTimeLeft(this.building.estate.destructionTick - this.building.estate.responseTick);
        busyCountdownColor = Effects.RED;
    }
    if (this.building && (this.building.type == BType.CAMPHALL))
    {
        if (this.building.estate.isSettingUp)
        {
            countDownText = Tick.toTimeLeft(this.building.estate.setUpTicksLeft);
            busyCountdownColor = Effects.GREEN;
        }
    }
    if (this.building && (this.building.type.name == BType.CATAPULTHALL.name))
    {
        if (this.building.estate.isSettingUp)
        {
            countDownText = Tick.toTimeLeft(this.building.estate.rechargingLeft);
            busyCountdownColor = Effects.GREEN;
        }
		if (this.building.estate.isRecharging) 
		{
			countDownText = Tick.toTimeLeft(this.building.estate.rechargingLeft);
			busyCountdownColor = Effects.GREEN;
		}
    }
    
    return {
    	"iconForStatus": iconForStatus,
    	"countDownText": countDownText,
    	"busyCountdownColor": busyCountdownColor
	};
}

BuildingView.prototype.drawIcon = function(img, x, y)
{
	if(this.ctx.main) try
	{
		//this.ctx.main.clearRect(x, y, img.width, img.height);
		this.ctx.main.drawImage(img, 0, 0, img.width, img.height,	x, y, img.width, img.height);
	} catch (e) {}
}

BuildingView.prototype.tick = function(builders, drawIcons)
{
    if((typeof builders == "undefined") || builders == null) 
    {
    	builders = -1;
    }
    
    var info = this.getExtraInfo(builders);
    
    var iconForStatus = info.iconForStatus;
    this.lastIconForStatus = iconForStatus;
    
    var countDownText = info.countDownText;
    var busyCountdownColor = info.busyCountdownColor;
    
    this.busyCountdownText = countDownText;
    
    var ctx = this.ctx.main;
    
    if (ctx && drawIcons)
    {
        var bLevel = this.building ? this.building.level : "";
        
        ctx.save();
        
        ctx.fillStyle = "#fff";
        ctx.font = (12*GAME_IMAGES_PACK) + "px pt-bold";
        ctx.shadowColor = "#000";
		ctx.shadowOffsetX = 1*GAME_IMAGES_PACK;
		ctx.shadowOffsetY = 1*GAME_IMAGES_PACK;
		ctx.shadowBlur = 8*GAME_IMAGES_PACK;
		
		var warningIcon = [CommonAssets.INPROGRESS_FROZEN, CommonAssets.BROKEN].indexOf(iconForStatus) >= 0;
		
		var img = null;
		if((!this.showCountdown || countDownText == "") && this.showLevel && this.building)
	    {
	    	img = APP.getPreloadImage(CommonAssets.BUILDING_LEVEL);
		}
		
		var cy = 0;
		if (iconForStatus == CommonAssets.STATUS_ON)
		{
    		img = APP.getPreloadImage(CommonAssets.STATUS_ON);
    		iconForStatus = null;
    		bLevel = "";
    		cy = 0;
		}
		else if (warningIcon)
    	{
    		img = APP.getPreloadImage(CommonAssets.BUILDING_LEVEL_WARNING);
    		iconForStatus = null;
    		bLevel = "!";
    		cy = 24;
    	}
		
		if(img)
		{
			this.drawIcon(img, this.buildingCenterX * GAME_IMAGES_PACK - img.width/2, this.buildingCenterY * GAME_IMAGES_PACK - img.height + cy);
			/*
			ctx.drawImage(img, 
				0, 0, img.width, img.height,
				this.buildingCenterX * GAME_IMAGES_PACK - img.width/2, this.buildingCenterY * GAME_IMAGES_PACK - img.height,
				img.width, img.height
			);
			*/
        	//ctx.drawImage(img, 0, 0, img.width, img.height, this.buildingCenterX - img.width/2, this.buildingCenterY - img.height, img.width, img.height);
		}
        
        if(this.getLevelVisibility() || warningIcon)
        {
        	//console.log(this.building.level);
        	ctx.fillText(bLevel,
        		this.buildingCenterX * GAME_IMAGES_PACK - ctx.measureText(bLevel).width/2,
        		this.buildingCenterY * GAME_IMAGES_PACK - img.height/2 + cy
			);
        }
        
        if(this.showLevel && this.building && this.building.isPerfect())
        {
        	var img = APP.getPreloadImage(CommonAssets.CRYSTAL_SIGN);
        	this.drawIcon(img, this.buildingCenterX * GAME_IMAGES_PACK - img.width/2, this.buildingCenterY * GAME_IMAGES_PACK - img.height);
        	/*
        	ctx.drawImage(img, 0, 0, img.width, img.height,
        		this.buildingCenterX * GAME_IMAGES_PACK - img.width/2,
        		this.buildingCenterY * GAME_IMAGES_PACK - img.height,
        		img.width, img.height
        	);
        	*/
        }
        
        if(iconForStatus)
        {
        	var img = APP.getPreloadImage(iconForStatus);
        	if (!img) 
        	{
        		img = new Image();
        		img.src = iconForStatus;
        		//throw Error(iconForStatus + " is not preloaded");
        	}
        	this.drawIcon(img, this.buildingCenterX * GAME_IMAGES_PACK - img.width/2, this.buildingCenterY * GAME_IMAGES_PACK - img.height);
        	//ctx.drawImage(img, 0, 0, img.width, img.height, this.buildingCenterX * GAME_IMAGES_PACK - img.width/2, this.buildingCenterY * GAME_IMAGES_PACK - img.height, img.width, img.height);
        }
        
        ctx.restore();
    }
	
	if(this.showCountdown && countDownText)
    {
    	if(!this.countDownView)
    	{
    		this.countDownView = COM.create("BuildingCountDown");
    		this.ctx.main.canvas.parentNode.appendChild(this.countDownView.DOM);
    	}
    	
    	this.countDownView.setColor(busyCountdownColor);
		this.countDownView.update(countDownText);
		this.countDownView.setPosition(this.buildingCenterX, this.buildingCenterY);
    	
    	/*
    	if(ctx)
    	{
	    	ctx.fillStyle = busyCountdownColor;
	    	ctx.font = (11 * GAME_IMAGES_PACK) + "px pt-bold";
	    	
	    	var w = ctx.measureText(countDownText).width;
	    	
	    	var img = APP.getPreloadImage(CommonAssets.COUNTDOWN_BG);
	    	ctx.drawImage(img, 0, 0, img.width, img.height, 
	    		(this.buildingCenterX - 6) * GAME_IMAGES_PACK - w / 2, 
	    		(this.buildingCenterY) * GAME_IMAGES_PACK - img.height/2, 
	    		(w + 12 * GAME_IMAGES_PACK), img.height + 4*GAME_IMAGES_PACK
	    	);
	    	
	    	ctx.fillText(countDownText, 
	    		this.buildingCenterX * GAME_IMAGES_PACK - ctx.measureText(countDownText).width/2, 
	    		(this.buildingCenterY + 5) * GAME_IMAGES_PACK
	    	);
    	}
    	*/
    }
    else
    {
    	this.hideCountDown();
    }
}

BuildingView.prototype.hideCountDown = function()
{
	if(this.countDownView)
	{
		this.countDownView.destroy();
		this.countDownView = null;
	}
}

BuildingView.prototype.updateTint = function(mouseOver)
{
    throw Error("not implemented");
    /*
    Effects.removeTint(buildingImage);
    if (building != null && building.isForeign) {
        Effects.tintRed(buildingImage);
    }
    if (mouseOver) {
        Effects.brighten(buildingImage);
    }
    */
}

BuildingView.prototype.setEffect = function(effect)
{
    console.warn("not implemented");
    /*
    var show:Boolean = effect != null;
    if (effectImage.visible != show) {
        UI.show(show, [effectImage]);
        effectImage.source = effect;
    }
    */
}