function Hex(params)
{
	this.ctx = null;
	
	this.__construct = function(params)
	{
		if (params)
		{
			$.extend(this, params);
		}
		
		if (!this.mainCanvas) this.mainCanvas = $('<canvas/>');
		if (!this.statusCanvas) this.statusCanvas = $('<canvas/>');
		if (!this.ctx) this.ctx = {main: this.mainCanvas[0].getContext('2d'), status: this.statusCanvas[0].getContext('2d')}
		
		this.buildingView = new BuildingView();
		this.buildingView.__construct({ctx: this.ctx});
		
		this.buildingView.x = this.x;
		this.buildingView.y = this.y;
		
		this.buildingView.buildingCenterX = this.x + APP.hexesOffset.left + this.WIDTH / 2;
		this.buildingView.buildingBottomY = this.y + APP.hexesOffset.top + this.HEIGHT;
	}
	
	this.__defineGetter__('building', function()
	{
        return this.estate ? this.estate.getCellByIndex(this.cellIndex).building : null;
	});
	
	this.WIDTH = APP.hexesSize.width;
	this.HEIGHT = APP.hexesSize.height;

	this.NEIGHBOUR_DX = APP.hexesSize.width;
	this.NEIGHBOUR_DY = APP.hexesSize.height;
	this.NEIGHBOUR_DXD = APP.hexesSize.width / 2;
	this.NEIGHBOUR_DYD = 40;

	this.cellIndex = -1;

	this.x = 0;
	this.y = 0;
	this.updated = false;
	this.cell = null;
	this.estate = null;

	// wall top
	var wall50 = null;
	var wall01 = null;
	var wall12 = null;
	var wall01a = null;
	var wall01b = null;
	var wall12b = null;
	// wall bottom
	var wall34 = null;
	var wall45 = null;
	var wall45a = null;
	var wall34a = null;
	var wall34b = null;
	// wall gate
	var wall23 = null;
	var wall23_area = null;

	var wallJoints = [];

	var cobbles;
	var resourceView;
	
	var onGateClick;
	
	this.oldCoble = null;
	this.oldBlocker = null;
	this.oldGateImage = null;
	
	this.updateWall = function(step)
	{
		var ns = this.neighbours;
		
		this.gate = false;
		
		if(step == 0)
		{
			var baseChildIndex = 1;
			wall50 = this.createJoint(wall50, "50", ns.hasSimpleJointAt(5, 0, this.estate), baseChildIndex, -46, -29, this.ctx.main);
			wall01 = this.createJoint(wall01, "01", ns.hasSimpleJointAt(0, 1, this.estate), baseChildIndex, -1, -40, this.ctx.main);
			wall12 = this.createJoint(wall12, "12", ns.hasSimpleJointAt(1, 2, this.estate), baseChildIndex, 46, -29, this.ctx.main);
			wall01a = this.createJoint(wall01a, "45", !ns.hasWallWith(0, this.estate) && ns.hasWallWith(1, this.estate), baseChildIndex, 10, -37, this.ctx.main);
			wall01b = this.createJoint(wall01b, "23", ns.hasWallWith(0, this.estate) && !ns.hasWallWith(1, this.estate), baseChildIndex, -10, -37, this.ctx.main);
			wall12b = this.createJoint(wall12b, "34", ns.hasWallWith(1, this.estate) && !ns.hasWallWith(2, this.estate), baseChildIndex, 45, -24, this.ctx.main);
		}
	
		if(step == 1)
		{
			baseChildIndex = 100;
			wall34 = this.createJoint(wall34, "34", ns.hasSimpleJointAt(3, 4, this.estate), baseChildIndex, -1, 23, this.ctx.main);
			wall45 = this.createJoint(wall45, "45", ns.hasSimpleJointAt(4, 5, this.estate), baseChildIndex, -46, 7, this.ctx.main);
			wall45a = this.createJoint(wall45a, "01", ns.hasWallWith(4, this.estate) && !ns.hasWallWith(5, this.estate), baseChildIndex, -47, 7, this.ctx.main);
			wall34a = this.createJoint(wall34a, "12", !ns.hasWallWith(3, this.estate) && ns.hasWallWith(4, this.estate), baseChildIndex, -10, 15, this.ctx.main);
			wall34b = this.createJoint(wall34b, "50", ns.hasWallWith(3, this.estate) && !ns.hasWallWith(4, this.estate), baseChildIndex, 10, 15, this.ctx.main);
			
			var wall23joint = ns.hasSimpleJointAt(2, 3, this.estate);
			
			if (wall23joint)
			{
				if (ns.hasGate())
				{
					this.gate = true;
					this.updateGateState(this.estate);
					this.enableGateArea();
				}
				else
				{
					this.gate = false;
					wall23 = this.createJoint(wall23, "23", true, baseChildIndex, 46, 7, this.ctx.main);
					this.disableGateArea();
				}
			}
		}
	}
	
}

Hex.prototype.prepareForUpdate = function(c)
{
	this.estate = c;
	if (this.checkInvalid(c))
	{
		this.buildingView.building = null;
	}
	else
	{
		this.buildingView.building = c.getCellByIndex(this.cellIndex).building;
		this.buildingView.order = c.getOrderByCellIndex(this.cellIndex);
	}
}

Hex.prototype.update = function(c, needToUpdateWall)
{
	this.prepareForUpdate(c);
	
	if (this.checkInvalid(c))
	{
		this.buildingView.hideCountDown();
		return;
	}

	// cobbles
	this.updateCobbles(c);
	
	if(needToUpdateWall)
	{
		this.updateWall(0);
	}

	var cell = c.getCellByIndex(this.cellIndex);
	var order = c.getOrderByCellIndex(this.cellIndex);

	this.updateBlocked(cell, c);
	this.updateBuilding(cell, order);
	
	if(needToUpdateWall)
	{
		this.updateWall(1);
	}
	
	if(this.showCellIndex)
	{
		var ctx = this.ctx.main;
		
		ctx.save();
		ctx.fillStyle = "#fff";
        ctx.font = (16*GAME_IMAGES_PACK) + "px pt-bold";
        ctx.shadowColor = "#000";
		ctx.shadowOffsetX = 1*GAME_IMAGES_PACK;
		ctx.shadowOffsetY = 1*GAME_IMAGES_PACK;
		ctx.shadowBlur = 8*GAME_IMAGES_PACK;
		
		var str = this.estate.getVisualNumber(this.cellIndex).toString();
		ctx.fillText (str,
				     (this.x + APP.hexesOffset.left + APP.hexesSize.width/2 - 2)*GAME_IMAGES_PACK - this.ctx.main.measureText(str).width/2,
				     (this.y + APP.hexesOffset.top + APP.hexesSize.height/2 + 6)*GAME_IMAGES_PACK);
		ctx.restore();
	}
}

Hex.prototype.isChanged = function()
{
	var changed = false;
	var cell = this.estate.getCellByIndex(this.cellIndex);
	var order = this.estate.getOrderByCellIndex(this.cellIndex);
	
	changed |= (this.oldCoble != this.getCobblesImage(this.estate));
	changed |= (this.oldBlocker != this.getBlockedImage(cell, this.estate));
	changed |= (this.gate && this.oldGateImage != this.getGateImage(this.estate));
	changed |= this.buildingView.isChanged(cell.building, order, this.estate.race);
	
	return changed;
}

Hex.prototype.checkInvalid = function(c)
{
	return c == null || !c.hasCell(this.cellIndex);
}

Hex.prototype.getCobblesImage = function(c)
{
	return (c.isInsideCityWall(this.cellIndex) && !c.isExpeditionReport) ? Assets.getRaceAssets(this.estate.owner.race).specific.CELL_COBBLES : null
}

Hex.prototype.updateCobbles = function(c)
{
	var img = this.getCobblesImage(c);
	this.oldCoble = img;
	if(img)
	{
		img = APP.getPreloadImage(img);
		try {
			this.ctx.main.drawImage(img, 0, 0, img.width, img.height, (this.x + APP.hexesOffset.left)*GAME_IMAGES_PACK, (this.y + APP.hexesOffset.top)*GAME_IMAGES_PACK, img.width, img.height);
		} catch (e) {}
	}
	
	img = null;
	var cell = c.getCellByIndex(this.cellIndex);
	
	if(this.showGrid)
	{
		img = APP.getPreloadImage(CommonAssets.HEX_EMPTY);
	}
	
	if(cell && cell.building && cell.building.isForeign)
	{
		img = APP.getPreloadImage(CommonAssets.HEX_ERROR);
	}
	
	if(img) try
	{
		this.ctx.main.drawImage(img, 0, 0, img.width, img.height, (this.x + APP.hexesOffset.left)*GAME_IMAGES_PACK, (this.y + APP.hexesOffset.top)*GAME_IMAGES_PACK, img.width, img.height);
	} catch (e) {}
}

Hex.prototype.getBlockedImage = function(cell, c)
{
	var img = null;
	if (cell.blocked)
	{
		img = c.terrain.blocker.asset;
	}
	else if (cell.hasResource())
	{
		img = Assets.getCellResource(cell.resource, !c.isExpeditionReport && c.isCellReachable(this.cellIndex));
	}
	
	return img;
}

Hex.prototype.updateBlocked = function(cell, c)
{
	var img = this.getBlockedImage(cell, c);
	
	this.oldBlocker = img;
	
	if(img)
	{
		img = APP.getPreloadImage(img);
		
		var w = img.width/GAME_IMAGES_PACK;
		var h = img.height/GAME_IMAGES_PACK;
		
		var left = this.x + APP.hexesOffset.left + (APP.hexesSize.width - w)/2;  
		var top = this.y + APP.hexesOffset.top + (APP.hexesSize.height - h)/2;
		
		left *= GAME_IMAGES_PACK;
		top *= GAME_IMAGES_PACK;
		
		try {
			this.ctx.main.drawImage(img, 0, 0, img.width, img.height, left, top, img.width, img.height);
		} catch (e) {}
	}
}

Hex.prototype.updateBuilding = function(cell, order)
{
	this.buildingView.buildingCenterX = this.x + APP.hexesOffset.left + this.WIDTH / 2;
	this.buildingView.buildingBottomY = this.y + APP.hexesOffset.top + this.HEIGHT;

	this.buildingView.update(cell.building, order, this.estate.race);
}

Hex.prototype.isInsideWall = function()
{
	return !this.estate.isExpeditionReport && this.estate.hasCell(this.cellIndex);
}

Hex.prototype.createJoint = function(wallJoint, section, reallyCreate, baseChildIndex, horizontalCenter, verticalCenter)
{
	if (reallyCreate)
	{
		var src = Assets.getRaceAssets(this.estate.owner.race).specific["WALL_" + section];
		
		img = APP.getPreloadImage(src);
		var w = img.width/GAME_IMAGES_PACK;
		var h = img.height/GAME_IMAGES_PACK;
		
		var left = this.x + APP.hexesOffset.left + (APP.hexesSize.width - w)/2 + horizontalCenter;  
		var top = this.y + APP.hexesOffset.top + (APP.hexesSize.height - h)/2 + verticalCenter + 2;
		
		left *= GAME_IMAGES_PACK;
		top *= GAME_IMAGES_PACK;
		
		try {
			this.ctx.main.drawImage(img, 0, 0, img.width, img.height, left, top, img.width, img.height);
		} catch (e) {}
		
		wallJoint = true;
	}
	else
	{
		wallJoint = false;
	}
	
	return wallJoint;
}

Hex.prototype.removeJoints = function()
{
	this.disableGateArea();
}

Hex.prototype.getNeighbourPoint = function(i)
{
	switch (i)
	{
		case 0:
			return {
				x : this.x - this.NEIGHBOUR_DXD,
				y : this.y - this.NEIGHBOUR_DYD
			};
		case 1:
			return {
				x : this.x + this.NEIGHBOUR_DXD,
				y : this.y - this.NEIGHBOUR_DYD
			};
		case 2:
			return {
				x : this.x + this.NEIGHBOUR_DX,
				y : this.y
			};
		case 3:
			return {
				x : this.x + this.NEIGHBOUR_DXD,
				y : this.y + this.NEIGHBOUR_DYD
			};
		case 4:
			return {
				x : this.x - this.NEIGHBOUR_DXD,
				y : this.y + this.NEIGHBOUR_DYD
			};
		case 5:
			return {
				x : this.x - this.NEIGHBOUR_DX,
				y : this.y
			};
	}
}

Hex.prototype.getNeighbourPoints = function()
{
	var res = [];
	for (var i = 0; i < 6; i++)
	{
		res.push(this.getNeighbourPoint(i));
	}
	return res;
}

Hex.prototype.getGateImage = function(estate)
{
	if(!this.gate)
	{
		return null;
	}
	
	var img = "";
	if (this.estate.isGateOpen())
	{
        img = Assets.WALL_23_OPEN(this.estate.owner.race, this.estate.hasBuff(BuffType.GATES_CANT_BE_CLOSED));
    } 
    else
    {
        img = Assets.WALL_23_CLOSED(this.estate.owner.race);
    }
    
    return img;
}

Hex.prototype.updateGateState = function(estate)
{
	if (!estate)
	{
		return;
	}
	
	var img = this.getGateImage(estate);
	
	this.oldGateImage = img;
	
	var horizontalCenter = 46;
	var verticalCenter = 7;
	
	img = APP.getPreloadImage(img);
	if(img)
	{
		var w = img.width/GAME_IMAGES_PACK;
		var h = img.height/GAME_IMAGES_PACK;
		
		var left = this.x + APP.hexesOffset.left + (APP.hexesSize.width - w)/2 + horizontalCenter;  
		var top = this.y + APP.hexesOffset.top + (APP.hexesSize.height - h)/2 + verticalCenter;
		
		left *= GAME_IMAGES_PACK;
		top *= GAME_IMAGES_PACK;
		  
		try {
			this.ctx.main.drawImage(img, 0, 0, img.width, img.height, left, top, img.width, img.height);
		} catch (e) {}
	}
}

Hex.prototype.tick = function(force)
{
	if (this.checkInvalid(this.estate))
	{
		return;
	}
	
	this.buildingView.buildingBottomY = this.y + APP.hexesOffset.top + this.HEIGHT;
	this.buildingView.buildingCenterX = this.x + APP.hexesOffset.left + this.WIDTH / 2;
	this.buildingView.buildingCenterY = this.buildingView.buildingBottomY - this.buildingView.buildingHeight / 2;
	this.buildingView.tick(null, force);
}

Hex.prototype.glowBuilding = function(doHighlight)
{
}

Hex.prototype.enableGateArea = function()
{
}

Hex.prototype.disableGateArea = function()
{
}