function AlliesTab()
{
}

extendClass(AlliesTab.prototype, DiplomatTabParent.prototype);

AlliesTab.prototype.__construct = function(dialog)
{
	COM(this.alliesList).createRenderer = $.proxy(this.createRenderer, this);
	
	this.offerAllianceButton = dialog.btnTabAction;
	COM(this.offerAllianceButton).caption = I18.f('OFFER_ALLIANCE');
	COM(this.offerAllianceButton).skin = 'green';
	this.onActionClick = $.proxy(this.onOfferAlliance, this);
	UI.show(true, [this.offerAllianceButton]);
	
	this.needUpdate = true;
}


AlliesTab.prototype.allyDialog = null; // AddEditAllyDialog

AlliesTab.prototype.tick = function()
{
    superMethod(this, 'tick')();
    if (this.diplomacy) 
    {
        if (this.offerAllianceButton)
        {
        	COM(this.offerAllianceButton).reason = this.getOfferAllianceReason();
        }
        
        for (var i = 0, len = this.diplomacy.allies.length; i < len; i++) 
        {
            var ally = this.diplomacy.allies[i];
            
            ally.refreshTimeLeft();
            if (Tick.hasPassed(ally.allianceBreakingTick)) 
            {
                this.needUpdate = true;
            }
        }
    }
    
    this.updateAlliesList();
}

AlliesTab.prototype.createRenderer = function(data)
{
    var renderer = COM.create('AllyDetails');
    
    renderer.readOnly = !this.estate.accessManageble;
    renderer.isEspionage = this.estate.isEspionage;
    renderer.onEdit = $.proxy(this.onEditAlliance, this);
    renderer.onCancel = $.proxy(this.onCancelAlliance, this);
    renderer.data = data;
    
	return renderer;
}

AlliesTab.prototype.updateAlliesList = function()
{
	if (!this.needUpdate) return;
    
	COM(this.alliesList).dataProvider = this.diplomacy.allies;
	
	this.needUpdate = false;
}

AlliesTab.prototype.onOfferAlliance = function()
{
    this.allyDialog = COM.create('AddEditAllyDialog');
    this.allyDialog.onResult = $.proxy(function (username, addToContacts, allowEstatesAccess, allowMissionsAccess)
    {
        var checkResult = AllianceConstraints.proposeAllianceValid(this.estate, this.diplomacy.allies, username);
        if (checkResult.isSuccess) 
        {
            Service.alliance.callMethod('offerAlliance', [username, addToContacts, allowEstatesAccess, allowMissionsAccess], 
            	$.proxy(this.handlerOnResult, this), $.proxy(this.handleFail, this)
            );
        } 
        else 
        {
            Alert.create(checkResult.text, I18.f('ERROR'));
        }
    }, this);
    
    this.allyDialog.toOfferAllianceMode();
    this.allyDialog.popup(this);
}

AlliesTab.prototype.onEditAlliance = function(view, ally)
{
    this.allyDialog = COM.create('AddEditAllyDialog');
    if (view.mode == AllyDetails.ALLY_MODE) 
    {
        this.allyDialog.onResult = $.proxy(function (username, addToContacts, allowEstatesAccess, allowMissionsAccess){
            Service.alliance.callMethod('editAlliance', [username, allowEstatesAccess, allowMissionsAccess], $.proxy(this.handlerOnResult, this), $.proxy(this.handleFail, this));
        }, this);
    } 
    else 
    {
        this.allyDialog.onResult = $.proxy(function (username, addToContacts, allowEstatesAccess, allowMissionsAccess){
            Service.alliance.callMethod('acceptProposal', [ally.user.username, addToContacts, allowEstatesAccess, allowMissionsAccess], $.proxy(this.handlerOnResult, this), $.proxy(this.handleFail, this));
        }, this);
    }
    
    this.allyDialog.toEditAllianceMode(ally, (view.mode == AllyDetails.PROPOSAL_MODE), (ally.user.username == this.diplomacy.myDeputy.person));
    this.allyDialog.popup(this);
}

AlliesTab.prototype.onCancelAlliance = function(view, ally)
{
	var handler = $.proxy(this.handlerOnResult, this);
    if (view.mode == AllyDetails.ALLY_MODE) 
    {
        switch (ally.status.name) 
        {
            case AllianceStatus.ACTIVE.name:
                Alert.create(I18.f('ALLIANCE_BREAK_CONFIRMATION', Tick.toHours(FConfig.BREAK_ALLIANCE_TIME)), '', function (){
                    Service.alliance.callMethod('breakAlliance', [ally.user.username], handler);
                });
                break;
            case AllianceStatus.PROPOSED.name:
                Service.alliance.callMethod('cancelOffer', [ally.user.username], handler);
                break;
        }
    } 
    else 
    {
        Service.alliance.callMethod('declineProposal', [ally.user.username], handler);
    }
}

AlliesTab.prototype.getOfferAllianceReason = function()
{
    if (!this.estate.accessManageble) 
    {
        return this.estate.accessDeputy ? I18.f('DEPUTY_CAN_NOT_DO_THIS') : I18.f('NO_ACCESS');
    } 
    else 
    {
        var capacity = this.estate.owner.getScienceValue(Science.DIPLOMACY);
        if (this.diplomacy.allies.length >= capacity) 
        {
            return I18.f('ALLIANCE_ERROR_CAPACITY_EXCEEDED', capacity);
        } 
    }
    return null;
}

AlliesTab.prototype.handlerOnResult = function(diplomacy)
{
    if (this.allyDialog) 
    {
        this.allyDialog.close();
        this.allyDialog = null;
    }
    
    this.diplomacy = diplomacy;
    this.estate.owner.diplomacy = diplomacy;
    this.handleResult(this.diplomacy);
    this.needUpdate = true;
    
    this.tick();
}
