function CapacityTipFactory()
{
	
}

CapacityTipFactory.addLine = function(tip)
{
	var args = Array.prototype.slice.call(arguments, 1);
	var tr, td;
	tr = tip.insertRow(tip.rows.length);
	td = tr.insertCell(0);
	td.style.textAlign = 'left';
	$(td).append(args.shift());
	if (args.length > 0)
	{
		td = tr.insertCell(1);
		td.style.textAlign = 'right';
		$(td).append(args.shift());
	}
	else
	{
		td.colSpan = 2;
	}
	return tr;
}
// called from manage & upgrade dialog
CapacityTipFactory.create = function(b, targetLevel, forPeople) 
{
	
	if (typeof targetLevel == 'undefined') targetLevel = -1;
    
    var tip = document.createElement('table');
    tip.className = 'tooltip-grid';
    
    if (targetLevel < 0 && !b.isWorking) 
    {
		CapacityTipFactory.addLine(tip, UI.styledText(b.getProblem(), 'negative'));
        return tip;
    }
    
    var baseCapacity = 0;
    var level = targetLevel > 0 ? targetLevel : b.level;
    var scienceBonusValue = 0;
    var scienceBonusText = "";
	var scienceName = "";
	if (b.type.isWarehouse()) 
	{
	    baseCapacity = b.getBaseWarehouseCapacity(level);
	    scienceName = I18.getScience(Science.WAREHOUSE);
	    scienceBonusValue = Science.getWarehouseCapacityBonus(b.estate.science.getLevel(Science.WAREHOUSE));
	    scienceBonusText = Strings.formatPercent(NumberUtils.floor(scienceBonusValue * 100), true);
	} 
	else if (b.type.isHouses()) 
	{
	    baseCapacity = b.getBasePeopleCapacityFor(level);
	    scienceName = I18.getScience(Science.SPEC_BENEFACTOR);
	    scienceBonusValue = NumberUtils.floor(Science.getHousesCapacityBonus(b.estate.science.getLevel(Science.SPEC_BENEFACTOR)));
	    scienceBonusText = "+" + scienceBonusValue;
    } 
    else if (b.type.isEstateHall()) 
    {
        baseCapacity = forPeople ? b.getPeopleCapacityFor(level) : b.getCapacityFor(level);
    } 
    else if (b.type.isMarket()) 
    {
        baseCapacity = b.getBaseWarehouseCapacity(level);
    } 
    else if (b.type.isClanReservation()) 
    {
        baseCapacity = b.getBasePeopleCapacityFor(level);
    } 
    else if (b.type.isClanWarehouse()) 
    {
        baseCapacity = b.getBaseWarehouseCapacity(level);
    }
    
	CapacityTipFactory.addLine(tip,
		b.isPerfect(level) ? I18.f('PERFECT_BUILDING') : I18.f('LEVEL_X', level),
		UI.styledText(baseCapacity, 'positive')
	);
    
    if (scienceBonusValue > 0) 
    {
		CapacityTipFactory.addLine(tip, scienceName, UI.styledText(scienceBonusText, 'positive'));
    }
    if (b.type.isWarehouse() || b.type.isClanCapacitor()) 
    {
        for(var i = 0, lst = b.estate.getBuildingBuffs(BuffType.BOOST_BUILDING, b.type), len = lst.length; i < len; i++) 
        {
            var buff = lst[i];
			CapacityTipFactory.addLine(tip, I18.getBuffCause(buff), I18.percentText(buff.value));
        }
    }
    
    return tip;
}

// called from manage dialog
CapacityTipFactory.createAsRaiser = function(b, targetLevel) 
{
	if (typeof targetLevel == 'undefined') targetLevel = -1;

    var tip = document.createElement('table');
    tip.className = 'tooltip-grid';
    
    if (targetLevel < 0 && !b.isWorking) 
    {
		CapacityTipFactory.addLine(tip, UI.styledText(b.getProblem(), 'negative'));
        return tip;
    }

    if (b.type.isHouses()) 
    {
        var level = targetLevel > 0 ? targetLevel : b.level;
        var baseValue = 0;
        var showBonuses = false;
        var overfilled = b.estate.isOverfilled(Resource.PEOPLE);

        //var tip:Tip = Tip.createGrid();
        var stopper = b.estate.getStoppingBuff(BType.HOUSES);
        if (stopper) 
        {
            showBonuses = false;
			CapacityTipFactory.addLine(tip, UI.styledText(I18.getBuffCause(stopper), 'negative'));
        } 
        else if (targetLevel < 0 && overfilled) 
        {
            baseValue = 0;
            showBonuses = false;
			CapacityTipFactory.addLine(tip, I18.f('HOUSES_OVERFILLED'), UI.styledText(I18.f('PER_TIME', 0), 'negative'));
        } 
        else 
        {
            baseValue = Math.floor(b.getBasePeopleProduction(level));
            showBonuses = true;
			CapacityTipFactory.addLine(tip, b.isPerfect(level) ? I18.f('PERFECT_BUILDING') : I18.f('LEVEL_X', level), UI.styledText(I18.f('PER_TIME', baseValue), 'positive'));
        }

        if (showBonuses) 
        {
            var bonus = Science.getHousesIncreaseBonus(b.estate.science.getLevel(Science.SPEC_BENEFACTOR));
            if (bonus > 0) 
            {
				CapacityTipFactory.addLine(tip, I18.getScience(Science.SPEC_BENEFACTOR), UI.styledText(Strings.formatBonusPerTime(bonus), 'positive'));
            }
        }
        if (showBonuses) 
        {
	        var buffs = b.estate.getBuildingBuffs(BuffType.BOOST_BUILDING, b.type);
	        for(var i = 0, len = buffs.length; i < len; i++) 
	        {
	            var buff = buffs[i];
				CapacityTipFactory.addLine(tip, I18.getBuffCause(buff), UI.styledText(Strings.formatBonusPerTime(buff.value), Style.forBonus(buff.value)));
            }
        }
    }
    
    return tip;
}
