function ClanCapacitorUI()
{
	this.building = null; // FBuilding
	this.clan = null; // FClan
	this.gateState = null; // GateState
}

ClanCapacitorUI.prototype.__construct = function()
{
	COM.bindExclusiveClick(this.transferButton, $.proxy(this.onTransferClick, this));
    $(this.membersList).on('select', $.proxy(this.tick, this));
    $(this.membersList).on('unselect', $.proxy(this.tick, this));
	COM.bindExclusiveClick(this.distributeButton, $.proxy(this.onDistributeClick, this));
}

ClanCapacitorUI.prototype.onTransfer = function(username, funds){}; // Function
ClanCapacitorUI.prototype.onTransferToCastle = function(xyz, funds){}; // Function
ClanCapacitorUI.prototype.onDistribute = function(funds){}; // Function

ClanCapacitorUI.prototype.building = null; // FBuilding
ClanCapacitorUI.prototype.clan = null; // FClan
ClanCapacitorUI.prototype.gateState = null; // GateState

ClanCapacitorUI.prototype.reset = function()
{
    COM(this.membersList).selectedIndex = -1;
    COM(this.resourceChooser).reset();
}

ClanCapacitorUI.prototype.getUsersAndCastles = function(clan)
{
    var res = [];
    for (var i = 0, len = clan.members.length; i < len; i++) 
    {
        res.push(clan.members[i].user.username);
    }
    for (var i = 0, len = clan.castles.length; i < len; i++) 
    {
        res.push(I18.formatEstateName(I18.f('CASTLE_NAME'), clan.castles[i]));
    }
    return res;
}

ClanCapacitorUI.prototype.update = function(building, clan, gateState)
{
    this.building = building;
    this.clan = clan;
    this.gateState = gateState;
    
    var funds = this.building.resources;
    var showResources = this.building.type.isClanReservation() ? [Resource.PEOPLE] : Resource.WAREHOUSE_RESOURCES;
    COM(this.resourceChooser).update(this.building.race, showResources, funds);
    COM(this.resources).alwaysShow = showResources;
    COM(this.resources).update(funds, this.building.race);
    
    this.capacityLabel.innerHTML = this.building.type.isClanReservation() 
    	? I18.f('CLAN_PEOPLE_CAPACITY') 
    	: I18.f('CLAN_RESOURCE_CAPACITY')
    ;
    this.currentCapacity.innerHTML = this.building.resources.sum().toString();
    this.maxCapacity.innerHTML = (this.building.type.isClanReservation() ? this.building.getPeopleCapacity() : this.building.getCapacity()).toString();

    var newMembers = this.getUsersAndCastles(this.clan);
    if (!COM(this.membersList).dataProvider || !Arrays.equals(COM(this.membersList).dataProvider, newMembers)) 
    {
        // trying to renew selection if found
        var selectedMember = COM(this.membersList).selectedItem;
        var foundIndex = -1;
        for (var index = 0; index < newMembers.length; index++) 
        {
            if (newMembers[index] === selectedMember) 
            {
                foundIndex = index;
            }
        }
        COM(this.membersList).dataProvider = newMembers;
        if (foundIndex > 0) 
        {
            COM(this.membersList).selectedItem = selectedMember;
            //membersList.scrollToIndex(foundIndex);
        }
    }
}

ClanCapacitorUI.prototype.tick = function()
{
    var canTransfer = Boolean(this.building.isOpenForTransfer() && !this.building.getProblem());
    var transferReason = [];
    if (!canTransfer)
    {
        if (this.gateState && this.gateState.isClosed()) 
        {
            transferReason.push(I18.f('TUNNEL_IS_CLOSED'));
        } 
        else if (this.building.isWorking && !canTransfer && this.gateState && this.gateState.isOpen()) 
        {
            transferReason.push(this.building.type.name == BType.CLAN_WAREHOUSE.name
				? I18.f('CLAN_TRANSFER_RESOURCES_OPEN_AT')
				: I18.f('CLAN_TRANSFER_SLAVES_OPEN_AT')
			);
        } 
        else 
        {
            transferReason.push(this.building.getProblem());
        }
    }
    else if (!this.building.accessManageble)
    {
    	transferReason = [I18.f('ACCESS_DENIED_ERROR_OWNER_LEVEL_NEEDED')];
    }
    else if (this.building.resources.isEmpty())
    {
    	transferReason = [I18.f('TRANSFER_NO_RESOURCES')];
    }
    COM(this.distributeButton).reason = transferReason.concat([]);

    if (!transferReason.length)
    {
    	if (COM(this.resourceChooser).value.sum() == 0)
    	{
    		transferReason.push(I18.f('TRANSFER_NO_RESOURCES'));
    	}
    	if (COM(this.membersList).selectedIndex < 0 || !COM(this.membersList).value)
    	{
    		transferReason.push(I18.f('TRANSFER_NO_TARGET'));
    	}
    }
    
    COM(this.transferButton).reason = transferReason;
    
    var uiFlag = this.building.isWorking && !canTransfer && this.gateState && this.gateState.isOpen();
    if (UI.show(uiFlag, [this.timeTillOpen]))
    {
        COM(this.timeTillOpen).value = Tick.ticksTill(this.building.startTransferTick);
    }
}

ClanCapacitorUI.prototype.onTransferClick = function()
{
    var name = COM(this.membersList).value;
    if (!name) return false;
    
    var castle = false;
    var xyz = null;
    for (var i = 0, len = this.clan.castles.length; i < len; i++) {
        if (name == I18.formatEstateName(I18.f('CASTLE_NAME'), this.clan.castles[i])) 
        {
            xyz = this.clan.castles[i];
            castle = true;
            break;
        }
    }

    var msg = this.building.type.isClanReservation()
		? (castle ? 'TRANSFER_CONFIRMATION_PEOPLE_CASTLE' : 'TRANSFER_CONFIRMATION_PEOPLE')
		: (castle ? 'TRANSFER_CONFIRMATION_RESOURCES_CASTLE' : 'TRANSFER_CONFIRMATION_RESOURCES')
	;
    
    Alert.create(I18.f(msg, name), '', $.proxy(function()
    {
    	var funds = COM(this.resourceChooser).value;
        if (castle) 
        {
            this.onTransferToCastle(xyz, funds);
        } 
        else 
        {
            this.onTransfer(name, funds);
        }
        
    }, this));
}

ClanCapacitorUI.prototype.onCapacityTip = function(e)
{
    //e.toolTip = CapacityTipFactory.create(this.building);
}

ClanCapacitorUI.prototype.onDistributeClick = function()
{
    var showResources = this.building.type.isClanReservation() ? [Resource.PEOPLE] : Resource.WAREHOUSE_RESOURCES;
    ResourceChooserDialog.create(I18.f('DISTRIBUTE_CLAN_RESOURCES_DESCRIPTION'), this.building.race, showResources, this.building.resources, this.onDistribute);
}