function DeputyTab()
{
	this.__defineGetter__('hasAllies', function()
	{
	    return this.activeAlliances.length > 1;
	});
	
	this.__defineGetter__('selectedDeputyName', function()
	{
	    var selected = COM(this.deputyCombo).selectedIndex;
	    return selected == 0 ? "" : this.activeAlliances[selected];
	});
	
	this.__defineGetter__('deputyDiffers', function()
	{
	    return (this.diplomacy.myDeputy.canReadMail != COM(this.allowMailAccess).checked) || (this.diplomacy.myDeputy.person != this.selectedDeputyName);
	});
	
	this.activeAlliances = []; // usernames only
	this.deputyOfEstate = null;
}

extendClass(DeputyTab.prototype, DiplomatTabParent.prototype);

DeputyTab.prototype.activeAlliances = []; // usernames only
DeputyTab.prototype.deputyOfEstate = null;

DeputyTab.prototype.__construct = function()
{
	$(this.deputyCombo).on('change', $.proxy(this.refreshDeputyButton, this));
	$(this.allowMailAccess).on('change', $.proxy(this.refreshDeputyButton, this));
	COM.bindExclusiveClick(this.setDeputyButton, $.proxy(this.onSetDeputy, this));
	
	COM.bindExclusiveClick(this.btnEnterDeputy, $.proxy(this.onEnterToDeputy, this));
	COM.bindExclusiveClick(this.refuseDeputyOfButton, $.proxy(this.dontWantBeDeputyOf, this));
}

DeputyTab.prototype.update = function(e)
{
    superMethod(this, 'update')(e);
    if (this.estate.isEspionage) 
    {
        COM(this.deputyCombo).disabled = true;
        COM(this.allowMailAccess).disabled = true;
        UI.show(false, [this.deputyCombo, this.allowMailAccess]);
        COM(this.allowMailAccess).checked = this.diplomacy.myDeputy.canReadMail;
        this.activeAlliances = [
        	I18.f('NO_DEPUTY'), 
        	this.diplomacy.myDeputy.person
        ];
        
        this.syncDeputyCombo();
        
        COM(this.deputyCombo).selectedIndex = diplomacy.myDeputy.isPresent ? 1 : 0;
    } 
    else 
    {
        UI.show(true, [this.deputyCombo, this.allowMailAccess]);
        this.fillActiveAlliances();
        this.refreshDeputyButton();
    }
    
    this.updateDeputyOf();
}

DeputyTab.prototype.onDeputyOfEstate = function(e)
{
	this.deputyOfEstate = e;
	this.updateDeputyOf();
}

DeputyTab.prototype.syncDeputyCombo = function()
{
    COM(this.deputyCombo).dataProvider = this.activeAlliances;
}

DeputyTab.prototype.updateDeputyOf = function()
{
    if (!this.diplomacy.deputyOf.capitalId) this.deputyOfEstate = null;
    
    var needRefresh = 
    	this.estate.accessManageble 
    	&& this.diplomacy.deputyOf.isPresent
    	&& this.diplomacy.deputyOf.capitalId
    	&& (!this.deputyOfEstate || this.deputyOfEstate.id.toString() != this.diplomacy.deputyOf.capitalId.toString())
    ;
    if (needRefresh)
    {
    	//return Service.city.callMethod('getById', [this.diplomacy.deputyOf.capitalId], $.proxy(this.onDeputyOfEstate, this));
    }
    
    if (this.diplomacy.deputyOf.isPresent) 
    {
        COM(this.deputyOfCombo).dataProvider = [this.diplomacy.deputyOf.person];
        COM(this.allowedReadMailOf).checked = this.diplomacy.deputyOf.canReadMail;
    } 
    else 
    {
        COM(this.allowedReadMailOf).checked = false;
        COM(this.deputyOfCombo).dataProvider = [I18.f('DEPUTY_OF_NOONE')];
    }
    
    //COM(this.btnEnterDeputy).enabled = !!this.estate.accessManageble;
    COM(this.btnEnterDeputy).reason = this.estate.accessManageble ? null : I18.f('ACCESS_DENIED_ERROR_DEPUTY_LEVEL_NEEDED');
    
    if (this.diplomacy.deputyOf.isPresent && this.diplomacy.deputyOf.capitalId) 
    {
    	COM(this.btnEnterDeputy).image.src = STATIC_INTERFACE_URL + "human/city_0.png";
    	COM(this.btnEnterDeputy).image.style.WebkitTransform = 'scale(0.7)';
    }
    else
    {
    	COM(this.btnEnterDeputy).reason = I18.f('DEPUTY_OF_NOONE');
    	COM(this.btnEnterDeputy).image.src = STATIC_INTERFACE_URL + "shared/icons/diplomatic/deputy-empty.png";
    	COM(this.btnEnterDeputy).image.style.WebkitTransform = 'scale(1.0)';
    }
    UI.show(this.diplomacy.deputyOf.isPresent && this.estate.accessManageble, [this.refuseDeputyOfButton]);
}

DeputyTab.prototype.fillActiveAlliances = function()
{
    var previousDeputy = this.activeAlliances.length > 0 ? this.selectedDeputyName : this.diplomacy.myDeputy.person;
    var previousMail = this.activeAlliances.length > 0 ? COM(this.allowMailAccess).checked : this.diplomacy.myDeputy.canReadMail;
    
    this.activeAlliances = [];
    
    var index = 0;
    for (var i = 0, len = this.diplomacy.allies.length; i < len; i++) 
    {
        var ally = this.diplomacy.allies[i];
        if (ally.isActive()) 
        {
            this.activeAlliances.push(ally.user.username);
            if (ally.user.username == previousDeputy) 
            {
                // do not subtract 1 - because the first element is "No deputy"
                index = this.activeAlliances.length;
            }
        }
    }
    this.activeAlliances.unshift(this.diplomacy.hasActiveAlliances() ? I18.f('NO_DEPUTY') : I18.f('NO_ALLIES'));

    this.syncDeputyCombo();
    COM(this.deputyCombo).disabled = !this.hasAllies || !this.estate.accessManageble;

    COM(this.deputyCombo).selectedIndex = index;
    COM(this.allowMailAccess).checked = previousMail;
}


DeputyTab.prototype.refreshDeputyButton = function()
{
    UI.show(this.estate.accessManageble, [this.setDeputyButton]);
    
    var index = COM(this.deputyCombo).selectedIndex;
    if (index == 0) 
    {
        COM(this.allowMailAccess).checked = false;
    }
    
    COM(this.allowMailAccess).disabled = index == 0 || !this.estate.accessManageble;
    var canBeSet = this.diplomacy.canBeDeputyOfMine(this.selectedDeputyName);
    UI.show(this.hasAllies && this.estate.accessManageble && this.deputyDiffers, [this.setDeputyButton]);
    COM(this.setDeputyButton).reason = canBeSet ? null : I18.f('PLAYER_ALREADY_HAS_DEPUTY', this.selectedDeputyName);
}

DeputyTab.prototype.dontWantBeDeputyOf = function()
{
    Alert.create(I18.f('REFUSE_BE_DEPUTY_OF_CONFIRMATION', this.diplomacy.deputyOf.person), '', 
    	$.proxy(function(){
    		Service.alliance.callMethod('refuseBeingDeputyOf', [this.diplomacy.deputyOf.person], $.proxy(this.handleResult, this));
    	}, this)
    )
}

DeputyTab.prototype.onSetDeputy = function()
{
	Service.alliance.callMethod('setDeputy', [this.selectedDeputyName, COM(this.allowMailAccess).checked], $.proxy(this.handleResult, this));
}

DeputyTab.prototype.onEnterToDeputy = function()
{
	GlobalSelectionCitiesCombo.returnEstateId = APP.getCityMediator().estate.id;
    Events.onShowCity.fire(this.diplomacy.deputyOf.capitalId);
    Dialog.find(this).close();
}
