var ManageClanDiplomatHall = function()
{
	callParentConstructor(ManageClanDiplomatHall.prototype, this);
	
	this.__defineGetter__('building', function() { return this.estate.findBuilding(BType.CLAN_DIPLOMAT); });

	this._warMode = null;
	this.__defineGetter__('warMode', function() { return this._warMode; });
	this.__defineSetter__('warMode', this.setWarMode);
	
	this.diplomacy = null;
	this.scroll = null;
}

extendClass(ManageClanDiplomatHall.prototype, ManageDialogChild.prototype);

ManageClanDiplomatHall.prototype.diplomacy = null;

ManageClanDiplomatHall.prototype.__construct = function()
{
	superMethod(ManageClanDiplomatHall.prototype, '__construct', this).apply(this, Array.prototype.slice.call(arguments, 0));
	
	this.updateData = $.proxy(this.updateData, this);
	
	COM.bindExclusiveClick(this.peace, $.proxy(function(){ this.warMode = false; }, this));
	COM.bindExclusiveClick(this.war, $.proxy(function(){ this.warMode = true; }, this));
	COM.bindExclusiveClick(this.add, $.proxy(this.onAddClick, this));

    this.scroll = new iScroll(this.list.parentNode, {});
    this.refreshScroll(300);
	
	this.setWarMode(false);
}

ManageClanDiplomatHall.prototype.refreshScroll = function(t)
{
	if (t) return setTimeout($.proxy(this.refreshScroll, this), t*1);
	if (this.scroll) this.scroll.refresh();
}

ManageClanDiplomatHall.prototype.reset = function()
{
	COM(this.add).caption = '';
	COM(this.add).reason = null;

	COM.clear(this.list);
	
	// activate tab button
	$(this.peace).toggleClass('active', !this.warMode);
	$(this.war).toggleClass('active', !!this.warMode);

	// update war or peace
	COM(this.add).caption = this.warMode ? I18.f('DECLARE_WAR') : I18.f('OFFER_PEACE');
}

ManageClanDiplomatHall.prototype.setWarMode = function(val)
{
	if (this._warMode === !!val) return;
	this._warMode = !!val;
	
	// clear old data
	this.reset();
	
	if (this.estate) this.update(this.estate);
}

ManageClanDiplomatHall.prototype.onAddClick = function()
{
	var title = I18.f(this.warMode ? 'DECLARE_WAR' : 'OFFER_PEACE');
	var action = I18.f(this.warMode ? 'DECLARE' : 'OFFER');
	var callback = $.proxy(function(clan){
		
		Service.alliance.callMethod((this.warMode ? 'declareWar' : 'offerPeace'), [clan], this.updateData, ClanException.errorHandler);
		
	}, this);
	SelectClanDialog.create(title, action, callback);
}

ManageClanDiplomatHall.prototype.update = function(estate)
{
    this.estate = estate;
	this.diplomacy = this.estate.owner.diplomacy.clan.diplomacy;

	if (!this.estate.accessClanDiplomacy)
	{
		COM(this.add).reason = I18.f('NO_ACCESS');
	}
	else if (!this.building.isWorking)
	{
		COM(this.add).reason = I18.f('BUILDING_IS_BROKEN_ALERT_TITLE').replace("\n", "<br/>");
	}
	else if (this.diplomacy.isDeclarationsLimit(!this.warMode))
	{
		COM(this.add).reason = I18.f((this.warMode ? 'WAR' : 'PEACE') + '_DECLARATIONS_LIMIT', FConfig.CLAN_DECLARATIONS_MAX_COUNT);
	}
	else
	{
		COM(this.add).reason = null;
	}
	
	var data = this.warMode ? this.diplomacy.warDeclarations : this.diplomacy.peaceDeclarations;
	this.updateItems(data);
}

ManageClanDiplomatHall.prototype.updateItems = function(data)
{
	var cls = this.warMode ? 'ClanWarRenderer' : 'ClanPeaceRenderer';
	
	var len = this.list.children.length, cnt = data.length;
	// clear if different type
	if (this.list.children.length && this.list.children[0].nodeName.toLowerCase() != cls.toLowerCase())
	{
		COM.clear(this.list);
		len = 0;
	}
	
	while (len > cnt) 
	{
		COM(this.list.children[0]).destroy();
		len--;
	}
	while (len < cnt) 
	{
		this.list.appendChild(COM.create(cls).DOM);
		len++;
	}
	
	for (var i = 0; i < len; i++)
	{
		COM(this.list.children[i]).update(data[i], this);
	}
	
	this.refreshScroll(300);
}

ManageClanDiplomatHall.prototype.updateData = function(diplomacy)
{
    this.estate.owner.diplomacy.clan.diplomacy = diplomacy;
    this.update(this.estate);
}


ManageClanDiplomatHall.prototype.tick = function()
{
	for (var i = 0; i < this.list.children.length; i++)
	{
		COM(this.list.children[i]).tick();
	}
}
