function ManageHall()
{
	callParentConstructor(ManageHall.prototype, this);
	
	this.__defineGetter__('innerDialog', this.getInnerDialog);
	this.__defineSetter__('innerDialog', this.setInnerDialog);

	this._innerDialog = null; // UpdatableEstateDialog
}

extendClass(ManageHall.prototype, ManageDialogChild.prototype);

ManageHall.TIP_MAX_ROWS_COUNT = 25;
ManageHall.TIP_SPACE_BETWEEN_COLUMNS = 30;

ManageHall.prototype._innerDialog = null; // UpdatableEstateDialog
ManageHall.prototype.scroll = null;

ManageHall.prototype.getInnerDialog = function()
{
	return this._innerDialog;
}

ManageHall.prototype.setInnerDialog = function(dialog)
{
    this._innerDialog = dialog;
    dialog.onClose = $.proxy(function() { this._innerDialog = null; }, this);
}

ManageHall.displayFunds = function(f, labels, problemResources, toHideResources)
{
    var resources = [Resource.GOLD, Resource.WOOD, Resource.STONE, Resource.GRAIN, Resource.IRON];
    for (var i = 0; i < 5; i++) 
    {
        if (labels[i]) 
        {
            var l = labels[i];
            UI.show(f.getValue(resources[i]) > 0 || !Arrays.contains(toHideResources, resources[i]), [l]);
            l.innerHTML = NumberUtils.floor(f.getValue(resources[i]));
            $(l).removeClass('positive negative').addClass(Arrays.contains(problemResources, resources[i]) ? 'negative' : 'positive');
        }
    }
}

ManageHall.prototype.__construct = function()
{
	COM.bindExclusiveClick(this.donateButton, $.proxy(this.onDonateCastle, this));
	COM.bindExclusiveClick(this.reorderBuildings, $.proxy(this.onReorderBuildings, this));
	COM.bindExclusiveClick(this.changeSkin, $.proxy(this.changeSkinClick, this));
	COM.bindExclusiveClick(this.increaseDefence, $.proxy(this.onIncreaseDefence, this));
	COM.bindExclusiveClick(this.populationIncreaseBox, $.proxy(this.onPeopleTip, this));
	COM.bindExclusiveClick(this.totalCapacityBox, $.proxy(this.onWarehouseCapacityTip, this));
	COM.bindExclusiveClick(this.prolongateLifetime, $.proxy(this.onProlongateLifetime, this));
	COM.bindExclusiveClick(this.brotherhood, $.proxy(this.onBrotherhood, this));
	COM.bindExclusiveClick(this.hallOfFameButton, $.proxy(this.onHallOfFame, this));
	COM.bindExclusiveClick(this.castleContributionButton, $.proxy(this.onCastleContributions, this));
	COM.bindExclusiveClick(this.cityRename, $.proxy(this.renameCity, this));
	$(this.retreatLimit).on('change', $.proxy(this.changeRetreatLimit, this));
	COM(this.ratioView).onChange = $.proxy(this.updateCapacity, this);
	
    var labels = [this.goldProduction, this.woodProduction, this.stoneProduction, this.grainProduction, this.ironProduction];
    COM.bindExclusiveClick(labels, $.proxy(this.createSpeedTip, this)).css({'cursor': 'pointer'});
	
	COM(this.retreatLimit).value = 60;
	
	this.scroll = new iScroll(this.content.parentNode, {
		onBeforeScrollMove: $.proxy(this.onScrollStart, this),
		onScrollEnd: $.proxy(this.onScrollEnd, this),
		scrollbarColor: '#784f0b',
		bounceLock: false,
		bounce: false,
	});
}

ManageHall.prototype.onClose = function()
{
	var ratioView = COM(this.ratioView);
	if (!ratioView.needUpdate)
	{
		Alert.create(I18.f('CONFIRM_CAPACITY_CHANGE'), I18.f('CONFIRM_CAPACITY_CHANGE_CAPTION'), function(){
			ratioView.onApply();
		});
	}
}

ManageHall.prototype.onScrollStart = function()
{
	APP.pauseTicks();
}

ManageHall.prototype.onScrollEnd = function()
{
	//APP.resumeTicks();
}

ManageHall.prototype.renameCity = function()
{
	RenameDialog.popup(this.estate, this);
}

ManageHall.prototype.updateDestructionTimer = function(estate)
{
    
    var timeBase = FEstate.tickNowForEstate(estate);
    if (UI.show(!Tick.isInfinity(estate.destructionTick), [this.destructionTimerLabel]))
    {
	    if (estate.shouldShowDestructionWarning()) 
	    {
	        var time = Tick.toTimeLeft(estate.destructionTick - timeBase);
	        this.destructionTimerLabel.innerHTML = I18.f('TIME_UNTIL_ESTATE_DESTRUCTION', this.building.getName(), time);
	        //$(this.destructionTimerLabel).addClass('negative');
	    } 
	    else 
	    {
	        var daysLeft = Tick.toDays(estate.destructionTick - timeBase);
	        this.destructionTimerLabel.innerHTML = I18.f('TIME_UNTIL_ESTATE_DESTRUCTION_IN_DAYS', this.building.getName(), daysLeft);
	    }
    }
}

ManageHall.prototype.update = function(e)
{
    COM(this.ratioView).update(this.estate);
    
    var uiFlag;
    var eClass = e.__CLS__.split('.').pop();
    var estateClass = this.estate.__CLS__.split('.').pop();
    
   	var skinIndex = this.estate.getSkinIndex(), b = this.building;
   	var src = (b.accessType ? Assets.B_findBy(b.race, b.type, b.resource, skinIndex) : Assets.B_locked(b.race));
    this.buildingImage.src = STATIC_URL + src;
    
    uiFlag = (estateClass == 'FCity') && e.accessManageble;
    UI.show(uiFlag, [this.cityRename]);
    
    uiFlag = (eClass == 'FLake' || eClass == 'FGrail');
    
    UI.show(uiFlag, [this.pearlProduction, this.pearlIcon]);
    if (uiFlag)
    {
    	this.pearlProduction.innerHTML = (eClass == 'FLake' ? FConfig.LAKE_PRODUCTION : FConfig.GRAIL_PRODUCTION);
    }
    
    // XXX: reorder buildings not implemented
    uiFlag = (estateClass == 'FCity' || estateClass == 'FPit' || estateClass == 'FCastle') && e.accessManageble;
    UI.show(false && uiFlag, [this.reorderBuildings]);

    var defenceDecl = ServiceHelper.getLocalServiceDecl(ServiceDecl.MAX_DEFENCE, 0);
    var isMaxDefenceActive = this.estate.maxDefenceFinishTick > this.estate.responseTick;
    var typeOK = false;
    if (defenceDecl) for (var i = 0; i < defenceDecl.estateTypes.length; i++)
    {
    	if (defenceDecl.estateTypes[i].name == this.estate.getEstateType().name)
    	{
    		typeOK = true;
    		break;
    	}
    }
    
    uiFlag = (e.accessManageble || isMaxDefenceActive) && typeOK;
    UI.show(uiFlag, [this.increaseDefence]);
    if (uiFlag)
    {
        this.increaseDefence.title = isMaxDefenceActive ? I18.activeMagicTip(this.estate.maxDefenceFinishTick) : I18.f('INCREASE_DEFENCE');
        COM(this.increaseDefence).skin = isMaxDefenceActive ? 'green' : 'yellow';
        COM(this.increaseDefence).enabled = e.accessManageble;
        Effects.pearlServiceActive(this.increaseDefence, isMaxDefenceActive);
    }
	
	uiFlag = e.accessManageble && (['FMine', 'FPit', 'FRuin'].indexOf(estateClass) >= 0);
	UI.show(uiFlag, [this.prolongateLifetime]);

	uiFlag = (estateClass != 'FCastle') && e.accessManageble && FUser.CURRENT_USER.brotherhoodEnabled;
	UI.show(uiFlag, [this.brotherhood]);

	this.updateCapacity(this.estate.capacity);
	ManageHall.displayFunds(this.estate.hiddenResourcesCapacity, [this.goldHidden, this.woodHidden, this.stoneHidden, this.grainHidden, this.ironHidden], [], []);
    
    var capacity = 0, buildings = this.estate.getWarehouseCapacitors();
    for (var i = 0; i < buildings.length; i++)
    {
        capacity += buildings[i].getCapacity();
    }
    this.warehouseCapacity.innerHTML = capacity;
    delete(buildings);
    
    uiFlag = this.estate.accessManageble && (estateClass == 'FCity' || estateClass == 'FCastle');
    UI.show(uiFlag, [this.changeSkin]);
    
    this.freePeopleIcon.src = STATIC_URL + Assets.getFreePeopleIcon(this.estate.race);

    var isBerserk = this.estate.hasBuff(BuffType.BERSERK);
    if (isBerserk) 
    {
        COM(this.retreatLimit).value = 100;
    } 
    else 
    {
        if (!Quarantine.contains('applyRetreatLimit')) 
        {
            var limit = Math.round(this.estate.retreatLimit / 10) * 10;
            COM(this.retreatLimit).value = this.estate.retreatLimit;
        }
    }
    
    COM(this.retreatLimit).disabled = this.estate.accessDeputy && !isBerserk ? null : 'disabled';
    this.defendersLabel.innerHTML = this.estate.killedUnitsCostDefender;
    this.attackersLabel.innerHTML = this.estate.killedUnitsCostAttacker;
    
    uiFlag = (eClass == 'FCastle') && (e.accessManageble || e.isAlly);
	UI.show(uiFlag, [this.donateButton]);
    
    uiFlag = (eClass == 'FCastle');
	UI.show(uiFlag, [this.hallOfFameButton]);
    
    uiFlag = ((eClass == 'FCastle') && e.owner.isInClan() && (e.accessManageble || e.isAlly || FUser.CURRENT_USER.role.canSpyAndObserve()));
	UI.show(uiFlag, [this.castleContributionButton]);
    
    if (this._innerDialog) 
    {
        this._innerDialog.update(this.estate);
    }
    
    var options = COM(COM(this.retreatLimit).list).items;
    for (var i = 0, len = options.length; i < len; i++)
    {
    	var o = COM(options[i]);
    	o.disabled = o.value < FConfig.MIN_RETREAT_PERCENT;
    }
    
	COM(this.singleFlags).updateFlags(e.flags);
	COM(this.sets).updateFlags(Flags.empty(), e.flagSetsCount);
    
    
    this.updateArrows();
}

ManageHall.prototype.onShow = function()
{
}

ManageHall.prototype.updateArrows = function()
{
	var hint = FUser.CURRENT_USER.findHint(HintType.UPGRADE_HALL);
    if (hint)
    {
    	HintArrow.CLICK_UPGRADE_MENU.show(this.dialog.actionUpgrade, hint, this.dialog.DOM, null, {x: 0, y: -5});
	}
}

ManageHall.prototype.updateCapacity = function(capacities)
{
    //ManageHall.displayFunds(capacities, [this.goldCapacity, this.woodCapacity, this.stoneCapacity, this.grainCapacity, this.ironCapacity], [], []);
}

ManageHall.prototype.tick = function()
{
    this.updateDestructionTimer(this.estate);

    var production = this.estate.getTotalProductionSpeedPerHour();
    var problemResources = Arrays.unionOf(
    	this.estate.notMaximalProductionResources(),
		this.estate.resourcesWithNegativeRaise(production),
		this.estate.getOverfilledResources()
	);
    ManageHall.displayFunds(production, [this.goldProduction, this.woodProduction, this.stoneProduction, this.grainProduction, this.ironProduction], problemResources, []);
    ManageHall.displayFunds(this.estate.getTotalRemainder(), [this.goldRemainder, this.woodRemainder, this.stoneRemainder, this.grainRemainder, this.ironRemainder], [], this.estate.getRemainderResourcesToHide());

    var warehouseResources = NumberUtils.floor(this.estate.bank.grain + this.estate.bank.wood + this.estate.bank.iron + this.estate.bank.stone + this.estate.bank.gold);
    this.resourcesSum.innerHTML = warehouseResources;

    this.peopleSpeed.innerHTML = Math.round(production.people);
    $(this.peopleSpeed)
    	.toggleClass('positive', production.people > 0)
    	.toggleClass('negative', production.people <= 0)
    ;

    this.peopleExtracted.innerHTML = NumberUtils.floor(this.estate.totalCitizens);
    this.peopleCapacity.innerHTML = NumberUtils.floor(this.estate.capacity.people);

    COM(this.ratioView).updateValues(this.estate.bank);
    COM(this.ratioView).tick();
    if (this._innerDialog) 
    {
        this._innerDialog.tick();
    }
    
    var resources = [Resource.GOLD, Resource.WOOD, Resource.STONE, Resource.GRAIN, Resource.IRON];
    for (var i = 0; i < 5; i++) 
    {
       	$(this[resources[i].name.toLowerCase()]).html(NumberUtils.floor(this.estate.bank.getValue(resources[i])));
    }
    COM(this.ratioView).fitLayout();
}

ManageHall.prototype.onShow = function()
{
    superMethod(this, 'onShow')();
    COM(this.ratioView).fitLayout();
}

ManageHall.prototype.showGridTip = function(tip)
{
    return tip.getLinesCount() > 0 ? tip : Tip.invisible();
}

ManageHall.prototype.addBuildingsToGridTip = function(grid, buildingValues, valueFormat, applyStyle)
{
    for (var i = 0, len = buildingValues.length; i < len; i++)
    {
    	var o = buildingValues[i];
        var building = o["building"];
        var value = o["value"];
        
        var row = grid.insertRow(-1), cell;
        $(row.insertCell(-1))
        	.html(I18.getBuildingName(building.type, building.race, building.resource))
        	.css({'width': '40%'})
        ;
        $(row.insertCell(-1))
        	.html(building.isPerfect() ? I18.f('PERFECT_BUILDING') : I18.f('LEVEL_X', building.level))
        	.css({'width': '40%'})
        ;
        $(row.insertCell(-1))
        	.html(this.constructValueView(value, (!building.hasSpeedProblems() && !building.isWarehouseDoesntWork() ? 'positive' : 'negative'), valueFormat))
        ;
    }
    return grid;
}

ManageHall.prototype.constructValueView = function(value, style, valueFormat)
{
    var valueView = $('<span class="' + style + '">' + (valueFormat ? I18.f(valueFormat, value) : value ) + '</span>');
    return valueView;
}

ManageHall.prototype.gridTipLine = function(typeText, levelText, value, style, valueFormat)
{
    var typeName = $('<span style="font-weight:bold;">' + typeText + '</span>');
    var valueLabel = this.constructValueView(value, style, valueFormat);
    return [typeName, levelText, valueLabel];
}

ManageHall.prototype.createSpeedTip = function(e)
{
    var r = Resource[e.currentTarget.getAttribute('resource')];
    var tip = document.createElement('div');
    
    var buildings = this.estate.getRaisersByResource(r);
    var buildingSpeeds = [];
    
    for (var i = 0; i < buildings.length; i++) 
    {
        var b = buildings[i];
        if (b.isConstructed) 
        {
            buildingSpeeds.push({"building":b, "value":NumberUtils.floor(b.getLastWorkSpeedPerHour().getValue(r))});
        }
    }
    
    var grid = document.createElement('table'), row, cell;
    grid.style.width = "100%";
    grid.className = 'text-narrow text-small highlight-odd';
    
    this.addBuildingsToGridTip(grid, buildingSpeeds, 'PER_TIME');
    
    var vanish = this.estate.getVanishPerHour(r), 
    	vanishes = NumberUtils.isPositive(vanish), 
    	value, style
    ;
    
    for (var i = 0, len = this.estate.buffs.length; i < len; i++) 
    {
    	var buff = this.estate.buffs[i];
    	value = null; style = null;
        if (buff.buffType.name == BuffType.GIVE_FUNDS_PER_HOUR.name) 
        {
            if (buff.funds.getValue(r) > 0) 
            {
                value = NumberUtils.ceil(buff.getConsumedFunds().getValue(r));
            }
        }
        if (buff.buffType.name == BuffType.TAKE_FUNDS_PER_HOUR.name) 
        {
            if (buff.funds.getValue(r) > 0) 
            {
                value = -NumberUtils.ceil(buff.getConsumedFunds().getValue(r));
            }
        }
        if (value !== null)
        {
	        row = grid.insertRow(-1);
	        $(row.insertCell(-1)).html(I18.getBuffCause(buff));
	        $(row.insertCell(-1)).html('&nbsp;');
	        $(row.insertCell(-1)).html(this.constructValueView(
	        	value, NumberUtils.compareTo(value, 0) > 0 ? 'positive' : 'negative', 'PER_TIME'
	        ));
        }
    }
    
    if (Resource.GOLD.name == r.name) 
    {
        var consume = this.estate.getConsumptionPerHour();
        var tax = this.estate.lastTaxPerHour;
        
        if (NumberUtils.equals(this.estate.lastTaxPerHour, 0) && this.estate.isOverfilled(r)) 
        {
	        row = grid.insertRow(0);
	        $(row.insertCell(-1)).html(I18.f('TAX')).css({'font-weight': 'bold'});
	        $(row.insertCell(-1)).html(I18.f('WAREHOUSES_OVERFILLED'));
	        $(row.insertCell(-1)).html(this.constructValueView(tax, 'negative', 'PER_TIME'));
        } 
        else 
        {
            var shopRatio = this.estate.shopRatioPerHour;
            var nominalTaxPerHour = this.estate.computeTaxPerHour();
            if (!this.estate.isFullTaxUsed() && !NumberUtils.equals(nominalTaxPerHour, 0)) 
            {
                shopRatio = this.estate.shopRatioPerHour * this.estate.lastTaxPerHour / nominalTaxPerHour;
            }
            var shopRatioString = Strings.formatNumber(shopRatio, 3);
            
	        row = grid.insertRow(0);
	        $(row.insertCell(-1)).html(I18.f('TAX')).css({'font-weight': 'bold'});
	        $(row.insertCell(-1)).html(Strings.format("{0} * ", shopRatioString));
	        $(row.insertCell(-1)).html(this.constructValueView(tax, this.estate.isFullTaxUsed() ? 'positive' : 'negative', 'PER_TIME'));
        }
        
        if (!FConfig.NO_ARMY_CONSUMPTION)
        {
	        row = grid.insertRow(0);
	        $(row.insertCell(-1)).html(I18.f('CONSUMPTION'));
	        $(row.insertCell(-1)).html('&nbsp;');
	        $(row.insertCell(-1)).html(this.constructValueView(NumberUtils.ceil(consume) * -1, 'negative', 'PER_TIME'));
        }
    }
    
    if (NumberUtils.compareTo(this.estate.bank.getValue(r), this.estate.capacity.getValue(r)) > 0) 
    {
        var lvl = this.estate.resourceDrainReduction == 0 
        	? " " 
        	: this.constructValueView(
        		Strings.formatNumberPercent(this.estate.resourceDrainReduction * 100, true, 0),
				this.estate.resourceDrainReduction < 0 ? 'positive' : 'negative'
			)
		;

        row = grid.insertRow(0);
        $(row.insertCell(-1)).html(I18.f('RESOURCES_DRAIN'));
        $(row.insertCell(-1)).html(lvl);
        $(row.insertCell(-1)).html(this.constructValueView(NumberUtils.ceil(vanish) * -1, 'negative', 'PER_TIME'));
    }
    
    if (grid.rows.length < 1) return false;
    tip.appendChild(grid);
    
    var a = TipDialog.create(tip, I18.getResourceName(r));
    a.userCanClose = true;
    a.setSubCaption(I18.f('PRODUCTION_PER_HOUR'));
    $(a.DOM).addClass('big');
    return a;
}

ManageHall.prototype.onWarehouseCapacityTip = function()
{
    var buildings = this.estate.getWarehouseCapacitors();
    var buildingCapacities = [];
    for (var i = 0; i < buildings.length; i++) 
    {
        var b = buildings[i];
        if (b.isConstructed) 
        {
            buildingCapacities.push({"building":b, "value":b.getCapacity()});
        }
    }
    
    var grid = document.createElement('table');
    grid.style.width = "100%";
    grid.className = 'text-narrow text-small highlight-odd';
    this.addBuildingsToGridTip(grid, buildingCapacities);
    
    var a = TipDialog.create(grid, I18.f('WAREHOUSES_TOTAL_CAPACITY'));
    a.userCanClose = true;
    $(a.DOM).addClass('big');
}

ManageHall.prototype.onPeopleTip = function()
{
    var tip, grid, buildings, buildingSpeeds, row, cell;
   
    tip = document.createElement('div');
    
    // PRODUCTION ---------------------------------------------------------------------
    $(tip).append('<div class="text-normal-bold text-middle">'+I18.f('POPULATION_INCREASE_PER_HOUR')+'</div>');
    
    buildings = this.estate.getOwnedBuildings(BType.HOUSES);
    buildingSpeeds = [];
    for (var i = 0, len = buildings.length; i < len; i++) 
    {
        var b = buildings[i];
        if (b.isConstructed) 
        {
            buildingSpeeds.push({"building":b, "value":NumberUtils.floor(b.getPeopleProductionSpeedPerHour())});
        }
    }
    
    var isOverfilled = this.estate.isOverfilled(Resource.PEOPLE);
    var reproductionFormula = isOverfilled ? I18.f('HOUSES_OVERFILLED') : Strings.format("{0} * ", Strings.formatNumber(FConfig.POPULATION_INCREASE_RATE));
    var reproductionValue = this.estate.getPeopleSelfreproductionPerHour();

    grid = document.createElement('table');
    grid.style.width = '100%';
    grid.className = 'text-narrow text-small highlight-odd';
    
    row = grid.insertRow(-1);
    $(row.insertCell(-1)).html(I18.f('REPRODUCTION'));
    $(row.insertCell(-1)).html(reproductionFormula);
    $(row.insertCell(-1)).html(this.constructValueView(reproductionValue, !isOverfilled ? 'positive' : 'negative', 'PER_TIME'));
    
    row = grid.insertRow(-1);
    $(row.insertCell(-1)).html(I18.getBuildingName(this.estate.hall.type, this.estate.hall.race, this.estate.hall.resource));
    $(row.insertCell(-1)).html(this.estate.hall.isPerfect() ? I18.f('PERFECT_BUILDING') : I18.f('LEVEL_X', this.estate.hall.level));
    $(row.insertCell(-1)).html(this.constructValueView(
    	this.estate.hall.getPeopleProductionSpeedPerHour(), !isOverfilled ? 'positive' : 'negative', 'PER_TIME'
    ));
    
    for (var i = 0, len = this.estate.buffs.length; i < len; i++)
    {
        var buff = this.estate.buffs[i], value = null;
        
        if (buff.buffType.name == BuffType.GIVE_FUNDS_PER_HOUR.name) 
        {
            if (NumberUtils.compareTo(buff.funds.people, 0) > 0) 
            {
			    value = NumberUtils.ceil(buff.getConsumedFunds().people);
            }
        }
        if (buff.buffType.name == BuffType.TAKE_FUNDS_PER_HOUR.name) 
        {
            if (NumberUtils.compareTo(buff.funds.people, 0) > 0) 
            {
			    value = -NumberUtils.ceil(buff.getConsumedFunds().people);
            }
        }
        if (value !== null)
        {
		    row = grid.insertRow(-1);
			$(row.insertCell(-1)).html(I18.getBuffCause(buff));
			$(row.insertCell(-1)).html('&nbsp;');
			$(row.insertCell(-1)).html(this.constructValueView(value, value > 0 ? 'positive' : 'negative', 'PER_TIME'));
        }
    }
    
    var vanish = this.estate.getVanishPerHour(Resource.PEOPLE);
    if (NumberUtils.compareTo(this.estate.bank.people, this.estate.capacity.people) > 0) 
    {
        var lvl = this.estate.peopleDrainReduction == 0 
        	? " " 
        	: this.constructValueView(
        		Strings.formatNumberPercent(this.estate.peopleDrainReduction * 100, true, 0), 
        		estate.peopleDrainReduction < 0 ? Style.FUNDS : Style.PROBLEM_INFO
        	)
        ;
		
	    row = grid.insertRow(-1);
		$(row.insertCell(-1)).html(I18.f('PEOPLE_DRAIN'));
		$(row.insertCell(-1)).html(lvl);
		$(row.insertCell(-1)).html(this.constructValueView(NumberUtils.ceil(vanish) * -1, 'negative', 'PER_TIME'));
    }

	this.addBuildingsToGridTip(grid, buildingSpeeds, 'PER_TIME');
	tip.appendChild(grid);
	
    // CAPACITY ---------------------------------------------------------------------
    $(tip).append('<div class="text-normal-bold text-middle">'+I18.f('PEOPLE_CAPACITY')+'</div>');
    buildings = this.estate.getPeopleCapacitors();
    buildingSpeeds = [];
    for (var i = 0, len = buildings.length; i < len; i++) 
    {
        var b = buildings[i];
        if (b.isConstructed) 
        {
            buildingSpeeds.push({"building":b, "value":b.getPeopleCapacity()});
        }
    }
    
    grid = document.createElement('table');
    grid.style.width = '100%';
    grid.className = 'text-narrow text-small highlight-odd';
    this.addBuildingsToGridTip(grid, buildingSpeeds, null, false);
    tip.appendChild(grid);
    
    var a = TipDialog.create(tip, I18.f('PEOPLE'));
    a.userCanClose = true;
    $(a.DOM).addClass('big');
}

ManageHall.prototype.boldTip = function(item)
{
    /*
    var label:Label = new Label();
    label.text = item.toString();
    label.setStyle("fontWeight", "bold");
    return Tip.create(label);
    */
}

ManageHall.prototype.changeRetreatLimit = function()
{
    if (!Quarantine.contains('applyRetreatLimit')) 
    {
        Quarantine.put('applyRetreatLimit');
       	
       	var btn = COM(COM(this.retreatLimit).button), enabled = btn.enabled;
       	
       	btn.enabled = false;
       	$(btn.DOM).css({opacity: 0.9});
       	$(this.retreatLimit).addClass('grayscale');
        Service.city.callMethod('setRetreatLimit', [this.estate.id, COM(this.retreatLimit).value], 
	        $.proxy(function(){
	        	btn.enabled = enabled;
       			$(btn.DOM).css({opacity: 1.0});
       			$(this.retreatLimit).removeClass('grayscale');
	        }, this),
	        $.proxy(function(){
	        	btn.enabled = enabled;
       			$(btn.DOM).css({opacity: 1.0});
       			$(this.retreatLimit).removeClass('grayscale');
	        }, this)
	    );
    }
}

ManageHall.prototype.reset = function()
{
    superMethod(this, 'reset')();
    //this.retreatLimit.min = FConfig.MIN_RETREAT_PERCENT;
    COM(this.ratioView).reset();
}

ManageHall.prototype.onPearlProductionTip = function(e)
{
    /*
    var tip:Tip = Tip.createGrid();
    tip.addLine(gridTipLine(I18.GOD_GIVES + ":", "", FConfig.i.LAKE_PRODUCTION, Style.FUNDS, I18.PER_TIME));
    e.toolTip = tip;
    */
}

ManageHall.prototype.onIncreaseDefence = function()
{
    if (this.estate.accessManageble) 
    {
        COM.create('IncreaseDefenceDialog').popup(this, this.estate);
    }
}

ManageHall.prototype.onProlongateLifetime = function()
{
    COM.create('ProlongateLifetimeDialog').popup(this, this.estate);
}

ManageHall.prototype.onReorderBuildings = function()
{
    Alert.error('Reordering is not supported!');
    //ManageDialog(dialog).switchDialog(ReorderBuildingsDialog.popup(null, this.estate));
}

ManageHall.prototype.changeSkinClick = function()
{
	if (!this.estate) return false;
	var estateClass = this.estate.__CLS__.split('.').pop();
	if (this.estate.accessManageble && (estateClass == 'FCity' || estateClass == 'FCastle'))
	{
		ChangeCitySkinDialog.create(this.estate).popup();
	}    
    
}

ManageHall.prototype.onRetreatLimitTip = function(event)
{
    if (this.estate.hasBuff(BuffType.BERSERK)) 
    {
        //event.toolTip = BuffCauseTip.create(this.estate.getBuffs(BuffType.BERSERK))(event);
    } 
    else 
    {
        //event.toolTip = Tip.invisible();
    }
}

ManageHall.prototype.onDonateCastle = function()
{
    CastleDonateDialog.create(this.estate, $.proxy(this.update, this));
}

ManageHall.prototype.onHallOfFame = function()
{
    if (this.innerDialog) this.innerDialog.close();
    this.innerDialog = HallOfFameDialog.create(this.estate);
}

ManageHall.prototype.onBrotherhood = function()
{
    Service.user.callMethod('visitBrotherhood', [], $.proxy(function(tick) { FUser.CURRENT_USER.settings.brotherhoodLastClickTick = tick; }, this), BrotherhoodException.errorHandler);
    BrotherhoodDialog.popup(this.estate.location);
}

ManageHall.prototype.onCastleContributions = function()
{
	CastleContributionsDialog.popup(this.estate.owner.diplomacy.clan);
}
