var ManageMilitaryUnitProducer = function()
{
	callParentConstructor(ManageMilitaryUnitProducer.prototype, this);
	
	this.__defineGetter__('level', function()
	{
        return this.estate.science.levelOf(this.utype);
	});
	
	this.__defineGetter__('consumption', function()
	{
	    if (this.estate.hasBuff(BuffType.NO_ARMY_CONSUMPTION) || FConfig.NO_ARMY_CONSUMPTION) 
	    {
	        return 0;
	    }
	    var treasury = this.estate.science.getSpecialValue(Science.TREASURY) / 100;
	    var localConsumption = (1 + this.estate.sumOfBuffs(BuffType.ARMY_CONSUMPTION, this.utype));
	    var globalConsumption = 1 + this.estate.owner.maxUnitTypeBuff(BuffType.GLOBAL_ARMY_CONSUMPTION, this.utype, FEstate.tickNowForEstate(this.estate));
	    var consumption = this.baseConsumption * (1 - treasury) * Math.max(FConfig.MIN_ARMY_CONSUMPTION, localConsumption * globalConsumption);
	    return NumberUtils.ceil(consumption * COM(this.count).value * 10) / 10;
	});
	
	this.__defineGetter__('baseConsumption', function()
	{
	    return FConfig.getStat(this.estate.race, this.utype, this.level).consume;
	});

}

extendClass(ManageMilitaryUnitProducer.prototype, ManageUnitProducerParent.prototype);

ManageMilitaryUnitProducer.prototype.onShow = function()
{
	superMethod(ManageMilitaryUnitProducer.prototype, 'onShow', this)();
}

ManageMilitaryUnitProducer.prototype.tick = function()
{
	superMethod(ManageMilitaryUnitProducer.prototype, 'tick', this)();
	
	var uv = COM(this.unitView);
	this.unitTip.update(uv.utype, uv._level, uv.race, -1, uv.aura, uv.buffs, -1, uv.time);
}

ManageMilitaryUnitProducer.prototype.onChangeUnitsCount = function() 
{
    superMethod(ManageMilitaryUnitProducer.prototype, 'onChangeUnitsCount', this)();
    this.updateConsumption();
}

ManageMilitaryUnitProducer.prototype.updateConsumption = function() 
{
    var f = new Funds();
    f.setValue(Resource.GOLD, this.consumption);
    COM(this.consumePerHour).update(f, this._b.race);
}

ManageMilitaryUnitProducer.prototype.onConsumeTip = function(e)
{
	var tip = document.createElement('div');
	tip.className = 'text-narrow text-middle';
	
	var funds = COM(this.consumePerHour).funds;
	Resource.FUNDS_RESOURCES.forEach($.proxy(function(r, idx, arr){
		
		if (!funds.getValue(r)) return;
		
		var box = document.createElement('table');
		box.style.width = '100%';
		tip.appendChild(box);
		
	    var stoppingBuffs = this.estate.getBuffs(BuffType.NO_ARMY_CONSUMPTION);
	    if (stoppingBuffs.length > 0 || FConfig.NO_ARMY_CONSUMPTION)  
	    {
	        stoppingBuffs.forEach(function(buff){
	        	
	        	$(box.insertRow(-1).insertCell(-1)).html(I18.getBuffCause(buff));
	        });
	        return;
	    }
	    
	    var cons = COM.create('IconAndValue');
	    cons.source = STATIC_INTERFACE_URL + CommonAssets.ICON_GOLD;
	    
        var tr = box.insertRow(-1);
	    if (this.estate.science.getLevel(Science.TREASURY) > 0) 
	    {
	        var treasury = this.estate.science.getSpecialValue(Science.TREASURY) / 100;
	        cons.value = I18.f('PER_TIME', NumberUtils.ceil(this.baseConsumption * COM(this.count).value * 10 * (1 - treasury)) / 10);
	        $(tr.insertCell(-1)).html(I18.box('LEVEL_X_TREASURY', this.level, -treasury));
	    } 
	    else 
	    {
	        cons.value = I18.f('PER_TIME', NumberUtils.ceil(this.baseConsumption * COM(this.count).value * 10) / 10);
	        $(tr.insertCell(-1)).html(I18.f('LEVEL_X', this.level));
	    }
        $(tr.insertCell(-1)).html(cons.DOM).addClass('align-center');
        
	    var buffs = this.estate.getBuffs(BuffType.ARMY_CONSUMPTION, this.utype);
	    buffs.forEach(function(buff){
	    	tr = box.insertRow(-1);
	    	$(tr.insertCell(-1)).html(I18.getBuffCause(buff));
	    	$(tr.insertCell(-1)).html(I18.percentReversedText(buff.value)).addClass('align-center');
	    });
	
	    var globalBuff = this.estate.owner.findMaxUnitTypeBuff(BuffType.GLOBAL_ARMY_CONSUMPTION, this.utype, FEstate.tickNowForEstate(this.estate));
	    if (globalBuff) 
	    {
	    	tr = box.insertRow(-1);
	    	$(tr.insertCell(-1)).html(I18.f('GLOBAL_ARMY_CONSUMPTION_CAUSE', I18.getBuffCause(globalBuff)));
	    	$(tr.insertCell(-1)).html(I18.percentReversedText(globalBuff.value)).addClass('align-center');
	    }
	
	    if (buffs.length > 0 || globalBuff) 
	    {
	        var localConsumption = (1 + this.estate.sumOfBuffs(BuffType.ARMY_CONSUMPTION, this.utype));
	        var globalConsumption = 1 + (globalBuff ? globalBuff.value : 0);
	        if (localConsumption * globalConsumption < FConfig.MIN_ARMY_CONSUMPTION) 
	        {
		    	tr = box.insertRow(-1);
		    	$(tr.insertCell(-1)).html(I18.f('MIN_ARMY_CONSUMPTION'));
		    	$(tr.insertCell(-1)).html(UI.styledText(Strings.formatPercent(Math.round((FConfig.MIN_ARMY_CONSUMPTION - 1) * 100), true), Style.OVERFLOW)).addClass('align-center');
	        }
	    }
		
	}, this));

	return TipDialog.create(tip, I18.f('PAYROLL'));
	//return ToolTip.createAtEvent(tip, e);
}

ManageMilitaryUnitProducer.prototype.autoProduceBuffType = function()
{
    return BuffType.AUTO_PRODUCE_MILITARY;
}

ManageMilitaryUnitProducer.prototype.getOneUnitCost = function()
{
    var resourcesDiscount = NumberUtils.limit(1 + this._b.estate.sumOfBuffs(BuffType.ECONOMY_TRAINING_MILITARY_RESOURCES, this.utype), FConfig.MIN_MILITARY_RESOURCES, 9);
    var peoplesDiscount = NumberUtils.limit(1 + this._b.estate.sumOfBuffs(BuffType.ECONOMY_TRAINING_MILITARY_PEOPLE, this.utype), FConfig.MIN_MILITARY_PEOPLE, 9);
    return superMethod(this, 'getOneUnitCost')().multiplyCopyResourcesPeople(resourcesDiscount, peoplesDiscount);
}
