function SwitchBuildingView()
{
	this.currentIndex = -1;
	this.cells = [];
	
	this.__defineGetter__('cellIndex', function()
	{
		try	{ return this.cells[this.currentIndex];	} catch (e){};
	});
}

SwitchBuildingView.prototype.currentIndex = -1;
SwitchBuildingView.prototype.cells = [];

SwitchBuildingView.prototype.__construct = function()
{
	var DOWN_EVENT = MOUSE_DOWN_EVENT;
	var MOVE_EVENT = MOUSE_MOVE_EVENT;
	var UP_EVENT = MOUSE_UP_EVENT;
	
	var doSwitch = $.proxy(function(dir){
		
		if (this.cells.length < 2 || listeners.x !== false) return false;
		
		if (dir)
		{
			this.setCurrentIndex(this.currentIndex + dir);
			var evt = $.Event('change', {target: this, value: this.cellIndex, dir: (dir > 0)});
			var anim = 'animate-' + (dir > 0 ? 'next' : 'prev');
			
			$(this.DOM)
				.removeClass('animate-prev animate-next')
				.addClass(anim)
				.trigger(evt)
			;
		} 
		
	}, this);
	
	var preventEvent = function(e){ e.preventDefault(); e.stopImmediatePropagation(); return false; }
	
	var listeners = 
	{
		d: 50, x: false,
		
		down: function(e)
		{
			if (this.cells.length > 1 && listeners.x === false)
			{
				listeners.x = e.originalEvent['touches'] ? e.originalEvent['touches'][0].pageX : e.pageX;
				window.addEventListener(MOVE_EVENT, listeners.move, false);
				window.addEventListener(UP_EVENT, listeners.up, false);
			}
			return preventEvent(e);
		},
		
		dir: function(e)
		{
			e = e['originalEvent'] || e;
			var touch = Array.prototype.slice.call((e['touches'] || []), 0).shift() || e;
			var x = (touch['pageX'] || listeners.x)*1, diff = x - listeners.x;
			return (Math.abs(diff) >= listeners.d) ? (diff < 0 ? 1 : -1) : 0;
		},
		
		move: function(e)
		{
			var dir = listeners.dir(e);
			if (dir)
			{
				return listeners.up(e, dir);
			}
		},
		
		up: function(e, dir)
		{
			dir = dir || listeners.dir(e);
			listeners.x = false;

			window.removeEventListener(MOVE_EVENT, listeners.move, false);
			window.removeEventListener(UP_EVENT, listeners.up, false);
			
			doSwitch(dir);
			return preventEvent(e);
		},
		
		unlock: function(e)
		{
			$(this.DOM).removeClass('animate-prev animate-next');
		}
	}
	
	listeners.down = $.proxy(listeners.down, this);
	listeners.move = $.proxy(listeners.move, this);
	listeners.up = $.proxy(listeners.up, this);
	listeners.unlock = $.proxy(listeners.unlock, this);
	
	COM.bindExclusiveClick(this.arrowLeft, function(e){
		doSwitch(-1);
		return preventEvent(e);
	});
	COM.bindExclusiveClick(this.arrowRight, function(e){
		doSwitch(1);
		return preventEvent(e);
	});

	$(this.arrowLeft).on(DOWN_EVENT, preventEvent);
	$(this.arrowRight).on(DOWN_EVENT, preventEvent);
	
	$(this.DOM)
		.on(DOWN_EVENT, listeners.down)
		.on('webkitAnimationEnd', listeners.unlock)
	;
}

SwitchBuildingView.prototype.getImage = function(b, o, race)
{
	//var src = null;
	var src = 'human/buildings/warehouse.png';
	if (b)
    {
    	var skinIndex = b.estate.getSkinIndex();
    	src = (b.accessType ? Assets.B_findBy(b.race, b.type, b.resource, skinIndex) : Assets.B_locked(b.race));
    }
    else if (o)
    {
    	src = Assets.B_findBy(race, o.type);
    }
    return src;
}

SwitchBuildingView.prototype.updateLinks = function(len)
{
	UI.show(len > 1, [this.arrowLeft, this.arrowRight]);
	
	var dotsThreshold = len > 7;
	
	if (UI.show(len > 1 && dotsThreshold, [this.links]))
	{
		this.links.innerHTML = this.currentIndex + 1 + ' / ' + len;
	}
	if (UI.show(len > 1 && !dotsThreshold, [this.dots]))
	{
		while (this.dots.children.length > len) this.dots.removeChild(this.dots.children[0]);
		while (this.dots.children.length < len) this.dots.appendChild(document.createElement('dot'));
		$('dot', this.dots).removeClass('active');
		this.dots.children[this.currentIndex].className = 'active';
	}
}

SwitchBuildingView.prototype.setCurrentIndex = function(idx)
{
	var len = this.cells.length;
	this.currentIndex = (len + idx) % len;
	this.updateLinks(len);
}

SwitchBuildingView.prototype.update = function(b, o, race, cells, idx)
{
	this.cells = cells || [];
	
	UI.show((this.cells.length > 0), [this.DOM]);
	
	this.image.src = STATIC_URL + this.getImage(b, o, race);
	this.image.onload = function()
	{
		var maxHeight = 80 * (new WebKitCSSMatrix(window.getComputedStyle(this).webkitTransform)).d;
		if (this.height < maxHeight) return;
		this.height *= maxHeight / this.height;
	}
	if (this.image.complete) this.image.onload.call(this.image); 
	
	this.setCurrentIndex(idx || 0);
	
	for (var i = 0; i < this.cells.length; i++)
	{
		if (this.cellIndex == this.cells[i])
		{
			if (b) this.DOM.title = b.estate.getCellLocationText(this.cellIndex);
			break;
		}
	}
}

SwitchBuildingView.prototype.tick = function(builders)
{
}
