function BrotherhoodOrderDetails()
{
	this.brotherhood = null;
	this.order = null;
}

BrotherhoodOrderDetails.prototype.brotherhood = null;
BrotherhoodOrderDetails.prototype.order = null;

BrotherhoodOrderDetails.prototype.__construct = function()
{
    COM.bindExclusiveClick(this.mailOwner, $.proxy(this.onSendMail, this));
    COM.bindExclusiveClick(this.mailExecutor, $.proxy(this.onSendMail, this));
    COM.bindExclusiveClick(this.cancelOrderButton, $.proxy(this.onCancelOrder, this));
    COM.bindExclusiveClick(this.takeOrderButton, $.proxy(this.onTakeOrder, this));
}

BrotherhoodOrderDetails.prototype.onResult = function (brotherhood){};

BrotherhoodOrderDetails.prototype.update = function(order)
{
    this.order = order;
    
    this.targetUser.innerHTML = I18.formatEstateName(order.targetUser.username, order.targetUser.capitalLocation);
    COM(this.timeView).value = order.nextUpdateTick;
    
    COM(this.mailOwner).username = order.owner ? order.owner.username : null;
    COM(this.mailExecutor).username = order.executor ? order.executor.username : null;
    
    var goalType = order.goal.__CLS__.split('.').pop();
    if (UI.show(goalType == 'KillArmyGoal', [this.killArmyView]))
    {
        this.orderGoal.innerHTML = I18.f('KILL_ARMY');
        COM(this.armyPrice).value = order.workToDo;
        COM(this.army).setUnits(order.targetUser.race, order.goal.levels, false);
        COM(this.army).setCount(order.goal.army);
        this.progress.innerHTML = Math.floor(100 * order.workDone / order.workToDo) + "%";
    }
    
    if (UI.show(goalType == 'DestroyBuildingsGoal', [this.destroyBuildingsView])) 
    {
        this.orderGoal.innerHTML = I18.f('DESTROY_BUILDINGS');
        this.buildingsCount.innerHTML = order.goal.count;
        this.btypeImage.src = STATIC_URL + Assets.B_findBy(order.targetUser.race, order.goal.btype);
        this.destroyBuildingsLocation.innerHTML = I18.formatEstateName(order.targetEstateName, order.goal.location);
        this.progress.innerHTML = order.workDone + "/" + order.workToDo;
    }
    
    if (UI.show(goalType == 'CaptureEstateGoal', [this.captureEstateView]))
    {
        this.orderGoal.innerHTML = I18.f('CAPTURE_ESTATE');
        this.captureEstateLocation.innerHTML = I18.formatEstateName(order.targetEstateName, order.goal.location);
        this.progress.text = I18.f(order.workDone ? 'YES' : 'NO');
    }
    
    var isOwner = !order.owner,
    	hasOwner = isOwner || !Strings.isEmpty(order.owner.username),
    	isExecutor = !order.executor,
    	hasExecutor = isExecutor || !Strings.isEmpty(order.executor.username)
    ;
    
    if (UI.show(!isOwner && hasOwner, [this.orderOwnerView]))
    {
        this.orderOwner.innerHTML = I18.formatEstateName(order.owner.username, order.owner.capitalLocation);
    }
    
    if (UI.show(!isExecutor && hasExecutor, [this.executorView]))
    {
        this.executor.innerHTML = I18.formatEstateName(order.executor.username, order.executor.capitalLocation);
    }
    
    UI.show(hasExecutor, [this.progressView]);
    
    COM(this.rewardPic).updateReward(order.reward);
    this.rewardName.innerHTML = COM(this.rewardPic).getTitle();
    
    UI.show(isOwner && !hasExecutor, [this.cancelOrderButton]);
    if (UI.show(!isOwner && !hasExecutor, [this.takeOrderButton]))
    {
        var reason = null;
        if (this.brotherhood.orderInExecution) reason = I18.f('BROTHERHOOD_ERROR_HAS_EXECUTOR');
        if (this.brotherhood.takeIsOnCooldown) reason = I18.f('BROTHERHOOD_ERROR_TAKE_COOLDOWN', Tick.toTimeFuture(this.brotherhood.nextTakeOrderTime));
        COM(this.takeOrderButton).reason = reason;
    }
}

BrotherhoodOrderDetails.prototype.onCancelOrder = function()
{
    var targetUsername = this.order.targetUser.username;
    Alert.create(I18.f('CANCEL_ORDER_CONFIRMATION', targetUsername), '', $.proxy(function (){
        Service.user.callMethod('cancelBrotherhoodOrder', [targetUsername], this.onResult, BrotherhoodException.errorHandler);
    }, this));
}

BrotherhoodOrderDetails.prototype.onTakeOrder = function()
{
    var targetUsername = this.order.targetUser.username;
    Alert.create(I18.f('TAKE_ORDER_CONFIRMATION', targetUsername), '', $.proxy(function ()
    {
        Service.user.callMethod('takeBrotherhoodOrder', [this.order.id], this.onResult, BrotherhoodException.errorHandler);
    }, this));
}

BrotherhoodOrderDetails.prototype.onExpired = function()
{
    Service.user.callMethod('getBrotherhood', [], this.onResult, BrotherhoodException.errorHandler);
}

BrotherhoodOrderDetails.prototype.onSendMail = function(e)
{
    var username = COM(e.currentTarget).username;
    if (!username) return false;
    MailBoxView.i().sendMessage(username);
}