function BrotherhoodOrderDialog()
{
	callParentConstructor(BrotherhoodOrderDialog.prototype, this);
	
	this.__defineGetter__('goalView', this.getGoalView);
	this.__defineGetter__('selectedAttacker', this.getSelectedAttacker);
	
	this.scroll = null;
}

extendClass(BrotherhoodOrderDialog.prototype, Dialog.prototype);

BrotherhoodOrderDialog.prototype.__construct = function()
{
	superMethod(BrotherhoodOrderDialog.prototype, '__construct', this)();
	
	this.fullScreen();
	
	COM.bindExclusiveClick(this.createButton, $.proxy(this.onCreate, this));
	COM.bindExclusiveClick(this.rewardBox, $.proxy(this.onChangeReward, this));
	$(this.attackers).on('change', $.proxy(this.updateGoalView, this));
	$(this.goals).on('change', $.proxy(this.onChangeGoal, this));
	$(this.anonymous).on('change', $.proxy(function(){
		this.offenderMessage.disabled = COM(this.anonymous).checked;
	}, this));
}

BrotherhoodOrderDialog.prototype.refreshScroll = function(timeout)
{
	if (timeout) return setTimeout($.proxy(this.refreshScroll, this), timeout*1);
	if (this.scroll) this.scroll.refresh();
}

BrotherhoodOrderDialog.prototype.onResult = null; // Function
BrotherhoodOrderDialog.prototype.reward = null; // OrderRewardType

BrotherhoodOrderDialog.popup = function(brotherhood, onResult)
{
    var d = COM.create('BrotherhoodOrderDialog');
    d.onResult = onResult;
    d.update(brotherhood);
    d.popup();
    return d;
}

BrotherhoodOrderDialog.prototype.onShow = function()
{
    var indexes = [{value:0, label:I18.f('WITHOUT_LIMITS')}];
    for (var i = 0; i < FConfig.BROTHERHOOD_EXECUTOR_RATING_INDEXES.length; i++) 
    {
        var index = FConfig.BROTHERHOOD_EXECUTOR_RATING_INDEXES[i];
        indexes.push({value:index, label:I18.f('RATING_TOP', index)});
    }
    COM(this.ratingIndexes).dataProvider = indexes;
    COM(this.ratingIndexes).selectedIndex = 0;
    
    COM(this.goals).selectedIndex = 0;
    
    COM(this.rewardPic).update(new FArtifact(ArtifactDeclaration.UNKNOWN_ARTIFACT), []);
    
    this.onChange = $.proxy(this.onChange, this);
    COM(this.goalsView).walkChildren(function(item){
    	COM(item).onChange = this.onChange;
    }, this);
    
    this.scroll = new iScroll(this.content, {});
    this.onChangeGoal();
    this.onChange();
}

BrotherhoodOrderDialog.prototype.update = function(brotherhood)
{
    var attackers = [];
    for (var i = 0; i < brotherhood.attackers.length; i++)
    {
    	attackers.push({label: brotherhood.attackers[i].username, value: brotherhood.attackers[i]});
    }
    COM(this.attackers).dataProvider = attackers;
    COM(this.attackers).selectedIndex = 0;
    this.updateGoalView();
}

BrotherhoodOrderDialog.prototype.onChange = function()
{
    if (!this.goalView) return;
    var errors = this.goalView.validate();
    if (!this.reward) 
    {
        errors.push(I18.f('CHOOSE_REWARD'));
    }
    COM(this.createButton).reason = errors;
    
    this.refreshScroll(300);
}

BrotherhoodOrderDialog.prototype.onChangeGoal = function()
{
    COM(this.goalsView).selectedIndex = COM(this.goals).selectedIndex;
    this.updateGoalView();
    this.onChange();
}

BrotherhoodOrderDialog.prototype.updateGoalView = function()
{
    if (!this.goalView || !this.selectedAttacker) return;
    this.goalView.update(this.selectedAttacker);
}

BrotherhoodOrderDialog.prototype.updateReward = function(reward)
{
    this.reward = reward;
    COM(this.rewardPic).updateReward(this.reward);
    this.rewardName.innerHTML = COM(this.rewardPic).getTitle();
    this.onChange();
}

BrotherhoodOrderDialog.prototype.onChangeReward = function()
{
    var d = ChooseOrderRewardDialog.create($.proxy(this.updateReward, this), this.reward);
}

BrotherhoodOrderDialog.prototype.onCreate = function()
{
    if (!this.selectedAttacker || !this.goalView) return;
    
    Service.user.callMethod('createBrotherhoodOrder', 
    	[
	    	this.selectedAttacker.username, 
	    	this.goalView.orderGoal.prepareForRequest(), 
	    	this.reward, 
	    	COM(this.ratingIndexes).value, 
	    	COM(this.anonymous).checked, 
	    	this.offenderMessage.value
    	], 
    	$.proxy(function(brotherhood)
    	{
			this.onResult(brotherhood);
			this.close();
		}, this),
		BrotherhoodException.errorHandler
    );
}

BrotherhoodOrderDialog.prototype.getGoalView = function()
{
    return COM(COM(this.goalsView).selectedChild);
}

BrotherhoodOrderDialog.prototype.getSelectedAttacker = function()
{
    return COM(this.attackers).value;
}

