function ClanActionsMenu()
{
	this.clan = null;
	this.asUser = null;
	this.allowControl = null;
	this.estateId = null;
	
	this.onResult = null;
}

ClanActionsMenu.prototype.clan = null;
ClanActionsMenu.prototype.asUser = null;
ClanActionsMenu.prototype.allowControl = null;
ClanActionsMenu.prototype.estateId = null;
ClanActionsMenu.prototype.onResult = null;

ClanActionsMenu.prototype.__construct = function()
{
	$(this.enterClanWarehouse).on(CLICK_EVENT, $.proxy(this.onEnterClanWarehouse, this));
	$(this.enterClanReservation).on(CLICK_EVENT, $.proxy(this.onEnterClanReservation, this));
	$(this.transformClan).on(CLICK_EVENT, $.proxy(this.onTransformClan, this));
	$(this.destroyClan).on(CLICK_EVENT, $.proxy(this.onDestroyClan, this));
}

ClanActionsMenu.prototype.update = function(clan, asUser, allowControl, estateId)
{
	this.clan = clan;
    this.asUser = asUser;
    this.allowControl = allowControl;
    this.estateId = estateId;
	
	if(!this.allowControl && asUser.username != FUser.CURRENT_USER.username)
	{
		var asUserMember = clan.getMember(asUser.username);
		if(asUserMember && asUserMember.hasRole(ClanRole.TREASURER)) this.allowControl = true;
	}
    
    var member = null;
    for (var i = 0, len = this.clan.members.length; i < len; i++)
    {
    	if (this.clan.members[i].user.username == this.asUser.username)
    	{
    		member = this.clan.members[i];
    		break;
    	}
    }
    var userIsMember = Boolean(member);
    var userIsLeader = this.clan.leaderName == this.asUser.username;
    var canChangeRole = userIsLeader && this.allowControl && !member.isLeader;
    var isImpeachment = this.clan.findQuestion(QuestionType.LEADER_IMPEACHMENT);
    
    //console.log(this.clan.leaderName, userIsMember, userIsLeader, canChangeRole, isImpeachment);
    
    var uiFlag = userIsMember && member.hasRole(ClanRole.TREASURER);
    UI.show(uiFlag, [this.enterClanReservation, this.enterClanWarehouse]);
	
	if (this.allowControl)
    {
	    COM(this.enterClanReservation).reason = this.clan.hasReservation ? null : I18.f('CLAN_ERROR_CLAN_HAS_NO_RESERVATION');
	    COM(this.enterClanWarehouse).reason = this.clan.hasWarehouse ? null : I18.f('CLAN_ERROR_CLAN_HAS_NO_WAREHOUSE');
		COM(this.destroyClan).reason = null;
    }
    else
    {
	    COM(this.enterClanReservation).reason = I18.f('NO_ACCESS');
	    COM(this.enterClanWarehouse).reason = I18.f('NO_ACCESS');
		COM(this.destroyClan).reason = I18.f('NO_ACCESS');
    }

    uiFlag = userIsMember && userIsLeader;
    UI.show(uiFlag, [this.destroyClan]);
    if (uiFlag)
    {
        if (this.clan.findQuestion(QuestionType.CLAN_DESTRUCTION)) 
        {
            COM(this.destroyClan).caption = I18.f('DESTROY_CLAN');
            COM(this.destroyClan).reason = I18.f('CLAN_ERROR_DESTRUCTION_RUNNING');
        } 
        else if (this.clan.isInDestruction()) 
        {
            COM(this.destroyClan).caption = I18.f('UNDESTROY_CLAN');
        } 
        else 
        {
            COM(this.destroyClan).caption = I18.f('DESTROY_CLAN');
        }
    }
    
    uiFlag = userIsLeader && userIsMember;
    UI.show(uiFlag, [this.transformClan]);
    if (uiFlag)
    {
        var reason = null;
        if (!this.allowControl) 
        {
            reason = I18.f('NO_ACCESS');
        } 
        else if (this.clan.type.name == ClanType.MONARCHICAL.name && this.clan.members.length < FConfig.DEMOCRATIC_CLAN_MIN_MEMBERS) 
        {
            reason = I18.f('CLAN_TRANSFORM_MEMBERS_LIMIT', FConfig.DEMOCRATIC_CLAN_MIN_MEMBERS);
        } 
        else if (this.clan.type.name == ClanType.DEMOCRATIC.name && this.clan.findQuestion(QuestionType.TRANSFORM_TO_MONARCHIC_CLAN)) 
        {
            reason = I18.f('CLAN_ERROR_TRANSFORMATION_RUNNING');
        } 
        COM(this.transformClan).reason = reason;
    }
}

ClanActionsMenu.prototype.tick = function()
{
	if (this.dialog)
	{
		this.dialog.tick();
	}
}

ClanActionsMenu.prototype.onEnterClanWarehouse = function()
{
	return this.onEnterClanCapacitor(BType.CLAN_WAREHOUSE);
}

ClanActionsMenu.prototype.onEnterClanReservation = function()
{
	return this.onEnterClanCapacitor(BType.CLAN_RESERVATION);
}

ClanActionsMenu.prototype.onEnterClanCapacitor = function(btype)
{
	Service.alliance.callMethod('getClanCapacitorManageData', [this.estateId, btype],
		$.proxy(function(data) {
			this.dialog = ClanCapacitorManagerDialog.create(this.estateId, data);
			this.dialog.onClose = $.proxy(function(){ this.dialog = null; }, this);
		}, this), 
		$.proxy(function (e){
		    if ((e.__className__ == 'ClanException')) 
		    {
		        Alert.create(I18.getClanErrorMessage(e.type));
		        return true;
		    }
		   return false;
		}, this)
    );
}

ClanActionsMenu.prototype.onTransformClan = function()
{
    switch (this.clan.type.name) {
        case ClanType.MONARCHICAL.name:
            var a = Alert.create(I18.f('TRANSFORM_TO_DEMOCRATIC_CLAN_CONFIRMATION'), htmlEscape(this.clan.name), 
	            $.proxy(function (){
	                Service.alliance.callMethod('transformClan', [null], this.onResult, ClanException.errorHandler);
	            }, this),
	            $.proxy(function (){
	            	return;
	            }, this)
            );
            a.userCanClose = true;
            break;
        case ClanType.DEMOCRATIC.name:
            ClanMessageConfirmDialog.create(I18.f('TRANSFORM_TO_MONARCHIC_CLAN_CONFIRMATION'), $.proxy(function (text) {
                Service.alliance.callMethod('transformClan', [text], this.onResult, ClanException.errorHandler);
            }, this));
    }
}

ClanActionsMenu.prototype.onDestroyClan = function()
{
    if (this.clan.isInDestruction()) 
    {
        Alert.create(I18.f('UNDESTROY_CLAN_CONFIRMATION'), htmlEscape(this.clan.clanName), 
	        $.proxy(function (){
			    Service.alliance.callMethod(
			    	'undestroyClan', [], 
			    	this.onResult, ClanException.errorHandler
			    );
	        }, this)
        );
    } 
    else 
    {
        if (this.clan.type == ClanType.DEMOCRATIC) 
        {
            ClanMessageConfirmDialog.create(I18.f('CLAN_DESTRUCTION_QUESTION_CONFIRM'), $.proxy(function (text){
                Service.alliance.callMethod('destroyClan', [text], this.onResult, ClanException.errorHandler);
            }, this));
        } 
        else 
        {
	        Alert.create(I18.f('DESTROY_CLAN_CONFIRMATION'), htmlEscape(this.clan.clanName), 
	        	$.proxy(function (){
				    Service.alliance.callMethod('destroyClan', [null], this.onResult, ClanException.errorHandler);
	        	}, this)
	        );
        }
    }
}

