function ClanMemberDialog()
{
	this.__defineGetter__('data', this.getData);
	this.__defineSetter__('data', this.setData);
	
	this.__defineGetter__('member', this.getMember);
	this.__defineSetter__('member', this.setMember);
	
	this._data = null; // FClanMember
	this.onResult = null; //Function
	this.allowControl = false; //Boolean
	this.espionageTick = 0; //Number
	this.asUser = ''; //String
	this.estateId = null; //EstateId
	this.clan = null; //FClan
	this.dialog = null; //ClanCapacitorManagerDialog
}

extendClass(ClanMemberDialog.prototype, Dialog.prototype);

ClanMemberDialog.prototype._data = null; // FClanMember
ClanMemberDialog.prototype.onResult = null; //Function
ClanMemberDialog.prototype.allowControl = false; //Boolean
ClanMemberDialog.prototype.espionageTick = 0; //Number
ClanMemberDialog.prototype.asUser = ''; //String
ClanMemberDialog.prototype.estateId = null; //EstateId
ClanMemberDialog.prototype.clan = null; //FClan
ClanMemberDialog.prototype.dialog = null; //ClanCapacitorManagerDialog

ClanMemberDialog.prototype.__construct = function()
{
    COM.bindExclusiveClick(this.sendMail, $.proxy(this.onSendMail, this));
    COM.bindExclusiveClick(this.expelButton, $.proxy(this.onExpel, this));
    COM.bindExclusiveClick(this.impeachmentButton, $.proxy(this.onInitiateImpeachment, this));
}

ClanMemberDialog.prototype.getData = function()
{
	return this._data;
}
ClanMemberDialog.prototype.setData = function(value)
{
    this._data = value;
    this.refresh();
}

ClanMemberDialog.prototype.getMember = function()
{
    return this._data;
}
ClanMemberDialog.prototype.setMember = function(val)
{
    return this.data = val;
}

ClanMemberDialog.prototype.onError = function(e)
{
    if ((e.__className__ == 'ClanException')) 
    {
        Alert.create(e.getMessage());
        return true;
    }
    return false;
}

ClanMemberDialog.prototype.expelFromClan = function(customText)
{
    Service.alliance.callMethod(
    	'expelFromClan', [this.member.user.username, customText], 
    	this.onResult, $.proxy(this.onError, this)
    );
    this.close();
}

ClanMemberDialog.prototype.onSendMail = function()
{
   	if (this.data) 
   	{
   		MailBoxView.i().sendMessage(this.data.user.username);
   	}
}

ClanMemberDialog.prototype.onExpel = function()
{
    var member = this.member;
    if (!member)
    {
    	return false;
    }
    
    if (member.hasAccepted) 
    {
        switch (this.clan.type.name) 
        {
            case ClanType.MONARCHICAL.name:
                Alert.create(I18.f('CLAN_EXPELING_CONFIRM', member.user.username), '', $.proxy(this.expelFromClan, this));
                break;
            
            case ClanType.DEMOCRATIC.name:
                ClanMessageConfirmDialog.create(I18.f('CONFIRM_MEMBER_EXPEL', member.user.username), $.proxy(this.expelFromClan, this));
                break;
        }
    } 
    else 
    {
        Alert.create(I18.f('CLAN_TAKE_AWAY_INVITATION_CONFIRM', member.user.username), '', 
        	$.proxy(function (){
			    Service.alliance.callMethod(
			    	'takeAwayClanInvitation', [member.user.username], 
			    	this.onResult, $.proxy(this.onError, this)
			    );
			    this.close();
        	}, this)
        );
    }
}

ClanMemberDialog.prototype.onGiveRole = function(role, member, checkbox)
{
    Alert.create(I18.s("GIVE", role.name, member.user.username), role.displayName, 
	    $.proxy(function (){
	        Service.alliance.callMethod('giveClanRole', [role, member.user.username], this.onResult, $.proxy(this.onError, this));
	    }, this),
	    $.proxy(function (){
	    	checkbox.checked = !checkbox.checked;
	    }, this)
    );
}

ClanMemberDialog.prototype.onTakeAwayRole = function(role, member, checkbox)
{
    Alert.create(I18.s("TAKEAWAY", role.name, member.user.username), role.displayName, 
	    $.proxy(function (){
	        Service.alliance.callMethod('takeAwayClanRole', [role, member.user.username], this.onResult, $.proxy(this.onError, this));
	    }, this),
	    $.proxy(function (){
	    	checkbox.checked = !checkbox.checked;
	    }, this)
    );
}

ClanMemberDialog.prototype.onInitiateImpeachment = function()
{
    ClanMessageConfirmDialog.create(I18.f('IMPEACHMENT_QUESTION_CONFIRM'), $.proxy(function(customText){
        Service.alliance.callMethod('initiateImpeachment', [customText], this.onResult, $.proxy(this.onError, this));
    }, this));
}

ClanMemberDialog.prototype.refresh = function()
{
    if (!this.data) 
    {
        return;
    }
    
    this.setCaption(this.member.user.username);
    
    this.memberCapitalImage.src = STATIC_URL + Assets.getRaceAssets(this.member.user.race).buildings[(FConfig.isNewYearAssets() ? "CITYHALL_NY_" : "CITYHALL_") + '0'];
    
    this.memberCapitalName.innerHTML = I18.fixName(this.member.user.capitalName);
    this.memberCapitalXYZ.innerHTML = this.member.user.capitalLocation.toString();
    
    COM(this.sacrificed).value = this.member.donatedGold;
    COM(this.sacrificedPearl).value = this.member.donatedPearl;
    
    $(this.DOM).toggleClass('grayscale', !this.member.hasAccepted);
    UI.show(this.allowControl, [this.sendMail]);
    
    this.updateRoles();
    this.tick();
}

ClanMemberDialog.prototype.tick = function()
{
    var canExpel = this.allowControl 
    	&& !this.member.isLeader 
    	&& this.member.user.username != this.asUser
		&& (this.member.hasAccepted ? this.clan.canExpel(this.asUser, this.member) : this.clan.canTakeAwayInvitation(this.asUser, this.member))
	;
    
    var uiFlag = canExpel && !this.member.isLeaving;
    UI.show(uiFlag, [this.expelButton]);
    if (uiFlag)
    {
        this.expelButton.title = this.member.hasAccepted ? I18.f('EXPEL_FROM_CLAN') : I18.f('TAKE_AWAY_CLAN_INVITATION');
    }
    
    uiFlag = this.member.isLeaving;
    UI.show(uiFlag, [this.tillLeave, this.leaveMessage]);
    if (uiFlag)
    {
        COM(this.tillLeave).value = this.espionageTick > 0 
        	? this.member.leaveClanTick - this.espionageTick 
        	: Tick.ticksTill(this.member.leaveClanTick)
        ;
        this.leaveMessage.innerHTML = I18.f('WILL_LEAVE_CLAN_IN', Tick.toTimeFuture(this.member.leaveClanTick));
    }
    
    uiFlag = this.allowControl && this.clan.type.name == ClanType.DEMOCRATIC.name && this.member.isLeader && this.member.user.username != this.asUser;
    UI.show(uiFlag, [this.impeachmentButton]);
    if (uiFlag)
    {
        var hasImpeachment = this.clan.findQuestion(QuestionType.LEADER_IMPEACHMENT);
        var hasElection = this.clan.findQuestion(QuestionType.LEADER_ELECTION_1) || this.clan.findQuestion(QuestionType.LEADER_ELECTION_2);
        if (hasImpeachment) 
        {
            COM(this.impeachmentButton).enabled = false;
            this.impeachmentButton.title = I18.f('CLAN_ERROR_IMPEACHMENT_RUNNING');
        } 
        else if (hasElection) 
        {
            COM(this.impeachmentButton).enabled = false;
            this.impeachmentButton.title = I18.f('CLAN_ERROR_ELECTION_RUNNING');
        } 
        else 
        {
            COM(this.impeachmentButton).enabled = true;
            this.impeachmentButton.title = I18.f('INITIATE_IMPEACHMENT');
        }
    }
    
    var info = this.member.user;
    if (UI.show(info.bannedTill > 0 && !info.deleted, [this.bannedBox]))
    {
		this.bannedTill.innerHTML = I18.f('IS_BANNED_TILL', Tick.toTimeFuture(info.bannedTill));
		this.bannedReason.innerHTML = I18.f('IS_BANNED_FOR', I18.getBanReason(info.banReason));
    }
    
    if (UI.show(!Strings.isEmpty(info.pwnedBy), [this.pwnedLabel]))
    {
        this.pwnedLabel.innerHTML = FUser.isMonster(info.pwnedBy) 
        	? I18.mega(I18.f('PWNED_BY_MONSTER')) 
        	: I18.mega(I18.f(PWNED_BY), info.pwnedBy)
        ;
    }
    
    if (UI.show(info.clanRole, [this.roleBox]))
    {
		this.roleLabel.innerHTML = info.clanRole.displayName;
		if (UI.show(info.nextElectionTick > 0, [this.nextLeaderElection]))
		{
			this.nextLeaderElection.innerHTML = I18.f('NEXT_LEADER_ELECTION', DateUtils.formatSmart(Tick.toMs(info.nextElectionTick)));
		}
    }
    
    if (this.dialog) 
    {
        this.dialog.tick();
    }
}

ClanMemberDialog.prototype.onTooltip = function(e)
{
    //e.toolTip = EstateStarsClanView.createAsTip(data.user, null, false, false, true)
}

ClanMemberDialog.prototype.updateRoles = function()
{
    var userIsMember = this.member.user.username == this.asUser;
    var userIsLeader = this.clan.leaderName == this.asUser;
    var canChangeRole = userIsLeader && this.allowControl && !this.member.isLeader;
    var isImpeachment = this.clan.findQuestion(QuestionType.LEADER_IMPEACHMENT);
    
    for (var i = 0, len = ClanRole.VALUES.length; i < len; i++) 
    {
        var r = ClanRole.VALUES[i];
        
        var roleContainer = document.createElement('table');
        roleContainer.style.width = '100%';
        
        var tr = roleContainer.insertRow(roleContainer.rows.length), td;
        
        td = tr.insertCell(0);
        var img = document.createElement('image');
        img.src = STATIC_INTERFACE_URL + CommonAssets.getClanRoleIcon(r);
        td.appendChild(img);
        $(td).css({'width': 50, 'height': 50, 'text-align': 'center', 'vertical-align': 'middle'});
        
        td = tr.insertCell(1);
        td.innerHTML = r.displayName;
        $(td).css({'text-align': 'left', 'vertical-align': 'middle'}).addClass('text-narrow text-middle');
        
        td = tr.insertCell(2);
        var roleCheckBox = COM.create('CheckBox');
        roleCheckBox.skin = 'round';
        roleCheckBox.value = r.name;
        roleCheckBox.checked = this.member.hasRole(r);
        roleCheckBox.disabled = !(canChangeRole 
        	&& (!this.member.role || this.member.role.name == r.name) 
        	&& (r.name != ClanRole.LEADER.name || !isImpeachment)
        );
        td.appendChild(roleCheckBox.DOM);
        $(td).css({'width': 50, 'height': 50, 'text-align': 'center', 'vertical-align': 'middle'});
        
        this.roles.appendChild(roleContainer);
        $(roleContainer).css({'border-bottom': '2px solid #d6c3ac'});
        $('td', roleContainer).css({'color': '#4a2108'});
        
        roleCheckBox.clickArea = tr;
        $(tr).css({'cursor': roleCheckBox.disabled ? 'default' : 'pointer'});
        $(roleCheckBox.DOM).on('change', $.proxy(this.checkboxClicker, this));
    }
}

ClanMemberDialog.prototype.checkboxClicker = function(e)
{
    var c = COM(e.target);
    r = ClanRole[c.value];
    if (!r) return false;
    
    if (c.checked)
    {
    	return this.onGiveRole(r, this.member, c);
    }
    else
    {
    	return this.onTakeAwayRole(r, this.member, c);
    }
}
