function ClanPeaceRenderer()
{
	this.__defineGetter__('building', function() { return this.parent.building; });
	this.__defineGetter__('estate', function() { return this.parent.estate; });
	this.__defineGetter__('diplomacy', function() { return this.estate.owner.diplomacy.clan.diplomacy; });
	
	this.parent = null;
	this.declaration = null;
}

ClanPeaceRenderer.prototype.parent = null;
ClanPeaceRenderer.prototype.declaration = null;

ClanPeaceRenderer.prototype.__construct = function()
{
	COM.bindExclusiveClick(this.cancelOfferButton, $.proxy(this.onCancelOffer, this));
	COM.bindExclusiveClick(this.acceptButton, $.proxy(this.onAccept, this));
	COM.bindExclusiveClick(this.rejectButton, $.proxy(this.onReject, this));
	COM.bindExclusiveClick(this.cancelButton, $.proxy(this.onCancel, this));
	COM.bindExclusiveClick(this.revertCancelButton, $.proxy(this.onRevertCancel, this));
	COM.bindExclusiveClick(this.sendMail, $.proxy(this.onSendMail, this));
    COM(this.timer).handler = $.proxy(this.onTimeout, this);
}

ClanPeaceRenderer.prototype.update = function(declaration, parent)
{
	this.parent = parent;
	this.declaration = declaration;
	
	this.status.src = STATIC_INTERFACE_URL + MailAssets.getPeaceStateIcon(this.declaration.state);
	if (this.declaration.state.name == ClanDeclarationState.EFFECTIVE.name && this.declaration.endTick > Tick.now())
	{
		this.status.src = STATIC_INTERFACE_URL + 'shared/mail/ally_status_breaking.png';
	}
	this.status.title = this.declaration.state.name;
	// status.layoutDirection = FlashVars.layoutDirection;
	
	this.clanName.innerHTML = htmlEscape(this.declaration.clan.name);
	
	var allowAccess = this.estate.accessClanDiplomacy && this.building.isWorking;
	
	UI.show(this.declaration.endTick > Tick.now(), [this.timerBox]);
	$(this.DOM).toggleClass('timer', this.declaration.endTick > Tick.now());
	
	UI.show(allowAccess && this.declaration.state.name == ClanDeclarationState.OUTCOMING.name, [this.cancelOfferButton]);
	
	if (UI.show(allowAccess && this.declaration.state.name == ClanDeclarationState.INCOMING.name, [this.acceptButton, this.rejectButton]))
	{
	    var limit = this.diplomacy.isDeclarationsLimit(true);
	    COM(this.acceptButton).reason = limit ? I18.f('PEACE_DECLARATIONS_LIMIT', FConfig.CLAN_DECLARATIONS_MAX_COUNT) : null;
	}
	UI.show(allowAccess && this.declaration.state.name == ClanDeclarationState.EFFECTIVE.name && this.declaration.endTick == 0, [this.cancelButton]);
	UI.show(allowAccess && this.declaration.state.name == ClanDeclarationState.EFFECTIVE.name && this.declaration.isCanceler, [this.revertCancelButton]);
	UI.show(allowAccess, [this.sendMail]);
	
	this.tick();
}

ClanPeaceRenderer.prototype.tick = function()
{
	if (this.declaration.endTick > Tick.now())
	{
	    COM(this.timer).value = this.declaration.endTick;
	}
}

ClanPeaceRenderer.prototype.onTimeout = function()
{
    if (!this.estate.isEspionage && this.building.isWorking) 
    {
        Service.alliance.callMethod('getClanDiplomacy', [], this.parent.updateData);
    }
}

ClanPeaceRenderer.prototype.onCancelOffer = function()
{
    Service.alliance.callMethod('cancelOfferPeace', [this.declaration.clan.name], this.parent.updateData, ClanException.errorHandler);
}

ClanPeaceRenderer.prototype.onAccept = function()
{
    Service.alliance.callMethod('acceptPeace', [this.declaration.clan.name], this.parent.updateData, ClanException.errorHandler);
}

ClanPeaceRenderer.prototype.onReject = function()
{
    Service.alliance.callMethod('rejectPeace', [this.declaration.clan.name], this.parent.updateData, ClanException.errorHandler);
}

ClanPeaceRenderer.prototype.onCancel = function()
{
    var self = this;
    Alert.create(I18.f('CONFIRM_CANCEL_PEACE'), '', $.proxy(function(){
    	Service.alliance.callMethod('cancelPeace', [self.declaration.clan.name], self.parent.updateData, ClanException.errorHandler);
    }, this));
}

ClanPeaceRenderer.prototype.onRevertCancel = function()
{
    var self = this;
    Alert.create(I18.f('CONFIRM_CANCEL_PEACE_REVERT'), '', $.proxy(function(){
    	Service.alliance.callMethod('cancelPeaceRevert', [self.declaration.clan.name], self.parent.updateData, ClanException.errorHandler);
    }, this));
}

ClanPeaceRenderer.prototype.onSendMail = function()
{
    MailBoxView.i().sendMessage(this.declaration.clan.name, "", null, "", MailReceiverType.CLAN_WARMINISTER);
}

