function QuestionsDialog()
{
	callParentConstructor(this);
	this.onResult = null; // Function
	this.innerDialog = null; // UpdatableEstateDialog
}

extendClass(QuestionsDialog.prototype, Dialog.prototype);

QuestionsDialog.prototype.onResult = null; // Function
QuestionsDialog.prototype.innerDialog = null; // UpdatableEstateDialog

QuestionsDialog.create = function(estate, onResult)
{
    var dialog = COM.create('QuestionsDialog');
    
    dialog.onResult = onResult;
    dialog.update(estate);
    
    dialog.popup();
    
    Service.alliance.callMethod('viewClanQuestions', [], function(tick){
    	FUser.CURRENT_USER.settings.clanQuestionsLastClickTick = tick;
    });
    
    return dialog;
}

QuestionsDialog.prototype.__construct = function()
{
	COM.bindExclusiveClick(this.createQuestionButton, $.proxy(this.createQuestion, this));
	COM.bindExclusiveClick(this.historyButton, $.proxy(this.showHistory, this));
	
	COM(this.questions).onItemSelect = $.proxy(this.showQuestion, this);
}

QuestionsDialog.prototype.update = function(estate)
{
    this.estate = estate;
    var clan = this.estate.owner.diplomacy.clan;
    if (!clan.questions) 
    {
        this.close();
        return;
    }
    
    var uiFlag = clan.questions.length == 0;
    UI.show(uiFlag, [this.noQuestionsText]);
    
    COM(this.questions).dataProvider = clan.questions;

    uiFlag = clan.leaderName == FUser.CURRENT_USER.username;
    UI.show(uiFlag, [this.createQuestionButton]);
    if (uiFlag)
    {
        var canCreate = clan.countQuestions(QuestionType.USER_QUESTION) < FConfig.USER_QUESTIONS_MAX_COUNT;
        COM(this.createQuestionButton).enabled = canCreate;
        this.createQuestionButton.title = canCreate ? null : I18.f('TOO_MANY_USER_QUESTIONS');
    }
    
    if (this.innerDialog) 
    {
        this.innerDialog.update(estate);
    }
    
    this.tick();
}

QuestionsDialog.prototype.tick = function()
{
    if (this.innerDialog) 
    {
        this.innerDialog.tick();
    }
}

QuestionsDialog.prototype.handleResult = function(diplomacy)
{
	this.estate.owner.diplomacy = diplomacy;
	this.update(this.estate);
	this.onResult(diplomacy);
}

QuestionsDialog.prototype.onCloseDialog = function()
{
    this.innerDialog = null;
}

QuestionsDialog.prototype.createQuestion = function()
{
    this.innerDialog = CreateQuestionDialog.create(this.estate, $.proxy(this.onResult, this));
    this.innerDialog.onClose = $.proxy(this.onCloseDialog, this);
}

QuestionsDialog.prototype.showQuestion = function()
{
    var question = COM(this.questions).value;
    if (!question) return false;
    
    if (this.innerDialog) this.innerDialog.close();
    this.innerDialog = ClanQuestionDialog.create(this.estate, question, $.proxy(this.handleResult, this));
    this.innerDialog.onClose = $.proxy(function(){
    	this.onCloseDialog();
    	COM(this.questions).selectedIndex = -1;
    }, this);
}

QuestionsDialog.prototype.showHistory = function()
{
    ClanArchiveDialog.create(this);
}
