function ArtifactListPageView()
{
	this.page = null; // int
	this.count = null; // int
	this.estate = null; // FEstate
	this.hero = null; // FHero
	this.parentDialog = null; // HeroDialog
	
	this._isStore = false; // Boolean
	this.__defineGetter__('isStore', this.getIsStore);
	this.__defineSetter__('isStore', this.setIsStore);
	
	this._isTrader = false; // Boolean
	this.__defineGetter__('isTrader', this.getIsTrader);
	this.__defineSetter__('isTrader', this.setIsTrader);
	
	this.__defineGetter__('caption', this.getCaption);
	this.__defineSetter__('caption', this.setCaption);
	
	this._pager = null;
	this.__defineGetter__('pager', this.getPager);
	this.__defineSetter__('pager', this.setPager);

	this._list = null; // FArtifactList
	this.__defineGetter__('list', this.getList);
	this.__defineSetter__('list', this.setList);
	
	this.filter = null;
	this.scroll = null;
}

ArtifactListPageView.prototype.__construct = function()
{
	this.caption = this.DOM.hasAttribute('caption') ? this.DOM.getAttribute('caption') : '';
	this.isStore = this.DOM.hasAttribute('isStore') && (this.DOM.getAttribute('isStore') == 'true');
	this.isTrader = this.DOM.hasAttribute('isTrader') && (this.DOM.getAttribute('isTrader') == 'true');
	
	this.initFilters();
	this.scroll = new iScroll(this.slotsWrapper, {});
}

ArtifactListPageView.prototype.refreshScroll = function(timeout)
{
	if (timeout) return setTimeout($.proxy(this.refreshScroll, this), timeout*1);
	if (this.scroll) this.scroll.refresh();
}

ArtifactListPageView.prototype.initFilters = function()
{
	var types = ['ANY', ArtifactType.ARTIFACT.name, ArtifactType.SCROLL.name, 'BOTTLE', ArtifactType.CRYSTAL.name, ArtifactType.RUNE.name];
	for (var i = 0; i < types.length; i++)
	{
		types[i] = {label: I18.f('ARTIFACT_FILTER_' + types[i]), value: types[i]}
	}
	COM(this.typeFilter).dataProvider = types;
	COM(this.typeFilter).selectedIndex = 0;
	
	var levels = FConfig.ARTIFACT_UPGRADE_COSTS.keys;
	for (var i = 0; i < levels.length; i++)
	{
		levels[i] = {label: I18.f('LEVEL_X', levels[i]), value: levels[i]}
	}
	levels.unshift({label: I18.f('ARTIFACT_FILTER_ANY_LEVEL', levels[i]), value: 0});
	
	COM(this.levelFilter).dataProvider = levels;
	COM(this.levelFilter).selectedIndex = 0;
	
	this.filter = new ArtifactFilter(null, COM(this.typeFilter).value, COM(this.levelFilter).value, true);
	
	$(this.typeFilter).on('change', $.proxy(function(){
		this.filter.type = COM(this.typeFilter).value;
		this.page = 0;
		this.updatePager();
	}, this));
	
	$(this.levelFilter).on('change', $.proxy(function(){
		this.filter.level = COM(this.levelFilter).value;
		this.page = 0;
		this.updatePager();
	}, this));
}

ArtifactListPageView.PAGE_SIZE = 24;

ArtifactListPageView.prototype.page = null; // int
ArtifactListPageView.prototype.count = null; // int
ArtifactListPageView.prototype.estate = null; // FEstate
ArtifactListPageView.prototype.hero = null; // FHero
ArtifactListPageView.prototype.parentDialog = null; // HeroDialog

ArtifactListPageView.prototype.getList = function()
{
	return this.filter.items;
}
ArtifactListPageView.prototype.setList = function(val)
{
	this.filter.items = val;
}

ArtifactListPageView.prototype._isStore = false; // Boolean
ArtifactListPageView.prototype.getIsStore = function(){return this._isStore} // Boolean
ArtifactListPageView.prototype.setIsStore = function(val)
{
	val = Boolean(val);
	if (val != this._isStore)
	{
		this._isStore = val;
		for (var i = 0, len = this.slots.children.length; i < len; i++)
		{
			COM(this.slots.children[i]).isStore = val;
		}
	}
	$(this.DOM).toggleClass('store', val);
}

ArtifactListPageView.prototype._isTrader = false; // Boolean
ArtifactListPageView.prototype.getIsTrader = function(){return this._isTrader} // Boolean
ArtifactListPageView.prototype.setIsTrader = function(val)
{
	val = Boolean(val);
	if (val != this._isTrader)
	{
		this._isTrader = val;
		for (var i = 0, len = this.slots.children.length; i < len; i++)
		{
			COM(this.slots.children[i]).isTrader = val;
		}
	}
	$(this.DOM).toggleClass('trader', val);
}

ArtifactListPageView.prototype.getCaption = function()
{
	return this.title.innerHTML;
}

ArtifactListPageView.prototype.setCaption = function(val)
{
	$(this.title).html(val || '');
	UI.show(!!val, [this.title.parentNode]);
}

ArtifactListPageView.prototype.getPager = function()
{
	return COM(this._pager);
}

ArtifactListPageView.prototype.setPager = function(val)
{
	if (this.pager)
	{
		COM(this.pager).onSelect = null;
		this._pager = null;
	}
	
	this._pager = COM(val) || null;
	if (this.pager)
	{
		COM(this.pager).onSelect = $.proxy(this.onChangePage, this);
		this.pager.page = this.page*1;
		this.pager.total = this.count*1;
	}
}

ArtifactListPageView.prototype.update = function(estate, hero, list, parentDialog)
{
	this.estate = estate;
	this.hero = hero;
	this.list = list;
	this.parentDialog = parentDialog;
	
	this.updatePager();
}

ArtifactListPageView.prototype.updatePager = function()
{
	this.count = NumberUtils.ceil(this.filter.indexes.length / ArtifactListPageView.PAGE_SIZE);
	
	var page = this.page >= this.count ? 0 : this.page;
	this.page = page;
	
	this.onChangePage(this.page);
	
    if (this.pager)
    {
	    this.pager.total = this.count*1;
	    this.pager.page = this.page*1;
	    if (UI.show(this.isStore, [this.pager.DOM]))
	    {
	    	this.pager.refreshMarker(true);
	    }
    }
}

ArtifactListPageView.prototype.onChangePage = function(page)
{
	var totalPages = Math.max(1, Math.ceil(this.filter.indexes.length / ArtifactListPageView.PAGE_SIZE));
	var oldPage = this.page*1;
	
	this.page = Math.max(0, Math.min(totalPages-1, page*1));
	
	var start = this.page * ArtifactListPageView.PAGE_SIZE;
	var end = start + ArtifactListPageView.PAGE_SIZE;
	
	if (this.isStore) 
	{
	    end = Math.min(end, this.filter.indexes.length);
	}
	var displayCount = end - start;
	
	var len = this.slots.children.length;
	var oldLen = len*1;
	while (len < displayCount) 
	{
	    var asv = COM.create('ArtifactSlotView');
	    asv.isArtifactStore = this.isStore;
	    asv.isTrader = this.isTrader;
	    this.slots.appendChild(asv.DOM);
	    len++;
	}
	while (len > displayCount) 
	{
	    COM(this.slots.children[len-1]).destroy();
	    len--;
	}
	
	for (var i = start; i < end; i++) 
	{
	    var idx = this.filter.indexes[i];
	    var a = this.filter.result[i];
	    var asv = COM(this.slots.children[i-start]);
	    asv.update(this.estate, this.hero, this.list, idx, a, [], this.parentDialog);
	}
	
	if (oldPage != this.page)
	{
		this.scroll.scrollTo(0,0,0);
		if (oldLen != displayCount)
		{
			this.refreshScroll(300);
		}
	}
}

ArtifactListPageView.prototype.fitLayout = function()
{
}

//-------------------------------------------------------------------------------------
function ArtifactFilter(list, type, level, includeEmpty)
{
	var _items = list || new FArtifactList(), 
		_type = type,
		_level = Math.max(0, Math.min(5, level*1)),
		_includeEmpty = (typeof includeEmpty == 'undefined') ? true : !!includeEmpty;
	;
	
	this.result = [];
	this.indexes = [];
	
	this.__defineGetter__('items', function(){ return _items; });
	this.__defineSetter__('items', function(val){ 
		_items = val;
		this.apply(); 
	});
	this.__defineGetter__('type', function(){ return _type || 'ANY'; });
	this.__defineSetter__('type', function(val){ 
		var str = this.toString();
		_type = val;
		if (this.toString() != str) this.apply(); 
	});
	this.__defineGetter__('level', function(){ return _level || 0; });
	this.__defineSetter__('level', function(val){ 
		var str = this.toString();
		_level = Math.max(0, Math.min(5, val*1));
		if (this.toString() != str) this.apply();
	});
	this.__defineGetter__('includeEmpty', function(){ return _includeEmpty; });
	this.__defineSetter__('includeEmpty', function(val){ 
		var str = this.toString();
		_includeEmpty = !!val;
		if (this.toString() != str) this.apply();
	});
	
	this.apply();
}

ArtifactFilter.prototype.toString = function()
{
	return [this.type, this.level, (this.includeEmpty ? 'EMPTY' : 'NOEMPTY')].join('_');
}

ArtifactFilter.prototype.checkLevel = function(a)
{
	if (!a || !a.declaration) return false;
	if (!this.level || (a.declaration.type.name == ArtifactType.CRYSTAL.name)) 
	{
		return true;
	}
	return a.declaration.level == this.level;
}

ArtifactFilter.prototype.checkType = function(a)
{
	if (!a || !a.declaration) return false;
	switch (this.type)
	{
		case 'ANY': 
			return true;
		case 'BOTTLE': 
			return a.declaration.isBottle();
		default: 
			return a.declaration.type.name == this.type;
	}
}

ArtifactFilter.prototype.check = function(a)
{
	return (a && a.declaration)
		? (this.checkLevel(a) && this.checkType(a))
		: this.includeEmpty
	;
}

ArtifactFilter.prototype.apply = function()
{
	this.result = [];
	this.indexes = [];
	for (var i = 0; i < this.items.artifacts.length; i++)
	{
		if (this.check(this.items.artifacts[i]))
		{
			this.result.push(this.items.artifacts[i]);
			this.indexes.push(i);
		}
	}
}

ArtifactFilter.prototype.compatible = function(f)
{
	if (!f) return false;
	if (this.type == 'ANY' || f.type == 'ANY' || this.type == f.type)
	{
		if (!this.level || !f.level || this.level == f.level)
		{
			return true;
		}
	}
	return f && (this.type == f.type) && (this.level == f.level);
}
