function ArtifactSlotView()
{
	this.__defineGetter__('hasArtifact', function(){
		return Boolean(this.artifact);
	});
	
	this.__defineSetter__('active', function(active){
		this._active = active;
		this.setGreyScale(!active);
	});
	
	this.__defineGetter__('dragAllowed', this.getDragAllowed);
	this.__defineSetter__('dragAllowed', this.setDragAllowed);
	
	this.list = null; // FArtifactList
	this.fromPosition = null; // int
	this.toPosition = null; // int
	this.artifact = null; // FArtifact
	this.estate = null; // FEstate
	this.hero = null; // FHero
	this.dragAllowed = true; // Boolean
	
	this.isTrader = false; // Boolean
	this.isArtifactStore = false; // Boolean
	this.isHeroDoll = false; // Boolean
	this.isCube = false; // Boolean
	
	this._active = null; // Boolean
	this.doHighlight = null; // Boolean
	this.isHighlightOk = null; // Boolean
	this.doesntWork = null; // Array
	
	this.parentDialog = null; // HeroDialog
	this.suppressClick = false; // Boolean
}

ArtifactSlotView.prototype.list = null; // FArtifactList
ArtifactSlotView.prototype.fromPosition = null; // int
ArtifactSlotView.prototype.toPosition = null; // int
ArtifactSlotView.prototype.artifact = null; // FArtifact
ArtifactSlotView.prototype.estate = null; // FEstate
ArtifactSlotView.prototype.hero = null; // FHero

ArtifactSlotView.prototype.isTrader = false; // Boolean
ArtifactSlotView.prototype.isArtifactStore = false; // Boolean
ArtifactSlotView.prototype.isHeroDoll = false; // Boolean

ArtifactSlotView.prototype._active = null; // Boolean
ArtifactSlotView.prototype.doHighlight = null; // Boolean
ArtifactSlotView.prototype.isHighlightOk = null; // Boolean
ArtifactSlotView.prototype.doesntWork = null; // Array

ArtifactSlotView.prototype.parentDialog = null; // HeroDialog
ArtifactSlotView.prototype.suppressClick = false; // Boolean

ArtifactSlotView.dropTargets = [];

ArtifactSlotView.prototype.__construct = function()
{
	if (this.DOM.hasAttribute('dragAllowed') && this.DOM.getAttribute('dragAllowed') == 'false')
	{
		this.dragAllowed = false;
	}
	ArtifactSlotView.dropTargets.push(this.DOM);
	DragManager.add(this);
}

ArtifactSlotView.prototype.__destruct = function()
{
	DragManager.remove(this.DOM);
	$(this.DOM).off('.dragndrop');
	
	for (var i = 0, len = ArtifactSlotView.dropTargets.length; i < len; i++)
	{
		if (ArtifactSlotView.dropTargets[i].isSameNode(this.DOM))
		{
			ArtifactSlotView.dropTargets.splice(i, 1);
			break;
		}
	}
}

ArtifactSlotView.prototype.showReason = function(txt, caption)
{
	if (this.reasonAlert) 
	{
		this.reasonAlert.close();
		this.reasonAlert = null;
	}
	this.reasonAlert = Alert.create(txt, caption);
}

//-----------------------------------------------------------------------------
// Drag'n'Drop interface
//-----------------------------------------------------------------------------

ArtifactSlotView.prototype._dragAllowed = true;
ArtifactSlotView.prototype.getDragAllowed = function()
{
	if (!this.artifact) 
	{
		return false;
	}
	if (!this.estate || !this.estate.accessDeputy) 
	{
	    return false;
	}
	if (this.isHeroDoll) 
	{
	    if (!this.hero || this.hero.isUpset() || this.estate.isEstateUnderAttack) 
	    {
	        if (this.hero && this.hero.isUpset()) this.showReason(I18.f('HERO_ERROR_MOVE_ARTIFACT_FOR_UPSET_HERO'));
	        return false;
	    }
	}
	return this._dragAllowed;
}
ArtifactSlotView.prototype.setDragAllowed = function(val)
{
	this._dragAllowed = Boolean(val);
}

ArtifactSlotView.prototype.createDragHelper = function()
{
	if (!this.artifact) return;
	
	var pic = COM.create('ArtifactPic');
	pic.update(this.artifact, this.doesntWork, 1);
	
	var helper = document.createElement('ArtifactSlotDragHelper');
	helper.appendChild(pic.DOM);
	
	return helper;
}

ArtifactSlotView.prototype.getDropTargets = function()
{
	return ArtifactSlotView.dropTargets;
}

ArtifactSlotView.checkDropTarget = function(source)
{
	for (var i = 0, len = ArtifactSlotView.dropTargets.length; i < len; i++)
	{
		if (ArtifactSlotView.dropTargets[i].isSameNode(source.DOM))
		{
			return true;
		}
	}
	return false;
}

ArtifactSlotView.prototype.acceptsDrop = function(source)
{
	return ArtifactSlotView.checkDropTarget(source);
};

ArtifactSlotView.prototype.canDropHere = function(source)
{
	if (source.ID == this.ID) 
	{
		return true;
	}
	if (this.estate && source && source.estate) 
	{
	    if (this.isTrader && this.estate.accessManageble && !source.isTrader) 
	    {//can sell to any slot of trader
	        return true;
	    } 
	    else if (!this.isTrader && source.isTrader && source.estate.accessManageble) 
	    {
	        return (!this.artifact || source.artifact.isStackableTo(this.artifact)) && this.acceptsDrop(source);
	    } 
	    else if (!this.isTrader && !source.isTrader) 
	    {
	        return (this.acceptsDrop(source) && (!this.artifact || ! source.artifact || this.artifact.isStackableTo(source.artifact) || source.acceptsDrop(this)));
	    }
	}
	return false;
}

ArtifactSlotView.prototype.onDragStart = function(d)
{
	if (d.source.DOM.isSameNode(this.DOM))
	{
		$(this.DOM).addClass('drag-source');
		$(d.helper).addClass('drag-helper');
	}
}

ArtifactSlotView.prototype.onDragTarget = function(d)
{
	var accept = this.canDropHere(d.source);
	$(this.DOM)
		.toggleClass('drop-target', this.acceptsDrop(d.source))
		.toggleClass('drop-target-accept', accept)
		.toggleClass('drop-target-deny', !accept)
	;
	this.highlight(true, accept);
}

ArtifactSlotView.prototype.onDragEnter = function(d)
{
	var accept = this.canDropHere(d.source);
	$(this.DOM).toggleClass('drop-target-active', accept);
	$(d.helper).toggleClass('helper-drop-accepted', accept);
	$(d.helper).toggleClass('helper-drop-denied', !accept);
}

ArtifactSlotView.prototype.onDragLeave = function(d)
{
	$(this.DOM).removeClass('drop-target-active');
	$(d.helper).removeClass('helper-drop-denied');
	$(d.helper).removeClass('helper-drop-accepted');
}

ArtifactSlotView.prototype.onDrop = function(d)
{
	var source = d.source;
	
	if (!source.artifact) return;
	if (source.ID == this.ID) return;
	if (source.list.id == this.list.id && source.fromPosition == this.fromPosition) return;
	
    var upsetHero = source.hero || this.hero;
    if (upsetHero && upsetHero.isUpset() && this.isHeroDoll)
    {
    	this.showReason(I18.f('HERO_ERROR_MOVE_ARTIFACT_FOR_UPSET_HERO'));
    	return false;
    }
    
	if (source.isTrader) 
	{
	    BuyArtifactDialog.create(this.estate, this.hero, source.fromPosition, this.list, this.toPosition);
	} 
	else if (this.isTrader) 
	{
	    SellArtifactDialog.create(this.estate, source.list, source.fromPosition, this.artifact ? -1 : this.toPosition);
	} 
	else 
	{
	    Service.hero.callMethod('moveArtifact', [this.estate.id, source.list.id, source.fromPosition, this.list.id, this.toPosition],
            $.proxy(function (e){}, this),
            $.proxy(function (e)
            {
                if (e.__className__ == 'HeroException') 
                {
                    Alert.create(e.getMessage());
                    return true;
                }
                return false;
                
            }, this)
	    );
	    
	    var src = {
	    	artifact: source.list.artifacts[source.fromPosition],
	    	doesntWork: source.doesntWork,
	    	count: source.count
	    }
	    var dest = {
	    	artifact: this.list.artifacts[this.fromPosition],
	    	doesntWork: this.doesntWork,
	    	count: this.count
	    }
	    this.update(this.estate, this.hero, this.list, this.position, src.artifact, src.doesntWork, this.parentDialog, src.count);
	    source.update(source.estate, source.hero, source.list, source.position, dest.artifact, dest.doesntWork, source.parentDialog, dest.count);
	}
}

ArtifactSlotView.prototype.onDragEnd = function(d)
{
	$(this.DOM).removeClass('drag-source drop-target drop-target-accept drop-target-deny drop-target-active');
	this.highlight(false);
}

//-----------------------------------------------------------------------------

ArtifactSlotView.prototype.update = function(estate, hero, list, position, artifact, doesntWork, parentDialog, count)
{
	if (typeof count == 'undefined') count = 1;
	this.estate = estate;
	this.hero = hero;
	this.list = list;
	this.fromPosition = position;
	this.toPosition = position;
	this.artifact = artifact;
	this.parentDialog = parentDialog;
	this.doesntWork = doesntWork;
	
	if (UI.show(this.artifact, [this.artifactView]))
	{
	    if (this.parentDialog && this.parentDialog.isInMission && this.hero && this.hero.manageAccess && this.artifact.declaration.type.name == ArtifactType.ARTIFACT.name) 
	    {
	        this.doesntWork.push(I18.f('DOESNT_WORK_IN_MISSION'));
	    }
	    if (this.estate && this.estate.accessDeputy && this.estate.isEstateUnderAttack && this.artifact.declaration.type.name == ArtifactType.ARTIFACT.name) 
	    {
	        this.doesntWork.push(I18.getMegaString(I18.f('HERO_ERROR_MOVE_ARTIFACT_UNDER_ATTACK'), FConfig));
	    }
	    
	    COM(this.artifactView).update(this.artifact, this.doesntWork, count);
	    COM(this.artifactView).setGreyScale(this.isHeroDoll && this.hero && this.hero.isUpset());
	    
		var action = {caption: I18.f('INFO'), skin: 'green', onclick: null, context: this};
		if (this.isTrader && this.estate && this.estate.accessManageble) 
		{
			action.caption = I18.f('BUY');
			action.onclick = this.buyArtifact;
			
			var estate = this.estate, av = COM(this.artifactView);
			av.onTip = $.proxy(function()
			{
			    this.closeTip();
			    if (this.artifact)
			    {
			    	this.tipDialog = ArtifactTip.alertWithCost(this.artifact, this.doesntWork, estate, this.actions);
			    	this.tipDialog.onClose = $.proxy(function(){ this.tipDialog = null; }, this);
			    }
			    
			}, av);
		} 
		else if (this.isArtifactStore && this.estate && this.estate.accessManageble) 
		{
			action.caption = I18.f('SELL');
			action.onclick = this.sellArtifact;
		} 
		else 
		{
			action = null;
		}
		
		COM(this.artifactView).actions = action;
	}
	
	this.highlight(this.doHighlight, this.isHighlightOk);
	
	$(this.DOM).toggleClass('simple', this.isTrader || this.isArtifactStore);
}

ArtifactSlotView.prototype.onClick = function(event)
{
	if (this.suppressClick) 
	{
    	this.suppressClick = false;
    	return;
	}
}

ArtifactSlotView.prototype.buyArtifact = function()
{
    COM(this.artifactView).closeTip();
    BuyArtifactDialog.create(this.estate, this.hero, this.toPosition);
}

ArtifactSlotView.prototype.sellArtifact = function()
{
    COM(this.artifactView).closeTip();
    SellArtifactDialog.create(this.estate, this.list, this.fromPosition, -1);
}

ArtifactSlotView.prototype.showArtifactInfo = function()
{
    //ArtifactInfoDialog.popup(estate, hero, artifact, parentDialog);
}

ArtifactSlotView.prototype.onDragDrop = function(e, d)
{
	var source = d.source;
}

ArtifactSlotView.prototype.onDragComplete = function()
{
	Events.onArtifactDragEnd.fire();
}

ArtifactSlotView.prototype.setMirrorMode = function(doSet, artifact)
{
	UI.show(doSet, [this.mirror]);
	if (doSet) 
	{
	    this.mirror.src = STATIC_INTERFACE_URL + Assets.getArtifact(artifact.declaration);
	    $(this.mirror).css('opacity', 0.5).addClass('grayscale');
	    //Effects.greyScale(mirror);
	}
}

ArtifactSlotView.prototype.highlight = function(doHighlight, isOk)
{
	this.doHighlight = doHighlight;
	this.isHighlightOk = isOk;
    
	if (doHighlight) 
	{
	    //if (!this.isTrader && !this.isArtifactStore)
	    if (isOk) 
	    {
	        this.slot.src = STATIC_INTERFACE_URL + 'shared/heroes/artifact_slot_accept.png';
	    } 
	    else 
	    {
	        this.slot.src = STATIC_INTERFACE_URL + 'shared/heroes/artifact_slot_deny.png';
	    }
	} 
	else 
	{
	    //if (!this.isTrader && !this.isArtifactStore)
    	this.slot.src = STATIC_INTERFACE_URL + HeroAssets.ARTIFACT_SLOT;
	}
}

ArtifactSlotView.prototype.setGreyScale = function(greyScale)
{
	if (this.DOM) $(this.DOM).toggleClass('grayscale', Boolean(greyScale));
}

ArtifactSlotView.prototype.assignCount = function(s, style)
{
	if (UI.show(s, [this.countLabel]))
	{
	    this.countLabel.innerHTML = s;
	    this.countLabel.className = style || '';
	}
}