function BuyArtifactDialog()
{
	callParentConstructor(BuyArtifactDialog.prototype, this);
	this.estate = null; // FEstate
	this.hero = null; // FHero
	this.fromPosition = null; // int
	this.toList = null; // FArtifactList
	this.toPosition = null; // int
	this.artifact = null; // FArtifact
}

extendClass(BuyArtifactDialog.prototype, Dialog.prototype);

BuyArtifactDialog.prototype.__construct = function()
{
	superMethod(BuyArtifactDialog.prototype, '__construct', this)();
	
	COM.bindExclusiveClick(this.buyButton, $.proxy(this.onBuy, this));
	COM.bindExclusiveClick(this.cancelButton, $.proxy(this.close, this));
	
}

BuyArtifactDialog.prototype.estate = null; // FEstate
BuyArtifactDialog.prototype.hero = null; // FHero
BuyArtifactDialog.prototype.fromPosition = null; // int
BuyArtifactDialog.prototype.toList = null; // FArtifactList
BuyArtifactDialog.prototype.toPosition = null; // int
BuyArtifactDialog.prototype.artifact = null; // FArtifact

BuyArtifactDialog.create = function(estate, hero, fromPosition, toList, toPosition)
{
	if (typeof toPosition == 'undefined') toPosition = -1;
	
	var result = COM.create('BuyArtifactDialog');
	result.hero = hero;
	result.fromPosition = fromPosition;
	result.toList = toList;
	result.toPosition = toPosition;
	result.artifact = estate.owner.traderArtifacts.artifacts[fromPosition];
	result.update(estate);
	result.popup(null);
	
	result.init();
}

BuyArtifactDialog.prototype.init = function()
{
	COM(this.artifactInfo).update(this.artifact);
	Events.onFEstateUpdated.add(this, this.update);
}

BuyArtifactDialog.prototype.update = function(e)
{
	this.estate = e;
	this.updatePrice();
}

BuyArtifactDialog.prototype.updatePrice = function()
{
	COM(this.artifactPrice).update(this.artifact, this.estate);
	
	var creditValue = Math.min(
		this.estate.owner.artifactTraderCredit,
	    Math.floor(this.artifact.price * FConfig.BUY_ARTIFACT_MAX_CREDIT_PART)
	);
	
	UI.show(this.artifact.price > 0 || creditValue < 0, [this.obtainPearlButton]);
	
	if (this.estate.owner.pearl < this.artifact.price - creditValue || !this.estate.bank.greaterOrEqualsTo(this.artifact.declaration.cost.resource)) 
	{
	    COM(this.buyButton).reason = [I18.f('NOT_ENOUGH_RESOURCES')];
	    return;
	}
	
	// store condition
	var store = this.estate.findBuilding(BType.ARTIFACT_STORE);
	if (!store) 
	{
	    COM(this.buyButton).reason = [I18.f('ARTIFACT_STORE_ABSENT')];
	    return;
	} 
	else if (!store.isWorking) 
	{
	    COM(this.buyButton).reason = [I18.f('ARTIFACT_STORE_NOT_WORKING')];
	    return;
	}
	if (!this.toList && (!this.hero || this.hero.bag.firstAddPosition(this.artifact) < 0) && store.storedArtifacts.firstAddPosition(this.artifact) < 0) 
	{
	    COM(this.buyButton).reason = [I18.f('ARTIFACT_STORE_HAS_NO_PLACE')];
	    return;
	}
}

BuyArtifactDialog.prototype.onBuy = function()
{
	// close must be here, not in success handler
	// because service call makes simult update -- NPE
	//close();
	var toArtifactStore;
	var store = this.estate.findBuilding(BType.ARTIFACT_STORE);
	if (!this.toList) 
	{
	    if (this.hero) 
	    {
	        this.toPosition = this.hero.bag.firstAddPosition(this.artifact);
	        if (this.toPosition >= 0) 
	        {
	            this.toList = this.hero.bag;
	            toArtifactStore = false;
	        }
	    }
	    if (!this.toList) 
	    {
	        this.toPosition = store.storedArtifacts.firstAddPosition(this.artifact);
	        this.toList = store.storedArtifacts;
	        toArtifactStore = true;
	    }
	} 
	else 
	{
	    toArtifactStore = store.storedArtifacts.id == this.toList.id;
	}
	
	APP.showLoader();
	Service.hero.callMethod('buyArtifact', [this.estate.id, this.fromPosition, this.toList.id, this.toPosition, this.artifact.declaration.name],
        $.proxy(function (e)
        {
			APP.hideLoader();
            this.close();
            Alert.create(toArtifactStore ? I18.f('ARTIFACT_BOUGHT_TO_STORE') : I18.f('ARTIFACT_BOUGHT_TO_HERO'));
            
        }, this),
        $.proxy(function (e)
        {
			APP.hideLoader();
            this.close();
            if (e.__className__ == 'HeroException') 
            {
                Alert.create(e.getMessage());
                return true;
            }
            return false;
            
        }, this)
	);
}

BuyArtifactDialog.prototype.onClose = function() 
{
    Events.onFEstateUpdated.remove(this);
}
