function HeroDialog()
{
	callParentConstructor(HeroDialog.prototype, this);
	
	this.__defineGetter__('store', this.getStore);
	this.__defineGetter__('hasStore', this.getHasStore);
	this.__defineGetter__('hasHero', this.getHasHero);
	this.__defineGetter__('hasAlchemy', this.getHasAlchemy);
	this.__defineGetter__('hasAlchemy2', this.getHasAlchemy2);

	this.mainHero = null; // FHero
	this.otherHero = null; // FHero
	this.race = null; // Race
	this.buffs = null; // ArrayCollection
	this.castTarget = null; // CastTarget
	this.isFromCity = null; // Boolean
	this.time = null; // Number
	this.cannotCast = false; // Boolean
	this.army = null; // Army
	this.isInMission = false; // Boolean
	
	Events.onMasterTimerTick.add(this, this.tick);
	this.onClose = $.proxy(function() {	Events.onMasterTimerTick.remove(this); }, this);
}

extendClass(HeroDialog.prototype, UpdatableEstateDialog.prototype);

HeroDialog.prototype.mainHero = null; // FHero
HeroDialog.prototype.otherHero = null; // FHero
HeroDialog.prototype.race = null; // Race
HeroDialog.prototype.buffs = null; // ArrayCollection
HeroDialog.prototype.castTarget = null; // CastTarget
HeroDialog.prototype.isFromCity = null; // Boolean
HeroDialog.prototype.time = null; // Number
HeroDialog.prototype.cannotCast = false; // Boolean
HeroDialog.prototype.army = null; // Army
HeroDialog.prototype.isInMission = false; // Boolean

HeroDialog.popupForCast = function(from, target)
{
	return HeroDialog.createInstance(from, from.garrisonHero, from.garrison, from.owner.race, from.buffs, target, false, NaN, false, false);
}

HeroDialog.create = function(estate, hero, army, race, buffs, time, cannotCast, isInMission)
{
	var target = estate ? CastTarget.fromOwnEstate(estate) : new CastTarget();
	return HeroDialog.createInstance(estate, hero, army, race, buffs, target, true, time, cannotCast, isInMission);
}

HeroDialog.createInstance = function(estate, hero, army, race, buffs, castTarget, isFromCity, time, cannotCast, isInMission)
{
	var instance = COM.create('HeroDialog')
	instance.mainHero = hero;
	instance.army = army;
	instance.race = race;
	instance.buffs = buffs;
	instance.castTarget = castTarget;
	instance.estate = estate;
	instance.isFromCity = isFromCity;
	instance.time = time;
	instance.cannotCast = cannotCast;
	instance.isInMission = isInMission;
	
    if (estate) Events.onFEstateUpdated.add(instance, instance.update);
    instance.update(estate);
    instance.resetTabs();

    instance.popup();
	
	return instance;
}

HeroDialog.prototype.__construct = function()
{
	superMethod(HeroDialog.prototype, '__construct', this)();
	this.fullScreen();
	
	COM.bindExclusiveClick(this.btnClose, $.proxy(this.close, this));
	
	this.tabData = 
	{
		HeroTab: 
		{
			button: COM(this.btnHero),
			enabled: true,
		},
		HeroSkillsTab: 
		{
			button: COM(this.btnSkills),
			enabled: true,
		},
		HeroSpellsTab: 
		{
			button: COM(this.btnSpells),
			enabled: true,
		},
		HeroStoreTab: 
		{
			button: COM(this.btnStore),
			enabled: true,
		},
		HeroTraderTab: 
		{
			button: COM(this.btnTrader),
			enabled: true,
		},
		HeroSetsTraderTab:
		{
			button: COM(this.btnSets),
			enabled: true,
		},
		HeroExchangeTab: 
		{
			button: null,//COM(this.btnExchange),
			enabled: true,
		},
		HeroAlchemyTab: 
		{
			button: COM(this.btnAlchemy),
			enabled: true,
		},
		HeroRuneMasterTab: 
		{
			button: COM(this.btnRuneMaster),
			enabled: true,
		},
	}
	
	for (var cls in this.tabData)
	{
		with (this.tabData[cls])
		{
			if (button)
			{
				COM.bindExclusiveClick(button.DOM, $.proxy(this.onTabClick, this)).attr('tab', cls);
			}
		}
	}
}

HeroDialog.prototype.onTabClick = function(e)
{
	this.showTab(e.currentTarget.getAttribute('tab'));
}

HeroDialog.prototype.tick = function()
{
	if (this.selectedChild)
	{
		this.selectedChild.tick(this.time);
	}
}

HeroDialog.prototype.update = function(estate)
{
	if (this.selectedChild)
	{
		this.selectedChild.race = this.race;
	}
	
	this.estate = estate;
	
	if (estate)
	{
		if (estate.heroes.length) 
		{
			if (this.mainHero)
			{
				this.mainHero = this.estate.findHero(this.mainHero.id);
			}
			if (!this.mainHero)
			{
				this.mainHero = this.estate.heroes[0];
			}
		}
	}
	if (!this.mainHero) this.close();
	
	if (!estate || !estate.accessManageble || !estate.heroes.length) 
	{
	    this.otherHero = null;
	} 
	else 
	{
	    if (this.otherHero) 
	    {
	        this.otherHero = this.estate.findHero(this.otherHero.id);
	    }
	    if (!this.otherHero) 
	    {
	        this.otherHero = this.estate.heroes[0];
	    }
	}
	
	this.updateTabData();
	
	if (this.selectedChild)
	{
		this.selectedChild.update.call(this.selectedChild, this);
	}
	
	this.tick();
}

HeroDialog.findHero = function(heroes, hero)
{
	if (!heroes || !$.isArray(heroes) || heroes.length == 0) return -1;
	for (var i = 0, len = heroes.length; i < len; i++)
	{
		if (hero.id == heroes[i].id)
		{
			return i;
		}
	}
	return -1;
}

HeroDialog.prototype.setMainHero = function(hero)
{
	var idx = HeroDialog.findHero(this.estate.heroes, hero);
	this.mainHero = this.estate.heroes[idx];
	this.update(this.estate);
}

HeroDialog.prototype.switchMainHero = function(dir)
{
	var idx = HeroDialog.findHero(this.estate.heroes, this.mainHero);
	var index = (idx + dir + this.estate.heroes.length) % this.estate.heroes.length;
	this.mainHero = this.estate.heroes[index];
	this.update(this.estate);
}
	
HeroDialog.prototype.setOtherHero = function(hero)
{
	var idx = HeroDialog.findHero(this.estate.heroes, hero);
	this.otherHero = this.estate.heroes[idx];
	this.update(this.estate);
}

HeroDialog.prototype.switchOtherHero = function(dir)
{
	var idx = HeroDialog.findHero(this.estate.heroes, this.mainHero);
	var index = (idx + dir + this.estate.heroes.length) % this.estate.heroes.length;
	this.otherHero = this.estate.heroes[index];
	this.update(this.estate);
}

HeroDialog.prototype.doClose = function(giveAttention)
{
	if (giveAttention && this.estate && this.estate.accessManageble && this.mainHero && this.mainHero.needsAttention) 
	{
	    Service.hero.callMethod('giveAttention', [this.estate.id, this.mainHero.id]);
	}
	this.mainHero = null;
	this.otherHero = null;
	superMethod(HeroDialog.prototype, 'close', this)();
}

HeroDialog.prototype.close = function()
{
	return this.doClose(true);
}
	
HeroDialog.prototype.deleteHero = function(hero)
{
	var heroId = (typeof hero == 'object') ? hero.id : hero;
	var self = this;
	
	var doDeleteHero = function()
	{
    	Service.hero.callMethod('deleteHero', [self.estate.id, heroId], 
    		function(e)
			{
		        if (e && !e.heroes.length)
		        {
		        	self.close();
		        }
		        Alert.create(I18.f('FIRE_HERO_DONE'));
			}, 
    		function(e)
			{
		        console.error('delete hero error', e, self.estate.id, heroId);
		        if (e.__className__ == 'HeroException') 
		        {
		            Alert.create(e.getMessage());
		            return true;
		        }
		        return false;
			}
    	); 
	}
	
	Alert.create(I18.f('FIRE_HERO_CONFIRMATION'), '', doDeleteHero);
}

HeroDialog.prototype.deleteMainHero = function()
{
	this.deleteHero(this.mainHero.id);
}

HeroDialog.prototype.deleteOtherHero = function()
{
	this.deleteHero(this.otherHero.id);
}

//-----------------------------------------------------------------------------
// TABS MANAGEMENT
//-----------------------------------------------------------------------------

HeroDialog.prototype.getStore = function()
{
	return this.estate ? this.estate.findBuilding(BType.ARTIFACT_STORE) : null;
}

HeroDialog.prototype.getHasStore = function()
{
	return Boolean(this.estate && this.estate.accessHideout && this.store && this.store.isWorking);
}

HeroDialog.prototype.getHasHero = function()
{
	return Boolean(this.mainHero);
}

HeroDialog.prototype.getHasAlchemy = function()
{
	return Boolean(FUser.CURRENT_USER.science.getLevel(Science.ALCHEMY) >= 1 || FUser.CURRENT_USER.role.canSpyAndObserve());
}

HeroDialog.prototype.getHasAlchemy2 = function()
{
	return Boolean(FUser.CURRENT_USER.science.getLevel(Science.ALCHEMY) >= 2 || FUser.CURRENT_USER.role.canSpyAndObserve());
}

HeroDialog.prototype.updateTabData = function()
{
	with (this.tabData.HeroTab)
	{
		enabled = this.hasHero;
	}
	
	with (this.tabData.HeroSkillsTab)
	{
		enabled = this.hasHero;
		UI.show(this.hasHero, [button]);
	}
	
	with (this.tabData.HeroSpellsTab)
	{
		enabled = this.hasHero;
		UI.show(this.hasHero, [button]);
	}
	
	with (this.tabData.HeroStoreTab)
	{
		enabled = this.hasStore && this.showStore;
		UI.show(this.hasStore, [button]);
	}
	
	with (this.tabData.HeroTraderTab)
	{
		enabled = this.hasStore;
		UI.show(this.hasStore, [button]);
	}
	
	with (this.tabData.HeroSetsTraderTab)
	{
		enabled = this.hasStore && this.estate.accessManageble && FConfig.SET_TRADER_ENABLED;
		UI.show(this.hasStore, [button]);
	}
	
	with (this.tabData.HeroExchangeTab)
	{
		button = this.selectedTab == 'HeroTab' ? this.selectedChild.btnExchange : null;
		enabled = this.estate && this.estate.accessManageble && this.estate.heroes.length > 1;
	}
	
	with (this.tabData.HeroAlchemyTab)
	{
		button.reason = this.hasAlchemy ? null : I18.f('ALCHEMY_REQUIRED');
		enabled = this.hasStore && this.hasAlchemy;
		UI.show(this.hasStore, [button]);
	}
	with (this.tabData.HeroRuneMasterTab)
	{
		button.reason = this.hasAlchemy2 ? null : I18.f('ALCHEMY2_REQUIRED');
		enabled = this.hasStore && (Science.ALCHEMY.getMaxLevel() >= 2) && this.hasAlchemy2;
		UI.show(this.hasStore && (Science.ALCHEMY.getMaxLevel() >= 2), [button]);
	}
}

HeroDialog.prototype.selectedTab = null;
HeroDialog.prototype.selectedChild = null;
HeroDialog.prototype.resetTabs = function()
{
	if (!this.isFromCity) 
	{
	    this.showTab('HeroSpellsTab');
	} 
	else if (this.mainHero && this.mainHero.needsAttention) 
	{
	    this.showTab('HeroTab');
	}
	else if (!this.selectedTab || !this.tabData[this.selectedTab].enabled)
	{
		for (var cls in this.tabData)
		{
			if (this.tabData[cls].enabled)
			{
				this.showTab(cls);
				break;
			}
		}
	}
}

HeroDialog.prototype.hideTab = function(cls)
{
	if (!this.selectedChild) return;
	if (cls && (this.selectedChild.DOM.nodeName).toUpperCase() == cls.toUpperCase()) return;
	
	this.selectedChild.destroy();
	this.selectedChild = null;
	
	if (this.tabData[this.selectedTab].button)
	{
		this.tabData[this.selectedTab].button.skin = '';
	}
	this.selectedTab = null;
}

HeroDialog.prototype.showTab = function(cls)
{
	if (cls && this.tabData[cls])
	{
		APP.showLoader();
		this.hideTab(cls);
		if (!this.selectedChild)
		{
			this.selectedChild = COM.create(cls);
			
			this.selectedTab = cls;
			this.tabData[this.selectedTab].button.skin = 'active';
			
			this.selectedChild.parentDialog = this;
			
			this.selectedChild.switchHero = $.proxy(this.setMainHero, this);
			this.selectedChild.deleteHero = $.proxy(this.deleteHero, this);
			if (cls == 'HeroExchangeTab')
			{
				this.selectedChild.switchOtherHero = $.proxy(this.setOtherHero, this);
				this.selectedChild.deleteOtherHero = $.proxy(this.deleteHero, this);
			}
			
			this.selectedChild.race = this.race;
			
			this.selectedChild.DOM.style.opacity = 0;
			this.content.appendChild(this.selectedChild.DOM);
			this.selectedChild.update.call(this.selectedChild, this);
			
			this.updateTabData();
			
			this.preloadImages(this.selectedChild.collectImages(), $.proxy(function(){
				setTimeout(APP.hideLoader, 500);
				this.selectedChild.DOM.style.opacity = 1;
			}, this));
		}
		else
		{
			setTimeout(APP.hideLoader, 500);
			//APP.hideLoader();
		}
	}
	return this.selectedChild;
}

HeroDialog.prototype.fitLayout = function()
{
	superMethod(HeroDialog.prototype, 'fitLayout', this).apply(this, Array.prototype.slice.call(arguments, 0));
	this.content.style.height = (this.content.offsetHeight + 30) + 'px';
	if (this.selectedChild)
	{
		this.selectedChild.fitLayout();
	}
}

HeroDialog.prototype.close = function()
{
	Events.onFEstateUpdated.remove(this);
	superMethod(HeroDialog.prototype, 'close', this)();
}

HeroDialog.prototype.preloadImages = function(images, callback)
{
	var cnt = images.length * 1, len = cnt * 1;
	callback = (typeof callback == 'function') ? callback : function(){};
	
	if (len < 1) return callback();
	
	function onLoad() 
	{ 
		if (--cnt == 0) callback(); 
	}
	for (var i = 0; i < len; i++)
	{
		var img = document.createElement('img');
		img.src = images[i];
		(img.complete ? onLoad.call(img) : (img.onload = onLoad)); 
	} 
}
