function HeroRight()
{
	this.__defineGetter__('store', this.getStore);
	this.__defineGetter__('hasStore', this.getHasStore);
	this.__defineGetter__('hasHero', this.getHasHero);
	this.__defineGetter__('hasAlchemy', this.getHasAlchemy);
	this.__defineGetter__('hasAlchemy2', this.getHasAlchemy2);
	
	this.race = null; // Race
	this.switchHero = null; // Function
	this.deleteHero = null; // Function
	
	this.showStore = true; // Boolean
	this.parentDialog = null; // HeroDialog
	
	this.estate = null; // FEstate
	this.mainHero = null; // FHero
	this.otherHero = null; // FHero
	this.army = null; // Army
	this.buffs = null; // ArrayCollection
	this.castTarget = null; // CastTarget
	this.isFromCity = null; // Boolean
	this.time = null; // Number
	
	this.selectedChild = null;
	this.selectedTab = false;
	this.scroll = null; // Number
	
}

HeroRight.prototype.__construct = function()
{
	this.tabData = {
		SkillsView: {
			button: COM(this.skillsViewButton),
			enabled: true,
			updateParams: [],
		},
		SpellBookView: {
			button: COM(this.spellsViewButton),
			enabled: true,
			updateParams: [],
		},
		ArtifactStoreView: {
			button: COM(this.storeViewButton),
			enabled: true,
			updateParams: [],
		},
		ArtifactTraderView: {
			button: COM(this.traderViewButton),
			enabled: true,
			updateParams: [],
		},
		SetTraderView: {
			button: COM(this.setTraderViewButton),
			enabled: true,
			updateParams: [],
		},
		HeroView: {
			button: COM(this.heroViewButton),
			enabled: true,
			updateParams: [],
		},
		AltarView: {
			button: COM(this.alchemyViewButton),
			enabled: true,
			updateParams: [],
		},
	}
	
	for (var cls in this.tabData)
	{
		with (this.tabData[cls])
		{
			COM.bindExclusiveClick(button.DOM, $.proxy(function(e){
				this.showTab(e.currentTarget.getAttribute('tab'));
			}, this))
		}
	}
	
	this.scroll = new iScroll(this.DOM, {
		hScroll: false, vScroll: true, 
		scrollbarColor: '#784f0b',
	});
}

HeroRight.prototype.race = null; // Race
HeroRight.prototype.switchHero = null; // Function
HeroRight.prototype.deleteHero = null; // Function

HeroRight.prototype.showStore = true; // Boolean
HeroRight.prototype.parentDialog = null; // HeroDialog

HeroRight.prototype.estate = null; // FEstate
HeroRight.prototype.mainHero = null; // FHero
HeroRight.prototype.otherHero = null; // FHero
HeroRight.prototype.army = null; // Army
HeroRight.prototype.buffs = null; // ArrayCollection
HeroRight.prototype.castTarget = null; // CastTarget
HeroRight.prototype.isFromCity = null; // Boolean
HeroRight.prototype.time = null; // Number

HeroRight.prototype.getStore = function()
{
	return this.estate ? this.estate.findBuilding(BType.ARTIFACT_STORE) : null;
}

HeroRight.prototype.getHasStore = function()
{
	return Boolean(this.estate && this.estate.accessHideout && this.store && this.store.isWorking);
}

HeroRight.prototype.getHasHero = function()
{
	return Boolean(this.mainHero);
}

HeroRight.prototype.getHasAlchemy = function()
{
	return Boolean(FUser.CURRENT_USER.science.getLevel(Science.ALCHEMY) >= 1 || FUser.CURRENT_USER.role.canSpyAndObserve());
}

HeroRight.prototype.getHasAlchemy2 = function()
{
	return Boolean(FUser.CURRENT_USER.science.getLevel(Science.ALCHEMY) >= 2 || FUser.CURRENT_USER.role.canSpyAndObserve());
}

HeroRight.prototype.firstUpdate = function()
{
	//if (this.parentDialog) $(this.parentDialog.headerRightBox).html(this.buttons);
	UI.show(false, [this.buttons]);
	this.firstUpdate = function(){};
}

HeroRight.prototype.update = function(estate, mainHero, otherHero, army, buffs, castTarget, isFromCity, time)
{
	this.estate = estate;
	this.mainHero = mainHero;
	this.otherHero = otherHero;
	this.army = army;
	this.buffs = buffs;
	this.castTarget = castTarget;
	this.isFromCity = isFromCity;
	this.time = time;
	
	this.firstUpdate();
	this.updateTabData();
	this.resetTabs();
	
	if (this.selectedChild)
	{
		this.selectedChild.update.apply(this.selectedChild, this.tabData[this.selectedTab].updateParams);
	}
	
	if (this.scroll)
	{
		setTimeout($.proxy(this.scroll.refresh, this.scroll), 300);
	}
}

HeroRight.prototype.tick = function(time)
{
	if (this.selectedChild && this.selectedChild['tick'])
	{
		this.selectedChild.tick(time);
	}
}

HeroRight.prototype.updateTabData = function()
{
	with (this.tabData.SkillsView)
	{
		enabled = this.hasHero;
		updateParams = [this.estate, this.mainHero, this.buffs];
	}
	
	with (this.tabData.SpellBookView)
	{
		enabled = this.hasHero;
		updateParams = [this.estate, this.mainHero, this.buffs, this.castTarget, this.parentDialog, this.isFromCity, this.time];
	}
	
	with (this.tabData.ArtifactStoreView)
	{
		enabled = this.hasStore && this.showStore;
		updateParams = [this.estate, this.mainHero, this.parentDialog];
	}
	
	with (this.tabData.ArtifactTraderView)
	{
		enabled = this.hasStore;
		updateParams = [this.estate, this.mainHero, this.parentDialog];
	}
	
	with (this.tabData.SetTraderView)
	{
		enabled = this.hasStore && this.estate.accessManageble && FConfig.SET_TRADER_ENABLED;
		updateParams = [this.estate];
	}
	
	with (this.tabData.HeroView)
	{
		enabled = Boolean(this.otherHero);
		updateParams = [this.estate, this.otherHero, this.army, [], this.parentDialog];
	}
	
	with (this.tabData.AltarView)
	{
		enabled = this.hasStore && this.hasAlchemy;
		updateParams = [this.estate, this.mainHero, this.parentDialog];
	}
	
	for (var tab in this.tabData)
	{
		with (this.tabData[tab])
		{
			UI.show(enabled, [button]);
		}
	}
}

// should be called on creationComplete (not on initialize) due to bug http://bugs.adobe.com/jira/browse/SDK-14157
HeroRight.prototype.resetTabs = function()
{
	if (!this.isFromCity) 
	{
	    this.showTab('SpellBookView');
	} 
	else if (this.mainHero && this.mainHero.needsAttention) 
	{
	    this.showTab('SkillsView');
	}
	else if (!this.selectedTab || !this.tabData[this.selectedTab].enabled)
	{
		for (var cls in this.tabData)
		{
			if (this.tabData[cls].enabled)
			{
				this.showTab(cls);
				break;
			}
		}
	}
}

HeroRight.prototype.hideTab = function(cls)
{
	if (!this.selectedChild) return;
	if (cls && (this.selectedChild.DOM.nodeName).toUpperCase() == cls.toUpperCase()) return;
	
	COM(this.selectedChild).destroy();
	
	this.selectedChild = null;
	
	this.tabData[this.selectedTab].button.skin = '';
	this.selectedTab = false;
}

HeroRight.prototype.showTab = function(cls)
{
	this.hideTab(cls);
	
	if (this.selectedChild) 
	{
		return this.selectedChild;
	}
	if (!cls || !this.tabData[cls])
	{
		return;
	}
	
	this.selectedChild = COM.create(cls);
	
	this.selectedTab = cls;
	this.tabData[this.selectedTab].button.skin = 'yellow';
	
	if (cls == 'SetTraderView')
	{
		this.selectedChild.reset();
	}
	if (cls == 'HeroView')
	{
		this.selectedChild.race = this.race;
		this.selectedChild.switchHero = this.switchHero;
		this.selectedChild.deleteHero = this.deleteHero;
	}
	this.selectedChild.parentDialog = this;
	this.selectedChild.update.apply(this.selectedChild, this.tabData[cls].updateParams);
	
	$(this.tabs).append(this.selectedChild.DOM);
	
	if (this.scroll)
	{
		this.scroll.refresh();
		this.scroll.scrollTo(0, 0, 0);
	}
	
	return this.selectedChild;
}
