function HeroView()
{
	this.tabVisible = true; // Boolean
	this.tabToolTip = I18.f('HERO'); // String
	
	this.estate = null; // FEstate
	this.army = null; // Army
	this.race = null; // Race
	this.hero = null; // FHero
	this.switchHero = null; // Function
	this.deleteHero = null; // Function
}

HeroView.prototype.__construct = function()
{
	COM.bindExclusiveClick(this.prevHero, $.proxy(function(){ return this.switchHero(-1); }, this));
	COM.bindExclusiveClick(this.nextHero, $.proxy(function(){ return this.switchHero(1); }, this));
	COM.bindExclusiveClick(this.manaRegen, $.proxy(this.onRegenToolTip, this));
}

HeroView.prototype.tabVisible = true; // Boolean
HeroView.prototype.tabToolTip = I18.f('HERO'); // String

HeroView.prototype.estate = null; // FEstate
HeroView.prototype.army = null; // Army
HeroView.prototype.race = null; // Race
HeroView.prototype.hero = null; // FHero
HeroView.prototype.switchHero = null; // Function
HeroView.prototype.deleteHero = null; // Function

// @param buffs - buffs on hero, are passed from outside : could be mission buffs, combat buffs, estate buffs
HeroView.prototype.update = function(estate, hero, army, buffs, parentDialog)
{
	if (this.hero && this.hero.id != hero.id) 
	{
	    COM(this.bagView).reset();
	    COM(this.castsView).reset();
	}
	
	this.hero = hero;
	this.army = army;
	this.estate = estate;
	
	//this.heroFace.title = canManage ? I18.f('RENAME_HERO_TIP') : "";
	COM(this.heroFace).update(hero, this.race);
	this.nameLabel.innerHTML = hero.name;
	$(this.descLabel).html(I18.f('HERO_DESCRIPTION', I18.getHeroClassName(hero.heroClass), hero.level));
	
	var canManage = this.estate && this.estate.accessManageble;
	
	COM(this.heroFace).onHeroClick = canManage ? $.proxy(this.showHeroRenameDialog, this) : null;
	
	COM(this.xpProgress).update(this.hero.currentLevelExp, this.hero.nextLevelExp);
    COM(this.xpProgress).toolTip1 = this.hero.maxLevelReached ? this.onMaxLevelTooltip : this.onExpToolTip;
	
	COM(this.dollView).update(this.estate, this.hero, buffs, parentDialog);
	COM(this.bagView).update(this.estate, this.hero, parentDialog);
	
	var castedSpells = Arrays.filter(this.hero.casts, function (c)
	{
	    return c.isVisible;
	});
	
	COM(this.castsView).update(this.estate, this.hero, castedSpells, FBuff.hasMagicDisable(buffs, FEstate.tickNowForEstate(this.estate)));
	UI.show(this.estate, [this.prevHero, this.nextHero]);
	this.tick();
}

HeroView.prototype.onMaxLevelTooltip = function(e)
{
	//var tip:Tip = new Tip();
	//tip.addLine(this.hero.level == FConfig.HERO_MAX_LEVEL ? I18.f('HERO_MAX_LEVEL_TOTAL') : I18.f('HERO_MAX_LEVEL_SCIENCE'));
	//e.toolTip = tip;
}

HeroView.prototype.updateManaPool = function(current, maximum)
{
	COM(this.manaPoolTrack).update(current, maximum);
}

HeroView.prototype.onMaxManaToolTip = function(e)
{
	//e.toolTip = NumberEffectTip.createTip("MAX_MANA", this.hero.buffs, 0, BuffType.HERO_MAX_MANA, FEstate.tickNowForEstate(this.estate));
}

HeroView.prototype.onManaToolTip = function(e)
{
	//e.toolTip = Tip.createGrid(UI.styledText(I18.f('MANA'), Style.BOLD));
}

HeroView.prototype.onMaxExpToolTip = function(e)
{
	//e.toolTip = Tip.createGrid(UI.styledText(I18.f('MAX_EXPERIENCE'), Style.BOLD));
}

HeroView.prototype.onExpToolTip = function(e)
{
	//e.toolTip = Tip.createGrid(UI.styledText(I18.f('EXPERIENCE'), Style.BOLD));
}

HeroView.prototype.onRegenToolTip = function()
{
	var tip;
	if (this.hero.manaRegenPerHour != 0) 
	{
	    tip = NumberEffectTip.createManaRegenTip(this.hero.buffs, this.army, FEstate.tickNowForEstate(this.estate));
	} 
	else if (this.hero.isMaxManaReached()) 
	{
	    tip = I18.f('MAX_MANA_LIMIT_REACHED');
	} 
	else 
	{
		tip = document.createElement('div');
	    if (!this.estate) 
	    {
	        $(tip).html(I18.f('MANA_REGEN_IN_MISSION', hero.manaRegenPerHour)).addClass('negative');
	    } 
	    else if (this.hero.isUpset()) 
	    {
	        if (this.hero.upsetByFight) 
	        {
	        	$(tip).html(I18.f('MANA_REGEN_UPSET', hero.manaRegenPerHour)).addClass('negative');
	        }
	        if (this.hero.upsetByTitle) 
	        {
	        	$(tip).html(I18.f('HERO_ERROR_REQUIRED_MAX_TITLE')).addClass('negative');
	        }
	    } 
	    else if (!this.estate.garrisonHero || this.estate.garrisonHero.id != this.hero.id) 
	    {
        	$(tip).html(I18.f('MANA_REGEN_IN_HIDEOUT', hero.manaRegenPerHour)).addClass('negative');
	    }
	}
    
    return Alert.create(tip, I18.f('MANA_REGEN'));
}

HeroView.prototype.onBagToolTip = function(e)
{
	//e.toolTip = NumberEffectTip.createTip("HERO_BAG_CAPACITY", this.hero.buffs, FConfig.HERO_BASE_BAG_SIZE, BuffType.HERO_BAG_SIZE, FEstate.tickNowForEstate(this.estate));
}

HeroView.prototype.tick = function()
{
	this.updateManaPool(NumberUtils.floor(this.hero.mana), this.hero.maxMana);
	this.manaRegen.innerHTML = I18.flow('MANA_REGEN_NUMBER', this.hero.manaRegenPerHour);
	var count = 0;
	for(var i = 0, len = this.hero.bag.artifacts.length; i < len; i++) 
	{
	    if (this.hero.bag.artifacts[i]) 
	    {
	        count++;
	    }
	}
	
	this.bagSizeLabel1.innerHTML = count;
	this.bagSizeLabel2.innerHTML = this.hero.bagSize;
}

HeroView.prototype.showHeroRenameDialog = function(h, r)
{
	var d = RenameHeroDialog.create(this.estate, this.hero, this.race, this);
	
	d.onDelete = $.proxy(function(){
		
		d.close();
		this.deleteHero();
		
	}, this);
}