function HireHeroView()
{
	this.hero = null;
	this.estate = null;
	
	this._selected = false;
	this.__defineGetter__('selected', function()
	{
		return this._selected;
	});
	this.__defineSetter__('selected', function(val)
	{
		val = Boolean(val);
		if (this._selected == val) return;
		
		this._selected = val;
		$(this.DOM).toggleClass('selected', val);
		if (val && typeof this.onSelect == 'function')
		{
			this.onSelect(this);
		}
	});
	
	this.__defineGetter__('label', function()
	{
		return this.faceLabel.innerHTML;
	});
	this.__defineSetter__('label', function(val)
	{
		$(this.faceLabel).html(val);
		UI.show(Boolean(this.faceLabel.innerHTML != ''), [this.faceLabel.parentNode]);
	});

	this._race = null;
	this.__defineGetter__('race', function()
	{
		return this._race;
	});
	this.__defineSetter__('race', function(val)
	{
		this._race = val;
		
		var back = $(this.heroBack);
		for (var i = 0, len = Race.ALL.length; i < len; i++)
		{
			var cls = Race.ALL[i].name.toLowerCase();
			back.toggleClass(cls, (Race.ALL[i].name == this.race.name));
		}
	});

}

HireHeroView.prototype.hero = null;
HireHeroView.prototype.estate = null;
HireHeroView.prototype.race = null;

HireHeroView.prototype._selected = false;
HireHeroView.prototype.onSelect = null;

HireHeroView.prototype.__construct = function()
{
	COM(this.costBox).alwaysShow = [Resource.GOLD];
	COM.bindExclusiveClick(this.hireButton, $.proxy(this.onHire, this));
	COM.bindExclusiveClick(this.heroBox, $.proxy(this.showHeroDoll, this));
}

HireHeroView.prototype.update = function(estate, h)
{
    this.estate = estate;

    if (h != null) 
    {
        if (!this.hero || (h.id != this.hero.id)) this.selected = false;
        
        this.hero = h;
        this.heroImage.src = STATIC_INTERFACE_URL + CommonHeroAssets[h.face];
        this.heroImage.style.cursor = 'pointer';
        
        this.nameLabel.innerHTML = h.name;
        //this.descLabel.innerHTML = I18.f('HERO_DESCRIPTION', I18.getHeroClassName(this.hero.heroClass), this.hero.level);
        this.descLabel.innerHTML = I18.getHeroClassName(this.hero.heroClass);
		this.label = I18.f('LEVEL_X', this.hero.level);
        this.specLabel.innerHTML = Arrays.join(Arrays.map(this.hero.getSpecBranches(), function(b) 
        {
            return I18.getBranchName(b.name);
            
        }), "/");
        
    	UI.show(true, [this.hireCostBox, this.hireButton]);
        COM(this.costBox).updateAll(h.hireCost, estate.bank, estate.owner.race);
    	COM(this.hireButton).enabled = true;
    	COM(this.hireButton).reason = this.getReason();
    } 
    else 
    {
        this.clear();
    }
}

HireHeroView.prototype.clear = function()
{
    this.heroImage.src = STATIC_INTERFACE_URL + Assets.getLockedHero(FUser.CURRENT_USER.race);
    this.heroImage.style.cursor = 'default';
    this.nameLabel.innerHTML = '';
    this.descLabel.innerHTML = '';
    this.specLabel.innerHTML = '';
	this.label = '';
	this.selected = false;
    UI.show(false, [this.hireCostBox, this.hireButton]);
    COM(this.hireButton).enabled = false;
    this.hero = null;
}

HireHeroView.prototype.showHeroDoll = function(e)
{
    if (this.hero) 
    {
        HeroDialog.create(null, this.hero, this.estate.garrison, this.estate.owner.race, [], NaN, true);
    }
    if (e)
    {
    	e.stopImmediatePropagation();
    	e.preventDefault();
    }
    return false;
}

HireHeroView.prototype.getReason = function()
{
    var reason = null;
    if (this.estate.isEstateUnderAttack) 
    {
        return I18.getMegaString(I18.f('HERO_ERROR_HIRE_UNDER_ATTACK'), FConfig);
    }
    if (!FUser.CURRENT_USER.hasTitleToHireHero) 
    {
        return I18.f('HERO_ERROR_HIRE_REQUIRED_MAX_TITLE');
    } 
    if (!FUser.CURRENT_USER.hasOneMoreHeroScienceSlot) 
    {
        return I18.f('HERO_ERROR_HIRE_MAX_HEROES');
    } 
    if (!this.estate.bank.greaterOrEqualsTo(this.hero.hireCost)) 
    {
        return I18.f('NOT_ENOUGH_RESOURCES');
    } 
    return null;
}

HireHeroView.prototype.onHire = function()
{
    if (this.getReason()) return false;
    
    var heroId = this.hero.id;
    Service.hero.callMethod('hireHero', [this.estate.id, heroId], 
    	$.proxy(function(e){
			
			Alert.create((e.garrisonHero && e.garrisonHero.id == heroId)
				? I18.f('HIRE_RESULT_GARRISON')
				: I18.f('HIRE_RESULT_HIDEOUT')
			);
			
    	}, this),
    	$.proxy(function(){
    		
    		Alert.create(e.message, I18.f('ERROR'));
    		
    	}, this)
    );
}
