function SetTraderView()
{
	this.tabVisible = true; // Boolean
	this.tabToolTip = I18.f('SET_TRADER'); // String
	
	this.needRefresh = true; // Boolean
	
	this.estate = null; // FEstate
	this.sets = null; // ArrayCollection //ArrayCollection<ArrayCollection<FArtifact>>
	this.setNames = null; // Array //<String>
	this.setLevels = null; // Array //<String>
	this.price = null; // Number
	this.priceWithDiscount = null; // Number
	
	this.__lastIndex = 0;
}

SetTraderView.prototype.__construct = function()
{
	$(this.setsCombo).on('change', $.proxy(this.onSelectSet, this));
	COM.bindExclusiveClick(this.buyButton, $.proxy(this.onBuyClicked, this));
}

SetTraderView.prototype.tabVisible = true; // Boolean
SetTraderView.prototype.tabToolTip = I18.f('SET_TRADER'); // String

SetTraderView.prototype.needRefresh = true; // Boolean

SetTraderView.prototype.estate = null; // FEstate
SetTraderView.prototype.sets = null; // ArrayCollection //ArrayCollection<ArrayCollection<FArtifact>>
SetTraderView.prototype.setNames = null; // Array //<String>
SetTraderView.prototype.setLevels = null; // Array //<String>
SetTraderView.prototype.price = null; // Number
SetTraderView.prototype.priceWithDiscount = null; // Number

SetTraderView.prototype.__lastIndex = 0;

SetTraderView.prototype.tryRefreshSetList = function()
{
	if (!this.estate || !this.estate.accessManageble || !this.needRefresh || !this.estate.findBuilding(BType.ARTIFACT_STORE)) 
	{
	    return;
	}
	this.needRefresh = false;
	
	Service.hero.callMethod('getSetsForTrader', [this.estate.id], $.proxy(this.onSetsForTrader, this));
}

SetTraderView.prototype.onSetsForTrader = function(setsList)
{
    this.sets = [];
    for (var i in setsList) if (!isNaN(parseInt(i))) this.sets.push(setsList[i]);
    
    this.setLevels = [];
    this.setNames = [];
    var forCombo = [];
    
    for (var i = 0; i < this.sets.length; i++) 
    {
        var decl = this.sets[i][0].declaration;
        this.setLevels.push(decl.level);
        this.setNames.push(decl.setName);
        forCombo.push(Texts.artifactSetName(decl.setName) + I18.f('ARTIFACT_LEVEL', decl.level));
    }
    COM(this.setsCombo).dataProvider = forCombo;
    if (this.sets.length > 0) 
    {
        COM(this.setsCombo).selectedIndex = Math.min(this.__lastIndex, this.sets.length);
        this.onSelectSet();
    }
}

SetTraderView.prototype.update = function(estate)
{
	this.estate = estate;
	COM(this.credit).update(this.estate.owner.artifactTraderCredit);
	this.creditText.title = I18.mega(I18.f('CREDIT_TIP'), FConfig.BUY_ARTIFACT_MAX_CREDIT_PART);
	UI.show(this.estate.accessManageble, [this.creditBox]);
	COM(this.newPrice).update(this.priceWithDiscount);
	this.tryRefreshSetList();
}

SetTraderView.prototype.onSelectSet = function()
{
	var idx = Math.max(0, COM(this.setsCombo).selectedIndex);
	this.__lastIndex = idx;
	this.displaySet(this.sets[idx]);
}

SetTraderView.prototype.displaySet = function(setList)
{
	for (var i = 0, len = this.dollView.children.length; i < len; i++)
	{
		COM(this.dollView.children[i]).update(this.estate, null, null, i, null, [], null);
	}
	
	this.price = 0;
	for (var i = 0, len = setList.length; i < len; i++)
	{
		var art = setList[i];
	    var index = art.declaration.parts[0].ordinal();
	    COM(this.dollView.children[index]).update(this.estate, null, null, index, art, [], null);
	    this.price += art.declaration.cost.pearl;
	}
	
	COM(this.rightHand).setMirrorMode(COM(this.leftHand).hasArtifact && COM(this.leftHand).artifact.declaration.twoHanded, COM(this.leftHand).artifact);
	COM(this.leftHand).setMirrorMode(COM(this.rightHand).hasArtifact && COM(this.rightHand).artifact.declaration.twoHanded, COM(this.rightHand).artifact);
	this.priceWithDiscount = this.price * (1 - FConfig.ARTIFACT_SETS_DISCOUNT);
	COM(this.newPrice).update(this.priceWithDiscount);
}

SetTraderView.prototype.onBuyClicked = function()
{
	var idx = COM(this.setsCombo).selectedIndex;
	BuySetDialog.create(this.estate, this.setNames[idx], this.setLevels[idx], this.price, this.priceWithDiscount);
}

SetTraderView.prototype.alwaysGreen = function(price)
{
	return true;
}

SetTraderView.prototype.checkPriceWithCredit = function(price)
{
	var creditValue = Math.min(this.estate.owner.artifactTraderCredit, NumberUtils.floor(this.price * FConfig.BUY_ARTIFACT_MAX_CREDIT_PART));
	return this.price - creditValue <= FUser.CURRENT_USER.pearl;
}

SetTraderView.prototype.priceTip = function(event)
{
	/*
	var tip:Tip = Tip.create();
	UIh.addLine(tip, PearlPrice.create(price, I18.OLD_PRICE.f()));
	UIh.addLine(tip, PearlPrice.create(price - priceWithDiscount, I18.f('YOU_SAVE')));
	event.toolTip = tip;
	*/
}

SetTraderView.prototype.reset = function()
{
	this.needRefresh = true;
}
