function SkillTip()
{
	this.branch = null; // SkillBranch
	this.skillIndex = null; // int
	this.hero = null; // FHero
	this.doesntWork = null; // Array
}

SkillTip.prototype.__construct = function()
{
	this.nextLevelTitle.innerHTML = I18.f('SCIENCE_NEXT_LEVEL');
}

SkillTip.skillTip = null; // SkillTip
SkillTip.prototype.branch = null; // SkillBranch
SkillTip.prototype.skillIndex = null; // int
SkillTip.prototype.hero = null; // FHero
SkillTip.prototype.doesntWork = null; // Array

SkillTip.alert = function(branch, skillIndex, hero, doesntWork, actions)
{
	var tip = SkillTip.create(branch, skillIndex, hero, doesntWork);
	
    var decls = FConfig.branches.getValueOrNull(branch.name);
    var decl = !decls || skillIndex >= decls.length ? null : decls[skillIndex];
    
	var caption = APP.isHeropedia ? I18.f('BUFF_CAUSE_PASSIVE_OWN') : I18.f('SKILL_NAME');
	var a = TipDialog.createWithActions(tip.DOM.outerHTML, caption, actions);
	
	tip.DOM = a.DOM.querySelector('SkillTip');
	a.tip = tip;
	
	tip.skillView = tip.DOM.querySelector('SkillView');
	COM(tip.skillView).DOM = tip.DOM.querySelector('SkillView');
	
	return a;
}

SkillTip.create = function(branch, skillIndex, hero, doesntWork)
{
	var tip = COM.create('SkillTip');
	tip.update(branch, skillIndex, hero, doesntWork);
	return tip;
}

SkillTip.prototype.update = function(branch, skillIndex, hero, doesntWork)
{
	this.branch = branch;
	this.skillIndex = skillIndex;
	this.hero = hero;
	this.doesntWork = doesntWork;
	
	this.passiveHolder.innerHTML = '';
	this.spellHolder.innerHTML = '';
	this.nextBuffs.innerHTML = '';
	this.nextSpells.innerHTML = '';
	
	var level = this.branch.levels[this.skillIndex];
	var decls = FConfig.branches.getValue(this.branch.name);
	var decl = (!decls || this.skillIndex >= decls.length) ? null : decls[this.skillIndex];
	var skillName = decl ? decl.name : null;
	
	var curSpellLevel = FConfig.getSpell(skillName, level); // SpellLevelDecl
	var curPassiveLevel = FConfig.getPassive(skillName, level); // ActionsDecl
	var maxLevel = FConfig.getMaxSkillLevel(decl);
	var nextSpellLevel = maxLevel > level ? FConfig.getSpell(skillName, level + 1) : null; // SpellLevelDecl
	var nextPassiveLevel = maxLevel > level ? FConfig.getPassive(skillName, level + 1) : null; // ActionsDecl
	
	var heropedia = APP.isHeropedia; // FlashVars.isHeropedia;
	if (UI.show(curSpellLevel || curPassiveLevel, [this.curLevelBox]))
	{
	    this.fillLevel(this.passiveHolder, this.spellHolder, skillName, curSpellLevel, curPassiveLevel);
	    if (heropedia) 
	    {
	    	this.fillDepends(this.deps, decl, level, branch, true);
	    	this.fillMinLevel(this.minLevel, decl, level, true);
	    }
	}
	
	if (UI.show(!heropedia && (nextSpellLevel || nextPassiveLevel), [this.nextLevelBox]))
	{
	    this.fillLevel(this.nextBuffs, this.nextSpells, skillName, nextSpellLevel, nextPassiveLevel);
	    this.fillDepends(this.nextDeps, decl, level + 1, this.branch);
	    this.fillMinLevel(this.nextMinLevel, decl, level + 1);
	}
	
	COM(this.doesntWorkText).update(doesntWork);
	
	var spellDecl = skillName ? FConfig.spells.getValueOrNull(skillName) : null;
	this.nameLabel.innerHTML = (spellDecl ? I18.f('SPELL_NAME') + " " : "") + Texts.skillName(skillName);
	
	COM(this.skillView).update(branch, skillIndex, null, hero, null, false);
	UI.show(false, [COM(this.skillView).nameLabel]);
}

SkillTip.safeGet = function(a, index)
{
	return  a && index < a.length ? a[index] : null;
}

SkillTip.prototype.fillLevel = function(passive, spell, skillName, spellLevelDecl, actionsDecl)
{
	if (UI.show(actionsDecl && actionsDecl.dtoActions.length > 0, [passive]))
	{
	    var passiveDesc = COM.create('PassiveDescription');
		passiveDesc.update(actionsDecl);
	    passive.appendChild(passiveDesc.DOM);
	}
	
	if (UI.show(spellLevelDecl && spellLevelDecl.allActions && spellLevelDecl.allActions.length > 0, [spell]))
	{
	    var spellDesc = COM.create('SpellDescription');
		spellDesc.update(skillName, spellLevelDecl, false, false);
	    spell.appendChild(spellDesc.DOM);
	}
}

SkillTip.prototype.fillDepends = function(label, decl, level, skillBranch, showAll)
{
	var depsText = this.getDependsText(decl, level, skillBranch, showAll);
	$(label).removeClass('negative');
	if (UI.show(depsText, [label]))
	{
	    label.innerHTML += I18.htmlFlow(I18.f('SCIENCE_REQUIRES') + " " + depsText);
	    $(label).addClass('negative');
	}
}

SkillTip.prototype.getDependsText = function(decl, level, skillBranch, showAll)
{
	var deps = decl.dependencies.getValueOrNull(level);
	if (!deps) 
	{
	    return null;
	}
	if (!showAll) 
	{
	    deps = skillBranch.getUnsatisfiedDeps(deps);
	}
	if (deps.size == 0) 
	{
	    return null;
	}
	
	var ret = Arrays.map(deps.keys, function(skillName) 
	{ 
		return Texts.skillName(skillName) + " " + deps.getValue(skillName);
	});
	return ret.join(', ');
}

SkillTip.prototype.fillMinLevel = function(label, decl, level, showAll)
{
    var minLevel = decl.minLevels.getValueOrDefault(level, 0);
    if (!showAll && minLevel <= this.hero.level) 
    {
        minLevel = 0;
    }
    
    if (UI.show(minLevel > 0, [label]))
    {
		label.innerHTML = I18.htmlFlow(I18.f('MIN_LEVEL') + " " + minLevel);
		$(label).addClass('negative');
    }
}