function SkillView()
{
	this.branch = null; // SkillBranch
	this.skillIndex = null; // int
	this.level = null; // int
	this.hero = null; // FHero
	this.estate = null; // FEstate
	this.buffs = null; // Array
	this.researchable = null; // Boolean
}

SkillView.prototype.__construct = function()
{
	$(this.DOM).one(CLICK_EVENT, $.proxy(this.onToolTip, this));
}

SkillView.prototype.branch = null; // SkillBranch
SkillView.prototype.skillIndex = null; // int
SkillView.prototype.level = null; // int
SkillView.prototype.hero = null; // FHero
SkillView.prototype.estate = null; // FEstate
SkillView.prototype.buffs = null; // Array
SkillView.prototype.researchable = null; // Boolean

SkillView.prototype.tipDialog = null; // SkillTip

SkillView.prototype.onToolTip = function(e)
{
	if (!this.tipDialog && this.enabled && !Quarantine.contains('SkillView.assignSkillPoint'))
	{
	    var actions = {
	    	"right": {
	    		"caption": '', 
	    		"skin": 'green', 
	    		"icon": STATIC_INTERFACE_URL + 'shared/heroes/button_up_skill.png', 
	    		"onclick": this.assignSkillPoint, 
	    		"context": this 
	    	} 
	    }
	    if (!this.estate || !this.researchable || this.hero.skillPointsLeft <= 0) actions = null;
	    
	    this.tipDialog = SkillTip.alert(this.branch, this.skillIndex, this.hero, this.getDoesntWork(), actions);
	    this.tipDialog.onClose = $.proxy(function(){
	    	this.tipDialog = null;
	    }, this);
	}
	$(this.DOM).one(CLICK_EVENT, $.proxy(this.onToolTip, this));
}

SkillView.prototype.assignSkillPoint = function()
{
	if (this.enabled && this.estate && !Quarantine.contains('SkillView.assignSkillPoint')) 
	{
	    Quarantine.put('SkillView.assignSkillPoint');
	    $(this.DOM).closest('SkillsView').addClass('grayscale').css({'opacity': 0.5});
	    if (this.researchable && this.hero.skillPointsLeft > 0) 
	    {
	        APP.showLoader();
	        Service.hero.callMethod('assignSkillPoint', [this.estate.id, this.hero.id, this.branch.name, this.skillIndex],
                $.proxy(this.onAssignSkillPoint, this), $.proxy(this.onAssignError, this)
	        );
	    }
	}
}

SkillView.prototype.onAssignSkillPoint = function(e)
{
    APP.hideLoader();
    $(this.DOM).closest('SkillsView').removeClass('grayscale').css({'opacity': 1.0});
    if (this.tipDialog)
    {
    	this.tipDialog.close();
    }
}

SkillView.prototype.onAssignError = function(e)
{
    APP.hideLoader();
    $(this.DOM).closest('SkillsView').removeClass('grayscale').css({'opacity': 1.0});
    if (GameException.isDisplayable(e)) 
    {
        Alert.create(e.getMessage());
        return true;
    }
    return false;
}

SkillView.prototype.getDoesntWork = function()
{
	var res = [];
	// only for passives
	var decls = FConfig.branches.getValueOrNull(this.branch.name);
	var decl = !decls || this.skillIndex >= decls.length ? null : decls[this.skillIndex];
	var skillName = decl ? decl.name : null;
	var passiveDecl = skillName ? FConfig.passives.getValueOrNull(skillName) : null;
	
	if (passiveDecl) 
	{
	    if (this.hero && this.hero.isUpset()) 
	    {
	        res.push(this.hero.upsetByTitle ? I18.f('HERO_ERROR_REQUIRED_MAX_TITLE') : I18.f('DOESNT_WORK_UPSET_HERO'));
	    }
	    if (this.hero && this.estate && !this.estate.isHeroInGarrison(this.hero.id)) 
	    {
	        res.push(this.estate.isHeroInHideout(this.hero.id) ? I18.f('HERO_IN_HIDEOUT') : I18.f('DOESNT_WORK_NON_GARRISON_HERO'));
	    }
	    if (FBuff.hasMagicDisable(this.buffs, FEstate.tickNowForEstate(this.estate))) 
	    {
	        res.push(I18.f('DOESNT_WORK_MAGIC_DISABLE'));
	    }
	    if (res.length > 0) 
	    {
	        res.unshift(I18.f('SKILL_DOESNT_WORK'));
	    }
	}
	if (this.estate && this.estate.accessManageble && this.estate.isEstateUnderAttack) 
	{
	    res.push(I18.getMegaString(I18.f('HERO_ERROR_CANT_ASSIGN_UNDER_ATTACK'), FConfig));
	}
	return res;
}

SkillView.prototype.update = function(branch, skillIndex, estate, hero, buffs, greyscaleZero)
{
	if (typeof greyscaleZero == 'undefined') greyscaleZero = true;
	
	this.branch = branch;
	this.skillIndex = skillIndex;
	this.level = branch.levels[this.skillIndex];
	this.hero = hero;
	this.estate = estate;
	this.buffs = buffs;
	
	UI.show(!this.branch.isDummy, [this.skillImage, this.levelBox, this.nameLabel]);
	$(this.DOM).toggleClass('dummy', !!this.branch.isDummy);
	
	if (this.branch.isDummy) 
	{
	    var src = Assets.getSkillStub(hero.heroClass);
	    if (src)
	    {
		    this.borderImage.src = STATIC_INTERFACE_URL + Assets.getSkillStub(hero.heroClass);
		    this.enabled = false;
		    this.DOM.style.cursor = 'default';
	    }
	    else
	    {
	    	UI.show(false, [this.borderImage]);
	    }
	} 
	else 
	{
	    var decls = FConfig.branches.getValueOrNull(this.branch.name);
	    var decl = !decls || this.skillIndex >= decls.length ? null : decls[this.skillIndex];
	
	    var isPassive = decl && FConfig.passives.getValueOrNull(decl.name);
	
	    this.borderImage.src = STATIC_INTERFACE_URL + (isPassive ? SkillAssets.SKILL_BORDER : SkillAssets.SKILL_ACTIVE_BORDER);
	    $(this.DOM).toggleClass('skill-active', !isPassive);
	    this.enabled = true;
	
	    var pic = (decl ? Assets.getSkillImage(decl.name) : null);
	    if (pic == SkillAssets.SKILL_UNKNOWN)
	    {
	    	$(this.DOM).addClass('skill-unknown');
	    }
	    this.skillImage.src = STATIC_INTERFACE_URL + pic;
	    $(this.DOM).toggleClass('grayscale', Boolean(this.level == 0 && this.greyscaleZero)); 
	
	    this.researchable = hero && this.isResearchable(decls, this.skillIndex, this.branch) && this.estate && this.estate.accessManageble && !this.estate.isEstateUnderAttack;
		this.DOM.style.cursor = this.researchable && this.hero.skillPointsLeft > 0 && this.estate ? 'pointer' : 'default';
	    var maxLevel = decl ? FConfig.getMaxSkillLevel(decl) : 0;
	    
	    var border = this.estate ? 'red' : '';
	    if (maxLevel <= this.level && maxLevel != 0) 
	    {
	        border = 'blue';
	    } 
	    else if (this.researchable) 
	    {
	        border = '';
	    }
	    
	    $(this.levelBox).removeClass('red blue');
	    if (border) $(this.levelBox).addClass(border);
		
	    this.curLevel.innerHTML = this.level + "/" + maxLevel;
	    
	    this.nameLabel.innerHTML = Texts.skillName(decl.name);
	}
}

SkillView.prototype.getMaxLevel = function(skillName)
{
	var levels = FConfig.spells.getValueOrNull(skillName);
	if (levels) 
	{
	    return levels.size;
	}
	
	var lvls = FConfig.passives.getValueOrNull(skillName);
	if (lvls) 
	{
	    return lvls.size;
	}
	
	return 0;
}

SkillView.prototype.isResearchable = function(decls, skillIndex, branch)
{
	var nextLevel = branch.levels[skillIndex] + 1;
	var decl = decls.length > skillIndex ? decls[skillIndex] : null;
	if (!decl) return false;
	
	var levelDecl = decl.dependencies ? decl.dependencies.getValueOrNull(nextLevel) : null;
	if (levelDecl && !levelDecl.isEmpty()) 
	{
	    for (var i = 0, len = levelDecl.raw.length; i < len; i++) 
	    {
	    	var skillName = levelDecl.raw[i].key;
	        var requredLevel = levelDecl.getValue(skillName);
	        var curLevel = branch.getLevel(skillName);
	        if (curLevel < requredLevel) 
	        {
	            return false;
	        }
	    }
	}
	var minLevel = decl.minLevels.getValueOrDefault(nextLevel, 0);
	return (nextLevel <= FConfig.getMaxSkillLevel(decl)) && (minLevel <= this.hero.level);
}
