function SpellBookView()
{
	this.tabVisible = true; // Boolean
	this.tabToolTip = I18.f('CASTS'); // String
	
	this.groupedSpells = null; // Array
	this.estate = null; // FEstate
	this.hero = null; // FHero
	this.buffs = null; // Array
	this.target = null; // CastTarget
	this.parentDialog = null; // HeroDialog
	this.fromCity = null; // Boolean
	this.time = null; // Number
	
	this.selectedView = null; // SpellView
}


SpellBookView.prototype.__construct = function()
{
}

SpellBookView.PAGE_SIZE = 100;

SpellBookView.prototype.tabVisible = true; // Boolean
SpellBookView.prototype.tabToolTip = I18.f('CASTS'); // String

SpellBookView.prototype.groupedSpells = null; // Array
SpellBookView.prototype.estate = null; // FEstate
SpellBookView.prototype.hero = null; // FHero
SpellBookView.prototype.buffs = null; // Array
SpellBookView.prototype.target = null; // CastTarget
SpellBookView.prototype.parentDialog = null; // HeroDialog
SpellBookView.prototype.fromCity = null; // Boolean
SpellBookView.prototype.time = null; // Number

SpellBookView.prototype.selectedView = null; // SpellView

SpellBookView.prototype.update = function(estate, hero, buffs, target, parentDialog, isFromCity, time)
{
	this.estate = estate;
	this.time = time;
	
	if (!this.hero || !hero || this.hero.id != hero.id) 
	{
	    this.selectedView = null;
	}
	this.hero = hero;
	
	this.target = target;
	this.parentDialog = parentDialog;
	this.fromCity = isFromCity;
	this.buffs = buffs;
	this.groupedSpells = [];
	
	for(var i = 0, spells = hero.getSpells(), len = spells.length; i < len; i++) 
	{
		var s = spells[i], added = false;
	    if (s.fromArtifact) 
	    {
			for(var j = 0, len2 = this.groupedSpells.length; j < len2; j++) 
	        {
	            var gSpell = this.groupedSpells[j].spell;
	            if (gSpell.fromArtifact && gSpell.level == s.level && gSpell.name == s.name) 
	            {
	                this.groupedSpells[j].count++;
	                added = true;
	                break;
	            }
	        }
	    }
	    if (!added) 
	    {
	        this.groupedSpells.push({spell: s, count: 1});
	    }
	}
	
	UI.show(false, [this.pager]);
	COM(this.pager).setIndexAndCount(COM(this.pager).index, Math.ceil(this.groupedSpells.length / SpellBookView.PAGE_SIZE));
	
	this.gotoPage(COM(this.pager).index);
}

SpellBookView.prototype.tick = function(time)
{
	for (var i = 0, len = this.spells.children.length; i < len; i++)
	{
	    COM(this.spells.children[i]).tick(time, this.parentDialog.cannotCast);
	}
	this.updateCastButton();
}

SpellBookView.prototype.createSpellView = function()
{
	var v = COM.create('SpellView');
	COM.bindExclusiveClick(v.DOM, $.proxy(this.onSelectSpell, this));
	return v;
}

SpellBookView.prototype.gotoPage = function(page)
{
	var skipped = SpellBookView.PAGE_SIZE * page;
	//var childCount = Math.min(SpellBookView.PAGE_SIZE, this.groupedSpells.length - skipped);
	var childCount = this.groupedSpells.length;
	UI.makeNChilds(this.spells, childCount, $.proxy(this.createSpellView, this));
	for (var i = 0, len = this.spells.children.length; i < len; i++)
	{
		var v = this.spells.children[i];
	    var spell = this.groupedSpells[i + skipped];
	    COM(v).update(this.estate, this.hero, this.buffs, spell.spell, spell.count, this.target, this.time, this.parentDialog.cannotCast);
	}
	
	this.updateCastButton();
}

SpellBookView.prototype.updateCastButton = function()
{
	var visible = this.hero && this.hero.owner == FUser.CURRENT_USER.username;
	var enabled = this.selectedView && COM(this.selectedView).canCast();
	UI.show(false && visible, [this.castButton]);
	COM(this.castButton).enabled = visible && enabled;
}

SpellBookView.prototype.onCastSpell = function(view)
{
	if (COM(view).canCast())
	{
		CastSpellDialog.create(this.estate, COM(view).getSpell(), this.target, this.parentDialog, this.fromCity);
	}
}

SpellBookView.prototype.onSelectSpell = function(e)
{
	var view = e.currentTarget, canCast = COM(view).canCast();
	if (canCast) 
	{
	    if (this.selectedView) 
	    {
	        COM(this.selectedView).deselect();
	    }
	    
	    this.selectedView = view;
		COM(view).select();
	    
	    this.updateCastButton();
	}
	
	COM(view).onClick = canCast ? $.proxy(this.onCastSpell, this) : null; 
	COM(view).onToolTip();
}