function SpellDescription()
{
}

SpellDescription.prototype.__construct = function()
{
    this.cantCastSourceUnderAttack.innerHTML = I18.getMegaString(I18.f('CANNOT_CAST_SOURCE_UNDER_ATTACK'), FConfig);
    this.cantCastTargetUnderAttack.innerHTML = I18.getMegaString(I18.f('CANNOT_CAST_TARGET_UNDER_ATTACK'), FConfig);
    this.cantCastSourceUnderSiege.innerHTML = I18.getMegaString(I18.f('CANNOT_CAST_SOURCE_UNDER_SIEGE'), FConfig);
    this.cantCastTargetUnderSiege.innerHTML = I18.getMegaString(I18.f('CANNOT_CAST_TARGET_UNDER_SIEGE'), FConfig);
    this.spellEffectsText.innerHTML = I18.f('SPELL_EFFECTS');
}

SpellDescription.EMPTY_ARRAY = [];
SpellDescription.prototype.showEffectsText = true; // Boolean

SpellDescription.prototype.update = function(spellName, decl, artifact, quest, buffs, cannotCastReasons)
{
	var contains = function(a,v)
	{ 
		for (var i=0,len=a.length;i<len;i++)
		{
			if (a[i].name==v.name) return a[i];
		}
		return null;
	}
	
	cannotCastReasons = cannotCastReasons || [];
	buffs = buffs || [];
	
	if (UI.show(!artifact && !quest && (decl.manaCost > 0), [this.spellMana]))
	{
	    this.spellMana.innerHTML = I18.f('SPELL_MANA', decl.manaCost);
	}
	
	if (UI.show(!artifact && !quest && (decl.cooldown != 0), [this.spellCooldown]))
	{
	    this.spellCooldown.innerHTML = I18.mega(I18.f('SPELL_COOLDOWN'), decl.cooldown);
	}
	
	if (UI.show(!quest && (decl.target.length != 0 || contains(decl.estateType, EstateType.EMPTY_FIELD)) && !(decl.target.length == 1 && contains(decl.target, SpellTargetType.SELF_ESTATE)), [this.spellRadius])) 
	{
	    this.spellRadius.innerHTML = '';
	    var baseRadius = decl.radius;
	    var radiusModifier = FBuff.sumOfBuffs(buffs, BuffType.CAST_RADIUS, null, -1);
	    
	    this.spellRadius.innerHTML = (baseRadius == 0) 
            ? ((radiusModifier == 0) 
            	? I18.f('SPELL_RADIUS_SELF_KINGDOM') 
            	: I18.f('SPELL_RADIUS_SELF_KINGDOM_MODIFIED', Math.max(0, radiusModifier))
            )
            : ((radiusModifier == 0) 
            	? I18.f('SPELL_RADIUS', baseRadius)
            	: I18.f('SPELL_RADIUS_MODIFIED', baseRadius, Math.max(-baseRadius, radiusModifier))
            )
	    ; 
	}
	
	UI.show(this.showEffectsText, [this.spellEffectsText]);
	
	if (quest || decl.estateType.length == 1 && decl.estateType[0].name == EstateType.EMPTY_FIELD.name) 
	{
	    UI.show(false, [this.spellTarget]);
	} 
	else 
	{
	    UI.show(true, [this.spellTarget]);
	    this.spellTarget.innerHTML = I18.f('SPELL_TARGET', this.targetDescription(spellName, decl));
	}
	
	var targetTypeDescription = [];
	if (decl.estateType.length == 0) 
	{
	    targetTypeDescription = [I18.f('ESTATE_TYPE_IMPOSIBLE')];
	} 
	else if (EstateType.isAnyEstate(decl.estateType)) 
	{
	    targetTypeDescription = [I18.f('ESTATE_TYPE_ANY_ESTATE')];
	} 
	else 
	{
	    for(var i = 0, len = decl.estateType.length; i < len; i++) 
	    {
	    	var tt = decl.estateType[i];
	        if (tt.name == EstateType.CASTLE.name && !FConfig.CASTLES_ENABLED) continue;
	        if (tt.name == EstateType.GROTTO.name && !FConfig.GROTTO_ENABLED) continue;
	        targetTypeDescription.push(I18.getEstateTypeName(tt));
	    }
	}
	
	if (UI.show(!quest, [this.spellTargetType]))
	{
	    this.spellTargetType.innerHTML = I18.f('SPELL_TARGET_TYPE', targetTypeDescription.join(', '));
	}
	
	UI.show(contains(decl.blocked, Blocked.SOURCE_UNDER_ATTACK), [this.cantCastSourceUnderAttack]);
	UI.show(contains(decl.blocked, Blocked.TARGET_UNDER_ATTACK), [this.cantCastTargetUnderAttack]);
	UI.show(contains(decl.blocked, Blocked.SOURCE_UNDER_SIEGE), [this.cantCastSourceUnderSiege]);
	UI.show(contains(decl.blocked, Blocked.TARGET_UNDER_SIEGE) || decl.immediate.containsReward("REWARD_FINISH_BUILDINGS"), [this.cantCastTargetUnderSiege]);
	
	this.spellEffects.innerHTML = '';
	EffectsView.createDescriptiveView(this.spellEffects, decl.immediate.dtoActions, false, true, false, false);
	
	if (decl.hero.dtoActions.length > 0) 
	{
	    $(this.spellEffects).append('<div>'+I18.f('SPELL_CARRIER_HERO')+'</div>');
	}
	EffectsView.createDescriptiveView(this.spellEffects, decl.hero.dtoActions, false, true, false, true);
	
	if (decl.estate.dtoActions.length > 0) 
	{
	    $(this.spellEffects).append('<div>'+I18.f('SPELL_CARRIER_ESTATE')+'</div>');
	}
	EffectsView.createDescriptiveView(this.spellEffects, decl.estate.dtoActions, false, true, false, false);
	
	if (decl.owner.dtoActions.length > 0) 
	{
	    $(this.spellEffects).append('<div>'+I18.f('SPELL_CARRIER_OWNER')+'</div>');
	}
	EffectsView.createDescriptiveView(this.spellEffects, decl.owner.dtoActions, false, true, false, false);
	
	UI.show(this.spellEffects.innerHTML, [this.spellEffects]);
	
	// errors display
	$('>*', this.spellProps).removeClass('negative');
	for (var i = 0; i < cannotCastReasons.length; i++)
	{
		$(this['spell' + cannotCastReasons[i]]).addClass('negative');
	}
	
	if (UI.show(decl.maxDuration > 0, [this.durationLabel]))
	{
	    this.durationLabel.innerHTML = I18.mega(I18.f('SPELL_DURATION'), decl.maxDuration); 
	    this.durationLabel.innerHTML += (decl.stopsAfterCombat 
	    	? " " + I18.f('STOPS_AFTER_COMBAT') 
	    	: ""
	    );
	}
}

SpellDescription.prototype.targetDescription = function(spellName, decl)
{
	var targetDescription = I18.getStringOrNull("SPELL_TARGET_" + spellName);
	return targetDescription ? targetDescription : this.buildTargetDescription(decl.target);
}

SpellDescription.prototype.buildTargetDescription = function(targets)
{
	var contains = function(a,v)
	{ 
		for (var i=0,len=a.length;i<len;i++)
		{
			if (a[i].name==v.name) return a[i];
		}
		return null;
	}
	
	var targetDescription = "";
	
	if (contains(targets, SpellTargetType.OWN_ESTATE) && contains(targets, SpellTargetType.SELF_ESTATE)) 
	{
	    targets = Arrays.filter(targets, function (target)
	    {
	        return target.name != SpellTargetType.SELF_ESTATE.name && target.name != SpellTargetType.OWN_ESTATE.name;
	    });
	    targetDescription += I18.f('SPELL_TARGET_TYPE_SELF_OWN_ESTATE') + (targets.length > 0 ? ", " : "");
	}
	
	targetDescription += Strings.join(Arrays.sorted(targets), ", ", function(target)
	{
	    return I18.getSpellTargetTypeName(target.name);
	});
	
	return targetDescription;
}
