function SpellView()
{
	this.estate = null; // FEstate
	this.hero = null; // FHero
	this.buffs = null; // Array
	this.spell = null; // FSpell
	this.count = null; // int
	this.target = null; // CastTarget
	
	this.cantCastReason = null; // Array
	this.cantCastAddidional = null; // Array
	
	this.isSelected = null; // Boolean
	this.canBeCasted = null; // Boolean
	
	this.onClick = null;
}

SpellView.prototype.__construct = function()
{
}

SpellView.prototype.estate = null; // FEstate
SpellView.prototype.hero = null; // FHero
SpellView.prototype.buffs = null; // Array
SpellView.prototype.spell = null; // FSpell
SpellView.prototype.count = null; // int
SpellView.prototype.target = null; // CastTarget

SpellView.prototype.cantCastReason = null; // Array
SpellView.prototype.cantCastAddidional = null; // Array

SpellView.prototype.isSelected = null; // Boolean
SpellView.prototype.canBeCasted = null; // Boolean

SpellView.prototype.onClick = null;

SpellView.CANNOT_CAST_REASON_CHECKERS =
[
    // radius
    function (s)
    {
        return !s.estate.location.isInRadiusFrom(s.target.location, Math.max(0, s.spell.config.radius + s.estate.sumOfBuffs(BuffType.CAST_RADIUS))) ? "Radius" : null; // Array
    },
    // mana
    function (s)
    {
        var h = s.estate.garrisonHero;
        if (s.spell.fromArtifact) 
        {
            return null;
        }
        return h != null && NumberUtils.compareTo(h.mana, s.spell.config.manaCost) < 0 ? "Mana" : null;
    },
    // cooldown
    function (s) 
    {
        return s.spell.isInCooldown(Tick.now()) ? "Cooldown" : null;
    },
    // target
    function (s)
    {
        if (s.spell.config.estateType.length == 1 && s.spell.config.estateType[0].name == EstateType.EMPTY_FIELD.name) 
        {
            return null;
        }
        var declTarget = s.spell.config.target;
        for (var i = 0, len = declTarget.length, t = s.target.getTargetType(s.estate.location); i < len; i++)
        {
        	if (declTarget[i].name == t.name)
        	{
        		return null;
        	}
        }
        return "Target";
    },
    function (s)
    {
        for (var i = 0, len = s.spell.config.estateType.length, t = s.target.estateType; i < len; i++)
        {
        	if (s.spell.config.estateType[i].name == t.name)
        	{
        		return null;
        	}
        }
        return "TargetType";
    }
];

SpellView.CANNOT_CAST_ADDITIONAL_CHECKERS =
[
    // camp is setting up
    function (s)
    {
        if (s.estate.__className__ == 'FCamp' && s.estate.isSettingUp) 
        {
            return I18.f('CAMP_IS_SETTING_UP'); // Array
        }
        return null;
    },
    // in garrison
    function (s)
    {
        if (s.hero && !s.estate.isHeroInGarrison(s.hero.id)) 
        {
            return s.estate.isHeroInHideout(s.hero.id) ? I18.f('HERO_IN_HIDEOUT') : I18.f('DOESNT_WORK_NON_GARRISON_HERO');
        }
        return null;
    },
    // gate
    function (s)
    {
        return s.estate.isGateClosed() ? I18.f('SPELL_REQUIRE_OPEN_GATES') : null;
    },
    // upset
    function (s) 
    {
        if (s.hero && s.hero.isUpset()) 
        {
            return s.hero.upsetByTitle ? I18.f('HERO_ERROR_REQUIRED_MAX_TITLE') : I18.f('DOESNT_WORK_UPSET_HERO')
        }
        return null;
    },
    // blocked
    function (s) 
    {
        return s.estate.isEstateUnderAttack && s.spell.config.blocked.contains(Blocked.SOURCE_UNDER_ATTACK) 
        	? I18.getMegaString(I18.f('CAST_ERROR_SOURCE_UNDER_ATTACK'), FConfig) 
        	: null
        ;
    },
    function (s)
    {
        return (Tick.ticksTill(s.target.nearestAttackTick) < FConfig.BEFORE_ATTACK_BLOCK) && s.spell.config.blocked.contains(Blocked.TARGET_UNDER_ATTACK) 
        	? I18.getMegaString(I18.f('CAST_ERROR_TARGET_UNDER_ATTACK'), FConfig)
        	: null
        ;
    },
    function (s)
    {
        return s.estate.isEstateUnderSiege && s.spell.config.blocked.contains(Blocked.SOURCE_UNDER_SIEGE) 
        	? I18.getMegaString(I18.f('CAST_ERROR_SOURCE_UNDER_SIEGE'), FConfig) 
        	: null
        ;
    },
    function (s)
    {
        return (Tick.ticksTill(s.target.nearestAttackTick) < FConfig.BEFORE_ATTACK_FEATURES_BLOCK) && s.spell.config.blocked.contains(Blocked.TARGET_UNDER_SIEGE) || s.spell.config.immediate.containsReward("REWARD_FINISH_BUILDINGS") && FUser.CURRENT_USER.isUserUnderSiege 
			? I18.getMegaString(I18.f('CAST_ERROR_TARGET_UNDER_SIEGE'), FConfig) 
			: null
		;
    },
    // antimagic (bkb)
    function (s)
    {
        return FBuff.hasMagicDisable(s.buffs, FEstate.tickNowForEstate(s.estate)) 
        	? I18.f('CAST_ERROR_SOURCE_HAS_MAGIC_DISABLE') 
        	: null
        ;
    },
    // silence
    function (s)
    {
        var buffs = FBuff.getBuffs(s.buffs, BuffType.CASTER_SILENCE, null, FEstate.tickNowForEstate(s.estate));
        if (buffs.length > 0) 
        {
            var spell = buffs[0].cast.spell;
            return I18.getMegaString(I18.f('CASTER_WAS_SILENCED'), {"spell":spell});
        }
        return null;
    },
    // need specialization
    function (s)
    {
        var spec = s.estate.science.getResearchedSpecial();
        if (!spec && s.spell.config.immediate.containsReward("REWARD_CHANGE_SPECIALIZATION")) 
        {
            return I18.f('CAST_ERROR_NO_SPECIALIZATION');
        }
        return null;
    }
];

SpellView.check = function(checkers, s)
{
	var result = [];
	for (var i = 0, len = checkers.length; i < len; i++) 
	{
		var checker = checkers[i];
	    var reason = checker(s);
	    if (reason) result.push(reason);
	}
	return result;
}

SpellView.prototype.update = function(e, h, buffs, s, count, castTarget, time, cannotCast)
{
	this.estate = e;
	if (this.isSelected && this.hero && this.hero.id != h.id) 
	{
	    this.isSelected = false;
	}
	
	this.hero = h;
	this.buffs = buffs;
	this.spell = s;
	this.count = count;
	this.target = castTarget;
	
	var isBottle = s.fromArtifact && s.artifactDeclaration.isBottle();
	$(this.DOM).toggleClass('artifact', !!s.fromArtifact);
	$(this.DOM).toggleClass('bottle', !!isBottle);
	this.spellImage.src = STATIC_INTERFACE_URL + (isBottle 
		? ArtifactAssets[s.artifactDeclaration.pic]
        : Assets.getSpellImageForScroll(s.fromArtifact ? s.artifactDeclaration.pic : s.name)
	);
	this.spellLevel.innerHTML = I18.f('LVL_X', this.spell.level);
	
	// reasons in spellDescription
	this.cantCastReason = [];
	this.cantCastAddidional = [];
	if (FUser.CURRENT_USER.username == h.owner) 
	{
	    if (e && e.accessManageble) 
	    {
	        this.cantCastReason = SpellView.check(SpellView.CANNOT_CAST_REASON_CHECKERS, this);
	        this.cantCastAddidional = SpellView.check(SpellView.CANNOT_CAST_ADDITIONAL_CHECKERS, this);
	    } 
	    else if (h.prisonOwnerName) 
	    {
	        this.cantCastAddidional = [I18.f('CANNOT_CAST_IN_PRISON')];
	    } 
	    else if (!cannotCast) 
	    {
	        this.cantCastAddidional = [I18.f('CANNOT_CAST_IN_MISSION')];
	    }
	}
	
	if (UI.show(this.spell.fromArtifact, [this.countLabel]))
	{
	    this.countLabel.innerHTML = this.count + "X";
	}
	
	this.tick(time, cannotCast);
	
	this.DOM.title = Texts.spellName(this.spell.name);
}

SpellView.prototype.tick = function(time, cannotCast)
{
	if (isNaN(time)) time = Tick.now();
	// http://jira.ely/browse/DEV-5652
	var locked = FUser.CURRENT_USER.username != this.hero.owner && this.spell.name == "TUMANNYI_FANTOM";
	UI.show(locked, [this.lockImage]);
	UI.show(!locked, [this.spellLevel]);
	
	if (UI.show(!locked && this.spell.isInCooldown(time), [this.cooldown.parentNode]))
	{
	    this.cooldown.innerHTML = Tick.toTimeLeft(this.spell.cooldownTicksLeft(time));
	}
	
	this.canBeCasted = !cannotCast 
		&& ((this.cantCastAddidional.length + this.cantCastReason.length) == 0) 
		&& !this.hero.isUpset()
	    && FUser.CURRENT_USER.username == this.hero.owner 
	    && !this.spell.isInCooldown(time)
	;
	//this.spellImage.style.cursor = this.canBeCasted ? 'pointer' : 'default';
	this.updateTint();
}

SpellView.prototype.getSpell = function()
{
	return this.spell;
}

SpellView.prototype.canCast = function()
{
	return this.canBeCasted;
}

SpellView.prototype.updateTint = function()
{
	$(this.DOM).toggleClass('selected', Boolean(!this.selected));
	$(this.spellImage.parentNode).toggleClass('grayscale', Boolean(!this.canBeCasted));
	$(this.spellLevel.parentNode)
		.toggleClass('green', this.canBeCasted)
		.toggleClass('dark', !this.canBeCasted)
	;
}

SpellView.prototype.select = function()
{
	this.isSelected = true;
	this.updateTint();
}

SpellView.prototype.deselect = function()
{
	this.isSelected = false;
	this.updateTint();
}

SpellView.prototype.onToolTip = function()
{
	var castCallback = (typeof this.onClick == 'function') 
		? $.proxy(function(){ this.onClick(this); }, this) 
		: null
	;
	
	var isBottle = this.spell.fromArtifact && this.spell.artifactDeclaration.isBottle();
	var icon = STATIC_INTERFACE_URL + (isBottle 
		? ArtifactAssets[this.spell.artifactDeclaration.pic]
        : Assets.getSpellImageForScroll(this.spell.fromArtifact ? this.spell.artifactDeclaration.pic : this.spell.name)
	)
	var castAction = castCallback 
		? {
			right:{ 
				caption: I18.f('MAKE_CAST'),
				skin: 'green', 
				onclick: castCallback 
			}
		} 
		: null
	;
	return SpellTip.alert(this.spell, this.buffs, this.cantCastReason, this.cantCastAddidional, castAction);
}
