function MarketAuction()
{
	callParentConstructor(MarketAuction.prototype, this);

	this.auction = null; // FAuction
	this.currentLotId = -1;
	this.nextUpdateTick = 0;
	this.updatePending = false;
	
	this.previousStake = "0";
	this.lastStakeInputKey = 0;

	this.lotDialog = null;
}

extendClass(MarketAuction.prototype, MarketTabParent.prototype);

MarketAuction.prototype.UPDATE_INTERVAL = 30;
MarketAuction.prototype.WARNING_LEFTTIME = 10;

MarketAuction.prototype.auction = null; // FAuction
MarketAuction.prototype.currentLotId = -1;
MarketAuction.prototype.nextUpdateTick = 0;
MarketAuction.prototype.updatePending = false;

MarketAuction.prototype.previousStake = "0";
MarketAuction.prototype.lastStakeInputKey = 0;

MarketAuction.prototype.lotDialog = null;

MarketAuction.prototype.__construct = function()
{
	superMethod(MarketAuction.prototype, '__construct', this)();
	
	COM(this.items).onItemClick = $.proxy(this.onSelectLot, this);
	this.lotsToBuy = COM(this.items);
	this.lotsToBuy.selectable = true;
	this.lotsToBuy.unselectable = false;
	
	COM.bindExclusiveClick(this.payButton, $.proxy(this.doPay, this));
	COM(this.payCitiesCombo).onCityChange = $.proxy(this.citySelected, this);
	COM(this.availableGold).alwaysShow = [Resource.GOLD];
	
	APP.showLoader();
}

MarketAuction.prototype.reset = function()
{
    superMethod(MarketAuction.prototype, 'reset', this)();
    this.updatePending = false;
    this.currentLotId = -1;
    if (this.lotDialog)
    {
    	this.lotDialog.close();
    }
}

MarketAuction.prototype.onShow = function() 
{
    superMethod(MarketBuy.prototype, "onShow", this)();
    this.performUpdate();
}

MarketAuction.prototype.onClose = function() 
{
    if (this.lotDialog)
    {
    	this.lotDialog.close();
    }
}

MarketAuction.prototype.performUpdate = function()
{
    if (!this.updatePending) 
    {
        this.updatePending = true;
        Service.market.callMethod('getAuction', [this.estate.id], $.proxy(this.updateAuction, this), $.proxy(this.handleError, this));
    }
}

MarketAuction.prototype.handleError = function(e)
{
    APP.hideLoader();
    if (e.__className__ == 'AuctionException') 
    {
        Alert.create(e.getMessage());
        if (Tick.hasPassed(e.bannedTill)) 
        {
            this.performUpdate();
        } 
        else 
        { 
        	//player is banned in auction
            this.close();
        }
        return true;
    }
    return false;
}

MarketAuction.prototype.refreshPickedLot = function(selectedId)
{
    var currentLot = null;
    this.lotsToBuy.selectedIndex = -1;
    this.currentLotId = -1;
    this.nextUpdateTick = Tick.now() + this.UPDATE_INTERVAL;
    
    var items = this.lotsToBuy.items;
    for (var i = 0; i < items.length; i++) 
    {
        var item = items[i];
        COM(item).refresh();
        
        var lot = COM(item).data;
        if (lot.lastCustomer == FUser.CURRENT_USER.username) 
        {
            this.currentLot = lot;
            this.currentLotId = lot.id;
        }

        //if (lot.id == selectedId) this.lotsToBuy.selectedIndex = i;
        if (lot.finishTick < this.nextUpdateTick) this.nextUpdateTick = lot.finishTick;
    }
    
    // select the item which was selected earlier
    if (selectedId >= 0) this.onSelectLot();
}

MarketAuction.prototype.updateAuction = function(auction)
{
    if (!this.updateInvoked) 
    {
        this.updateInvoked = true;
    }
    this.updatePending = false;
    var selectedLot = this.getSelectedLot();
    var selectedId = selectedLot ? selectedLot.id : -1;
    this.auction = auction;
    
    UI.show(!this.auction.hasDebt, [this.lotsToBuy]);
    UI.show(this.auction.hasDebt, [this.debtBox]);
    
    this.lotsToBuy.update(auction);
    
    if (!this.auction.hasDebt) 
    {
        this.refreshPickedLot(selectedId);
    }
    else
    {
		this.nextUpdateTick = auction.debt.finishTick;
		COM(this.debtCount).value = auction.debt.count;
		var funds = new Funds(); funds.gold = auction.debt.lastPrice;
		COM(this.debtSum).update(funds, FUser.CURRENT_USER.race);
		
		//var filteredLinks = filterCities(auction.estate.links);
		var filteredLinks = Arrays.filter(auction.estate.links, function (link)
		{
			return link.estateInfo.estateType.name == EstateType.CITY.name; 
		});

		COM(this.payCitiesCombo).updateCities(filteredLinks);
		COM(this.payCitiesCombo).cityId = auction.estate.id;
		this.citySelected(COM(this.payCitiesCombo).city);
    }
    
    if (this.lotDialog)
    {
    	this.lotDialog.update(this.estate, selectedLot, this.auction);
    }
    
    Events.onFEstateUpdated.fire(this.auction.estate);
    APP.hideLoader();
}

MarketAuction.prototype.tick = function()
{
    if (!this.auction) return; // not loaded...
    
    var lot = this.getSelectedLot();
    if (Tick.now() >= this.nextUpdateTick) 
    {
        this.performUpdate();
    }
    
    UI.show(!this.auction.hasDebt, [this.lotsToBuy])
    if (UI.show(this.auction.hasDebt, [this.debtBox]))
    {
    	this.updatePending = true;
    	var left = this.auction.debt.finishTick - Tick.now();
    	
    	COM(this.payTime).value = left;
        $(this.payTime)
        	.toggleClass('negative', left <= 10)
        	.toggleClass('positive', left > 10)
        ;
    }
    
    if (this.lotDialog)
    {
    	this.lotDialog.tick();
    }
}

MarketAuction.prototype.hasReason = function()
{
    var auctionBanned = !Tick.hasPassed(this.estate.auctionBanTill);
    //this.auction.enabled = this.estate.getMarket().isConstructed && !auctionBanned && this.estate.accessManageble && this.dialog.estate.owner.isAuctionAvailable;
    if (!this.estate.owner.isAuctionAvailable) 
    {
        return I18.f('AUCTION_UNAVAILABLE', FConfig.AUCTION_OPEN_RATING);
    } 
    else if (auctionBanned) 
    {
        return I18.f('AUCTION_ERROR_BAN', Tick.toTimeFuture(this.estate.auctionBanTill));
    } 
    else if (!this.estate.accessManageble) 
    {
        return I18.f('NO_ACCESS');
    }
    return false;
}

MarketAuction.prototype.sendMessage = function()
{
    MailBoxView.i().sendMessage(this.getSelectedLot().lastCustomer);
}

MarketAuction.prototype.onSelectLot = function(item, list)
{
    var lot = item ? COM(item).data : this.getSelectedLot();
    if (!lot) return;
    
    if (item)
    {
	    this.lotDialog = MarketAuctionDialog.create(this.estate, lot, this.auction);
		this.lotDialog.onSuccess = $.proxy(this.updateAuction, this);
		this.lotDialog.onError = $.proxy(this.handleError, this);
		this.lotDialog.onClose = $.proxy(function(){ 
			this.lotDialog = null;
			this.lotsToBuy.selectedIndex = -1; 
		}, this);
    }
}

// current user's lot
MarketAuction.prototype.getSelectedLot = function()
{
    return this.lotsToBuy.value;
}

MarketAuction.prototype.onGetRowColor = function(item, defaultColor)
{
    return this.currentLotId != -1 && this.currentLotId == item.id ? Effects.GREEN : defaultColor;
}

MarketAuction.prototype.compareByPriceRatio = function(a, b, fields)
{
    return NumberUtils.strictCompare(a.lastPrice / a.count, b.lastPrice / b.count);
}


MarketAuction.prototype.filterCities = function(links)
{
    return Arrays.filter(links, function(link){
        return link.estateInfo.estateType.name == EstateType.CITY.name;
    });
}

MarketAuction.prototype.citySelected = function(c)
{
        var available = new Funds; available.gold = c.bank.gold;
        COM(this.availableGold).update(available, FUser.CURRENT_USER.race);
        COM(this.debtSum).capacity = available;
        COM(this.payButton).reason = available.gold >= this.auction.debt.lastPrice ? null : I18.f('NOT_ENOUGH_RESOURCES');
}

MarketAuction.prototype.doPay = function()
{
	APP.showLoader();
	Service.market.callMethod('payForLot', 
		[this.estate.id, COM(this.payCitiesCombo).city.id], 
		$.proxy(this.updateAuction, this), 
		$.proxy(this.handleError, this)
	);
}
