function MarketAuctionList()
{
	callParentConstructor(MarketAuctionList.prototype, this);
	this.lots = [];
}

extendClass(MarketAuctionList.prototype, ScrollableList.prototype);

MarketAuctionList.prototype.rendererClass = 'MarketAuctionItem';

MarketAuctionList.prototype.lots = [];
MarketAuctionList.prototype.order = {col: -1, dir: 0};

MarketAuctionList.prototype.__construct = function()
{
	superMethod(MarketAuctionList.prototype, '__construct', this)();

	this.initHeaders();	
	this.visibleItems = 0; // no auto resize, list must be resized on fitLayout
}

MarketAuctionList.prototype.initHeaders = function()
{
	var headers = [
		$('<sort col="0"><img src="' + STATIC_INTERFACE_URL + CommonAssets.ICON_PEARL + '" />' + I18.f('LOT') + '</sort>'),
		$('<sort col="1">' + I18.f('PRICE_SHORT') + '</sort>'),
		$('<sort col="2"><img src="' + STATIC_INTERFACE_URL + CommonAssets.ICON_GOLD + '" />' + I18.f('TOTAL_PRICE') + '</sort>'),
		$('<sort col="3">' + I18.f('AUCTIONER') + '</sort>'),
		$('<sort col="4">' + I18.f('FINISH') + '</sort>'),
	];
	
	for (var i = 0; i < headers.length; i++)
	{
		headers[i].on(CLICK_EVENT, $.proxy(this.onHeaderClick, this));
	}
	
	this.setHeader(headers);
}

MarketAuctionList.prototype.onHeaderClick = function(e)
{
	var col = e.currentTarget.getAttribute('col') * 1 || 0;
	this.orderBy(col);
}

MarketAuctionList.prototype.update = function(auction)
{
	this.lots = auction.lots.slice(0);
	this.orderBy(this.order.col, this.order.dir);
}

MarketAuctionList.prototype.orderBy = function(col, dir)
{
	if (col < 0) col = -1;
	if (typeof dir != 'undefined') 
	{
		this.order.dir = dir*1;
	}
	else
	{
		if (this.order.col != col || !this.order.dir) this.order.dir = 1;
		else if (this.order.dir == 1) this.order.dir = -1;
		else if (this.order.dir == -1) this.order.dir = 0;
	}

	this.order.col = col;
	this.dataProvider = this.orderedData();

	$('sort', this.headerRow).removeClass('asc desc');
	if (this.order.dir)
	{
		$('sort[col=' + this.order.col + ']', this.headerRow).addClass(this.order.dir > 0 ? 'asc' : 'desc');
	}
}

MarketAuctionList.prototype.orderedData = function()
{
	var ret = this.lots.slice(0), o = this.order;
	for (var i = 0; i < ret.length; i++)
	{
		ret[i].refresh();
	}
	
	if (o.dir)
	{
		function doOrder(lot1, lot2)
		{
			var val1 = lot1.id, val2 = lot2.id, col = o.col*1 || 0;
			if (col == 0)
			{
				val1 = lot1.count*1;
				val2 = lot2.count*1;
			}
			if (col == 1)
			{
				val1 = (lot1.lastPrice / lot1.count);
				val2 = (lot2.lastPrice / lot2.count);
			}
			if (col == 2)
			{
				val1 = lot1.lastPrice*1;
				val2 = lot2.lastPrice*1;
			}
			if (col == 3)
			{
				val1 = lot1.lastCustomer.toLowerCase();
				val2 = lot2.lastCustomer.toLowerCase();
			}
			if (col == 4)
			{
				val1 = lot1.finishTick*1;
				val2 = lot2.finishTick*1;
			}
			
			if (val1 == val2) return 0;
			return (val1 > val2 ? o.dir : -o.dir);
		}
		
		ret = ret.sort(doOrder);
	}
	return ret;
}
