function MarketBuyDialog()
{
	callParentConstructor(MarketBuyDialog.prototype, this);
	
	this.__defineGetter__('amount', this.getAmount);
	this.__defineSetter__('amount', this.setAmount);
	this.buyLot = null;
}

extendClass(MarketBuyDialog.prototype, Dialog.prototype);

MarketBuyDialog.prototype.estate = null;
MarketBuyDialog.prototype.fmarket = null;
MarketBuyDialog.prototype.info = null;
MarketBuyDialog.prototype.lot = null;
MarketBuyDialog.prototype.buyLot = null;

MarketBuyDialog.create = function(estate, fmarket, info, lot)
{
	var d = COM.create('MarketBuyDialog');
	d.update(estate, fmarket, info, lot);
	d.popup.call(d);
	return d;
}

MarketBuyDialog.prototype.__construct = function()
{
	superMethod(MarketBuyDialog.prototype, '__construct', this)();
	
	COM(this.totalCost).showZero = false;
	COM(this.totalCost).alwaysShow = [Resource.GOLD];
	
	$(this.amountToBuy).on('change', $.proxy(function(e){this.amount = COM(e.currentTarget).value;}, this));
	
	COM.bindExclusiveClick(this.buyButton, $.proxy(this.onBuyClick, this));
}

MarketBuyDialog.prototype.getAmount = function()
{
	return COM(this.amountToBuy).value*1;
}

MarketBuyDialog.prototype.setAmount = function(val)
{
	val = Math.max(COM(this.amountToBuy).min, Math.min(COM(this.amountToBuy).max, val*1));
	if (val == COM(this.amountToBuy).value) return;
	COM(this.amountToBuy).value = val;
	this.tick();
}

MarketBuyDialog.prototype.firstUpdate = function()
{
	this.firstUpdate = function(){};
   	COM(this.amountToBuy).value = COM(this.amountToBuy).max;
}

MarketBuyDialog.prototype.update = function(estate, fmarket, info, lot)
{
	this.estate = estate;
	this.fmarket = fmarket;
	this.info = info;
	this.lot = lot;
	
	this.tick();
	this.firstUpdate();
}

MarketBuyDialog.prototype.tick = function()
{
    COM(this.totalCost).capacity = this.estate.bank;

    var merchantsAvailable = this.estate.countFreeSpecialUnits(UnitType.MERCHANT);
    var canAffordAmount = Math.floor(this.estate.bank.gold / this.lot.price);
    var capacity = Math.floor(FConfig.getMerchantCapacity(this.estate) * (1.0 + this.estate.sumOfBuffs(BuffType.MERCHANTS_CAPACITY)));
    
    COM(this.amountToBuy).max = Math.min(this.lot.amount, canAffordAmount, merchantsAvailable * capacity);
    this.amount = COM(this.amountToBuy).value;
    
    var totalGold = new Funds();
    totalGold.gold = Math.ceil(this.amount * this.lot.price);
    COM(this.totalCost).update(totalGold, this.estate.owner.race);

    var merchantsNeeded = Math.max(1, Math.ceil(COM(this.amountToBuy).value / capacity));

    var canSendMission = (merchantsNeeded <= merchantsAvailable), reason = null;
    if (!this.estate.accessDeputy)
    {
    	reason = I18.f('NO_ACCESS');
    }
    if (!this.lot)
    {
    	reason = I18.f('AUCTION_ERROR_NO_SUCH_LOT');
    }
    else if (this.fmarket.missionsCount >= this.estate.owner.getScienceValue(Science.ARMY_MANAGEMENT) + FConfig.MAX_MISSIONS) 
    {
        reason = I18.f('MISSIONS_LIMIT_REACHED');
    } 
    else if (merchantsAvailable == 0 || (!canSendMission)) 
    {
        reason = I18.f('NO_MERCHANTS_FOR_TRADE');
    } 
    else if (this.amount <= 0)
    {
    	reason = I18.f('NEED_TO_FILL_DATA');
    }
    else if (canAffordAmount == 0) 
    {
        reason = I18.f('MARKET_ERROR_NOT_ENOUGH_GOLD');
    } 
    
    $(this.merchantsCount).html(I18.f('NEEDED_X_MECHANTS_FROM_Y', Math.ceil(merchantsNeeded), Math.floor(merchantsAvailable)));
    
    COM(this.buyButton).reason = reason;
    $(this.merchantsCount)
    	.toggleClass("negative", Boolean(!canSendMission))
    	.toggleClass("positive", Boolean(canSendMission))
    ;
    
    //COM(this.buyButton).enabled = this.lot && this.estate.accessDeputy && (this.amount > 0) && canSendMission;
    //COM(this.amountToBuy).disabled = !(this.lot && this.estate.accessDeputy && merchantsAvailable > 0);
}

MarketBuyDialog.prototype.onBuyClick = function()
{
	if (this.buyLot) 
	{
		this.buyLot(this.lot, this.amount);
	}
}
