function ApprenticeRenderer()
{
	callParentConstructor(ApprenticeRenderer.prototype, this);
	
	this.__defineGetter__('apprentice', this.getApprentice);
	this.__defineSetter__('apprentice', this.setApprentice);
	
	this.allowControl = false;
	this.canExpand = false;
	this.onResult = null;
	this.rewardDialog = null;
}

extendClass(ApprenticeRenderer.prototype, ScrollableListItem.prototype);

ApprenticeRenderer.prototype.allowControl = false;
ApprenticeRenderer.prototype.canExpand = false;
ApprenticeRenderer.prototype.onResult = null; // Function

ApprenticeRenderer.prototype.detailsDialog = null;

ApprenticeRenderer.prototype.__construct = function()
{
	superMethod(ApprenticeRenderer.prototype, '__construct', this)();
	
	COM.bindExclusiveClick(this.deleteButton, $.proxy(this.onExpel, this));
	COM.bindExclusiveClick(this.releaseButton, $.proxy(this.onRelease, this));
	
	$(this.DOM).on('select', $.proxy(this.showDetails, this));
}

ApprenticeRenderer.prototype.showDetails = function()
{
	this.selected = false;
	if (!this.apprentice || !this.canExpand) return;
	if (this.detailsDialog) this.detailsDialog.close();
	this.detailsDialog = ApprenticeQuestsDialog.create(this.apprentice, $.proxy(this.onGetReward, this), this.allowControl);
}

ApprenticeRenderer.prototype.getApprentice = function()
{
    return this.data;
}

ApprenticeRenderer.prototype.setApprentice = function(val)
{
    this.data = val;
}

ApprenticeRenderer.prototype.refresh = function()
{
	if (!this._data) return;
	
    var app = this.apprentice;

    UI.show(this.allowControl && app.readyToRelease, [this.releaseButton]);
    UI.show(this.allowControl && !app.readyToRelease && Tick.isInfinity(this.apprentice.expelTick), [this.deleteButton]);
	
    if (app.hasRejected || !app.user || !app.isValid) 
    {
        this.deleteButton.title = I18.f('REMOVE');
        this.canExpand = !app.hasRejected;
       	this.apprenticeName.style.fontStyle = 'italic';
       	this.apprenticeName.className = '';
        if (app.hasRejected)
        {
       		this.apprenticeName.className = 'negative';
        	this.apprenticeName.style.textDecoration = 'line-through';
        	$(this.apprenticeName).addClass('negative');
        }
    } 
    else if (app.hasAccepted) 
    {
   		this.apprenticeName.className = 'text-normal-bold positive';
        this.deleteButton.title = I18.f('EXPEL_APPRENTICE');
        this.canExpand = true;
    } 
    else 
    {
        this.deleteButton.title = I18.f('CANCEL_APPRENTICE_INVITATION');
        this.canExpand = false;
       	this.apprenticeName.className = '';
        this.apprenticeName.style.fontStyle = 'italic';
        this.apprenticeName.style.textDecoration = 'underline';
    }
    
    this.apprenticeName.innerHTML = app.username;
    this.progressLabel.innerHTML = I18.getMegaString('{0}/{1}', [app.questProgress(), app.quests.length]);
    
    this.tick();
}

ApprenticeRenderer.prototype.onGetReward = function(quest)
{
    var d = COM.create('DisplayMentorRewardDialog');
    d.onChooseReward = $.proxy(function(data)
    {
        Service.alliance.callMethod('getMentorReward', [this.apprentice.username, quest.name, data], this.onResult, $.proxy(this.onFail, this));
    }, this);
    
    d.update(quest);
    d.popup(this);
    
	if (this.detailsDialog) this.detailsDialog.close();
}

ApprenticeRenderer.prototype.onExpel = function()
{
    if (!this.apprentice.hasAccepted) 
    {
	    Alert.create(I18.f('CANCEL_APPRENTICE_INVITATION_CONFIRMATION', this.apprentice.username), '', 
	    	$.proxy(function(){
	        	Service.alliance.callMethod('cancelApprenticeInvitation', [this.apprentice.username], this.onResult, $.proxy(this.onFail, this));
	    	}, this)
	    );
    } 
    else if (this.apprentice.isValid) 
    {
	    Alert.create(I18.f('EXPEL_APPRENTICE_CONFIRMATION', this.apprentice.username), '', 
	    	$.proxy(function(){
	        	Service.alliance.callMethod('expelApprentice', [this.apprentice.username], this.onResult, $.proxy(this.onFail, this));
	    	}, this)
	    );
    } 
    else 
    {
	    Alert.create(I18.f('EXPEL_APPRENTICE_CONFIRMATION', this.apprentice.username), '', 
	    	$.proxy(function(){
	        	Service.alliance.callMethod('expelApprentice', [this.apprentice.username], this.onResult, $.proxy(this.onFail, this));
	    	}, this)
	    );
    }
    return false;
}

ApprenticeRenderer.prototype.onRelease = function()
{
   	Alert.create(I18.f('RELEASE_APPRENTICE_CONFIRMATION', this.apprentice.username), null, $.proxy(function(){
   		Service.alliance.callMethod('releaseApprentice', [this.apprentice.username], this.onResult, $.proxy(this.onFail, this));
   	}, this))
    return false;
}

ApprenticeRenderer.prototype.onFail = function(e)
{
    if (GameException.isDisplayable(e)) 
    {
        Alert.create(e.getMessage());
        return true;
    }
    return false;
}

ApprenticeRenderer.prototype.tick = function()
{
    if (!this.apprentice.user)
    {
    	this.timeToRemove.innerHTML = '<span class="text-narrow text-small">'+I18.f('APPRENTICE_DIED')+'</span>';
    }
    else if (!Tick.isInfinity(this.apprentice.expelTick))
    {
    	this.timeToRemove.innerHTML = Tick.toTimeLeft(this.apprentice.expelTick - Tick.now());
    	this.timeToRemove.title = I18.getMegaString('MENTOR_REFUSING_STARTED_BODY', {tick:this.apprentice.expelTick, apprentice:this.apprentice.user.username});
    }
    else
    {
    	this.timeToRemove.innerHTML = '&nbsp;';
    }
    
    $(this.DOM)
    	.toggleClass('not-accepted', (this.apprentice.user && !this.apprentice.hasAccepted && !this.apprentice.hasRejected))
    	.toggleClass('rejected', !!(this.apprentice.user && this.apprentice.hasRejected))
    	.toggleClass('dead', !this.apprentice.user)
    ;
}

