var FCastle = function()
{
	callParentConstructor(FCastle.prototype, this);
};

extendClass(FCastle.prototype, FOuterEstate.prototype);

$.extend(FCastle.prototype,
{
	__className__ : "FCastle",

	canManageOnSectors : function()
	{
		return false;
	},

	getCellByIndex : function(cellIndex)
	{
		if (cellIndex == FCastle.GATE_FAKE_CELL_INDEX)
		{
			return getFakeCell(this.castleGate);
		}
		return superMethod(FCastle, "getCellByIndex", this)(cellIndex);
	},

	isOnSector : function(cellIndex)
	{
		return cellIndex < this.CELLS_ON_SECTORS;
	},

	getVisualNumber : function(cellIndex)
	{
		if (this.isOnSector(cellIndex))
		{
			return cellIndex % this.CELLS_PER_SECTOR + 1;
		}
		return superMethod(FCastle, "getVisualNumber", this)(this.getIndexWithoutSectors(cellIndex));
	},

	getCellByIndex : function(cellIndex)
	{
		if (cellIndex == FCastle.GATE_FAKE_CELL_INDEX)
		{
			return this.getFakeCell(this.castleGate);
		}
		return superMethod(FCastle, "getCellByIndex", this)(cellIndex);
	},

	isMaxHallLevel : function()
	{
		return this.hall.level >= this.hall.type.getLevels(Race.CASTLE_MONSTER).length - 1;
	},

    canUpgradeHall : function()
    {
        return this.accessDeputy && !this.hall.isBuildingOrUpgrading && !this.isMaxHallLevel();
    },

	getMaxCellIndex : function()
	{
		return this.cells.length + this.CELLS_ON_SECTORS;
	},

	isInsideCityWall : function(cellIndex)
	{
		return cellIndex >= this.CELLS_ON_SECTORS && this.getIndexWithoutSectors(cellIndex) < this.insideWallCellCount;
	},

	isCellReachable : function(cellIndex)
	{
		return cellIndex == FCastle.GATE_FAKE_CELL_INDEX || (!this.isOnSector(cellIndex) && this.isInsideCityWall(cellIndex));
	},

	getInsideWallCellCount : function()
	{
		return this.insideWallCellCount;
	},

	getSkinIndex : function()
	{
		return this.skinIndex;
	},
	
	getEstateType: function()
	{
		return EstateType.CASTLE;
	},
	
    calcGateGarrisonHelp: function()
    {
        var sum = 0;
        for (var i = 0; i < UnitType.MILITARY.length; i++) 
        {
            var ut = UnitType.MILITARY[i];
            var stat = FConfig.getStat(this.race, ut, this.owner.science.unitLevels[i]);
            var buff = FBuff.sumOfBuffs(this.buffs, BuffType.HEALTH_BONUS, ut, FEstate.tickNowForEstate(this));
            sum += this.garrison.getValue(ut) * stat.health * (1 + buff);
        }
        var result = NumberUtils.floor(sum * FConfig.CASTLEGATE_GARRISON_HELP);
        return result;
    },
	
	tick: function(delta)
	{
		superMethod(FCastle.prototype, "tick", this)(delta);
		
        if (!this.isExpeditionReport)
        {
            if (this.castleGate)
            {
                this.castleGate.tick(delta);
                if (this.castleGate.isRepairing && this.castleGate.builders > 0 && this.castleGate.workDone < this.castleGate.workToDo)
                {
                    var healthToRestore = this.castleGate.getLevel().health - this.castleGate.currentHealth;
                    var workToRestoreHit = NumberUtils.ceil(this.castleGate.workToDo / healthToRestore);
                    var hits = NumberUtils.floor(this.castleGate.workDone / workToRestoreHit);
                    var work = hits * workToRestoreHit;
                    this.castleGate.workDone -= work;
                    this.castleGate.workToDo -= work;
                    this.castleGate.currentHealth += hits;
                }
            }
        }
	},
	
	fillSpecialMilitaryInfo: function(info)
	{
        info.gateLevel = !this.castleGate ? 0 : this.castleGate.level;
        info.gateHealth = !this.castleGate ? 0 : this.castleGate.currentHealth;
    }
});

FCastle.GATE_FAKE_CELL_INDEX = -2;
