function FCity()
{
	callParentConstructor(FCity.prototype, this);

	this.__defineGetter__('isCity', function()
	{
		return true;
	});

	this.__defineGetter__('ownCells', function()
	{
		return this.cells.length;
	});

	this.__defineGetter__('name', function()
	{
		return this._name == FCity.CITY_DEFAULT_NAME ? I18.f('CITY_DEFAULT_NAME') : this._name;
	});

	this.__defineSetter__('cityName', function(value)
	{
		this._name = value;
	});

	this.__defineGetter__('isFakeRegister', function()
	{
		return this._name == "" && this.owner.username == "Fake";
	});

	this._name = '';
	this.nextLevelCells = 0;
	this.isCapital = false;
	this.insideWallCellCount = 0;
	this.extraCellsCount = 0;
	this.requestRename = false;
	this.skinIndex = 0;
	this.nextCorruptionTick = 0;
};

extendClass(FCity.prototype, FEstate.prototype);

FCity.prototype.__className__ = "FCity";

FCity.CITY_DEFAULT_NAME = "%CITY_DEFAULT_NAME%";

FCity.prototype.canManageOnSectors = function()
{
	return true;
}

FCity.prototype.isOnSector = function(cellIndex)
{
	return cellIndex < this.CELLS_ON_SECTORS;
}

FCity.prototype.getVisualNumber = function(cellIndex)
{
	if (this.isOnSector(cellIndex))
	{
		return cellIndex % this.CELLS_PER_SECTOR + 1;
	}
	return superMethod(FCity, "getVisualNumber", this)(this.getIndexWithoutSectors(cellIndex));
}

FCity.prototype.getMaxCellIndex = function()
{
	return this.ownCells + this.CELLS_ON_SECTORS;
}

FCity.prototype.isMaxHallLevel = function()
{
	return this.insideWallCellCount == this.ownCells;
}

FCity.prototype.areSectorsReachable = function()
{
	return this.isMaxHallLevel() && this.isGateOpen() && this.isSectorsScienceLearned();
}

FCity.prototype.isInsideCityWall = function(cellIndex)
{
	return cellIndex >= this.CELLS_ON_SECTORS && this.getIndexWithoutSectors(cellIndex) < this.insideWallCellCount;
}

FCity.prototype.isCellReachable = function(cellIndex)
{
	return this.isOnSector(cellIndex) ? this.areSectorsReachable() : this.isInsideCityWall(cellIndex);
}

FCity.prototype.getInsideWallCellCount = function()
{
	return this.insideWallCellCount;
}

FCity.prototype.getSkinIndex = function()
{
	return this.skinIndex;
}

FCity.prototype.canUpgradeHall = function()
{
	return this.accessDeputy && !this.hall.isBuildingOrUpgrading && !this.isMaxHallLevel();
}

FCity.prototype.canManageWhenGateClosed = function(c)
{
	return c.onSector ? this.isGateOpen() : true;
}

FCity.prototype.canManageOnSector = function(cell)
{
	return cell.onSector ? this.isMaxHallLevel() : true;
}

FCity.prototype.canDeleteBuilding = function(c)
{
	return superMethod(FCity.prototype, 'canDeleteBuilding', this)(c) && !c.building.isForeign;
}

FCity.prototype.canManageWorkers = function(c)
{
	return this.canManageOnSector(c) && this.canManageWhenGateClosed(c) && superMethod(FCity.prototype, 'canManageWorkers', this)(c);
}

FCity.prototype.canManageBuilders = function(c)
{
	return this.canManageOnSector(c) && this.canManageWhenGateClosed(c) && superMethod(FCity.prototype, 'canManageBuilders', this)(c);
}

FCity.prototype.getTotalRemainder = function()
{
	var remainder = new Funds();
	var cellulars = Arrays.clone(this.sectors);
	cellulars.push(this);
	for (var i = 0; i < cellulars.length; i++)
	{
		var cellular = cellulars[i];
		for (var j = 0; j < cellular.cells.length; j++)
		{
			var cell = cellular.cells[j];
			if (!cell.resource.isNull())
			{
				remainder.addResource(cell.resource, cell.remainder);
			}
		}
	}
	return remainder;
}

FCity.prototype.getRemainderResourcesToHide = function()
{
	return [Resource.GOLD, Resource.WOOD];
}

FCity.prototype.isCellReachable = function(cellIndex)
{
	return this.isOnSector(cellIndex) ? this.areSectorsReachable() : this.isInsideCityWall(cellIndex);
}

FCity.prototype.getCellLocationText = function(cellIndex)
{
	if (this.isOnSector(cellIndex))
	{
		return I18.f('CELL_POSITION_SECTOR', this.getVisualNumber(cellIndex), Math.floor(cellIndex / this.CELLS_PER_SECTOR) + 1);
	}
	return superMethod(FCity.prototype, 'getCellLocationText', this)(cellIndex);
}

FCity.prototype.isExtraCell = function(cellIndex)
{
	return (this.ownCells - this.getIndexWithoutSectors(cellIndex)) <= this.extraCellsCount;
}

FCity.prototype.hasCompletedBuilding = function()
{
	return this.ownedBuildings.length > 1 && this.ownedBuildings[1].state.name == BuildingState.NORMAL.name;
}
// todo: take this info from server
FCity.prototype.isFirstEnteringToFirstCity = function()
{
	return this.requestRename && this.isCapital && this.accessManageble && this.links.length == 1 && this.science.getLevel(Science.COLONIZATION) == 0;
}

FCity.prototype.getInsideWallCellCount = function()
{
	return this.insideWallCellCount;
}

FCity.prototype.getNextLevelInsideWallCellCount = function()
{
	return this.nextLevelCells;
}

FCity.prototype.getEstateType = function()
{
	return EstateType.CITY;
}
