function FClan()
{
	this.__defineGetter__('size', function()
	{
		return this.members.length + this.invitees.length;
	});
	
	this.__defineGetter__('allUsers', function()
	{
	    var ret = [];
	    for (var i = 0, len = this.members.length; i < len; i++) ret.push(this.members[i]);
	    for (var i = 0, len = this.invitees.length; i < len; i++) ret.push(this.invitees[i]);
	    return ret;
	});
	
	this.clanName = '';
	this.type = null; // ClanType
	this.leaderName = '';
	this.members = []; // ArrayCollection//FClanMember
	this.invitees = []; // ArrayCollection//FClanMember
	this.isRegistered = false;
	this.hasReservation = false;
	this.hasWarehouse = false;
	this.destructionTick = 0;
	this.level = 0;
	this.donation = 0;
	//todo (AB) fields can be calculated by clan level
	this.capacity = 0;
	this.maxDonation = 0;
	this.nextLevelCapacity = 0;
	this.donationPearl = 0;
	this.nextLevelPearl = 0;
	this.castles = []; // ArrayCollection//XYZ
	this.questions = []; // ArrayCollection//FQuestion
	this.lastQuestionStartTick = 0;
	this.hasDiplomatHall = false;
	this.diplomacy = null; // FClanDiplomacy
	this.nearestCastleAttackTick = 0;
	
}

FClan.prototype.clanName = '';
FClan.prototype.type = null; // ClanType
FClan.prototype.leaderName = '';
FClan.prototype.members = []; // ArrayCollection//FClanMember
FClan.prototype.invitees = []; // ArrayCollection//FClanMember
FClan.prototype.isRegistered = false;
FClan.prototype.hasReservation = false;
FClan.prototype.hasWarehouse = false;
FClan.prototype.destructionTick = 0;
FClan.prototype.level = 0;
FClan.prototype.donation = 0;
//todo (AB) fields can be calculated by clan level
FClan.prototype.capacity = 0;
FClan.prototype.maxDonation = 0;
FClan.prototype.nextLevelCapacity = 0;
FClan.prototype.donationPearl = 0;
FClan.prototype.nextLevelPearl = 0;
FClan.prototype.castles = []; // ArrayCollection//XYZ
FClan.prototype.questions = []; // ArrayCollection//FQuestion
FClan.prototype.lastQuestionStartTick = 0;
FClan.prototype.hasDiplomatHall = false;
FClan.prototype.diplomacy = null; // FClanDiplomacy
FClan.prototype.nearestCastleAttackTick = 0;

FClan.prototype.__construct = function()
{
    this.clanName = htmlEscape(this.clanName);
}

FClan.prototype.isInDestruction = function()
{
    return this.destructionTick > 0;
}

FClan.prototype.canBeUpgraded = function()
{
    return this.maxDonation > 0;
}

FClan.prototype.getMember = function(username)
{
    if (this.members) for (var i = 0, len = this.members.length; i < len; i++) 
    {
    	var fcm = this.members[i];
        if (FUser.isSameName(fcm.user.username, username)) 
        {
            return fcm;
        }
    }
    return null;
}

FClan.prototype.canSendInvitation = function(invitor)
{
    var member = this.getMember(invitor);
    
    if (member) 
    {
    	if (this.type.name == ClanType.MONARCHICAL.name)
    	{
    		return member.role && member.role.canInviteAndExpel;
    	}
    	
    	if (this.type.name == ClanType.DEMOCRATIC.name)
    	{
    		return !this.findInvitee(invitor);
    	}
    }
    
    return false;
}

FClan.prototype.canTakeAwayInvitation = function(invitor, invitee)
{
    var member = this.getMember(invitor);
    
    if (member) 
    {
    	if (this.type.name == ClanType.MONARCHICAL.name)
    	{
    		return member.role && member.role.canInviteAndExpel;
    	}
    	
    	if (this.type.name == ClanType.DEMOCRATIC.name)
    	{
    		return invitee.invitor == invitor && !this.findQuestion(QuestionType.INVITE_MEMBER, invitor);
    	}
    }
    
    return false;
}

FClan.prototype.canExpel = function(initiator, user)
{
    var member = this.getMember(initiator);
    
    if (member) 
    {
    	if (this.type.name == ClanType.MONARCHICAL.name)
    	{
    		return member.role && member.role.canInviteAndExpel;
    	}
    	
    	if (this.type.name == ClanType.DEMOCRATIC.name)
    	{
    		return !this.findQuestion(QuestionType.EXPEL_MEMBER, initiator) && !this.findQuestionByParam(QuestionType.EXPEL_MEMBER, "user", user.user.username);
    	}
    }
    
    return false;
}

FClan.prototype.findQuestion = function(type, owner)
{
    if (this.questions) for (var i = 0, len = this.questions.length; i < len; i++)
    {
    	var question = this.questions[i];
        if (question.type.name == type.name && (!owner || (question.owner && question.owner.user.username == owner))) 
        {
            return question;
        }
    }
    return null;
}

FClan.prototype.countQuestions = function(type)
{
    var count = 0;
    if (this.questions) for (var i = 0, len = this.questions.length; i < len; i++)
    {
    	var question = this.questions[i];
        if (question.type.name == type.name) count++;
    }
    return count;
}

FClan.prototype.findQuestionByParam = function(type, param, value)
{
    if (this.questions) for (var i = 0, len = this.questions.length; i < len; i++)
    {
    	var question = this.questions[i];
        if (question.type.name == type.name && question.params && question.params[param] == value) 
        {
            return question;
        }
    }
    return null;
}

FClan.prototype.findInvitee = function(invitor)
{
    if (this.invitees) for (var i = 0, len = this.invitees.length; i < len; i++)
    {
    	var member = this.invitees[i];
        if (member.invitor == invitor) 
        {
            return member;
        }
    }
    return null;
}
