function FEstate()
{
	this.__defineGetter__('consumptionDeficit', function()
	{
	    return this._consumptionDeficit;
	});
	this.__defineGetter__('name', function()
	{
	    return I18.getEstateTypeName(this.getEstateType());
	});
	this.__defineGetter__('freeCitizens', function()
	{
	    return this.bank.people;
	});
	this.__defineGetter__('plodders', function()
	{
	    return this.builders + this.workers;
	});
	this.__defineGetter__('totalCitizens', function()
	{
	    return this.freeCitizens + this.plodders;
	});
	this.__defineGetter__('race', function()
	{
	    return !this.owner ? FUser.CURRENT_USER.race : this.owner.race;
	});
	this.__defineGetter__('science', function()
	{
	    return !this.owner ? FUser.CURRENT_USER.science : this.owner.science;
	});
	this.__defineGetter__('lastTaxPerHour', function()
	{
	    return this._lastTaxPerHour;
	});
	this.__defineGetter__('lastNominalTaxPerHour', function()
	{
	    return this._lastNominalTaxPerHour;
	});
	this.__defineGetter__('isGarrisonBlocked', function()
	{
	    return Tick.now() <= this.garrisonUnblockTick;
	});
	this.__defineGetter__('terrainProductivityBonus', function()
	{
	    return FConfig.getTerrainConfig(this.terrain).productivity;
	});
	this.__defineGetter__('isCity', function()
	{
	    return false;
	});
	this.__defineGetter__('bank', function()
	{
	    return this._bank;
	});
	this.__defineSetter__('bank', function(value) 
	{
	    if (value instanceof FundsBuilder) 
	    {
	        this._bank = value.funds;
	    } 
	    else 
	    {
	        this._bank = value;
	    }
	});
	this.__defineGetter__('isFakeRegister', function()
	{
	    return false;
	});
	this.__defineGetter__('garrisonHeroId', function()
	{
	    return this.garrisonHero ? this.garrisonHero.id : -1;
	});
	this.__defineGetter__('buildersBonusOverflow', function()
	{
	    return NumberUtils.equals(this.effectiveBuilders(1), 1.0 / FConfig.MIN_CONSTRUCTION_RATE);
	});
	this.__defineGetter__('accessClanDiplomacy', function()
	{
	    return this.accessOrderUnit;
	});
	this.__defineGetter__('isEstateUnderAttack', function()
	{
	    return Tick.ticksTill(this.nearestIncomingAttackTick) <= FConfig.BEFORE_ATTACK_BLOCK;
	});
	this.__defineGetter__('isEstateUnderSiege', function()
	{
	    return Tick.ticksTill(this.nearestIncomingAttackTick) <= FConfig.BEFORE_ATTACK_FEATURES_BLOCK;
	});
};

extendClass(FEstate.prototype, FCellular.prototype);

FEstate.prototype.__className__ = "FEstate";

FEstate.CELLS_PER_SECTOR = 8;
FEstate.SECTORS_COUNT = 4;
FEstate.CELLS_ON_SECTORS = FEstate.CELLS_PER_SECTOR * FEstate.SECTORS_COUNT;
FEstate.HIDEOUT_ACCESS_LEVEL = 4;
FEstate.GARRISON_ACCESS_LEVEL = 3;
FEstate.CASTS_ACCESS_LEVEL = 3;
FEstate.SCIENCE_ACCESS_LEVEL = 3;
FEstate.BUILDING_TYPE_ACCESS_LEVEL = 2;
FEstate.BUILDING_INFO_ACCESS_LEVEL = 4;
FEstate.HALL_FAKE_CELL_INDEX = -1;

FEstate.prototype.CELLS_PER_SECTOR = FEstate.CELLS_PER_SECTOR;
FEstate.prototype.SECTORS_COUNT = FEstate.SECTORS_COUNT;
FEstate.prototype.CELLS_ON_SECTORS = FEstate.CELLS_ON_SECTORS;
FEstate.prototype.HIDEOUT_ACCESS_LEVEL = FEstate.HIDEOUT_ACCESS_LEVEL;
FEstate.prototype.GARRISON_ACCESS_LEVEL = FEstate.GARRISON_ACCESS_LEVEL;
FEstate.prototype.CASTS_ACCESS_LEVEL = FEstate.CASTS_ACCESS_LEVEL;
FEstate.prototype.SCIENCE_ACCESS_LEVEL = FEstate.SCIENCE_ACCESS_LEVEL;
FEstate.prototype.BUILDING_TYPE_ACCESS_LEVEL = FEstate.BUILDING_TYPE_ACCESS_LEVEL;
FEstate.prototype.BUILDING_INFO_ACCESS_LEVEL = FEstate.BUILDING_INFO_ACCESS_LEVEL;
FEstate.prototype.HALL_FAKE_CELL_INDEX = FEstate.HALL_FAKE_CELL_INDEX;

FEstate.prototype.terrain = null; // Terrain
FEstate.prototype.blamba = ''; // String

FEstate.prototype.id = null; // EstateId

FEstate.prototype.ownedBuildings = null; // Array    // FBuilding[]
FEstate.prototype.foreignBuildings = null; // Array  // FBuilding[]
FEstate.prototype._buildingByTypeCache = new FMap(); // FMap = new FMap()  // transient Map<BType, FCityBuilding[]>

FEstate.prototype.location = null; // XYZ
FEstate.prototype.links = null; // Array  // FKingdomEstateLink[]

FEstate.prototype.owner = null; // FUser

FEstate.prototype._bank = null; // Funds
FEstate.prototype.capacity = null; // Funds
FEstate.prototype.hiddenResourcesCapacity = null; // Funds

FEstate.prototype.hall = null; // FBuilding
FEstate.prototype.builders = 0; // int
FEstate.prototype.workers = 0; // int
FEstate.prototype.buildingsInMaintenanceCount = 0; // int
FEstate.prototype.repairingBuildingsCount = 0; // int
FEstate.prototype.shopRatioPerHour = 0; // Number
FEstate.prototype.resourceDrainReduction = 0; // Number
FEstate.prototype.peopleDrainReduction = 0; // Number

FEstate.prototype.hideout = null; // Army
FEstate.prototype.garrison = null; // Army
FEstate.prototype.garrisonBonuses = null; // Array
FEstate.prototype.garrisonUnblockTick = null; // Number
FEstate.prototype.garrisonHero = null; // FHero
FEstate.prototype.garrisonFaded = null; // Boolean
FEstate.prototype.heroes = null; // ArrayCollection
FEstate.prototype.maxDefenceFinishTick = null; // Number
FEstate.prototype.buffs = null; // ArrayCollection//List<FBuff>

FEstate.prototype.destructionTick = null; // Number

FEstate.prototype.drainCalculator = new FDrainCalculator(); // FDrainCalculator = new FDrainCalculator()
FEstate.prototype.retreatLimit = 0; // int
FEstate.prototype.sectors = null; // Array // FSector[]

FEstate.prototype.assistances = null; // ArrayCollection // List<List<FMission>>
FEstate.prototype.assistantBonuses = null; // ArrayCollection // List<AuraBonus[]>

FEstate.prototype.gateState = null; // GateState
FEstate.prototype.gateChangeTick = 0; // Number
FEstate.prototype.gateClosingByMagic = false; // Boolean
FEstate.prototype.gateAfterCityMove = false; // Boolean

FEstate.prototype.nearestIncomingAttackTick = 0; // Number

// accessDeputy is true for owner
FEstate.prototype.accessManageble = false; // Boolean
FEstate.prototype.accessWarManageble = false; // Boolean
FEstate.prototype.accessOrderUnit = false; // Boolean
FEstate.prototype.accessDeputy = false; // Boolean
FEstate.prototype.accessMail = false; // Boolean
FEstate.prototype.accessHideout = false; // Boolean
FEstate.prototype.accessGarrison = false; // Boolean
FEstate.prototype.accessScience = false; // Boolean
FEstate.prototype.accessInsideView = false; // Boolean
FEstate.prototype.accessCombo = false; // Boolean
FEstate.prototype.accessMissions = false; // Boolean
FEstate.prototype.accessCasts = false; // Boolean
FEstate.prototype.isEspionage = false; // Boolean
FEstate.prototype.isAlly = false; // Boolean
// for espionage
FEstate.prototype.canLoot = null; // Funds
FEstate.prototype.lootFromWarehouses = null; // Funds
FEstate.prototype.lootFromMarket = null; // Funds
FEstate.prototype.lootFromClanBuildings = null; // Funds
FEstate.prototype.lootDiscount = 0; // int

FEstate.prototype.killedUnitsCostAttacker = 0; // Number
FEstate.prototype.killedUnitsCostDefender = 0; // Number

FEstate.prototype.auctionBanTill = 0; // Number

FEstate.prototype.isExpeditionReport = false; // Boolean
FEstate.prototype.mailBoxStatus = null; // FMailBoxStatus

FEstate.prototype.buildQueue = null; // ArrayCollection//<BuildOrder>
FEstate.prototype.queueRequire = null; // Funds
FEstate.prototype.queueRequirePearl = 0; // int

FEstate.prototype.reason = ''; // String   // for debug

FEstate.prototype._consumptionBuffer = 0; // Number = 0  // transient
FEstate.prototype._consumptionDeficit = 0; // Number  // transient
FEstate.prototype._lastNominalTaxPerHour = 0; // Number  // transient
FEstate.prototype._lastTaxPerHour = 0; // Number  // transient

FEstate.prototype._cacheConsumptionPerHour = 0; // Number  // transient
FEstate.prototype._cacheGoldProductionPerTick = 0; // Number  // transient
FEstate.prototype._cacheTotalShopProductionPerHour = 0; // Number  // transient

FEstate.prototype._fromMail = false; // Boolean = false // transient

FEstate.prototype.prolongationOffer = 0; // Number
FEstate.prototype.prolongationOfferCost = 0; // Number

FEstate.prototype.lastCastTick = 0; // Number
FEstate.prototype.casts = null; // ArrayCollection //<FCast>

FEstate.prototype.flags = null; // Flags //<Boolean>
FEstate.prototype.flagSetsCount = 0; // int

FEstate.prototype.servicesValues = null; // FMap // <String, Long>
FEstate.prototype.servicesAutoOrder = null; // ArrayCollection // <String>
FEstate.prototype.friendOwnerIsInactive = false; // Boolean

FEstate.prototype.canPerformOneMoreTask = function()
{
	return this.buildingsInMaintenanceCount < this.getMaxConcurrentBuildings();
}

FEstate.prototype.getMaxConcurrentBuildings = function()
{
	return Science.getMaxConcurrentBuildings(this.owner.getScienceValue(Science.BUILDING));
}

FEstate.prototype.canManageOnSectors = function()
{
	return false;
}

FEstate.prototype.getCellByIndex = function(cellIndex)
{
	if (cellIndex == this.HALL_FAKE_CELL_INDEX)
	{
		return this.getFakeCell(this.hall);
	}

	if (!this.sectors)
	{
		return superMethod(FEstate, "getCellByIndex", this)(cellIndex);
	}

	if (cellIndex < this.CELLS_ON_SECTORS)
	{
		var index = Math.floor(cellIndex / this.CELLS_PER_SECTOR);
		return this.sectors[index].getCellByIndex(cellIndex % this.CELLS_PER_SECTOR);
	}
	else
	{
		return superMethod(FEstate, "getCellByIndex", this)(this.getIndexWithoutSectors(cellIndex));
	}
}

FEstate.prototype.getOrderByCellIndex = function(index)
{
	if (this.buildQueue)
	{
		for (var i = 0; i < this.buildQueue.length; i++)
		{
			if (this.buildQueue[i].cellIndex == index)
			{
				return this.buildQueue[i];
			}
		}
	}
	return null;
}

FEstate.prototype.getFakeCell = function(b)
{
	var fakeCell = new FCell();
	fakeCell.building = b;
	return fakeCell;
}

FEstate.prototype.getIndexWithoutSectors = function(cellIndex)
{
	return cellIndex - this.CELLS_ON_SECTORS;
}

FEstate.prototype.hasBuff = function(type, ut)
{
	return this.buffs == null ? false : FBuff.hasBuff(this.buffs, type, ut, FEstate.tickNowForEstate(this));
}

FEstate.prototype.getOwnedBuildings = function(type)
{
	var ret = [];

	if (!type)
	{
		return this.ownedBuildings;
	}

	for (var i = 0; i < this.ownedBuildings.length; i++)
	{
		if (type.name == this.ownedBuildings[i].typeName)
			ret.push(this.ownedBuildings[i]);
	}

	return ret;
}

FEstate.prototype.countBuildings = function(type, withQueue)
{
	var ownedBuildingCount = this.getOwnedBuildings(type).length;
	if (withQueue)
	{
		ownedBuildingCount += this.countBuildingsInQueue(type);
	}
	return ownedBuildingCount;
}

FEstate.prototype.countBuildingsInQueue = function(type)
{
	if (!type) return this.buildQueue.length;
	var self = this;
	return this.buildQueue.filter(function(b){
		return (type.name == b.type.name) && (self.getCellByIndex(b.cellIndex).hasBuildingWithType(type));
	}).length;
}

FEstate.prototype.isGateOpen = function()
{
	return this.gateState != null && this.gateState.isOpen();
}

FEstate.prototype.isSectorsScienceLearned = function()
{
	// if we don't know science level in espionage -- act as if owner knows SECTORS
	if (this.isEspionage)
	{
		if (this.accessScience)
		{
			return this.science.getLevel(Science.SECTORS) > 0;
		}
		else
		{
			return true;
		}
	}
	else
	{
		return this.science.getLevel(Science.SECTORS) > 0;
	}
}

FEstate.prototype.equals = function(other)
{
	return other && this.id.equals(other.id);
}

FEstate.prototype.getInsideWallCellCount = function()
{
	return this.cells.length;
}

FEstate.prototype.hasCell = function(cellIndex)
{
	if (cellIndex < 0)
	{
		return false;
	}
	if (this.sectors)
	{
		return cellIndex < this.CELLS_ON_SECTORS || this.getIndexWithoutSectors(cellIndex) < this.cells.length;
	}
	else
	{
		return cellIndex < this.cells.length;
	}
}

FEstate.prototype.getSkinIndex = function()
{
	return -1;
}

FEstate.prototype.getTotalProductionSpeedPerHour = function()
{
	var res = new Funds();
	var buildings = this.getOwnedBuildings();
	$.each(buildings, function(n, b)
	{

		if (b.type.isExtractor() || b.type.isProducer())
		{
			res.add(b.getLastWorkSpeedPerHour());
		}
		if (b.type.isHouses() || b.type.isEstateHall())
		{
			res.addResource(Resource.PEOPLE, b.getPeopleProductionSpeedPerHour());
		}

	});

	var consumption = this.getConsumptionPerHour();
	res.subtractResource(Resource.GOLD, consumption);
	res.addResource(Resource.GOLD, this._lastTaxPerHour);
	res.addResource(Resource.PEOPLE, this.getPeopleSelfreproductionPerHour());

	var reduced = new Funds();
	if (this.drainCalculator != null)
	{
		reduced = this.drainCalculator.getDrainPerHour(this).ceil();
	}
	else
	{
		var self = this;
		$.each(Resource.FUNDS_RESOURCES, function(n, r)
		{
			reduced.setValue(r, self.getVanishPerHour(r));
		})
	}

	res.subtract(reduced.ceil(), false);

	var buff;
	$.each(this.buffs, function(n, buff)
	{
		if (buff.buffType == BuffType.GIVE_FUNDS_PER_HOUR)
		{
			res.add(buff.getConsumedFunds());
		}
		if (buff.buffType == BuffType.TAKE_FUNDS_PER_HOUR)
		{
			res.subtract(buff.getConsumedFunds(), false);
		}
	})

	return res;
}

FEstate.prototype.getCapacityBonus = function(type)
{
	var result = 0;
	if (type.isWarehouse())
	{
		result += Science.getWarehouseCapacityBonus(this.science.getLevel(Science.WAREHOUSE));
	}
	if (type.isWarehouse() || type.isClanWarehouse())
	{
		result += this.sumOfBuildingBuffs(BuffType.BOOST_BUILDING, type);
	}
	return result;
}

FEstate.prototype.isOverfilled = function(r)
{
	return !NumberUtils.isPositive(this.getBankFreeSpaceForResource(r));
}

FEstate.prototype.getBankFreeSpaceForResource = function(r)
{
	if (r.name == Resource.PEOPLE.name)
	{
		return Math.max(this.capacity.people - this.totalCitizens, 0);
	}
	else
	{
		return Math.max(this.capacity.getValue(r) - this.bank.getValue(r), 0);
	}
}

FEstate.prototype.getStoppingBuff = function(btype)
{
	if (this.buffs)
	{
		for (var i = 0; i < this.buffs.length; i++)
		{
			var buff = this.buffs[i];
			if (buff.buffType == BuffType.STOP_BUILDING && buff.btype == btype)
			{
				return buff;
			}
		}
	}
	return null;
}

FEstate.prototype.getConsumptionPerHour = function()
{
	if (isNaN(this._cacheConsumptionPerHour))
	{
		if (this.hasBuff(BuffType.NO_ARMY_CONSUMPTION) || this.owner.isOnVacation() || FConfig.NO_ARMY_CONSUMPTION)
		{
			this._cacheConsumptionPerHour = 0;
		}
		else
		{
			this._cacheConsumptionPerHour = this.computeConsumptionSpeedPerHour(this.garrison) + this.computeConsumptionSpeedPerHour(this.hideout);
		}
	}
	return this._cacheConsumptionPerHour;
}

FEstate.prototype.computeConsumptionSpeedPerHour = function(a)
{
	var treasury = this.science.getSpecialValue(Science.TREASURY) / 100.0;
	var result = 0;
	for (var i = 0; i < UnitType.MILITARY.length; i++)
	{
		var ut = UnitType.MILITARY[i];
		var count = a.getValue(ut);
		if (count > 0)
		{
			var base = FConfig.getStat(this.race, ut, this.science.levelOf(ut)).consume;
			var localConsumption = 1 + this.sumOfBuffs(BuffType.ARMY_CONSUMPTION, ut);
			var globalConsumption = 1 + this.owner.maxUnitTypeBuff(BuffType.GLOBAL_ARMY_CONSUMPTION, ut, FEstate.tickNowForEstate(this));
			result += base * count * (1 - treasury) * Math.max(FConfig.MIN_ARMY_CONSUMPTION, localConsumption * globalConsumption);
		}
	}
	return result;
}

FEstate.prototype.getWarehouseCapacitors = function()
{
	var res = [];

	res.push(this.hall);
	var buildings = this.getOwnedBuildings(BType.WAREHOUSE);
	for (var i = 0; i < buildings.length; i++)
	{
		res.push(buildings[i]);
	}
	return res;
}

FEstate.prototype.isGateClosed = function()
{
	return this.gateState != null && this.gateState.isClosed();
}

FEstate.prototype.shouldShowDestructionWarning = function()
{
	return Tick.ticksTill(this.destructionTick) <= FConfig.OUTER_ESTATE_DESTRUCTION_WARNING;
}

FEstate.prototype.effectiveBuilders = function(b)
{
	var speedUp = this.science.getSpecialValue(Science.SPEC_ARCHITECT) / 100;
	return b / Math.max(FConfig.MIN_CONSTRUCTION_RATE, 1 + this.sumOfBuffs(BuffType.CONSTRUCTION_TIME) - speedUp);
}

FEstate.prototype.sumOfBuffs = function(bt, ut)
{
	return !this.buffs ? 0 : FBuff.sumOfBuffs(this.buffs, bt, ut, FEstate.tickNowForEstate(this));
}

FEstate.prototype.tick = function(delta)
{
    if (!this.isExpeditionReport)
    {
        for(var i=0; i<this.ownedBuildings.length; i++)
        {
        	this.ownedBuildings[i].tick(delta);
        }
        
        this.clearGoldCache();

        this._consumptionBuffer = delta * this.getConsumptionPerHour() / Tick.TICKS_PER_HOUR;
        
        if(this.foreignBuildings)
        {
	        for(var i=0; i<this.foreignBuildings.length; i++)
	        {
	        	this.foreignBuildings[i].tick(delta);
	        }
        }

        var goldDeficit = 0;
        var buff;
        
        for(var i=0; i<this.buffs.length; i++)
        {
        	buff = this.buffs[i];
        	if (buff.buffType.name == BuffType.TAKE_FUNDS_PER_HOUR.name)
        	{
                var funds = buff.funds.multiplyCopy(delta / Tick.TICKS_PER_HOUR);
                var taken = this.subtractFundsFromBank(funds);
                goldDeficit += funds.gold - taken.gold;
                taken.multiply(Tick.TICKS_PER_HOUR / delta);
                buff.setConsumedFunds(taken);
            }
        }
        
        this.collectTax(delta);

        // tick people self-production
        var peoplePerTick = this.getPeopleSelfreproductionPerHour() / Tick.TICKS_PER_HOUR;
        this.addResourceToBank(Resource.PEOPLE, peoplePerTick * delta);
        var given;
        for(var i=0; i<this.buffs.length; i++)
        {
        	buff = this.buffs[i];
            if (buff.buffType.name == BuffType.GIVE_FUNDS_PER_HOUR.name)
            {
                funds = buff.funds.multiplyCopy(delta / Tick.TICKS_PER_HOUR);
                given = this.addFundsToBank(funds);
                given.multiply(Tick.TICKS_PER_HOUR / delta);
                buff.setConsumedFunds(given);
            }
        }

        // tick consumption and corruption
        this._consumptionDeficit = Math.min(this._consumptionBuffer, this.bank.extractAndReturnOverflow(Resource.GOLD, this._consumptionBuffer + goldDeficit));
        if (this.accessDeputy)
        {
            this.drainCalculator.tick(this, delta);
        }
        
        if (this.canStartNextBuilding())
        {
			this.nextUpdateTick = Tick.now();
		}

        if (this.garrisonHero && !this.isEspionage)
        {
            this.garrisonHero.tick(delta);
        }
    }
}

FEstate.prototype.canStartNextBuilding = function()
{
    if (!this.canPerformOneMoreTask())
    {
        return false;
    }

    if(!this.buildQueue || this.buildQueue.length == 0)
    {
        return false;
    }

    var order = this.buildQueue[0];
    var cell = this.getCellByIndex(order.cellIndex);
    
    if (cell.onSector && !this.gateState.isOpen())
    {
        return false;
    }

    var params = order.type.getParams(this.race);
    var nextLevel = !cell.building ? 1 : cell.building.level + 1;
    if (!params || !params.levels || params.levels.length <= nextLevel)
    {
        return false;
    }

    var bLevel = params.levels[nextLevel];
    
    if (!this.bank.greaterOrEqualsTo(bLevel.cost))
    {
        return false;
    }

    if (this.owner.pearl < bLevel.pearl)
    {
        return false;
    }

    if (!FEstate.haveEnoughCrystals(bLevel.artCost, CrystalType.getCrystals(this)))
    {
        return false;
    }

    if (this.hasBuff(BuffType.CONSTRUCTION_CHAOS))
    {
        return false;
    }
    
    return true;
}

FEstate.haveEnoughCrystals = function(artCost, currentCounts)
{
    if(artCost)
    {
        var ct;
        for(var i=0; i<CrystalType.ALL.length; i++)
        {
        	ct = CrystalType.ALL[i];
        	if (ct.getOrdinal() < artCost.length && artCost[ct.getOrdinal()] != 0)
        	{
                var have = currentCounts[ct.getOrdinal()];
                var need = artCost[ct.getOrdinal()];
                if (need > have)
                {
                    return false;
                }
            }
        }
    }
    return true;
}

FEstate.prototype.addResourceToBank = function(r, value)
{
    var servedForConsumption = 0;
    if (Resource.GOLD.equals(r))
    {
        var oldConsumptionBuffer = this._consumptionBuffer;
        var diff = this._consumptionBuffer - value;
        this._consumptionBuffer = Math.max(0, diff);
        value = diff < 0 ? -diff : 0;
        servedForConsumption = oldConsumptionBuffer - this._consumptionBuffer;
    }
    var freeSpace = this.getBankFreeSpaceForResource(r);
    var addedToBank = freeSpace > 0 ? Math.min(freeSpace, value) : 0;
    this.bank.addResource(r, addedToBank);
    return addedToBank + servedForConsumption;
}

FEstate.tickNowForEstate = function(e)
{
	return (e != null && (e.isEspionage || e.isExpeditionReport)) ? e.responseTick : Tick.now();
}

//-----------------------------------------------------------------------------------------------------------

FEstate.prototype.collectTax = function(delta)
{
    this._lastNominalTaxPerHour = this.computeTaxPerHour();
    var tax = delta * this._lastNominalTaxPerHour / Tick.TICKS_PER_HOUR;
    var taxAddedToBank = this.addResourceToBank(Resource.GOLD, tax);
    this._lastTaxPerHour = NumberUtils.ceil(taxAddedToBank * Tick.TICKS_PER_HOUR / delta);
}

FEstate.prototype.clearGoldCache = function()
{
    this._cacheConsumptionPerHour = NaN;
    this._cacheGoldProductionPerTick = NaN;
    this._cacheTotalShopProductionPerHour = NaN;
}

FEstate.prototype.addFundsToBank = function(funds)
{
    var consumedFunds = new Funds();
    var r;
    for(var i=0; i<Resource.FUNDS_RESOURCES.length; i++)
    {
    	r = Resource.FUNDS_RESOURCES[i];
    	var value = funds.getValue(r);
        if (value > 0)
        {
            consumedFunds.addResource(r, this.addResourceToBank(r, value));
        }
    }
    return consumedFunds;
}

FEstate.prototype.subtractFundsFromBank = function(funds)
{
    var consumedFunds = new Funds();
    
    if(this.resources)
    {
	    for (var i = 0; i < this.resources.length; i++)
	    {
	    	var r = this.resources[i];
	        var value = funds.getValue(r);
	        if (value > 0) 
	        {
	            consumedFunds.addResource(r, this.subtractResourceFromBank(r, value));
	        }
	    }
    }
    return consumedFunds;
}

FEstate.prototype.subtractResourceFromBank = function(r, value)
{
    var subtracted = Math.min(this.bank.getValue(r), value);
    this.bank.subtractResource(r, subtracted);
    return subtracted;
}

FEstate.prototype.getLink = function()
{
    for (var i = 0; i < this.links.length; i++)
    {
    	var l = this.links[i];
        if (this.id && this.links[i].estateInfo.id && this.id.equals(this.links[i].estateInfo.id)) 
        {
            return l;
        }
    }
    throw new Error("Could not find city link on self");
}

FEstate.prototype.getWorkingPlaces = function()
{
    var res = 0;
    var buildings = this.getOwnedBuildings();
    for (var i = 0; i < buildings.length; i++)
    {
    	var b = buildings[i];
        if (!b.isOnSector || this.isGateOpen()) 
        {
            if (b.type.isRequiresWorkers(this.race) && !b.isBroken && !b.isSuppressed) 
            {
                res += b.getLevel().workers - b.workers;
            }
            if (b.isBuildingOrUpgrading) 
            {
                res += b.getLevel(b.level + 1).builders - b.builders;
            }
            if (b.isRepairing) 
            {
                res += b.getLevel().builders - b.builders;
            }
        }
    }
    return res;
}

FEstate.prototype.canPerformOneMoreRepairing = function()
{
    return this.repairingBuildingsCount < this.getMaxConcurrentRepairings();
}

FEstate.prototype.getMaxConcurrentRepairings = function()
{
    return Science.getMaxConcurrentRepairings(this.owner.getScienceValue(Science.REPAIRING));
}

FEstate.prototype.getPeopleSelfreproductionPerHour = function(freeCitizens)
{
    if (typeof freeCitizens == undefined) freeCitizens = -1;
    var count = freeCitizens >= 0 ? freeCitizens : this.freeCitizens;
    var rate = count * FConfig.POPULATION_INCREASE_RATE;
    return !this.isOverfilled(Resource.PEOPLE) ? Math.floor(rate) : 0;
}

FEstate.prototype.getOverfilledResources = function()
{
    var res = [];
    for (var i = 0; i < Resource.WAREHOUSE_RESOURCES.length; i++)
    {
    	var r = Resource.WAREHOUSE_RESOURCES[i];
        if (this.isOverfilled(r)) 
        {
            res.push(r);
        }
    }
    return res;
}

FEstate.prototype.computeTaxPerHour = function()
{
    return NumberUtils.floor(this.shopRatioPerHour * this.freeCitizens);
}

FEstate.prototype.computeConsumptionSpeedPerTick = function(a)
{
    return this.computeConsumptionSpeedPerHour(a) / Tick.TICKS_PER_HOUR;
}

FEstate.prototype.hasMarket = function()
{
    return this.countBuildings(BType.MARKET) > 0;
}

FEstate.prototype.hasClanReservation = function()
{
    return this.countBuildings(BType.CLAN_RESERVATION) > 0;
}

FEstate.prototype.hasClanWarehouse = function()
{
    return this.countBuildings(BType.CLAN_WAREHOUSE) > 0;
}

FEstate.prototype.getMarket = function()
{
    var markets = this.getOwnedBuildings(BType.MARKET);
    if (markets.length > 0) 
    {
        return markets[0];
    }
    return null;
}

FEstate.prototype.getDiplomatHall = function()
{
    var diplomats = this.getOwnedBuildings(BType.DIPLOMAT);
    if (diplomats.length > 0) 
    {
        return diplomats[0];
    }
    return null;
}

FEstate.prototype.isInsideCityWall = function(cellIndex)
{
    throw new Error('abstractMethod');
}
FEstate.prototype.isCellReachable = function(cellIndex)
{
    throw new Error('abstractMethod');
}

FEstate.prototype.isOnSector = function(cellIndex)
{
    throw new Error('abstractMethod');
}

FEstate.prototype.hasSectorBonus = function(cellIndex)
{
    return this.canManageOnSectors() && this.isOnSector(cellIndex);
}

FEstate.prototype.canUpgradeBuilding = function(cellIndex)
{
    if (cellIndex == FEstate.HALL_FAKE_CELL_INDEX)
    {
    	return this.canUpgradeHall();
    }
    var c = this.getCellByIndex(cellIndex);
    return this.accessDeputy && c.hasBuilding() && !c.building.isForeign && c.building.canUpgrade() && this.isCellReachable(cellIndex);
}
FEstate.prototype.canDeleteBuilding = function(c)
{
    if (!c || !c.building) return false;
    return this.accessManageble && c.hasBuilding() && !c.building.type.isIndestructible(this.race);
}
FEstate.prototype.canManageWorkers = function(c)
{
    var b = c.building;
    return !b.isBroken && !b.isRepairing && b.isConstructed && !b.isSuppressed;
}
FEstate.prototype.canManageBuilders = function(c)
{
    var b = c.building;
    return (b.isBuildingOrUpgrading || b.isRepairing) && !b.isBroken;
}

FEstate.prototype.countFreeSpecialUnits = function(unitType)
{
    var total = 0;
    var buildings = this.getOwnedBuildings();
    for (var i = 0; i < buildings.length; i++)
    {
    	var b = buildings[i];
        if (b.type.isSpecialUnitProducer() && b.type.getUnitType(this.race).equals(unitType)) 
        {
            if (this.isGateOpen() || !b.isOnSector) 
            {
                total += b.unitsAvailable;
            }
        }
    }
    var buffs = this.getBuffs(BuffType.SUMMON);
    for (var i = 0; i < buffs.length; i++)
    {
    	var fb = buffs[i];
        if (unitType.equals(fb.unitType)) 
        {
            total += NumberUtils.floor(fb.value);
        }
    }
    return total;
}

FEstate.prototype.hasFreeSpecialUnits = function(unitType)
{
    return this.countFreeSpecialUnits(unitType) > 0;
}

FEstate.prototype.isBuildingOrUpgraiding = function()
{
    var buildings = this.getOwnedBuildings();
    for (var i = 0; i < buildings.length; i++)
    {
    	var b = buildings[i];
        if (b.isBuildPending || b.isUpgradePending) 
        {
            return true;
        }
    }
    return false;
}

FEstate.prototype.hasTraining = function(btype)
{
    var buildings = this.getOwnedBuildings();
    for (var i = 0; i < buildings.length; i++)
    {
    	var b = buildings[i];
        if (b.type.equals(btype) && b.unitsQueued > 0) 
        {
            return true;
        }
    }
    return false;
}

/* returns true if units are present in any building, including non-working */
FEstate.prototype.hasSpecialUnits = function(unitType)
{
    var buildings = this.getOwnedBuildings();
    for (var i = 0; i < buildings.length; i++)
    {
    	var b = buildings[i];
        if (b.type.isSpecialUnitProducer() && b.type.getUnitType(this.race).equals(unitType) && b.unitsAvailable > 0) 
        {
            return true;
        }
    }
    return false;
}

FEstate.prototype.getBonus = function(ut)
{
    for (var i = 0, im = UnitType.MILITARY.length; i < im; i++) 
    {
        if (UnitType.MILITARY[i] == ut) 
        {
            var bonus = new AuraBonus();
            bonus.damage = this.garrisonBonuses[i].damage;
            bonus.defence = this.garrisonBonuses[i].defence;
            bonus.health = this.garrisonBonuses[i].health;
            bonus.persecution = this.garrisonBonuses[i].persecution;
            return bonus;
        }
    }
    return new AuraBonus();
}

FEstate.prototype.notMaximalProductionResources = function()
{
    var result = [], tmp = [];
    var buildings = this.getOwnedBuildings();
    for (var i = 0; i < buildings.length; i++)
    {
    	var b = buildings[i];
        if ((b.type.isProducer() || b.type.isExtractor()) && b.hasSpeedProblems()) 
        {
            if (tmp.indexOf(b.resource.name) < 0) 
            {
                result.push(b.resource);
                tmp.push(b.resource.name)
            }
        }
    }
    return result;
}

FEstate.prototype.resourcesWithNegativeRaise = function(totalProductionSpeed)
{
    var result = [];
    for (var i = 0; i < Resource.FUNDS_RESOURCES.length; i++)
    {
    	var r = Resource.FUNDS_RESOURCES[i];
        if (NumberUtils.compareTo(totalProductionSpeed.getValue(r), 0) < 0) 
        {
            result.push(r);
        }
    }
    return result;
}

FEstate.prototype.getTotalRemainder = function()
{
    var res = new Funds();
    for (var i = 0; i < this.cells.length; i++)
    {
    	var cell = this.cells[i];
        if (!cell.resource.isNull()) 
        {
            res.addResource(cell.resource, cell.remainder);
        }
    }
    return res;
}

// used only for view
FEstate.prototype.getRemainderResourcesToHide = function()
{
    return [];
}

FEstate.prototype.getPeopleCapacitors = function()
{
    return Arrays.unionOf([this.hall], this.getOwnedBuildings(BType.HOUSES));
}

FEstate.prototype.getRaisersByResource = function(r)
{
    var res = [];
    var buildings = this.getOwnedBuildings();
    for (var i = 0; i < buildings.length; i++)
    {
    	var b = buildings[i];
        if (!b.type.isRecovery() && b.resource && b.resource.name == r.name) 
        {
            res.push(b);
        }
    }
    return res;
}

FEstate.prototype.isFullTaxUsed = function()
{
    return !this.bank || (this.freeCitizens > 0 && NumberUtils.equals(this._lastTaxPerHour, this._lastNominalTaxPerHour));
}

FEstate.prototype.isGateChanging = function()
{
    return this.gateState.name == GateState.OPENING_NOW.name || this.gateState.name == GateState.CLOSING_NOW.name;
}

FEstate.prototype.canUpgradeHall = function()
{
    return false;
}

FEstate.prototype.getNameAndLocation = function()
{
    return I18.formatEstateName(this.name, this.location);
}

FEstate.prototype.getProductivityBonus = function(cellIndex)
{
    var bonus = this.terrainProductivityBonus.multiplyCopy(1.0 / 100.0);
    var addSectorBonus = this.hasSectorBonus(cellIndex);
    var addPitBonus = (this instanceof FPit);
    for (var i = 0; i < Resource.FUNDS_RESOURCES.length; i++)
    {
    	var r = Resource.FUNDS_RESOURCES[i];
        bonus.addResource(r, addSectorBonus ? FConfig.ratio(FConfig.PRODUCTIVITY_RATIO_IN_SECTOR) : 0);
        bonus.addResource(r, addPitBonus ? FConfig.ratio(FConfig.PRODUCTIVITY_BONUS_IN_PIT.getValue(r)) : 0);
    }
    return bonus;
}

FEstate.prototype.getVanishPerHour = function(r)
{
    var reducer;
    if (this.drainCalculator) 
    {
        if (Resource.PEOPLE.equals(r)) 
        {
            reducer = this.drainCalculator.getPopulationReducer();
        } 
        else 
        {
            reducer = this.drainCalculator.getResourceReducer(r);
        }
    } 
    else 
    {
        if (Resource.PEOPLE.equals(r)) 
        {
            reducer = new PopulationReducer();
        } 
        else 
        {
            reducer = new ResourceReducer(r);
        }
    }
    return reducer.getVanishPerHour(this);
}

FEstate.prototype.isMaxHallLevel = function()
{
    throw new Error('abstractMethod');
}

FEstate.prototype.getNextLevelInsideWallCellCount = function()
{
    return this.cells.length;
}

FEstate.prototype.canAddToBuildQueue = function()
{
    return this.buildQueue && this.buildQueue.length < FConfig.BUILD_QUEUE_SIZE;
}

FEstate.prototype.getPeopleCapacityBonus = function(type)
{
    var result = 0;
    if (type.isClanReservation()) 
    {
        result += this.sumOfBuildingBuffs(BuffType.BOOST_BUILDING, type);
    }
    return result;
}

FEstate.prototype.getPeopleCapacityAbsoluteBonus = function(type)
{
    var result = 0;
    if (type.isHouses()) 
    {
        result += Science.getHousesCapacityBonus(this.science.getLevel(Science.SPEC_BENEFACTOR));
    }
    return result;
}

FEstate.prototype.getPeopleProductionBonus = function(type)
{
    var result = 0;
    if (type.isHouses()) 
    {
        result += Science.getHousesIncreaseBonus(this.science.getLevel(Science.SPEC_BENEFACTOR));
        result += this.sumOfBuildingBuffs(BuffType.BOOST_BUILDING, type);
    }
    return result;
}

FEstate.prototype.sumOfBuildingBuffs = function(bt, btype)
{
    return !this.buffs ? 0 : FBuff.sumOfBuildingBuffs(this.buffs, bt, btype, FEstate.tickNowForEstate(this));
}

FEstate.prototype.maxBuff = function(bt)
{
    var buff = !this.buffs ? null : FBuff.findMaxBuff(this.buffs, bt, null, FEstate.tickNowForEstate(this));
    return !buff ? 0 : buff.value;
}

FEstate.prototype.findMaxBuff = function(bt)
{
    return !this.buffs ? null : FBuff.findMaxBuff(this.buffs, bt, null, FEstate.tickNowForEstate(this));
}

FEstate.prototype.getBuffs = function(bt, unitType)
{
    return !this.buffs ? [] : FBuff.getBuffs(this.buffs, bt, unitType, FEstate.tickNowForEstate(this));
}

FEstate.prototype.getBuildingBuffs = function(bt, btype)
{
    return !this.buffs ? [] : FBuff.getBuildingBuffs(this.buffs, bt, btype, FEstate.tickNowForEstate(this));
}

FEstate.prototype.getLongestBuff = function(bt)
{
    var result = null;
    var maxTill = Number.MIN_VALUE;

    var all = this.getBuffs(bt);
    for (var i = 0; i < all.length; i++)
    {
    	var b = all[i];
        if (b.cast.tillTick > maxTill) 
        {
            result = b;
            maxTill = b.cast.tillTick;
        }
    }
    return result;
}

FEstate.prototype.isMagicActive = function(btype)
{
    return this.getMagicActiveTick(btype) >= Tick.now();
}

FEstate.prototype.getMagicActiveTick = function(btype)
{
    for (var i = 0; i < this.buffs.length; i++)
    {
    	var buff = this.buffs[i];
        if (buff.cast.cause.name == CastCause.SERVICE.name && buff.cast.spell == btype.name) 
        {
            return buff.cast.tillTick;
        }
    }
    return this.owner.getBoostTick(btype.name);
}

FEstate.prototype.getServicesActiveTill = function()
{
    var r = new FMap();
    for (var i = 0; i < this.buffs.length; i++)
    {
    	var buff = this.buffs[i];
        if (buff.cast.cause == CastCause.SERVICE) 
        {
            r.put(buff.cast.spell, buff.cast.tillTick);
        }
    }
    return r;
}

FEstate.prototype.fillSpecialMilitaryInfo = function(info)
{
    // override if needed
}

FEstate.prototype.getMilitaryInfo = function()
{
    var info = new FEstateMilitaryInfo();
    info.id = this.id;
    info.terrain = this.terrain;
    info.estateType = this.getEstateType();
    info.fundsWoHiddenAndDiscount = (!this.canLoot || this.lootDiscount == 0) ? new Funds() :
                         this.canLoot.copy().setValue(Resource.PEOPLE, 0);
    info.pearlsInRuin = this.__className__ == "FRuin" ? this.owner.pearl : 0;
    info.defenderLevel = this.getScienceLevel(Science.SPEC_DEFENDER);
    info.maxDefence = this.maxDefenceFinishTick >= this.responseTick;

    info.fortifications = this.getLevels(BType.FORTIFICATION);
    info.towers = this.getLevels(BType.TOWER);
    info.magicTowers = this.getLevels(BType.MAGIC_TOWER);

    info.defenders = [];
    info.casts = this.casts;
    info.buffs = FBuff.getBuffsForGarrison(this, this.buffs);
    info.heroes = this.heroes;
    info.prisonRatio = 0;
    var prison = this.findBuilding(BType.PRISON);
    if (prison && prison.isWorking)
    {
        info.prisonRatio = prison.getLevel().capture;
    }
    var def0 = new Army();
    if (this.garrison)
    {
        def0.add(this.garrison);
    }
    if (this.hideout)
    {
        def0.add(this.hideout);
    }
    var army = new ArmyInBattle();
    army.army = def0.toArray();
    army.levels = !this.owner.science ? new Army().toArray() : this.owner.science.unitLevels;
    army.race = this.owner.race;
    army.betrayer = this.owner.betrayerTill >= this.responseTick;
    army.neutral = this.owner.isFake();
    army.retreatLimit = this.retreatLimit;
    army.username = this.owner.username;
    army.estateName = this.name;
    army.estateXYZ = this.location;
    army.titleIndex = this.owner.titleIndex;
    army.armyFaded = this.garrisonFaded;
    army.hero = this.garrisonHero;
    army.nonHeroCasts = this.casts;
    info.defenders.push(army);

    var handleAssistance = $.proxy(function(assistances)
    {
        var def = new Army();
        var defHero = null;
        var mt1;
        for(var i=0; i<assistances.length; i++)
		{
			mt1 = assistances[i];
			def.add(mt1.army);
            if (mt1.hero)
            {
                defHero = mt1.hero;
            }
		}
        
        var mt = assistances[0];
        army = new ArmyInBattle();
        army.army = def.toArray();
        army.levels = mt.ownerUnitLevels;
        army.race = mt.owner.race;
        army.betrayer = mt.owner.betrayerTill >= this.responseTick;
        army.neutral = mt.owner.isFake();
        army.retreatLimit = 100;
        army.username = mt.owner.username;
        army.estateName = mt.from.name;
        army.estateXYZ = mt.from.location;
        army.titleIndex = mt.owner.titleIndex;
        army.armyFaded = mt.armyFaded;
        army.hero = defHero;
        army.nonHeroCasts = mt.nonHeroCasts;
        info.defenders.push(army);
    }, this);

    var missions;
    for(var i=0; i<this.assistances.length; i++)
    {
    	missions = this.assistances[i];
    	if (missions.length > 0)
    	{
            handleAssistance(missions);
        }
    }
    
    this.fillSpecialMilitaryInfo(info);
    return info;
}

FEstate.prototype.getLevels = function(btype)
{
    var result = [];
    var buildings = this.getOwnedBuildings();
    for (var i = 0; i < buildings.length; i++)
    {
    	var b = buildings[i];
        if (btype.equals(b.type) && b.isWorking) 
        {
            result.push(b.level);
        }
    }
    return result;
}

FEstate.prototype.findBuilding = function(btype)
{
    for (var i = 0; i < this.cells.length; i++)
    {
    	var cell = this.cells[i];
        if (cell.building != null && cell.building.type == btype) 
        {
            return cell.building;
        }
    }
    return null;
}

FEstate.prototype.findHero = function(id)
{
    if (id < 0) 
    {
        return null;
    }
    return Arrays.firstWith(this.heroes, function(h)
    {
        return h != null && h.id == id;
    });
}

FEstate.prototype.timeNeededToBuild = function(level, builders, science)
{
    var effective = !science ? builders : this.effectiveBuilders(builders);
    return NumberUtils.ceil(level.time * level.builders / effective);
}

FEstate.prototype.effectiveRepairmen = function(b)
{
    var speedUp = this.science.getSpecialValue(Science.SPEC_ARCHITECT) / 100;
    return b / Math.max(FConfig.MIN_REPAIR_RATE, 1 + this.sumOfBuffs(BuffType.REPAIR_TIME) - speedUp);
}

FEstate.prototype.getEstateType = function()
{
    throw new Error('abstractMethod');
}

FEstate.prototype.isHeroInGarrison = function(heroId)
{
    return this.garrisonHeroId == heroId;
}

FEstate.prototype.isHeroInHideout = function(heroId)
{
    // hero in the estate and not in the garrison
    return !this.isHeroInGarrison(heroId) && this.findHero(heroId);
}

FEstate.prototype.getScienceLevel = function(science)
{
    return this.owner && this.owner.science ? this.owner.science.getLevel(science) : 0;
}

FEstate.prototype.getGateCell = function()
{
    return HexNeighbours.getGateIndex(this);
}

FEstate.prototype.monsterRace = function()
{
    return FBuff.hasBuff(this.buffs, BuffType.MONSTER_RACE, null, FEstate.tickNowForEstate(this));
}
