function FScience()
{
	this.speedPerHour = 0;
	this.magicBoost = false;
	this.knowledge = 0;
	this.scienceEndTick = 0;
	this.currentScience = null; // Science
	this.isChargedScienceVisited = false;
	
	this.contributors = null; // FMap <Username, int>
	this.clanKnowledge = 0;
	this.clanEndTick = 0;
	this.clanSpeedPerHour = 0;
	
	this._scienceLevels = null;    // FMap <Science, int>
	this.chargedSciences = null;    // ArrayCollection <Science>
	this._unitLevels = null;       // Array transient: int[]
	
	this.scienceQueue = null; // ArrayCollection

	this.__defineGetter__("speedPerTick", this.getSpeedPerTick);
	
	this.__defineGetter__("scienceLevels", function(){ return this._scienceLevels; });
	this.__defineSetter__("scienceLevels", function(val)
	{ 
	    this._scienceLevels = val;
	    this._unitLevels = [];
	    var unitTypes = UnitType.MILITARY;
	    for (var i = 0; i < unitTypes.length; i++) 
	    {
	        this._unitLevels.push(this.levelOf(unitTypes[i]));
	    }
	});
	this.__defineGetter__("unitLevels", function(){ return this._unitLevels; });
	this.__defineGetter__("speedPerTick", function(){ return this.speedPerHour / Tick.TICKS_PER_HOUR; });
	this.__defineGetter__("clanSpeedPerTick", function(){ return this.clanSpeedPerHour / Tick.TICKS_PER_HOUR; });
};

FScience.prototype.speedPerHour = 0;
FScience.prototype.magicBoost = false;
FScience.prototype.knowledge = 0;
FScience.prototype.scienceEndTick = 0;
FScience.prototype.currentScience = null; // Science
FScience.prototype.isChargedScienceVisited = false;

FScience.prototype.contributors = null; // FMap <Username, int>
FScience.prototype.clanKnowledge = 0;
FScience.prototype.clanEndTick = 0;
FScience.prototype.clanSpeedPerHour = 0;

FScience.prototype._scienceLevels = null;    // FMap <Science, int>
FScience.prototype.chargedSciences = null;    // ArrayCollection <Science>
FScience.prototype._unitLevels = null;       // Array transient: int[]

FScience.prototype.scienceQueue = null; // ArrayCollection

FScience.prototype.getLevel = function(science)
{
	var level = this.scienceLevels.getValueOrNull(science);
	//var level = 0;
	return !level ? 0 : level;
}

FScience.prototype.getSpecialValue = function(science)
{
	var level = this.getLevel(science);
	return (level == 0) ? 0 : science.getLevel(level - 1).value;
}

FScience.prototype.getNextLevelCost = function(s)
{
	return s.getLevel(this.getLevel(s)).cost;
}

FScience.prototype.levelOf = function(utype)
{
    if (!utype.science)
    {
        return 1;   // unit does not require science
    }
    return this.getLevel(utype.science);
}
   
FScience.prototype.isScienceInProgress = function()
{
    return this.scienceQueue.length > 0 || (this.currentScience && this.knowledge < this.getCurrentScienceCost());
}

FScience.prototype.getCurrentScienceCost = function()
{
    return this.getNextLevelCost(this.currentScience);
}

FScience.prototype.hasChargedSciences = function()
{
    return this.chargedSciences.length > 0;
}

FScience.prototype.getSpeedPerTick = function()
{
	return this.speedPerHour / Tick.TICKS_PER_HOUR;
}

FScience.prototype.isCharged = function(s)
{
    for(var i=0; i<this.chargedSciences.length; i++)
    {
    	if(this.chargedSciences[i].name == s.name) return true;
    }
    return false;
}

FScience.prototype.getUnsatisfiedDeps = function(s)
{
    var res = new FMap();

    var nextLevel = this.getLevel(s) + 1;
    var price = s.getLevel(nextLevel - 1);
    
    var dep, needed, current;
    
    for(var i=0; i<price.dependencies.raw.length; i++)
    {
    	dep = price.dependencies.raw[i].key;
    	
    	needed = price.dependencies.getValue(dep);
    	current = 0;
    	
    	if (this.scienceLevels.containsKey(dep))
    	{
            current = this.scienceLevels.getValue(dep);
        }
        if (current < needed)
        {
            res.put(dep, needed);
        }
    }

    return res;
}

FScience.prototype.isResearchable = function(s, user)
{
    return this.getLevel(s) < s.getMaxLevel() &&
           this.getUnsatisfiedDeps(s).isEmpty() && !this.isCharged(s) &&
           (!s.isClannish() || user.isInRegisteredClan());
}

FScience.prototype.isAllScienceResearched = function()
{
    var s;
    for(var i=0; i<FConfig.science.raw.length; i++)
    {
    	s = FConfig.science.raw[i].key;
    	if (!Science.CAPITAL.equals(s))
    	{
            var maxLevel = FConfig.science.getValue(s).length;
            var curLevel = this.getLevel(s);
            if (curLevel < maxLevel)
            {
                return false;
            }
        }
    }
    return true;
}

FScience.prototype.isQueueLimitReached = function()
{
    return this.scienceQueue.length >= FConfig.PREMIUM_SCIENCE_QUEUE_SIZE;        
}

FScience.prototype.queueContainsSpecial = function()
{
    for(var i=0; i<this.scienceQueue.length; i++)
    {
    	if(this.scienceQueue[i].isSpecialization())
    	{
    		return true;
    	}
    }
    return false;
}

FScience.prototype.queueContainsScience = function(science)
{
	for(var i=0; i<this.scienceQueue.length; i++)
    {
    	if(this.scienceQueue[i].name == science.name)
    	{
    		return true;
    	}
    }
    return false;
}

FScience.prototype.canAddToQueue = function(science)
{
    return this.currentScience.name != science.name && !this.isQueueLimitReached() && !this.queueContainsScience(science)
            && !(science.isSpecialization() && this.queueContainsSpecial()); 
}

FScience.prototype.hasChargedSciences = function()
{
    return this.chargedSciences.length > 0;
}

FScience.prototype.getUnitUpgradeCost = function(utype, race)
{
    var curLevel = this.getLevel(utype.science);

    var curLevelCost = FConfig.getStat(race, utype, curLevel).cost;
    var nextLevelCost = FConfig.getStat(race, utype, curLevel + 1).cost;
    
    var res = nextLevelCost.copy();
    
    res.extract(curLevelCost);
    return res;
}

FScience.prototype.getResearchedSpecial = function()
{
    var s;
    for(var i=0; i<Science.SPECIALIZATION.length; i++)
    {
    	s = Science.SPECIALIZATION[i];
    	if (this.getLevel(s) > 0)
    	{
            return s;
        }
    }
    return null;
}

FScience.prototype.isAnotherSpecialResearched = function(science)
{
    var spec = this.getResearchedSpecial();
    return spec && !spec.equals(science);
}

FScience.prototype.isAnotherSpecialAlreadyInQueue = function(science)
{
	var s;
	for(var i=0; i<this.scienceQueue.length; i++)
    {
    	s = this.scienceQueue[i];
    	if (s  && s.isSpecialization() && !s.equals(science))
    	{
            return true;
        }
    }
    return false;
}
