var BType = function(name, id, requiredScience)
{
	this.name = name;

	if ( typeof id == "undefined")
	{
		id = null;
	}
	if ( typeof requiredScience == "undefined")
	{
		requiredScience = null;
	}

	this.name = name;
	this.id = id;
	this._requiredScience = requiredScience;
	
	this.__defineGetter__("requiredScience", this.getRequiredScience);
};

extendClass(BType.prototype, Enum.prototype);

BType.__isEnum__ = true;

BType.prototype.__CLS__ = "game.model.kingdom.building.BType";
BType.prototype.id = null;
BType.prototype.name = null;
BType.prototype._requiredScience = null;

BType.prototype.getRequiredScience = function()
{
    return (FConfig.IGNORE_SCIENCE_BUILDINGS && FConfig.IGNORE_SCIENCE_BUILDINGS.indexOf(this) >= 0) ? null : this._requiredScience;
}

BType.prototype.isExtractor  = function()
{
	return BType.WINDMILL.equals(this) || BType.MINE_IRON.equals(this) || BType.MINE_GOLD.equals(this) || BType.MINE_STONE.equals(this);
}

BType.prototype.getBuildCost  = function(r)
{
	return this.getFirstLevel(r).cost;
}

BType.prototype.isRaiser  = function()
{
	return this.isExtractor() || this.isProducer();
}

BType.prototype.isProducer  = function()
{
	return this.isSawmill() || (this.isEstateHall() && !this.isLakeHall() && !this.isGrailHall()) || this.isShop();
}

BType.prototype.isGrailHall = function()
{
    return BType.GRAILHALL.equals(this);
}

BType.prototype.isLakeHall  = function()
{
	return BType.LAKEHALL.equals(this);
}

BType.prototype.isRecovery  = function()
{
	return BType.RECOVERY.equals(this);
}

BType.prototype.isUnitProducer  = function()
{
	return this.isMilitaryUnitProducer() || this.isSpecialUnitProducer();
}

BType.prototype.checkPlaceConstraints = function(mask, value)
{
	for(var i=0; i<mask.length; i++)
	{
		if(mask[i].name == value.name)
		{
			return true;
		}
	}
	return false;
}

BType.prototype.canBeBuiltOnSector  = function(r)
{
	return this.checkPlaceConstraints(this.getParams(r).whereIn.mask, BWhere.SECTOR);
}

BType.prototype.canBeBuiltInPit  = function(r)
{
	return this.checkPlaceConstraints(this.getParams(r).whereIn.mask, BWhere.PIT);
}

BType.prototype.canBeBuiltIn  = function(r, type)
{
	var mask = this.getParams(r).whereIn.mask;
	switch (type)
	{
		case EstateType.PIT:
			return this.checkPlaceConstraints(mask, BWhere.PIT);
		case EstateType.MINE:
			return false;
		case EstateType.LAKE:
			return this.checkPlaceConstraints(mask, BWhere.LAKE) || this.checkPlaceConstraints(mask, BWhere.SECTOR);
		case EstateType.GRAIL:
            return this.checkPlaceConstraints(mask, BWhere.GRAIL) || this.checkPlaceConstraints(mask, BWhere.SECTOR);
		case EstateType.RUIN:
			return false;
		case EstateType.CAMP:
			return false;
		case EstateType.CASTLE:
			return this.checkPlaceConstraints(mask, BWhere.CASTLE);
		default:
			throw new Error(Strings.format("Unknown estate type \"{0}\"", type));
	}
}

BType.prototype.isMilitaryUnitProducer  = function()
{
	return BType.MELEE.equals(this) || BType.SCAVENGER.equals(this) || BType.RANGER.equals(this) || BType.HEALER.equals(this) || BType.MINER.equals(this) || BType.CAVALRY.equals(this) || BType.FLYER.equals(this) || BType.MAGE.equals(this);
}

BType.prototype.isSpecialUnitProducer  = function()
{
	return BType.MERCHANT.equals(this) 
		|| BType.SETTLER.equals(this) 
		|| BType.SPYGUILD.equals(this) 
		|| BType.ANTISPYGUILD.equals(this) 
		|| BType.EXPEDITION.equals(this) 
		|| BType.PROPHET_TOWER.equals(this) 
		|| BType.MIRROR_TOWER.equals(this) 
		|| BType.ABANDONED_MERCHANT.equals(this)
		|| BType.WORKSHOP.equals(this)
	;
}

BType.prototype.isShop  = function()
{
	return BType.SHOP.equals(this);
}

BType.prototype.isSawmill  = function()
{
	return BType.SAWMILL.equals(this);
}

BType.prototype.isEstateHall  = function()
{
	return BType.CITYHALL.equals(this) || this.isOuterEstateHall();
}

BType.prototype.isOuterEstateHall  = function()
{
	return BType.PITHALL.equals(this) || BType.MINEHALL.equals(this) || BType.LAKEHALL.equals(this) || BType.CAMPHALL.equals(this) || BType.CASTLEHALL.equals(this) || BType.GROTTOHALL.equals(this) || BType.GRAILHALL.equals(this);
}

BType.prototype.isCapacitor  = function()
{
	return this.isWarehouse() || this.isHouses() || this.isEstateHall();
}

BType.prototype.isMarket  = function()
{
	return BType.MARKET.equals(this);
}

BType.prototype.isExpedition  = function()
{
	return BType.EXPEDITION.equals(this);
}

BType.prototype.isDiplomatHall  = function()
{
	return BType.DIPLOMAT.equals(this);
}

BType.prototype.isTemple  = function()
{
	return BType.TEMPLE.equals(this);
}

BType.prototype.isHouses  = function()
{
	return BType.HOUSES.equals(this);
}

BType.prototype.isWarehouse  = function()
{
	return BType.WAREHOUSE.equals(this);
}

BType.prototype.isTerraformer  = function()
{
	return BType.TERRAFORMER.equals(this);
}

BType.prototype.isAlchemist  = function()
{
	return BType.ALCHEMIST.equals(this);
}

BType.prototype.isTower  = function()
{
	return BType.TOWER.equals(this) || BType.MAGIC_TOWER.equals(this);
}

BType.prototype.isFortification  = function()
{
	return BType.FORTIFICATION.equals(this);
}

BType.prototype.isSettler  = function()
{
	return BType.SETTLER.equals(this);
}

BType.prototype.isResurrector  = function()
{
	return BType.RESURRECTOR.equals(this);
}

BType.prototype.isClanWarehouse  = function()
{
	return BType.CLAN_WAREHOUSE.equals(this);
}

BType.prototype.isClanReservation  = function()
{
	return BType.CLAN_RESERVATION.equals(this);
}

BType.prototype.isClanCapacitor  = function()
{
	return this.isClanReservation() || this.isClanWarehouse();
}

BType.prototype.getFirstLevel  = function(r)
{
	// level 0 is null-level, does nothing, contains nulls and zeros
	return this.getParams(r).levels[1];
}

BType.prototype.isRequiresWorkers  = function(r)
{
	return this.getParams(r).requiresWorkers;
}

BType.prototype.isIndestructible  = function(r)
{
	return this.getParams(r).indestructible;
}

BType.prototype.getMaxCountInCity  = function(r)
{
	return this.getParams(r).maxCountInCity;
}

BType.prototype.getEmptyCellSpeed  = function(r)
{
	return this.getParams(r).emptyCellSpeed;
}

BType.prototype.getUnitType  = function(r)
{
	return this.getParams(r).unitType;
}

BType.prototype.getLevels  = function(r)
{
	return this.getParams(r).levels;
}

BType.prototype.getWhereIn  = function(r)
{
	return this.getParams(r).whereIn;
}

BType.prototype.getWhereOn  = function(r)
{
	return this.getParams(r).whereOn;
}

BType.prototype.setLevels  = function(levels, r)
{
	this.getParams(r).levels = levels;
}

BType.prototype.setUnitType  = function(type, r)
{
	this.getParams(r).unitType = type;
}

BType.prototype.getParams  = function(r)
{
	var raceParam = FConfig.buildingsMap.getValue(r);
	if (raceParam.containsKey(this))
	{
		return raceParam.getValue(this) || new BuildingParams();
	}
	else
	{
		return FConfig.buildingsMap.getValue(Race.MONSTER).getValue(this) || new BuildingParams();
	}
}

BType.byId = function(id)
{
	for (var i = 0, len = BType.values.length; i < len; i++)
	{
		if (id == BType.values[i].id) return BType.values[i];
	}
	throw new Error(Strings.format("Unknown building type: {0}", id));
}

BType.NULL = new BType("NULL", 10000);
BType.CITYHALL = new BType("CITYHALL", 1001);
BType.PITHALL = new BType("PITHALL", 1002);
BType.MINEHALL = new BType("MINEHALL", 1003);
BType.RUINHALL = new BType("RUINHALL", 1004);
BType.LAKEHALL = new BType("LAKEHALL", 1005);
BType.CAMPHALL = new BType("CAMPHALL", 1006);
BType.EVILHALL = new BType("EVILHALL", 1007);
BType.CASTLEHALL = new BType("CASTLEHALL", 1008);
BType.CASTLEGATE = new BType("CASTLEGATE", 1009);
BType.GROTTOHALL = new BType("GROTTOHALL", 1010);
BType.GRAILHALL = new BType("GRAILHALL" , 1011);
BType.STONEHENGEHALL = new BType("STONEHENGEHALL", 1012);
BType.CATAPULTHALL = new BType("CATAPULTHALL", 1013);
BType.MINE_IRON = new BType("MINE_IRON", 4);
BType.MINE_STONE = new BType("MINE_STONE", 5);
BType.MINE_GOLD = new BType("MINE_GOLD", 3);
BType.WINDMILL = new BType("WINDMILL", 7);
BType.ALCHEMIST = new BType("ALCHEMIST", 2);
BType.SAWMILL = new BType("SAWMILL", 6);
BType.SHOP = new BType("SHOP", 8);
BType.RECOVERY = new BType("RECOVERY", 9);
BType.HOUSES = new BType("HOUSES", 1);
BType.WAREHOUSE = new BType("WAREHOUSE", 11);
BType.TERRAFORMER = new BType("TERRAFORMER", 12, Science.TERRAFORMING);
BType.MARKET = new BType("MARKET", 13, Science.MARKET);
BType.DIPLOMAT = new BType("DIPLOMAT", 14, Science.DIPLOMACY);
BType.CLAN_WAREHOUSE = new BType("CLAN_WAREHOUSE", 15, Science.DIPLOMACY);
BType.CLAN_RESERVATION = new BType("CLAN_RESERVATION", 16, Science.DIPLOMACY);
BType.TOWER = new BType("TOWER", 101, Science.TOWER);
BType.FORTIFICATION = new BType("FORTIFICATION", 102, Science.FORTIFICATION);
BType.MAGIC_TOWER = new BType("MAGIC_TOWER", 103, Science.MAGIC_TOWER);
BType.MELEE = new BType("MELEE", 201, Science.MELEE);
BType.SCAVENGER = new BType("SCAVENGER", 202, Science.SCAVENGER);
BType.RANGER = new BType("RANGER", 203, Science.RANGER);
BType.HEALER = new BType("HEALER", 204, Science.HEALER);
BType.MINER = new BType("MINER", 205, Science.MINER);
BType.CAVALRY = new BType("CAVALRY", 206, Science.CAVALRY);
BType.FLYER = new BType("FLYER", 207, Science.FLYER);
BType.MAGE = new BType("MAGE", 208, Science.MAGE);
BType.MERCHANT = new BType("MERCHANT", 301, Science.MERCHANT);
BType.SETTLER = new BType("SETTLER", 302, Science.COLONIZATION);
BType.SPYGUILD = new BType("SPYGUILD", 303, Science.ESPIONAGE);
BType.ANTISPYGUILD = new BType("ANTISPYGUILD", 304, Science.ANTI_ESPIONAGE);
BType.EXPEDITION = new BType("EXPEDITION", 305, Science.CARTOGRAPHY);
BType.ABANDONED_MERCHANT = new BType("ABANDONED_MERCHANT", 306);
BType.PROPHET_TOWER = new BType("PROPHET_TOWER", 307, Science.OBSERVATION);
BType.MIRROR_TOWER = new BType("MIRROR_TOWER", 308, Science.ANTI_OBSERVATION);
BType.TEMPLE = new BType("TEMPLE", 400, Science.TEMPLE);
BType.RESURRECTOR = new BType("RESURRECTOR", 401, Science.RESURRECTOR);
BType.TAVERN = new BType("TAVERN", 501, Science.HEROES);
BType.ARTIFACT_STORE = new BType("ARTIFACT_STORE", 502, Science.HEROES);
BType.PRISON = new BType("PRISON", 503, Science.PRISON);
BType.ARENA = new BType("ARENA", 601);
BType.CLAN_DIPLOMAT = new BType("CLAN_DIPLOMAT", 602);
BType.WORKSHOP = new BType("WORKSHOP", 603);

BType.values = [
	//BType.NULL,
	BType.CITYHALL,
	BType.PITHALL,
	BType.MINEHALL,
	BType.RUINHALL,
	BType.LAKEHALL,
	BType.CAMPHALL,
	BType.EVILHALL,
	BType.CASTLEHALL,
	BType.CASTLEGATE,
	BType.GROTTOHALL,
	BType.GRAILHALL,
	BType.STONEHENGEHALL,
	BType.CATAPULTHALL,
	BType.MINE_IRON,
	BType.MINE_STONE,
	BType.MINE_GOLD,
	BType.WINDMILL,
	BType.ALCHEMIST,
	BType.SAWMILL,
	BType.SHOP,
	BType.RECOVERY,
	BType.HOUSES,
	BType.WAREHOUSE,
	BType.TERRAFORMER,
	BType.MARKET,
	BType.DIPLOMAT,
	BType.CLAN_WAREHOUSE,
	BType.CLAN_RESERVATION,
	BType.TOWER,
	BType.FORTIFICATION,
	BType.MAGIC_TOWER,
	BType.MELEE,
	BType.SCAVENGER,
	BType.RANGER,
	BType.HEALER,
	BType.MINER,
	BType.CAVALRY,
	BType.FLYER,
	BType.MAGE,
	BType.MERCHANT,
	BType.SETTLER,
	BType.SPYGUILD,
	BType.ANTISPYGUILD,
	BType.EXPEDITION,
	BType.ABANDONED_MERCHANT,
	BType.PROPHET_TOWER,
	BType.MIRROR_TOWER,
	BType.TEMPLE,
	BType.RESURRECTOR,
	BType.TAVERN,
	BType.ARTIFACT_STORE,
	BType.PRISON,
	BType.ARENA,
	BType.CLAN_DIPLOMAT,
	BType.WORKSHOP,
];
