function Alert()
{
	callParentConstructor(Alert.prototype, this);
	
	this.yesHandler = null;
	this.noHandler = null;
	this.onClose = null;
	
	this.userCanClose = false;
	this.actions = [];
	
	this._hideDefaultActions = true;
	this.__defineGetter__('hideDefaultActions', function(){ return this._hideDefaultActions; });
	this.__defineSetter__('hideDefaultActions', function(val)
	{ 
		this._hideDefaultActions = val;
		this.refreshDefaultButtons()
	});
	
	if(APP && APP.hideLoader) APP.hideLoader();
}

extendClass(Alert.prototype, Dialog.prototype);

Alert.prototype.constructMessage = function(message)
{
	if(!$.isArray(message)) message = [message];
	
	var ret = "", s = "";
	for(var i=0; i<message.length; i++)
	{
		s = message[i];
		
		if (s.DOM) s = s.DOM;
		if (s instanceof HTMLElement) s = s.outerHTML;
		
	    ret += s;
	}
	
	return ret;
}

Alert.prototype.open = function(message, caption)
{
	
	$(this.message).html(this.constructMessage(message) || '');
	
	if(!caption)
	{
		caption = I18.f('MESSAGE');
	}
	
	$(this.caption).html(caption || '');
	
	$(this.yesButton).one(CLICK_EVENT, $.proxy(this.onYesButton, this));
	$(this.noButton).one(CLICK_EVENT, $.proxy(this.onNoButton, this));
	
	if(!this.yesHandler && !this.noHandler)
	{
		COM(this.yesButton).caption = I18.f("OK");
		this.noButton.style.display = "none";
	}
	else
	{
		COM(this.yesButton).skin = 'green';
		COM(this.noButton).skin = 'red';
	}
}

Alert.prototype.refreshDefaultButtons = function()
{
	var flag = this._hideDefaultActions ? (this.actions.length == 0) : true;
	UI.show(flag, [this.yesButton]);
	UI.show(flag && (this.yesHandler || this.noHandler), [this.noButton]); 
}

Alert.prototype.onYesButton = function()
{
	if(this.yesHandler)
	{
		this.yesHandler();
	}
	
	this.close();
}

Alert.prototype.onNoButton = function()
{
	if(this.noHandler)
	{
		this.noHandler();
	}
	
	this.close();
}

Alert.prototype.__construct = function(params)
{
}

Alert.prototype.onShow = function()
{
	this.scroll = new iScroll(this.message.parentNode, {scrollbarColor: '#784f0b'});
	setTimeout($.proxy(this.scroll.refresh, this.scroll), 500);
}

Alert.findDuplicateAlert = function(message)
{
	if (message && (message instanceof HTMLElement)) message = message.outerHTML;
	var lst = document.querySelectorAll('Alert'), len = lst.length;
	for (var i = 0; i < len; i++)
	{
		var a = COM(lst[i]);
		if (a.message.innerHTML == message)
		{
			return a;
		}
	}
	return null;
}

Alert.create = function(message, caption, yesHandler, noHandler)
{
	var obj = Alert.findDuplicateAlert(message);
	if (obj) return obj;
	
	obj = COM.create('Alert');
	
	obj.yesHandler = yesHandler;
	obj.noHandler = noHandler;
	
	obj.open.call(obj, message, caption);
	
	obj.popup();
	
	return obj;
}

Alert.error = function(message, caption, onClose)
{
	if(!caption)
	{
		caption = I18.f('ERROR');
	}
	
	var obj = Alert.create(message, caption, onClose);
	$(obj.DOM).addClass('error');
	return obj;
}

Alert.createWithActions = function(message, caption, actions, context)
{
	var d = Alert.create(message, caption);
	
	if (!actions) actions = [];
	var a = { "common": [], "left": [], "right": [] }, buttons = [];
	if ($.isArray(actions))
	{
		a.common = actions;
	}
	else if (actions && (typeof actions == 'object'))
	{
		var full = (actions['common'] || actions['left'] || actions['right']);
		if (full)
		{
			for (var p in a)
			{
				if (actions[p])
				{
					if ($.isArray(actions[p])) a[p] = actions[p];
					else if (actions[p]) a[p].push(actions[p]);
				}
			}
		}
		else
		{
			a.common.push(actions);
		}
	}
	
	Alert.addActions(d, d.actionsBox, a.common, context);
	Alert.addActions(d, d.headerLeftBox, a.left, context);
	Alert.addActions(d, d.headerRightBox, a.right, context);
	
	d.refreshDefaultButtons(); 
	d.userCanClose = true;
	
	return d;
}

Alert.addActions = function(dialog, box, actions, context)
{
	if (typeof context == 'undefined') context = dialog;
	var actionDecl = null, buttons = [];
	while (typeof (actionDecl = actions.shift()) == 'object')
	{
		if (actionDecl instanceof HTMLElement)
		{
			actionDecl = {DOM: actionDecl};
		}
		var btn = (actionDecl['DOM']) 
			? actionDecl 
			: SimpleButton.createForAction(actionDecl, context)
		;
		COM.bindExclusiveClick(btn.DOM, $.proxy(dialog.close, dialog));
		$(box).append(btn.DOM);
		dialog.actions.push(btn);
	}
}
