function CheckBoxButtonsGroup()
{
	this.items = [];
	this.multiSelect = false;
	this.noOneSelect = false;
	this.buttonSkin = "yellow";
	
	this._enabled = true;
	this.__defineGetter__("enabled", this.getEnabled);
	this.__defineSetter__("enabled", this.setEnabled);
	
	this.__defineGetter__("value", this.getValue);
	this.__defineSetter__("value", this.setValue);
	
	this.__defineGetter__("skin", this.getSkin);
	this.__defineSetter__("skin", this.setSkin);
	
	this.change = $.proxy(this.change, this);
	
	this.onChange = null;
}

CheckBoxButtonsGroup.prototype.__construct = function()
{
	var self = this;
	
	this.multiSelect = this.DOM.hasAttribute('multiselect') ? this.DOM.getAttribute('multiselect') == 'true' : false;
	
	// init options
	if (this.__inlineData)
	{
		var dp = [], data = $('<span>'+this.__inlineData+'</span>'), self = this;
		$('item', data).each(function(n, obj)
		{
			var label = $(obj).html();
			var value = obj.hasAttribute('value') ? obj.getAttribute('value') : label;
			var disabled = obj.hasAttribute('disabled');
			var checked = obj.hasAttribute('checked');
			
			var opt = self.addItem(label, value, checked, disabled);
		});
	}
}

CheckBoxButtonsGroup.prototype.addItem = function(label, value, checked, disabled)
{
	var item = COM.create("CheckBoxButton");
	item.skin = this.buttonSkin;
	item.caption = label;
	item.value = value;
	item.checked = checked;
	item.enabled = !disabled;
	
	$(item.DOM).on("change", this.change);
	
	this.items.push(item);
	
	this.DOM.appendChild(item.DOM);
	
	return item;
}

CheckBoxButtonsGroup.prototype.removeItem = function(item)
{
	if (item) item.destroy();
	for (var i = 0; i < this.items.length; i++)
	{
		if (this.items[i].ID === item.ID)
		{
			this.items.splice(i, 1);
			break;
		}
	}
}

CheckBoxButtonsGroup.prototype.clear = function()
{
	while (this.items.length) 
	{
		COM(this.items.shift()).destroy();
	}
	this.items = [];
}

CheckBoxButtonsGroup.prototype.change = function(e)
{
	if(!this.multiSelect)
	{
		var changed = false;
		var item = COM(e.currentTarget);
		for(var i = 0; i < this.items.length; i++)
		{
			if (this.items[i] !== item)
			{
				if(this.items[i].checked)
				{
					changed = true;
				}
				this.items[i].checked = false;
			}
		}
		
		if(!changed && !this.noOneSelect)
		{
			item.checked = true;
		}
	}
	
	if(typeof this.onChange == "function")
	{
		this.onChange(this.value);
	}
}

CheckBoxButtonsGroup.prototype.getSkin = function()
{
	return this.buttonSkin;
}

CheckBoxButtonsGroup.prototype.setSkin = function(value)
{
	if(this.buttonSkin === value) return;
	this.buttonSkin = value;
}


CheckBoxButtonsGroup.prototype.getEnabled = function()
{
	return this._enabled;
}

CheckBoxButtonsGroup.prototype.setEnabled = function(val)
{
	val = Boolean(val);
	
	if(this._enabled === val)
	{
		return;
	}
	
	this._enabled = val;
	
	for(var i=0; i<this.items.length; i++)
	{
		this.items[i].enabled = val;
	}
}

CheckBoxButtonsGroup.prototype.getValue = function()
{
	var ret = [];
	for (var i = 0; i < this.items.length; i++)
	{
		if (this.items[i].checked)
		{
			ret.push(this.items[i].value);
			if (this.multiSelect) break;
		}
	}
	return this.multiSelect ? ret : ret.shift(); 
}

CheckBoxButtonsGroup.prototype.setValue = function(val)
{
	if(!$.isArray(val)) val = (typeof val == "undefined") ? [] : [val];
	if (!this.multiSelect) val = [val.shift()]; // 1 value
	
	for (var i = 0; i < this.items.length; i++)
	{
		this.items[i].checked = false;
		for (var n = 0; n < val.length; n++)
		{
			if(val[n] == this.items[i].value)
			{
				this.items[i].checked = true;
			}
		}
	}
}