function SimpleButton()
{
	this._visible = true;
	this._skin = 'gray';
	this._enabled = true;
	this._icon = [null, null];
	
	this.__defineGetter__("visible", this.getVisible);
	this.__defineSetter__("visible", this.setVisible);
	
	this.__defineGetter__("caption", this.getCaption);
	this.__defineSetter__("caption", this.setCaption);
	
	this.__defineGetter__("subCaption", this.getSubCaption);
	this.__defineSetter__("subCaption", this.setSubCaption);
	
	this.__defineGetter__("enabled", this.getEnabled);
	this.__defineSetter__("enabled", this.setEnabled);
	
	this.__defineGetter__("skin", this.getSkin);
	this.__defineSetter__("skin", this.setSkin);
	
	this.__defineGetter__("icon", this.getIcon);
	this.__defineSetter__("icon", this.setIcon);
	
	this.disabledCSSClass = "disabled";
	
	this._reason = [];
	this._oldSkin = null;
	this.sound = null;
	
	this.__defineGetter__ ("reason", function()
	{
		return ($.isArray(this._reason) && this._reason.length) ? this._reason : null;
	});
	
	this.__defineSetter__ ("reason", function(val)
	{
		if (!$.isArray(val)) val = val ? [val] : [];
		
		this._reason = val;
		
		var w = $(this.wrapper);
		if (this._skin)
		{
			w.toggleClass(this._skin, Boolean(!this.reason));
		}
		w.toggleClass('red', Boolean(this.reason));
	});
}

SimpleButton.prototype.__construct = function()
{
	$(this.DOM).addClass('SimpleButton');
	$(this.wrapper).toggleClass('flat', $(this.DOM).hasClass('flat'));
	$(this.wrapper).toggleClass('disabled', $(this.DOM).hasClass('disabled'));
	
	var attr = $(this.DOM).attr();
	
	this.caption = (attr['caption'] ? attr['caption'] : this.__inlineData);
	
	this.subCaption = (attr['subcaption'] ? attr['subcaption'] : '');
	this.skin = (attr['skin'] ? attr['skin'] : 'yellow');
	this.enabled = (attr['enabled'] ? attr['enabled'] != false : true);
	
	this.icon = [
		(attr['icon'] ? attr['icon'] : null),
		(attr['rightIcon'] ? attr['rightIcon'] : null)
	]; 
	
	$(this.DOM).on(CLICK_EVENT, $.proxy(function(e)
	{
		if(isKeyboardVisible())
        {
            e.stopImmediatePropagation();
            return false;
        }
        
        if (this.reason)
		{
			var content = document.createElement('div'), reason = this.reason;
			for (var i = 0, len = reason.length; i < len; i++)
			{
				if (typeof reason[i] == 'function') 
				{
					reason[i]();
				}
				else
				{
					var msg = document.createElement('div');
					msg.style.margin = '0.5em';
					$(msg).append(reason[i]);
					$(content).append(msg);
				}
			}
			if (content.innerHTML.length) 
			{
				APP.playSound();
				Alert.create(content);
			}
			e.stopImmediatePropagation();
			return false;
		}
		
		if (this.sound) APP.specialSound = this.sound;
		
		if(!this.enabled)
		{
			e.stopImmediatePropagation();
			return false;
		}
		
	}, this));
}

SimpleButton.prototype.getVisible = function()
{
	return this._visible;
}

SimpleButton.prototype.setVisible = function(value)
{
	if(this._visible == value) return;
	
	this._visible = value;
	
	UI.show(this._visible, [this.DOM]);
}

SimpleButton.prototype.getCaption = function()
{
	return this.captionText.innerHTML;
}

SimpleButton.prototype.setCaption = function(value)
{
	UI.show((value || value == "0") && String(value).length, [this.captionText.parentNode]);
	if(this.getCaption() === value) return;
	this.captionText.innerHTML = value;
}

SimpleButton.prototype.getSubCaption = function()
{
	return this.subCaptionText.innerHTML;
}

SimpleButton.prototype.setSubCaption = function(value)
{
	if(this.getSubCaption() == value) return;
	
	if(!value || value == '')
	{
		this.subCaptionText.style.display = "none";
	}
	else
	{
		this.subCaptionText.style.display = "";
	}
	
	this.subCaptionText.innerHTML = value;
}

SimpleButton.prototype.getSkin = function()
{
	return this._skin;
}

SimpleButton.prototype.setSkin = function(value)
{
	if(this._skin === value) return;
	if(!this.reason) 
	{
		$(this.wrapper).removeClass(this._skin).addClass(value);
	}
	this._skin = value;
}

SimpleButton.prototype.getEnabled = function()
{
	return this._enabled;
}

SimpleButton.prototype.setEnabled = function(value)
{
	value = Boolean(value);
	if(this._enabled == value) return;
	
	this._enabled = value;
	
	$(this.wrapper).toggleClass('disabled', !value);
}

SimpleButton.prototype.getIcon = function()
{
	return this._icon;
}

SimpleButton.prototype.setIcon = function(value)
{
	if (!value || typeof value['push'] == 'undefined')
	{
		value = [value, null];
	}
	
	var icons = (this.leftImage && this.rightImage) ? [this.leftImage, this.rightImage] : [this.iconImage];
	var len = icons.length;
	for (var i = 0; i < len; i++)
	{
		this._icon[i] = value[i];
		icons[i].src = this._icon[i] || '';
		icons[i].style.display = this._icon[i] ? '' : 'none';
	}
	var bothIcons = len >= 2 && !(Boolean(this._icon[0]) ^ Boolean(this._icon[1]));
	if (this.subCaptionText) this.subCaptionText.className = bothIcons ? 'align-center' : '';
}

SimpleButton.createForAction = function(actionDecl, context, callback)
{
	if (!actionDecl) actionDecl = {};
	
	var btn = COM.create('SimpleButton');
	
	if (actionDecl['onclick']) 
	{
		callback = actionDecl['onclick'];
		delete(actionDecl['onclick']);
	}
	
	var ctx = actionDecl['context'] 
		? actionDecl['context'] 
		: (context || btn)
	;
	delete(actionDecl['context']);
	
	for (var prop in actionDecl)
	{
		btn[prop] = actionDecl[prop];
	}
	
	if (typeof callback == 'function') callback = [callback];
	if ($.isArray(callback))
	{
		var fn;
		while (callback.length > 0)
		{
			fn = callback.shift();
			if (typeof fn == 'function') 
			{
				COM.bindExclusiveClick(btn.DOM, $.proxy(fn, ctx));
			}
		}
	}
		
	return btn;
}
