function SliderDialog()
{
	callParentConstructor(SliderDialog.prototype, this);
	
	this.data = null;
	this.initCountFrom = null;
	this.countFrom = null;
	this.initCountTo = null;
	this.countTo = null;
	
	this.stepIndex = 0;
	this.multipliers = [1, 10, 100, 1000];
	this.changeMultiplierCallback = null;
	this.onMove = function(){};
}

extendClass(SliderDialog.prototype, Dialog.prototype);

SliderDialog.prototype.__construct = function()
{
	this.header.style.display = "none";
}

SliderDialog.create = function(data, countFrom, countTo, onMove, stepIndex, multipliers, changeMultiplierCallback)
{
	var d = COM.create('SliderDialog');
	
	if (multipliers) d.multipliers = multipliers;
	d.stepIndex = (typeof stepIndex != 'undefined') ? stepIndex*1 : 0;
	d.onMove = onMove || function(){};
	d.changeMultiplierCallback = changeMultiplierCallback;
	
	d.update(data, countFrom*1, countTo*1);
	d.popup();
	
	return d;
}

SliderDialog.prototype.popup = function(parent)
{
	superMethod(SliderDialog.prototype, "popup", this)(parent);
	
	this.skin = "black";
	this.headerCloseBox.style.display = "none";
	
	COM(this.moveDown).icon = STATIC_INTERFACE_URL + "icons/left_arrow.png";
	COM(this.moveUp).icon = STATIC_INTERFACE_URL + "icons/right_arrow.png";
	
	COM.bindExclusiveClick(this.changeMultiplier, $.proxy(this.onChangeMultiplier, this));
	COM.bindExclusiveClick(this.moveDown, $.proxy(this.onMoveDown, this));
	COM.bindExclusiveClick(this.moveUp, $.proxy(this.onMoveUp, this));
	COM.bindExclusiveClick(this.okButton, $.proxy(this.doMove, this));
	
	COM(this.slider).onChange = $.proxy(this.onSliderChanged, this);
	COM(this.slider).snapPicker = true;
}

SliderDialog.prototype.onShow = function()
{
	COM(this.slider).updateUI();
}

SliderDialog.prototype.update = function(data, countFrom, countTo)
{
	this.data = data;
	this.initCountFrom = countFrom;
	this.countFrom = countFrom;
	this.initCountTo = countTo;
	this.countTo = countTo;
	
	var s = COM(this.slider);
	s.min = 0;
	s.max = this.initCountFrom + this.initCountTo;
	s.value = this.initCountTo;
	
	s.enabled = !!(this.initCountFrom || this.initCountTo);
	
	COM(this.changeMultiplier).enabled = s.enabled;
	COM(this.moveDown).enabled = s.enabled;
	COM(this.moveUp).enabled = s.enabled;
	
	this.updateCounts();
	this.updateMultiplier();
}

SliderDialog.prototype.onApply = function()
{
	this.close();
}

SliderDialog.prototype.onChangeMultiplier = function()
{
    this.stepIndex = (this.stepIndex + 1) % this.multipliers.length;
	this.updateMultiplier();
	
	if(this.changeMultiplierCallback) this.changeMultiplierCallback(this.stepIndex);
}

SliderDialog.prototype.updateMultiplier = function()
{
	COM(this.changeMultiplier).caption = this.multipliers[this.stepIndex];
	COM(this.slider).step = this.multipliers[this.stepIndex];
}

SliderDialog.prototype.onMoveDown = function()
{
	this.calcCounts(-1);
}

SliderDialog.prototype.onMoveUp = function()
{
	this.calcCounts(1);
}

SliderDialog.prototype.calcCounts = function(direct)
{
	var n = this.multipliers[this.stepIndex];
	
	this.countFrom -= direct*n;
	this.countTo += direct*n;
	if(this.countFrom < 0)
	{
		this.countFrom = 0;
	}
	if(this.countTo < 0)
	{
		this.countTo = 0;
	}
	if(this.countFrom > this.initCountFrom + this.initCountTo)
	{
		this.countFrom = this.initCountFrom + this.initCountTo;
	}
	if(this.countTo > this.initCountFrom + this.initCountTo)
	{
		this.countTo = this.initCountFrom + this.initCountTo;
	}
	
	COM(this.slider).value = this.countTo;
	this.updateCounts();
}

SliderDialog.prototype.onSliderChanged = function(val)
{
	this.countTo = val;
	this.countFrom = this.initCountFrom + this.initCountTo - this.countTo;
	this.updateCounts();
}

SliderDialog.prototype.updateCounts = function()
{
	this.countFromLabel.innerHTML = Math.floor(this.countFrom);
	this.countToLabel.innerHTML = Math.floor(this.countTo);
}

SliderDialog.prototype.doMove = function()
{
	this.onMove(this.data, this.countTo);
	this.close();
}

SliderDialog.prototype.close = function()
{
	superMethod(SliderDialog.prototype, "close", this)();
}