function ArmyInput()
{
	this.race = null;
	this.aura = new EmptyUnitAura();
	this.buffs = null;
	this.heroes = null;
	this.selectedHero = null;
	this.editableLevels = true;
	
	this.onChangeUnits = function(){};
	this.onChangeHero = function(){};
	this.onAllChangeLevels = function(){};
	
	this.currentUnit = null;
	
	this.showCountInput = $.proxy(this.showCountInput, this);
}

ArmyInput.prototype.__construct = function()
{
	COM(this.armyView).zeroAsCardsAllowed = false;
   
	var units = COM(this.armyView).units;
	for(var i=0; i<units.length; i++)
	{
		$(units[i].DOM).off(CLICK_EVENT);
		COM.bindExclusiveClick(units[i].DOM, this.showCountInput)
	}
	
	COM(this.armyView).onHeroClick = null;
	//COM.bindExclusiveClick(COM(this.armyView).heroView, $.proxy(this.onChooseHero, this));
	$(COM(this.armyView).heroView).on(CLICK_EVENT, $.proxy(this.onChooseHero, this));
}

ArmyInput.MAX_CHARS = 8;

ArmyInput.prototype.showCountInput = function(e)
{
	this.currentUnit = COM(e.currentTarget);
	
	var input = COM.create("CalcUnitProps");
	input.editableLevels = this.editableLevels;
	input.popup(this.currentUnit.level, this.currentUnit.countInt, $.proxy(this.onChangeCurrentUnitLevel, this), $.proxy(this.onChangeCurrentUnitCount, this));
	input.onClose = $.proxy(function()
	{
		input.countChanged();
		this.currentUnit = null;
	}, this);
}

ArmyInput.prototype.onChangeCurrentUnitCount = function(count)
{
	this.currentUnit.initCount(count, this.buffs);
	this.onChangeUnits();
}

ArmyInput.prototype.onChangeCurrentUnitLevel = function(level)
{
	this.currentUnit.initUnitLevel(level);
	this.onLevelChanged();
}

ArmyInput.prototype.onLevelChanged = function()
{
    var level = 0;
    if (this.armyView)
    {
        var units = COM(this.armyView).units;
        for(var i=0; i<units.length; i++)
        {
        	level = level > 0 ? level : units[i].level;
            if (level != units[i].level)
            {
                level = 0;
                break;
            }
        }
    }
    this.onAllChangeLevels(level);
}

ArmyInput.prototype.hideCountInput = function()
{
    if (this.currentUnit)
    {
        this.currentUnit.initCount(this.currentUnit.getCountInt(), this.buffs);
    }
    this.currentUnit = null;
}

ArmyInput.prototype.refresh = function()
{
    this.update(this.getArmy(), this.getLevelsArray());
}

ArmyInput.prototype.update = function(army, levels)
{
    var av = COM(this.armyView);
    
    av.setAllBacks(this.race);
    av.setUnits(this.race, levels, this.buffs ? FBuff.hasBuff(this.buffs, BuffType.MONSTER_RACE, null, Tick.now()) : false, this.aura, this.buffs);
    av.setCount(army);
    this.setHero(this.selectedHero);
    this.onLevelChanged();
}

ArmyInput.prototype.setAllLevels = function(level)
{
    var units = COM(this.armyView).units;
    for(var i=0; i<units.length; i++)
    {
    	units[i].initUnitLevel(level);
    }
}

ArmyInput.prototype.getArmy = function()
{
    return Army.create(this.getArmyArray());
}

ArmyInput.prototype.getLevelsArray = function()
{
    var ret = [];
    
    var units = COM(this.armyView).units;
    for(var i=0; i<units.length; i++)
    {
    	ret.push(units[i].level);
    }
    
    return ret;
}

ArmyInput.prototype.getArmyArray = function()
{
    var ret = [];
    
    var units = COM(this.armyView).units;
    for(var i=0; i<units.length; i++)
    {
    	ret.push(units[i].getCountInt());
    }
    
    return ret;
}

ArmyInput.prototype.setShowHero = function(show)
{
    //UI.show(show, [buttonHeroOut]);
    //slotsView.setShowHero(show);
    COM(this.armyView).showHero = show;
}

ArmyInput.prototype.onChooseHero = function(event)
{
	this.setHero(null);
    
    if(this.heroes && this.heroes.length > 0)
    {
		var chooseHero = COM.create("ChooseHero");
		chooseHero.onHeroSelect = $.proxy(this.onHeroSelected, this);
		chooseHero.show(this.heroes, this.race);
		chooseHero.popup();
	}
}

ArmyInput.prototype.onHeroSelected = function(fh)
{
    this.setHero(fh);
}

ArmyInput.prototype.setHero = function(fh)
{
    COM(this.armyView).setHero(fh, this.race);
    if (this.selectedHero != fh)
    {
        this.selectedHero = fh;
        this.onChangeHero();
    }
}