function UnitTip()
{
	this.__defineGetter__('showUnitType', function(){ return this.typeBox.style.display != 'none'; })
	this.__defineSetter__('showUnitType', function(v){ UI.show(Boolean(v), [this.typeBox]); })
	
	this.__defineGetter__('showStat', function(){ return this.statBox.style.display != 'none'; })
	this.__defineSetter__('showStat', function(v){ UI.show(Boolean(v), [this.statBox]); })
	
	this.__defineGetter__('showSpecialAbility', function(){ return this.specialAbility.style.display != 'none'; })
	this.__defineSetter__('showSpecialAbility', function(v){ UI.show(Boolean(v), [this.specialAbility]); })
}


UnitTip.prototype.__construct = function()
{
	
}

UnitTip.STAT_FORMAT = "{0}";
UnitTip.STAT_FORMAT_AURA = "{0} ({1})";

UnitTip.militaryTips = new Array(8);

UnitTip.updateMilitaryTip = function(type, level, r, aura, selectedAttack, buffs, round, tick)
{
    if (type.isMilitary() < 0) return;
    UnitTip.militaryTips[type.ordinal].update(type, level, r, selectedAttack, aura, buffs, round, tick);
}

UnitTip.initMilitaryTip = function(type, level, r, aura, selectedAttack, buffs, round, tick)
{
    if (type.isMilitary() < 0) return;
    UnitTip.militaryTips[type.ordinal].initUpdate(type, level, r, selectedAttack, aura, buffs, round, tick);
}

UnitTip.getMilitaryTip = function(type, level, r, aura, selectedAttack, buffs, round, tick)
{
    if (type.isMilitary() < 0) return;
    var ord = type.ordinal;
    if (!UnitTip.militaryTips[ord]) 
    {
        UnitTip.militaryTips[ord] = COM.create('UnitTip');
        UnitTip.initMilitaryTip(type, level, r, aura, selectedAttack, buffs, round, tick);
    } 
    else 
    {
        UnitTip.updateMilitaryTip(type, level, r, aura, selectedAttack, buffs, round, tick);
    }
    return UnitTip.militaryTips[ord];
}

UnitTip.create = function(type, level, r, aura, selectedAttack, buffs, round, tick)
{
    var t;
    if (type.isMilitary() >= 0) 
    {
        t = UnitTip.getMilitaryTip(type, level, r, aura, selectedAttack, buffs, round, tick);
    } 
    else 
    {
        var dom = document.createElement('div');
        dom.innerHTML = I18.getSpecialUnitDescription(type);
        t = {DOM: dom};
    } 
    return t;
}

UnitTip.clear = function()
{
	for(var i=0; i<UnitTip.militaryTips.length; i++)
	{
		if(UnitTip.militaryTips[i] && UnitTip.militaryTips[i].destroy)
		{
			UnitTip.militaryTips[i].destroy();
		}
	}
	UnitTip.militaryTips = [];
}

UnitTip.prototype.initUpdate = function(type, level, r, selectedAttack, aura, buffs, round, tick)
{
    if (type.isMilitary() < 0) return;
    
    var ourVictim = FConfig.getVictimFor(type);
    UI.show(ourVictim, [this.ourVictimBox]);
    if (ourVictim)
    {
        this.thisAsPersecutor.src = STATIC_URL + Assets.getUnitTypeIcon(type);
        this.victim.src = STATIC_URL + Assets.getUnitTypeIcon(ourVictim);
    }
    
    var ourPersecutor = FConfig.getPersecutorFor(type);
    UI.show(ourPersecutor, [this.ourPersecutorBox]);
    if (ourPersecutor)
    {
        this.thisAsVictim.src = STATIC_URL + Assets.getUnitTypeIcon(type);
        this.persecutor.src = STATIC_URL + Assets.getUnitTypeIcon(ourPersecutor);
    }
    
    UI.show(ourVictim || ourPersecutor, [this.persecutorsBox]);
    
    var hasAbility = Arrays.contains([UnitType.SCAVENGER.name, UnitType.MINER.name, UnitType.HEALER.name, UnitType.MAGE.name], type.name);
    UI.show(hasAbility, [this.specialAbility]);

    this.update(type, level, r, selectedAttack, aura, buffs, round, tick);
}

UnitTip.prototype.update = function(type, level, r, selectedAttack, aura, buffs, round, tick)
{
    if (type.isMilitary() < 0) return;
    var stat = FConfig.getStat(r, type, level);
    var bonus = aura.getBonus(type);
    var bs = new BuffSummator(buffs, type, tick, false);
    this.typeIcon.src = STATIC_URL + Assets.getUnitTypeIcon(type);
    this.unitName.innerHTML = I18.getUnitName(type, level, r);

    var damageBonus = CombatBonusUtil.getDamageBonus(bonus, bs, round);
    var defenseBonus = FBuff.hasBuff(buffs, BuffType.MAX_DEFENCE, type, tick) ? FConfig.MAX_MAX_DEFENCE : bonus.defence + bs.sum(BuffType.DEFENSE_BONUS);
    var healthBonus = bonus.health / 100 + bs.sum(BuffType.HEALTH_BONUS);
    var healingBonus = bs.sum(BuffType.HEALING_BONUS);
    var towerPreventionBonus = bs.sum(BuffType.MAGE_TOWER_RESISTANCE);
    var scavengerCapacityBonus = bs.sum(BuffType.SCAVENGERS_CAPACITY);
    var persecutionBonus = bonus.persecution + bs.sum(BuffType.PERSECUTION_BONUS);

    var damageString = (selectedAttack != -1 ? selectedAttack : stat.damage).toString();
	//console.log(this.damage, COM(this.damage));
	COM(this.damage).update(damageString, damageBonus, FConfig.MIN_DAMAGE_PERCENT - 1);
    COM(this.defence).updateWithLimit(stat.defence, defenseBonus,
            NumberUtils.clamp(FConfig.MAX_DEFENCE + NumberUtils.floor(FBuff.sumOfBuffs(buffs, BuffType.INCREASE_MAX_DEFENCE, null, tick)), 0, FConfig.MAX_MAX_DEFENCE), 0);
    COM(this.health).update(stat.health.toString(), healthBonus, FConfig.MIN_HEALTH_PERCENT - 1);

    this.persecutorBonus.innerHTML = Strings.formatBonusPercent(persecutionBonus);
    UI.stylePercentBonus(persecutionBonus, this.persecutorBonus, FConfig.MIN_DAMAGE_PERCENT - 1);

    //special ability
    if (this.showSpecialAbility) 
    {
        $(this.specialAbility).html('');
        switch (type.name) 
        {
            case UnitType.SCAVENGER.name:
            	$(this.specialAbility).append(scavengerCapacityBonus != 0
					? I18.box(I18.f('UNIT_ABILITY_SCAVENGER_BONUS'), stat.capacity, scavengerCapacityBonus)
					: I18.f('UNIT_ABILITY_SCAVENGER', stat.capacity)
				);
                break;
            
            case UnitType.MINER.name:
                $(this.specialAbility).append(I18.f('UNIT_ABILITY_MINER'));
                break;
            
            case UnitType.HEALER.name:
                $(this.specialAbility).append(healingBonus != 0
					? I18.box(I18.f('UNIT_ABILITY_HEALER_BONUS'), stat.heal, healingBonus)
					: I18.f('UNIT_ABILITY_HEALER', stat.heal)
                );
                break;
            
            case UnitType.MAGE.name:
                $(this.specialAbility).append(towerPreventionBonus != 0
                    ? I18.box(I18.f('UNIT_ABILITY_MAGE_BONUS'), stat.towerResistance, towerPreventionBonus)
                    : I18.f('UNIT_ABILITY_MAGE', stat.towerResistance)
                );
                break;
        }
    }
}
