var BaseResourceBox = function()
{
	this.showZero = false; // Boolean = false
	this.allZeroText = ""; // String = ""
	this.showFraction = false; // Boolean = false
	this.megaStringKey = ""; // String
	this.allPeople = false; // Boolean = false
	this._view = null; // BaseFundsView
	this._funds = new Funds(); // Funds = new Funds()
	this._race = new Race(); // Race
	this._created = false; // Boolean = false
	this._alwaysShow = []; // Array = []       // Resource[]
	this._alwaysHide = []; // Array = []       // Resource[]
	this._neverHighlight = []; // Array = []   // Resource[]
	this._tipChanged = false; // Boolean = false
	this._onTip = function(r){};
	this._itemWidth = -1; // Number = -1
	
	this.__defineSetter__('onTip', function(value)
	{
	    this._onTip = value;
	    this._tipChanged = true;
	    if (this._created) 
	    {
	        this.invalidateTip();
	    }
	});
	
	this.__defineSetter__('alwaysHide', function(value)
	{
	    this._alwaysHide = value;
	    if (this._funds) 
	    {
	        this.update(this._funds, null);
	    }
	});
	
	this.__defineSetter__('alwaysShow', function(value)
	{
	    this._alwaysShow = value;
	    if (this._funds) 
	    {
	        this.update(this._funds, null);
	    }
	});
	
	this.__defineGetter__('itemWidth', function()
	{
	    return this._itemWidth;
	});
	
	this.__defineSetter__('itemWidth', function(value)
	{
	    this._itemWidth = value;
	    this.updateItemWidth();
	});
	
	this.__defineSetter__('type', function(t)
	{
	    var viewClass = BaseResourceBox.VIEWS[t];
	    if (!viewClass)
	    {
	        console.errors("Could not find view class \"%s\" for ResourceBox", t);
	    }
	    this.createView(viewClass);
	});
	
	this.__defineGetter__('funds', function()
	{
	    return this._funds;
	});
	
	this.__defineGetter__('allZero', function()
	{
	    return this._view.allZero;
	});
	
	this.shortView = false;
	this.shortViewCharsLimit = 6;
}

BaseResourceBox.FRACTION_MIN_VALUABLE = 0.0001;
BaseResourceBox.FRACTION_PRECISION = 3;

BaseResourceBox.VIEWS = 
{
    hbox: 'HBoxFundsView',
    tile: 'TiledFundsView',
    fixed: 'FixedFundsView'
};
BaseResourceBox.DEFAULT_VIEW = BaseResourceBox.VIEWS.hbox;

BaseResourceBox.prototype.showZero = false; // Boolean = false
BaseResourceBox.prototype.allZeroText = ""; // String = ""
BaseResourceBox.prototype.showFraction = false; // Boolean = false
BaseResourceBox.prototype.megaStringKey = ""; // String
BaseResourceBox.prototype.allPeople = false; // Boolean = false
BaseResourceBox.prototype._view = null; // BaseFundsView
BaseResourceBox.prototype._funds = new Funds(); // Funds = new Funds()
BaseResourceBox.prototype._race = null; // Race
BaseResourceBox.prototype._created = false; // Boolean = false
BaseResourceBox.prototype._alwaysShow = []; // Array = []       // Resource[]
BaseResourceBox.prototype._alwaysHide = []; // Array = []       // Resource[]
BaseResourceBox.prototype._neverHighlight = []; // Array = []   // Resource[]
BaseResourceBox.prototype._tipChanged = false; // Boolean = false
BaseResourceBox.prototype._onTip = function(r)
{
    //return Tip.invisible();
};
BaseResourceBox.prototype._itemWidth = -1; // Number = -1

BaseResourceBox.prototype.__construct = function()
{
	this.boxTooltipHandler = $.proxy(this.boxTooltipHandler, this);
	
	var el = $(this.DOM);
	
	this.showZero = Boolean(el.attr('showZero'));
	this.allZeroText = el.attr('allZeroText') || "";
	this.showFraction = Boolean(el.attr('showFraction'));
	this.type = el.attr('view') || 'hbox';

    if (this._view == null) 
    {
        this.createView(BaseResourceBox.DEFAULT_VIEW);
    }
    this.invalidateTip();
    this._created = true;
}

BaseResourceBox.prototype.deattachTip = function()
{
    return; // deprecated
}

BaseResourceBox.prototype.invalidateTip = function()
{
	var self = this;
	$('iconandvalue', this._view.DOM).each(function(n, iav)
	{
		$(iav).off(CLICK_EVENT);
		COM.bindExclusiveClick(iav, self.boxTooltipHandler);
	});
}

BaseResourceBox.prototype.boxTooltipHandler = function(e)
{
    if(!this._onTip) return;
    
    var res = Resource[$(e.currentTarget).attr('name').toUpperCase()];
    this._onTip(res);
}

BaseResourceBox.prototype.update = function(f, race)
{
	this._funds = f;
    if (race && !race.isSameRace(this._race)) 
    {
        this._race = race;
        this.updatePeopleIcon();
    }
    
    for (var i = 0; i < Resource.FUNDS_RESOURCES.length; i++)
    {
    	var r = Resource.FUNDS_RESOURCES[i];
    	this.setVal(r, f.getValue(r));
    }
    if (this.allZero) 
    {
        if (f.isEmpty())
        {
        	this.allZero.innerHTML = this.allZeroText;
        	this.allZero.style.display = "";
        }
        else
        {
        	this.allZero.style.display = "none";
        }
    }
    this.refreshHighlight();
}

BaseResourceBox.prototype.updatePeopleIcon = function()
{
    COM.get(this.getIcon(Resource.PEOPLE)).source = STATIC_INTERFACE_URL + (this.allPeople ? Assets.getAllPeopleIcon(this._race) : Assets.getFreePeopleIcon(this._race));
}

BaseResourceBox.prototype.applyStyle = function(resources, s)
{
	for (var i = 0; i < resources.length; i++)
	{
		$(this.getIcon(resources[i])).addClass(s);
	}
}

BaseResourceBox.prototype.styleFunds = function(resources, isBadValue)
{
	for (var i = 0; i < resources.length; i++)
	{
		COM(this.getIcon(resources[i])).styleFunds(isBadValue);
	}
}

BaseResourceBox.prototype.getIcon = function(r)
{
    return this._view.getIcon(r);
}

BaseResourceBox.prototype.setVal = function(r, value)
{
    var stringValue = this.toStringForm(value);
    
    if(this.shortView && stringValue.length > this.shortViewCharsLimit)
	{
		stringValue = Math.floor(value/1000) + "K";
	}
    
    var doShow = (stringValue != "0" || this.showZero);
    
    for (var i = 0; i < this._alwaysHide.length; i++)
    {
    	if (r.name == this._alwaysHide[i].name) 
    	{
    		doShow = false;
    	}
    }
    
    for (var i = 0; i < this._alwaysShow.length; i++)
    {
    	if (r.name == this._alwaysShow[i].name) 
    	{
    		doShow = true;
    	}
    }
    
    COM(this.getIcon(r)).value = (!this.megaStringKey || this.megaStringKey == "") ? stringValue : I18.getMegaString(this.megaStringKey, [stringValue]);
    
    if(doShow)
    {
    	this.getIcon(r).style.display = "";
    }
    else
    {
    	this.getIcon(r).style.display = "none";
    }
}

BaseResourceBox.prototype.refreshHighlight = function()
{
    // do nothing
}

BaseResourceBox.prototype.toStringForm = function(value)
{
    if (isNaN(value)) 
    {
        return "0";
    }
    if (!this.hasValuableFraction(value)) 
    {
        return Math.round(value).toString();
    }
    if (!this.showFraction) 
    {
        return NumberUtils.floor(value).toString();
    }
    return Strings.formatNumber(value, BaseResourceBox.FRACTION_PRECISION);
}

BaseResourceBox.prototype.createView = function(viewClass)
{
    this._view = COM.create(viewClass);
    this.updateItemWidth();
    
    this.DOM.innerHTML = "";
    this.DOM.appendChild(this._view.DOM);
}

BaseResourceBox.prototype.updateItemWidth = function()
{
    if (this._itemWidth < 0) 
    {
        return;
    }

    this._view.itemWidth = this._itemWidth;
}

BaseResourceBox.prototype.hasValuableFraction = function(v)
{
    return Math.abs(v - Math.round(v)) > BaseResourceBox.FRACTION_MIN_VALUABLE;
}

BaseResourceBox.prototype.showIconBacks = function(val)
{
	if(val) $("iconandvalue", this.DOM).addClass("darkBack");
	else $("iconandvalue", this.DOM).removeClass("darkBack");
}