function HSlider()
{
	this.__defineGetter__('min', this.getMin);
	this.__defineSetter__('min', this.setMin);
	
	this.__defineGetter__('max', this.getMax);
	this.__defineSetter__('max', this.setMax);
	
	this.__defineGetter__('limit', this.getLimit);
	this.__defineSetter__('limit', this.setLimit);
	
	this.__defineGetter__('value', this.getValue);
	this.__defineSetter__('value', this.setValue);
	
	this.__defineGetter__('enabled', this.getEnabled);
	this.__defineSetter__('enabled', this.setEnabled);
	
	this._limit = false;
	this._active = false;
}

HSlider.prototype._limit = false;
HSlider.prototype._active = false;

HSlider.prototype.__construct = function()
{
	try
	{
		this.min = this.DOM.hasAttribute('min') ? this.DOM.getAttribute('min')*1 : 0;
		this.max = this.DOM.hasAttribute('max') ? this.DOM.getAttribute('max')*1 : 1;
		if (this.DOM.hasAttribute('step')) this.input.step = this.DOM.getAttribute('step')*1 || 1;
		this.value = this.DOM.hasAttribute('value') ? this.DOM.getAttribute('value')*1 : this.min;
		
		$(this.input).on('input', $.proxy(this.triggerChange, this));
		$(this.input).on('dblclick', function(e){
			e.preventDefault();
			e.stopImmediatePropagation();
			return false;
		});
		
		this.triggerSelect = $.proxy(this.triggerSelect, this);
		this.DOM.addEventListener(MOUSE_DOWN_EVENT, $.proxy(function(){
			document.addEventListener(MOUSE_MOVE_EVENT, this.preventIScroll, false);
			document.addEventListener(MOUSE_UP_EVENT, this.triggerSelect, false);
		}, this));
	}
	catch (e) { alert(e.message); }
}

HSlider.prototype.preventIScroll = function(e)
{
	e.stopPropagation();
	return true;
}

HSlider.prototype.triggerChange = function()
{
	if (this.value > this.limit) this.value = this.limit;
	if (typeof this['onChange'] == 'function')
	{
		this.onChange(this.value, this);
	}
	$(this.DOM).trigger($.Event('change'));
}

HSlider.prototype.triggerSelect = function()
{
	document.removeEventListener(MOUSE_MOVE_EVENT, this.preventIScroll, false);
	document.removeEventListener(MOUSE_UP_EVENT, this.triggerSelect, false);
	if (this.value > this.limit) this.value = this.limit;
	if (typeof this['onSelect'] == 'function')
	{
		this.onSelect(this.value, this);
	}
	$(this.DOM).trigger($.Event('select'));
	$(this.DOM).trigger($.Event('finish'));
}

HSlider.prototype.setValues = function(val, min, max)
{
	if (typeof val == 'undefined') return;
	if (typeof min == 'undefined') min = this._min;
	if (typeof max == 'undefined') max = this._max;
	this.min = Math.min(min, max);
	this.max = Math.max(min, max);
	this.value = Math.max(this._min, Math.min(this._max, val));
}

HSlider.prototype.getMin = function() { return ~~this.input.min; }
HSlider.prototype.setMin = function(v)
{
	this.input.min = ~~v;
	if (this.value < this.min) this.value = this.min;
}

HSlider.prototype.getMax = function() { return ~~this.input.max; }
HSlider.prototype.setMax = function(v)
{
	this.input.max = ~~v;
	if (this.value > this.max) this.value = this.max;
}

HSlider.prototype.getLimit = function() 
{ 
	return isNaN(parseFloat(this._limit)) 
		? this.max 
		: Math.min(this.max, this._limit)
	; 
}
HSlider.prototype.setLimit = function(v)
{
	v = parseFloat(v);
	if (isNaN(v)) 
	{
		this._limit = false; // no limits
		return;
	}
	
   	this._limit = v;
   	if (this._value > v) this.value = v;
}

HSlider.prototype.getValue = function() { return ~~this.input.value; }
HSlider.prototype.setValue = function(v)
{
	this.input.value = Math.max(this.min, Math.min(this.limit, ~~v));
}

HSlider.prototype.getEnabled = function() { return !this.input.disabled; }
HSlider.prototype.setEnabled = function(v)
{
	this.input.disabled = !v;
	$(this.DOM).toggleClass('disabled', !v);
}

HSlider.prototype.updateUI = function(){}
