function TeeterVSlider()
{
    this.size = TeeterVSlider.BIG;

    this._movingFast = 0;
    this._multiplier = 1;
    this._startFastTick = Number.MAX_VALUE;
    this._msSinceLastStep = 0;
      
    this._tickerPeriod = 100;
    this._tickerInitialDelay = 300;
    this._ticker = new Ticker(this._tickerPeriod, $.proxy(this.onTick, this));
    
    this.__defineSetter__("multiplier", this.setMultiplier);
    this.__defineGetter__("_isPressed", this.get_isPressed);
	this.__defineGetter__("value", this.getValue);
	this.__defineSetter__("value", this.setValue);
	this.__defineGetter__("maximum", this.getMaximum);
	this.__defineSetter__("maximum", this.setMaximum);
	this.__defineGetter__("sliderEnabled", this.getSliderEnabled);
	this.__defineSetter__("sliderEnabled", this.setSliderEnabled);
}

TeeterVSlider.BIG = {source:CommonAssets.EQUALIZER_SLIDER, height:87};
TeeterVSlider.SMALL = {source:CommonAssets.EQUALIZER_SLIDER_SMALL, height:60};
TeeterVSlider.TINY = {source:CommonAssets.EQUALIZER_SLIDER_TINY, height:36};

TeeterVSlider.prototype.__construct = function()
{
	$(this.up).on(MOUSE_DOWN_EVENT, $.proxy(this.startFastUp, this));
	$(this.up).on("mouseout", $.proxy(this.stopFast, this));
	$(this.up).on(MOUSE_UP_EVENT, $.proxy(this.onReleaseUp, this));
	
	$(this.down).on(MOUSE_DOWN_EVENT, $.proxy(this.startFastDown, this));
	$(this.down).on("mouseout", $.proxy(this.stopFast, this));
	$(this.down).on(MOUSE_UP_EVENT, $.proxy(this.onReleaseDown, this));
	
	this.value = 0;
	this.slider.maximum = 100;
	this.slider.visible = true;
}

TeeterVSlider.prototype.onChange = function() {};

TeeterVSlider.prototype.setMultiplier = function(val)
{
    this._multiplier = val;
}

TeeterVSlider.prototype.initialDelayPassed = function()
{
    return Tick.timeInMs() - this._startFastTick > this._tickerInitialDelay;
}

TeeterVSlider.prototype.onTick = function(delta)
{
    if (this._isPressed && this.initialDelayPassed())
    {
        this._msSinceLastStep += delta * 1000; // delta is in ms
        var fullSteps = this._msSinceLastStep / this._tickerPeriod;
        var shift = this._movingFast * fullSteps;
        this._msSinceLastStep -= fullSteps * this._tickerPeriod;
        
        this.moveSlider(shift);
    }
}

TeeterVSlider.prototype.get_isPressed = function()
{
    return this._movingFast != 0;
}

TeeterVSlider.prototype.getValue = function()
{
    return this.slider.value;
}

TeeterVSlider.prototype.setValue = function(v)
{
    this.slider.value = v;
    this.slider.innerHTML = Math.floor(v);
}

TeeterVSlider.prototype.getMaximum = function()
{
    return this.slider.maximum;
}

TeeterVSlider.prototype.setMaximum = function(v)
{
    this.slider.maximum = v;
}

TeeterVSlider.prototype.getSliderEnabled = function()
{
    return this.slider.visible;
}

TeeterVSlider.prototype.setSliderEnabled = function(v)
{
    this.slider.visible = v;
    this.up.disabled = !v;
    this.down.disabled = !v;
}

TeeterVSlider.prototype.stopFast = function(e)
{
    this._movingFast = 0;
    this._startFastTick = Number.MAX_VALUE;
    this._msSinceLastStep = 0;
    this._ticker.stop();
}

TeeterVSlider.prototype.startFast = function(mult)
{
    this._movingFast = mult;
    this._startFastTick = Tick.timeInMs();
    this._ticker.start();
}

TeeterVSlider.prototype.startFastUp = function(e)
{
    this.startFast(1);
}

TeeterVSlider.prototype.startFastDown = function(e)
{
    this.startFast(-1);
}

TeeterVSlider.prototype.onRelease = function(value)
{
    if (this._isPressed && this._movingFast == value)
    {
        this.moveSlider(this._movingFast);
    }
    this.stopFast();
}

TeeterVSlider.prototype.onReleaseUp = function(e)
{
    this.onRelease(1);
}

TeeterVSlider.prototype.onReleaseDown = function(e)
{
    this.onRelease(-1);
}

TeeterVSlider.prototype.onSliderChange = function(e)
{
    /*
    if (e.value != slider.maximum) {
        var cnt:int = (e.value + _multiplier / 2) / _multiplier;
        e.value = cnt * _multiplier;
    }
    onChange(this, e.value);
    */
}

TeeterVSlider.prototype.moveSlider = function(delta)
{
    if (this.sliderEnabled)
    {
        this.value += delta * this._multiplier;
        if (this.value != this.maximum)
        {
            if (delta > 0)
            {
                //this.value = NumberUtils.roundDownToDivide(this.value, this._multiplier);
                if(this.value > this.maximum)
                {
                	this.value = this.maximum;
                }
            }
            else
            {
                //this.value = NumberUtils.roundUpToDivide(this.value, this._multiplier);
                if(this.value < 0)
                {
                	this.value = 0;
                }
            }
        }
    }
    
    this.onChange(this, this.value);
}