function FinanceInputTab()
{
	this.selectedPayment = null;
    this.selectedTariffRenderer = null;
    this.selectedPaymentChanged = null;
    this.currentActionHandler = null;
    this.account = null;
    this.receiverUsername = null;
    this.servers = null;
    this.showSinglePortalAccount = null;
    this.phoneMin = 10;
    this.phoneMax = 10;
    
    this.parentDialog = null;

    this.showMandatory = false;
    this.showPortalPurseButton = false;

    this.dialog = null;
    
    this.__defineGetter__("serverId", this.getServerId);
    this.__defineGetter__("selectedPaymentOption", this.getSelectedPaymentOption);
}

FinanceInputTab.prototype.__construct = function()
{
	this.onPurchaseComplete = $.proxy(this.onPurchaseComplete, this);
	
	$(this.chooseServer).on("change", $.proxy(this.onServerChange, this));
	COM(this.tariffs).onItemClick = $.proxy(this.onPaymentClick, this);
	
	if(window.PurchasesManager)
    {
        PurchasesManager.purchaseCompleteCallback = this.onPurchaseComplete;
        PurchasesManager.errorCallback = APP.hideLoader;
    }
	
	$(this.refreshButton).on(CLICK_EVENT, $.proxy(function(){
		this.updateTariffs();
	}, this));
	
	UI.show(ANDROID_MODE, [this.refreshButton]);
	
	//$(this.paymentOptions).on("change", $.proxy(this.onPaymentOptionChange, this));
	//COM.bindExclusiveClick(this.financeHelpButton, $.proxy(this.showFinanceHelp, this));
	
	//this.paymentMethodLabel.innerHTML = I18.f("CHOOSE_PAYMENT_METHOD");
	//this.frodDisabledWarning.innerHTML = I18.f("CLIENT_KNOWS_ERROR_FROD_PAYMENTS_DISABLED");
	//this.chooseTariffLabel.innerHTML = I18.f("CHOOSE_TARIFF");
	//this.remarkLabel.innerHTML = I18.f("REMARK");
	
	//COM(this.financeHelpButton).caption = I18.f("FINANCE_HELP");
	//COM(this.chatButton).caption = I18.f("ASK_QUESTION_IN_CHAT");
}

FinanceInputTab.prototype.__destruct = function()
{
	if(window.PurchasesManager)
    {
        PurchasesManager.purchaseCompleteCallback = null;
        PurchasesManager.errorCallback = null;
    }
}

FinanceInputTab.prototype.update = function(acc, serverId, showMandatory)
{
    this.account = acc;
    this.showMandatory = !!showMandatory;

    var newServers = [], gs;
    for(var i=0; i<this.account.servers.length; i++)
    {
    	gs = this.account.servers[i];
    	if (gs.avatarExists && gs.config.moneyTransferAllowed)
    	{
            newServers.push(gs);
        }
    }

    // adding portal as first server
    var portal = new FGameServer();
    portal.id = null;
    portal.displayName = I18.f("PORTAL");
    newServers.unshift(portal);

    this.servers = newServers;
    if (!serverId)
    {
        serverId = this.servers.length > 0 ? this.servers[0].id : null;
    }
    
    var s, selected;
    var cb = COM(this.chooseServer);
    cb.clear();
    
    for(var i=0; i<this.servers.length; i++)
    {
    	s = this.servers[i];
    	cb.addOption(this.servers[i], this.servers[i].displayName, false);

    	if (s.id == serverId)
    	{
			selected = s;
        }
    }
    cb.selectedItem = selected;
    cb.refresh();
    
    UI.show(Boolean(this.receiverUsername), [this.receiverUsernameLabel], $.proxy(function()
    {
    	this.receiverUsernameLabel.innerHTML = I18.f("RECEIVER_USERNAME", this.receiverUsername);
    }, this));
    
    /*
    UI.show(!this.account.canFrod(this.receiverUsername), [this.frodDisabledWarning], $.proxy(function()
    {
        this.frodDisabledWarning.innerHTML = this.account.frodEnabled ? I18.f("CLIENT_KNOWS_ERROR_FROD_GIFTS_DISABLED") : I18.f("CLIENT_KNOWS_ERROR_FROD_PAYMENTS_DISABLED");
    }, this));
    
    this.toPortalWarning.innerHTML = FinanceInputTab.getPaymentOnPortalWarningString();
    */

	this.updateTariffs();
	
	this.onServerChange();
}

FinanceInputTab.prototype.fitLayout = function(container)
{
	this.tariffs.style.height = (container.height - this.headerRow.offsetHeight - 8) + "px";
	COM(this.tariffs).refresh();
}

FinanceInputTab.prototype.getServerId = function()
{
    return COM(this.chooseServer).selectedItem.data.id;
}

FinanceInputTab.prototype.onServerChange = function()
{
    this.buyTo.innerHTML = this.isFromServer() ? I18.f("BUY_PEARL_TO") : I18.f("BUY_MDOLLARS_TO", I18.f("PORTAL_CURRENCY"));
    //UI.show(!this.isFromServer(), [this.toPortalWarning]);
    this.showPaymentMethods();
}

FinanceInputTab.prototype.isFromServer = function()
{
    return Boolean(this.serverId);
}

FinanceInputTab.prototype.showPaymentMethods = function()
{
    if(this.parentDialog)
    {
    	this.parentDialog.showTransferButton(this.isFromServer() && !this.receiverUsername);
    	
    	this.parentDialog.setTransferButtonListener($.proxy(function()
    	{
            TransferDialog.popup(null, this.serverId, $.proxy(function()
            {
                PortalService.account.callMethod("getAccount", [], $.proxy(function(acc)
                {
                    this.account = acc;
                }, this));
            }, this));
        }, this));
    }
}

FinanceInputTab.prototype.getCurrentPaymentSystem = function()
{
	var id = "apple";
	if(ANDROID_MODE) id = "google";
	
	for(var i=0; i<FPortalConfig.PAYMENTS.length; i++)
	{
		if(FPortalConfig.PAYMENTS[i].id == id) return FPortalConfig.PAYMENTS[i];
	}
	
	return FPortalConfig.PAYMENTS[0];
}

FinanceInputTab.prototype.updateTariffs = function()
{
	if(!FPortalConfig.PAYMENTS || !FPortalConfig.PAYMENTS[0]) return;
	
	var system = this.getCurrentPaymentSystem();
	var option = system.options[0];
	var tariffs = option.tariffs;
	
	var ids = [];
	for(var i=0; i<tariffs.length; i++)
	{
		ids.push(tariffs[i].id);
	}
	PurchasesManager.refresh(ids, $.proxy(this.renderTariffs, this));
}

FinanceInputTab.parsePrice = function(price)
{
	return String(price).replace(new RegExp('[^0-9\.]', 'g'), '') * 1 || 0;
}

FinanceInputTab.sortItems = function(items)
{
	items.sort(function(item1, item2)
	{
		return FinanceInputTab.parsePrice(item1.price) > FinanceInputTab.parsePrice(item2.price) ? 1 : -1;
	});
	
	return items;
}

FinanceInputTab.prototype.renderTariffs = function(products)
{
	var items = PurchasesManager.prepareItems(products);
	items = FinanceInputTab.sortItems(items);
	
	var cb = COM(this.tariffs);
	var row;
	
	if (cb)
	{
		UI.show(true, [this.tariff]);
		cb.dataProvider = []; 
		for (var i=0; i<items.length; i++)
		{
			row = COM.create("ScrollableListItem");
			row.data = [items[i].title, items[i].price];
			
			row.productId = items[i].id;
			
			cb.addItem(row);
		}
		UI.show(false, [this.refreshButton]);
	}
	else
	{
		console.error("No tariffs container!");
	}
    
    //this.account = null;
    //this.receiverUsername = null;
}

FinanceInputTab.prototype.showTariffs = function()
{
    /*
    page.selectedChild = tariff;
    selectedTariffRenderer = null;
    var p:Payment = selectedPayment;
    if (selectedPaymentChanged) {
        var localOption:PaymentOption = null;
        var otherOption:PaymentOption = null;
        for (var i:int = p.options.length - 1; i >= 0; --i) {
            var option:PaymentOption = PaymentOption(p.options.getItemAt(i));
            if (option.isFrod && !account.canFrod(receiverUsername)) {
                p.options.removeItemAt(i);
            } else if (localOption == null && option.id.substr(0, 2) == account.country) {
                localOption = option;
            } else if (otherOption == null && option.id.substr(0, 5) == "other") {
                otherOption = option;
            }
        }
        paymentOptions.dataProvider = p.options;
        if (p.options.length > 0) {
            if (localOption != null) {
                paymentOptions.selectedItem = localOption;
            } else if (otherOption != null) {
                paymentOptions.selectedItem = otherOption;
            } else {
                paymentOptions.selectedIndex = 0;
            }
        }
        selectedPaymentChanged = false;
    }
    UI.show(p.options.length > 1, [paymentOptions]);
    refreshTarrifs();
    refreshChatButton();
    */
}

FinanceInputTab.prototype.refreshTarrifs = function()
{
    /*
    var count:int = ((selectedPaymentOption.isFrod && !account.canFrod(receiverUsername)) || selectedPaymentOption.tariffs == null) ? 0 : selectedPaymentOption.tariffs.length;
    while (tariffs.getChildren().length < count) {
        var tr:TariffRenderer = new TariffRenderer();
        tr.onClick = onTariffClick;
        tariffs.addChild(tr);
    }
    while (count < tariffs.getChildren().length) {
        tariffs.removeChildAt(tariffs.getChildren().length - 1);
    }
    for (var i:int = 0; i < count; i++) {
        var renderer:TariffRenderer = tariffs.getChildAt(i) as TariffRenderer;
        var t:Tariff = selectedPaymentOption.tariffs.getItemAt(i) as Tariff;
        renderer.init(i, t);
    }
    var remark:String = selectedPaymentOption.remark == null ? "" : selectedPaymentOption.remark;
    remarkTxt.text = remark.split("\\n").join("\n");
    UI.show(remark.length > 0, [remarkBox]);
    */
}

FinanceInputTab.prototype.refreshChatButton = function()
{
    /*
    var active:Boolean = selectedPaymentOption.chatUrl != null;
    if (active) {
        chatButton.addEventListener(MouseEvent.CLICK, function (event:Event):void {
            navigateToURL(new URLRequest(selectedPaymentOption.chatUrl), "_blank");
        });
    }
    UI.show(active, [chatButton]);
    */
}

FinanceInputTab.prototype.onPaymentClick = function(pr)
{
    if (!FPortalConfig.PAYMENTS_ALLOWED && this.account.role.name == Role.USER.name)
    {
        Alert.create(I18.f("PAYMENTS_NOT_ALLOWED"));
        return;
    }
    
    if(!pr.productId) return;
    
    APP.showLoader();
    
	var customer = this.account.username;
	var receiver = this.receiverUsername || customer;
	var server = COM(this.chooseServer).selectedItem.data.id || '';
    var payload = [customer, receiver, server].join(':');
    PurchasesManager.purchase(pr.productId, payload);
    
    /*
    if (pr.getPayment() != selectedPayment) {
        selectedPaymentChanged = true;
    }
    selectedPayment = pr.getPayment();
    if (pr.getPayment().options.length == 0) {
        currentActionHandler = pr.getPayment().onAction(receiverUsername, account, null, null, serverId);
    } else {
        showTariffs();
    }
    */
}

FinanceInputTab.prototype.onPurchaseComplete = function(transactionId, productId, receipt)
{
    var resultHandler = function()
    {
        Alert.create(I18.f("PURCHASE_COMPLETE"));
        Events.onFinanceChanged.fire();
        APP.hideLoader();
    }
    var receiver = this.receiverUsername ? this.receiverUsername : this.account.username;
    var server = COM(this.chooseServer).selectedItem.data.id;

    if (arguments.length == 1)
    {
    	PurchasesManager.sendToServerFull(receiver, server, transactionId, resultHandler, APP.hideLoader);
    }
    else
    {
    	PurchasesManager.sendToServer(receiver, server, productId, receipt, resultHandler, APP.hideLoader, transactionId);
    }
    
}

FinanceInputTab.prototype.onPaymentOptionChange = function()
{
    /*
    refreshTarrifs();
    refreshChatButton();
    */
}

FinanceInputTab.getPaymentSystem = function(payment, option)
{
    //return (option.paymentSystem != PaymentSystem.OTHER) ? option.paymentSystem : payment.paymentSystem;
}

FinanceInputTab.prototype.onTariffClick = function(tr)
{
    /*
    var t:Tariff = tr.getTariff();
    selectedTariffRenderer = tr;
    var system:PaymentSystem = getPaymentSystem(selectedPayment, selectedPaymentOption);
    if (system == PaymentSystem.IFREE) {
        page.selectedChild = smsCodePanel;
        smsCountry.textFlow = I18.SMS_COUNTRY.flow(selectedPaymentOption.name);
        smsDescription.textFlow = I18.SMS_DESCRIPTION.flow(t.serviceNumber, t.getSmsText());
        smsRemarkTxt.text = !Strings.isEmpty(selectedPaymentOption.smsRemark1)
                ? selectedPaymentOption.smsRemark1
                : I18.SMS_PAYMENT_REMARK.f();
        smsRemarkLink.text = I18.SMS_PAYMENT_REMARK_URL_TEXT.f();
        smsRemarkLink.url = I18.SMS_PAYMENT_REMARK_URL_ADDRESS.f(t.serviceNumber, t.country);
        UI.show(true, [smsRemarkBox, enterCodeBox]);
        smsRemarkTxt2.textFlow = I18.htmlFlow(selectedPaymentOption.smsRemark2);
        UI.show(!Strings.isEmpty(selectedPaymentOption.smsRemark2), [smsRemarkBox2]);
    } else if (system == PaymentSystem.EPAY_MOBILE || system == PaymentSystem.XSOLLA_2PAY_MOBILE) {
        page.selectedChild = epayMobileBox;
        epayRemark.text = t.getSmsText() != null ? t.getSmsText() : selectedPaymentOption.smsRemark1;
        phoneMin = selectedPaymentOption.phoneMinLength;
        phoneMax = selectedPaymentOption.phoneMaxLength;
    } else {
        currentActionHandler = selectedPayment.onAction(receiverUsername, account, selectedPaymentOption, t, serverId);
    }
    */
}

FinanceInputTab.prototype.onEnterCode = function()
{
    /*
    enterCode.enabled = false;
    PortalService.account.enterSMSCode(receiverUsername, smsCode.text, isFromServer() ? serverId : null).handle(
            function (result:FPaymentResult):void {
                enterCode.enabled = true;
                account = result.account;
                page.selectedChild = payment;
                GameAlert.doShow(I18.SUCCESSULL_SMS_PAY.f(result.getAmountText())).onClose = function ():void {
                    if (!isFromServer()) {
                        GameAlert.doShow(getPaymentOnPortalWarningString()).width = 550;
                    }
                };
            },
            function (e:Error):Boolean {
                enterCode.enabled = true;
                if (e is InvalidSMSCodeException) {
                    GameAlert.doShow(I18.SMS_CODE_ACTIVATION_ERROR.f());
                    return true;
                }
                return false;
            }
    );
    */
}

FinanceInputTab.getPaymentOnPortalWarningString = function()
{
    return I18.f("PAYMENT_ON_PORTAL_WARNING");
}

FinanceInputTab.prototype.onEnterPhone = function()
{
    /*
    enterPhone.enabled = false;
    if (selectedPaymentOption.customUrl != null) {
        currentActionHandler = selectedPayment.onAction(receiverUsername, account, selectedPaymentOption, selectedTariffRenderer.getTariff(), serverId, phone.text);
    } else {
        PortalService.account.logPaymentClick(selectedTariffRenderer.getTariff().id);
        PortalService.account.makeEpayMobileInvoice(receiverUsername, selectedPaymentOption.id, selectedTariffRenderer.getTariff().id, serverId, phone.text).handle(
                function (result:String):void {
                    enterPhone.enabled = true;
                    GameAlert.doShow(result == null ? I18.EPAY_MOBILE_INVOICE_OK.f() : I18.EPAY_MOBILE_INVOICE_ERROR.f(result));
                });
    }
    */
}

FinanceInputTab.prototype.getSelectedPaymentOption = function()
{
    //return PaymentOption(paymentOptions.selectedItem);
}

FinanceInputTab.prototype.onClose = function()
{
    /*
    if (currentActionHandler != null) {
        currentActionHandler.onClose();
        currentActionHandler = null;
    }
    */
}

FinanceInputTab.prototype.hideDestinationCombo = function()
{
    UI.show(false, [this.destinationCombo]);
}

FinanceInputTab.showFinanceHelp = function()
{
    /*
    var url:String = I18.FINANCE_HELP_URL.f();
    navigateToURL(new URLRequest(url.indexOf("/") == 0 ? PortalService.getUrl(url) : url), "_blank");
    */
}

FinanceInputTab.forwardEnabled = function(phone, min, max)
{
    //return phone.length >= min && phone.length <= max;
}