function FCast()
{
	this.spell = ""; // String
	this.level = 0; // int
	this.startTick = 0; // Number
	this.tillTick = 0; // Number
	this.caster = ""; // String
	this.cause = new CastCause(); // CastCause
	this.artifact = ""; // String
	this.fromOwner = false; // Boolean
	this.baseBuffCount = 0; // int
	
	this.buffs = []; // ArrayCollection//List<FBuff>
	
	this.__defineGetter__('isVisible', this.getIsVisible);
	this.__defineGetter__('actions', this.getActions);
}

FCast.prototype.__CLS__ = "game.service.heroes.FCast";

FCast.prototype.spell = null; // String
FCast.prototype.level = null; // int
FCast.prototype.startTick = null; // Number
FCast.prototype.tillTick = null; // Number
FCast.prototype.caster = null; // String
FCast.prototype.cause = null; // CastCause
FCast.prototype.artifact = null; // String
FCast.prototype.fromOwner = null; // Boolean
FCast.prototype.baseBuffCount = 0; // int

FCast.prototype.buffs = null; // ArrayCollection//List<FBuff>

FCast.prototype.prepareForRequest = function()
{
	var ret = {
		__CLS__: this.__CLS__,
		spell: this.spell,
		level: this.level,
		startTick: this.startTick,
		tillTick: this.tillTick,
		caster: this.caster,
		cause: this.cause,
		artifact: this.artifact,
		fromOwner: this.fromOwner
	};
	
	var buffs = [];
	for(var i=0; i<this.buffs.length; i++)
	{
		buffs[i] = this.buffs[i].prepareForRequest();
	}
	ret.buffs = buffs;
	
	return ret;
}
	
FCast.prototype.isStarted = function(tick)
{
    return this.startTick <= tick;
}

FCast.prototype.isMilitary = function(attacker, ally)
{
    for (var i = 0, len = this.buffs.length; i < len; i++) 
    {
    	var b = this.buffs[i];
        if (b.buffType.isMilitary(b, attacker, ally)) 
        {
            return true;
        }
    }
    return false;
}

FCast.prototype.isMagicDisable = function()
{
    return this.hasBuff(BuffType.MAGIC_DISABLE);
}

FCast.prototype.isGlobal = function()
{
    return this.hasBuff(BuffType.SCIENCE_VALUE);
}

FCast.prototype.hasBuff = function(type)
{
    var tname = type.name ? type.name : type;
    
    for (var i = 0, len = this.buffs.length; i < len; i++) 
    {
        var b = this.buffs[i];
        if (b.buffType.name == tname) 
        {
            return true;
        }
    }
    return false;
}

FCast.prototype.getIsVisible = function()
{
    return this.cause.name == CastCause.SPELL.name || this.cause.name == CastCause.CASTLE_STANDARD.name;
}

FCast.prototype.getActions = function()
{
    switch (this.cause.name) 
    {
        case CastCause.SPELL.name:
            var spells = FConfig.getSpell(this.spell, this.level);
            return spells ? spells.allActions : [];
        default:
            var passives = FConfig.getPassive(this.spell, this.level);
            return passives ? passives.dtoActions : [];
    }
}

FCast.checkBuffTarget = function(buff, buffTarget)
{
	for (var i = 0, len = buff.targets.length; i < len; i++)
	{
		if (buff.targets[i].name == buffTarget.name) return true;
	}
	return false;
}

FCast.workingBuffs = function(casts, buffTarget, terrain)
{
    var result = [];
    if (casts) 
    {
        var magicDisabled = FCast.containsMagicDisabled(casts);
        for (var i = 0, len = casts.length; i < len; i++) 
        {
        	var cast = casts[i];
            if (!magicDisabled || !cast.isMagicDisableable() || cast.isMagicDisable()) 
            {
		        for (var j = 0, len2 = cast.buffs.length; j < len2; j++) 
		        {
		        	var buff = cast.buffs[j];
                    if (
                    	(!buffTarget || FCast.checkBuffTarget(buff, buffTarget)) 
                    	&& 
                    	(!terrain || !buff.terrain || buff.terrain.name == terrain.name)
                    ) 
					{
                        result.push(buff);
                    }
                }
            }
        }
    }
    return result;
}

FCast.containsMagicDisabled = function(casts)
{
    for (var i = 0, len = casts.length; i < len; i++) 
    {
        if (casts[i].isMagicDisable()) 
        {
            return true;
        }
    }
    return false;
}

FCast.prototype.isMagicDisableable = function()
{
    return CastCause.isMagicDisableable(this.cause);
}

FCast.prototype.toString = function()
{
    return "FCast{spell=" + String(this.spell) + ",level=" + String(this.level) + ",tillTick=" + String(this.tillTick) + ",caster=" + String(this.caster) + ",cause=" + String(this.cause) + ",artifact=" + String(this.artifact) + ",fromOwner=" + String(this.fromOwner) + "}";
}

