function FHero()
{

	this.__defineGetter__('hireCost', function(){
	    return this.heroLevel.cost;
	});
	
	this.__defineGetter__('nextLevelExp', function(){
	    return this.heroLevel.exp;
	});
	
	this.__defineGetter__('heroLevel', function(){
	    return FConfig.heroLevels[this.level - 1];
	});
	
	this.__defineGetter__('hasSkillsToReset', function(){
	    return this.hasCustomBranchSelected() || this.getDistributedSkillPoints() > 0;
	});
	
}

FHero.BASE_BRANCH_DEFAULT_LEVEL = 1;
FHero.DEFAULT_BRANCHES_COUNT = 2;

FHero.prototype.id = null; // Number
FHero.prototype.name = ''; // String
FHero.prototype.owner = ''; // String
FHero.prototype.heroClass = ''; // String
FHero.prototype.level = 0; // int
FHero.prototype.currentLevelExp = 0; // Number
FHero.prototype.maxLevelReached = false; // Boolean
FHero.prototype.face = ''; // String
FHero.prototype.male = false; // Boolean
FHero.prototype.body = null; // FArtifactList
FHero.prototype.bag = null; // FArtifactList
FHero.prototype.upsetByFight = false; // Boolean
FHero.prototype.upsetTillTick = 0; // Number
FHero.prototype.upsetByTitle = false; // Boolean
FHero.prototype.branches = null; // ArrayCollection // List<SkillBranch>
FHero.prototype.skillPointsLeft = 0; // int
FHero.prototype.mana = 0; // Number
FHero.prototype.maxMana = 0; // int
FHero.prototype.manaRegenPerHour = 0; // int
FHero.prototype.bagSize = 0; // int
FHero.prototype.needsAttention = false; // Boolean
FHero.prototype.cooldowns = null; // FMap  // name 2 expirationTick
FHero.prototype.casts = null; // ArrayCollection;//List<FCast> casts
FHero.prototype.buffs = null; // ArrayCollection//List<FBuff>

FHero.prototype.prisonOwnerName = ''; // String
FHero.prototype.prisonEstateName = ''; // String
FHero.prototype.prisonXYZ = null; // XYZ
FHero.prototype.prisonCostBonus = 0; // Number
FHero.prototype.manageAccess = false; // Boolean

FHero.prototype.__construct = function()
{
	this.name = htmlEscape(this.name);
}

FHero.prototype.prepareForRequest = function()
{
	//var ret = prepareRequestParams([this])[0];
	var ret = cloneRequestParam(this);
	
	ret.name = htmlUnescape(ret.name);
	ret.bag = this.bag.prepareForRequest();
	ret.body = this.body.prepareForRequest();
	
	for(var i=0; i<ret.buffs.length; i++)
	{
		ret.buffs[i] = this.buffs[i].prepareForRequest();
	}
	ret.buffs = ret.buffs;
	
	for(var i=0; i<ret.casts.length; i++)
	{
		ret.casts[i] = this.casts[i].prepareForRequest();
	}
	ret.casts = ret.casts;
	
	var external = [];
	var e;
    for (i=0; i<this.cooldowns.raw.length; i++)
    {
        e = this.cooldowns.raw[i];
        external.push({__CLS__: "game.util.FMap$Entry", key: e.key, value: e.value});
    }
    
    cd = {__CLS__: 'game.util.FMap'};
    cd.data = external;
    ret.cooldowns = cd;
	
	return ret;
}

FHero.prototype.isUpset = function()
{
    return this.upsetByFight || this.upsetByTitle;
}

FHero.prototype.getBranchByName = function(branchName)
{
    return Arrays.firstWith(this.branches, function(b) { return b.name == branchName; });
}

FHero.prototype.getBranch = function(index)
{
    return index < this.branches.length ? this.branches[index] : SkillBranch.DUMMY_BRANCH;
}

FHero.prototype.getSpells = function()
{
    var res = [];
    var cooldownTick;
    // from skills
    
    for (var i = 0; i < this.branches.length; i++) 
    {
        var branch = this.branches[i];
        var skills = FConfig.branches.getValueOrNull(branch.name);
        var index = 0;
        for (var j = 0; j < skills.length; j++) 
        {
            var skill = skills[j];
            var level = branch.levels[index];
            if (level > 0 && skill.hasSpell()) 
            {
                cooldownTick = this.cooldowns.containsKey(skill.name) ? this.cooldowns.getValue(skill.name) : 0;
                res.push(new FSpell(skill.name, level, null, false, cooldownTick));
            }
            index++;
        }
    }
    
    // from artifacts in bag (scrolls)
    for (var i = 0; i < this.bag.artifacts.length; i++) 
    {
        var art = this.bag.artifacts[i]
        if (art && art.declaration.spell) 
        {
            level = art.declaration.level;
            if (level > 0 && art.declaration.givesSpell()) 
            {
                res.push(new FSpell(art.declaration.spell, level, art.declaration));
            }
        }
    }
    return res;
}

FHero.prototype.tick = function(delta)
{
    if (this.mana < this.maxMana) 
    {
        this.mana += delta * this.manaRegenPerHour / Tick.TICKS_PER_HOUR;
        if (this.mana > this.maxMana) 
        {
            this.mana = this.maxMana + 0;
        }
    }
    if (this.upsetByFight) 
    {
        this.upsetByFight = (this.upsetTillTick >= Tick.now());
    }
}

// working buffs only from casted on this hero/passives
FHero.prototype.getWorkingBuffs = function(terrain)
{
    return FCast.workingBuffs(this.casts, BuffTarget.SELF, terrain);
}

FHero.prototype.hasWorkingBuff = function(t, terrain, time)
{
    return FBuff.hasBuff(this.getWorkingBuffs(terrain), t, null, time);
}

FHero.prototype.hasCustomBranchSelected = function()
{
    return this.branches.length > FHero.DEFAULT_BRANCHES_COUNT;
}

FHero.prototype.getDistributedSkillPoints = function()
{
    var baseBranch = this.branches[0];
    var points = baseBranch.getDistributedSkillPoints(FHero.BASE_BRANCH_DEFAULT_LEVEL);

    for (var i = 1; i < this.branches.length; i++) 
    {
        var branch = this.branches[i];
        points += branch.getDistributedSkillPoints(0);
    }
    return points;
}

FHero.prototype.compareTo = function(other)
{
    return this.id - other.id;
}

FHero.prototype.getSpecBranches = function()
{
    var specBranches = [];
    for (var i = 1; i < this.branches.length; i++) 
    {
        specBranches.push(this.branches[i]);
    }
    return specBranches;
}

FHero.prototype.isMaxManaReached = function()
{
    return this.mana >= this.maxMana;
}

FHero.getRequiredLevelForBranch = function(branchIndex)
{
    var n = 1;
    for (var lvl in FConfig.heroLevels) 
    {
        if (FConfig.heroLevels[lvl].branchesCount >= branchIndex) 
        {
            return n;
        }
        n++;
    }
    throw new Error("Cannot find level for branchIndex " + branchIndex);
}

