function HintArrow(name, direction, mediator)
{
	this.name = name;
	this.direction = direction;
    this.mediator = mediator;
    
	this.arrow = null;
	this.understoodHints = [];
	
	this.dialogRoot = null;
	this.component = null;
	this.parent = null;
	this.hint = null;
	
	this.updateListenerId = 0;
	this.scrollerParents = [];
}

HintArrow.ALIGN =
{
	left: 0,
	center: 1,
	right: 2
}

HintArrow.VALIGN =
{
	top: 0,
	middle: 1,
	bottom: 2
}

HintArrow.TOP = {source: CommonAssets.HINT_ARROW_TOP, align: HintArrow.ALIGN.center, valign: HintArrow.VALIGN.top, cssClass: "hintArrowTop"};
HintArrow.BOTTOM = {source: CommonAssets.HINT_ARROW_BOTTOM, align: HintArrow.ALIGN.center, valign: HintArrow.VALIGN.bottom, cssClass: "hintArrowBottom"};
HintArrow.RIGHT = {source: CommonAssets.HINT_ARROW_RIGHT, align: HintArrow.ALIGN.right, valign: HintArrow.VALIGN.middle, cssClass: "hintArrowRight"};
HintArrow.LEFT = {source: CommonAssets.HINT_ARROW_LEFT, align: HintArrow.ALIGN.left, valign: HintArrow.VALIGN.middle, cssClass: "hintArrowLeft"};

HintArrow.MEDIATOR_CITY = "city";
HintArrow.MEDIATOR_KINGDOM = "kingdom";

HintArrow.OPEN_ADVISOR = new HintArrow("OPEN_ADVISOR", HintArrow.TOP, HintArrow.MEDIATOR_CITY);
HintArrow.EXPAND_QUEST = new HintArrow("EXPAND_QUEST", HintArrow.TOP);
HintArrow.GET_REWARD = new HintArrow("GET_REWARD", HintArrow.TOP);
HintArrow.CLICK_ON_CELL = new HintArrow("CLICK_ON_CELL", HintArrow.TOP, HintArrow.TOPMEDIATOR_CITY);
HintArrow.CLICK_BUILDING_TAB = new HintArrow("CLICK_BUILDING_TAB", HintArrow.BOTTOM);
HintArrow.SELECT_BUILDING = new HintArrow("SELECT_BUILDING", HintArrow.RIGHT);
HintArrow.PRESS_CONSTRUCT = new HintArrow("PRESS_CONSTRUCT", HintArrow.BOTTOM);
HintArrow.OPEN_SCIENCE = new HintArrow("OPEN_SCIENCE", HintArrow.TOP);
HintArrow.SELECT_SCIENCE = new HintArrow("SELECT_SCIENCE", HintArrow.LEFT);
HintArrow.OPEN_KINGDOM = new HintArrow("OPEN_KINGDOM", HintArrow.TOP, HintArrow.MEDIATOR_CITY);
HintArrow.OPEN_CITY = new HintArrow("OPEN_CITY", HintArrow.TOP, HintArrow.MEDIATOR_KINGDOM);
HintArrow.OPEN_KINGDOM_MENU = new HintArrow("OPEN_KINGDOM_MENU", HintArrow.TOP, HintArrow.MEDIATOR_KINGDOM);
HintArrow.CLICK_KINGDOM_MENU_MAIL = new HintArrow("CLICK_KINGDOM_MENU_MAIL", HintArrow.TOP);
HintArrow.TYPE_LETTER_TEXT = new HintArrow("TYPE_LETTER_TEXT", HintArrow.TOP);
HintArrow.CLICK_SEND_BUTTON = new HintArrow("CLICK_SEND_BUTTON", HintArrow.LEFT);
HintArrow.NAVIGATE_KINGDOM = new HintArrow("NAVIGATE_KINGDOM", HintArrow.TOP, HintArrow.MEDIATOR_KINGDOM);
HintArrow.OPEN_MAILBOX = new HintArrow("OPEN_MAILBOX", HintArrow.TOP);
HintArrow.SELECT_SOME_MESSAGE = new HintArrow("SELECT_SOME_MESSAGE", HintArrow.LEFT);
HintArrow.CLICK_ARCHIVE = new HintArrow("CLICK_ARCHIVE", HintArrow.TOP);
HintArrow.OPEN_CITYHALL_MENU = new HintArrow("OPEN_CITYHALL_MENU", HintArrow.TOP, HintArrow.MEDIATOR_CITY);
HintArrow.CLICK_UPGRADE_MENU = new HintArrow("CLICK_UPGRADE_MENU", HintArrow.BOTTOM);
/////
HintArrow.CLICK_UPGRADE_BUTTON = new HintArrow("CLICK_UPGRADE_BUTTON", HintArrow.LEFT);

HintArrow.enabled = true;

HintArrow.SYSTEM_MANAGER = APP;

HintArrow.initArrows = function(c)
{
    return [HintArrow.OPEN_ADVISOR,
			HintArrow.EXPAND_QUEST,
			HintArrow.GET_REWARD,
			HintArrow.CLICK_ON_CELL,
			HintArrow.CLICK_BUILDING_TAB,
			HintArrow.SELECT_BUILDING,
			HintArrow.PRESS_CONSTRUCT,
			HintArrow.OPEN_SCIENCE,
			HintArrow.SELECT_SCIENCE,
			HintArrow.OPEN_KINGDOM,
			HintArrow.OPEN_CITY,
			HintArrow.OPEN_KINGDOM_MENU,
			HintArrow.CLICK_KINGDOM_MENU_MAIL,
			HintArrow.TYPE_LETTER_TEXT,
			HintArrow.CLICK_SEND_BUTTON,
			HintArrow.NAVIGATE_KINGDOM,
			HintArrow.OPEN_MAILBOX,
			HintArrow.SELECT_SOME_MESSAGE,
			HintArrow.CLICK_ARCHIVE,
			HintArrow.OPEN_CITYHALL_MENU,
			HintArrow.CLICK_UPGRADE_MENU,
			HintArrow.CLICK_UPGRADE_BUTTON];
}

HintArrow.ALL = HintArrow.initArrows(HintArrow);

HintArrow.anyVisible = function()
{
    for(var i=0; i<HintArrow.ALL.length; i++)
    {
    	if (HintArrow.ALL[i].isShowing())
    	{
            //console.log("NOW SHOWING: ", HintArrow.ALL[i].name);
            return true;
        }
    }
    return false;
}

HintArrow.hideAll = function()
{
    for(var i=0; i<HintArrow.ALL.length; i++)
    {
    	HintArrow.ALL[i].hide()
    }
}

HintArrow.clearUnderstood = function()
{
	if(HintArrow.anyVisible()) return;
	
	for(var i=0; i<HintArrow.ALL.length; i++)
    {
    	if (HintArrow.ALL[i] != HintArrow.OPEN_SCIENCE &&
    		HintArrow.ALL[i] != HintArrow.OPEN_MAILBOX /*&&
    		HintArrow.ALL[i] != HintArrow.CLICK_ON_CELL*/)
    	{
    		HintArrow.ALL[i].understoodHints = [];
    	}
    }
}

HintArrow.prototype.show = function(component, hint, container, directPosition, correctPos)
{
    //console.log(hint.type.name, !HintArrow.enabled, HintArrow.anyVisible(), this.isUnderstood(hint));
    
    if (!HintArrow.enabled || HintArrow.anyVisible() || this.isUnderstood(hint) /*|| (dialog is Dialog && Dialog(dialog).dialog == null)*/)
    {
        return;
    }
   
	if(component && component.DOM) component = component.DOM;

    HintArrow.hideAll();

    this.component = component;
    this.hint = hint;

    this.arrow = document.createElement("img");
    this.arrow.style.position = "absolute";
    this.arrow.src = STATIC_INTERFACE_URL + this.direction.source;
    this.arrow.className = this.direction.cssClass;
    this.arrow.style.display = "none";
    this.arrow.style.left = "-1024px";
    
    this.alignPosition = $.proxy(function()
    {
    	setTimeout($.proxy(function()
    	{
    		if(!this.arrow) return;
    		
    		var cw, ch, left, top;
    	
	    	if(directPosition)
		    {
		    	left = directPosition.left;
			    top = directPosition.top;
			    cw = directPosition.width;
			    ch = directPosition.height;
		    }
		    else
		    {
			    left = getObjectX(component);
			    top = getObjectY(component);
			    cw = component.offsetWidth;
			    ch = component.offsetHeight;
		    }
	    	
	    	if(this.direction.align == HintArrow.ALIGN.left)
		    {
		    	left -= this.arrow.offsetWidth;
		    }
		    if(this.direction.align == HintArrow.ALIGN.right)
		    {
		    	left += cw;
		    }
		    if(this.direction.align == HintArrow.ALIGN.center)
		    {
		    	left += cw/2 - this.arrow.offsetWidth/2;
		    }
		    
		    if(this.direction.valign == HintArrow.VALIGN.top)
		    {
		    	top -= this.arrow.offsetHeight;
		    }
		    if(this.direction.valign == HintArrow.VALIGN.middle)
		    {
		    	top += ch/2 - this.arrow.offsetHeight/2;
		    }
		    if(this.direction.valign == HintArrow.VALIGN.bottom)
		    {
		    	top += ch;
		    }
		    
		    if(correctPos)
		    {
		    	left += correctPos.x;
		    	top += correctPos.y;
		    }
		    
		    this.arrow.style.left = left+"px";
	    	this.arrow.style.top = top+"px";
    	}, this), 10);
    }, this);
    
    //$(window).on("fitlayout", this.alignPosition);
    
    setTimeout($.proxy(function()
    {
    	if(!this.arrow) return;
    	
    	this.arrow.style.display = "";
    	
	    if(container)
	    {
	    	$(container).append(this.arrow);
	    }
	    else
	    {
	    	$("#popup").append(this.arrow);
	    }
	    this.arrow.style.zIndex = 10000;
    
	    this.alignPosition();
	    
	    clearInterval(this.alignTimer);
	    this.alignTimer = setInterval(this.alignPosition, 500);
	    
    }, this), 500);
    
    var d = COM(container);
    if(d && d.hintArrows) d.hintArrows.push(this);
}

HintArrow.prototype.understood = function(quest)
{
    if (this.hint && (!quest || quest == this.hint.quest) && !this.isUnderstood(this.hint))
    {
        this.understoodHints.push(this.hint);
        this.hide();
        
        APP.updateHints();
    }
}

HintArrow.prototype.isUnderstood = function(hint)
{
    if (!hint)
    {
        return false;
    }
    for(var i=0; i<this.understoodHints.length; i++)
    {
        if (this.understoodHints[i].equals(hint))
        {
            return true;
        }
    }
    return false;
}

HintArrow.prototype.hide = function()
{
	if(this.alignPosition)
	{
		//$(window).off("fitlayout", this.alignPosition);
		this.alignPosition = null;
	}
	
	clearInterval(this.alignTimer);
	
    $(this.arrow).remove();
    this.arrow = null;
    this.dialogRoot = null;
    this.component = null;
    this.parent = null;
}

HintArrow.prototype.isShowing = function()
{
    return Boolean(this.arrow);
}

HintArrow.prototype.canShow = function(hint)
{
    return !this.isShowing() && !this.isUnderstood(hint);
}
