function KingdomFieldInfo()
{
	callParentConstructor(KingdomFieldInfo.prototype, this);
	
	this.terrain = null;
	this.location = null;
	this.field = null;
}

extendClass(KingdomFieldInfo.prototype, Dialog.prototype);

KingdomFieldInfo.createFromField = function(f)
{
    return KingdomFieldInfo.create(f.terrain, f.getLocationXYZ(), f);
}

KingdomFieldInfo.create = function(terrain, location, field)
{
    var kingdomFieldInfo = COM.create("KingdomFieldInfo");
    kingdomFieldInfo.location = location;
    kingdomFieldInfo.terrain = terrain;
    kingdomFieldInfo.field = field;
    kingdomFieldInfo.init();
    return kingdomFieldInfo;
}

KingdomFieldInfo.prototype.__construct = function()
{
}

KingdomFieldInfo.prototype.init = function(container)
{
	this.setCaption(I18.f("INFORMATION"));
	this.terrainName.innerHTML = this.terrain.displayName + " " + this.location;
    
    COM(this.view).terrain = this.terrain;
    COM(this.view).init();
    
    if(this.field)
    {
    	UI.show(!this.field.hasEstate() && this.field.exploreTick > 0, [this.explored]);
		this.explored.innerHTML = I18.f("EXPLORED_DATE", Tick.toDays(Tick.now() - this.field.exploreTick));
		
		UI.show(this.field.hasEstate() && this.field.estate.hasOwner() && this.field.estate.owner.isOnVacation(), [this.vacation]);
    }
    
    var f = this.field;
    
    var showGifts = f.hasEstate() && (f.estate.gifts.length > 0
            || f.estate.owner.tournamentVictoriesCount > 0
            || f.estate.owner.tournamentGrottoVictoriesCount > 0
            || f.estate.owner.frags > 0);
            
    UI.show(showGifts, [this.giftView, this.giftHeader], $.proxy(function()
    {
        $(this.giftView).empty();
        
        var gifts = FGift.buildGifts(
                f.estate.owner.tournamentVictoriesCount,
                f.estate.owner.tournamentGrottoVictoriesCount,
                f.estate.owner.frags,
                f.estate.gifts
        );
        
        for(var i=0; i<gifts.length; i++)
        {
        	var view = COM.create("GiftView");
            view.update(gifts[i]);
            this.giftView.appendChild(view.DOM);
        }
        
    }, this));
    
    UI.show(f.hasEstate(), [this.estateBox], $.proxy(function()
    {
        var estateName;
        if (f.hasCity())
        {
            estateName = I18.getKingdomCityTip(f.estate.owner.race);
            if (f.hasFakeRuin())
            {
                estateName = I18.f("RUIN_NAME");
            }
        }
        else if (f.hasOuterEstate() && f.estate.hasOwner())
        {
            var outerEstate = f.estate;
            estateName = I18.getKingdomOuterEstateTip(outerEstate.owner.race, outerEstate.type, outerEstate.resources);
        }
        else
        {
            estateName = f.estate.getName();
        }
        COM(this.estateBox).update(null, estateName, f.estate.type, f.estate.info);
        COM(this.estateBox).iconImage.style.display = "none";
    }, this));
    
    if(f.hasEstate() && f.estate.hasOwner() && f.estate.owner.isBetrayer())
    {
        this.betrayer.style.display = "";
        this.betrayer.innerHTML = I18.f("BETRAYER_TILL", Tick.toTimeFuture(f.estate.owner.betrayerTill));
    }
    else this.betrayer.style.display = "none";
    
    this.scroll = new iScroll(this.windowWrapper, {scrollbarColor: '#784f0b'});
	setTimeout($.proxy(this.scroll.refresh, this.scroll), 500);
}