function KingdomMediator()
{
	callParentConstructor(KingdomMediator.prototype, this);
	
	this.__defineGetter__("kv", this.getKv);
	
	
	this.kingdom = null;
    this.ticker = new Ticker(1000, $.proxy(this.tick, this));
    this.xy = null;
    this.z = -1;

    this.updatableDialog = null;
}

extendClass(KingdomMediator.prototype, BaseMediator.prototype);

KingdomMediator.prototype.__construct = function()
{
	superMethod(KingdomMediator, "__construct", this)(COM.create("KingdomView"));
	
	var self = this;
    
    this.kv.onCityAttack = this.missionHandlerFactory("MissionAttackDialog");
    this.kv.onCityAssist = this.missionHandlerFactory("MissionAssistanceDialog");
    this.kv.onMoveArmy = this.missionHandlerFactory("MissionMoveArmyDialog");
    this.kv.onMoveResources = this.missionHandlerFactory("MissionMoveResourcesDialog");
    this.kv.onSettler = this.missionHandlerFactory("MissionSettlerDialog");
    this.kv.onMakeCamp = this.missionHandlerFactory("MissionCampDialog");
    this.kv.onExpedition = this.missionHandlerFactory("MissionExpeditionDialog");
    this.kv.onInitiateEspionage = this.missionHandlerFactory("EspionageDialog");
    this.kv.onObserve = this.missionHandlerFactory("ObservationDialog");

    this.kv.onSendMessage = $.proxy(MailBoxView.i().sendMessage, MailBoxView.i());
    this.kv.onSendMonsterMessage = $.proxy(MailBoxView.i().sendMessageToMonsters, MailBoxView.i());

    this.kv.onCityDelete = function(f)
    {
        if (f.hasCity())
        {
            Alert.create(I18.f("KINGDOM_CITY_DELETE_CONFIRM"), "", function()
            {
                Service.kingdom.callMethod("deleteCity", [f.estate.id],
                	$.proxy(function(k)
                	{
                		self.update(k, true);
                	}, self),
                    function(e)
                    {
                        if (e.__className__ == "CityDeleteException")
                        {
                            Alert.create(e.getMessage());
                            return true;
                        }
                        return false;
                    });
            });
        }
    };
    
    this.kv.onLeaveOuterEstate = $.proxy(function(f)
    {
        if (f.hasOuterEstate())
        {
            Alert.create(I18.f("KINGDOM_LEAVE_ESTATE_CONFIRM"), "", $.proxy(function()
            {
                if (f.hasCatapult())
                {
                    Service.kingdom.callMethod("deleteCatapult", [f.estate.id],
                        $.proxy(function(k) { self.update(k, true); }, self),
                        function(e)
                        {
                            Alert.create(e.getMessage());
                            return true;
                        });
                }
                else
                {
                    this.kv.globalForceUpdate = true;

                    Service.kingdom.callMethod("leaveOuterEstate", [f.estate.id],
                        $.proxy(function(k) { self.update(k, true); }, self),
                        function(e)
                        {
                            Alert.create(e.getMessage());
                            return true;
                        });
                }
            }, this));
        }
    }, this);

    this.kv.onNavigateKingdom = function(newX1, newY1)
    {
        var newX = newX1 - 1;
        var newY = newY1 - 1;
        if (newX >= 0 && newX < FConfig.EMPIRE_SIZE.x && newY >= 0 && newY < FConfig.EMPIRE_SIZE.y)
        {
            self.xy = new XY(newX, newY);
            self.z = -1;
            Service.kingdom.callMethod("get", [newX, newY], $.proxy(function(k)
            {
            	self.update(k);
            }, self));
            APP.showLoader();
        }
    };

    this.kv.onShowInfo = function(f)
    {
        KingdomFieldInfo.createFromField(f).popup(this.kv);
    };

    this.kv.onForgiveBetrayer = function()
    {
        self.updatableDialog = ForgiveBetrayerDialog.popup(self.kv, function()
        {
            var requireUpdate = self.updatableDialog.requireUpdate;
            self.updatableDialog = null;
            if (requireUpdate)
            {
                Service.kingdom.callMethod("get", [self.kingdom.location.x, self.kingdom.location.y]);
            }
        });
    };
    
    this.kv.onMakeGift = function(f)
    {
        COM.create("MakeGiftDialog").popup(self.kv, self.kingdom.location, f.estate.owner.username);
    };
    
    this.kv.onShareOuterEstate = function(f)
    {
        COM.create("ShareOuterEstateDialog").popup(self.kv, f.getLocationXYZ());
    };
    
    this.kv.onCastSpell = function(f)
    {
        Service.city.callMethod("getById", [self.kv.citiesCombo.selected.id], function(from)
        {
            if (!from.garrisonHero)
            {
                Alert.create(I18.f("CAST_ERROR_NO_GARRISON_HERO"));
                return;
            }
            var target = CastTarget.fromField(f);
            var dialog = HeroDialog.popupForCast(from, target);
            var ticker = new EstateTicker($.proxy(dialog.update, dialog), $.proxy(dialog.tick, dialog));
            dialog.onClose = $.proxy(ticker.stop, ticker);
            ticker.start(from.id, from);
        });
    };

    this.kv.onMoveCity = function(f)
    {
        var w = COM.create("MoveCityDialog");
        w.popup(self.kv, self.kv.citiesCombo.selected.estateInfo, f.getLocationXYZ());
    };
}

KingdomMediator.prototype.fitLayout = function(container)
{
	this.kv.fitLayout(container);
}

KingdomMediator.prototype.show = function(xyz)
{
    this.xy = xyz;
    this.z = !xyz ? -1 : xyz.z;
    this.showKingdom();
}

KingdomMediator.prototype.showKingdom = function()
{
    if (this.xy)
    {
        this.viewComponent.layersWrapper.style.display = "none";
    }
    else if (this.kingdom)
    {
        this.xy = this.kingdom.location;
    }
    else
    {
        this.viewComponent.layersWrapper.style.display = "none";
        this.xy = new XY();
    }
    
    Service.kingdom.callMethod("get", [this.xy.x, this.xy.y], $.proxy(function(k)
    {
        this.ticker.pause();
        Events.onFKingdomUpdated.add(this, this.update);
		
        this.update(k);
        /*
        if (this.z >= 0)
        {
            this.kv.dragToZ(this.z);
        }
        */

        this.viewComponent.layersWrapper.style.display = "block";
        
    }, this),
    
    $.proxy(function(e)
    {
    	//console.log(this.kingdom);
    	
    	this.xy = this.kingdom.location;
    	this.update(this.kingdom);
    	this.viewComponent.layersWrapper.style.display = "block";
    }, this)
    
    );
    
    APP.showLoader();
}

KingdomMediator.prototype.onHide = function()
{
    Events.onFKingdomUpdated.remove(this.update);
    this.ticker.stop();
}

KingdomMediator.prototype.onConnectionGain = function()
{
    if (!this.kingdom)
    {
        this.showKingdom();
    }
    else
    {
        this.ticker.pause();
        this.ticker.start();
    }
}

KingdomMediator.prototype.getKv = function()
{
    return this.viewComponent;
}


KingdomMediator.prototype.update = function(k, noCenterMap)
{
    if (!k.location.equals(this.xy))
    {
        return;
    }
    
    this.kingdom = k;
    this.kv.update(k, noCenterMap);
    this.ticker.pause();
    Tick.updateTick(k.responseTick);
    this.ticker.start();
    
    if (this.kingdom.links.length == 0 && !FUser.pwnedDialogShown)
    {
        Service.user.callMethod("getCapital");
    }
    
    APP.hideLoader();
}

KingdomMediator.prototype.tick = function(delta)
{
    FUser.CURRENT_USER.tick(delta);        
    if (this.updatableDialog)
    {
        this.updatableDialog.tick();
    }
    if (Tick.now() > this.kingdom.nextUpdateTick)
    {
        Service.kingdom.callMethod("get", [this.kingdom.location.x, this.kingdom.location.y]);
    }
    
    this.kv.tick();
}

KingdomMediator.prototype.missionHandlerFactory = function(missionDialogClass)
{
    var self = this;
    return function(f)
    {
        if(COM.getData(missionDialogClass))
        {
	        var md = MissionDialogParent.create(missionDialogClass);
	        var t = new PrepareMissionTicker(md, f);
	        t.start(md.getOriginId(self.kv.citiesCombo));
        }
        else
        {
        	Alert.error(missionDialogClass + " is not implemented");
        }
    };
}

KingdomMediator.prototype.stopAllTickers = function()
{
    this.ticker.stop();
}

KingdomMediator.prototype.updateCurrentUserFooter = function()
{
    this.kv.dashboard.update(FUser.CURRENT_USER);
}