function KingdomView()
{
	this._kingdom = null;
	
	this.scroll = null;
	this.viewLevel = 0;
	
	//this.dblClickHandler = $.proxy(this.dblClickHandler, this);
	this.swypeStartHandler = $.proxy(this.swypeStartHandler, this);
	this.swypeMoveHandler = $.proxy(this.swypeMoveHandler, this);
	this.swypeEndHandler = $.proxy(this.swypeEndHandler, this);
	this.swypeNavigateMode = false;
	
	//Constants
	this.tileWidth = 297;
	this.tileHeight = 215;
	
	this.cellWidth = 149;
	this.cellHeight = 108;
	
	this.tilesPadding = {left: -30, top: -30};
	this.cellsPadding = {left: 0, top: 10};
	
	this.miniMapCellSize = 8;
	
	this.fieldMenu = null;
	
	this.hintField = null;
	
	this.fieldsHistory = [];
	for (var i = 0; i < FKingdom.HEIGHT; i++) this.fieldsHistory[i] = [];
	
	this.globalForceUpdate = false;
	
	this.__defineGetter__("estateId", this.getEstateId);
}

KingdomView.prototype.viewLevels = 
[
	{
		showGrid: false,
		showAnimations: true,
		showExtraInfo: false
	},
	
	{
		showGrid: true,
		showAnimations: true,
		showExtraInfo: true
	}
];

KingdomView.prototype.__construct = function()
{
	COM(this.returnButton).icon = STATIC_INTERFACE_URL + "kingdom/locale.png";
	COM(this.searchButton).icon = STATIC_INTERFACE_URL + "kingdom/search.png";
	
	COM.bindExclusiveClick(this.returnButton, $.proxy(this.gotoCity, this));
	COM.bindExclusiveClick(this.searchButton, $.proxy(this.gotoCoords, this));
	
	this.citiesCombo = COM.create('GlobalSelectionCitiesCombo');
	this.citiesCombo.forKingdom = true;
	this.citiesCombo.onChange = $.proxy(this.updateEstateButton, this);
	this.DOM.appendChild(this.citiesCombo.DOM);
	
	COM.bindExclusiveClick(this.locker, $.proxy(this.hidePanels, this));
	COM.bindExclusiveClick(this.changeEstateButton, $.proxy(this.showRightPanel, this));
	
	this.serverTimeLabel = COM.create('ServerTimeLabel');
	this.DOM.appendChild(this.serverTimeLabel.DOM);
	
	COM.bindExclusiveClick(this.navigationButton, $.proxy(function()
	{
		//HintArrow.OPEN_CITY.understood();
		HintArrow.hideAll();
		HintArrow.clearUnderstood();
		
		APP.showCity(this.citiesCombo.selected.id, true);
		
		this.citiesCombo.selected = null;
	}, this));
	
	COM.bindExclusiveClick(this.mailButton, $.proxy(function()
	{
		MailBoxView.i().popup(FUser.CURRENT_USER.username);
	}, this));
	
	this.tiles = document.createElement("canvas");
	this.setupLayer(this.tiles);
	
	this.miniMapView.ctx = this.miniMapView.getContext("2d");
	this.miniMapViewBig.ctx = this.miniMapViewBig.getContext("2d");
	
	this.swypePosition = {obj: null, x: 0, y: 0};
	
	$(this.navigateToLeft).on(MOUSE_DOWN_EVENT, this.swypeStartHandler);
	$(this.navigateToLeft).on(MOUSE_MOVE_EVENT, this.swypeMoveHandler);
	$(this.navigateToLeft).on(MOUSE_UP_EVENT, this.swypeEndHandler);
	this.navigateToLeft.directX = -1;
	this.navigateToLeft.directY = 0;
	
	$(this.navigateToRight).on(MOUSE_DOWN_EVENT, this.swypeStartHandler);
	$(this.navigateToRight).on(MOUSE_MOVE_EVENT, this.swypeMoveHandler);
	$(this.navigateToRight).on(MOUSE_UP_EVENT, this.swypeEndHandler);
	this.navigateToRight.directX = 1;
	this.navigateToRight.directY = 0;
	
	$(this.navigateToTop).on(MOUSE_DOWN_EVENT, this.swypeStartHandler);
	$(this.navigateToTop).on(MOUSE_MOVE_EVENT, this.swypeMoveHandler);
	$(this.navigateToTop).on(MOUSE_UP_EVENT, this.swypeEndHandler);
	this.navigateToTop.directX = 0;
	this.navigateToTop.directY = -1;
	
	$(this.navigateToBottom).on(MOUSE_DOWN_EVENT, this.swypeStartHandler);
	$(this.navigateToBottom).on(MOUSE_MOVE_EVENT, this.swypeMoveHandler);
	$(this.navigateToBottom).on(MOUSE_UP_EVENT, this.swypeEndHandler);
	this.navigateToBottom.directX = 0;
	this.navigateToBottom.directY = 1;
	
	$(this.navigateToLeft).css({opacity: 0});
	$(this.navigateToRight).css({opacity: 0});
	$(this.navigateToTop).css({opacity: 0});
	$(this.navigateToBottom).css({opacity: 0});
	
	$(this.layersWrapper).on(MOUSE_DOWN_EVENT, $.proxy(this.globalSwypeStartHandler, this));
	$(this.layersWrapper).on(MOUSE_UP_EVENT, $.proxy(this.globalSwypeEndHandler, this));
	
	COM.bindExclusiveClick(this.miniMap, $.proxy(this.showMiniMapBig, this));
	COM.bindExclusiveClick(this.miniMapBig, $.proxy(this.showMiniMap, this));
	
	COM(this.searchX).min = 1;
	COM(this.searchX).max = FConfig.EMPIRE_SIZE.x;
	COM(this.searchY).min = 1;
	COM(this.searchY).max = FConfig.EMPIRE_SIZE.y;
	
	COM(this.changeViewLevelButton).icon = STATIC_INTERFACE_URL + "icons/show_grid.png";
	COM.bindExclusiveClick(this.changeViewLevelButton, $.proxy(this.changeViewLevel, this));
	
	COM(this.showInfoButton).icon = STATIC_INTERFACE_URL + "icons/show_info.png";
	COM.bindExclusiveClick(this.showInfoButton, $.proxy(this.showInfo, this));
	
	COM.bindExclusiveClick(this.logoutButton, Service.goToPortal);
	
	COM.bindExclusiveClick(this.showMissions, $.proxy(function()
	{
		MissionListDialog.popup(null, FUser.CURRENT_USER, this.getEstateId());
	}, this));
	
	HintArrow.hideAll();
}

KingdomView.prototype.updateMissionsButton = function()
{
	var b = COM(this.showMissions);
	
	if (FUser.CURRENT_USER.attackWarning)
	{
        b.skin = "red";
    }
    else
    {
        b.skin = "yellow";
    }

    b.caption = FUser.CURRENT_USER.currentMissionsCount + "/" + FUser.CURRENT_USER.missionsLimit;
}

KingdomView.prototype.getEstateId = function()
{
    return !this.citiesCombo.selected ? null : this.citiesCombo.selected.id;
}

KingdomView.prototype.setupLayer = function(layer)
{
	layer.width = (FKingdom.WIDTH+2) * this.cellWidth;
	layer.height = (FKingdom.HEIGHT+2) * this.cellHeight;
	
	layer.style.position = "absolute";
	layer.style.left = "0px";
	layer.style.top = "0px";
	
	layer.ctx = layer.getContext("2d");
	
	this.layersContainer.appendChild(layer);
}

KingdomView.prototype.fitLayout = function(container)
{
	$(this.locker).css({
		width: container.width,
		height: container.height
	});
	
	$("toppanel", this.DOM).css("width", container.width);
	
	$(this.layersWrapper).css({
		width: container.width,
		height: container.height - $("toppanel", this.DOM).height()
	});
	
	$(this.citiesCombo.DOM).css("left", container.width);
	$("> wrapper", this.citiesCombo.DOM).css("height", container.height - $("> serverinfo", this.citiesCombo.DOM).height());
	
	$(this.serverTimeLabel.DOM).css("top", container.height - $("> wrapper", this.serverTimeLabel.DOM).height() + 3);
	$(this.serverTimeLabel.DOM).css("left", (container.width - $("> wrapper", this.serverTimeLabel.DOM).width())/2);
	
	$(this.navigationButton).css("left", container.width - $(this.navigationButton).width());
	$(this.navigationButton).css("top", container.height - $(this.navigationButton).height());
	
	$(this.mailButton).css("left", container.width - $(this.navigationButton).width());
	$(this.mailButton).css("top", container.height - $(this.navigationButton).height());
	
	$(this.miniMap).css("left", 0);
	$(this.miniMap).css("top", container.height - $(this.miniMap).height());
	
	$(this.miniMapBig).css("left", 0);
	$(this.miniMapBig).css("top", container.height - $(this.miniMapBig).height());
	
	var h = container.height - $("toppanel", this.DOM).height();
	var sz = 65;
	
	$(this.navigateToLeft).css({
		left: 0,
		top: 0,
		width: sz,
		height: h
	});
	
	$(this.navigateToRight).css({
		left: container.width-sz,
		top: 0,
		width: sz,
		height: h
	});
	
	$(this.navigateToTop).css({
		left: 0,
		top: 0,
		width: container.width,
		height: sz
	});
	
	$(this.navigateToBottom).css({
		left: 0,
		top: h-sz,
		width: container.width,
		height: sz
	});
	
	if(!this.scroll)
	{
		var scale = KINGDOM_SCALE;
		
		this.scroll = new iScroll(this.layersScrollWrapper.parentNode, {
			bounce: false,
			bounceLock: false,
			onBeforeScrollMove: $.proxy(function()
			{
				this.closeFieldMenu();
			}, this),
			momentum: false,
			sensitivity: 12,
			scrollbarColor: 'rgba(255,255,255,0.5)',
			zoom: false, zoomMin: scale, zoomMax: scale
		});
	}
	
	setTimeout($.proxy(this.scroll.refresh, this.scroll), 500);
}

KingdomView.prototype.getMouseCoords = function(e)
{
	if (e.originalEvent.touches)
	{
		var touches = e.originalEvent.changedTouches.length > 0 ? e.originalEvent.changedTouches : e.originalEvent.touches;  
		e = touches[0];
	}
	
	return {x: e.pageX, y: e.pageY};
}

KingdomView.prototype.swypeStartHandler = function(e)
{
	this.swypePosition.obj = e.currentTarget;
	var c = this.getMouseCoords(e);
	this.swypePosition.x = c.x;
	this.swypePosition.y = c.y;
	
	e.stopImmediatePropagation();
	return false;
}

KingdomView.prototype.swypeMoveHandler = function(e)
{
	if(this.swypePosition.obj != e.currentTarget)
	{
		return;
	}
	
	this.swypeNavigateMode = true;
	$(e.currentTarget).css({opacity: 1});
}

KingdomView.prototype.swypeEndHandler = function(e)
{
	if(this.swypePosition.obj != e.currentTarget)
	{
		return;
	}
	
	this.globalSwypeEndHandler(e);
	
	e.stopImmediatePropagation();
	return false;
}

KingdomView.prototype.closeFieldMenu = function(noReloadArrows)
{
	if(this.fieldMenu)
	{
		this.fieldMenu.destroy();
		this.fieldMenu = null;
		
		HintArrow.hideAll();
		
		if(!noReloadArrows)
		{
			setTimeout($.proxy(function()
			{
				HintArrow.clearUnderstood();
				this.updateEstateArrows();
				this.updateArrows();
			}, this), 500);
		}
	}
}

KingdomView.prototype.globalSwypeStartHandler = function(e)
{
	
}

KingdomView.prototype.globalSwypeEndHandler = function(e)
{
	var doNavigate = false;
	
	if(e && this.swypePosition.obj)
	{
		var c = this.getMouseCoords(e);
		
		var diffX = this.swypePosition.x - c.x;
		var diffY = this.swypePosition.y - c.y;
		
		if(this.swypePosition.obj.directX == NumberUtils.sign(diffX) && Math.abs(diffX) > Math.abs(diffY) && Math.abs(diffX) > 10)
		{
			this.onNavigateKingdom(this._kingdom.location.x+this.swypePosition.obj.directX+1, this._kingdom.location.y+1);
			doNavigate = true;
		}
		
		if(this.swypePosition.obj.directY == NumberUtils.sign(diffY) && Math.abs(diffY) > Math.abs(diffX) && Math.abs(diffY) > 10)
		{
			this.onNavigateKingdom(this._kingdom.location.x+1, this._kingdom.location.y+this.swypePosition.obj.directY+1);
			doNavigate = true;
		}
	}
	
	$(this.navigateToLeft).css({opacity: 0});
	$(this.navigateToRight).css({opacity: 0});
	$(this.navigateToTop).css({opacity: 0});
	$(this.navigateToBottom).css({opacity: 0});
	
	this.swypePosition.obj = null;
	
	var swypeNavigateMode = this.swypeNavigateMode;
	this.swypeNavigateMode = false;
	
	if(doNavigate)
	{
		this.closeFieldMenu();
		return;
	}
	else
	{
		if(this.fieldMenu)
		{
			this.closeFieldMenu();
			return;
		}
		
		if (e.originalEvent.touches && e.originalEvent.touches.length > 1)
		{
			return;
		}
		
		if(swypeNavigateMode)
		{
			return;
		}
		
		if(e && !APP.ticksIsPaused)
		{
			if (e.originalEvent.touches)
			{
				var touches = e.originalEvent.changedTouches.length > 0 ? e.originalEvent.changedTouches : e.originalEvent.touches;  
				e = touches[0];
			}
			
			var container = $(this.layersScrollWrapper);
			var x = e.pageX - container.offset().left - (this.cellWidth + this.cellsPadding.left);
			var y = e.pageY - container.offset().top - (this.cellHeight + this.cellsPadding.top);
			
			var scale = (this.scroll && this.scroll.scale) ? this.scroll.scale : 1;
			
			x /= scale;
			y /= scale;
			
			var tx = Math.floor(x/this.cellWidth);
			var ty = Math.floor(y/this.cellHeight);
			
			if(tx >= 0 && ty >= 0 && tx < FKingdom.WIDTH && ty < FKingdom.HEIGHT)
			{
				this.showMenuFor({field: this._kingdom.getField(tx, ty)});
				
				if(this.fieldMenu)
				{
					var parentWidth = $(this.layersWrapper).width();
					var parentHeight = $(this.layersWrapper).height();
					
					var menuWidth = this.fieldMenu.DOM.offsetWidth;
					var menuHeight = this.fieldMenu.DOM.offsetHeight;
					
					var scrollX = this.scroll ? this.scroll.x : 0;
					var scrollY = this.scroll ? this.scroll.y : 0;
					
					var centerX = (tx+1) * this.cellWidth * scale + this.cellsPadding.left * scale + scrollX + this.cellWidth * scale / 2;
					x = centerX - menuWidth/2;
					y = (ty) * this.cellHeight * scale + this.cellsPadding.top * scale + scrollY;
					
					if(x < 0)
					{
						x = 0;
					}
					
					if(y < 0)
					{
						y = (ty+2) * this.cellHeight * scale + this.cellsPadding.top * scale + scrollY;
						this.fieldMenu.pointer.className = "top";
					}
					
					if(x + menuWidth > parentWidth)
					{
						x = parentWidth - menuWidth;
					}
					
					if(y + menuHeight > parentHeight)
					{
						y = parentHeight - menuHeight;
					}
					
					this.fieldMenu.pointer.style.left = (centerX - x - this.fieldMenu.pointer.offsetWidth/2) + "px";
					
					this.fieldMenu.DOM.style.left = x + "px";
					this.fieldMenu.DOM.style.top = y + "px";
					
					this.fieldMenu.DOM.style.opacity = 1;
				}
				
				//e.stopImmediatePropagation();
				//return false;
			}
		}
	}
}

KingdomView.prototype.showInfo = function()
{
	KingdomInfoDialog.popup(this._kingdom);
}

KingdomView.prototype.showMenuFor = function(f)
{
	if (Quarantine.contains(f.field))
	{
        return;
    }
    
    if (f.field.hasEstate() && f.field.estate.hasOwner() && f.field.estate.owner.isOnVacation())
    {
        this.onShowInfo(f.field);
        return;
    }
    
    if (f.field.isVoid && !f.field.hasEstate())
    {
        return;
    }

    var menu = COM.create("KingdomMenu");
    this.layersWrapper.appendChild(menu.DOM);
    
    if(f.field.city &&
       f.field.city.type.name != EstateType.RUIN.name &&
       f.field.city.type.name != EstateType.CASTLE.name &&
       f.field.city.type.name != EstateType.CATAPULT.name)
    {
    	$(menu.DOM).addClass("showStars");
    	COM(menu.stars).titleIndex = f.field.estate.ownerTitleIndex;
    }
    else
    {
    	$(menu.DOM).removeClass("showStars");
    }
    
    UI.show(f.field.canSendSettler() && !FUser.isSpectator(), [menu.btnSettle]);
    menu.onBuild = $.proxy(function()
    {
        this.closeFieldMenu();
        this.onSettler(f.field);
    }, this);
    
    UI.show(!f.field.hasEstate() && !FUser.isSpectator(), [menu.btnExpedition]);
    menu.onExpedition = $.proxy(function()
    {
        this.closeFieldMenu();
        this.onExpedition(f.field);
    }, this);
    
    UI.show(true, [menu.showInfo]);
    menu.onShowInfo = $.proxy(function()
    {
        this.closeFieldMenu();
        this.onShowInfo(f.field);
    }, this);
    
    var hasScienceCamps = FUser.CURRENT_USER.science.getLevel(Science.CAMPS) > 0;
    UI.show(hasScienceCamps && f.field.canSendSettler() && !FUser.isSpectator(), [menu.btnCamp]);
    menu.onMakeCamp = $.proxy(function()
    {
        this.closeFieldMenu();
        this.onMakeCamp(f.field);
    }, this);

    var hasScienceGeology = FUser.CURRENT_USER.science.getLevel(Science.GEOLOGY) > 0;
    var selectedType = this.citiesCombo.selected ? this.citiesCombo.selected.estateInfo.estateType.name : "";
    UI.show(FConfig.PEARL_MOVE_CITY_PRICE > 0 && selectedType == EstateType.CITY.name && f.field.canSendSettler() && !FUser.isSpectator(), [menu.btnMoveCityHere]);

    if(hasScienceGeology)
    {
        menu.onMoveCity = $.proxy(function()
        {
            this.closeFieldMenu();
            this.onMoveCity(f.field);
        }, this);
    }
    else
    {
        menu.onMoveCity = $.proxy(function()
        {
            this.closeFieldMenu();
            Alert.create(I18.mega(I18.f("UNAVAILABLE_NO_SCIENCE"), Science.GEOLOGY));
        }, this);
    }
    
    var canCast = !FUser.isSpectator() && this.citiesCombo.selected.hasGarrisonHero;
    UI.show(!FUser.isSpectator() && canCast, [menu.castSpell]);
    menu.onCastSpell = $.proxy(function()
    {
        this.closeFieldMenu();
        this.onCastSpell(f.field);
    }, this);
    
    UI.show(!FUser.isSpectator(), [menu.btnEnter]);
    
    if (f.field.hasEstate())
    {
    	var estate = f.field.estate;
        var ownedByMe = estate.hasOwner() && FUser.CURRENT_USER.equals(estate.owner);
        var myClanCastle = estate.type.name == EstateType.CASTLE.name && FUser.CURRENT_USER.isClanLeader()
                && FUser.CURRENT_USER.diplomacy.clan.clanName == estate.info.clanName;

		/*
		var myClanCatapult = estate.type.name == EstateType.CATAPULT.name && FUser.CURRENT_USER.isClanLeader()
			&& FUser.CURRENT_USER.diplomacy.clan.clanName == estate.info.clanName;
		*/
        var myClanCatapult = estate.canDeleteCatapult;

        var immune = FConfig.IS_SURVIVAL && FUser.CURRENT_USER.race.isSameSide(estate.owner.race) && !ownedByMe && !estate.owner.isFake();

        //menu.btnEnter.style.display = "";

        if (estate.canEnter)
        {
            menu.onEnter = $.proxy(function()
            {
                this.closeFieldMenu();
                if (f.field.hasEstate())
                {
                    Events.onShowCity.fire(estate.id);
                    GlobalSelectionCitiesCombo._selectedCityId = estate.id;
                }
            }, this);
        }
        else
        {
            $(menu.btnEnter).addClass("spy");
            menu.onEnter = $.proxy(function()
            {
                this.closeFieldMenu();
                this.onInitiateEspionage(f.field);
            }, this);
            
            UI.show(!immune, [menu.btnEnter]);
        }

        UI.show((ownedByMe && f.field.hasCity() && !f.field.city.isCapital || myClanCatapult && f.field.hasCatapult()) && !FUser.isSpectator(), [menu.btnDestroy]);
        UI.show((ownedByMe || myClanCastle) && f.field.hasOuterEstate() && !f.field.hasRuin() && !f.field.hasGrotto() && !FUser.isSpectator(), [menu.btnRefuse]);

        menu.onDestroy = $.proxy(function()
        {
            this.closeFieldMenu();
            if (f.field.hasEstate())
            {
                if (f.field.hasOuterEstate())
                {
                    this.onLeaveOuterEstate(f.field);
                }
                else
                {
                    this.onCityDelete(f.field);
                }
            }
        }, this);

        if (FUser.isSpectator() || FUser.CURRENT_USER.equals(estate.owner) || estate.ownedByAlly || estate.ownedByClanMate)
        {
            //menu.btnAttack.setup(false, false);
            UI.show(false, [menu.btnAttack]);
        }
        else if (estate.ownerInPeace)
        {
            //menu.btnAttack.setup(true, false, I18.CANNOT_ATTACK_IN_PEACE.f());
            UI.show(false, [menu.btnAttack]);
        }
        else if (immune)
        {
            //menu.btnAttack.setup(true, false, I18.CANT_ATTACK_SAME_SIDE.f());
            UI.show(false, [menu.btnAttack]);
        }
        else if (estate.ownedByFriend)
        {
            //menu.btnAttack.setup(true, false, I18.CANT_ATTACK_FRIEND.f());
            UI.show(false, [menu.btnAttack]);
        }
        else
        {
            //menu.btnAttack.setup(true, true, I18.ATTACK.f());
            UI.show(true, [menu.btnAttack]);
        }
        
        menu.onAttack = $.proxy(function()
        {
            this.closeFieldMenu();
            this.onCityAttack(f.field);
        }, this);

		UI.show(estate.canAssist && !FUser.isSpectator(), [menu.btnAssist]);
        menu.onAssist = $.proxy(function()
        {
            this.closeFieldMenu();
            this.onCityAssist(f.field);
        }, this);
        
        UI.show(ownedByMe && (!estate.id.equals(this.estateId)), [menu.btnMoveArmy]);
        menu.onMoveArmy = $.proxy(function()
        {
            this.closeFieldMenu();
            this.onMoveArmy(f.field);
        }, this);
        
        UI.show(ownedByMe && (!estate.id.equals(this.estateId)), [menu.btnMoveResources]);
        menu.onMoveResources = $.proxy(function()
        {
            this.closeFieldMenu();
            this.onMoveResources(f.field);
        }, this);

        UI.show(ownedByMe && FUser.CURRENT_USER.isBetrayer() && (f.field.hasCity() || f.field.hasCamp()), [menu.btnForgiveBetrayer]);
        menu.onForgiveBetrayer = $.proxy(function()
        {
            this.closeFieldMenu();
            this.onForgiveBetrayer();
        }, this);

        UI.show(!ownedByMe && estate.hasRealOwner() && (f.field.hasCity() || f.field.hasCamp() || (f.field.hasCastle() && !Strings.isEmpty(estate.info.clanName))), [menu.btnMakeGift]);
        menu.onMakeGift = $.proxy(function()
        {
            this.closeFieldMenu();
            this.onMakeGift(f.field);
        }, this);

		UI.show((!f.field.hasCity() || (!ownedByMe && f.field.hasFakeRuin())) && !f.field.hasCamp() && !f.field.hasCastle() && !(ownedByMe && f.field.hasRuin()) && !FUser.isSpectator(), [menu.btnShareOuterEstate]);
        menu.onShareOuterEstate = $.proxy(function()
        {
            this.closeFieldMenu();
            this.onShareOuterEstate(f.field);
        }, this);

        UI.show(!ownedByMe && !f.field.hasCastle() && !f.field.hasCatapult()  && (estate.hasOwner() || f.field.hasRuin()), [menu.btnSendMessage]);
        menu.onSendMessage = $.proxy(function()
        {
            this.closeFieldMenu(true);
            if (f.field.hasRuin())
            {
                this.onSendMonsterMessage(estate.id);
            }
            else if (f.field.hasEstate())
            {
                this.onSendMessage(estate.owner.username, "", null, "", MailReceiverType.USER);
            }
        }, this);

        UI.show(estate.canObserve && !FUser.isSpectator(), [menu.btnObserve]);
        menu.onObserve = $.proxy(function()
        {
            this.closeFieldMenu();
            this.onObserve(f.field);
        }, this);
        if (immune)
        {
            UI.show(false, [menu.btnObserve]);
        }
        
        HintArrow.OPEN_KINGDOM_MENU.understood();
        var hint = FUser.CURRENT_USER.findHint(HintType.SEND_LETTER);
        if (hint)
        {
            HintArrow.CLICK_KINGDOM_MENU_MAIL.show(menu.btnSendMessage, hint, this.DOM);
        }
    }
    else
    {
    	$(menu.btnEnter).addClass("viewReport");
    	
        if (f.field.exploreTick > 0)
        {
            if (UI.show(FUser.CURRENT_USER.isPremium(), [menu.btnEnter]))
            {
                menu.onEnter = $.proxy(function()
                {
                    this.closeFieldMenu();
                    Service.kingdom.callMethod("getExploredField", [f.field.getLocationXYZ()], function(e)
                    {
                        Events.onShowExpeditionResult.fire(e);
                    });
                }, this);
            }
        }
        else
        {
            UI.show(false, [menu.btnEnter])
        }
        
        UI.show(false, [
        	menu.btnDestroy, menu.btnRefuse, menu.btnAttack, menu.btnAssist, menu.btnMoveArmy,
        	menu.btnMoveResources, menu.btnSendMessage, menu.btnObserve, menu.btnForgiveBetrayer,
        	menu.btnMakeGift, menu.btnShareOuterEstate
        ]);
    }
   
	menu.DOM.style.opacity = 0;
	this.fieldMenu = menu;
}

KingdomView.prototype.changeViewLevel = function()
{
	this.viewLevel++;
	if(this.viewLevel >= this.viewLevels.length)
	{
		this.viewLevel = 0;
	}
	
	APP.showLoader();
	
	if (FUser.CURRENT_USER && FUser.CURRENT_USER.settings)
	{
		showGrid = this.viewLevel > 0;
		FUser.CURRENT_USER.settings.kingdomGridVisible = showGrid;
		Service.user.callMethod("updateKingdomGrid", [showGrid]);
	}
	
	setTimeout($.proxy(function()
	{
		this.update(this._kingdom, true, true);
	}, this), 10);
}

KingdomView.prototype.hidePanels = function()
{
	//this.css({left: "0px"});
	//this.DOM.style.webkitTransitionDuration = "300ms";
	//this.DOM.style.webkitTransform = "translate3d(0px, 0px, 0px)";
	$(this.DOM).removeClass("panelExpanded");
	this.leftPanelExpanded = false;
	this.rightPanelExpanded = false;
	this.locker.style.display = "none";
}

KingdomView.prototype.showLeftPanel = function()
{
	//this.css({left: "480px"});
	//this.DOM.style.webkitTransitionDuration = "300ms";
	//this.DOM.style.webkitTransform = "translate3d(480px, 0px, 0px)";
	//$(this.DOM).addClass("panelExpanded");
	this.leftPanelExpanded = true;
	this.locker.style.display = "block";
	this.closeFieldMenu();
}

KingdomView.prototype.showRightPanel = function()
{
	//this.css({"left": "-480px"});
	//alert("show");
	$(this.DOM).addClass("panelExpanded");
	this.rightPanelExpanded = true;
	this.locker.style.display = "block";
	this.closeFieldMenu();
}

KingdomView.prototype.updateEstateButton = function()
{
	this.hidePanels();
	
	var b = COM(this.changeEstateButton);
	if(this.citiesCombo.selected)
	{
	    b.DOM.style.display = "";
	    b.caption = this.citiesCombo.selected.estateInfo.getName();
	    b.subCaption = this.citiesCombo.selected.estateInfo.id.toString();
	    b.icon = STATIC_INTERFACE_URL + this.citiesCombo.selected.estateInfo.getEstateIcon();
	}
	else
	{
		b.DOM.style.display = "none";
	}
}

KingdomView.prototype.showMiniMap = function()
{
	this.miniMap.style.display = "block";
	this.miniMapBig.style.display = "none";
}

KingdomView.prototype.showMiniMapBig = function()
{
	this.miniMap.style.display = "none";
	this.miniMapBig.style.display = "block";
}

KingdomView.prototype.updateSearch = function()
{
	COM(this.searchX).value = this._kingdom.location.x + 1;
	COM(this.searchY).value = this._kingdom.location.y + 1;
}

KingdomView.prototype.addCacheImage = function(cache, src)
{
	var cnt = 0;
	for(var i in cache)
	{
		if(cache[i] == src) return cache;
		
		cnt++;
	}
	
	cache["img"+cnt] = src;
	
	return cache;
}

KingdomView.prototype.gotoCoords = function()
{
	//APP.showKingdom({x: this.searchX.value-1, y: this.searchY.value-1});
	this.onNavigateKingdom(COM(this.searchX).value, COM(this.searchY).value);
}

KingdomView.prototype.gotoCity = function()
{
	//APP.showKingdom(this.citiesCombo.selected.estateInfo.id);
	
	var id = this.getEstateId();
	if(id.x+1 == COM(this.searchX).value && id.y+1 == COM(this.searchY).value)
	{
		this.centerMap();
	}
	else
	{
		this.onNavigateKingdom(this.citiesCombo.selected.estateInfo.id.x+1, this.citiesCombo.selected.estateInfo.id.y+1);
	}
}

KingdomView.prototype.getCellImage = function(f)
{
	var img = null;
	
	if (f.hasEstate())
	{
	    if (f.estate.__CLS__ == "game.service.kingdom.FKingdomCity")
	    {
	        img = Assets.getKingdomCityImage(f.estate.owner.race, f.estate.skinIndex);
	        if (f.estate.type.name == EstateType.RUIN.name)
	        {
	            img = KingdomAssets.RUIN;
	        }
	    }
	    else if (f.estate.__CLS__ == "game.service.kingdom.FKingdomOuterEstate")
	    {
	        img = Assets.getKingdomOuterEstateImage(f.estate);
	    }
	}
	
	return img;
}

KingdomView.prototype.getMagicImage = function(f)
{
	var img = null;
	
	if (f.estate && Tick.now() <= f.estate.magicDisableTillTick)
	{
        img = KingdomAssets["MAGIC_DISABLE_STATIC"];
    }
    else if (f.estate && Tick.now() <= f.estate.magicDefenseTillTick)
    {
        img = KingdomAssets["MAGIC_DEFENSE_STATIC"];
    }
    else
    {
    }
    
    return img;
}

KingdomView.prototype.getBuffImage = function(f)
{
	var img = null;
	
	var buff = FBuff.findFirst(f.buffs, BuffType.KINGDOM_ANIMATION, null, Tick.now());
    if (buff && f.hasEstate())
    {
        img = KingdomAssets["KINGDOM_STATIC_" + buff.value];
    }
    else
    {
    }
    
    return img;
}

KingdomView.UNDERGROUND_TILE_REPLACE = KingdomAssets.T_STEPPE_0;

KingdomView.prototype.update = function(k, noCenterMap, force)
{
	if(FUser.CURRENT_USER && FUser.CURRENT_USER.settings)
	{
		this.viewLevel = FUser.CURRENT_USER.settings.kingdomGridVisible ? 1 : 0;
	}
	
	COM(this.navigationButton).image.src = STATIC_URL + Assets.getKingdomCityImage(FUser.CURRENT_USER.race, 0);
	COM(this.mailButton).image.src = STATIC_INTERFACE_URL + "city/dashboard/messages.png";
	
	if(k.links.length > 0)
	{
		this.logoutButton.style.display = "none";
		this.mailButton.style.display = "none";
		this.navigationButton.style.display = "";
		this.changeEstateButton.style.display = "";
		this.returnButton.style.display = "none";
	}
	else
	{
		this.logoutButton.style.display = "";
		this.mailButton.style.display = "";
		this.navigationButton.style.display = "none";
		this.changeEstateButton.style.display = "none";
		this.returnButton.style.display = "none";
	}

	var newKindom = !this._kingdom || !this._kingdom.location.equals(k.location);
	
	this._kingdom = k;
	
	if(this.scroll) setTimeout($.proxy(function()
	{
		this.scroll.refresh();
	}, this), 100);
	
	COM(this.pearlBox).update(FUser.CURRENT_USER);
	
	var cache = {};
    var cimg, mapIsChanged = false, cityName, cityOwnerName;
    
    cache.mini_map = KingdomAssets.MINI_MAP;
    cache.VAGONETTE_IRON = KingdomAssets.VAGONETTE_IRON;
    cache.VAGONETTE_STONE = KingdomAssets.VAGONETTE_STONE;
    cache.VAGONETTE_GOLD = KingdomAssets.VAGONETTE_GOLD;
    cache.ANCIENTS_DEFENSE = KingdomAssets.ANCIENTS_DEFENSE;
	cache.CLOSED_GATE_ICON = KingdomAssets.CLOSED_GATE_ICON;
	cache.FLAG = KingdomAssets.FLAG;
	cache.UNDERGROUND_TILE_REPLACE = KingdomView.UNDERGROUND_TILE_REPLACE;
    
    for (var i = -1; i < FKingdom.HEIGHT; i++)
    {
        for (var j = -1; j < FKingdom.WIDTH; j++)
        {
            var f = k.getField(j < 0 ? 0 : j, i < 0 ? 0 : i);
            
            cache = this.addCacheImage(cache, f.terrain.findTile(f.terrainSubType, f.hasEstate()));
            
            cimg = this.getCellImage(f);
			if(cimg)
			{
				cache = this.addCacheImage(cache, cimg);
			}
			
            if(i >= 0 && j >= 0)
            {
            	if(!this.fieldsHistory[i][j]) this.fieldsHistory[i][j] = {};
            	
            	if(this.fieldsHistory[i][j].image !== cimg)	mapIsChanged = true;
            	this.fieldsHistory[i][j].image = cimg;
            	
            	cimg = this.getMagicImage(f);
	            if(cimg)
				{
					cache = this.addCacheImage(cache, cimg);
					if(this.fieldsHistory[i][j].magic !== cimg) mapIsChanged = true;
					this.fieldsHistory[i][j].magic = cimg;
				}
				
	            cimg = this.getBuffImage(f);
	            if(cimg)
				{
					cache = this.addCacheImage(cache, cimg);
					if(this.fieldsHistory[i][j].buff !== cimg) mapIsChanged = true;
					this.fieldsHistory[i][j].buff = cimg;
				}
				
				var gateClosed = f.hasEstate() && f.estate.gatesClosed;
				if(this.fieldsHistory[i][j].gateClosed !== gateClosed) mapIsChanged = true;
				this.fieldsHistory[i][j].gateClosed = gateClosed;
				
				cityName = "";
				cityOwnerName = "";
				
				if(f.hasEstate())
				{
				    cityName = this.getCityName(f);
				    cityOwnerName = (f.estate.hasOwner() && !f.hasRuin() && !f.hasFakeRuin()) ? this.getCityOwnerName(f) : "";
				    
				    if(this.fieldsHistory[i][j].cityName !== cityName) mapIsChanged = true;
				    if(this.fieldsHistory[i][j].cityOwnerName !== cityOwnerName) mapIsChanged = true;
				}
				
				this.fieldsHistory[i][j].cityName = cityName;
                this.fieldsHistory[i][j].cityOwnerName = cityOwnerName;
            }
        }
    }
	
	this.citiesCombo.links = k.links;
	
    if (!this.citiesCombo.selected && k.links.length > 0)
    {
        var id = APP.getCityMediator()._estate.id;
        for(var i=0; i<k.links.length; i++)
        {
        	if(k.links[i].id.equals(id))
        	{
        		this.citiesCombo.selected = k.links[i];
        	}
        }
        //this.citiesCombo.selected = k.getCapitalLink();
        //this.setSelected(this.citiesCombo.selected);
    }
    
    var present = false;
	var capital = null;
	for(var i=0; i<k.links.length; i++)
	{
		if(k.links[i].isCapital) capital = k.links[i];
		if(this.citiesCombo.selected && k.links[i].id.equals(this.citiesCombo.selected.id)) present = true;
	}
	
	if(!present && capital)
	{
		this.citiesCombo.selected = capital;
		GlobalSelectionCitiesCombo._selectedCityId = capital.id;
	}
	
	this.updateEstateButton();
	
	//console.log("UPDAE");
	
	if(!newKindom && !force && !mapIsChanged && !this.globalForceUpdate)
	{
		HintArrow.hideAll();
		this.updateArrows();
		return;
	}
	
	this.globalForceUpdate = false;
	
	this.updateEstateArrows();
	
	this.closeFieldMenu();
    
    this.showInfoButton.style.display = (FConfig.IS_SURVIVAL || k.currentUser.canAttackAnyRuin()) ? "inline-table" : "none";
    
    this.updateSearch();
    
    APP.clearImagesCache();
    APP.addPreloadImages(cache, STATIC_INTERFACE_URL);
	
	APP.waitPreloadImages($.proxy(function()
	{
		this.draw(noCenterMap);
	}, this));
	
    if (FUser.CURRENT_USER.isOnVacation())
    {
        OnVacationDialog.spawn();
    }

	this.tick();
}

KingdomView.prototype.getActiveCells = function()
{
	var cells = [];
	var l;
	
	var clan = FUser.CURRENT_USER.diplomacy.clan;
	if(clan)
	{
		for(var i=0; i<clan.castles.length; i++)
		{
			cells.push({"location": clan.castles[i], "color": Effects.ORANGE});
		}

        for(var i=0; i<clan.otherEstates.length; i++)
        {
            cells.push({"location": clan.otherEstates[i], "color": Effects.ORANGE});
        }
	}
	
	if (this._kingdom.allies)
	{
		for(var i=0; i<this._kingdom.allies.length; i++)
		{
			cells.push({"location": this._kingdom.allies[i], "color": Effects.ORANGE});
		}
	}
	
	if (this._kingdom.clanmates)
	{
		for(var i=0; i<this._kingdom.clanmates.length; i++)
		{
			cells.push({"location": this._kingdom.clanmates[i], "color": Effects.ORANGE});
		}
	}
	
	for(var i=0; i<this._kingdom.links.length; i++)
	{
		cells.push({"location": this._kingdom.links[i].estateInfo.location, "color": Effects.YELLOW});
	}
	
	return cells;
}

KingdomView.prototype.drawCellText = function(ctx, text, x, y, cellWidth, noSplit)
{
	ctx.save();
	
	/*
	ctx.shadowColor = "#000";
	ctx.shadowOffsetX = 0;
	ctx.shadowOffsetY = 0;
	ctx.shadowBlur = 3;
	*/
	ctx.lineWidth = 4;
	ctx.strokeStyle = "#000";
	
	text = htmlUnescape(text);
	text = text.split("\n").join(" ");
	text = text.split("\r").join(" ");
	
	if(noSplit)
	{
	    ctx.strokeText(text, x + (cellWidth-ctx.measureText(text).width)/2, y);
        ctx.fillText(text, x + (cellWidth-ctx.measureText(text).width)/2, y);
	}
	else
	{
    	var tmp = text.split(" ");
    	var s = "", cs = "", len;
    	while(tmp.length > 0)
    	{
    		cs = tmp.shift();
    		len = ctx.measureText(s + cs);
    		if(len.width > cellWidth)
    		{
    			if(s != "")
    			{
                    ctx.strokeText(s, x + (cellWidth-ctx.measureText(s).width)/2, y);
                    ctx.fillText(s, x + (cellWidth-ctx.measureText(s).width)/2, y);
                    y += 20*GAME_IMAGES_PACK;
    			}
    			s = cs;
    		}
    		else
    		{
    			s += cs + " ";
    		}
    		
    		if(tmp.length == 0)
    		{
    			ctx.strokeText(s, x + (cellWidth-ctx.measureText(s).width)/2, y);
    			ctx.fillText(s, x + (cellWidth-ctx.measureText(s).width)/2, y);
    		}
    	}
	}
	
	ctx.restore();
}

KingdomView.prototype.getCityName = function(f)
{
    var cityName = "";
    
    cityName = f.estate.getName();
                
    if (f.hasFakeRuin())
    {
        cityName = I18.f("RUIN_NAME");
    }
    
    cityName = f.estate.hasLevel() ? I18.f("ESTATE_NAME_AND_LEVEL_FORMAT", [cityName, f.hasFakeRuin() ? f.estate.info.titleIndex : f.estate.level]) : cityName;
    
    return cityName;
};

KingdomView.prototype.getCityOwnerName = function(f)
{
    if (f.estate.hasOwner() && !f.hasRuin() && !f.hasFakeRuin()) return FUser.getUserNameForMessages(f.estate.owner.username);
    else return "";
}; 

KingdomView.prototype.draw = function(noCenterMap)
{
	APP.hideLoader();
	
	var img, x, y, cimg;
	
	var vl = this.viewLevels[this.viewLevel];
	
	this.tiles.width = this.tiles.width;
	//this.cells.width = this.cells.width;
	//this.animations.width = this.animations.width;
	//this.cityNames.width = this.cityNames.width;
	
	var cellX, cellY, cellIX=0;
	//var centerTo;
	
	this.tiles.ctx.strokeStyle = "#fff";
	this.tiles.ctx.lineWidth = 1;
	
	/*
	if(this.tiles.ctx.setLineDash)
	{
		this.tiles.ctx.setLineDash([5,2]);
	}
	*/
	
	this.tiles.ctx.font = (18*GAME_IMAGES_PACK) + "px pt-bold";
    
    /*
    this.tiles.ctx.shadowColor = "#000";
	this.tiles.ctx.shadowOffsetX = 1*GAME_IMAGES_PACK;
	this.tiles.ctx.shadowOffsetY = 1*GAME_IMAGES_PACK;
	this.tiles.ctx.shadowBlur = 8*GAME_IMAGES_PACK;
	*/
	
	var f, f2;
	//draw tiles
	for (var i = -2; i <= FKingdom.HEIGHT; i++)
    {
        for (var j = -2; j <= FKingdom.WIDTH; j++)
        {
            x = j;
            if(x < 0) x = 0;
            if(x > FKingdom.WIDTH - 1) x = FKingdom.WIDTH - 1;
            
            y = i;
            if(y < 0) y = 0;
            if(y > FKingdom.HEIGHT - 1) y = FKingdom.HEIGHT - 1;
            
            f = this._kingdom.getField(x, y);
            
            img = APP.getPreloadImage(f.terrain.findTile(f.terrainSubType, f.hasEstate()));
            
            if(i < 0 || j < 0 || i >= FKingdom.HEIGHT || j >= FKingdom.WIDTH)
            {
            	if(f.terrain.name == Terrain.UNDERGROUND.name)
            	{
            		//f2 = this._kingdom.getField(x == 0 ? 1 : x-1, y == 0 ? 1 : y-1);
            		//img = APP.getPreloadImage(f2.terrain.findTile(f2.terrainSubType, f2.hasEstate()));
            		img = APP.getPreloadImage(KingdomView.UNDERGROUND_TILE_REPLACE);
            	}
            }
            
            cellX = (j+1)*this.cellWidth + this.cellsPadding.left;
            cellY = (i+1)*this.cellHeight + this.cellsPadding.top;
            
            try {
            	this.tiles.ctx.drawImage(img, 0, 0, img.width, img.height, cellX + this.tilesPadding.left + this.cellsPadding.left, cellY - this.cellHeight/2 + this.tilesPadding.top + this.cellsPadding.top, img.width, img.height);
            } catch (e) {}
		}
	}
	
	if(vl.showGrid)
	{
		this.tiles.ctx.beginPath();
		this.tiles.ctx.moveTo(this.cellWidth + this.cellsPadding.left, this.cellHeight + this.cellsPadding.top);
		this.tiles.ctx.lineTo(this.cellWidth + this.cellsPadding.left + this.cellWidth*FKingdom.WIDTH, this.cellHeight + this.cellsPadding.top);
		this.tiles.ctx.lineTo(this.cellWidth + this.cellsPadding.left + this.cellWidth*FKingdom.WIDTH, this.cellHeight + this.cellsPadding.top + this.cellHeight*FKingdom.HEIGHT);
		this.tiles.ctx.stroke();
	}
	
	var fieldInfoText, cityName, cityOwnerName, color, eif;
	//draw estates
	var extraInfo = [];
	for (var i = 0; i < FKingdom.HEIGHT; i++)
    {
        extraInfo[i] = [];
        for (var j = 0; j < FKingdom.WIDTH; j++)
        {
        	eif = {};
        	
        	fieldInfoText = "";
            cityName = "";
            cityOwnerName = "";
            color = "#fff";
            
            cellX = (j+1)*this.cellWidth + this.cellsPadding.left;
            cellY = (i+1)*this.cellHeight + this.cellsPadding.top;
            
            eif.cellX = cellX;
            eif.cellY = cellY;
            
        	if(vl.showGrid)
        	{
            	this.tiles.ctx.beginPath();
				this.tiles.ctx.moveTo(cellX, cellY);
				this.tiles.ctx.lineTo(cellX, cellY+this.cellHeight);
				this.tiles.ctx.lineTo(cellX+this.cellWidth, cellY+this.cellHeight);
				this.tiles.ctx.stroke();
			}
			
			f = this._kingdom.getField(j, i);
        	
        	cimg = this.getCellImage(f);
        	if(cimg)
        	{
        		if(f.hasEstate())
        		{
        			var glow = null;
        			
        			if(f.estate.id.equals(this.getEstateId()))
        			{
        				glow = "#0059b2";
        			}
        			
        			if(f.hasRuin() && f.estate.isSpecial)
        			{
        				glow = "#e2281c";
        			}
        			
        			if(glow)
        			{
        				this.tiles.ctx.save();
        				this.tiles.ctx.globalAlpha = 0.4;
        				this.tiles.ctx.scale(1.3, 1);
        				this.tiles.ctx.fillStyle = glow;
        				this.tiles.ctx.beginPath();
						this.tiles.ctx.arc((cellX + this.cellWidth/2)/1.3, cellY + this.cellHeight/2, this.cellHeight/2, 0, Math.PI*2);
						this.tiles.ctx.fill();
						this.tiles.ctx.restore();
        			}
        		}
        		
        		img = APP.getPreloadImage(cimg);
        		
        		var y = cellY + this.cellHeight/6*5 - img.height;
        		if(f.city.type.name == EstateType.EVIL.name)
        		{
        			y = cellY - img.height/2;
        		}
        		
        		try {
        			this.tiles.ctx.drawImage(img, 0, 0, img.width, img.height, cellX + (this.cellWidth-img.width)/2, y, img.width, img.height);
        		} catch (e) {}
        		
        		if(f.estate.__CLS__ == "game.service.kingdom.FKingdomOuterEstate")
			    {
			        if (f.estate.type.name == EstateType.PIT.name)
			        {
			            var resources = f.estate.resources;
			            var rimg;
			            for(var r=0; r<resources.length; r++)
			            {
			            	rimg = null;
			            	if(resources[r].name == Resource.IRON.name) rimg = KingdomAssets.VAGONETTE_IRON;
			            	if(resources[r].name == Resource.STONE.name) rimg = KingdomAssets.VAGONETTE_STONE;
			            	if(resources[r].name == Resource.GOLD.name) rimg = KingdomAssets.VAGONETTE_GOLD;
			            	
			            	if(rimg)
			            	{
			            		rimg = APP.getPreloadImage(rimg);
			            		eif.resource = rimg;
			            		//this.tiles.ctx.drawImage(rimg, 0, 0, rimg.width, rimg.height, cellX + (this.cellWidth-rimg.width)/2, cellY + this.cellHeight/4*3 - rimg.height, rimg.width, rimg.height);
			            	}
			            }
			        }
			    }
        		
        		//console.log(f.estate);
        		cityName = this.getCityName(f);
	            
	            if (f.estate.hasOwner() && !f.hasRuin() && !f.hasFakeRuin())
	            {
	                cityOwnerName = this.getCityOwnerName(f);
	                if (f.estate.ownedByMe || f.estate.ownedByAlly || f.estate.ownedByClanMate || f.estate.ownerInPeace || f.estate.ownedByFriend)
	                {
	                    color = Effects.ALLY;
	                }
	                else if (f.estate.ownedByEnemy)
	                {
	                    color = Effects.ENEMY;
	                }
	                else if (f.estate.owner.isBetrayer())
	                {
	                    color = Effects.BETRAYER;
	                }
	                else
	                {
	                    color = Effects.WHITE;
	                }
	            }
	            
	            /*
	            if(f.estate.id.equals(this.getEstateId()))
	            {
	            	centerTo = i*FKingdom.WIDTH + j;
	            }
	            */
        	}
        	else
        	{
            	if (f.exploreTick > 0)
            	{
	                if (f.cellCount > 0) 
	                {
	                    fieldInfoText = I18.f("CELL_INFO_USER_BUILDABLE", f.cellCount);
	                }
	                else
	                {
	                    fieldInfoText = I18.f("CELL_INFO_SYSTEM");
	                }
	            }
            }
            
            if(vl.showAnimations)
            {
	            cimg = this.getMagicImage(f);
	            if(cimg)
	            {
	            	img = APP.getPreloadImage(cimg);
	            	eif.magic = img;
            		//this.tiles.ctx.drawImage(img, 0, 0, img.width, img.height, cellX + (this.cellWidth-img.width)/2, cellY + (this.cellHeight-img.height)/2 - this.cellHeight/6, img.width, img.height);
	            }
	            
	            cimg = this.getBuffImage(f);
	            if(cimg)
	            {
	            	img = APP.getPreloadImage(cimg);
	            	eif.buff = img;
            		//this.tiles.ctx.drawImage(img, 0, 0, img.width, img.height, cellX + (this.cellWidth-img.width)/2, cellY + (this.cellHeight-img.height)/2 - this.cellHeight/6, img.width, img.height);
	            }
            }
            
            if(f.hasEstate() && f.estate.hasFlags)
    		{
    			var fimg = APP.getPreloadImage(KingdomAssets.FLAG);
    			eif.flag = fimg;
    			//this.tiles.ctx.drawImage(fimg, 0, 0, fimg.width, fimg.height, cellX + this.cellWidth-fimg.width, cellY + fimg.height, fimg.width, fimg.height);
    		}
    		
    		if(f.hasEstate() && f.estate.ancientsDefense)
    		{
    			var fimg = APP.getPreloadImage(KingdomAssets.ANCIENTS_DEFENSE);
    			eif.ancients = fimg;
    			//this.tiles.ctx.drawImage(fimg, 0, 0, fimg.width, fimg.height, cellX + this.cellWidth-fimg.width, cellY, fimg.width, fimg.height);
    		}
    		
    		if(f.hasEstate() && f.estate.gatesClosed)
    		{
    			var fimg = APP.getPreloadImage(KingdomAssets.CLOSED_GATE_ICON);
    			eif.gate_closed = fimg;
    			//this.tiles.ctx.drawImage(fimg, 0, 0, fimg.width, fimg.height, cellX + this.cellWidth-fimg.width, cellY, fimg.width, fimg.height);
    		}
            
            cellIX++;
            eif.cellIX = cellIX;
            eif.color = color;
            
            if(vl.showExtraInfo)
            {
	            /*
	            this.tiles.ctx.fillStyle = "#fff0e0";
	            this.tiles.ctx.font = (20*GAME_IMAGES_PACK) + "px pt-bold";
            	this.tiles.ctx.fillText(cellIX, cellX + 10*GAME_IMAGES_PACK, cellY + 28*GAME_IMAGES_PACK);
            	
            	this.tiles.ctx.font = (18*GAME_IMAGES_PACK) + "px pt-bold";
            	*/
        	}
            
            
            //this.tiles.ctx.fillStyle = color;
            
            eif.cityName = cityName;
            eif.cityOwnerName = cityOwnerName;
            eif.fieldInfoText = fieldInfoText;
            
            extraInfo[i][j] = eif;
            
            /*
            if(cityName)
            {
            	this.drawCellText(this.tiles.ctx, cityName, cellX, cellY + this.cellHeight/4*3, this.cellWidth);
            }
            
            if(cityOwnerName)
            {
            	this.drawCellText(this.tiles.ctx, cityOwnerName, cellX, cellY + this.cellHeight/4*3 + (20*GAME_IMAGES_PACK), this.cellWidth);
            }
            
            if(vl.showExtraInfo && fieldInfoText)
            {
            	this.drawCellText(this.tiles.ctx, fieldInfoText, cellX, cellY + this.cellHeight/2, this.cellWidth);
            }
            */
        }
	}
	
	for (var i = 0; i < FKingdom.HEIGHT; i++)
    {
        for (var j = 0; j < FKingdom.WIDTH; j++)
        {
        	eif = extraInfo[i][j];
        	
        	if(eif.resource) try
        	{
        		this.tiles.ctx.drawImage(eif.resource, 0, 0, eif.resource.width, eif.resource.height, eif.cellX + (this.cellWidth-eif.resource.width)/2, eif.cellY + this.cellHeight/4*3 - eif.resource.height, eif.resource.width, eif.resource.height);
        	} catch (e) {}
        	
        	if(eif.magic) try
        	{
        		this.tiles.ctx.drawImage(eif.magic, 0, 0, eif.magic.width, eif.magic.height, eif.cellX + (this.cellWidth-eif.magic.width)/2, eif.cellY + (this.cellHeight-eif.magic.height)/2 - this.cellHeight/6, eif.magic.width, eif.magic.height);
        	} catch (e) {}
        	
        	if(eif.buff) try
        	{
				this.tiles.ctx.drawImage(eif.buff, 0, 0, eif.buff.width, eif.buff.height, eif.cellX + (this.cellWidth-eif.buff.width)/2, eif.cellY + (this.cellHeight-eif.buff.height)/2 - this.cellHeight/6, eif.buff.width, eif.buff.height);
			} catch (e) {}
			
			if(eif.flag) try
			{
				this.tiles.ctx.drawImage(eif.flag, 0, 0, eif.flag.width, eif.flag.height, eif.cellX + this.cellWidth-eif.flag.width, eif.cellY + eif.flag.height, eif.flag.width, eif.flag.height);
			} catch (e) {}
			
			if(eif.ancients) try
			{
				this.tiles.ctx.drawImage(eif.ancients, 0, 0, eif.ancients.width, eif.ancients.height, eif.cellX + this.cellWidth-eif.ancients.width-16, eif.cellY + 6, eif.ancients.width*1.5, eif.ancients.height*1.5);
			} catch (e) {}
			
			if(eif.gate_closed) try
			{
				this.tiles.ctx.drawImage(eif.gate_closed, 0, 0, eif.gate_closed.width, eif.gate_closed.height, eif.cellX + this.cellWidth - eif.gate_closed.width, eif.cellY + this.cellHeight/2 - eif.gate_closed.height/2, eif.gate_closed.width, eif.gate_closed.height);
			} catch (e) {}
			
			if(vl.showExtraInfo)
            {
            	this.tiles.ctx.fillStyle = "#fff0e0";
				this.tiles.ctx.font = (20*GAME_IMAGES_PACK) + "px pt-bold";
				this.tiles.ctx.lineWidth = 3;
				this.tiles.ctx.strokeStyle = "#ff8000";
				
				this.tiles.ctx.strokeText(eif.cellIX, eif.cellX + 10*GAME_IMAGES_PACK, eif.cellY + 28*GAME_IMAGES_PACK);
				this.tiles.ctx.fillText(eif.cellIX, eif.cellX + 10*GAME_IMAGES_PACK, eif.cellY + 28*GAME_IMAGES_PACK);
				
				this.tiles.ctx.font = (18*GAME_IMAGES_PACK) + "px pt-bold";
			}
			
			this.tiles.ctx.fillStyle = eif.color;
			
			if(eif.cityName)
			{
				this.drawCellText(this.tiles.ctx, eif.cityName, eif.cellX, eif.cellY + this.cellHeight/4*3, this.cellWidth, true);
			}
			
			if(eif.cityOwnerName)
			{
				this.drawCellText(this.tiles.ctx, eif.cityOwnerName, eif.cellX, eif.cellY + this.cellHeight/4*3 + (20*GAME_IMAGES_PACK), this.cellWidth, true);
			}
			
			if(vl.showExtraInfo && eif.fieldInfoText)
			{
				this.drawCellText(this.tiles.ctx, eif.fieldInfoText, eif.cellX, eif.cellY + this.cellHeight/2, this.cellWidth);
			}
        }
	}
	
	APP.clearImagesCache();
	
	//mini map
	img = APP.getPreloadImage(KingdomAssets.MINI_MAP);
	
	var x = this._kingdom.location.x - 5;
	var y = this._kingdom.location.y - 6;
	
	if(x < 0)
	{
		x = 0;
	}
	
	if(y < 0)
	{
		y = 0;
	}
	
	if(x + 10 >= FConfig.EMPIRE_SIZE.x)
	{
		x = FConfig.EMPIRE_SIZE.x - 11;
	}
	
	if(y + 13 >= FConfig.EMPIRE_SIZE.y)
	{
		y = FConfig.EMPIRE_SIZE.y - 13;
	}
	
	//console.log(x, y, FConfig.EMPIRE_SIZE.x, FConfig.EMPIRE_SIZE.y);
	
	try {
		this.miniMapView.ctx.drawImage(img, x*this.miniMapCellSize, y*this.miniMapCellSize, this.miniMapCellSize*11, this.miniMapCellSize*13, 0, 0, this.miniMapCellSize*11, this.miniMapCellSize*13);
	} catch (e) {}
	
	this.miniMapView.ctx.lineWidth = 1;
	
	var cells = this.getActiveCells();
	for(var i=0; i<cells.length; i++)
	{
		this.miniMapView.ctx.fillStyle = cells[i].color;
		this.miniMapView.ctx.fillRect((cells[i].location.x - x)*this.miniMapCellSize+1, (cells[i].location.y - y)*this.miniMapCellSize+1, this.miniMapCellSize-1, this.miniMapCellSize-1);
	}
	
	//current position
	this.miniMapView.ctx.strokeStyle = "#f00";
	this.miniMapView.ctx.strokeRect((this._kingdom.location.x - x)*this.miniMapCellSize, (this._kingdom.location.y - y)*this.miniMapCellSize, this.miniMapCellSize, this.miniMapCellSize);
	
	//big mini map
	try {
		this.miniMapViewBig.ctx.drawImage(img, 0, 0, img.width, img.height, 0, 0, img.width, img.height);
	} catch (e) {}
	
	for(var i=0; i<cells.length; i++)
	{
		this.miniMapViewBig.ctx.fillStyle = cells[i].color;
		this.miniMapViewBig.ctx.fillRect(cells[i].location.x*this.miniMapCellSize+1, cells[i].location.y*this.miniMapCellSize+1, this.miniMapCellSize-1, this.miniMapCellSize-1);
	}
	this.miniMapViewBig.ctx.strokeStyle = "#f00";
	this.miniMapViewBig.ctx.strokeRect(this._kingdom.location.x*this.miniMapCellSize, this._kingdom.location.y*this.miniMapCellSize, this.miniMapCellSize, this.miniMapCellSize);
	
	if(ANDROID_MODE)
	{
		this.tiles.style.display = "none";
		setTimeout($.proxy(function()
		{
			this.tiles.style.display = "block";
		}, this), 10);
	}
	
	//center map
	if(noCenterMap) return;
	if(!this.scroll) return;
	
	//if(!centerTo) centerTo = Math.floor(FKingdom.HEIGHT/2)*FKingdom.WIDTH + Math.floor(FKingdom.WIDTH/2);
	//this.dragToZ(centerTo);
	
	this.centerMap();
}

KingdomView.prototype.centerMap = function()
{
	var centerTo;
	
	for (var i = 0; i < FKingdom.HEIGHT; i++)
    {
        for (var j = 0; j < FKingdom.WIDTH; j++)
        {
        	var f = this._kingdom.getField(j, i);
        	if(f.estate && f.estate.id.equals(this.getEstateId()))
            {
            	centerTo = i*FKingdom.WIDTH + j;
            }
        }
	}
	
	if(!centerTo) centerTo = Math.floor(FKingdom.HEIGHT/2)*FKingdom.WIDTH + Math.floor(FKingdom.WIDTH/2);
	
	if(this.hintField && HintArrow.OPEN_KINGDOM_MENU.isShowing()) 
	{
		centerTo = this.hintField;
	}
	
	this.dragToZ(centerTo);
}

KingdomView.prototype.tick = function()
{
    COM(this.serverTimeLabel).tick();
    this.updateMissionsButton();
    
    /*
    if (FUser.CURRENT_USER.attackWarning) {
        dashboard.showMissions.source = CommonAssets.getPlatform(FlashVars.isSocialNetwork).BUTTON_MISSIONS_RED;
    } else {
        dashboard.showMissions.source = CommonAssets.getPlatform(FlashVars.isSocialNetwork).BUTTON_MISSIONS;
    }
    dashboard.showMissions.text = FUser.CURRENT_USER.currentMissionsCount + '/' + FUser.CURRENT_USER.missionsLimit;
    dashboard.showScience.update(FUser.CURRENT_USER.science);

    if (theBestGame != null) {
        UI.show(Tick.inFuture(FConfig.i.BEST_GAME_END_TICK), [theBestGame], function ():void {
            theBestGame.update();
        });
    }
    if (_kingdom.currentUser.canAttackAnyRuin()) {
        attackAnyRuinTime.text = Tick.toTimeLeft(_kingdom.currentUser.canAttackAnyRuinTill - Tick.now());
    }
    */
}

KingdomView.prototype.updateEstateArrows = function()
{
    if(!APP.currentScreenIsKingdom()) return;
    
    var estateHint = FUser.CURRENT_USER.findHint(HintType.FIND_OTHER_ESTATE);

    if (estateHint && this._kingdom)
    {
        var otherField = this.findOtherEstate();
        if (otherField < 0)
        {
            HintArrow.OPEN_KINGDOM_MENU.hide();
            HintArrow.NAVIGATE_KINGDOM.show(navigator.navigateNorth, estateHint, this.DOM);
        }
        else
        {
            HintArrow.NAVIGATE_KINGDOM.understood();
            if (HintArrow.OPEN_KINGDOM_MENU.canShow(estateHint))
            {
                this.dragToZ(otherField);
                var c = this.getCellCoordsByZ(otherField);
                HintArrow.OPEN_KINGDOM_MENU.show(null, estateHint, this.layersContainer, {
					left: c.x,
					top: c.y,
					width: this.cellWidth,
					height: this.cellHeight
				});
				
				this.hintField = otherField;
            }
        }
    }
}

KingdomView.prototype.dragToZ = function(z)
{
	var scale = (this.scroll && this.scroll.scale) ? this.scroll.scale : 1;
	
	var maxScrollX = $(this.layersWrapper).width() - $(this.layersContainer).width() * scale;
	var maxScrollY = $(this.layersWrapper).height() - $(this.layersContainer).height() * scale;
	
	var c = this.getCellCoordsByZ(z);
	var cellX = c.x + this.cellWidth/2;
	var cellY = c.y + this.cellHeight/2;
	
	cellX -= $(this.layersWrapper).width() / scale / 2;
	cellY -= $(this.layersWrapper).height() / scale / 2;
	
	cellX *= scale;
	cellY *= scale;
	
	cellX *= -1;
	cellY *= -1;
	
	if(cellX > 0)
	{
		cellX = 0;
	}
	
	if(cellX < maxScrollX)
	{
		cellX = maxScrollX;
	}
	
	if(cellY > 0)
	{
		cellY = 0;
	}
	
	if(cellY < maxScrollY)
	{
		cellY = maxScrollY;
	}
	
	if(this.scroll) this.scroll.scrollTo(cellX, cellY, 0);
}

KingdomView.prototype.getCellCoordsByZ = function(z)
{
    var xy = FKingdom.getFieldLocationByZ(z);
    var cellX = (xy.x+1)*this.cellWidth + this.cellsPadding.left;
	var cellY = (xy.y+1)*this.cellHeight + this.cellsPadding.top;
	
	return {x: cellX, y: cellY};
}

KingdomView.prototype.findOtherEstate = function()
{
    var applicable = [];
    var array, f;
    for(var i=0; i<this._kingdom.fields.length; i++)
    {
    	array = this._kingdom.fields[i];
    	for(var n=0; n<array.length; n++)
    	{
    		f = array[n];
    		if (f.estate && f.estate.__className__ == "FKingdomCity" && !f.estate.owner.isOnVacation() && !FUser.isCurrentUser(f.estate.owner.username))
    		{
                applicable.push(f.z);
            }
    	}
    }
    return applicable.length > 0 ? applicable[Math.floor(Math.random() * applicable.length)] : -1;
}

KingdomView.prototype.updateArrows = function()
{
    if(!APP.currentScreenIsKingdom()) return;
    
    var cityHint = FUser.CURRENT_USER.findHint(HintType.CONSTRUCT_FIRST_BUILDING);
    cityHint = !cityHint ? FUser.CURRENT_USER.findHint(HintType.UPGRADE_HALL) : cityHint;
	cityHint = !cityHint ? FUser.CURRENT_USER.findHint(HintType.GET_REWARD) : cityHint;
	cityHint = !cityHint ? FUser.CURRENT_USER.findHint(HintType.READ_QUEST) : cityHint;
    
    if (cityHint)
    {
        HintArrow.OPEN_CITY.show(this.navigationButton, cityHint, this.DOM);
    }
}