function ShareOuterEstateDialog()
{
	callParentConstructor(ShareOuterEstateDialog.prototype, this);
	
	this.location = null;
	
	this.onCheckBoxClicked = $.proxy(this.onCheckBoxClicked, this);
	this.onPlayerChange = $.proxy(this.onPlayerChange, this);
}

extendClass(ShareOuterEstateDialog.prototype, Dialog.prototype);

ShareOuterEstateDialog.prototype.__construct = function()
{
	COM.bindExclusiveClick(this.toClan, this.onCheckBoxClicked);
	COM.bindExclusiveClick(this.toApprentices, this.onCheckBoxClicked);
	$(this.player).on("change", this.onPlayerChange);
	
	COM.bindExclusiveClick(this.okButton, $.proxy(this.onShare, this));
	
	this.setCaption(I18.f("SHARE_OUTER_ESTATE_WITH_PLAYER"));
	
	this.comment.maxlength = Math.max(FConfig.MAX_MAIL_COMMENT_LENGTH, 60);
}

ShareOuterEstateDialog.prototype.popup = function(parent, location)
{
    this.location = location;
    superMethod(ShareOuterEstateDialog.prototype, "popup", this)();
    
    UI.show(FUser.CURRENT_USER.isInClan(), [this.toClan]);
    UI.show(FUser.CURRENT_USER.diplomacy.isMentor, [this.toApprentices]);
    UI.show(false, [this.warningText]);
    
    this.onPlayerChange();
}

ShareOuterEstateDialog.prototype.handleSuccess = function(ignore)
{
    this.close();
    this.showDoneMessage(COM(this.toClan).checked ?
            I18.f("SHARE_OUTER_ESTATE_TO_CLAN_DONE", this.location) :
            I18.f("SHARE_OUTER_ESTATE_DONE", [COM(this.player).value, this.location]));
}

ShareOuterEstateDialog.prototype.handleFail = function(e)
{
    if (GameException.isDisplayable(e))
    {
        //this.warningText.innerHTML = e.getMessage();
        //UI.show(true, [this.warningText]);
        Alert.create(e.getMessage());
        return true;
    }
    return false;
}

ShareOuterEstateDialog.prototype.onPlayerChange = function()
{
	var val = COM(this.player).value;
	//COM(this.okButton).enabled = val && val.length > 0;
	COM(this.okButton).reason = (val && val.length > 0) ? null : I18.f("SELECT_ADDRESSEE");
}

ShareOuterEstateDialog.prototype.showDoneMessage = function(txt)
{
	txt = txt.split("[").join("<br/>[");
    Alert.create(txt);
}

ShareOuterEstateDialog.prototype.onShare = function()
{
    if (COM(this.toClan).checked)
    {
        Service.kingdom.callMethod("shareOuterEstateToClan", [this.location, this.comment.value],
	        $.proxy(function()
	        {
	            this.showDoneMessage(I18.f("SHARE_OUTER_ESTATE_TO_CLAN_DONE", this.location.toString()));
	            this.close();
	        }, this),
	        $.proxy(this.handleFail, this));
    }
    else if (COM(this.toApprentices).checked)
    {
        Service.kingdom.callMethod("shareOuterEstateToApprentices", [this.location, this.comment.value],
	        $.proxy(function(ignore)
	        {
	            this.showDoneMessage(I18.f("SHARE_OUTER_ESTATE_TO_APPRENTICES_DONE", this.location.toString()));
	            this.close();
	        }, this),
	        $.proxy(this.handleFail, this));
    }
    else
    {
        var usernames = FUser.splitNames(COM(this.player).value);
        
        Service.kingdom.callMethod("shareOuterEstateToUsers", [this.location, usernames, this.comment.value],
	        $.proxy(function(ignore)
	        {
				this.showDoneMessage(I18.mega("SHARE_OUTER_ESTATE_DONE", [usernames, this.location.toString()]));
	            this.close();
	        }, this),
	        $.proxy(this.handleFail, this));
    }
}

ShareOuterEstateDialog.prototype.onCheckBoxClicked = function(e)
{
    var checkbox = e.currentTarget;

    COM(this.toClan).checked = e.currentTarget == this.toClan && COM(checkbox).checked;
    COM(this.toApprentices).checked = e.currentTarget == this.toApprentices && COM(checkbox).checked;
    
    if (COM(this.toClan).checked)
    {
		COM(this.player).value = I18.f("CLAN");
        COM(this.player).disabled = true;
    }
    else if (COM(this.toApprentices).checked)
    {
        COM(this.player).value = I18.f("APPRENTICES");
        COM(this.player).disabled = true;
    }
    else
    {
        COM(this.player).value = "";
        COM(this.player).disabled = false;
    }
    
    this.onPlayerChange();
}