function CombatReportDialog()
{
	callParentConstructor(CombatReportDialog.prototype, this);
	this.scroll = null;
	
	this.bold = "text-bold";
	this.bold12p = "text-bold12p";
	
}

extendClass(CombatReportDialog.prototype, Dialog.prototype);

CombatReportDialog.prototype.__construct = function()
{
	this.scroll = new iScroll(this.windowWrapper, {scrollbarColor: '#784f0b'});
}

CombatReportDialog.prototype.fitLayout = function(container)
{
	superMethod(CombatReportDialog.prototype, "fitLayout", this)(container);
	
	this.windowWrapper.style.width = this.content.offsetWidth + "px";
	this.windowWrapper.style.height = (this.content.offsetHeight - 8) + "px";
}

CombatReportDialog.prototype.popup = function(message, details)
{
	superMethod(CombatReportDialog.prototype, "popup", this)();
	
	this.fullScreen();		
		
	this.setCaption(I18.f("DETAILS"));

    var n = 0;
    var prev = null;
    
    for (var i=0; i<details.rounds.length; i++)
    {    	
    	var round = details.rounds[i];
        var d = prev == null ? 0 : CombatReportDialog.diff(round.attackers, prev.attackers) + CombatReportDialog.diff(round.defenders, prev.defenders);
        
        switch (round.type.name) 
        {
            case RoundType.INITIAL.name:
            	if (details.missions != null)
                {
                    for (var j=0; j<details.missions.length; j++)
                    {
                    	var mr = details.missions[j];

                        this.rounds.appendChild(I18.box("MISSION_REPORT", FUser.getUserNameForMessages(mr.owner),
                                I18.formatEstateName(mr.fromEstateName, mr.fromField), mr.startTick));
                    }
                }                
                break;
            case RoundType.PARTIAL_RETREAT.name:
                if (d > 0) 
                {
                    CombatReportDialog.showPartialRetreatRound(this.rounds, message, round, prev);
                }
                break;
            case RoundType.FORTIFICATIONS.name:
                CombatReportDialog.showFortificationsRound(this.rounds, message, round, prev);
                break;
            case RoundType.FIGHT.name:
                n += 1;
                CombatReportDialog.showFight(this.rounds, n, message, round, prev);
                break;
            case RoundType.BREAK.name:
                CombatReportDialog.showResultMessage(this.rounds, message, n, details);
                break;
            case RoundType.HEAL.name:
                if (d > 0) 
                {
                    CombatReportDialog.showHealing(this.rounds, message, round, prev);
                }
                break;
            case RoundType.REJOIN.name:
                if (d > 0) 
                {
                    CombatReportDialog.showRejoin(this.rounds, message, round, prev);
                }
                break;
            case RoundType.NECROMANCY.name:
                if (d > 0) 
                {
                    CombatReportDialog.showNecromancy(this.rounds, message, round, prev);
                }
                break;
            default:
                throw Error("Unknown round type");
        }
        prev = round;
    }
    
    setTimeout($.proxy(this.scroll.refresh, this.scroll), 300);
}

CombatReportDialog.diff = function(now/*:Array*/, before/*:Array*/)
{
    if (now == null) 
    {
        return 0;
    }
    if (before == null) 
    {
        return 0;
    }
    var res = 0;
    for (var i = 0; i < now.length; i++) 
    {
        var nowA = now[i];
        var beforeA = before[i];
        if ((nowA != null) && (beforeA != null)) 
        {
            for (var j = 0; j < nowA.result.length; j++) 
            {
                res += Math.abs(nowA.result[j] - beforeA.result[j]);
            }
        }
    }
    return res;
}

CombatReportDialog.addSpacer = function(parent)
{
	var p = document.createElement("p");
    parent.appendChild(p);    
}

CombatReportDialog.addLabel = function(parent, text, style)
{
    var label = document.createElement("div");
  	$(label).addClass("text-"+style);
    label.innerHTML = Strings.safeHtml(text);
    parent.appendChild(label);
}

CombatReportDialog.showArmy = function(parent, summary, current, prev, sign, showAttack, round, tick)
{
	var a = CombatViewUtils.createArmyViewX(summary, current, prev, sign, showAttack, round, tick);
	parent.appendChild(a.DOM);
}

CombatReportDialog.showTowers = function(parent, message, round)
{
	parent.appendChild(CombatViewUtils.createTowersBoxForRound(message, round));
}

CombatReportDialog.showGate = function(parent, message, round, prev)
{
    parent.appendChild(CombatViewUtils.createGateBoxForRound(message, round, prev));
}

CombatReportDialog.showUnknownArmy = function(parent)
{
	var unknownArmy = COM.create("ArmyView");
   	unknownArmy.setAccessLevelRequired(6);
    parent.appendChild(unknownArmy.DOM);
}

CombatReportDialog.showUnknownTowers = function(parent)
{
    var unknownTowers = CombatViewUtils.towerBox(false);
    unknownTowers.appendChild(TowerView.createEmpty(false).DOM);
    unknownTowers.appendChild(TowerView.createEmpty(false).DOM);
    parent.appendChild(unknownTowers);
}

CombatReportDialog.retreatMessage = function(isYours, summary)
{
    var cityName = I18.formatEstateName(summary.fromEstateName, summary.fromFieldXYZ);
    if (isYours) 
    {
        if (summary.lossesPercent == 100) 
        {
            return I18.f("YOUR_ARMY_IS_DEAD",cityName);
        } 
        else 
        {
            return I18.f("YOUR_ARMY_RETREATS",cityName, summary.lossesPercent);
        }
    } 
    else 
    {
        var username = FUser.getUserNameForMessages(summary.username);
        if (summary.lossesPercent == 100) 
        {
            return I18.f("PLAYER_ARMY_IS_DEAD",username, cityName);
        } 
        else 
        {
            return I18.f("PLAYER_ARMY_RETREATS",username, cityName, summary.lossesPercent);
        }
    }
}

CombatReportDialog.showRetreatInfo = function(parent, message, index)
{
    for (var i=0; i<message.armiesInfo.length; i++)
    {
    	var summary = message.armiesInfo[i];
        if (summary.retreatRound == index) 
        {
            CombatReportDialog.addLabel(parent, CombatReportDialog.retreatMessage(!message.isForwarded && FUser.isSameName(message.receiver, summary.username), summary));
        }
    }
}

CombatReportDialog.showResultMessage = function(parent, message, total, details)
{
    if (message.roundsLimitReached) 
    {
        CombatReportDialog.addLabel(parent, I18.f("COMBAT_ROUNDS_LIMIT_REACHED"), "bold12p");
    }
    var result = "";
    if (message.receiverWonCombat()) 
    {
        // if only fortification and healing rounds are present
        if (total == 0) 
        {
            result = I18.f("COMBAT_DETAILS_YOU_DOMINATING");
        } 
        else 
        {
            if (message.defenderVisible) 
            {
                result = I18.f("COMBAT_DETAILS_YOU_WON",total);
            } 
            else 
            {
                result = I18.f("COMBAT_DETAILS_YOU_DOMINATING_RATIO", FConfig.BATTLE_DOMINANCE_RATIO);
            }
        }
    } 
    else if (message.receiverLoseCombat()) 
    {
        if (total == 0) 
        {
            if (FArmyRound.everybodyDies(details.getFortificationsRound().attackers)) 
            {
                result = I18.f("COMBAT_DETAILS_YOU_LOSE_ON_FORTIFICATIONS");
            }
        } 
        else 
        {
            if (message.defenderVisible)
            {
                result = I18.f("COMBAT_DETAILS_YOU_LOSE",total);
            } 
            else 
            {
                result = I18.f("COMBAT_DETAILS_ENEMY_DOMINATING_RATIO", FConfig.BATTLE_DOMINANCE_RATIO);
            }
        }
    } 
    else 
    {
		if (total == 0)
        {
            result = I18.f("COMBAT_DETAILS_DEFENDERS_DOMINATING");
        } 
        else 
        {
            if (message.defenderVisible) 
            {
                if (message.result.name == MissionResult.SUCCESS.name) 
                {
                    result = I18.f("COMBAT_DETAILS_ATTEKERS_WON",total);
                } 
                else 
                {
                    result = I18.f("COMBAT_DETAILS_DEFENDERS_WON",total);
                }
            } 
            else 
            {
                result = I18.f("COMBAT_DETAILS_DEFENDERS_DOMINATING", FConfig.BATTLE_DOMINANCE_RATIO);
            }
        }
    }
    CombatReportDialog.addLabel(parent, result, "bold12p");
}

CombatReportDialog.addUserName = function(parent, message, username, isAttacker)
{
    var d = CombatViewUtils.messageBodyLabel(message, Strings.safeHtml(FUser.getUserNameForMessages(username)), isAttacker);
    d.style.display = "block";
    d.style.width = "40%";
    d.style.margin = "auto";
    parent.appendChild(d);
}

CombatReportDialog.showFight = function(parent, n, message, round, prev)
{
    CombatReportDialog.addLabel(parent, I18.f("ROUND") + " " + n,"bold");
    CombatReportDialog.showFightArmies(parent, message, round, n, prev, true);
    if (prev.gate != 0) 
    {
        CombatReportDialog.showFightGates(parent, message, round, prev);
    } 
    else 
    {
        CombatReportDialog.showFightArmies(parent, message, round, n, prev, false);
    }
    CombatReportDialog.showRetreatInfo(parent, message, n);
    CombatReportDialog.addSpacer(parent);
}

CombatReportDialog.showFightArmies = function(parent, message, round, n, prev, isAttacker)
{
    var summaries = isAttacker ? message.attackersSummary : message.defendersSummary;
    var armies = isAttacker ? round.attackers : round.defenders;
    var initial = isAttacker ? prev.attackers : prev.defenders;
    for (var i = 0; i < summaries.length; i++) 
    {
        var summary = summaries[i];
        if (summary == null) 
        {
            continue;
        }
        for (var j=0; j<message.armiesInfo.length; j++)
        {
        	s = message.armiesInfo[j];
            if (s.username == summary.username) 
            {
                if ((s.retreatRound != -1) && (s.retreatRound < n)) 
                {
                    summary = null;
                    break;
                }
            }
        }
        if (summary == null) 
        {
            continue;
        }
        var host = !isAttacker && i == 0;
        if (armies != null) 
        {
            var army = armies[i];
            var before = initial[i];
            if (!ArmyUtils.isEmpty(before.result) || (host && message.cityHasTowers())) 
            {
                CombatReportDialog.addUserName(parent, message, summary.username, isAttacker);
                CombatReportDialog.showArmy(parent, summary, army, before, "-", true, n, message.tick);
                if (host && message.cityHasTowers()) 
                {
                	var children = parent.childNodes;
                	CombatReportDialog.showTowers(children[children.length - 1], message, round);
                }
            }
        } 
        else 
        {
            CombatReportDialog.addUserName(parent, message, summary.username, isAttacker);
            CombatReportDialog.showUnknownArmy(parent);
            if (host) 
            {
            	var children = parent.childNodes;
                CombatReportDialog.showUnknownTowers(children[children.length - 1]);
            }
        }
    }
}

CombatReportDialog.showFightGates = function(parent, message, round, prev)
{
    var summaries = message.defendersSummary;
    for (var i = 0; i < 1; i++) 
    {
        var summary = summaries[i];
        if (summary == null) 
        {
            continue;
        }
        if (round.defenders != null) 
        {
            CombatReportDialog.addUserName(parent, message, summary.username, false);
            CombatReportDialog.showGate(parent, message, round, prev);
            if (message.cityHasTowers()) 
            {
            	var children = parent.childNodes;
                CombatReportDialog.showTowers(children[children.length - 1], message, round);
            }
        } 
        else 
        {
            CombatReportDialog.addUserName(parent, message, summary.username, false);
            CombatReportDialog.showUnknownArmy(parent);
            var children = parent.childNodes;
            CombatReportDialog.showUnknownTowers(children[children.length - 1]);
        }
    }
}

CombatReportDialog.gatherAdditionalFortificationLossesReasons = function(armyOwnerSummary)
{
    var casts = [];
 
    for (var i=0; i<armyOwnerSummary.length; i++)
    {
    	var faos = armyOwnerSummary[i];
        for (var j=0; j<faos.buffs.length; j++)
        
        {
        	var buff = faos.buffs[j];
            if (buff.buffType == BuffType.ADDITIONAL_FORTIFICATION_KILLS) 
            {
                casts.push(I18.getBuffCause(buff, "_R"));
            }
            if (buff.buffType == BuffType.EARTHQUAKE) 
            {
                casts.push(I18.getBuffCause(buff, "_R"));
            }
        }
    }
    return casts;
}

CombatReportDialog.gatherAdditionalFortificationLossesCount = function(armyOwnerSummary)
{
    var count = 0;
    for (var i=0; i<armyOwnerSummary.length; i++)
    {
    	var faos = armyOwnerSummary[i];
        for (var j=0; j<faos.buffs.length; j++)
        {
        	var buff = faos.buffs[j];
            if (buff.buffType == BuffType.ADDITIONAL_FORTIFICATION_KILLS) 
            {
                count += buff.value;
            }
        }
    }
    return count;
}

CombatReportDialog.gatherEarthquakeLossesCount = function(armyOwnerSummary)
{
    var count = 0;
    for (var i=0; i<armyOwnerSummary.length; i++)
    {
    	var faos = armyOwnerSummary[i];
        for (var j=0; j<faos.buffs.length; j++)
        {
        	var buff = faos.buffs[j];
            if (buff.buffType == BuffType.EARTHQUAKE) 
            {
                count += buff.value;
            }
        }
    }
    return count;
}

CombatReportDialog.showFortificationsRound = function(parent, message, round, prev)
{
    CombatReportDialog.addLabel(parent, I18.f("FORTIFICATIONS_ROUND"), "bold");

    var totalAttackersDied = CombatReportDialog.diff(round.attackers, prev.attackers);
    var additionalAttackersDied = Math.min(CombatReportDialog.gatherAdditionalFortificationLossesCount(message.defendersSummary) + CombatReportDialog.gatherEarthquakeLossesCount(message.attackersSummary), totalAttackersDied);
    if (additionalAttackersDied > 0) 
    {
        CombatReportDialog.addLabel(parent, I18.f("COMBAT_DETAILS_LOST_ON_FORTIFICATIONS_ATTACKERS_ADDITIONAL",additionalAttackersDied, CombatReportDialog.gatherAdditionalFortificationLossesReasons(message.defendersSummary)));
    }
    CombatReportDialog.addLabel(parent, I18.f("COMBAT_DETAILS_LOST_ON_FORTIFICATIONS_ATTACKERS",totalAttackersDied - additionalAttackersDied));
    CombatReportDialog.showFortificationsArmies(parent, message, round, prev, true);

    if (CombatReportDialog.diff(round.defenders, prev.defenders) > 0) 
    {
        CombatReportDialog.addLabel(parent, I18.f("COMBAT_DETAILS_LOST_ON_FORTIFICATIONS_DEFENDERS",CombatReportDialog.diff(round.defenders, prev.defenders), CombatReportDialog.gatherAdditionalFortificationLossesReasons(message.attackersSummary).join(", ")));
        CombatReportDialog.showFortificationsArmies(parent, message, round, prev, false);
    }
    CombatReportDialog.showRetreatInfo(parent, message, 0);
    CombatReportDialog.addSpacer(parent);
}

CombatReportDialog.showFortificationsArmies = function(parent, message, round, prev, isAttacker)
{
    var summaries = isAttacker ? message.attackersSummary : message.defendersSummary;
    var armies = isAttacker ? round.attackers : round.defenders;
    var initial = isAttacker ? prev.attackers : prev.defenders;
    for (var i = 0; i < summaries.length; i++) 
    {
        var summary = summaries[i];
        if (summary != null) 
        {
            if (armies != null) 
            {
                var army = armies[i];
                var before = initial[i];
                if (!ArmyUtils.isEmpty(before.result)) 
                {
                    CombatReportDialog.addUserName(parent, message, summary.username, isAttacker);
                    CombatReportDialog.showArmy(parent, summary, army, before, "-", false, 0, message.tick);
                }
            } 
            else 
            {
                CombatReportDialog.addUserName(parent, message, summary.username, isAttacker);
                CombatReportDialog.showUnknownArmy(parent);
            }
        }
    }
}




CombatReportDialog.showHealing = function(parent, message, round, prev)
{
    CombatReportDialog.addLabel(parent, I18.f("HEALING_ROUND"), "bold");

    if (CombatReportDialog.diff(round.attackers, prev.attackers) > 0) 
    {
        CombatReportDialog.showHealingArmies(parent, message, round, prev, true);
    }
    if (CombatReportDialog.diff(round.defenders, prev.defenders) > 0) 
    {
        CombatReportDialog.showHealingArmies(parent, message, round, prev, false);
    }
    CombatReportDialog.addSpacer(parent);
}




CombatReportDialog.showHealingArmies = function(parent, message, round, prev, isAttacker)
{
    var summaries = isAttacker ? message.attackersSummary : message.defendersSummary;
    var armies = isAttacker ? round.attackers : round.defenders;
    var initial = isAttacker ? prev.attackers : prev.defenders;
    var healings = isAttacker ? message.attackersHealing : message.defendersHealing;
    for (var i = 0; i < summaries.length; i++) 
    {
        var summary = summaries[i];
        if (summary != null) 
        {
            if (armies != null) 
            {
                var army = armies[i];
                var before = initial[i];
                var d = ArmyUtils.diff(army, before);
                if (d > 0) 
                {
                    for (var j = 0; j < summaries.length; j++) 
                    {
                        if (healings[j][i] > 0) 
                        {
                            if (!message.isForwarded && FUser.isSameName(summaries[j].username, message.receiver)) 
                            {
                                CombatReportDialog.addLabel(parent, I18.f("COMBAT_DETAILS_HEALERS_HEALING_PLAYER",healings[j][i]));
                            } 
                            else 
                            {
                                CombatReportDialog.addLabel(parent, I18.f("COMBAT_DETAILS_HEALERS_HEALING",FUser.getUserNameForMessages(summaries[j].username), healings[j][i]));
                            }
                        }
                    }
                    CombatReportDialog.addUserName(parent, message, summary.username, isAttacker);
                    CombatReportDialog.showArmy(parent, summary, army, before, "+", false, 0, message.tick);
                }
            }
        }
    }
}



CombatReportDialog.showPartialRetreatRound = function(parent, message, round, prev)
{
    CombatReportDialog.addLabel(parent, I18.f("PARTIAL_RETREAT_ROUND"), "bold");

    if (CombatReportDialog.diff(round.attackers, prev.attackers) > 0) 
    {
        CombatReportDialog.showPartialRetreatedArmies(parent, message, round, prev, true);
    }
    if (CombatReportDialog.diff(round.defenders, prev.defenders) > 0) 
    {
        CombatReportDialog.showPartialRetreatedArmies(parent, message, round, prev, false);
    }
    CombatReportDialog.addSpacer(parent);
}

CombatReportDialog.showPartialRetreatedArmies = function(parent, message, round, prev, isAttacker)
{
    var summaries = isAttacker ? message.attackersSummary : message.defendersSummary;
    var armies = isAttacker ? round.attackers : round.defenders;
    var initial = isAttacker ? prev.attackers : prev.defenders;
    for (var i = 0; i < summaries.length; i++) 
    {
        var summary = summaries[i];
        if (summary != null) 
        {
            if (armies != null) 
            {
                var army = armies[i];
                var before = initial[i];
                var d = ArmyUtils.diff(army, before);
                if (d > 0) 
                {
                    CombatReportDialog.addUserName(parent, message, summary.username, isAttacker);
                    CombatReportDialog.showArmy(parent, summary, army, before, "-", false, 0, message.tick);
                }
            }
        }
    }
}



CombatReportDialog.showRejoin = function(parent, message, round, prev)
{
    CombatReportDialog.addLabel(parent, I18.f("REJOIN_ROUND"), "bold");

    if (CombatReportDialog.diff(round.attackers, prev.attackers) > 0) 
    {
        CombatReportDialog.showRejoinArmies(parent, message, round, prev, true);
    }
    if (CombatReportDialog.diff(round.defenders, prev.defenders) > 0) 
    {
        CombatReportDialog.showRejoinArmies(parent, message, round, prev, false);
    }
    CombatReportDialog.addSpacer(parent);
}

CombatReportDialog.showRejoinArmies = function(parent, message, round, prev, isAttacker)
{
    var summaries = isAttacker ? message.attackersSummary : message.defendersSummary;
    var armies = isAttacker ? round.attackers : round.defenders;
    var initial = isAttacker ? prev.attackers : prev.defenders;
    for (var i = 0; i < summaries.length; i++) 
    {
        var summary = summaries[i];
        if (summary != null) 
        {
            if (armies != null) 
            {
                var army = armies[i];
                var before = initial[i];
                var d = ArmyUtils.diff(army, before);
                if (d > 0) 
                {
                    CombatReportDialog.addUserName(parent, message, summary.username, isAttacker);
                    CombatReportDialog.showArmy(parent, summary, army, before, "+", false, 0, message.tick);
                }
            }
        }
    }
}

CombatReportDialog.showNecromancy = function(parent, message, round, prev)
{
    CombatReportDialog.addLabel(parent, I18.f("NECROMANCY_ROUND"), "bold");

    if (CombatReportDialog.diff(round.attackers, prev.attackers) > 0) 
    {
        CombatReportDialog.showRejoinArmies(parent, message, round, prev, true);
    }
    if (CombatReportDialog.diff(round.defenders, prev.defenders) > 0) 
    {
        CombatReportDialog.showRejoinArmies(parent, message, round, prev, false);
    }
    CombatReportDialog.addSpacer(parent);
}
