function CombatViewUtils()
{
}

CombatViewUtils.createArmyView = function(summary, forces, time)
{
    var result = COM.create("ArmyListView");
    result.setUnits(summary.race, summary.levels, FBuff.hasBuff(summary.buffs, BuffType.MONSTER_RACE, null, time),
            new UnitAura(summary.bonuses), summary.buffs);
    result.setHero(summary.hero, summary.race);
    result.setFrozenTime(time);
    result.setCountInitial(forces.result, forces.initial, true, true);
    result.showAllUnitsCount();
    result.hideUnitsCountFake();
    result.showAsReport();
    return result;
}

CombatViewUtils.createArmyViewX = function(summary, forces, prev, sign, showSelectedAttack, round, tick)
{
    var result = COM.create("ArmyListView");
    result.setUnits(summary.race, summary.levels, FBuff.hasBuff(summary.buffs, BuffType.MONSTER_RACE, null, tick), new UnitAura(summary.bonuses), summary.buffs, round);
    result.setHero(summary.hero, summary.race);
    result.setFrozenTime(tick);
    result.setCountInitial(forces.result, prev.result, sign == "-", sign != "-");
    result.showAllUnitsCount();
    result.hideUnitsCountFake();
    if (forces.selectedAttack && showSelectedAttack)
    {
        result.setSelectedAttack(forces.selectedAttack);
    }
    return result;
}

CombatViewUtils.createTowersBox = function(message)
{
    var miniTowers = false;
    var box = CombatViewUtils.towerBox(miniTowers);

    var summary = message.defendersSummary[0];
    var garrisonAttack = CombatBonusUtil.getGarrisonAttackForCombatReport(summary.race, summary.levels, summary.bonuses, message.defendersForces[0].initial, summary.buffs, message.tick);

    var bt;
    if (message.firstTowerLevel > 0 || message.secondTowerLevel > 0 && miniTowers)
    {
        bt = message.firstTowerLevel > 0 ? BType.byId(message.firstTowerTypeId) : null;
        box.appendChild(CombatViewUtils.createTower(bt, message.firstTowerLevel, summary.race, garrisonAttack,
                message.firstTowerBonus, CombatViewUtils.getTowerTotalBonusPercent(bt, message), miniTowers).DOM);
    }
    if (message.secondTowerLevel > 0 || message.firstTowerLevel > 0 && miniTowers)
    {
        bt = message.secondTowerLevel > 0 ? BType.byId(message.secondTowerTypeId) : null;
        box.appendChild(CombatViewUtils.createTower(bt, message.secondTowerLevel, summary.race, garrisonAttack,
                message.secondTowerBonus, CombatViewUtils.getTowerTotalBonusPercent(bt, message), miniTowers).DOM);
    }

    return box;
}

CombatViewUtils.createTower = function(bt, level, r, garrisonAttack, bonusConst, bonusPercent, mini)
{
    var attack = new Range();
    if (level > 0)
    {
        var blt = bt.getLevels(r)[level];
        if (bt.name == BType.TOWER.name)
        {
            attack = blt.attack.copy();
        }
        else
        {
            attack = NumberUtils.floor(blt.garrisonAttackBonus * garrisonAttack);
        }
    }
    return TowerView.create(r, bt, level, attack, bonusPercent, bonusConst, mini);
}

CombatViewUtils.getTowerTotalBonusPercent = function(bt, message)
{
    return (bt.name == BType.TOWER.name) ? message.towerBuffBonus : message.magicTowerBuffBonus;
}

CombatViewUtils.towerBox = function(miniTowers)
{
    /*
    var box = miniTowers ? new VBox() : new HBox();
    if (miniTowers) {
        box.setStyle("verticalGap", "1");
    } else {
        box.setStyle("horizontalGap", "6");
    }
    */
	var box = document.createElement("div");
	box.style.textAlign = "center";
    return box;
}

CombatViewUtils.createTowersBoxForRound = function(message, round)
{
    var miniTowers = false;
    var box = CombatViewUtils.towerBox(miniTowers);
    var r = message.defendersSummary[0].race;
    var stats;

    var bt;
    if (message.firstTowerLevel > 0 || message.secondTowerLevel > 0 && miniTowers)
    {
        bt = message.firstTowerLevel > 0 ? BType.byId(message.firstTowerTypeId) : null;
        stats = round.towers[0] ? round.towers[0] : new TowerInRound();
        if (stats.level > 0)
        {
            box.appendChild(TowerView.create(r, bt, stats.level, stats.baseAttack, stats.bonusPercent, stats.bonusConst, miniTowers).DOM);
        }
    }
    if (message.secondTowerLevel > 0 || message.firstTowerLevel > 0 && miniTowers)
    {
        bt = message.secondTowerLevel > 0 ? BType.byId(message.secondTowerTypeId) : null;
        stats = round.towers[1] ? round.towers[1] : new TowerInRound();
        if (stats.level > 0)
        {
            box.appendChild(TowerView.create(r, bt, stats.level, stats.baseAttack, stats.bonusPercent, stats.bonusConst, miniTowers).DOM);
        }
    }

    return box;
}

CombatViewUtils.createGateBoxForRound = function(message, round, prev)
{
    return CastleGateView.create(message, prev.gate, round.gate, true).DOM;
}

CombatViewUtils.armyName = function(message, username)
{
    return (!message.isForwarded && FUser.isSameName(username, message.receiver))
            ? I18.f("YOUR_ARMY")
            : I18.f("OPPONENT_ARMY", FUser.getUserNameForMessages(username));
}

CombatViewUtils.heroName = function(message, username, heroname)
{
    return (!message.isForwarded && FUser.isSameName(username, message.receiver))
            ? I18.f("YOUR_HERO", heroname)
            : I18.f("OPPONENT_HERO", heroname, FUser.getUserNameForMessages(username));
}

CombatViewUtils.messageBodyLabel = function(message, username, attacker)
{
    return CombatViewUtils.createLabelWithColor(CombatViewUtils.armyName(message, username), CombatViewUtils.getBattleResultColor(message, attacker));
}

CombatViewUtils.getBattleResultColor = function(message, attacker, defaultColor)
{
    if(typeof defaultColor == "undefined")
    {
    	defaultColor = "#fff";
    }
    
    switch (message.result.name)
    {
        case MissionResult.SUCCESS.name:
            return CombatViewUtils.getBattleStatusColor(attacker, true);
        case MissionResult.DEFEAT.name:
            return CombatViewUtils.getBattleStatusColor(!attacker, true);
    }
    return defaultColor;
}

CombatViewUtils.getBattleStatusColor = function(youWon, youFight)
{
    return youFight ? (youWon ? Effects.LIGHT_GREEN : Effects.LIGHT_RED) : Effects.LIGHT_ORANGE;
}

CombatViewUtils.createLabelWithColor = function(text, color)
{
    var res = document.createElement("span");
    res.innerHTML = Strings.safeHtml(text);
    res.style.backgroundColor = color;
    return res;
}

CombatViewUtils.createArmyOwnerDescription = function(message, username, attacker)
{
    var mainXYZ = attacker ? message.fromEstateXYZ : message.toFieldXYZ;
    var color = CombatViewUtils.getBattleResultColor(message, attacker);
    var cities = [];

    var summary;
    for(var i=0; i<message.armiesInfo.length; i++)
    {
    	summary = message.armiesInfo[i];

    	if (summary.username == username && (!FUser.isHomelessMonster(username) || !mainXYZ || mainXYZ.equals(summary.fromFieldXYZ)))
    	{
            var from = I18.formatEstateName(summary.fromEstateName, summary.fromFieldXYZ);
            var ok = true;
            for(var j=0; j<cities.length; j++)
            {
            	if(cities[j] == from)
            	{
            		ok = false;
            		break;
            	}
            }
            if(ok)
            {
            	cities.push(from);
            }
        }
    }

    var text = "";
    var ownMessage = !message.isForwarded && FUser.isSameName(message.receiver, username);
    var info = cities.length > 0 ? cities[0] : "";
    var info2 = cities.length > 1 ? cities[1] : "";

    switch (cities.length)
    {
        case 0: return null;
        case 1:
            text = ownMessage ? (info == I18.NOWHERE ? I18.f("YOUR_ARMY") : I18.f("YOUR_ARMY_ONE", info)) : CombatViewUtils.playerOrClanDescription(username, info);
            break;
        case 2:
            text = ownMessage ? I18.f("YOUR_ARMY_TWO", [info, info2]) : I18.f("PLAYER_ARMY_TWO", [username, info, info2]);
            break;
        default:
            text = ownMessage ? I18.f("YOUR_ARMY_MANY", [info, info2]) : I18.f("PLAYER_ARMY_MANY", [username, info, info2]);
            break;

    }

    return CombatViewUtils.createLabelWithColor(Strings.safeHtml(text), color);
}

CombatViewUtils.playerOrClanDescription = function(username, info)
{
    if (!Strings.isEmpty(username) && username.charAt(0) == '$')
    {
        return info == I18.NOWHERE ? I18.f("CLAN_ARMY", username.substring(1)) : I18.f("CLAN_ARMY_ONE", [username.substring(1), info]);
    }
    else
    {
        return info == I18.NOWHERE ? I18.f("OPPONENT_ARMY", FUser.getUserNameForMessages(username)) : I18.f("PLAYER_ARMY_ONE", [FUser.getUserNameForMessages(username), info]);
    }
}