function MailBoxView()
{
	if (MailBoxView._instance)
	{
        throw Error("Could not create multiple instances of MailBoxView");
    }

    Events.onFEstateUpdated.add(this, function(e)
    {
        if (!(e.__className__ == "FCastle" || e.__className__ == "FCatapult"))
        {
            this.ownerUsername = e.owner.username;
        }
        this.haveMailAccess = e.accessMail;
        if (this.haveMailAccess && e.mailBoxStatus)
        {
            this.requestNewMessages(e.mailBoxStatus);
        }
    });
    
    Events.onFKingdomUpdated.add(this, function(k)
    {
        this.ownerUsername = FUser.CURRENT_USER.username;
        haveMailAccess = true;
        this.requestNewMessages(k.mailBoxStatus);
    });
	
	callParentConstructor(MailBoxView.prototype, this);

    this.ownerUsername = "";

    // variables that define client mailbox state
    this.currentInboxFilter = "filterAll";
    this.currentInboxPage = 0;
    this.currentOutboxPage = 0;
    this.currentArchivePage = 0;
    this.currentInboxOpenedMessage = -1;
    this.currentOutboxOpenedMessage = -1;
    this.currentArchiveOpenedMessage = -1;

    this.haveMailAccess = true;
    this.filterAdminButton = null;
    this.dialog = null;

    this.mailBox = null;
    this.isOwner = true;
    this.lettersText = "";

    this.contactList = null;
    this.addContactButton = null;
    this.magicInContacts = null;
    this.clanList = null;
    this.sendMessageToClanButton = null;
    this.ignoreList = null;
    this.ignoreContactButton = null;
    this.clanTab = null;
    this.mailBoxFolders = null;
    
    this.createInboxMessageRenderer = $.proxy(this.createInboxMessageRenderer, this);
    this.createOutboxMessageRenderer = $.proxy(this.createOutboxMessageRenderer, this);
    this.createArchiveMessageRenderer = $.proxy(this.createArchiveMessageRenderer, this);
    
    this.onExpandInboxMessage = $.proxy(this.onExpandInboxMessage, this);
	this.onCollapseInboxMessage = $.proxy(this.onCollapseInboxMessage, this);
	this.onExpandOutboxMessage = $.proxy(this.onExpandOutboxMessage, this);
	this.onCollapseOutboxMessage = $.proxy(this.onCollapseOutboxMessage, this);
	this.onExpandArchiveMessage = $.proxy(this.onExpandArchiveMessage, this);
	this.onCollapseArchiveMessage = $.proxy(this.onCollapseArchiveMessage, this);
	
	this.onChangeInboxPage = $.proxy(this.onChangeInboxPage, this);
	this.onChangeOutboxPage = $.proxy(this.onChangeOutboxPage, this);
	this.onChangeArchivePage = $.proxy(this.onChangeArchivePage, this);
	this.onPagerChanged = $.proxy(this.onPagerChanged, this);
	
	this.onAbuse = $.proxy(this.onAbuse, this);
	this.onRefuseMentor = $.proxy(this.onRefuseMentor, this);
	this.onAskMentorResources = $.proxy(this.onAskMentorResources, this);
	this.onExpelApprentice = $.proxy(this.onExpelApprentice, this);
	this.onReleaseApprentice = $.proxy(this.onReleaseApprentice, this);
	
	this.toggleContants = $.proxy(this.toggleContants, this);
	
	this.onChangeSelection = $.proxy(this.onChangeSelection, this);
    
    this.__defineGetter__("currentPage", this.getCurrentPage);
}

extendClass(MailBoxView.prototype, Dialog.prototype);

MailBoxView._instance = null;

MailBoxView.i = function()
{
    if (!MailBoxView._instance)
    {
        MailBoxView._instance = COM.create("MailBoxView");
    }
    return MailBoxView._instance;
}

MailBoxView.toggle = function(hide)
{
    if (MailBoxView._instance)
    {
        MailBoxView._instance.DOM.style.display = hide ? "none" : "";
        MailBoxView._instance.fitLayout();
    }
}

MailBoxView.isVisible = function()
{
    return MailBoxView._instance && MailBoxView._instance.DOM.style.display != "none";
}

MailBoxView.prototype.__construct = function()
{
	var filters = [
		{icon: MailAssets.BUTTON_FILTER_ALL, name: I18.f("MESSAGE_FILTER_ALL_TOOLTIP"), callback: this.showInbox},
		{icon: MailAssets.BUTTON_SENT, name: I18.f("SENT_MESSAGES"), callback: this.showOutbox},
		{icon: MailAssets.BUTTON_FILTER_SYSTEM, name: I18.f('MESSAGE_FILTER_SYSTEM_TOOLTIP'), callback: this.setFilter, filter: "filterInfoMessages"},
		{icon: MailAssets.BUTTON_FILTER_MILITARY, name: I18.f('MESSAGE_FILTER_MILITARY_TOOLTIP'), callback: this.setFilter, filter: "filterMilitaryMissionMessages"},
		{icon: MailAssets.BUTTON_FILTER_MAGIC, name: I18.f('MESSAGE_FILTER_MAGIC_TOOLTIP'), callback: this.setFilter, filter: "filterMagicMessages"},
		{icon: MailAssets.BUTTON_FILTER_USER, name: I18.f('MESSAGE_FILTER_USER_TOOLTIP'), callback: this.setFilter, filter: "filterUserMessages"},
		{icon: MailAssets.BUTTON_FILTER_CLAN, name: I18.f('MESSAGE_FILTER_CLAN_TOOLTIP'), callback: this.setFilter, filter: "filterClanMessages"},
		{icon: MailAssets.BUTTON_FILTER_ADMIN, name: I18.f('MESSAGE_FILTER_ADMIN_TOOLTIP'), callback: this.setFilter, filter: "filterAdminMessages"},
		{icon: MailAssets.BUTTON_ARCHIVE, name: I18.f("ARCHIVE"), callback: this.showArchive},
	];
	
	var f;
	for(var i=0; i<filters.length; i++)
	{
		f = COM.create("MailFilterItem");
		f.init(filters[i].icon, filters[i].name);
		f.filter = filters[i].filter;
		f.onClick = $.proxy(filters[i].callback, this);
		this.filters.appendChild(f.DOM);
		
		if(i == 0)
		{
			f.select();
		}
	}
	
	this.filtersScroll = new iScroll(this.filters.parentNode, {scrollbarColor: '#2a0f04'});
	
	this.contactsScroll = new iScroll(this.contactListHolder, {
		onBeforeScrollMove: APP.pauseTicks,
		onScrollEnd: APP.resumeTicks,
		scrollbarColor: '#2a0f04',
	});
	
	COM(this.addContactButton).caption = I18.f('ADD_CONTACT');
	COM.bindExclusiveClick(this.addContactButton, $.proxy(function()
	{
		this.addUserHandler(this.mailBox);
	}, this));
	
	COM(this.ignoreContactButton).caption = I18.f('ADD_TO_IGNORE');
	COM.bindExclusiveClick(this.ignoreContactButton, $.proxy(function()
	{
		this.ignoreUserHandler(this.mailBox);
	}, this));
	
	COM(this.sendMessageButton).icon = STATIC_INTERFACE_URL + "shared/mail/new_mail_icon.png";
	COM.bindExclusiveClick(this.sendMessageButton, $.proxy(function()
	{
		this.sendMessage();
	}, this));
	
	COM.bindExclusiveClick(this.sendMessageToClanButton, $.proxy(function()
	{
		this.sendMessageToReceiverType(MailReceiverType.CLAN);
	}, this));
	
	COM.bindExclusiveClick(this.archiveCount, $.proxy(this.showPremiumDialog, this));
	COM.bindExclusiveClick(this.contactListCapacity, $.proxy(this.showPremiumDialog, this));
	
	this.skin = "darkwood-border";
	
	COM(this.showContactsButton).caption = I18.f("CONTACT_LIST");
	COM.bindExclusiveClick(this.showContactsButton, this.toggleContants);
	COM.bindExclusiveClick(this.closeContactsShader, this.toggleContants);
	
	//this.contactListTitleText.innerHTML = I18.f("CONTACT_LIST").toUpperCase();
	
	var ct = COM(this.contactsTabs);
	
	var b = ct.addItem("", "contactList", true);
	b.icon = STATIC_INTERFACE_URL + "shared/mail/users.png";
	
	this.clanListTab = ct.addItem("", "clanList");
	this.clanListTab.icon = STATIC_INTERFACE_URL + "shared/mail/clans.png";
	
	var b = ct.addItem("", "ignoreList");
	b.icon = STATIC_INTERFACE_URL + "shared/mail/black_list.png";
	
	ct.onChange = $.proxy(function(val)
	{
		this.showContactTab(this[ct.value]);
	}, this);
	
	//this.toggleContants();
}

MailBoxView.prototype.toggleContants = function()
{
	if($(this.DOM).hasClass("movedLeft"))
	{
		this.closeContactsShader.style.display = "none";
	}
	else
	{
		this.closeContactsShader.style.display = "block";
	}
	
	$(this.DOM).toggleClass("movedLeft");
}

MailBoxView.prototype.fitLayout = function(container)
{
	/*
	if("ontouchstart" in window && document.body.scrollTop > 1)
	{
		return;
	}
	*/
	
	superMethod(MailBoxView.prototype, "fitLayout", this)(container);
	
	if(!container) container = {width: window.innerWidth, height: window.innerHeight};
	
	this.closeContactsShader.style.width = container.width + "px";
	this.closeContactsShader.style.height = container.height + "px";
	
	var padding = parseFloat($("> contentwrapper", this.wrapper).css("padding-right"));
	
	this.contactListContainer.style.left = (container.width-padding) + "px";
	this.contactListContainer.style.height = container.height + "px";
	
	this.contactListHolder.style.height = (container.height-padding-this.contactListTitle.offsetHeight-this.contactListPanel.offsetHeight) + "px";
	
	var w = $("> contentwrapper > content", this.wrapper);
	var c = {width: w.width(),  height: w.height()};
	
	c.width += padding;
	
	this.windowWrapper.style.width = c.width+"px";
	this.windowWrapper.style.height = c.height+"px";
	
	this.navWrapper.style.height = c.height+"px";
	
	COM(this.mailBoxFolders).fitLayout({width: c.width-this.navWrapper.offsetWidth,  height: c.height});
}

MailBoxView.prototype.setFilter = function(item)
{
	if(item && item.filter && item.filter != "")
	{
		var mbf = COM(this.mailBoxFolders);
		mbf.showTab(mbf.inboxPage);
		if(this.currentInboxFilter != item.filter)
		{
			this.filterChanged(item.filter);
		}
	}
}

MailBoxView.prototype.showInbox = function()
{
	var mbf = COM(this.mailBoxFolders);
	mbf.showTab(mbf.inboxPage);
	if(this.currentInboxFilter != "filterAll")
	{
		this.filterChanged("filterAll");
	}
}

MailBoxView.prototype.showOutbox = function()
{
	var mbf = COM(this.mailBoxFolders);
	mbf.showTab(mbf.outboxPage);
	if(this.currentInboxFilter != "filterAll")
	{
		this.filterChanged("filterAll");
	}
}

MailBoxView.prototype.showArchive = function()
{
	var mbf = COM(this.mailBoxFolders);
	mbf.showTab(mbf.archivePage);
	if(this.currentInboxFilter != "filterAll")
	{
		this.filterChanged("filterAll");
	}
}

MailBoxView.prototype.popup = function(ownerUsername)
{
	if(!ownerUsername) return;
	
	if(!this.DOM.parentNode)
	{
		superMethod(MailBoxView.prototype, "popup", this)();
		this.setCaption(I18.f("MAILBOX").toUpperCase());
		this.fullScreen();
	}
	else
	{
		this.DOM.style.display = "";
		this.onShow();
	}
	
	this.ownerUsername = ownerUsername;
    this.isOwner = FUser.isCurrentUser(ownerUsername);
    
    //this.currentInboxFilter = FUser.CURRENT_USER.settings.getMailBoxFilter();

    this.clearMailboxState();
    this.refresh();
}

MailBoxView.prototype.onShow = function()
{
	this.fitLayout();
	this.refreshScroll();
}

MailBoxView.prototype.refreshScroll = function()
{
	if($(this.DOM).is(':visible'))
	{
		setTimeout($.proxy(this.filtersScroll.refresh, this.filtersScroll), 500);
		setTimeout($.proxy(this.contactsScroll.refresh, this.contactsScroll), 500);
	}
}

MailBoxView.prototype.close = function()
{
	Dialog.fullScreenDialogsCount--;
	MailBoxView.toggle(true);
	
	COM(COM(this.mailBoxFolders)).hidePanel();
	
	this.destroyChildArrows();
}

MailBoxView.prototype.hideAllContactTabs = function()
{
	this.contactList.style.display = "none";
	this.clanList.style.display = "none";
	this.ignoreList.style.display = "none";
}

MailBoxView.prototype.showContactTab = function(tab)
{
	this.hideAllContactTabs();
	tab.style.display = "";
	
	this.addContactButton.style.display = (tab == this.contactList ? "" : "none");
	this.sendMessageToClanButton.style.display = (tab == this.clanList ? "" : "none");
	this.ignoreContactButton.style.display = (tab == this.ignoreList ? "" : "none");
	
	this.refreshScroll();
	this.contactsScroll.scrollTo(0, 0);
}

MailBoxView.prototype.requestNewMessages = function(mbs)
{
    if (mbs.newMessages && MailBoxView.isVisible())
    {
        this.refresh();
    }
    
    if (MailBoxView.isVisible())
    {
        this.updateAdminFilterBlinking(mbs);
    }
}

MailBoxView.prototype.updateAdminFilterBlinking = function(mbs)
{
    if (this.filterAdminButton)
    {
        this.filterAdminButton.icon = this.mailBox && mbs.newAdminMessages && this.mailBox.inbox.length > 1 && this.filterAdminButton.enabled ?
                CommonAssets.BUTTON_FILTER_ADMIN_BLINKING : MailAssets.BUTTON_FILTER_ADMIN;
    }
}

MailBoxView.prototype.mailBoxState = function()
{
    var res = new ClientMailBoxState();
    res.inbox.page = this.currentInboxPage;
    res.outbox.page = this.currentOutboxPage;
    res.archive.page = this.currentArchivePage;
    res.inbox.openedMessageId = -1;//new FLong(this.currentInboxOpenedMessage);
    res.outbox.openedMessageId = -1;//new FLong(this.currentOutboxOpenedMessage);
    res.archive.openedMessageId = -1;//new FLong(this.currentArchiveOpenedMessage);
    res.inboxFilter = this.currentInboxFilter;
    return res;
}

MailBoxView.prototype.refresh = function()
{
    Service.mail.callMethod("getMailBox", [this.ownerUsername, this.mailBoxState()], $.proxy(this.updateMailBox, this));
    
    setTimeout(APP.showLoader, 10);
}

MailBoxView.prototype.updateMailBoxStatus = function(mbs)
{
    Events.onMailboxStatusChange.fire(mbs);
    this.requestNewMessages(mbs);
}

MailBoxView.prototype.handlerOnResult = function(dialog, message, components, update)
{
    if(typeof update == "undefined") update = true;
    
    return $.proxy(function(mail)
    {
        if (dialog)
        {
            dialog.close();
        }
        if (message)
        {
            var items = [message];
            if (components)
            {
                items = items.concat(components);
            }
            
            var d = $("<div/>");
            var c;
            for(var i=0; i<items.length; i++)
            {
            	c = $("<div/>");
            	c.append(items[i].DOM ? items[i].DOM : items[i]);
            	d.append(c);
            }
            
            Alert.create(d[0]);
        }
        if (update)
        {
            this.updateMailBox(mail);
        }
        
        APP.hideLoader();
    }, this);
}

MailBoxView.handlerOnFail = function(e)
{
    if(!e) return false;
    
    if (e.__className__ == "AvatarNotFoundException" || e.__className__ == "MailException" || e.__className__ == "MentorshipException")
    {
        Alert.create(e.getMessage());
        return true;
    }
    return false;
}

MailBoxView.prototype.ignoreContact = function(dialog, username, comment)
{
    Quarantine.put(dialog);
    
    Service.mail.callMethod("ignoreContact", [this.ownerUsername, username, comment],
    
    $.proxy(function(mail)
    {
        this.handlerOnResult(dialog)(mail);
	}, this),
	
	MailBoxView.handlerOnFail
        
    );
}

MailBoxView.prototype.addContact = function(forUsername, dialog, username, comment, showStatus)
{
    Quarantine.put(dialog);
    Service.mail.callMethod("addContact", [forUsername, username, comment, showStatus],
	
	$.proxy(function(mail)
	{
		this.handlerOnResult(dialog, null, null, forUsername == this.ownerUsername)(mail);
	}, this),
	
	MailBoxView.handlerOnFail
	
    );
}

MailBoxView.prototype.updateMailBox = function(mail)
{
    if (mail.state != null)
    {
        this.currentInboxPage = mail.state.inbox.page;
        this.currentOutboxPage = mail.state.outbox.page;
        this.currentArchivePage = mail.state.archive.page;
        this.currentInboxFilter = mail.state.inboxFilter;
    }
    
    this.updateMailBoxStatus(mail.mailBoxStatus);
    this.update(mail);
    
    COM(this.mailBoxFolders).resetScroll();
    
    setTimeout(APP.hideLoader, 200);
}

MailBoxView.prototype.update = function(mail)
{
    if (!this.mailBox)
    {
        this.mailBox = mail;
    }
    
    this.updateContacts(mail);
    
    if (mail.state)
    {
        this.mailBox = mail;
        this.updateInboxPage();
        this.updateOutboxPage();
        this.updateArchivePage();
        this.updateMailBoxFolders();
    }
    
    this.updateArrows();
}

MailBoxView.prototype.updateContacts = function(mail)
{
    if (!mail)
    {
        return;
    }
    
    this.isOwner = FUser.isCurrentUser(mail.ownerUsername);
    this.mailBox.cannotAskResourcesTill = mail.cannotAskResourcesTill;
    this.mailBox.contactList = mail.contactList;
    this.mailBox.clanList = mail.clanList;
    
    if (this.contactList)
    {
        var el = COM(this.contactList);
        el.onCreateRenderer = this.createContactRenderer;
		el.dataProvider = mail.notIgnoredList;
		el.update(true);
    }
    
    if (this.clanList)
    {
        var el = COM(this.clanList);
        el.onCreateRenderer = this.createClanContactRenderer;
		el.dataProvider = mail.clanList;
		el.update();
    }
    
    this.clanListTab.reason = (mail.clanList && mail.clanList.length > 0) ? "" : I18.f("NOT_CLAN_MEMBER");
    
    if (this.ignoreList)
    {
        var el = COM(this.ignoreList);
        el.onCreateRenderer = this.createContactRenderer;
		el.dataProvider = mail.ignoreList;
		el.update();
    }
    
    COM(this.contactListCapacity).caption = mail.friendsList.length + "/" + mail.getMaxContacts();
    COM(this.addContactButton).enabled = (mail.friendsList.length < mail.getMaxContacts());
    
    /*
    if (this.magicInContacts)
    {
        this.magicInContacts.activate(mail.ownerPremiumUntil);
    }
    */
    
    this.refreshScroll();
}

MailBoxView.prototype.createContactRenderer = function()
{
    var renderer = COM.create("ContactDetails");
    
    var self = MailBoxView.i();
    
    renderer.update = function(cd)
    {
        this.removeButton.style.display = self.isOwner ? "" : "none";
        this.editButton.style.display = self.isOwner ? "" : "none";
        this.onEdit = self.onEditContact;
        
        this.ask.style.display = "none";
        this.allianceStatus.style.display = "";

        switch (cd.relation.name)
        {
            case Relation.MENTOR.name:
                this.onRemove = self.onRefuseMentor;
                //COM(this.removeButton).caption = I18.f("REFUSE_MENTOR");
                this.onAsk = self.onAskMentorResources;
                this.ask.style.display = "";
                this.allianceStatus.style.display = "none";
                this.ask.style.display = self.isOwner && Tick.now() > self.mailBox.cannotAskResourcesTill ? "" : "none";
                break;
            case Relation.APPRENTICE.name:
                this.onRemove = cd.readyToRelease ? self.onReleaseApprentice : self.onExpelApprentice;
                //COM(this.removeButton).caption = cd.readyToRelease ? I18.f("RELEASE_APPRENTICE") : I18.f("EXPEL_APPRENTICE");
                break;
            case Relation.FRIEND.name:
                this.onRemove = self.onDeleteFriend;
                break;
            case Relation.IGNORE.name:
                this.onRemove = self.onDeleteIgnoreContact;
                this.onEdit = self.onEditIgnoreContact;
                break;
        }
    };
    return renderer;
}

MailBoxView.prototype.createClanContactRenderer = function()
{
    var renderer = COM.create("ContactDetails");

    renderer.update = function(cd)
    {
        this.ask.style.display = "none";
        this.editButton.style.display = "none";
        this.removeButton.style.display = "none";
    }

    return renderer;
}

MailBoxView.prototype.addUserHandler = function(mail, presetName)
{
    if(typeof presetName == "undefined") presetName = "";
    
    var dialog = COM.create("AddEditContactDialog");
    dialog.onResult = function(username, comment, showStatus)
    {
        var checkResult = MailConstraints.addContactValid(mail.ownerUsername, username, mail.notIgnoredList);
        
        if (checkResult.isSuccess)
        {
            var self = MailBoxView.i();
            
            if (FMailBox.getContact(mail.ignoreList, username))
            {
                Alert.create(I18.f("ADD_CONTACT_FROM_IGNORE"), "", function()
                {
                    self.addContact(mail.ownerUsername, dialog, username, comment, showStatus);
                });
            }
            else
            {
                self.addContact(mail.ownerUsername, dialog, username, comment, showStatus);
            }
        }
        else
        {
            Alert.create(checkResult.text);
        }
    };
    
    dialog.popup();
    COM(dialog.player).value = presetName;
    dialog.toAddContactMode();
}

MailBoxView.prototype.ignoreUserHandler = function(mail, presetName)
{
    if(typeof presetName == "undefined") presetName = "";
    
    var dialog = COM.create("AddEditContactDialog");
    
    dialog.onResult = function(username, comment, showStatus)
    {
        var checkResult = MailConstraints.ignoreContactValid(FUser.CURRENT_USER.username, username, mail.ignoreList);
        
        if (checkResult.isSuccess)
        {
            var self = MailBoxView.i();
            
            if (FMailBox.getContact(mail.friendsList, username))
            {
                Alert.create(I18.f("IGNORE_CONTACT_FROM_CONTACTS"), "", function()
                {
					self.ignoreContact(dialog, username, comment);
                });
            }
            else
            {
                self.ignoreContact(dialog, username, comment);
            }
        }
        else
        {
            Alert.create(checkResult.text);
        }
    };
    
    dialog.popup();
    dialog.toIgnoreContactMode();
    COM(dialog.player).value = presetName;
}

MailBoxView.prototype.onEditContact = function(contact)
{
    var dialog = COM.create("AddEditContactDialog");
    dialog.onResult = function(username, comment, showStatus)
    {
        var self = MailBoxView.i();
        
        Service.mail.callMethod("editContact", [self.ownerUsername, contact.user.username, comment, showStatus],
          function(mail)
          {
        	  self.handlerOnResult(dialog)(mail);
          },
	      MailBoxView.handlerOnFail);
    };
    
    dialog.popup();
    COM(dialog.player).value = contact.user.username;
    COM(dialog.player).disabled = true;
    COM(dialog.comment).value = contact.comment;
    COM(dialog.comment).disabled = contact.relation.name != Relation.FRIEND.name;
    COM(dialog.checkBox).checked = contact.showStatus;
    dialog.toAddContactMode(true);
}

MailBoxView.prototype.onDeleteIgnoreContact = function(contact)
{
    Alert.create(I18.f("DELETE_CONTACT_CONFIRMATION", contact.user.username), "", function()
    {
		var self = MailBoxView.i();
		
		Service.mail.callMethod("deleteIgnoreContact", [self.ownerUsername, contact.user.username],
		
		function(mail)
	    {
	        MailBoxView.i().handlerOnResult()(mail);
		},
		
		MailBoxView.handlerOnFail
		
		);
	});
}

MailBoxView.prototype.onEditIgnoreContact = function(contact)
{
    var dialog = COM.create("AddEditContactDialog");
    
    dialog.onResult = function(username, comment, showStatus)
    {
        var self = MailBoxView.i();
        
        Service.mail.callMethod("editIgnoreContact", [self.ownerUsername, contact.user.username, comment],
		
		function(mail)
		{
			self.handlerOnResult(dialog)(mail);
		},
		
		MailBoxView.handlerOnFail
        
        );
    };
    
    dialog.popup();
    COM(dialog.player).value = contact.user.username;
    COM(dialog.player).disabled = true;
    COM(dialog.comment).value = contact.comment;
    dialog.toIgnoreContactMode(true);
}

MailBoxView.prototype.onDeleteFriend = function(contact)
{
    Alert.create(I18.f("DELETE_CONTACT_CONFIRMATION", contact.user.username), "", function()
    {
        var self = MailBoxView.i();
        
        Service.mail.callMethod("deleteContact", [self.ownerUsername, contact.user.username], 
			function(mail)
			{
				self.handlerOnResult()(mail);
			},
			MailBoxView.handlerOnFail
        );
    });
}

MailBoxView.prototype.updateArrows = function()
{
    var hint = FUser.CURRENT_USER.findHint(HintType.ARCHIVE_LETTER);
    var mbf = COM(this.mailBoxFolders);
    
    if (!hint || mbf.getCurrentTab() != mbf.inboxPage)
    {
        HintArrow.SELECT_SOME_MESSAGE.hide();
        HintArrow.CLICK_ARCHIVE.hide();
        return;
    }

    var list = COM(COM(mbf.inboxPage).list);
    
    if (this.isSomethingSelected(list))
    {
        HintArrow.SELECT_SOME_MESSAGE.hide();
        HintArrow.CLICK_ARCHIVE.show(mbf.archiveButton, hint, this.DOM, null, {x: 0, y: -mbf.actionsPanel.offsetHeight});
    }
    else if (list.items.length > 0)
    {
        HintArrow.CLICK_ARCHIVE.hide();
        HintArrow.SELECT_SOME_MESSAGE.show(COM(list.items[0]).checkbox, hint, this.DOM);
    }
}

MailBoxView.prototype.updateInboxPage = function()
{
    if (!this.mailBoxFolders || !this.mailBox)
    {
        return;
    }
    
    var el = COM(COM(this.mailBoxFolders).inboxPage);
    
    el.onCreateRenderer = this.createInboxMessageRenderer;
    el.onExpand = this.onExpandInboxMessage;
    el.onCollapse = this.onCollapseInboxMessage;
    COM(el.pager).onSelect = this.onChangeInboxPage;
    COM(el.pager).onChange = this.onPagerChanged;
    el.dataProvider = this.mailBox.inbox;
    
    el.update();

    //updateSelections(mailBoxFolders.inboxPage.list.renderers, mailBox.inbox);
    
    if (this.mailBox.state)
    {
		COM(el.pager).setIndexAndCount(this.mailBox.state.inbox.page, this.mailBox.inboxPagesCount);
	}
}

MailBoxView.prototype.updateOutboxPage = function()
{
    if (!this.mailBoxFolders || !this.mailBox)
    {
        return;
    }
    
    var el = COM.get(COM.get(this.mailBoxFolders).outboxPage);
    
    el.onCreateRenderer = this.createOutboxMessageRenderer;
    el.onExpand = this.onExpandOutboxMessage;
    el.onCollapse = this.onCollapseOutboxMessage;
    COM(el.pager).onSelect = this.onChangeOutboxPage;
    
    el.dataProvider = this.mailBox.sentMessages;
    
    el.update();
    
    if (this.mailBox.state)
    {
        COM(el.pager).setIndexAndCount(this.mailBox.state.outbox.page, this.mailBox.outboxPagesCount);
    }

    /*
    updateSelections(mailBoxFolders.outboxPage.list.renderers, mailBox.sentMessages);
    */
}

MailBoxView.prototype.updateArchivePage = function()
{
    if (!this.mailBoxFolders || !this.mailBox)
    {
        return;
    }
    
    var el = COM(COM(this.mailBoxFolders).archivePage);
    
    el.onCreateRenderer = this.createArchiveMessageRenderer;
    el.onExpand = this.onExpandArchiveMessage;
    el.onCollapse = this.onCollapseArchiveMessage;
    COM(el.pager).onSelect = this.onChangeArchivePage;
    
    el.dataProvider = this.mailBox.archive;
    
    el.update();
    
    if (this.mailBox.state)
    {
        COM(el.pager).setIndexAndCount(this.mailBox.state.archive.page, this.mailBox.archivePagesCount);
    }

    /*
    updateSelections(mailBoxFolders.archivePage.list.renderers, mailBox.archive);
    */
}

MailBoxView.prototype.onPagerChanged = function()
{
	COM(this.mailBoxFolders).onPagerChanged();
}

MailBoxView.prototype.clearMailboxState = function()
{
    this.currentInboxPage = 0;
    this.currentOutboxPage = 0;
    this.currentArchivePage = 0;
    this.currentInboxOpenedMessage = -1;
    this.currentOutboxOpenedMessage = -1;
    this.currentArchiveOpenedMessage = -1;
}

MailBoxView.prototype.createInboxMessageRenderer = function()
{
    return this.createMessageRenderer().setOnChangeCheckbox(this.onChangeSelection).showFrom(true);
}

MailBoxView.prototype.createOutboxMessageRenderer = function()
{
    return this.createMessageRenderer().setOnChangeCheckbox(this.onChangeSelection).showTo(true);
}

MailBoxView.prototype.createArchiveMessageRenderer = function()
{
    return this.createMessageRenderer().setOnChangeCheckbox(this.onChangeSelection).showFrom(true).showTo(true);
}

MailBoxView.prototype.onChangeSelection = function()
{
	var selMails = this.getSelectedMails(this.currentPage.list);
    
    if(selMails.length > 0)
    {
    	COM(this.mailBoxFolders).setSelectCount(selMails.length);
    	COM(this.mailBoxFolders).showPanel();
    }
    else
    {
    	COM(this.mailBoxFolders).hidePanel();
    }
    
    //this.mailBoxFolders.deleteButton.disabled = !(somethingSelected && this.isOwner);
    //this.mailBoxFolders.archiveButton.disabled = !somethingSelected;
    //this.mailBoxFolders.markReadButton.disabled = !somethingSelected;
    this.updateArrows();
}

MailBoxView.prototype.isSomethingSelected = function(list)
{
	var items = COM(list).items;
	
	var item;
	for(var i=0; i<items.length; i++)
	{
		item = COM(items[i]);
		if(COM(item.checkbox).checked)
		{
			return true;
		}
	}
	
	return false;
}

MailBoxView.prototype.createMessageRenderer = function()
{
    var renderer = COM.create("MessageDetails");
    renderer.onIgnoreContactFromMessage = $.proxy(this.onIgnoreFromMessage, this);
    renderer.onAddContactFromMessage = $.proxy(this.onAddFromMessage, this);
    renderer.onAbuseContactFromMessage = $.proxy(this.onAbuse, this);
    renderer.onForwardMessage = $.proxy(this.onForward, this);
    renderer.onReplyOnMessage = $.proxy(function(receiver, subject, adminSubject, messageBody)
    {
        this.sendMessage(receiver, subject, adminSubject, messageBody);
    }, this);
    renderer.onReplyToAdmin = $.proxy(function(message)
    {
        this.sendMessageToAdmin(new SubjectInput(message.createReplySubject(), false), message.adminSubject, message.createReplyMessageBody());
    }, this);
    return renderer;
}

MailBoxView.prototype.onExpandInboxMessage = function(renderer)
{
    this.onOpenInboxMessage(renderer.data);
}

MailBoxView.prototype.onCollapseInboxMessage = function(renderer)
{
    this.onCloseInboxMessage(renderer.data);
}

MailBoxView.prototype.onExpandOutboxMessage = function(renderer)
{
    this.onOpenOutboxMessage(renderer.data);
}

MailBoxView.prototype.onCollapseOutboxMessage = function(renderer)
{
    this.onCloseOutboxMessage(renderer.data);
}

MailBoxView.prototype.onExpandArchiveMessage = function(renderer)
{
    this.onOpenArchiveMessage(renderer.data);
}

MailBoxView.prototype.onCollapseArchiveMessage = function(renderer)
{
    this.onCloseArchiveMessage(renderer.data);
}

MailBoxView.prototype.onOpenInboxMessage = function(m)
{
    if (m.unread)
    {
        Service.mail.callMethod("markMessagesRead", [this.ownerUsername, [m.id]], $.proxy(this.updateMailBoxStatus, this));
    }
    this.currentInboxOpenedMessage = m.id;
}

MailBoxView.prototype.onCloseInboxMessage = function(m)
{
    this.currentInboxOpenedMessage = -1;
}

MailBoxView.prototype.onOpenOutboxMessage = function(m)
{
    this.currentOutboxOpenedMessage = m.id;
}

MailBoxView.prototype.onCloseOutboxMessage = function(m)
{
    this.currentOutboxOpenedMessage = -1;
}

MailBoxView.prototype.onOpenArchiveMessage = function(m)
{
    if (m.unread)
    {
        Service.mail.callMethod("markMessagesRead", [this.ownerUsername, [m.id]], $.proxy(this.updateMailBoxStatus, this));
    }
    this.currentArchiveOpenedMessage = m.id;
}

MailBoxView.prototype.onCloseArchiveMessage = function(m)
{
    this.currentArchiveOpenedMessage = -1;
}

MailBoxView.prototype.onChangeInboxPage = function(index)
{
    this.currentInboxPage = index;
    this.currentInboxOpenedMessage = -1;
    this.refresh();
}

MailBoxView.prototype.onChangeOutboxPage = function(index)
{
    this.currentOutboxPage = index;
    this.currentOutboxOpenedMessage = -1;
    this.refresh();
}

MailBoxView.prototype.onChangeArchivePage = function(index)
{
    this.currentArchivePage = index;
    this.currentArchiveOpenedMessage = -1;
    this.refresh();
}

MailBoxView.prototype.onChangeFilter = function(filter)
{
    this.currentInboxFilter = filter;
    this.currentInboxOpenedMessage = -1;
    this.refresh();
}

MailBoxView.prototype.showPremiumDialog = function()
{
    PaymentDialog.popup(FUser.CURRENT_USER, null, null);
}

MailBoxView.prototype.sendMessage = function(receiver, subject, adminSubject, messageBody, receiverType)
{
    this.sendMessageImpl(this.ownerUsername, receiver, subject, adminSubject, messageBody, receiverType);
}

MailBoxView.prototype.sendOwnMessage = function(receiver, subject, adminSubject, messageBody, receiverType)
{
    this.sendMessageImpl(FUser.CURRENT_USER.username, receiver, subject, adminSubject, messageBody, receiverType);
}

MailBoxView.prototype.sendMessageToAdmin = function(subjectComponent, adminSubject, messageBody)
{
    if(!messageBody) messageBody = "";
    
    if (!subjectComponent)
    {
        subjectComponent = new SubjectInput();
    }
    var dialog = COM.create("NewMessageDialog");
    dialog.onResult = function(username, subject, messageBody, receiverType)
    {
        APP.showLoader();
        var self = MailBoxView.i();
        
        adminSubject = adminSubject ? adminSubject : subjectComponent.adminSubject;
        if(subject == adminSubject) subject = I18.f(subject);
        
        Service.mail.callMethod("sendMessageToAdmin", [self.ownerUsername, subject, adminSubject, messageBody, self.mailBoxState()],
        
        	$.proxy(function(mail)
            {
            	self.handlerOnResult(dialog, I18.f("ADMIN_MESSAGE_SENT"))(mail);
            }, this),
            
            MailBoxView.handlerOnFail
        );
    };
    dialog.popup(null, I18.f("ADMINISTRATION_MESSAGE_SENDER"), subjectComponent, messageBody, MailReceiverType.USER, true);
    COM(COM(dialog.receiverSelector).receiverInput).disabled = true;
}

MailBoxView.prototype.sendMessageToMonsters = function(id)
{
    var dialog = COM.create("NewMessageDialog");
    var self = MailBoxView.i();
    
    dialog.onResult = function(username, subject, messageBody, receiverType)
    {
        APP.showLoader();
        Service.mail.callMethod("sendMessageToMonster", [self.ownerUsername, id, I18.f("MONSTER_PLAYER"), subject, messageBody, self.mailBoxState()],
            
            $.proxy(function(mail)
            {
            	self.handlerOnResult(dialog)(mail);
            }, self),
            
            MailBoxView.handlerOnFail
        );
    };
    dialog.popup(null, I18.f("MONSTER_PLAYER"), new SubjectInput(), "", MailReceiverType.USER, true);
    
    COM(COM(dialog.receiverSelector).receiverInput).disabled = true;
}

MailBoxView.prototype.sendMessageImpl = function(fromUsername, receiver, subject, adminSubject, messageBody, receiverType)
{
    if(typeof fromUsername == "undefined") fromUsername = "";
    if(typeof receiver == "undefined") receiver = "";
    if(typeof subject == "undefined") subject = "";
    if(typeof adminSubject == "undefined") adminSubject = "";
    if(typeof messageBody == "undefined") messageBody = "";
    
    var dialog = COM.create("NewMessageDialog");
    dialog.onResult = function(username, subject, messageBody, rt, endCallBack)
    {
        APP.showLoader();
        var self = MailBoxView.i();
        
        Service.mail.callMethod("sendMessage", [fromUsername, username, subject, adminSubject, messageBody, self.mailBoxState(), rt],
            
            $.proxy(function(mail)
            {
            	if(endCallBack) endCallBack();
            	self.handlerOnResult(dialog, null, null, fromUsername == self.ownerUsername)(mail);
            }, this),
            
            MailBoxView.handlerOnFail
        );
    };
    var sc = new SubjectInput(subject);
    dialog.popup(null, receiver, sc, messageBody, receiverType);
    dialog.silencedTill = FUser.CURRENT_USER.silencedTill;
}

MailBoxView.prototype.sendMessageToReceiverType = function(rt)
{
    this.sendMessage("", "", "", "", rt);
}

MailBoxView.prototype.getCurrentPage = function()
{
    var m = COM(this.mailBoxFolders);
    
    if(m.inboxPage.style.display != "none")
    {
    	return COM(m.inboxPage);
    }
    
    if(m.outboxPage.style.display != "none")
    {
    	return COM(m.outboxPage);
    }
    
    if(m.archivePage.style.display != "none")
    {
    	return COM(m.archivePage);
    }
    
    return null;
}

MailBoxView.prototype.getSelectedMails = function(list)
{
	var messages = $("messagedetails", list);
    
    var res = [];
    var cb, m;
    for(var i=0; i<messages.length; i++)
    {
    	m = COM(messages[i]);
    	if(COM(m.checkbox).checked)
    	{
    		res.push(m);
    	}
    }
    
    return res;
}

MailBoxView.prototype.markRead = function()
{
	var mails = this.getSelectedMails(this.currentPage.list);
	var res = [];
	for(var i=0; i<mails.length; i++)
	{
		if(mails[i].data.unread)
		{
			mails[i].data.unread = false;
			mails[i].unread = false;
			res.push(mails[i].data.id);
		}
		COM(mails[i].checkbox).checked = false;
		mails[i].skin = "";
	}
    
    Service.mail.callMethod("markMessagesRead", [this.ownerUsername, res], $.proxy(function()
    {
    	COM(this.mailBoxFolders).showInfo(I18.f('MAILS_MARK_AS_READ'));
    	this.updateMailBoxStatus();
    }, this));
	
	this.onChangeSelection();
}

MailBoxView.prototype.onArchiveClick = function(e)
{
    HintArrow.SELECT_SOME_MESSAGE.understood();
	HintArrow.CLICK_ARCHIVE.understood();
	
	this.noClearHints = true;
    
    var mails = this.getSelectedMails(this.currentPage.list);
    
    if(mails.length > 0)
    {
    	if (this.mailBox.archiveMessageCount + mails.length <= this.mailBox.archiveCapacity)
    	{
            var idList = [];
            for(var i=0; i<mails.length; i++)
            {
            	COM(mails[i].checkbox).checked = false;
            	mails[i].skin = "";
            	idList.push(mails[i].data.id);
            }
            
            Service.mail.callMethod("archiveMessages", [this.ownerUsername, idList, this.mailBoxState()],
				$.proxy(function(mail)
				{
					COM(this.mailBoxFolders).showInfo(I18.f('MAILS_MOVED_TO_ARCHIVE'));
					this.handlerOnResult()(mail);
					this.onChangeSelection();
				}, this),
				MailBoxView.handlerOnFail
            );
        }
        else
        {
            Alert.create(I18.f("ARCHIVE_CAPACITY_EXCEEDED_WARN"));
        }
    }
}

MailBoxView.prototype.onDelete = function(e)
{
	var mails = this.getSelectedMails(this.currentPage.list);
	var idList = [];
	for(var i=0; i<mails.length; i++)
	{
		idList.push(mails[i].data.id);
	}
    
    if (idList.length > 0)
    {
        Alert.create(I18.f("DELETE_MESSAGES_CONFIRMATION"), "",
                $.proxy(function()
                {
                    Service.mail.callMethod("deleteMessages", [this.ownerUsername, idList, this.mailBoxState()],
						
						$.proxy(function(mail)
						{
							COM(this.mailBoxFolders).showInfo(I18.f('MAILS_DELETED'));
							this.handlerOnResult()(mail);
							this.onChangeSelection();
						}, this),
						MailBoxView.handlerOnFail
                    );
                    
                }, this));
    }
}

MailBoxView.prototype.filterChanged = function(filterName)
{
    COM(this.mailBoxFolders).clear();
    this.onChangeFilter(filterName);
    this.updateArrows();
}

MailBoxView.prototype.updateMailBoxFolders = function()
{
    if (!this.mailBoxFolders || !this.mailBox)
    {
        return;
    }
	
	COM(this.archiveCount).caption = this.mailBox.archiveMessageCount + "/" + this.mailBox.archiveCapacity;
    //this.onChangeSelection();
}

MailBoxView.prototype.onForward = function(message)
{
    var dialog = COM.create("ForwardMailDialog");
    dialog.onResult = $.proxy(function(username, receiverType)
    {
        Service.mail.callMethod("forwardMessage", [this.ownerUsername, username, message.id, this.mailBoxState(), receiverType],
			$.proxy(function(mail)
			{
				this.handlerOnResult(dialog)(mail);
			}, this),
			MailBoxView.handlerOnFail
        );
    }, this);
    dialog.popup();
}

MailBoxView.prototype.onAddFromMessage = function(sender)
{
	this.addUserHandler(this.mailBox, sender);
}

MailBoxView.prototype.onIgnoreFromMessage = function(sender)
{
    this.ignoreUserHandler(this.mailBox, sender);
}

MailBoxView.prototype.onAbuse = function(message, isSpam, abuseButton)
{
    var empty = false;
    
    if (message.__CLS__ == "game.service.mail.FSystemMessage")
    {
        empty = Strings.isEmpty(message.text);
    }
    
    if (empty)
    {
        Alert.create(I18.f("NO_ABUSE_TO_EMPTY_MESSAGE"));
        return;
    }

    Alert.create(I18.f("ABUSE_WARNING"), "", $.proxy(function()
    {
		if(abuseButton) abuseButton.style.display = "none";
		
		Service.mail.callMethod("reportAbuse", [this.ownerUsername, message.id, isSpam, this.mailBoxState()],
                $.proxy(function(mail)
                {
                	this.handlerOnResult(null, I18.f("ABUSE_REPORTED"))(mail);
                }, this),
                MailBoxView.handlerOnFail
        );
    }, this));
}

MailBoxView.prototype.onRefuseMentor = function(contact)
{
    var message = I18.f("REFUSE_MENTOR_CONFIRMATION", contact.user.username);
    if (!FUser.CURRENT_USER.diplomacy.canBecomeApprentice)
    {
        message += "<br/><br/>" + I18.f("REFUSE_MENTOR_SELF_EDUCATION");
    }
    Alert.create(message, "",
            $.proxy(function(mail)
            {
                Service.mail.callMethod("refuseMentor", [contact.user.username],
                        $.proxy(function(mail)
                        {
                        	this.handlerOnResult()(mail);
                        }, this),
                        MailBoxView.handlerOnFail
                );
            }, this));
}

MailBoxView.prototype.onAskMentorResources = function(contact)
{
    Service.mail.callMethod("askResourcesFromMentor", [contact.user.username],
    		$.proxy(function(mail)
    		{
    			this.handlerOnResult(null, I18.f("ASKED_MENTOR_RESOURCES"), [FundsBox.createAskMentorFunds()])(mail);
    		}, this), 
            MailBoxView.handlerOnFail
    );
}

MailBoxView.prototype.onExpelApprentice = function(contact)
{
    Alert.create(I18.f("EXPEL_APPRENTICE_CONFIRMATION", contact.user.username), "",
            $.proxy(function()
            {
                Service.mail.callMethod("expelApprentice", [contact.user.username],
                        $.proxy(function(mail)
                        {
                        	this.handlerOnResult()(mail);
                        }, this),
                        MailBoxView.handlerOnFail
                );
            }, this));
}

MailBoxView.prototype.onReleaseApprentice = function(contact)
{
    Alert.create(I18.f("EXPEL_APPRENTICE_CONFIRMATION", contact.user.username), "",
            $.proxy(function()
            {
                Service.mail.callMethod("releaseApprentice", [contact.user.username],
                        $.proxy(function(mail)
                        {
                        	this.handlerOnResult()(mail);
                        }, this),
                        MailBoxView.handlerOnFail
                );
            }, this));
}