function MailConstraints()
{
}

MailConstraints.SEND_MESSAGE_CHECK_ROUTINES =
{
    template: function(username, subject, messageBody)
    {
        return CheckResult.success();
    },

    checkLength: function(username, subject, messageBody)
    {
        if (subject.length > FConfig.SUBJECT_LENGTH_MAX)
        {
            return CheckResult.fail(I18.f("MESSAGE_SUBJECT_TOO_LONG"));
        }
        if (messageBody.length > FConfig.MESSAGE_BODY_LENGTH_MAX)
        {
            return CheckResult.fail(I18.f("MESSAGE_BODY_TOO_LONG"));
        }
        return CheckResult.success();
    }
};

MailConstraints.ADD_CONTACT_CHECK_ROUTINES =
{
    template: function(owner, username, contacts)
    {
        return CheckResult.success();   // do nothing
    },

    checkMyself: function(owner, username, contacts)
    {
        return MailConstraints.checkMyselfGeneral(owner, username, I18.f("ADD_CONTACT_MYSELF_ERROR"));
    },

    checkContactTwice: function(owner, username, contacts)
    {
        return MailConstraints.checkContactTwiceGeneral(owner, username, contacts, I18.f("ADD_CONTACT_TWICE_ERROR"));
    }
};

MailConstraints.IGNORE_CONTACT_CHECK_ROUTINES =
{
    template: function(owner, username, contacts)
    {
        return CheckResult.success();   // do nothing
    },

    checkMyself: function(owner, username, contacts)
    {
        return MailConstraints.checkMyselfGeneral(owner, username, I18.f("IGNORE_CONTACT_MYSELF_ERROR"));
    },

    checkContactTwice: function(owner, username, contacts)
    {
        return MailConstraints.checkContactTwiceGeneral(owner, username, contacts, I18.f("IGNORE_CONTACT_TWICE_ERROR"));
    },

    checkCapacity: function(owner, username, contacts)
    {
        return MailConstraints.checkCapacityGeneral(contacts, FConfig.IGNORE_LIST_CAPACITY, I18.f("IGNORE_LIST_CAPACITY_EXCEEDED"));
    }
};

MailConstraints.checkMyselfGeneral = function(owner, username, object)
{
    if (FUser.isSameName(owner, username))
    {
        return CheckResult.fail(object);
    }
    return CheckResult.success();
}

MailConstraints.checkContactTwiceGeneral = function(owner, username, contacts, object)
{
    for(var i=0; i<contacts.length; i++)
    {
    	if (FUser.isSameName(contacts[i].user.username, username)) 
    	{
            return CheckResult.fail(object);
        }
    }
    return CheckResult.success();
}

MailConstraints.checkCapacityGeneral = function(contacts, limit, object)
{
    if (contacts.length >= limit)
    {
        return CheckResult.fail(object);
    }
    return CheckResult.success();
}

MailConstraints.addContactValid = function(owner, username, contacts)
{
    var f;
    for(var i in MailConstraints.ADD_CONTACT_CHECK_ROUTINES)
    {
        f = MailConstraints.ADD_CONTACT_CHECK_ROUTINES[i];
        
        var result = f(owner, username, contacts);
        if (!result.isSuccess)
        {
            return result;
        }
    }
    return CheckResult.success();
}

MailConstraints.sendMessageValid = function(username, subject, messageBody)
{
    var f;
    for(var i in MailConstraints.SEND_MESSAGE_CHECK_ROUTINES)
    {
        f = MailConstraints.SEND_MESSAGE_CHECK_ROUTINES[i];
        
        var result = f(username, subject, messageBody);
        if (!result.isSuccess)
        {
            return result;
        }
	}
    
    return CheckResult.success();
}

MailConstraints.ignoreContactValid = function(owner, username, contacts)
{
    var f;
    for(var i in MailConstraints.IGNORE_CONTACT_CHECK_ROUTINES)
    {
        f = MailConstraints.IGNORE_CONTACT_CHECK_ROUTINES[i];
    
        var result = f(owner, username, contacts);
        if (!result.isSuccess)
        {
            return result;
        }
    }
    return CheckResult.success();
}