function ObservationReportMessageView()
{
	callParentConstructor(ObservationReportMessageView.prototype, this);
	
	this.isVictim = true;
    this.isInitiator = false;
}

extendClass(ObservationReportMessageView.prototype, SystemMessageView.prototype);

ObservationReportMessageView.prototype.getMessage = function()
{
    return this._message;
}

ObservationReportMessageView.prototype.getSubject = function()
{
    return this.isVictim
            ? I18.f("OBSERVATION_VICTIM_SUBJECT", Strings.safeHtml(this.message.initiatorUsername), Strings.safeHtml(this.message.estateNameAndLocation))
            : I18.f("OBSERVATION_INITIATOR_SUBJECT", Strings.safeHtml(this.message.estateNameAndLocation));
}

ObservationReportMessageView.prototype.createChildren = function()
{
    superMethod(ObservationReportMessageView.prototype, "createChildren", this)();
        
    var result = this.message.result;
    var cityOwner = result.targetOwner;
    
    var prophetsLabel = document.createElement("div");
    prophetsLabel.style.display = "inline-block";
    prophetsLabel.style.width = "50%";
    prophetsLabel.style.textAlign = "center";
    prophetsLabel.className = "text-small";
    prophetsLabel.innerHTML = this.isInitiator ? I18.f("YOUR_PROPHETS") : I18.f("PROPHETS_OF", Strings.safeHtml(this.message.initiatorUsername));
    
    var mirrorsLabel = document.createElement("div");
    mirrorsLabel.style.display = "inline-block";
    mirrorsLabel.style.width = "50%";
    mirrorsLabel.style.textAlign = "center";
    mirrorsLabel.className = "text-small";
    mirrorsLabel.innerHTML = this.isVictim ? I18.f("YOUR_MIRRORS") : I18.f("MIRRORS_OF", Strings.safeHtml(cityOwner.getUsernameForMessages()));
    
    this.addLine(prophetsLabel, mirrorsLabel);
    
    var prophetsView = COM.create("UnitView");
    prophetsView.race = this.message.initiatorRace;
    prophetsView.utype = UnitType.PROPHET;
    prophetsView.showZero = true;
    prophetsView.grayscaleZero = false;
    prophetsView.showCount = true;
    prophetsView.update(1, result.prophetsKilled, -1);
    prophetsView.setInitialKilledPresent(result.prophetsTotal, result.prophetsKilled);
    
    
    var mirrorsView = COM.create("UnitView");
    mirrorsView.race = cityOwner.race;
    mirrorsView.utype = UnitType.MIRROR; 
    mirrorsView.showZero = true;
    mirrorsView.grayscaleZero = false; 
    mirrorsView.showCount = true;
    
    if (result.mirrorsPresent) 
    {
        mirrorsView.update(1, result.mirrorsKilled, -1);
        mirrorsView.setInitialKilledPresent(this.isVictim ? this.message.mirrorsTotal : -1, result.mirrorsKilled);
    } 
    else 
    {
        mirrorsView.showUnitBackIfZero = true;
        mirrorsView.update(1, 0, -1);
    }        
    
    var c1 = document.createElement("div");
    c1.style.display = "inline-block";
    c1.style.width = "50%";
    c1.style.textAlign = "center";
    c1.appendChild(prophetsView.DOM);
    
    var c2 = document.createElement("div");
    c2.style.display = "inline-block";
    c2.style.width = "50%";
    c2.style.textAlign = "center";
    c2.appendChild(mirrorsView.DOM);
    
    this.addLine(c1,c2);       
    
    var accessDescription = document.createElement("div");
    accessDescription.className = "text-small";

    if (result.isSuccess) 
    {
        accessDescription.innerHTML = this.isVictim
                ? I18.f("YOU_WERE_OBSERVED", Strings.safeHtml(this.message.initiatorUsername))
                : I18.f("YOU_GOT_ACCESS_TO_MISSIONS", Strings.safeHtml(cityOwner.getUsernameForMessages()));
        //UI.show(!this.isVictim,[showReport]);
    } 
    else 
    {
        accessDescription.innerHTML = this.isVictim
                ? I18.f("YOU_WERE_OBSERVED_NO_ACCESS", Strings.safeHtml(this.message.initiatorUsername))
                : I18.f("YOU_GOT_NO_ACCESS_TO_MISSIONS", Strings.safeHtml(cityOwner.getUsernameForMessages()));       
        //UI.show(false,[showReport]);
    }
    
    this.addLine(accessDescription);
}

ObservationReportMessageView.prototype.setReportButton = function()
{
	var showReport = COM.create("SimpleButton");
	COM.bindExclusiveClick(showReport.DOM, $.proxy(function()
    {
    	this.onShowReport();
    	//this.parent.close();
    }, this));
    
    if (this.message.result.isSuccess) 
    {
        UI.show(!this.isVictim,[showReport]);
    } 
    else 
    {
        UI.show(false,[showReport]);
    }
    return showReport;
}

ObservationReportMessageView.prototype.setForwardButton = function()
{
	var mailForvard = COM.create("SimpleButton");
    COM.bindExclusiveClick(mailForvard.DOM, $.proxy(function()
    {
    	this.renderer.onForwardMessage(this.message);
    }, this));    
    return mailForvard;
}

ObservationReportMessageView.prototype.setReplyButton = function()
{
	if(this.isVictim)
    {    	    	
    	var replyButton = COM.create("SimpleButton");
    	COM.bindExclusiveClick(replyButton.DOM, $.proxy(function()
	    {
	    	this.renderer.onReplyOnMessage(this.message.initiatorUsername, '', '', '');
	    }, this));
	    
	    return replyButton;
    }
}

ObservationReportMessageView.prototype.setAddContactButton = function()
{
	if(this.isVictim)
    {
	    var addContactButton = COM.create("SimpleButton");
    	COM.bindExclusiveClick(addContactButton.DOM, $.proxy(function()
	    {
	    	this.renderer.onAddContactFromMessage(this.message.initiatorUsername);
	    }, this));
	    return addContactButton;
    }	
}

ObservationReportMessageView.prototype.setMessage = function(m)
{
    superMethod(ObservationReportMessageView.prototype, "setMessage", this)(m);
    this.isVictim = !this.message.isForwarded && this.message.initiatorUsername != this.message.receiver;
    this.isInitiator = !this.message.isForwarded && this.message.initiatorUsername == this.message.receiver;
}

ObservationReportMessageView.prototype.onShowReport = function()
{
	Service.mail.callMethod("getObservationDetails", [MailBoxView.i().ownerUsername, this.message.id],
		function(result)
		{
			MissionListDialog.showObservationResult(result)
		});
}