function SystemMessageView()
{
	callParentConstructor(SystemMessageView.prototype, this);
	this._message = null;
}

extendClass(SystemMessageView.prototype, MessageView.prototype);

SystemMessageView.create = function(message, renderer)
{
	var view = SystemMessageView.createViewByMessage(message);
    view.renderer = renderer;
    //console.log(message.messageClass);
    view.setMessage(message);
    $(renderer.DOM).addClass('Message-' + message.messageClass);
    return view;
}

SystemMessageView.createViewByMessage = function(message)
{
	switch (message.messageClass)
    {
    	case "BuildingSystemMessage":
        {
	        if(message.buildingType.name == BType.RECOVERY.name)
	        {
	            return new RecoveryMessageView();
	        }
	        else if (message.buildingType.name == BType.TERRAFORMER.name)
	        {
	            return new TerraformerMessageView();
	        }
     	}
        break;
        case "ResourceCellExtractedSystemMessage":
             return new ResourceCellExtractedView();
        case "ScienceSystemMessage":
        	return new ScienceResearchCompleteView();
        case "GateOpenMessage":
        	return new GateWasOpenedView();
        case "TunnelClosedSystemMessage":
        	return new PlainTextView(I18.f("SYSTEM_MESSAGE_TUNNEL_CLOSED_SUBJECT", message.estateNameAndLocation),
                    I18.f("SYSTEM_MESSAGE_TUNNEL_CLOSED_BODY"));
        case "MissionSettlerMessage":
        	return new MissionSettlerResultView();
        case "MissionCatapultMessage":
        	return new MissionCatapultResultView();
        case "MissionMoveResourcesMessage":
        	return new MissionMoveResourcesView();
        case "MissionTradeResultMessage":
        	return new MissionTradeResultMessageView();
        case "MissionMoveArmyMessage":
        	return new MissionMoveArmyView();
        case "MissionCampMessage":
        	return new MissionCampView();
        case "NotEnoughPaymentSystemMessage":
        	return new PlainTextView(I18.f("SYSTEM_MESSAGE_SUBJECT_NOT_ENOUGH_PAYMENT"),
                    I18.f("SYSTEM_MESSAGE_BODY_NOT_ENOUGH_PAYMENT", message.estateNameAndLocation));
        case "NotEnoughStorageSystemMessage":
        	return new PlainTextView(I18.f("SYSTEM_MESSAGE_SUBJECT_NOT_ENOUGH_STORAGE"),
                    I18.f("SYSTEM_MESSAGE_BODY_NOT_ENOUGH_STORAGE", message.estateNameAndLocation));
        case "NotEnoughHousesSystemMessage":
        	return new PlainTextView(I18.f("SYSTEM_MESSAGE_SUBJECT_NOT_ENOUGH_HOUSES"),
                    I18.f("SYSTEM_MESSAGE_BODY_NOT_ENOUGH_HOUSES", message.estateNameAndLocation));
        case "MissionCaptureMessage":
        case "MissionLootMessage":
        case "MissionDestroyBuildingsMessage":
        case "MissionSlaveTakingMessage":
        case "MissionDestroyCampMessage":
        case "MissionDestroyCatapultMessage":
        case "MissionDestroyCityMessage":
        case "MissionRescueHeroMessage":
        	return new MissionAttackMessageView();
        case "CombatCalculationMessage":
        	return new CombatCalculationMessageView();
        case "EspionageMessage":
        	return new EspionageReportMessageView();
        case "ObservationMessage":
        	return new ObservationReportMessageView();
        case "GateClosedByPlayerMessage":
        	return new GateWasClosedView();
        case "GateClosedMissionFailedMessage":
        	var cityStr = message.toEstateNameAndLocation();
            return new PlainTextView(I18.f("SYSTEM_MESSAGE_SUBJECT_GATE_CLOSED_MISSION_FAILED", cityStr),
                    I18.f("SYSTEM_MESSAGE_BODY_GATE_CLOSED_MISSION_FAILED", cityStr));
        case "UserOnVacationMissionFailedMessage":
        	var fmmm = message;
            if (!fmmm.estateOwner.username)
            {
                return new PlainTextView(I18.f("SYSTEM_MESSAGE_SUBJECT_USER_ON_VACATION_MISSION_FAILED"),
                    I18.f("SYSTEM_MESSAGE_BODY_USER_ON_VACATION_MISSION_FAILED", fmmm.toEstateNameAndLocation(), fmmm.toEstateOwnerUsername));
            }
            else
            {
                return new PlainTextView(I18.f("SYSTEM_MESSAGE_SUBJECT_OWNER_ON_VACATION_MISSION_FAILED"),
                    I18.f("SYSTEM_MESSAGE_BODY_OWNER_ON_VACATION_MISSION_FAILED", fmmm.toEstateNameAndLocation(), fmmm.toEstateOwnerUsername));
            }
        case "MissionAssistanceMessage":
        	return new MissionAssistanceMessageView();
        case "MissionAncientsAssistanceMessage":
        	return new MissionAncientsAssistanceMessageView();    
        case "GateClosedBuildingsCorruptedMessage":
        	cityStr = message.estateNameAndLocation;
            return new PlainTextView(I18.f("SYSTEM_MESSAGE_SUBJECT_GATE_CLOSED_BUILDINGS_DAMAGED", cityStr),
                    I18.f("SYSTEM_MESSAGE_BODY_GATE_CLOSED_BUILDINGS_DAMAGED", cityStr));
        case "AllianceOfferMessage":
        	return new AllianceMessageView(I18.f("ALLIANCE_OFFER_MESSAGE_SUBJECT"), I18.f("ALLIANCE_OFFER_MESSAGE_BODY", message.username));
        case "AllianceOfferCancelledMessage":
        	return new AllianceMessageView(I18.f("ALLIANCE_OFFER_CANCELLED_MESSAGE_SUBJECT"), I18.f("ALLIANCE_OFFER_CANCELLED_MESSAGE_BODY", message.username));
        case "AllianceOfferAcceptedMessage":
        	return new AllianceMessageView(I18.f("ALLIANCE_OFFER_ACCEPTED_MESSAGE_SUBJECT"), I18.f("ALLIANCE_OFFER_ACCEPTED_MESSAGE_BODY", message.username));
        case "AllianceProposalDeclinedMessage":
        	return new AllianceMessageView(I18.f("ALLIANCE_PROPOSAL_DECLINED_MESSAGE_SUBJECT"), I18.f("ALLIANCE_PROPOSAL_DECLINED_MESSAGE_BODY", message.username));
        case "AllianceBrokenMessage":
        	return new AllianceMessageView(I18.f("ALLIANCE_BROKEN_MESSAGE_SUBJECT"), I18.f("ALLIANCE_BROKEN_MESSAGE_BODY", message.username));
        case "AllianceBreakStartedMessage":
        	return new AllianceBreakStartedMessageView(I18.f("ALLIANCE_BREAK_STARTED_MESSAGE_SUBJECT"), "ALLIANCE_BREAK_STARTED_MESSAGE_BODY");
        case "GroupMissionAttackInvitationMessage":
        	return new PlainTextView(I18.f("GROUP_ATTACK_INVITATION_MESSAGE_SUBJECT"),
                    I18.f("GROUP_ATTACK_INVITATION_MESSAGE_BODY", message.username, message.estateNameAndLocation, message.getAttackGoalText()));
		case "MissionExpeditionMessage":
        	return new MissionExpeditionResultView(message);
        case "ExpeditionFailedMessage":
        	return new ExpeditionFailedView(message);
        case "MissionCanceledByInitiatorMessage":
        	var enemyEstateName = message.toEstateNameAndLocation();
            var enemyName = FUser.getUserNameForMessages(message.toEstateOwnerUsername);
            return new PlainTextView(
                    I18.f("SYSTEM_MESSAGE_ATACK_CANCELED_BY_INITIATOR_SUBJECT"),
                    I18.f("SYSTEM_MESSAGE_ATACK_CANCELED_BY_INITIATOR_BODY", enemyName, enemyEstateName));
        case "SpecialUnitDiedInEstateMessage":
        	return new SpecialUnitDiedInEstateView(message.unitType);
        case "MissionEscapeMessage":
        	return new MissionEscapeView();
        case "BanExpiredMessage":
        	return new BanExpiredView();
        case "MonsterMessage":
        	return new MonsterMessageView();
        case "CautiousAttackersMessage":
        	return new CautiousAttackerMessageView();
        case "AutoHideoutMessage":
        	return new PlainTextView(
                    I18.f("SYSTEM_MESSAGE_AUTO_HIDEOUT_NO_MONEY_SUBJECT"),
                    I18.f("SYSTEM_MESSAGE_AUTO_HIDEOUT_NO_MONEY_BODY", message.estateNameAndLocation));
        case "AuctionPayFailMessage":
        	return new AuctionPayFailView();
        case "AuctionWinMessage":
        	return new AuctionWinView();
        case "AuctionStakeOutbidMessage":
        	return new AuctionStakeOutbidView();
        case "GiftMessage":
        	return new GiftMessageView();
        case "ShareOuterEstateMessage":
        	return new ShareOuterEstateMessageView();
        case "BrotherhoodExecutedMessage":
        	return new BrotherhoodExecutedMessageView();
        case "ClanInvitationRejectedMessage":
        	return new ClanInvitationRejectedMessageView();
        case "LeaveClanMessage":
        	return new LeaveClanMessageView();
        case "ClanLeaderChangeMessage":
        	return new ClanLeaderChangeMessageView();
        case "ClanReportMessage":
        	return new ClanReportMessageView();
        case "ClanResourcesReceivedMessage":
        	return new ClanResourceReceivedMessageView();
        case "DeputyAssignmentMessage":
        	return new PlainTextView(I18.f("DEPUTY_ASSIGNMENT_SUBJECT"), I18.f("DEPUTY_ASSIGNMENT_BODY", message.ally));
        case "DeputyCancelMessage":
        	if (message.canceledByOwner)
            {
                return new PlainTextView(I18.f("DEPUTY_CANCELED_BY_OWNER_SUBJECT"), I18.f("DEPUTY_CANCELED_BY_OWNER_BODY", message.ally));
            }
            else
            {
                return new PlainTextView(I18.f("DEPUTY_ASSIGNMENT_ROLLBACK_SUBJECT"), I18.f("DEPUTY_ASSIGNMENT_ROLLBACK_BODY", message.ally));
            }
        case "ShowOuterEstatesMessage":
        	return new ShowOuterEstatesView();
        case "PersonalEventMessage":
        	return new PersonalEventMessageView();
        case "CastMessage":
        	return new CastMessageView();
        case "ApprenticeInvitationMessage":
        	return new ApprenticeInvitationView();
        case "PhantomMissionMessage":
        	return new PlainTextView(
                    I18.f("PHANTOM_MISSION_ARRIVED_SUBJECT", FUser.getUserNameForMessages(message.estateOwner.username), message.fromEstateNameAndLocation()),
                    I18.f("PHANTOM_MISSION_ARRIVED_BODY", message.toEstateNameAndLocation(), FUser.getUserNameForMessages(message.estateOwner.username), message.fromEstateNameAndLocation()));
		case "RuinLegacyMessage":
        	return new RuinLegacyMessageView();
        case "ApprenticeAskedResourcesMessage":
        	return new ApprenticeAskedResourcesView();
        case "MentorGaveResourcesMessage":
        	return new MentorGaveResourcesView();
        case "BuildingsEffectsMessage":
        	return new BuildingsEffectsView();
        case "TournamentFinishedMessage":
        	return new TournamentFinishedMessageView();
        case "CatapultSiegeMessage":
			return new CatapultSiegeView();
        default:
        {
        	return new SimpleMessage();
        }
    }
    throw Error("Could not find message view by class " + message.messageClass);
}

SystemMessageView.prototype.setMessage = function(m)
{
    this._message = m;
}

SystemMessageView.prototype.addLine = function(items)
{	
    if(!$.isArray(items)) items = Array.prototype.slice.call(arguments, 0);    
    var box = document.createElement("div");

    for(var i=0; i<items.length; i++)
    {
    	var d = items[i];
    	$(box).append(d);
   	}
   	this.DOM.appendChild(box);
   	return box;
}

SystemMessageView.prototype.imageWithToolTip = function(source, tooltipFactoryMethod, interfacePath)
{
    var i = document.createElement("img");
    i.align = "absmiddle";
    
    if(source && (typeof source != "string") && source.src) i.src = source.src;
    else i.src = (interfacePath ? STATIC_INTERFACE_URL : STATIC_URL) + source;
    
    return i;
}

SystemMessageView.prototype.assignToolTip = function(c, tooltipFactoryMethod)
{
}

SystemMessageView.prototype.addText = function(text)
{
    this.addLine(this.createText(text));
}

SystemMessageView.prototype.addReactionView = function()
{
    var mrv = COM.create("MessageReactionView");
    mrv.renderer = this.renderer;
    mrv.update();

    this.DOM.appendChild(mrv.DOM);
}

SystemMessageView.prototype.getBuildingName = function(bt)
{
    return I18.getBuildingName(!bt ? this._message.buildingType : bt, FUser.CURRENT_USER.race);
}

SystemMessageView.prototype.handleMentorShipSuccess = function(result)
{
    this.parent.close();
    MailBoxView.i().refresh();
}

SystemMessageView.prototype.handleMentorshipError = function(e)
{
    if ((e.__className__ == 'NotEnoughResourcesException')) 
    {
        Alert.create(e.getMessage());
        return true;
    }
    if ((e.__className__ == 'MentorshipException')) 
    {
        Alert.create(e.getMessage());
        MailBoxView.i().refresh();
        return true;
    }
    return false;
}

SystemMessageView.prototype.createText = function(text)
{
	var lbl = document.createElement("span");
    lbl.innerHTML = Strings.safeHtml(text);
    return lbl;
}

SystemMessageView.prototype.addImage = function(src, gamePack)
{
	var img = document.createElement("img");
    img.src = (gamePack ? STATIC_URL : STATIC_INTERFACE_URL) + src;
    return img;
}

//////////////////////////////////

function BuildingMessageView()
{
	callParentConstructor(BuildingMessageView.prototype, this);
}

extendClass(BuildingMessageView.prototype, SystemMessageView.prototype);

BuildingMessageView.prototype.createBuildingImage = function(bt)
{
    return this.imageWithToolTip(this.findImage(FUser.CURRENT_USER.race, bt));
}

BuildingMessageView.prototype.getText = function()
{
}

BuildingMessageView.prototype.createChildren = function()
{
    superMethod(BuildingMessageView.prototype, "createChildren", this)();
    this.addLine(this.createBuildingImage(this.getBuildingType()), this.createText(this.getText()));
}

BuildingMessageView.prototype.getBuildingType = function()
{
    return this._message.buildingType;
}

BuildingMessageView.prototype.findImage = function(r, bt)
{
	var result = Assets.B_findByOrNull(r, bt, null, 0, null);
	if (!result) 
	{
		result = Assets.B_findBy(Race.MONSTER, bt, null, 0, null);
	}
	return result;
}

//////////////////////////////////

function AllianceMessageView(subject, body)
{
	callParentConstructor(AllianceMessageView.prototype, this);
	
	this.subject = subject;
    this.body = body;
}

extendClass(AllianceMessageView.prototype, BuildingMessageView.prototype);

AllianceMessageView.prototype.getSubject = function()
{
	return this.subject;
}

AllianceMessageView.prototype.getText = function()
{
	return this.body;
}

AllianceMessageView.prototype.getBuildingType = function()
{
    return BType.DIPLOMAT;
}

AllianceMessageView.prototype.imageWithToolTip = function(source, tooltipFactoryMethod)
{
    var i = document.createElement("img");
    i.align = "absmiddle";
    i.src = STATIC_URL+source;
    return i;
}

//////////////////////////////////

function AllianceBreakStartedMessageView(subject, body)
{
	callParentConstructor(AllianceBreakStartedMessageView.prototype, this, subject, body);
	this.subject = subject;
	this.body = body;
}

extendClass(AllianceBreakStartedMessageView.prototype, AllianceMessageView.prototype);




AllianceBreakStartedMessageView.prototype.getText = function()
{	
	return I18.f(this.body,this._message.username, Tick.happensAt(this._message.allianceBreakingTick));
}

AllianceBreakStartedMessageView.prototype.imageWithToolTip = function(source)
{
    var i = document.createElement("img");
    i.src = STATIC_URL + source;
    return i;
}

//////////////////////////////////

function TerraformerMessageView()
{
	callParentConstructor(TerraformerMessageView.prototype, this);
}

extendClass(TerraformerMessageView.prototype, BuildingMessageView.prototype);

TerraformerMessageView.prototype.getSubject = function()
{
    return I18.f("SYSTEM_MESSAGE_TERRAFORMER_HEADER");
}

TerraformerMessageView.prototype.getText = function()
{
	var text = I18.f("SYSTEM_MESSAGE_TERRAFORMER_BODY", this._message.estateNameAndLocation, this._message.cellNo + 1);
    if (this._message.cellResource.name != Resource.NULL.name)
    {
        text += ".\n" + I18.f("SYSTEM_MESSAGE_TERRAFORMER_EXTRA", I18.getResourceName(this._message.cellResource));
    }
    
    return text;
}

//////////////////////////////////

function RecoveryMessageView()
{
	callParentConstructor(RecoveryMessageView.prototype, this);
}

extendClass(RecoveryMessageView.prototype, BuildingMessageView.prototype);

RecoveryMessageView.prototype.getSubject = function()
{
    return I18.f("SYSTEM_MESSAGE_RECOVERY_HEADER");
}

RecoveryMessageView.prototype.getText = function()
{
    return I18.f("SYSTEM_MESSAGE_RECOVERY_BODY", this._message.estateNameAndLocation,
            this.getBuildingName(),
            I18.getResourceName(this._message.cellResource),
            this._message.cellNo + 1);
}

//////////////////////////////////

function BanExpiredView()
{
	callParentConstructor(BanExpiredView.prototype, this);
}

extendClass(BanExpiredView.prototype, BuildingMessageView.prototype);

BanExpiredView.prototype.getSubject = function()
{
    return I18.f("SYSTEM_MESSAGE_SUBJECT_BAN_EXPIRED", this._message.username);
}

BanExpiredView.prototype.getText = function()
{
    return I18.f("SYSTEM_MESSAGE_BODY_BAN_EXPIRED", this._message.username);
}

BanExpiredView.prototype.getBuildingType = function()
{
    return BType.TEMPLE;
}

//////////////////////////////////

function MonsterMessageView()
{
	callParentConstructor(MonsterMessageView.prototype, this);
}

extendClass(MonsterMessageView.prototype, SystemMessageView.prototype);

MonsterMessageView.prototype.createChildren = function()
{
    superMethod(MonsterMessageView.prototype, "createChildren", this)();
    this.addLine();
    this.addLine(this.imageWithToolTip(KingdomAssets.RUIN, null, true), this.createText(this._message.text));

    if (this._message.username && this._message.text.length > 0 || this._message.canForward)
    {
        this.addReactionView();
    }
}

//////////////////////////////////

function CautiousAttackerMessageView()
{
	callParentConstructor(CautiousAttackerMessageView.prototype, this);
}

extendClass(CautiousAttackerMessageView.prototype, SystemMessageView.prototype);

CautiousAttackerMessageView.prototype.getSubject = function()
{
	var m = this._message;
    var armyName = CombatViewUtils.armyName(m, m.username);
    var heroName = CombatViewUtils.heroName(m, m.username, m.heroname);
    
    return m.isCautiousDefender()
            ? I18.f("CAUTIOUS_DEFENDER_SUBJECT", heroName, armyName)
            : m.attackCanceled
                ? I18.f("CAUTIOUS_ALL_SUBJECT", heroName, armyName)
                : I18.f("CAUTIOUS_ONE_SUBJECT", heroName, armyName);
}

CautiousAttackerMessageView.prototype.createChildren = function()
{
	superMethod(CautiousAttackerMessageView.prototype, "createChildren", this)();
    
    var m = this._message;
    var armyName = CombatViewUtils.armyName(m, m.username);
    var heroName = CombatViewUtils.heroName(m, m.username, m.heroname);
    var text = m.isCautiousDefender()
            ? I18.f("CAUTIOUS_DEFENDER_BODY", heroName, armyName)
            : m.attackCanceled
                ? I18.f("CAUTIOUS_ALL_BODY", heroName, armyName)
                : I18.f("CAUTIOUS_ONE_BODY", heroName, armyName);
    
    var c = CastView.create(m.cast, [], true);
	this.addLine(c.DOM, this.createText(text));
    
    if (m.isCautiousDefender())
    {
    	    	
    	var f = COM.create("FundsBox");
    	f.update(m.resources, FUser.CURRENT_USER.race);
    	    	
        this.addLine(this.createText(I18.f("PRICE_SHORT")), f.DOM);
	}
}

//////////////////////////////////

function ScienceResearchCompleteView()
{
	callParentConstructor(ScienceResearchCompleteView.prototype, this);
}

extendClass(ScienceResearchCompleteView.prototype, SystemMessageView.prototype);

ScienceResearchCompleteView.prototype.getSubject = function()
{
    var s = I18.getScience(this._message.science);
    return this._message.auto ?
           I18.f("SYSTEM_MESSAGE_SUBJECT_SCIENCE_COMPLETE", s) : I18.f("SYSTEM_MESSAGE_SUBJECT_SCIENCE_CHARGED", s);
}

ScienceResearchCompleteView.prototype.createChildren = function()
{
    superMethod(ScienceResearchCompleteView.prototype, "createChildren", this)();
    var body =  this._message.auto ? "SYSTEM_MESSAGE_BODY_SCIENCE_COMPLETE" : "SYSTEM_MESSAGE_BODY_SCIENCE_CHARGED";
    var img = this.addImage(Assets.getScienceImage(this._message.science));
    img.align = "left";
    this.addLine(img, this.createText(I18.f(body,I18.getScience(this._message.science), this._message.scienceLevel + 1)));
}

//////////////////////////////////

function ResourceCellExtractedView()
{
	callParentConstructor(ResourceCellExtractedView.prototype, this);
}

extendClass(ResourceCellExtractedView.prototype, SystemMessageView.prototype);

ResourceCellExtractedView.prototype.getSubject = function()
{
    return I18.f("SYSTEM_MESSAGE_SUBJECT_RESOURCE_CELL_EXTRACTED");
}

ResourceCellExtractedView.prototype.createChildren = function()
{
    superMethod(ResourceCellExtractedView.prototype, "createChildren", this)();
	var s = NumberUtils.equals(this._message.sectorNo, -1)
            ? "SYSTEM_MESSAGE_BODY_RESOURCE_CELL_EXTRACTED_ESTATE"
            : "SYSTEM_MESSAGE_BODY_RESOURCE_CELL_EXTRACTED_SECTOR";
            
    this.addLine(this.createText(I18.f(s, this._message.estateNameAndLocation, this._message.cellNo + 1, I18.getResourceName(this._message.cellResource))));
}

//////////////////////////////////

function GateWasOpenedView()
{
	callParentConstructor(GateWasOpenedView.prototype, this);
}

extendClass(GateWasOpenedView.prototype, SystemMessageView.prototype);

GateWasOpenedView.prototype.getSubject = function()
{
    var message = this._message;
    var location = this._message.estateNameAndLocation;

    var s = (message.estateType.name == EstateType.CITY.name ?
            (message.gateDestroyed ? I18.f("SYSTEM_MESSAGE_SUBJECT_GATE_DESTROYED",location) : I18.f("SYSTEM_MESSAGE_SUBJECT_GATE_OPENED",location)) :
            (message.gateDestroyed ? I18.f("SYSTEM_MESSAGE_TUNNEL_DESTROYED_SUBJECT",location) : I18.f("SYSTEM_MESSAGE_TUNNEL_OPEN_SUBJECT",location)));

    return s;
}

GateWasOpenedView.prototype.createChildren = function()
{
    superMethod(GateWasOpenedView.prototype, "createChildren", this)();
    var message = this._message;
    if (message.estateType.name == EstateType.CITY.name)
    {
    	this.addLine(this.addImage(Assets.WALL_23_OPEN(FUser.CURRENT_USER.race, message.gateDestroyed), true),
                this.createText(I18.f("SYSTEM_MESSAGE_BODY_GATE_OPENED", this.getBuildingName(BType.CITYHALL))));
    }
    else
    {
		this.addLine(this.createText(I18.f("SYSTEM_MESSAGE_TUNNEL_OPEN_BODY")));
	}
}

//////////////////////////////////

function GateWasClosedView()
{
	callParentConstructor(GateWasClosedView.prototype, this);
}

extendClass(GateWasClosedView.prototype, SystemMessageView.prototype);

GateWasClosedView.prototype.getSubject = function()
{
    return I18.f("SYSTEM_MESSAGE_SUBJECT_GATE_CLOSED", this._message.estateNameAndLocation);
}

GateWasClosedView.prototype.createChildren = function()
{
    superMethod(GateWasClosedView.prototype, "createChildren", this)();
    var corruptionDays = Math.floor(Tick.toDays(FConfig.CORRUPT_BUILDINGS_CLOSED_GATE_TIMEOUT));

    var text = FConfig.GATE_STATE_AFFECTS_EXPEDITORS ? I18.f("SYSTEM_MESSAGE_BODY_GATE_CLOSED_WITH_EXPEDITORS", corruptionDays)
            : I18.f("SYSTEM_MESSAGE_BODY_GATE_CLOSED", corruptionDays);
    this.addLine(this.addImage(Assets.WALL_23_CLOSED(FUser.CURRENT_USER.race), true), this.createText(text));
}

//////////////////////////////////

function PlainTextView(subject, text)
{
	callParentConstructor(PlainTextView.prototype, this);
	
	this.subject = subject;
    this.text = text;
}

extendClass(PlainTextView.prototype, SystemMessageView.prototype);

PlainTextView.prototype.getSubject = function()
{
    return this.subject;
}

PlainTextView.prototype.createChildren = function()
{
    superMethod(PlainTextView.prototype, "createChildren", this)();
    this.addLine(this.createText(this.text));
}

//////////////////////////////////

function AuctionWinView()
{
	callParentConstructor(AuctionWinView.prototype, this);
}

extendClass(AuctionWinView.prototype, SystemMessageView.prototype);

AuctionWinView.prototype.getSubject = function()
{
    return I18.f("AUCTION_WIN");
}

AuctionWinView.prototype.createChildren = function()
{
    superMethod(AuctionWinView.prototype, "createChildren", this)();
    
    with (this.DOM.appendChild(document.createElement('inline-table')))
    {
	    with (appendChild(document.createElement('row')))
	    {
		    with (appendChild(document.createElement('cell')))
		    {
		    	innerHTML = I18.f("LOT_WON");
		    	style.textAlign = 'left';
		    	style.padding = '8px';
		    }
		    with (appendChild(document.createElement('cell')))
		    {
		    	innerHTML = '<img src="' + STATIC_INTERFACE_URL + CommonAssets.ICON_PEARL + '" />';
		    	style.padding = '8px';
		    }
		    with (appendChild(document.createElement('cell')))
		    {
		    	innerHTML = this._message.pearlCount;
		    	style.textAlign = 'left';
		    	style.padding = '8px';
		    }
	    }
	    
	    with (appendChild(document.createElement('row')))
	    {
		    with (appendChild(document.createElement('cell')))
		    {
		    	innerHTML = I18.f("YOUR_STAKE");
		    	style.textAlign = 'left';
		    	style.padding = '8px';
		    }
		    with (appendChild(document.createElement('cell')))
		    {
		    	innerHTML = '<img src="' + STATIC_INTERFACE_URL + CommonAssets.ICON_GOLD + '" />';
		    	style.padding = '8px';
		    }
		    with (appendChild(document.createElement('cell')))
		    {
		    	innerHTML = this._message.price;
		    	style.textAlign = 'left';
		    	style.padding = '8px';
		    }
	    }
    }
}

//////////////////////////////////

function AuctionPayFailView()
{
	callParentConstructor(AuctionPayFailView.prototype, this);
}

extendClass(AuctionPayFailView.prototype, SystemMessageView.prototype);

AuctionPayFailView.prototype.getSubject = function()
{
    return I18.f("PAY_FAIL_MESSAGE_SUBJECT");
}

AuctionPayFailView.prototype.createChildren = function()
{
    superMethod(AuctionPayFailView.prototype, "createChildren", this)();
    
    this.addText(I18.f("PAY_FAIL_MESSAGE_BODY", Tick.toTimeFuture(this._message.bannedTill)));
}

//////////////////////////////////

function AuctionStakeOutbidView()
{
	callParentConstructor(AuctionStakeOutbidView.prototype, this);
}

extendClass(AuctionStakeOutbidView.prototype, SystemMessageView.prototype);

AuctionStakeOutbidView.prototype.getSubject = function()
{
    return I18.f("STAKE_OUTBID_MESSAGE_SUBJECT");
}

AuctionStakeOutbidView.prototype.createChildren = function()
{
    superMethod(AuctionStakeOutbidView.prototype, "createChildren", this)();
    this.addText(I18.f("STAKE_OUTBID_MESSAGE_BODY", this._message.username));
}

//////////////////////////////////

function SpecialUnitWithTextView(subject, text, type)
{
	callParentConstructor(SpecialUnitWithTextView.prototype, this);
	
	this.subject = subject;
    this.text = text;
    this.type = type;
}

extendClass(SpecialUnitWithTextView.prototype, SystemMessageView.prototype);

SpecialUnitWithTextView.prototype.getSubject = function()
{
    return this.subject;
}

SpecialUnitWithTextView.prototype.createChildren = function()
{
    superMethod(SpecialUnitWithTextView.prototype, "createChildren", this)();
    this.addLine(this.addImage(Assets.getUnitImageFor(FUser.CURRENT_USER.race, this.type, 1)), this.createText(this.text));
}

SpecialUnitWithTextView.prototype.getUnitName = function()
{
    return I18.getUnitName(this.type, 1, FUser.CURRENT_USER.race);
}

//////////////////////////////////

function SpecialUnitDiedInEstateView(type)
{
	callParentConstructor(SpecialUnitDiedInEstateView.prototype, this, "", "", type);
}

extendClass(SpecialUnitDiedInEstateView.prototype, SpecialUnitWithTextView.prototype);

SpecialUnitDiedInEstateView.prototype.getSubject = function()
{
    return I18.f("SYSTEM_MESSAGE_SPEC_UNIT_DIED_SUBJECT", this.getUnitName(), this._message.fromEstateNameAndLocation());
}

SpecialUnitDiedInEstateView.prototype.createChildren = function()
{
	this.text = I18.f("SYSTEM_MESSAGE_SPEC_UNIT_DIED_BODY", this.getUnitName(), this._message.fromEstateNameAndLocation());
	superMethod(SpecialUnitDiedInEstateView.prototype, "createChildren", this)();
}

//////////////////////////////////

function MissionExpeditionResultView(m)
{
	callParentConstructor(MissionExpeditionResultView.prototype, this);
	
	this.m = m;
}

extendClass(MissionExpeditionResultView.prototype, SystemMessageView.prototype);

MissionExpeditionResultView.prototype.getSubject = function()
{
    var estateType = this.m.expeditionResult.getEstateType();
    var estateName = !estateType ? "" : I18.getOuterEstateName(estateType, this.m.resource ? [this.m.resource.name] : null);
    var subject = I18.s("EXPEDITOR", this.m.expeditionResult.name + "_SUBJECT", [this.getSource(), estateName, this.m.estateLevel == 0 ? "?" : this.m.estateLevel]);
    
    if (this.m.foundFlag >= 0)
    {
        subject += I18.f("EXPEDITOR_FOUND_FLAG_SUBJECT_SUFFIX", I18.getFlagName(this.m.foundFlag));
    }
    return subject;
}

MissionExpeditionResultView.prototype.getSource = function()
{
    switch (this.m.source.name)
    {
        case MissionExpeditionSource.EXPLORER.name:
            return I18.f("EXPEDITOR_UNIT_SUBJECT", this.m.toFieldXYZ.toString());
        case MissionExpeditionSource.SETTLER.name:
            return I18.f("EXPEDITOR_SETTLER_SUBJECT", this.m.toFieldXYZ.toString());
        case MissionExpeditionSource.CATAPULT.name:
            return I18.f("EXPEDITOR_CATAPULT_SUBJECT", this.m.toFieldXYZ.toString());
        case MissionExpeditionSource.CAMPING.name:
            return I18.f("EXPEDITOR_CAMPING_SUBJECT", [this.m.heroName, this.m.toFieldXYZ.toString()]);
        case MissionExpeditionSource.CITY_MOVE.name:
            return I18.f("EXPEDITOR_CITY_MOVE_SUBJECT", this.m.toFieldXYZ.toString());
        case MissionExpeditionSource.CAST.name:
            if (this.m.heroName)
            {
                return I18.f("EXPEDITOR_HERO_SUBJECT", [this.m.heroName, this.m.toFieldXYZ.toString()]);
            }
            else
            {
                return I18.f("EXPEDITOR_CAST_SUBJECT", this.m.toFieldXYZ.toString());
            }
        default:
            return "???";
    }
}

MissionExpeditionResultView.prototype.getBody = function()
{
    switch (this.m.expeditionResult.name)
    {
        case MissionExpeditionResult.FOUND_BUILDABLE_FIELD.name:
            return I18.f("EXPEDITOR_FOUND_BUILDABLE_FIELD_BODY", [this.m.toFieldXYZ.toString(), this.m.cellCount, this.m.maxHallLevel, this.m.terrain ? this.m.terrain.displayName : ""]);
        case MissionExpeditionResult.MINE_FOUND.name:
            return I18.f("EXPEDITOR_MINE_FOUND_BODY", [this.m.toFieldXYZ.toString(), I18.getMineName(this.m.resource.name)]);
        default:
            return I18.s("EXPEDITOR", this.m.expeditionResult.name + "_BODY", this.m.toFieldXYZ.toString());
    }
}

MissionExpeditionResultView.prototype.shouldShowDetails = function()
{
    return (this.m.expeditionResult.name == MissionExpeditionResult.FOUND_BUILDABLE_FIELD.name || this.m.expeditionResult.name == MissionExpeditionResult.FOUND_NOT_BUILDABLE_FIELD.name);
}

MissionExpeditionResultView.prototype.getImage = function()
{
    var founder = null;
    switch (this.m.source.name)
    {
        case MissionExpeditionSource.EXPLORER.name:
            founder = Assets.getUnitImageFor(this.m.expeditionRace, UnitType.EXPEDITOR, 1);
            break;
        case MissionExpeditionSource.SETTLER.name:
            founder = Assets.getUnitImageFor(this.m.expeditionRace, UnitType.SETTLER, 1);
            break;
        case MissionExpeditionSource.CAMPING.name:
        case MissionExpeditionSource.CAST.name:
            if (this.m.heroName)
            {
                var c = document.createElement("div");
                c.style.backgroundImage = "url('" + STATIC_INTERFACE_URL + Assets.getHeroBack(this.m.expeditionRace) + "')";
                c.style.backgroundRepeat = "no-repeat";
                c.style.display = "inline-block";
                /*
                var b = document.createElement("img");
                b.src = STATIC_INTERFACE_URL + Assets.getHeroBack(this.m.expeditionRace);
                c.appendChild(b);
                */
                var f = document.createElement("img");
                f.src = STATIC_INTERFACE_URL + Assets.getHeroFace(this.m.heroFace);
                c.appendChild(f);
                founder = c;
            }
            else
            {
                founder = Assets.getUnitImageFor(this.m.expeditionRace, UnitType.MAGE, 1);
            }
            break;
    }
    
    if(typeof founder == "string")
    {
	    var d = document.createElement("div");
	    var img = document.createElement("img");
	    img.src = STATIC_INTERFACE_URL + founder;
	    d.appendChild(img);
	    
	    return d;
    }
    else
    {
    	return founder;
    }
}

MissionExpeditionResultView.prototype.createChildren = function()
{
    superMethod(MissionExpeditionResultView.prototype, "createChildren", this)();
    this.addLine(this.getImage(), this.createText(this.getBody()));
    this.addLine("&nbsp;");
    this.addLine("&nbsp;");

    if (this.m.foundFlag >= 0)
    {
    	var f = FlagsView.createOne(this.m.foundFlag);
    	this.addLine(this.createText(I18.f("EXPEDITOR_FOUND_FLAG")), f.DOM);
    }
    
    
    /*
    if (this.shouldShowDetails())
    {
    	var details = COM.create("SimpleButton");
        details.setCaption(I18.f("DETAILS"));
        
        COM.bindExclusiveClick(details.DOM, $.proxy(function()
        {
        	Service.mail.callMethod("getExpeditionDetails", [MailBoxView.i().ownerUsername, this._message.id],
        	$.proxy(function(result)
        	{
                result._fromMail = true;
                Events.onShowExpeditionResult.fire(result);
                this.parent.close();
            }, this));
        }, this));

        var forward = COM.create("SimpleButton");
        forward.setCaption(I18.f("MAIL_FORWARD"));
        
        COM.bindExclusiveClick(forward.DOM, $.proxy(function()
        {
        	this.renderer.onForwardMessage(this._message);
        }, this));
        
        this.addLine(details.DOM, forward.DOM);

        
    }
    */
    
    
}


MissionExpeditionResultView.prototype.setDetailsButton = function()
{
	if (this.shouldShowDetails())
    {
    	var details = COM.create("SimpleButton");
        COM.bindExclusiveClick(details.DOM, $.proxy(function()
        {
        	Service.mail.callMethod("getExpeditionDetails", [MailBoxView.i().ownerUsername, this._message.id],
        	$.proxy(function(result)
        	{
                result._fromMail = true;
                Events.onShowExpeditionResult.fire(result);
                this.parent.close();
            }, this));
        }, this));

        return details;
	}
	else return false;
}

MissionExpeditionResultView.prototype.setForwardButton = function()
{
	if (this.shouldShowDetails())
    {
        var forward = COM.create("SimpleButton"); 
        COM.bindExclusiveClick(forward.DOM, $.proxy(function()
        {
        	this.renderer.onForwardMessage(this._message);
        }, this));
        return forward;
    }
    else return false;
}


//////////////////////////////////

function ExpeditionFailedView(m)
{
	callParentConstructor(ExpeditionFailedView.prototype, this);
	this.m = m;
}

extendClass(ExpeditionFailedView.prototype, SystemMessageView.prototype);

ExpeditionFailedView.prototype.getSubject = function()
{
    return I18.f("EXPEDITOR_UNIT_DIED_SUBJECT", this.m.estateNameAndLocation);
}

ExpeditionFailedView.prototype.createChildren = function()
{
    superMethod(ExpeditionFailedView.prototype, "createChildren", this)();
 	this.addLine(this.getImage(Assets.getUnitImageFor(this.m.expeditionRace, UnitType.EXPEDITOR, 1)), this.createText(I18.getMegaString(I18.f("EXPEDITOR_UNIT_DIED_BODY"), [this.m.toFieldXYZ, this.m.deathRate]) + "\n\n"));

}

ExpeditionFailedView.prototype.getImage = function(src)
{
	var f = document.createElement("img");
	f.src = STATIC_INTERFACE_URL + src;
	return f;
}

//////////////////////////////////

function MissionSettlerResultView()
{
	callParentConstructor(MissionSettlerResultView.prototype, this);
}

extendClass(MissionSettlerResultView.prototype, SystemMessageView.prototype);

MissionSettlerResultView.prototype.getSubject = function()
{
    return this._message.result.equals(MissionResult.SUCCESS) ?
           I18.f("SYSTEM_MESSAGE_SUBJECT_SETTLER_SUCCESS") : I18.f("SYSTEM_MESSAGE_SUBJECT_SETTLER_FAIL");
}

MissionSettlerResultView.prototype.createChildren = function()
{
    superMethod(MissionSettlerResultView.prototype, "createChildren", this)();
    var text;
    switch (this._message.settlerResult.name)
    {
        case MissionSettlerResult.SUCCESS.name:
            text = I18.f("SYSTEM_MESSAGE_BODY_SETTLER_SUCCESS", this._message.toFieldXYZ);
            break;
        case MissionSettlerResult.FAIL_UNKNOWN_ESTATE_PRESENT.name:
            text = I18.f("SYSTEM_MESSAGE_BODY_SETTLER_FAIL_ESTATE_PRESENT", this._message.toFieldXYZ);
            break;
        case MissionSettlerResult.FAIL_CITY_PRESENT.name:
            text = I18.f("SYSTEM_MESSAGE_BODY_SETTLER_FAIL_CITY_PRESENT", this._message.toFieldXYZ);
            break;
        case MissionSettlerResult.FAIL_RUIN_PRESENT.name:
            text = I18.f("SYSTEM_MESSAGE_BODY_SETTLER_FAIL_RUIN_PRESENT", this._message.toFieldXYZ, this._message.toEstateLevel);
            break;
        case MissionSettlerResult.FAIL_NON_BUILDABLE.name:
            text = I18.f("SYSTEM_MESSAGE_BODY_SETTLER_FAIL_NON_BUILDABLE", this._message.toFieldXYZ);
            break;
    }
    this.addLine(this.addImage(Assets.getUnitImageFor(FUser.CURRENT_USER.race, UnitType.SETTLER)), this.createText(text));
}

//////////////////////////////////

function MissionCatapultResultView()
{
	callParentConstructor(MissionCatapultResultView.prototype, this);
}

extendClass(MissionCatapultResultView.prototype, SystemMessageView.prototype);

MissionCatapultResultView.prototype.getSubject = function()
{
    return this._message.result.equals(MissionResult.SUCCESS) 
		? I18.f("SYSTEM_MESSAGE_SUBJECT_CATAPULT_SUCCESS") 
		: I18.f("SYSTEM_MESSAGE_SUBJECT_CATAPULT_FAIL")
	;
}

MissionCatapultResultView.prototype.createChildren = function()
{
    superMethod(MissionCatapultResultView.prototype, "createChildren", this)();
    var text;
    switch (this._message.catapultResult.name)
    {
        case MissionCatapultResult.SUCCESS.name:
            text = I18.f("SYSTEM_MESSAGE_BODY_CATAPULT_SUCCESS", this._message.toFieldXYZ);
            break;
        case MissionCatapultResult.FAIL_NON_BUILDABLE.name:
            text = I18.f("SYSTEM_MESSAGE_BODY_CATAPULT_FAIL_NON_BUILDABLE", this._message.toFieldXYZ);
            break;
        case MissionCatapultResult.FAIL_NO_CASTLE.name:
            text = I18.f("SYSTEM_MESSAGE_BODY_CATAPULT_FAIL_NO_CASTLE", this._message.toFieldXYZ);
            break;
    }
    this.addLine(this.addImage(Assets.getUnitImageFor(Race.CASTLE_MONSTER, UnitType.CATAPULT)), this.createText(text));
}

//////////////////////////////////

function CatapultSiegeView()
{
	callParentConstructor(CatapultSiegeView.prototype, this);
}

extendClass(CatapultSiegeView.prototype, BuildingMessageView.prototype);

CatapultSiegeView.prototype.getText = function()
{
	var csm = this._message;
	
	if (csm.catapultOwner)
	{
		if (CatapultSiegeResult.NO_TARGET.name == csm.result.name) 
		{
			return I18.f('YOUR_CATAPULT_SIEGE_NO_TARGET_BODY', csm.catapultLocation, csm.estateXYZ);
		}
		if (CatapultSiegeResult.NO_DAMAGE.name == csm.result.name) 
		{
			return I18.f('YOUR_CATAPULT_SIEGE_NO_DAMAGE_BODY', csm.catapultLocation, csm.estateXYZ);
		}
		if (CatapultSiegeResult.BUILDING_DAMAGED.name == csm.result.name) 
		{
			return I18.f('YOUR_CATAPULT_SIEGE_BUILDING_DAMAGED_BODY', csm.catapultLocation, csm.estateXYZ);
		}
		if (CatapultSiegeResult.BUILDING_DESTROYED.name == csm.result.name) 
		{
			return I18.f('YOUR_CATAPULT_SIEGE_BUILDING_DESTROYED_BODY', csm.catapultLocation, csm.estateXYZ);
		}
	}
	else
	{
		if (CatapultSiegeResult.NO_DAMAGE.name == csm.result.name) 
		{
			return I18.f('CATAPULT_SIEGE_HAS_ATTACKED_YOUR_ESTATE_BODY', csm.catapultLocation, csm.estateXYZ);
		}
		if (CatapultSiegeResult.BUILDING_DAMAGED.name == csm.result.name) 
		{
			return I18.f('CATAPULT_SIEGE_HAS_DAMAGED_YOUR_BUILDING_BODY', csm.catapultLocation, csm.estateXYZ);
		}
		if (CatapultSiegeResult.BUILDING_DESTROYED.name == csm.result.name) 
		{
			return I18.f('CATAPULT_SIEGE_HAS_DESTROYED_YOUR_BUILDING_BODY', csm.catapultLocation, csm.estateXYZ);
		}
	}
	console.warn("Could not find body for", csm);
}

CatapultSiegeView.prototype.getSubject = function()
{
	var csm = this._message;

	if (csm.catapultOwner)
	{
		if (CatapultSiegeResult.NO_TARGET.name == csm.result.name) 
		{
			return I18.f('YOUR_CATAPULT_SIEGE_NO_TARGET_SUBJECT', csm.catapultLocation, csm.estateXYZ);
		}
		if (CatapultSiegeResult.NO_DAMAGE.name == csm.result.name) 
		{
			return I18.f('YOUR_CATAPULT_SIEGE_NO_DAMAGE_SUBJECT', csm.catapultLocation, csm.estateXYZ);
		}
		if (CatapultSiegeResult.BUILDING_DAMAGED.name == csm.result.name) 
		{
			return I18.f('YOUR_CATAPULT_SIEGE_BUILDING_DAMAGED_SUBJECT', csm.catapultLocation, csm.estateXYZ);
		}
		if (CatapultSiegeResult.BUILDING_DESTROYED.name == csm.result.name) 
		{
			return I18.f('YOUR_CATAPULT_SIEGE_BUILDING_DESTROYED_SUBJECT', csm.catapultLocation, csm.estateXYZ);
		}
	}
	else
	{
		if (CatapultSiegeResult.NO_DAMAGE.name == csm.result.name) 
		{
			return I18.f('CATAPULT_SIEGE_HAS_ATTACKED_YOUR_ESTATE_SUBJECT', csm.catapultLocation, csm.estateXYZ);
		}
		if (CatapultSiegeResult.BUILDING_DAMAGED.name == csm.result.name) 
		{
			return I18.f('CATAPULT_SIEGE_HAS_DAMAGED_YOUR_BUILDING_SUBJECT', csm.catapultLocation, csm.estateXYZ);
		}
		if (CatapultSiegeResult.BUILDING_DESTROYED.name == csm.result.name) 
		{
			return I18.f('CATAPULT_SIEGE_HAS_DESTROYED_YOUR_BUILDING_SUBJECT', csm.catapultLocation, csm.estateXYZ);
		}
	}
	console.warn("Could not find subject for", csm);
}

//////////////////////////////////

function MissionMoveResourcesView()
{
	callParentConstructor(MissionMoveResourcesView.prototype, this);
}

extendClass(MissionMoveResourcesView.prototype, SystemMessageView.prototype);

MissionMoveResourcesView.prototype.getSubject = function()
{
    return this._message.result.equals(MissionResult.SUCCESS) ?
           I18.f("SYSTEM_MESSAGE_SUBJECT_MOVE_RESOURCES_SUCCESS") : I18.f("SYSTEM_MESSAGE_SUBJECT_MOVE_RESOURCES_FAIL");
}

MissionMoveResourcesView.prototype.createChildren = function()
{
    superMethod(MissionMoveResourcesView.prototype, "createChildren", this)();
    var m = this._message;
    
    //console.log(m);
    var view = document.createElement("div");
   
    switch (m.result.name)
    {
        case MissionResult.SUCCESS.name:
            var c = document.createElement("div");
            c.innerHTML = I18.f("FROM") + " " + "<b>"
            	+ Strings.safeHtml(m.fromEstateNameAndLocation()) 
            	+ "</b><br>" + I18.f("TO") + " " 
            	+ "<b>" + Strings.safeHtml(m.toEstateNameAndLocation()) + "</b>"
            ;
            view = c;
            break;

        case MissionResult.FAIL.name:
            view = this.createText(I18.f("SYSTEM_MESSAGE_BODY_MOVE_RESOURCES_FAIL", Strings.safeHtml(m.fromEstateNameAndLocation()), m.toFieldXYZ));
            break;
    }
        
    this.addLine(this.addImage(Assets.getUnitImageFor(FUser.CURRENT_USER.race, UnitType.MERCHANT)), view);
    //this.addLine(this.getImage(), view);
	      
	var f = COM.create("FundsBox");
 	//f.update(m.resources, m.estateOwner.race);
 	f.update(m.resources, FUser.CURRENT_USER.race);
 	this.addLine(this.createText(I18.f("RESOURCES")),f.DOM);
	   
   	if (m.merchantsDied > 0)
    {
        this.addLine(this.createText(I18.f("MERCHANTS_DIED", m.merchantsDied)));
    }
            
    if (m.deathRate > 0)
    {
        this.addLine(this.createText(I18.getMegaString(I18.f("DEATH_RATE_MESSAGE"), m.deathRate)));
    }
}


//////////////////////////////////

function MissionTradeResultMessageView()
{
	callParentConstructor(MissionTradeResultMessageView.prototype, this);
}

extendClass(MissionTradeResultMessageView.prototype, SystemMessageView.prototype);

MissionTradeResultMessageView.prototype.getSubject = function()
{
    if (this._message.isOwnMission)
    {
        return this._message.result.equals(MissionResult.SUCCESS) ?
        I18.f("SYSTEM_MESSAGE_SUBJECT_RES_BUY_SUCCESS") : I18.f("SYSTEM_MESSAGE_SUBJECT_RES_BUY_FAIL");
    }
    else
    {
        return I18.f("SYSTEM_MESSAGE_SUBJECT_RES_SELL_SUCCESS");
    }
}

MissionTradeResultMessageView.prototype.createChildren = function()
{
    superMethod(MissionTradeResultMessageView.prototype, "createChildren", this)();
    
    if (this._message.isOwnMission)
    {
        this.createBuyMessageBody();
    }
    else
    {
        this.createSellMessageBody();
    }
}

MissionTradeResultMessageView.prototype.createSellMessageBody = function()
{
    var message = this._message;
    
    this.addLine(this.addImage(Assets.B_findBy(FUser.CURRENT_USER.race, BType.MARKET), true));
            
    this.addText(I18.f("SYSTEM_MESSAGE_BODY_RES_SELL_SUCCESS", message.toEstateNameAndLocation(), message.username));
    
    var grid = document.createElement("inline-table");
    
   	var got = new Funds();
    got.gold = message.spent;  
            
    var sold = new Funds();
    sold.setValue(message.resource, message.bought);

	var f1 = COM.create("FundsBox");
 	f1.update(got, FUser.CURRENT_USER.race);
	
	//this.addLine(this.createText(I18.f("GOT")), f1.DOM);
	
	var f2 = COM.create("FundsBox");
	f2.update(sold, FUser.CURRENT_USER.race);
	
	//this.addLine(this.createText(I18.f("SOLD")), f2.DOM);
	
	var r = document.createElement("row");
    
    var c = document.createElement("cell");
    c.style.textAlign = "left";
    c.style.paddingRight = "12px";
    c.appendChild(this.createText(I18.f("GOT")));
    r.appendChild(c);
    
    var c = document.createElement("cell");
    c.appendChild(f1.DOM);
    r.appendChild(c);
    
    grid.appendChild(r);
    
    var r = document.createElement("row");
    
    var c = document.createElement("cell");
    c.style.textAlign = "left";
    c.style.paddingRight = "12px";
    c.appendChild(this.createText(I18.f("SOLD")));
    r.appendChild(c);
    
    var c = document.createElement("cell");
    c.appendChild(f2.DOM);
    r.appendChild(c);
    
    grid.appendChild(r);

	this.addLine(grid);

	if (message.peacePoints > 0) 
    {
        this.addLine(this.createText(I18.f("GOT_PEACE_POINTS")), IconAndValue.create(CommonAssets.PEACE_POINT, NumberUtils.format(message.peacePoints)));
    }
}

MissionTradeResultMessageView.prototype.createBuyMessageBody = function()
{
    var m = this._message;
    var view;
	this.addLine(this.getImage(Assets.getUnitImageFor(FUser.CURRENT_USER.race, UnitType.MERCHANT, 1)));
	switch (m.result.name) 
    {
        case "SUCCESS":
            this.addLine(this.createText(I18.f("SYSTEM_MESSAGE_BODY_RES_BUY_SUCCESS",m.fromEstateNameAndLocation(), m.toEstateNameAndLocation())));
           
            var bought = new Funds();
            bought.setValue(m.resource, m.bought);
            
            var f1 = COM.create("FundsBox");
		 	f1.update(bought, FUser.CURRENT_USER.race);
            
            var spent = new Funds();
            spent.gold = m.spent;
			
			var f2 = COM.create("FundsBox");
		 	f2.update(spent, FUser.CURRENT_USER.race);

            this.addLine(this.createText(I18.f("BOUGHT")), f1.DOM);
            this.addLine(this.createText(I18.f("SPENT")), f2.DOM);
            
            if (m.peacePoints > 0)
            {
                this.addLine(UIh.createLine(I18.f("GOT_PEACE_POINTS"), IconAndValue.create(STATIC_INTERFACE_URL + CommonAssets.PEACE_POINT, I18.formatNumber(m.peacePoints))));
            }
            
            break;

        case "FAIL":
            this.addLine(this.createText(I18.f("SYSTEM_MESSAGE_BODY_RES_BUY_FAIL",m.fromEstateNameAndLocation(), m.toEstateNameAndLocation())));
            break;
    }
}


MissionTradeResultMessageView.prototype.getImage = function(src)
{
    var founder = null;
    var img = document.createElement("img");
    img.src = STATIC_INTERFACE_URL + src;
    return img;
}

//////////////////////////////////

function GiftMessageView()
{
	callParentConstructor(GiftMessageView.prototype, this);
}

extendClass(GiftMessageView.prototype, SystemMessageView.prototype);

GiftMessageView.prototype.getSubject = function()
{
    return this._message.receiverType.name == MailReceiverType.CLAN.name ? I18.f("CLAN_GIFT_MESSAGE_SUBJECT") : I18.f("GIFT_MESSAGE_SUBJECT");
}

GiftMessageView.prototype.createChildren = function()
{
    //throw Error("Not implemented");
    
    superMethod(GiftMessageView.prototype, "createChildren", this)();
    
    var gift = this._message.gift;
    var t = this._message.receiverType == MailReceiverType.CLAN ? "CLAN_GIFT_MESSAGE_BODY" : "GIFT_MESSAGE_BODY";
    var text = I18.f(t,this._message.username, I18.getGiftName(gift.giftName), Tick.toTimeFuture(gift.finishTick));
    this.addLine(this.addImage(Assets.getGiftImage(gift.giftName)),this.createText(text));

    if (this._message.text != null && this._message.text.length > 0) 
    {    	   	   	
        var label = I18.f("GIFT_MESSAGE_BODY_POSTFIX",this._message.username, this._message.text);
        this.addLine(this.createText(label));
        this.addReactionView();
    }
    
}

//////////////////////////////////

function ShareOuterEstateMessageView()
{
	callParentConstructor(ShareOuterEstateMessageView.prototype, this);
}

extendClass(ShareOuterEstateMessageView.prototype, SystemMessageView.prototype);

ShareOuterEstateMessageView.prototype.getSubject = function()
{
    return I18.f("SHARE_OUTER_ESTATE_SUBJECT");
}

ShareOuterEstateMessageView.prototype.getSender = function()
{
    return this._message.username;
}

ShareOuterEstateMessageView.prototype.createChildren = function()
{
    superMethod(ShareOuterEstateMessageView.prototype, "createChildren", this)();
    
    if (this._message.text && this._message.text.length > 0)
    {
        this.addText(I18.f("SHARE_OUTER_ESTATE_BODY_COMMENT", this._message.username, this._message.estateNameAndLocation, this._message.text));
    }
    else
    {
        this.addText(I18.f("SHARE_OUTER_ESTATE_BODY", this._message.username, this._message.estateNameAndLocation));
    }
    
    this.addReactionView();
}

//////////////////////////////////

function BrotherhoodExecutedMessageView()
{
	callParentConstructor(BrotherhoodExecutedMessageView.prototype, this);
}

extendClass(BrotherhoodExecutedMessageView.prototype, SystemMessageView.prototype);

BrotherhoodExecutedMessageView.prototype.getSubject = function()
{
    return I18.f("BROTHERHOOD_EXECUTION_COMPLETED_TARGET_SUBJECT", this._message.username);
}

BrotherhoodExecutedMessageView.prototype.getSender = function()
{
    return this._message.username;
}

BrotherhoodExecutedMessageView.prototype.createChildren = function()
{
    superMethod(BrotherhoodExecutedMessageView.prototype, "createChildren", this)();
    this.addText(I18.getMegaString("BROTHERHOOD_EXECUTION_COMPLETED_TARGET_BODY",
            {
            	owner: this._message.username,
            	executor: this._message.executor,
            	action: I18.f(this._message.goalType.name),
            	message: this._message.text
            }));
    this.addReactionView();
}

//////////////////////////////////

function SimpleMessage()
{
	callParentConstructor(SimpleMessage.prototype, this);
}

extendClass(SimpleMessage.prototype, SystemMessageView.prototype);

SimpleMessage.prototype.getSubject = function()
{
	if (this._message.template)
    {
    	return I18.getMegaString(this._message.template + "_SUBJECT", this._message.params);
    }
    return I18.getMegaString("MESSAGE_" + this._message.messageClass + "_SUBJECT", this._message);
}


SimpleMessage.prototype.createChildren = function()
{
    superMethod(SimpleMessage.prototype, "createChildren", this)();
	
	if (this._message.template)
    {
    	var icon = null;
        if (this._message.iconPath) 
        {
        	var dotIndex = this._message.iconPath.indexOf(".");
            if (dotIndex > 0) 
            {
            	var assetClass = this._message.iconPath.substring(0, dotIndex);
                var p = this._message.iconPath.substring(dotIndex + 1);
                var path = function(assetClass, p)
                {
	                switch (assetClass)
	                {
	                	case "Assets": return Assets[p];
		                case "MailAssets": return MailAssets[p];
		                case "SmileAssets": return SmileAssets[p];
		                case "CommonAssets": return CommonAssets[p];
		                case "GiftAssets": return GiftAssets[p];
		                case "ArtifactAssets": return ArtifactAssets[p];
		                case "SkillAssets": return SkillAssets[p];
		                case "KingdomAssets": return KingdomAssets[p];
		                case "HeroAssets": return HeroAssets[p] ? HeroAssets[p] : CommonHeroAssets[p];
	                }
                };
                
                icon = this.addImage(path(assetClass, p));
            } 
            else if (this._message.iconPath == "advisor") 
            {
            	icon = Assets.getAdvisor(FUser.CURRENT_USER.race); //think about more precise race determination
            }
        }
        var body;
        if (I18.getStringOrNull(this._message.template + "_BODY_HTML") != null)
        {
            var body = I18.getMegaString(this._message.template + "_BODY_HTML", this._message.params);
        } 
        else if (!this._message.params.isEmpty()) 
        {
            body = I18.getMegaBox(this._message.template + "_BODY", this._message.params);
        }
        else 
        {
            body = this.createText(I18.getMegaString(this._message.template + "_BODY", this._message.params));
        }       
        
        $(body).addClass("inline-simple-mes");
        
        if(icon) this.addLine(this.imageWithToolTip(icon, null, true), body);
        else this.addLine(body);
	}
    else
    {
    	this.addText(I18.getMegaString("MESSAGE_" + this._message.messageClass + "_BODY", this._message));
    }
}

//////////////////////////////////

function ClanInvitationRejectedMessageView()
{
	callParentConstructor(ClanInvitationRejectedMessageView.prototype, this);
}

extendClass(ClanInvitationRejectedMessageView.prototype, SystemMessageView.prototype);

ClanInvitationRejectedMessageView.prototype.getSubject = function()
{
    return I18.f("CLAN_INVITATION_REJECTED_MESSAGE_SUBJECT", htmlEscape(this._message.clanName));
}

ClanInvitationRejectedMessageView.prototype.createChildren = function()
{
    superMethod(ClanInvitationRejectedMessageView.prototype, "createChildren", this)();
    
    this.addLine(this.createText(I18.f("CLAN_INVITATION_REJECTED_MESSAGE_BODY", htmlEscape(this._message.clanName), this._message.invitee)));
}

//////////////////////////////////

function ApprenticeInvitationView()
{
	callParentConstructor(ApprenticeInvitationView.prototype, this);
}

extendClass(ApprenticeInvitationView.prototype, SystemMessageView.prototype);

ApprenticeInvitationView.prototype.getSubject = function()
{
    return I18.f("MESSAGE_APPRENTICE_INVITATION_SUBJECT", this._message.username);
}

ApprenticeInvitationView.prototype.createChildren = function()
{
    superMethod(ApprenticeInvitationView.prototype, "createChildren", this)();
    
   	if (this._message.text != null) 
    {
        this.addLine(this.createText(this._message.text));
    }

    this.addLine(this.createText(I18.getMegaString("MESSAGE_APPRENTICE_INVITATION_BODY", this._message)));

	var okButton = COM.create("SimpleButton");
    okButton.caption = I18.f("YES");
    
    COM.bindExclusiveClick(okButton.DOM, $.proxy(function(e)
    {
    	Service.alliance.callMethod("acceptApprenticeInvitation", [this._message.username],
    		$.proxy(this.handleMentorShipSuccess, this),
    		$.proxy(this.handleMentorshipError, this));
    }, this)); 
           
    var cancelButton = COM.create("SimpleButton");
    cancelButton.caption = I18.f("NO");
    COM.bindExclusiveClick(cancelButton.DOM, $.proxy(function(e)
    {
    	Service.alliance.callMethod("rejectApprenticeInvitation",[this._message.username],
    		$.proxy(this.handleMentorShipSuccess, this),
    		$.proxy(this.handleMentorshipError, this));
    }, this));
    
    if (!this._message.invitationValid || !this._message.isOwner)
    {
        if (!this._message.isOwner) 
        {
            okButton.reason = I18.f("NO_ACCESS");
            cancelButton.reason = I18.f("NO_ACCESS");
        } 
        else if (!FUser.CURRENT_USER.diplomacy.canBecomeApprentice) 
        {
            okButton.reason = I18.f("CANT_BECOME_APPRENTICE");
            cancelButton.reason = I18.f("CANT_BECOME_APPRENTICE");
        } 
        else if (FUser.CURRENT_USER.diplomacy.currentMentor) 
        {
            okButton.reason = I18.f("APPRENTICE_INVITATION_HAS_MENTOR");
            cancelButton.reason = I18.f("APPRENTICE_INVITATION_HAS_MENTOR");
        } 
        else 
        {
            okButton.reason = I18.f("APPRENTICE_INVITATION_INVALID");
            cancelButton.reason = I18.f("APPRENTICE_INVITATION_INVALID");
        }
    }
    
    this.DOM.appendChild(okButton.DOM);
    this.DOM.appendChild(cancelButton.DOM);
    this.addReactionView();
    
}

//////////////////////////////////

function LeaveClanMessageView()
{
	callParentConstructor(LeaveClanMessageView.prototype, this);
}

extendClass(LeaveClanMessageView.prototype, SystemMessageView.prototype);

LeaveClanMessageView.prototype.getSubject = function()
{
    return this._message.expelledByLeader
        ? I18.f("YOU_WERE_EXPELLED_CLAN_MESSAGE_SUBJECT")
        : I18.f("PLAYER_LEAVE_CLAN_MESSAGE_SUBJECT");
}

LeaveClanMessageView.prototype.createChildren = function()
{
	    superMethod(LeaveClanMessageView.prototype, "createChildren", this)();
    
    this.addText(this._message.expelledByLeader
        ? I18.f("YOU_WERE_EXPELLED_CLAN_MESSAGE_BODY", htmlEscape(this._message.clanName))
        : I18.f("PLAYER_LEAVE_CLAN_MESSAGE_BODY", this._message.username, htmlEscape(this._message.clanName)));
}

//////////////////////////////////

function ClanLeaderChangeMessageView()
{
	callParentConstructor(ClanLeaderChangeMessageView.prototype, this);
}

extendClass(ClanLeaderChangeMessageView.prototype, SystemMessageView.prototype);

ClanLeaderChangeMessageView.prototype.getSubject = function()
{
    return I18.f("YOU_ARE_NEW_CLAN_LEADER");
}

ClanLeaderChangeMessageView.prototype.createChildren = function()
{
    superMethod(ClanLeaderChangeMessageView.prototype, "createChildren", this)();
    
    var text = this._message.clanLeaderLeft
        ? I18.f("PREVIOUS_CLAN_LEADER_LEFT", htmlEscape(this._message.clanName), this._message.username)
        : I18.f("PREVIOUS_CLAN_LEADER_CHOSE", htmlEscape(this._message.clanName), this._message.username)
    
    this.addLine(this.createText(text));
}

//////////////////////////////////

function ClanReportMessageView()
{
	callParentConstructor(ClanReportMessageView.prototype, this);
}

extendClass(ClanReportMessageView.prototype, SystemMessageView.prototype);

ClanReportMessageView.prototype.getSubject = function()
{
    return I18.psf("SYSTEM_MESSAGE", this._message.template, "SUBJECT", [DateUtils.format(Tick.toMs(this._message.reportPeriod), "DD MMM YYYY")]);
}

ClanReportMessageView.prototype.createChildren = function()
{
    superMethod(ClanReportMessageView.prototype, "createChildren", this)();
        
    var body = I18.psf("SYSTEM_MESSAGE", this._message.template, "BODY");
    if (body != "") 
    {
        this.addLine(this.createText(body));
    }
    if (!this._message.resourceAdders.isEmpty()) 
    {
        var grid = document.createElement("inline-table");
		grid.className = "clan-report";
        
        for (var i=0;i<this._message.resourceAdders.keys.length;i++) 
        {
        	var adder = this._message.resourceAdders.keys[i];
        	
        	var row = document.createElement("row");
        	
        	var cell = document.createElement("cell");
        	cell.style.textAlign = "left";
        	cell.style.paddingRight = "16px";
        	cell.innerHTML = I18.psf("SYSTEM_MESSAGE", this._message.template, "ITEM", adder);
        	row.appendChild(cell);
        	
        	var cell = document.createElement("cell");
        	cell.style.textAlign = "left";
        	var f = COM.create("FundsBox");
 			f.update(this._message.resourceAdders.getValue(adder), FUser.CURRENT_USER.race);
 			cell.appendChild(f.DOM);
 			row.appendChild(cell);
 			
 			grid.appendChild(row);
        }
        
		this.addLine(grid);
    } 
    else 
    {
        I18.psf("SYSTEM_MESSAGE", this._message.template, "EMPTY_BODY");
    }
}

//////////////////////////////////

function ClanResourceReceivedMessageView()
{
	callParentConstructor(ClanResourceReceivedMessageView.prototype, this);
}

extendClass(ClanResourceReceivedMessageView.prototype, SystemMessageView.prototype);

ClanResourceReceivedMessageView.prototype.getSubject = function()
{
    return I18.getClanResourcesReceivedSubject(this._message.buildingType);
}

ClanResourceReceivedMessageView.prototype.createChildren = function()
{
    superMethod(ClanResourceReceivedMessageView.prototype, "createChildren", this)();
    var f = COM.create("FundsBox");
    f.update(this._message.resources, FUser.CURRENT_USER.race);
    this.addLine(this.createText(I18.getClanResourcesReceivedBody(this._message.buildingType, this._message.toSelf, this._message.username)));
	this.addLine(f.DOM);
}

//////////////////////////////////

function ShowOuterEstatesView()
{
	callParentConstructor(ShowOuterEstatesView.prototype, this);
	
	this.__defineGetter__("estateType", this.getEstateType);
}

extendClass(ShowOuterEstatesView.prototype, SystemMessageView.prototype);

ShowOuterEstatesView.prototype.getEstateType = function()
{
    return this._message.estatesInfo[0][1].resolve();
}

ShowOuterEstatesView.prototype.getSubject = function()
{
    return I18.sf("SHOW_OUTER_ESTATES_SUBJECT", this._message.template);
}

ShowOuterEstatesView.prototype.createChildren = function()
{
    superMethod(ShowOuterEstatesView.prototype, "createChildren", this)();
    
    var names = [];
    var info;
    for(var i=0; i<this._message.estatesInfo.length; i++)
    {
    	info = this._message.estatesInfo[i];
    	var location = info[0];
        var type = info[1].resolve();
        var res = [info[2]];
        names.push(I18.formatEstateName(I18.getOuterEstateName(type, res), location));
    }
    this.addLine(this.createText(I18.sf("SHOW_OUTER_ESTATES_BODY", this._message.template, names.join("\n"))));
}

//////////////////////////////////

function PersonalEventMessageView()
{
	callParentConstructor(PersonalEventMessageView.prototype, this);
}

extendClass(PersonalEventMessageView.prototype, SystemMessageView.prototype);

PersonalEventMessageView.prototype.getSubject = function()
{
    var msg = this._message;
    return I18.s("PERSONAL_EVENT_" + msg.eventType + "_" + msg.lifeCycle, "SUBJECT");
}

PersonalEventMessageView.prototype.createChildren = function()
{
    superMethod(PersonalEventMessageView.prototype, "createChildren", this)();
    
    var msg = this._message;
    this.addText(I18.getMegaString("PERSONAL_EVENT_" + msg.eventType + "_" + msg.lifeCycle + "_BODY", [msg.value]));
}

//////////////////////////////////

function CastMessageView()
{
	callParentConstructor(CastMessageView.prototype, this);
}

extendClass(CastMessageView.prototype, SystemMessageView.prototype);

CastMessageView.prototype.getSubject = function()
{
    var msg = this._message;
    var subj;

    if (msg.hasEffect)
    {
        if (msg.receiver == msg.casterPlayer) subj = "CAST_MESSAGE_SUBJECT_SELF";
        else if (msg.targetPlayer && msg.targetPlayer.charAt(0) == '$') subj = "CAST_MESSAGE_SUBJECT_CASTLE";
        else subj = "CAST_MESSAGE_SUBJECT";
    }
    else
    {
        if (msg.receiver == msg.casterPlayer) subj = "CAST_MESSAGE_NO_EFFECT_SUBJECT_SELF";
        else if (msg.targetPlayer && msg.targetPlayer.charAt(0) == '$') subj = "CAST_MESSAGE_NO_EFFECT_SUBJECT_CASTLE";
        else subj = "CAST_MESSAGE_NO_EFFECT_SUBJECT";
    }

    return I18.f(subj, Texts.spellName(msg.spellName));
}

CastMessageView.prototype.createChildren = function()
{
    //superMethod(CastMessageView.prototype, "createChildren", this)();
    
    var msg = this._message;
    var text;
    if (msg.infection)
    {
        if (msg.targetHero)
        {
            if (msg.receiver == msg.casterPlayer)
            {
                text = "CAST_MESSAGE_INFECTION_ON_HERO_BODY_SELF";
            }
            else
            {
                text = "CAST_MESSAGE_INFECTION_ON_HERO_BODY";
            }
        }
        else
        {
            if (msg.receiver == msg.casterPlayer)
            {
                text = "CAST_MESSAGE_INFECTION_ON_ESTATE_BODY_SELF";
            }
            else
            {
                text = "CAST_MESSAGE_INFECTION_ON_ESTATE_BODY";
            }
        }
    }
    else
    {
        if (msg.targetHero)
        {
            text = "CAST_MESSAGE_ON_HERO_BODY";
        }
        else
        {
            if (msg.targetPlayer && msg.targetPlayer.charAt(0) == '$')
            {
                text = "CAST_MESSAGE_ON_CASTLE_BODY";
            }
            else
            {
                text = "CAST_MESSAGE_ON_ESTATE_BODY";
            }
        }
    }
    
    this.addText(I18.getMegaString(text, msg));
    
    var pic = document.createElement("img");
    pic.src = STATIC_INTERFACE_URL + (Assets.getArtifactSpellImage(msg.spellName) || Assets.getSpellImage(msg.spellName));
    this.addLine(pic);
    
    if (msg.activeTill != 0)
    {
        this.addText(I18.f("CAST_MESSAGE_DURATION", Tick.toTimeFuture(msg.activeTill)));
    }
    
    var showNoEffect = !msg.hasEffect;
    if (msg.castEffects)
    {
        var e;
        for(var i=0; i<msg.castEffects.length; i++)
        {
        	e = msg.castEffects[i];
        	if (e)
        	{
                this.addText(I18.getCastEffectMessage(e));
                if (CastEffectType.MAGIC_DEFENSE.equals(e.type) || CastEffectType.UNKNOWN.equals(e.type))
                {
                    showNoEffect = false;
                }
            }
        }
    }
    if (showNoEffect)
    {
        this.addText(I18.f("CAST_MESSAGE_NO_EFFECT"));
    }
}

//////////////////////////////////

function RuinLegacyMessageView()
{
	callParentConstructor(RuinLegacyMessageView.prototype, this);
}

extendClass(RuinLegacyMessageView.prototype, SystemMessageView.prototype);

RuinLegacyMessageView.prototype.getSubject = function()
{
    return I18.f("SYSTEM_MESSAGE_SUBJECT_RUIN_LEGACY");
}

RuinLegacyMessageView.prototype.createChildren = function()
{
    superMethod(RuinLegacyMessageView.prototype, "createChildren", this)();
    
    var msg = this._message;
    this.addText(I18.f("SYSTEM_MESSAGE_BODY_RUIN_LEGACY", msg.ruinXYZ));
    
    for (var i = 0; i < msg.beneficiaries.length; i++)
    {
 		var text = I18.f("SYSTEM_MESSAGE_DETAIL_RUIN_LEGACY",FUser.getUserNameForMessages(msg.beneficiaries[i]),
    	I18.fixEstateName(msg.estates[i]),msg.locations[i]);
    	
    	text = text.substring(0,text.length-9);
    	
    	var f = COM.create("FundsBox");
	    f.update(msg.amounts[i], FUser.CURRENT_USER.race);
	    this.addLine(this.createText(text),f.DOM);
 
    	/*
        this.addLine(I18.box(I18.f("SYSTEM_MESSAGE_DETAIL_RUIN_LEGACY"), FUser.getUserNameForMessages(msg.beneficiaries[i]),
                I18.fixEstateName(msg.estates[i]), msg.locations[i], msg.amounts[i]));
        */
    }
}

//////////////////////////////////

function ApprenticeAskedResourcesView()
{
	callParentConstructor(ApprenticeAskedResourcesView.prototype, this);
}

extendClass(ApprenticeAskedResourcesView.prototype, SystemMessageView.prototype);

ApprenticeAskedResourcesView.prototype.getSubject = function()
{
	return I18.f("APPRENTICE_ASKED_RESOURCES_SUBJECT", this._message.params.getValue("apprentice"));
}

ApprenticeAskedResourcesView.prototype.createChildren = function()
{
    superMethod(ApprenticeAskedResourcesView.prototype, "createChildren", this)();
    var satisfied = this._message.params.getValue("satisfied");
    var apprentice = this._message.params.getValue("apprentice")
    this.addLine(this.createText(I18.f("APPRENTICE_ASKED_RESOURCES_BODY", apprentice)));

    this.addLine(FundsBox.createAskMentorFunds().DOM);
    
    var button = COM.create("SimpleButton");
    button.caption = I18.f("GIVE_FROM_CAPITAL");
    COM.bindExclusiveClick(button, $.proxy(function()
    {
    	Service.mail.callMethod("giveResourcesToApprentice", [this._message.id],
    		$.proxy(this.handleMentorShipSuccess, this),
    		$.proxy(this.handleMentorshipError, this));
    }, this));
    
    button.enabled = (!satisfied && FUser.isCurrentUser(this._message.receiver));
    this.addLine(button.DOM);
}

ApprenticeAskedResourcesView.prototype.handleMentorShipSuccess = function(result)
{
	superMethod(ApprenticeAskedResourcesView.prototype, "handleMentorShipSuccess", this)(result);
}

//////////////////////////////////

function MentorGaveResourcesView()
{
	callParentConstructor(MentorGaveResourcesView.prototype, this);
}

extendClass(MentorGaveResourcesView.prototype, SystemMessageView.prototype);

MentorGaveResourcesView.prototype.getSubject = function()
{
    return I18.f("MENTOR_GAVE_RESOURCES_SUBJECT");
}

MentorGaveResourcesView.prototype.createChildren = function()
{
    superMethod(MentorGaveResourcesView.prototype, "createChildren", this)();
    
    
    this.addLine(this.createText(I18.f("MENTOR_GAVE_RESOURCES_BODY")));
    var f = COM.create("FundsBox");
    f.update(FConfig.APPRENTICE_ASK_RESOURCES, FUser.CURRENT_USER.race);
    this.addLine(f.DOM);
    this.addLine(this.createText(I18.f("RESOURCES_MOVED_TO_CAPITAL")));
}

//////////////////////////////////

function BuildingsEffectsView()
{
	callParentConstructor(BuildingsEffectsView.prototype, this);
}

extendClass(BuildingsEffectsView.prototype, SystemMessageView.prototype);

BuildingsEffectsView.prototype.getSubject = function()
{
	return I18.f("BUILDING_EFFECT_MESSAGE_SUBJECT");
}

BuildingsEffectsView.prototype.createChildren = function()
{
	/*
	modifyed EffectsView
	*/
	superMethod(BuildingsEffectsView.prototype, "createChildren", this)();
				
	//var effectsList = FBuildingsEffectsMessage(this._message).generationEffects;
  
  	var effectsList = this._message.generationEffects;
    
  	var effectsMap = FMap.groupBy(effectsList, function(e)
    {
        return e.btype;
    });
	
	var military = [];
    	
    for (var i=0; i < effectsMap.keys.length; i++)
    {	 	
    	var btype = effectsMap.keys[i];
   	
    	if (btype.isMilitaryUnitProducer())
        {
            military = military.concat(effectsMap.getValue(btype));
        } 
        else 
        {
            this.addSection(I18.psf("BUILDING_EFFECT", btype.name, "TITLE"), effectsMap.getValue(btype));
        }
    }
	// aggregate military units in single section
        
    if (military.length > 0)
    {
        this.addSection(I18.f("BUILDING_EFFECT_UNITS_TITLE"), military);
    }
}

BuildingsEffectsView.prototype.addSection = function(title, effects)
{
    //throw Error("Not implemented");
    //this.addLine(UI.label(title));
    
    this.addLine(this.createText(title));
    var d = EffectsView.createPerfectBuildingEffectView(effects, FUser.CURRENT_USER.race);
    
    
    
    $(d).addClass("inline-div");
    this.addLine(d);
    
}

//////////////////////////////////

function TournamentFinishedMessageView()
{
	callParentConstructor(TournamentFinishedMessageView.prototype, this);
}

extendClass(TournamentFinishedMessageView.prototype, SystemMessageView.prototype);

TournamentFinishedMessageView.prototype.getMessage = function()
{
    return this._message;
}

TournamentFinishedMessageView.prototype.getSubject = function()
{
    return I18.mega(I18.f("TOURNAMENT_WINNERS_SUBJECT"), this.getMessage().type);
}

TournamentFinishedMessageView.prototype.createChildren = function()
{
    superMethod(TournamentFinishedMessageView.prototype, "createChildren", this)();
    
    this.addLine(I18.mega(I18.f("TOURNAMENT_WINNERS_BODY"), this.getMessage().creationTick, this.getMessage().type, this.getMessage().winners.length));
    var table = document.createElement("inline-table");
    this.addSection(table, I18.f("TOURNAMENT_WINNERS_PLACE"), I18.f("TOURNAMENT_WINNERS_USERNAME"), I18.f("REWARD"));
    for(var i = 0; i < this.getMessage().winners.length; i++)
    {
        var wi = this.getMessage().winners[i];
        this.addSection(table, "" + (i+1), wi.username, wi.king ? I18.f('TOURNAMENT_KING_ARTIFACT') : Texts.artifactTypeNameLvl(wi.type, wi.level));
    }
    table.className = "tournament-finished-message-table";
    this.addLine(table);
}

TournamentFinishedMessageView.prototype.addBox = function(line, text, width)
{
	var c = document.createElement("cell");
	c.innerHTML = text;
	line.appendChild(c);
}

TournamentFinishedMessageView.prototype.addSection = function(table, place, username, reward)
{
    var line = document.createElement("row");
    this.addBox(line, place, 50);
    this.addBox(line, username, 150);
    this.addBox(line, reward, 300);
    table.appendChild(line);
}