function FMilitaryMissionMessage()
{
	callParentConstructor(FMilitaryMissionMessage.prototype, this);
	
	this.attackersForces = null;
    this.defendersForces = null;
    this.attackersSummary = null;
    this.defendersSummary = null;

    this.defenderVisible = null;
    this.roundsLimitReached = null;

    this.defenderSpecSkill = null;
    this.towerBuffBonus = null;
    this.magicTowerBuffBonus = null;

    this.firstTowerTypeId = null;
    this.firstTowerLevel = null;
    this.firstTowerBonus = null;
    this.secondTowerTypeId = null;
    this.secondTowerLevel = null;
    this.secondTowerBonus = null;
    this.gateLevel = null;
    this.gateInitialHealth = null;
    this.gateResultHealth = null;
    this.gateGarrisonHelp = null;

    this.buildingsDamaged = null;
    this.buildingsDestroyed = null;
    this.savedByBuilding = null;
    this.failReason = null;
    this.armiesInfo = null;
    this.attackersHealing = null;
    this.defendersHealing = null;
    this.predictionPolicy = null;
    this.attackersHealingOverflow = null;
    this.defendersHealingOverflow = null;
}

extendClass(FMilitaryMissionMessage.prototype, FMissionMessage.prototype);

FMilitaryMissionMessage.prototype.__CLS__ = "game.service.mail.FMilitaryMissionMessage";

FMilitaryMissionMessage.prototype.getIcon = function()
{
    return !this.clanName ? MailAssets.MESSAGE_ICON_MILITARY : MailAssets.MESSAGE_ICON_CLAN;
}

FMilitaryMissionMessage.prototype.getActualSender = function()
{
    return I18.f("MILITARY_MESSAGE_SENDER");
}

FMilitaryMissionMessage.prototype.receiverWonCombat = function()
{
    if (this.isForwarded)
    {
        return false;
    }
    else if (this.isOwnMission || this.receiverAttacked())
    {
        return this.result.name == MissionResult.SUCCESS.name;
    }
    else
    {
        return this.result.name == MissionResult.DEFEAT.name;
    }
}

FMilitaryMissionMessage.prototype.receiverLoseCombat = function()
{
    if (this.isForwarded)
    {
        return false;
    }
    else if (this.isOwnMission || this.receiverAttacked())
    {
        return this.result.name == MissionResult.DEFEAT.name;
    }
    else
    {
        return this.result.name == MissionResult.SUCCESS.name;
    }
}

FMilitaryMissionMessage.prototype.receiverAttacked = function()
{
    return !this.isForwarded && this.userAttacked(this.receiver);
}

FMilitaryMissionMessage.prototype.receiverDefend = function()
{
    return !this.isForwarded && this.userDefend(this.receiver);
}

FMilitaryMissionMessage.prototype.userAttacked = function(name)
{
    var summary;
    for(var i=0; i<this.attackersSummary.length; i++)
    {
    	summary = this.attackersSummary[i];
    	
    	if (summary && FUser.isSameName(summary.username, name))
    	{
            return true;
        }
    }
    
    return false;
}

FMilitaryMissionMessage.prototype.userDefend = function(name)
{
    var summary;
    for(var i=0; i<this.defendersSummary.length; i++)
    {
    	summary = this.defendersSummary[i];
    	
    	if (summary && FUser.isSameName(summary.username, name))
    	{
            return true;
        }
    }
    
    return false;
}

FMilitaryMissionMessage.prototype.getOwnCombatSummary = function()
{
    for (var i = 0; i < this.attackersSummary.length; i++)
    {
        if (this.attackersSummary[i] && FUser.isSameName(this.attackersSummary[i].username, this.receiver))
        {
            return this.attackersForces[i];
        }
    }
    
    for (i = 0; i < this.defendersSummary.length; i++)
    {
        if (this.defendersSummary[i] && FUser.isSameName(this.defendersSummary[i].username, this.receiver))
        {
            return this.defendersForces[i];
        }
    }
    
    return null;
}

FMilitaryMissionMessage.prototype.cityHasTowers = function()
{
    return this.firstTowerLevel != 0 || this.secondTowerLevel != 0;
}

FMilitaryMissionMessage.prototype.defenderUserAndLocation = function()
{
    var defenderUsername = "-";
    if (this.defendersSummary && this.defendersSummary.length > 0 &&
        this.defendersSummary[0].username && this.defendersSummary[0].username.length > 0)
    {
        defenderUsername = this.defendersSummary[0].username;
    }
    
    return I18.formatEstateName(FUser.getUserNameForMessages(defenderUsername), this.toFieldXYZ);
}

FMilitaryMissionMessage.prototype.attackerUserAndLocation = function()
{
    var attackerUsername = "-";
    if (this.attackersSummary && this.attackersSummary.length > 0 &&
        this.attackersSummary[0].username && this.attackersSummary[0].username.length > 0)
	{
        attackerUsername = this.attackersSummary[0].username;
    }
    return I18.formatEstateName(FUser.getUserNameForMessages(attackerUsername), this.fromEstateXYZ);
}