function FUserMessage()
{
	callParentConstructor(FUserMessage.prototype, this);
	
	this.subject = null;
    this.adminSubject = null;
    this.messageBody = null;
    this.sender = null;
    this.senderCapital = null;
    this.createdWithPremium = null;
    
    this.__defineGetter__("isSentByMe", this.getIsSentByMe);
    this.__defineGetter__("canReply", this.getCanReply);
    this.__defineGetter__("canForward", this.getCanForward);
}

extendClass(FUserMessage.prototype, FBaseMessage.prototype);

FUserMessage.prototype.__CLS__ = "game.service.mail.FUserMessage";

FUserMessage.prototype.__construct = function()
{
	this.subject = htmlEscape(this.subject);
	this.messageBody = htmlEscape(this.messageBody);
}

FUserMessage.prototype.getIcon = function()
{
    return MailAssets.MESSAGE_ICON_USER;
}

FUserMessage.prototype.otherUser = function()
{
    return FUser.isCurrentUser(this.getOriginalSender()) ? this.receiver : this.getOriginalSender();
}

FUserMessage.prototype.createReplyMessageBody = function()
{
    return "\n\n" + this.getOriginalSender() + " " + I18.f("WROTE") + "\n" + this.getBody();
}

FUserMessage.prototype.createReplySubject = function()
{
    var subj;
    if (this.subject.indexOf("RE:") == 0)
    {
        subj = "RE[2]:\u200E " + StringUtil.trim(this.subject.substr(4));
    }
    else
    {
        // count replies
        var replies = this.numberOfReplies(this.subject);
        if (replies <= 0)
        {
            subj = "RE:\u200E " + this.subject;
        }
        else
        {
            var subjStart = "RE[]: ".length + ("" + replies).length;
            subj = "RE[" + (replies + 1) + "]:\u200E " + StringUtil.trim(this.subject.substr(subjStart));
        }
    }
    
    return Strings.ellipsis(subj, FConfig.SUBJECT_LENGTH_MAX);
}

FUserMessage.prototype.numberOfReplies = function(subj)
{
    var firstIndex = "RE[".length;
    var secondIndex = subj.indexOf("]:");
    // subject does not contain correctly formatted replies
    // to prevent integer overflow in the reply count
    if (secondIndex < 0 || secondIndex > firstIndex + 8)
    {
        return 0;
    }
    
    try
    {
        var number = subj.substr(firstIndex, secondIndex - firstIndex);
        return parseInt(StringUtil.trim(number));
    }
    catch (ignore)
    {
    }
    
    return 0;
}

FUserMessage.prototype.getOriginalSender = function()
{
    return this.isForwarded ? this.forwarder : this.sender;
}

FUserMessage.prototype.getActualSender = function()
{
    return this.formatSender(this.sender);
}

FUserMessage.prototype.getSubject = function()
{
    return this.subject && StringUtil.trim(this.subject) != "" ? this.subject : I18.f("NO_SUBJECT");
}

FUserMessage.prototype.createView = function(renderer)
{
    //var view = COM.create("UserMessageView");
    var view = new UserMessageView();
    view.renderer = renderer;
    return view;
}

FUserMessage.prototype.getBody = function()
{
    return this.isForwarded ? this.sender + " " + I18.f("WROTE") + "\n" + this.messageBody : this.messageBody;
}

FUserMessage.prototype.getIsSentByMe = function()
{
    return FUser.isCurrentUser(this.getSender());
}

FUserMessage.prototype.getCanReply = function()
{
    //return this.replyable && !this.isForwarded && !this.isSentByMe;
    return this.replyable;
}

FUserMessage.prototype.getCanForward = function()
{
    return this.replyable;
}

