function CombatResultBox()
{
	this.message = null;
    this.showAsCalculatorReport = false;
}

CombatResultBox.prototype.addLine = function()
{
    var items = Array.prototype.slice.call(arguments, 0);
    return UIh.addLine(this.container, Arrays.antidot(items));
}

CombatResultBox.empty2nothing = function(f)
{
    return f.isZero() ? I18.f("NOTHING") : f;
}

CombatResultBox.prototype.createChildren = function()
{
    var text, box, color;
    var magicDisabled = false;
    var summary;
    var tmp = this.message.attackersSummary.concat(this.message.defendersSummary);
    for(var i=0; i<tmp.length; i++)
    {
    	summary = tmp[i];
    	if (FBuff.hasMagicDisable(summary.buffs, this.message.tick))
    	{
            magicDisabled = true;
            break;
        }
    }

    var predictionPolicy = !this.message.predictionPolicy ? "" : " (" + this.message.predictionPolicy.toString() + ")";
    if (this.message.isForwarded)
    {
        text = (this.message.result.name == MissionResult.SUCCESS.name ? I18.f("ATTACKER_WIN") : I18.f("ATTACKER_LOSE")) + predictionPolicy;
        color = Effects.LIGHT_ORANGE;
        box = UI.line(text, color, "bold", true);
        box.style.textAlign = "center";
        this.container.appendChild(box);
    }
    else if (this.showAsCalculatorReport)
    {
        var youFight = this.message.receiverAttacked() || this.message.receiverDefend();
        color = CombatViewUtils.getBattleStatusColor(this.message.receiverWonCombat(), youFight);
        if (youFight)
        {
            text = (this.message.receiverWonCombat() ? I18.f("YOU_WIN") : I18.f("YOU_LOSE")) + predictionPolicy + "<br/>"
                    + I18.f("PREDICTION_BASED_ON", FConfig.COMBAT_CALCULATIONS_AVG_OF);
        }
        else
        {
            text = (this.message.result.name == MissionResult.SUCCESS.name ? I18.f("ATTACKER_WIN") : I18.f("ATTACKER_LOSE")) + predictionPolicy + "<br/>"
                    + I18.f("PREDICTION_BASED_ON", FConfig.COMBAT_CALCULATIONS_AVG_OF);
        }
        box = UI.line(text, color, "bold", true);
        box.style.textAlign = "center";
        this.container.appendChild(box);
    }

    var tick = this.message.tick;
    for (var ind = 0; ind < this.message.attackersSummary.length; ind++)
    {
        summary = this.message.attackersSummary[ind];
        if (summary)
        {
            var armyOwnerDesc = CombatViewUtils.createArmyOwnerDescription(this.message, summary.username, true);
            if (armyOwnerDesc)
            {
				this.addLine(CombatRole.ATTACKER.toString() + ":", armyOwnerDesc);
                this.addLine(CombatViewUtils.createArmyView(summary, this.message.attackersForces[ind], tick));
                this.addHeroLines(summary, this.message.result.name == MissionResult.SUCCESS.name, true, ind > 0, magicDisabled);
                this.addCasts(summary.nonHeroCasts, false, true, false, magicDisabled, null);
                this.extraResourcesLines(summary);
                if (summary.releasedHeroes.length > 0)
                {
                    this.addLine(I18.f("RELEASED_HEROES"), this.heroesView(summary.releasedHeroes, summary.race));
                }
                if (summary.imprisonedHeroes.length > 0)
                {
                    this.addLine(I18.f("IMPRISONED_HEROES"), this.heroesView(summary.imprisonedHeroes, summary.race));
                }
            }
        }
    }
    if (this.showAsCalculatorReport)
    {
        if (this.message.attackersHealingOverflow > 0)
        {
            UIh.addLine(this.container, I18.f("HEALING_OVERFLOW", this.message.attackersHealingOverflow));
        }
    }

    this.extraBodyLines();

    if (!this.message.isOwnMission && !this.message.defenderVisible)
    {
        this.addLine(I18.f("SYSTEM_MESSAGE_BODY_YOU_UNKNOWN"));
    }
    
    if (!this.message.isOwnMission || this.message.defenderVisible)
    {
        for (ind = 0; ind < this.message.defendersSummary.length; ind++)
        {
            summary = this.message.defendersSummary[ind];
            if (summary)
            {
                this.addLine(CombatRole.DEFENDER.toString() + ":", CombatViewUtils.createArmyOwnerDescription(this.message, summary.username, false));
                var armyView = CombatViewUtils.createArmyView(summary, this.message.defendersForces[ind], tick);
                this.addLine(armyView.DOM);
                if (ind == 0)
                {
                    this.addLine(CombatViewUtils.createTowersBox(this.message));
                }
                if (ind == 0 && this.message.gateInitialHealth != 0)
                {
                    $("row", armyView.unitBox)[0].appendChild(CastleGateView.create(this.message, this.message.gateInitialHealth, this.message.gateResultHealth, true).DOM);
                }
                this.addHeroLines(summary, this.message.result.name == MissionResult.DEFEAT.name, false, ind > 0, magicDisabled);
                this.addCasts(summary.nonHeroCasts, false, false, false, magicDisabled, null);
                this.extraCrystalLines(summary);
                this.extraResourcesMarauderedLines(summary);
                if (this.showAsCalculatorReport)
                {
                    UIh.addLine(this.container, I18.f("LOSSES_IN_GOLD") + ":", CombatResultBox.empty2nothing(Funds.createWithGold(summary.losesCost)));
                }
            }
        }
    }
    else
    {
        for (ind = 0; ind < this.message.defendersSummary.length; ind++)
        {
            if (this.message.defendersSummary[ind])
            {
                this.addLine(CombatRole.DEFENDER.toString() + ":", CombatViewUtils.messageBodyLabel(this.message, this.message.defendersSummary[ind].username, false));
                var levels = new FMap();
                for (var i = 0; i < UnitType.MILITARY.length; i++)
                {
                    levels.put(UnitType.MILITARY[i], 0);
                }
                this.addLine(I18.f("SYSTEM_MESSAGE_BODY_ENEMY_UNKNOWN"));
                var unknownArmy = COM.create("ArmyView");
                unknownArmy.setAccessLevelRequired();
                if (ind == 0)
                {
                    var miniTowers = false;
                    var unknownTowers = CombatViewUtils.towerBox(miniTowers);
                    unknownTowers.appendChild(TowerView.createEmpty(miniTowers).DOM);
                    unknownTowers.appendChild(TowerView.createEmpty(miniTowers).DOM);
                    unknownArmy.DOM.appendChild(unknownTowers);
                }
                this.addLine(unknownArmy.DOM);
                this.extraCrystalLines(this.message.defendersSummary[ind]);
                this.extraResourcesMarauderedLines(this.message.defendersSummary[ind]);
            }
        }
    }
    if (this.showAsCalculatorReport)
    {
        if (this.message.defendersHealingOverflow > 0)
        {
            UIh.addLine(this.container, I18.f("HEALING_OVERFLOW", this.message.defendersHealingOverflow));
        }
    }
}

CombatResultBox.prototype.addHeroLines = function(summary, won, attacker, ally, magicDisabled)
{
    if (summary.hero)
    {
        if (this.showAsCalculatorReport && summary.prisonRatio != 0)
        {
            this.addLine(I18.getMegaString("IMPRISONMENT_PROBABILITY", {prisonRatio:Math.min(1, summary.prisonRatio)}));
        }
        this.addCasts(summary.hero.casts, true, attacker, ally, magicDisabled, summary.hero);

        var lbl = document.createElement("span");
        lbl.style.fontWeight = "bold";
        var xpGained = summary.heroExp;
        if (summary.hero.maxLevelReached)
        {
            xpGained = 0;
            lbl.style.color = Effects.DARK_RED;
            /*
            lbl.setStyle("textDecoration", "underline");
            lbl.toolTip = summary.hero.level == FConfig.i.HERO_MAX_LEVEL
                    ? I18.HERO_MAX_LEVEL_TOTAL.f()
                    : I18.HERO_MAX_LEVEL_SCIENCE.f();
			*/
        }
        else if (FBuff.hasBuff(summary.buffs, BuffType.HERO_EXPERIENCE, null, this.message.tick))
        {
            lbl.style.color = Effects.DARK_GREEN;
            lbl.style.textDecoration = "underline";
            
            COM.bindExclusiveClick(lbl, function()
            {
                var b, c, tip = $("<div/>");
                for(var i=0; i<summary.buffs.length; i++)
                {
                	b = summary.buffs[i];
                	if(b.buffType.name == BuffType.HERO_EXPERIENCE.name)
                	{
                		c = $("<div/>");
                		c.append(I18.getBuffCause(b));
                		c.append(I18.percentText(b.value));
                		
                		tip.append(c);
                	}
                }
                
                TipDialog.create(tip[0]);
            });
        }
        lbl.innerHTML = NumberUtils.format(xpGained, 0, ".", " ");
        this.addLine(I18.f("HERO_GAINED_EXP"), lbl);
        if (summary.manaGain > 0)
        {
            this.addLine(I18.f("HERO_GAINED_MANA", summary.manaGain));
        }
        if (summary.manaLose > 0)
        {
            this.addLine(I18.f("HERO_LOSED_MANA", summary.manaLose));
        }
        if (!won)
        {
            var happy = false;
            var cast, buff;
            for(var i=0; i<summary.hero.casts.length; i++)
            {
            	cast = summary.hero.casts[i];
            	for(var j=0; j<cast.buffs.length; j++)
            	{
            		buff = cast.buffs[j];
            		if (buff.buffType.name == BuffType.HERO_NO_UPSET.name)
            		{
                        happy = true;
                    }
            	}
            }
            if (happy)
            {
                this.addLine(I18.f("HERO_DOUBLE_UPSET"));
            }
        }
    }
}

CombatResultBox.prototype.addCasts = function(casts, isHeroCasts, attacker, ally, magicDisabled, hero)
{
    var tmp = [];
    for(var i=0; i<casts.length; i++)
    {
    	if(casts[i].isMilitary(attacker, ally) && !casts[i].cause.system)
    	{
    		tmp.push(casts[i]);
    	}
    }
    casts = tmp;
    if (casts.length > 0)
    {
        var cv = COM.create("CastsView");
		cv.visibleItems = 10;
		cv.vertical = false;
		cv.update(null, hero, casts, magicDisabled);
        this.addLine((isHeroCasts ? I18.f("HERO_EFFECTS") : I18.f("OTHER_EFFECTS")) + ":", cv);
    }
}

CombatResultBox.prototype.extraResourcesMarauderedLines = function(summary)
{
    if (!summary.building_marauder.isZero())
    {
        UIh.addLineRaceDependent(this.container, summary.race, I18.f("MARAUDERED_FROM_BUILDINGS") + ":", summary.building_marauder);
    }
    if (!summary.army_marauder.isZero())
    {
        UIh.addLineRaceDependent(this.container, summary.race, I18.f("MARAUDERED_FROM_ARMY") + ":", summary.army_marauder);
    }
    if (summary.convertedSlaves != 0)
    {
        UIh.addLineRaceDependent(this.container, summary.race, I18.f("CONVERTED_SLAVES"),
                new Funds().setValue(Resource.PEOPLE, summary.convertedSlaves));
    }
    if (summary.convertedSlavesToClan != 0)
    {
        UIh.addLineRaceDependent(this.container, summary.race, I18.f("CONVERTED_SLAVES_TO_CLAN"),
                new Funds().setValue(Resource.PEOPLE, summary.convertedSlavesToClan));
    }
}

CombatResultBox.prototype.extraCrystalLines = function(summary)
{
    if (summary.crystals.length > 0 || summary.bagOverflowByCrystal)
    {
        var l = UI.labelWithTip(I18.f("CRYSTALS"));
        COM.bindExclusiveClick(l, function()
        {
            TipDialog.create(ArtifactPic.createArray(summary.crystals, !summary.bagOverflowByCrystal ? null : UI.styledLabel(I18.f("NO_ARTIFACT_BAG_OVERFLOW"), Style.PROBLEM_INFO), I18.f("CRYSTALS")));
        });
        UIh.addLine(this.container, I18.f("LOOT") + ":", l);
    }
}

CombatResultBox.prototype.extraWarPointsLine = function(summary)
{
    if (summary.warPoints > 0)
    {
        this.addLine(I18.f("GOT_WAR_POINTS") + ":", IconAndValue.create(STATIC_URL + CommonAssets.WAR_POINT, NumberUtils.format(summary.warPoints, 2)));
    }
}

CombatResultBox.prototype.getCapturedEstate = function()
{
    if (this.message.savedByBuilding)
    {
        return I18.f("CAPTURE_FAILED_CAUSE_DEFENSE_BUILDING");
    }
    else if (this.message.result.name == MissionResult.SUCCESS.name)
    {
        return I18.fixName(this.message.toEstateName);
    }
    return I18.f("NOTHING");
}

CombatResultBox.prototype.extraBodyLines = function()
{
    if (this.message.messageClass == "MissionCaptureMessage")
    {
        this.addLine(I18.f("CAPTURED_ESTATE") + ":", this.getCapturedEstate());
        this.addLine(I18.f("BUILDINGS_DESTROYED_COUNT") + ":", this.message.buildingsDestroyed);
        this.addLine(I18.f("BUILDINGS_DAMAGED_COUNT") + ":", this.message.buildingsDamaged);
    }
    else if (this.message.messageClass == "MissionDestroyBuildingsMessage")
    {
        this.addLine(I18.f("TARGET_BUILDING") + ":", this.getBuildingName());
        this.addLine(I18.f("BUILDINGS_DESTROYED_COUNT") + ":", this.message.buildingsDestroyed);
        this.addLine(I18.f("BUILDINGS_DAMAGED_COUNT") + ":", this.message.buildingsDamaged);
    }
    else if (this.message.messageClass == "MissionDestroyCampMessage")
    {
        if (this.message.result.name == MissionResult.SUCCESS.name)
        {
            this.addLine(I18.f("CAMP_WAS_DESTROYED"));
        }
    }
    else if (this.message.messageClass == "MissionDestroyCityMessage")
    {
        if (this.message.result.name == MissionResult.SUCCESS.name)
        {
            this.addLine(I18.f("CITY_WAS_DESTROYED"));
        }
        else if (this.message.savedByBuilding)
        {
            this.addLine(I18.f("DESTROY_FAILED_CAUSE_DEFENSE_BUILDING"));
        }
        else
        {
            this.addLine(I18.f("CITY_DESTROY_FAILED"));
        }
    }
}

CombatResultBox.prototype.getBuildingName = function(bt)
{
    return I18.getBuildingName(!bt ? this.message.buildingType : bt, this.message.defendersSummary[0].race);
}

CombatResultBox.prototype.heroesView = function(heroes, race)
{
    var box = UI.denseHBox();
    for (var i = 0; i < heroes.length; i++)
    {
        var h = heroes[i];
        var lbl = document.createElement("span");
        lbl.innerHTML = Strings.safeHtml(i == heroes.length - 1 ? h.name : h.name + ',');
        lbl.style.textDecoration = "underline";
        lbl.style.fontWeight = "bold";
        
        COM.bindExclusiveClick(lbl, function()
        {
            var hv = HeroFaceView.createWith(h, race);
            hv.update(h, race);
            Alert.create(Tip.create(hv.DOM).DOM);
        });

        box.appendChild(lbl);
    }
    return box;
}

CombatResultBox.prototype.extraResourcesLines = function(summary)
{
    if (this.message.messageClass == "MissionCaptureMessage")
    {
        this.addLine(I18.f("BURNED") + ":", !summary.burn.isZero() ? summary.burn : I18.f("NOTHING"));
        this.extraCrystalLines(summary);
        this.extraWarPointsLine(summary);
        this.extraResourcesMarauderedLines(summary);
    }
    else if (this.message.messageClass == "MissionDestroyBuildingsMessage")
    {
        this.extraCrystalLines(summary);
        this.extraWarPointsLine(summary);
        this.extraResourcesMarauderedLines(summary);
    }
    else if (this.message.messageClass == "MissionDestroyCampMessage")
    {
        this.extraCrystalLines(summary);
        this.extraWarPointsLine(summary);
        this.extraResourcesMarauderedLines(summary);
    }
    else if (this.message.messageClass == "MissionRescueHeroMessage")
    {
        this.extraCrystalLines(summary);
        this.extraWarPointsLine(summary);
        this.extraResourcesMarauderedLines(summary);
    }
    else
    {
        var race = this.message.defendersSummary[0].race;
        var lootItems = [];
        lootItems.push(I18.f("LOOT") + ":");
        if (!summary.loot.isZero())
        {
            lootItems.push(summary.loot);
        }
        if (!summary.flags.isEmpty())
        {
            var l = UI.labelWithTip(I18.f("FLAG"));
            lootItems.push(l);
            COM.bindExclusiveClick(l, function(e)
            {
                TipDialog.create(FlagsView.createFlags(summary.flags, 0, false).DOM, I18.f("FLAG"));
            });
        }
        if (summary.crystals.length > 0 || summary.bagOverflowByCrystal)
        {
            var l = UI.labelWithTip(I18.f("CRYSTALS"));
            lootItems.push(l);
            COM.bindExclusiveClick(l, function()
            {
                TipDialog.create(ArtifactPic.createArray(summary.crystals, !summary.bagOverflowByCrystal ? null : UI.styledLabel(I18.f("NO_ARTIFACT_BAG_OVERFLOW"), Style.PROBLEM_INFO), I18.f("CRYSTALS")));
            });
        }
        if (summary.standard)
        {
            var l = UIh.createLine(
                    UI.labelWithTip(I18.f("STANDARD") + ' '),
                    I18.f("TO_ESTATE", [I18.formatEstateName(I18.f("CASTLE_NAME"), summary.standardCastle)])
            );
            lootItems.push(l);
            
            COM.bindExclusiveClick(l, function()
            {
                TipDialog.create(
                        '<img src="' + STATIC_URL + CommonAssets["STANDARD_" + summary.standard.name] + '"/>',
                        I18.f("CAST_LASTS_TILL", Tick.toTimeFuture(summary.standard.till))
                );
            });
        }
        if (lootItems.length == 1 && summary.crystals.length == 0 && !summary.standard)
        {
            lootItems.push(I18.f("NOTHING"));
        }

        if (this.showAsCalculatorReport)
        {
            UIh.addLine(this.container, I18.f("LOSSES_IN_GOLD") + ":", CombatResultBox.empty2nothing(Funds.createWithGold(summary.losesCost)));
        }
        UIh.addLineRaceDependent(this.container, race, lootItems);
        this.extraWarPointsLine(summary);
        if (this.showAsCalculatorReport)
        {
            var profit = Funds.createWithGold(summary.loot.toCost() - summary.losesCost);
            UIh.addLine(this.container, I18.f("PROFIT") + ":", CombatResultBox.empty2nothing(profit));
        }

        if (!summary.clan.isZero())
        {
            UIh.addLineRaceDependent(this.container, race, summary.clan.people > 0 ? I18.f("CLANED_RESERVATION") : I18.f("CLANED_WAREHOUSE"), summary.clan);
            if (!summary.burn.isZero())
            {
                UIh.addLineRaceDependent(this.container, race, I18.f("BURNED") + ":", summary.burn);
            }
        }
        else
        {
            UIh.addLineRaceDependent(this.container, race, I18.f("BURNED") + ":", !summary.burn.isZero() ? summary.burn : I18.f("NOTHING"));
        }
        this.extraResourcesMarauderedLines(summary);
        var line = [];
        if (summary.lootPearls != 0)
        {
            var pearlsNumber = document.createElement("span");
            pearlsNumber.innerHTML = summary.lootPearls.toString();
            Style.FUNDS.applyTo(pearlsNumber);
            line.push(I18.f("GOD_GIVES") + ":");
            line.push(CommonAssets.ICON_PEARL);
            line.push(pearlsNumber);
        }
        if (summary.artifact || summary.bagOverflow)
        {
            if (line.length > 0)
            {
                var sp = document.createElement("span");
                sp.style.marginRight = "20px";
                sp.innerHTML = "&nbsp;";
                line.push(sp);
            }
            line.push(I18.f("DETECTED") + ":");
            
            var l = UI.labelWithTip(I18.f("MAGIC_ITEM"));
            
            COM.bindExclusiveClick(l, function()
            {
            	summary.bagOverflow
                        ? TipDialog.create(UI.styledLabel(I18.f("NO_ARTIFACT_BAG_OVERFLOW"), Style.PROBLEM_INFO))
                        : TipDialog.create(ArtifactPic.create(summary.artifact, []).DOM, I18.f("MAGIC_ITEM"))
            });
            
            line.push(l);
        }
        
        if (summary.rune || summary.bagOverflowByRune)
        {
            if (line.length > 0)
            {
                /*
                sp = new Spacer();
                sp.width = 20;
                line.addItem(sp);
                */
            }
            line.push(I18.f("LOOTED"));
            var l = UI.styledLabel(I18.f("RUNE"));
            l.style.textDecoration = "underline";
            
            COM.bindExclusiveClick(l, function()
            {
                TipDialog.create(summary.bagOverflowByRune
                        ? UI.styledLabel(I18.f("NO_ARTIFACT_BAG_OVERFLOW"), Style.PROBLEM_INFO)
                        : ArtifactPic.create(summary.rune, []).DOM);
            });
            
            line.push(l);
        }
        
        if (line.length > 0)
        {
            this.addLine(line);
        }
    }
}