function MissionAttackMessageView()
{
	callParentConstructor(MissionAttackMessageView.prototype, this);
}

extendClass(MissionAttackMessageView.prototype, SystemMessageView.prototype);

MissionAttackMessageView.prototype.getMessage = function()
{
    return this._message;
}

MissionAttackMessageView.prototype.setMessage = function(msg)
{
    this._message = msg;
}

MissionAttackMessageView.prototype.getSubject = function()
{
    if (this.message.template)
    {
        return I18.getMegaString(this.message.template + "_SUBJECT", this.message.params);
    }
    else
    {
        var s;
        switch (this.message.result.name)
        {
            case MissionResult.SUCCESS.name:
                if (this.message.isForwarded)
                {
                    s = I18.f("SYSTEM_MESSAGE_SUBJECT_ATTACK_OTHER_SUCCESS");
                }
                else if (this.message.isOwnMission)
                {
                    s = I18.f("SYSTEM_MESSAGE_SUBJECT_ATTACK_SUCCESS");
                }
                else
                {
                    s = I18.f("SYSTEM_MESSAGE_SUBJECT_ATTACK_DEFEAT");
                }
                break;
            case MissionResult.DEFEAT.name:
                if (this.message.isForwarded)
                {
                    s = I18.f("SYSTEM_MESSAGE_SUBJECT_ATTACK_OTHER_DEFEAT");
                }
                else if (this.message.isOwnMission)
                {
                    s = I18.f("SYSTEM_MESSAGE_SUBJECT_ATTACK_DEFEAT");
                }
                else
                {
                    s = I18.f("SYSTEM_MESSAGE_SUBJECT_ATTACK_SUCCESS");
                }
                break;
            case MissionResult.FAIL.name:
                return I18.getAttackFailSubject(this.message.failReason, this.message.toFieldXYZ, this.message.toEstateNameAndLocation());
            default:
                throw Error("Cannot find subject for mission result " + message.result.name);
        }
        
        return I18.megaTemplate(s, [this.message.attackerUserAndLocation(), this.message.defenderUserAndLocation(), this.getMissionGoal()]);
    }
}

MissionAttackMessageView.prototype.getHeaderColor = function()
{
    if (this.message.isForwarded)
    {
        return Effects.LIGHT_ORANGE;
    }
    
    return CombatViewUtils.getBattleResultColor(this.message, this.message.isOwnMission, MissionAttackMessageView.prototype.headerColor);
}

MissionAttackMessageView.prototype.createChildren = function()
{
    superMethod(MissionAttackMessageView.prototype, "createChildren", this)();
           
    if (this.message.template)
    {
        var s = I18.getStringOrNull(this.message.template + "_BODY");
        if (s)
        {
            this.addLine(I18.getMegaString(this.message.template + "_BODY", this.message.params));
        }
    }
    if (this.message.result.equals(MissionResult.FAIL))
    {
        this.addLine(I18.f("SYSTEM_MESSAGE_BODY_ATTACK_FAIL"));
    }
    else
    {
    	var crb = COM.create("CombatResultBox");
    	crb.message = this.message;
        crb.createChildren();
        this.addLine(crb.DOM);
        

		/*
        var details:Button = new Button();
        details.label = I18.DETAILS.f();
        details.addEventListener(MouseEvent.CLICK, onDetailsClicked);

        var forward:Button = new Button();
        forward.label = I18.MAIL_FORWARD.f();
        forward.addEventListener(MouseEvent.CLICK, function(e:MouseEvent):void{
            renderer.onForwardMessage(message);
        });
        forward.visible = message.template == null;
        */
        /*
	    var details = COM.create("SimpleButton");
	    details.setCaption(I18.f("DETAILS"));
	    COM.bindExclusiveClick(details.DOM, $.proxy(function()
	    {
	    	this.onDetailsClicked(this.message);
	    }, this));
        


	    var forward = COM.create("SimpleButton");
	    forward.setCaption(I18.f("MAIL_FORWARD"));
	    COM.bindExclusiveClick(forward.DOM, $.proxy(function()
	    {
	    	this.renderer.onForwardMessage(this.message);
	    }, this));
        
        UI.show((this.message.template == null),[forward]); 
        this.addLine(details.DOM, forward.DOM);
        
        
        */
        
    }
}

MissionAttackMessageView.prototype.setDetailsButton = function()
{
	if (this.message.result.equals(MissionResult.FAIL)) return false;
	var details = COM.create("SimpleButton");
    COM.bindExclusiveClick(details.DOM, $.proxy(function()
    {
    	this.onDetailsClicked(this.message);
    }, this));
    return details;
}

MissionAttackMessageView.prototype.setForwardButton = function()
{
	if (this.message.result.equals(MissionResult.FAIL)) return false;	
	var forward = COM.create("SimpleButton");
    COM.bindExclusiveClick(forward.DOM, $.proxy(function()
    {
    	this.renderer.onForwardMessage(this.message);
    }, this));    
    UI.show((this.message.template == null),[forward]); 
	return forward;
}

MissionAttackMessageView.prototype.onDetailsClicked = function(e)
{
	APP.showLoader();
	Service.mail.callMethod("getCombatReport",[MailBoxView.i().ownerUsername, this.message.id],$.proxy(function (result/*:FCombatReport*/)
    {
		var d = COM.create("CombatReportDialog");
		d.popup(this.message, result);
    }, this));
}

MissionAttackMessageView.prototype.getMissionGoal = function()
{
    if (this.message.messageClass == "MissionCaptureMessage")
    {
        return I18.f("MISSION_GOAL_CAPTURE");
    }
    else if (this.message.messageClass == "MissionDestroyBuildingsMessage")
    {
        return I18.f("MISSION_GOAL_DESTROYBUILDINGS");
    }
    else if (this.message.messageClass == "MissionLootMessage")
    {
        return I18.f("MISSION_GOAL_LOOT");
    }
    else if (this.message.messageClass == "MissionSlaveTakingMessage")
    {
        return I18.f("MISSION_GOAL_SLAVESTAKING");
    }
    else if (this.message.messageClass == "MissionDestroyCampMessage")
    {
        return I18.f("MISSION_GOAL_DESTROYCAMP");
    }
    else if (this.message.messageClass == "MissionDestroyCityMessage")
    {
        return I18.f("MISSION_GOAL_DESTROYCITY");
    }
	else if (this.message.messageClass == "MissionDestroyCatapultMessage") 
	{
		return I18.f('MISSION_GOAL_DESTROYCATAPULT');
	}    
    else if (this.message.messageClass == "MissionRescueHeroMessage")
    {
        return I18.f("MISSION_GOAL_RESCUEHERO");
    }
    throw Error("Mission goal not found");
}