function MissionEscapeView()
{
	callParentConstructor(MissionEscapeView.prototype, this);
}

extendClass(MissionEscapeView.prototype, SystemMessageView.prototype);

MissionEscapeView.prototype.getSubject = function()
{
    var m = this._message;
    switch(m.failReason)
    {
        case "ESTATE_RECAPTURED":
            return I18.f("ESCAPE_MISSION_MESSAGE_ESCAPE_FROM_ESTATE_SUBJECT", m.fromEstateNameAndLocation());
        case "ESTATE_LEFT":
            return I18.f("ESCAPE_MISSION_MESSAGE_YOU_HAVE_LEFT_ESTATE_SUBJECT", m.fromEstateNameAndLocation());
        case "ESTATE_SELFDESTRUCTED":
        case "ESTATE_SELFDESTRUCTED_GROTTO":
            return I18.f("ESCAPE_MISSION_MESSAGE_ESTATE_SELFDESTRUCTED_SUBJECT", m.fromEstateNameAndLocation());
        case "ESTATE_DESTROYED":
            return I18.f("ESCAPE_MISSION_MESSAGE_ESTATE_DESTROYED_SUBJECT", m.fromEstateNameAndLocation());
        case "ESTATE_MOVED":
            return I18.f("ESCAPE_MISSION_MESSAGE_ESTATE_MOVED_SUBJECT", m.fromEstateNameAndLocation());
        case "NO_ESTATE":
            return I18.f("ESCAPE_MISSION_MESSAGE_NO_ESTATE_SUBJECT", m.fromEstateNameAndLocation());
        default:
            throw Error("unknown mission escape cause");
    }
}

MissionEscapeView.prototype.createChildren = function()
{
	superMethod(MissionEscapeView.prototype, "createChildren", this)();
	var message = this._message;
	if (ArmyUtils.isEmpty(message.attackersForces[0].initial) && message.estateOwner.hero == null) 
	{
        // estate is selfdestructed w/o army
		
		if (FUser.isClanUser(message.estateOwner.username))
		{
            this.addLine(this.createText(I18.f("CLAN_ESTATE_ELIMINATED", message.fromEstateNameAndLocation())));
		}
		else
		{
			this.addLine(this.createText(I18.f("NO_ARMY_IN_ELIMINATED_ESTATE")));
		}
    }
	else
	{
        var initialArmy = ArmyUtils.createArmy(message.attackersForces[0].initial);
        var diedArmy = new Army(initialArmy);
 		diedArmy.subtract(ArmyUtils.createArmy(message.attackersForces[0].result));
        if (message.failReason == "ESTATE_RECAPTURED")
        {
            this.addLine(this.createText(I18.f("YOUR_ARMY_DISCOVER_RECAPTURED_PIT",message.fromEstateNameAndLocation(), message.toEstateNameAndLocation())));
        } 
        else
        {
            this.addLine(this.createText(I18.f("YOUR_ARMY_RETREATED_IN_ESTATE",message.toEstateNameAndLocation())));
        }
        
        var view = COM.create("ArmyView");
		view.setUnits(message.estateOwner.race, message.estateOwner.levels, FBuff.hasBuff(message.estateOwner.buffs, BuffType.MONSTER_RACE, null, 0));
        view.setCount(initialArmy);
        //if (message.estateOwner.hero) 
        view.setHero(message.estateOwner.hero, message.estateOwner.race);
		this.addLine(view.DOM);

        if (message.estateOwner.hero != null)
        {
            this.addLine(this.createText(I18.f("HERO_RETREATS_TO_HIDEOUT")));
        }

		if (!diedArmy.isZero())
        {
            var text = ArmyUtils.count(message.attackersForces[0].result)==0 ? I18.f("ALL_ARMY") : diedArmy.sum().toString();
        	this.addLine(this.createText(I18.f("DESERTED_CHARACTERS")+' '+text));
                       
            var view2 = COM.create("ArmyView");
            view2.setHero(null, message.estateOwner.race);
            view2.setUnits(message.estateOwner.race, message.estateOwner.levels, FBuff.hasBuff(message.estateOwner.buffs, BuffType.MONSTER_RACE, null, message.tick));
            view2.setCount(diedArmy);
            this.addLine(view2.DOM);
        }
    }
	
	if (message.failReason == "ESTATE_SELFDESTRUCTED_GROTTO")
	{
		this.addLine(this.createText(I18.f("GROTTO_COMPLETE")));
	}
}
