function MilitaryMissionDialog()
{
	callParentConstructor(MilitaryMissionDialog.prototype, this);
	
	this.garrison = new Army();
    this.targetUnitAura = new EmptyUnitAura();
    this.heroDialog = null;
    
    this.__defineGetter__("military", this.getMilitary);
    this.__defineGetter__("selectedBuffs", this.getSelectedBuffs);
}

extendClass(MilitaryMissionDialog.prototype, MissionDialogChild.prototype);

MilitaryMissionDialog.prototype.__construct = function()
{
	superMethod(MilitaryMissionDialog.prototype, "__construct", this)();
	
	COM(this.equalizer).onChange = $.proxy(this.onChange, this);
	this.initEqualizer();
}

MilitaryMissionDialog.prototype.checkMissionConstraints = function(estate)
{
    if (!this.checkMissionCommonConstraints(estate))
    {
        return false;
    }
    var b;
    
    var equalizer = COM(this.equalizer);
    
    if (this.parentDialog.isMissionsLimitReached())
    {
        this.setNotification(I18.f("MISSIONS_LIMIT_REACHED"));
        return false;
    }
    else if (this.garrison.isZero() && estate.heroes.length == 0)
    {
        this.setNotification(I18.f("MISSION_EMPTY_GARRISON"));
        return false;
    }
    else if (estate.isGateClosed())
    {
        this.setNotification(I18.getMissionEntranceClosedMessage(estate.__className__ == "FOuterEstate"));
        return false;
    }
    else if (!this.parentDialog.isEnoughCost())
    {
        this.setNotification(I18.f("MISSION_NOT_ENOUGH_RESOURCES"));
        return false;
    }
    else if (estate.isGarrisonBlocked)
    {
        this.setNotification(I18.f("MISSION_BLOCKED_GARRISON") + " " + NumberUtils.ceil(Tick.ticksTill(estate.garrisonUnblockTick)));
        return false;
    }
    else if (equalizer.selectedArmy.isZero() && !equalizer.heroSelected)
    {
        this.setNotification(I18.f("MISSION_EMPTY_SQUAD"));
        return false;
    }
    else if (!equalizer.selectedArmy.isZero() && estate.hasBuff(BuffType.ARMY_CANNOT_MOVE))
    {
        b = estate.getLongestBuff(BuffType.ARMY_CANNOT_MOVE);
        this.setNotification(I18.f("EFFECT_ACTIVE_TILL", [I18.getBuffCause(b), Tick.toTimeFuture(b.cast.tillTick)]));
        return false;
    }
    else if (this.parentDialog.getRealRetreatLimit() < FConfig.MIN_RETREAT_PERCENT || this.parentDialog.getRealRetreatLimit() > 100) 
    {
        this.setNotification(I18.f("MISSION_WRONG_RETREAT_LIMIT"));
        return false;
    }
    else if (equalizer.heroSelected && estate.garrisonHero.isUpset())
    {
        if (estate.garrisonHero.upsetByTitle)
        {
            this.setNotification(I18.f("HERO_ERROR_REQUIRED_MAX_TITLE"));
        }
        else
        {
            this.setNotification(I18.f("MISSION_HERO_CANNOT_MOVE"));
        }
        return false;
    }
    else if (equalizer.heroSelected && estate.hasBuff(BuffType.HERO_CANNOT_MOVE))
    {
        b = estate.getLongestBuff(BuffType.HERO_CANNOT_MOVE);
        this.setNotification(I18.f("EFFECT_ACTIVE_TILL", [I18.getBuffCause(b), Tick.toTimeFuture(b.cast.tillTick)]));
        return false;
    }
    return true;
}

MilitaryMissionDialog.prototype.isAttack = function()
{
    return false;
}

MilitaryMissionDialog.prototype.updateUnitBonuses = function()
{
    var attackerBonuses = [];

    var attacker = this.parentDialog.estate.owner;
    var terrain = this.target.terrain;
    var ownerRace = !this.target.estate ? Race.MONSTER : this.target.estate.owner.race;
    
    var betrayingBonusPercent = this.isAttack() ? this.target.estate.owner.isFake() || this.target.estate.owner.isBetrayer() ? 0 : CombatBonusUtil.calcBetrayerAttackBonus(attacker.race, ownerRace) : 0;
    var betrayer = this.isAttack() ? attacker.isBetrayer() || !this.target.estate.owner.isFake() && attacker.race.equals(ownerRace) : false;

    for (var i = 0, im = UnitType.MILITARY.length; i < im; i++)
    {
        var ut = UnitType.MILITARY[i];
        var attackingTypeBonus = new AuraBonus();
        attackingTypeBonus.damage = CombatBonusUtil.calcAttackBonus(attacker.race, betrayer, terrain, ut) + betrayingBonusPercent;
        attackingTypeBonus.defence = CombatBonusUtil.calcDefenceBonus(attacker.race, betrayer, terrain, ut);
        attackerBonuses.push(attackingTypeBonus);
    }
    
    this.targetUnitAura = new UnitAura(attackerBonuses);
}

MilitaryMissionDialog.prototype.init = function()
{
    superMethod(MilitaryMissionDialog.prototype, "init", this)();
    this.updateUnitBonuses();
}

MilitaryMissionDialog.prototype.update = function(info)
{
    superMethod(MilitaryMissionDialog.prototype, "update", this)(info);

    if (this.heroDialog)
    {
        this.heroDialog.update(this.estate);
    }
    if (this.estate.hasBuff(BuffType.ARMY_RATION))
    {
        COM(this.parentDialog.cost).onTip = this.costTooltipHandler;
    }
    
    COM(this.equalizer).toField = this.military.getToField();
    COM(this.equalizer).update(this.estate, this.targetUnitAura);
    COM(this.equalizer).tick();
}

MilitaryMissionDialog.prototype.tick = function()
{
    superMethod(MilitaryMissionDialog.prototype, "tick", this)();
    
    this.garrison = this.estate.garrison;
    if (COM(this.equalizer).estate)
    {
        COM(this.equalizer).tick();
    }
    if (this.heroDialog)
    {
        this.heroDialog.tick();
    }
    if (this.canApply)
    {
        this.canApply = this.checkMissionConstraints(this.estate);
    }
    
    this.parentDialog.capacity.innerHTML = NumberUtils.floor(this.military.army.getCapacity(this.estate.owner.race, this.estate.science) *
            (1 + FBuff.sumOfBuffs(this.selectedBuffs, BuffType.SCAVENGERS_CAPACITY, null, FEstate.tickNowForEstate(this.estate)))).toString();
    if (FBuff.hasBuff(this.selectedBuffs, BuffType.SCAVENGERS_CAPACITY, null, FEstate.tickNowForEstate(this.estate)))
    {
        //Style.assignTip(parentDialog.capacity, " ");
    }
    else
    {
        //Style.removeTip(parentDialog.capacity);
    }
}

MilitaryMissionDialog.prototype.costTooltipHandler = function(r)
{
    /*
    var tip:Tip = Tip.createGrid();

    var cost:Funds = new Funds();
    cost.setValue(Resource.GOLD, military.getDefaultCost());
    tip.addLine(UI.styledText(I18.MISSION_COST, Style.BOLD), cost);

    var buffs:Array = estate.getBuffs(BuffType.ARMY_RATION);
    for each (var buff:FBuff in buffs) {
        tip.addLine(I18h.getBuffCause(buff), I18h.percentReversedText(buff.value));
    }

    return tip;
    */
}

MilitaryMissionDialog.prototype.showRetreatLimit = function()
{
    UI.show(true, [this.parentDialog.retreatLimitBox]);
    COM(this.parentDialog.retreatLimit).value = FUser.CURRENT_USER.settings.getMissionRetreatLimit();
}

MilitaryMissionDialog.prototype.onChange = function()
{
    this.military.army = COM(this.equalizer).selectedArmy;
    this.military.heroId = COM(this.equalizer).selectedHeroId;
    this.military.setFromEstate(this.estate);
    this.military.refreshTravelDuration();
    this.parentDialog.tick();
}

MilitaryMissionDialog.prototype.reset = function()
{
    superMethod(MilitaryMissionDialog.prototype, "reset", this)();
	COM(this.equalizer).reset();
}

MilitaryMissionDialog.prototype.initEqualizer = function()
{
    /*
    var equalizer = COM(this.equalizer);
    var av = COM(COM(equalizer.availableArmyView).armyView);
    var sv = COM(COM(equalizer.selectedArmyView).armyView);
    
    av.onHeroClick = this.heroClickFactory(ArmyView.createBuffsGetter(av));
    sv.onHeroClick = this.heroClickFactory(ArmyView.createBuffsGetter(sv));
    
    COM(equalizer.heroHideout).onHeroClick = this.heroClickFactory(ArmyView.createBuffsGetter(null));
    */
}

MilitaryMissionDialog.prototype.heroClickFactory = function(buffsGetter)
{
    return $.proxy(function(h, r)
    {
        this.heroDialog = HeroDialog.create(this.estate, h, COM(this.equalizer).selectedArmy, r, buffsGetter(), NaN, false);
        heroDialog.onClose = $.proxy(function()
        {
            this.heroDialog = null;
        }, this);
    }, this);
}

MilitaryMissionDialog.prototype.onArriveTimeTooltip = function(e)
{
    this.createSpeedTip(this.estate, this.military.getBaseTravelTime(), this.military.outcomingSpeedBuffType, this.military.incomingSpeedBuffType, null, 0, this.military);
}

MilitaryMissionDialog.prototype.onReturnTimeTooltip = function(e)
{
    this.createSpeedTip(this.estate, this.military.getBaseTravelTime() * 2, this.military.outcomingSpeedBuffType, this.military.incomingSpeedBuffType, null, 0, this.military);
}

MilitaryMissionDialog.prototype.onCapacityTooltip = function(e)
{
    var tip = NumberEffectTip.createTip("BASE_SCAVENGER_CAPACITY", this.selectedBuffs, this.military.army.getCapacity(this.estate.owner.race, this.estate.science), BuffType.SCAVENGERS_CAPACITY, FEstate.tickNowForEstate(this.estate));
    TipDialog.create(tip);
}

MilitaryMissionDialog.prototype.getMilitary = function()
{
    return this.missionOrder;
}

MilitaryMissionDialog.prototype.getSelectedBuffs = function()
{
    return COM(COM(COM(this.equalizer).selectedArmyView).armyView).getBuffs();
}

MilitaryMissionDialog.prototype.onResult = function(mission)
{
    superMethod(MilitaryMissionDialog.prototype, "onResult", this)(mission);
    
    if (COM(this.equalizer).stepIndex != FUser.CURRENT_USER.settings.resourceChooserStepIndex)
    {
        Events.onArmyChooserStepIndexChanged.fire(COM(this.equalizer).stepIndex);
    }
}