function MissionAttackDialog()
{
	callParentConstructor(MissionAttackDialog.prototype, this);
	
	this.inviteList = [I18.f("INVITE_NOONE"), I18.f("INVITE_ALLIES"), I18.f("INVITE_CLAN")];
	this.missionGroups = [];
	this.joinList = [];
	
	this.onGoalSelected = $.proxy(this.onGoalSelected, this);
	this.onAssistanceDurationChange = $.proxy(this.onAssistanceDurationChange, this);
	
	this.__defineGetter__("attackOrder", this.getAttackOrder);
	this.__defineGetter__("selectedGroup", this.getSelectedGroup);
}

extendClass(MissionAttackDialog.prototype, MilitaryMissionDialog.prototype);

MissionAttackDialog.prototype.setupParentComponentsVisibility = function()
{
    UI.show(true, [this.parentDialog.capacityBox, this.parentDialog.capacity, this.parentDialog.etaGridRow, this.parentDialog.missionCostBox,
        this.parentDialog.goalRadioHBox, this.parentDialog.selectButton, this.parentDialog.selectButtonLabel, this.parentDialog.alliedAttackBox]);
    this.showTravelMissionsComponents();
    COM(this.parentDialog.etaGridRow).showReturnTime = true;
}

MissionAttackDialog.prototype.reset = function()
{
    superMethod(MissionAttackDialog.prototype, "reset", this)();
    
    $(this.parentDialog.selectButton).off(CLICK_EVENT, this.onGoalSelected);
    $(this.parentDialog.goalRadioButtons).off("change", this.onGoalSelected);
    $(this.parentDialog.assistanceDuration).off("change", this.onAssistanceDurationChange);
    //COM(this.parentDialog.selectButton).icon = STATIC_URL + CommonAssets.BLOCKER_DARKTREE;
    COM(this.parentDialog.selectButton).image.src = STATIC_URL + CommonAssets.BLOCKER_DARKTREE;
    this.attackOrder.typeToDestroy = null;
}

MissionAttackDialog.prototype.isAttack = function()
{
    return true;
}

MissionAttackDialog.prototype.setupParentComponentsProperties = function()
{
    this.parentDialog.costLabel.innerHTML = I18.f("MISSION_SQUAD_WAGE");
    COM(this.parentDialog.assistanceDuration).min = 0;
    COM(this.parentDialog.assistanceDuration).value = 0;

    this.showRetreatLimit();
    COM.bindExclusiveClick(this.parentDialog.selectButton, this.onGoalSelected);
    COM(this.parentDialog.goalRadioButtons).selectedIndex = 0;
    $(this.parentDialog.goalRadioButtons).on("change", this.onGoalSelected);
    
    $(this.parentDialog.assistanceDuration).on("change", this.onAssistanceDurationChange);

    this.parentDialog.onInviteOrAssistChange = $.proxy(this.updateCombos, this);
    this.parentDialog.onInviteComboChange = $.proxy(function(e)
    {
        this.parentDialog.updateGoal();
        if (COM(this.parentDialog.inviteOrAssist).selectedIndex == 1)
        {
            this.updateSelectBuildingButton();
        }
        this.updateGroupBuffs();
    }, this);
}

MissionAttackDialog.prototype.createMissionOrder = function()
{
    this.missionOrder = new MissionAttackOrder();
}

MissionAttackDialog.prototype.getAttackOrder = function()
{
    return this.missionOrder;
}

MissionAttackDialog.prototype.checkMissionConstraints = function(estate)
{
    var prepare = this.parentDialog.prepareInfo;
    var alreadyHaveAttack = this.selectedGroup && this.selectedGroup.numberOfUserMissions > 0;
    if (this.target.estate.isOuterEstate && this.target.estate.type.name == EstateType.RUIN.name)
    {
        if (!alreadyHaveAttack && !estate.owner.canAttackAnyRuin() && estate.owner.titleIndex > this.target.estate.maxTitleIndexToAttack)
        {
            this.setNotification(I18.f("CANNOT_ATTACK_WEAK_RUIN"));
            return false;
        }
    }
    else if (this.target.estate.isOuterEstate && this.target.estate.type.name == EstateType.CASTLE.name)
    {
        if (!alreadyHaveAttack && estate.owner.diplomacy.clan && estate.owner.diplomacy.clan.castles.length > this.target.estate.maxTitleIndexToAttack)
        {
            this.setNotification(I18.f("CANNOT_ATTACK_WEAK_CASTLE"));
            return false;
        }
    }
    else if (this.target.estate.isOuterEstate
            && this.target.estate.type.name == EstateType.GROTTO.name
            && !FUser.CURRENT_USER.tournamentGrotto.isInTournament)
	{
            this.setNotification(I18.f("CANNOT_ATTACK_NOT_IN_TOURNAMENT"));
            return false;
    }
    else if (!alreadyHaveAttack && estate.owner.titleIndex > this.target.estate.maxTitleIndexToAttack && !prepare.ignoreTitleIndex)
    {
        this.setNotification(I18.f("CANNOT_ATTACK_PLAYER_WITH_LOWER_TITLE"));
        return false;
    }
    
    if (prepare.limitedBan)
    {
        this.setNotification(I18.f("MISSION_START_ERROR_BANPROTECTION"));
        return false;
    }
    else if (prepare.godprotection)
    {
        this.setNotification(I18.f("MISSION_START_ERROR_GODPROTECTION"));
        return false;
    }
    else if (estate.owner.isClanMate(this.target.estate.owner.username))
    {
        this.setNotification(I18.f('MISSION_START_ERROR_CANNOT_ATTACK_CLANMATE'));
        return false;
    }
    else if (estate.owner.isAlly(this.target.estate.owner.username))
    {
        this.setNotification(I18.f("MISSION_START_ERROR_CANNOT_ATTACK_ALLY"));
        return false;
    }
    else if (FBuff.hasBuff(this.target.buffs, BuffType.PROHIBIT_ATTACK, null, FEstate.tickNowForEstate(estate)))
    {
        this.setNotification(I18.f("MISSION_START_ERROR_ATTACK_PROHIBITED"));
        return false;
    }
    else if (this.parentDialog.captureRadio.selected && this.target.estate && this.target.estate.type.name != EstateType.GROTTO.name && prepare.outerEstatesLimitReached && !this.selectedGroup)
    {
        this.setNotification(I18.f("MISSION_START_ERROR_OUTER_ESTATES_LIMIT_REACHED"));
        return false;
    }
    else if (this.parentDialog.captureRadio.selected && this.target.estate && this.target.estate.type.name == EstateType.GROTTO.name && prepare.grottoLimitReached && !selectedGroup)
    {
        this.setNotification(I18.f("MISSION_START_ERROR_GROTTO_LIMIT_REACHED"));
        return false;
    }
    else if (this.parentDialog.captureCastleRadio.selected && prepare.castlesLimitReached && !this.selectedGroup)
    {
        this.setNotification(I18.f("MISSION_START_ERROR_CASTLES_LIMIT_REACHED"));
        return false;
    }
    else if (prepare.nextTimeWhenAttackIsAvailable > this.attackOrder.arriveTick && !alreadyHaveAttack)
    {
        if (prepare.ruinLootLimitReached)
        {
            this.setNotification(I18.f("MISSION_START_ERROR_RUIN_LOOTS_LIMIT_REACHED", [FConfig.RUIN_LOOT_LIMIT, Tick.toTimeFuture(prepare.nextTimeWhenAttackIsAvailable)]));
        }
        else
        {
            this.setNotification(I18.f("MISSION_START_ERROR_ATTACK_LIMIT_REACHED", [FConfig.ATTACK_LIMIT_PER_CITY_COUNT, Tick.toTimeFuture(prepare.nextTimeWhenAttackIsAvailable)]));
        }
        return false;
    }
    else if (!this.attackOrder.willJoinInTime(this.selectedGroup))
    {
        this.setNotification(I18.f("WILL_NOT_JOIN_IN_TIME"));
        return false;
    }
    else if (this.parentDialog.destroyRadio.selected && !this.selectedGroup && !this.attackOrder.typeToDestroy)
    {
        this.setNotification(I18.f("MISSION_BUILDING_NOT_SELECTED"));
        return false;
    }
    else if (this.selectedGroup && this.selectedGroup.withMyHero && this.attackOrder.heroId != -1)
    {
        this.setNotification(I18.f("MISSION_START_ERROR_ONE_HERO_IN_GROUP"));
        return false;
    }
    
    return superMethod(MissionAttackDialog.prototype, "checkMissionConstraints", this)(estate);
}

MissionAttackDialog.prototype.update = function(info)
{
    superMethod(MissionAttackDialog.prototype, "update", this)(info);
    this.missionGroups = info.groups;

    var inviteSelection = COM(this.parentDialog.inviteCombo).selectedIndex;

    this.joinList = [];
	
    if (!info.groups || info.groups.length == 0)
    {
        this.joinList.push(I18.f("NO_MISSION_TO_JOIN"));
    }
    else
    {
	    for(var i=0; i<info.groups.length; i++)
	    {
	    	this.joinList.push({label: info.groups[i].toString(), value: info.groups[i]});
	    }
    }

    this.updateCombos();
    this.updateSelectBuildingButton();

    COM(this.parentDialog.inviteCombo).selectedIndex = inviteSelection >= 0 ? inviteSelection : 0;
    
    this.updateCaption();
}

MissionAttackDialog.prototype.updateGroupBuffs = function()
{
    COM(this.equalizer).update(this.estate, this.targetUnitAura, this.selectedGroup ? this.selectedGroup.buffs : []);
    COM(this.equalizer).tick();
}

MissionAttackDialog.prototype.updateCombos = function(e)
{
    var ic = COM(this.parentDialog.inviteCombo);
    var wasInvite = ic.dataProvider == this.inviteList;
    if (COM(this.parentDialog.inviteOrAssist).selectedIndex == 0)
    {
        if (!wasInvite)
        {
            ic.dataProvider = this.inviteList;
            ic.selectedIndex = 0;
        }
        ic.disabled = false;
    }
    else
    {
        ic.dataProvider = this.joinList;
        if (wasInvite)
        {
            ic.selectedIndex = 0;
        }
        ic.disabled = !(this.missionGroups && this.missionGroups.length > 0);
    }
    
    ic.refresh();

    this.parentDialog.updateGoal();

    this.updateGroupBuffs();
}

MissionAttackDialog.prototype.updateSelectBuildingButton = function()
{
    var b = COM(this.parentDialog.selectButton);
    
    var mg = this.selectedGroup;
    if (mg && mg.typeToDestroy)
    {
        this.setSelectedBuildingButtonIcon(mg.typeToDestroy);
    }
    else if (this.attackOrder.typeToDestroy)
    {
        this.setSelectedBuildingButtonIcon(this.attackOrder.typeToDestroy);
    }
    else
    {
        //b.icon = STATIC_URL + CommonAssets.BLOCKER_DARKTREE;
        b.image.src = STATIC_URL + CommonAssets.BLOCKER_DARKTREE;
    }
    
    UI.show(this.parentDialog.destroyRadio.selected, [b, this.parentDialog.selectButtonLabel]);
    b.enabled = !this.selectedGroup;
}

MissionAttackDialog.prototype.setSelectedBuildingButtonIcon = function(bt)
{
    var race = this.target.estate.owner.race;
    if (race.name == Race.CASTLE_MONSTER.name && this.target.estate.type.name != EstateType.CASTLE.name)
    {
        race = this.parentDialog.estate.owner.race;
    }
    //COM(this.parentDialog.selectButton).icon = STATIC_URL + Assets.B_findBy(race, bt);
    COM(this.parentDialog.selectButton).image.src = STATIC_URL + Assets.B_findBy(race, bt);
}

MissionAttackDialog.prototype.tick = function()
{
    superMethod(MissionAttackDialog.prototype, "tick", this)();

    this.updateSelectBuildingButton();

    if (!this.isNotificationSet()
            && !this.target.estate.owner.isFake()
            && !this.target.estate.owner.isBetrayer()
            && this.target.estate.owner.race.name == this.info.estate.owner.race.name
            && !this.info.estate.owner.isBetrayer())
	{
        //this.setNotification(I18.f("WARNING_BECOME_BETRAYER"), Effects.ORANGE);
        COM(this.parentDialog.apply).warning = I18.f("WARNING_BECOME_BETRAYER");
        COM(this.parentDialog.apply).skin = "orange";
    }

    if (!this.isNotificationSet() && this.info.estate.hasBuff(BuffType.HERO_MISSION_INVISIBLE) && COM(this.equalizer).heroSelected)
    {
        //this.setNotification(I18.f("WARNING_MISSION_IS_INVISIBLE"), Effects.GREEN);
        COM(this.parentDialog.apply).warning = I18.f("WARNING_MISSION_IS_INVISIBLE");
        COM(this.parentDialog.apply).skin = "orange";
    }

    var terrain = this.attackOrder.getToField().terrain;
    var isBerserk = this.attackOrder.heroId != -1 && this.estate.garrisonHero.hasWorkingBuff(BuffType.BERSERK, terrain, FEstate.tickNowForEstate(this.estate));
    if (isBerserk)
    {
        COM(this.parentDialog.retreatLimit).disabled = true;
        COM(this.parentDialog.retreatLimit).value = 100;
        this.parentDialog.onRetreatLimitTip = $.proxy(function(e)
        {
            //e.toolTip = BuffCauseTip.create(FBuff.getBuffs(estate.garrisonHero.getWorkingBuffs(terrain), BuffType.BERSERK, null, FEstate.tickNowForEstate(estate)));
        }, this);
        //parentDialog.retreatLimit.toolTip = " ";
    }
    else if (this.selectedGroup && this.selectedGroup.retreatLimit != -1)
    {
        //this.parentDialog.onRetreatLimitTip = null;
        COM(this.parentDialog.retreatLimit).disabled = true;
        var group = this.selectedGroup;
        COM(this.parentDialog.retreatLimit).value = group ? group.retreatLimit : 0;
        //this.parentDialog.retreatLimit.toolTip = I18.ATTACKS_MUST_HAVE_SAME_RETREAT_LIMIT.f();
    }
    else
    {
        //parentDialog.onRetreatLimitTip = null;
        COM(this.parentDialog.retreatLimit).disabled = false;
        //parentDialog.retreatLimit.toolTip = "";
    }
}

MissionAttackDialog.prototype.onApply = function()
{
    this.attackOrder.goal = this.parentDialog.selectedGoal;
    this.attackOrder.retreatLimit = COM(this.parentDialog.retreatLimit).value*1;
    this.attackOrder.joinMissionId = this.selectedGroup ? this.selectedGroup.id : -1;
    this.attackOrder.typeToDestroy = this.selectedGroup ? this.selectedGroup.typeToDestroy : this.attackOrder.typeToDestroy;
    this.attackOrder.assistanceDurationTicks = this.parentDialog.assistanceBox.style.display != "none" 
    	? COM(this.parentDialog.assistanceDuration).value * Tick.TICKS_PER_HOUR 
    	: 0
    ;
    
    if (COM(this.parentDialog.inviteOrAssist).selectedIndex == 0)
    {
        this.attackOrder.alliesInvited = COM(this.parentDialog.inviteCombo).selectedIndex == 1;
        this.attackOrder.clanInvited = COM(this.parentDialog.inviteCombo).selectedIndex == 2;
    }
}

MissionAttackDialog.prototype.onAssistanceDurationChange = function(e)
{
    this.attackOrder.assistanceDurationTicks = this.parentDialog.assistanceBox.style.display != "none" 
    	? COM(this.parentDialog.assistanceDuration).value * Tick.TICKS_PER_HOUR 
    	: 0
    ;
    this.parentDialog.tick();
}

MissionAttackDialog.prototype.onGoalSelected = function(e)
{
    this.updateCaption();
    
    UI.show(this.parentDialog.selectedGoal.name == MissionAttackGoal.CAPTURE_CASTLE.name, [this.parentDialog.assistanceBox]);

    if (!this.target.estate || this.parentDialog.selectedGoal.name != MissionAttackGoal.DESTROY_BUILDINGS.name)
    {
        return;
    }
    
    var race = this.target.estate.owner.race.name != Race.MONSTER.name ? this.target.estate.owner.race : FUser.CURRENT_USER.race;
    var t = this.target.estate.isOuterEstate ? this.target.estate.type : null;
    
    SelectBuildingToDestroyDialog.popup(race, t, $.proxy(function(type)
    {
        //COM(this.parentDialog.selectButton).icon = STATIC_URL + Assets.B_findBy(race, type);
        COM(this.parentDialog.selectButton).image.src = STATIC_URL + Assets.B_findBy(race, type);
        this.attackOrder.typeToDestroy = type;
    }, this));
}

MissionAttackDialog.prototype.updateCaption = function()
{
	this.parentDialog.setCaption(COM(this.parentDialog.goalRadioButtons).selectedItem.label);
}

MissionAttackDialog.prototype.updateTicks = function()
{
    this.attackOrder.updateTicks(this.selectedGroup);
}

MissionAttackDialog.prototype.getSelectedGroup = function()
{
    if ((COM(this.parentDialog.inviteOrAssist).selectedIndex == 1) && COM(this.parentDialog.inviteCombo).selectedItem && (COM(this.parentDialog.inviteCombo).selectedItem.value.__className__ == "FMissionGroup"))
    {
        return COM(this.parentDialog.inviteCombo).selectedItem.value;
    }

    return null;
}