function MissionBodyView()
{
	callParentConstructor(MissionBodyView.prototype, this);
	
	this.mission = null;
    this.calculateHandler = null;
    this.cancelHandler = null;
    this.snapshotTick = null;
    this.heroDialog = null;
    this.lastUpdateTick = null;
}

extendClass(MissionBodyView.prototype, Dialog.prototype);

MissionBodyView.create = function(m, calculateHandler, cancelHandler, snapshotTick)
{
    var mdv = COM.create("MissionBodyView");
    mdv.calculateHandler = calculateHandler;
    mdv.cancelHandler = cancelHandler;
    mdv.snapshotTick = snapshotTick;
    mdv.update(m);
    mdv.popup();
    
    return mdv;
}

MissionBodyView.prototype.__construct = function()
{
	COM.bindExclusiveClick(this.cancelButton, $.proxy(this.onCancel, this));
	COM.bindExclusiveClick(this.calculateButton, $.proxy(this.onCalculate, this));
	
	this.scroll = new iScroll(this.scrollWrapper);
}

MissionBodyView.prototype.update = function(m)
{
    COM(this.unitView).race = m.owner.race;
    UI.show(!m.army.isZero() || m.hero || m.isPhantom, [this.armyBox, this.lootCaption], $.proxy(function()
    {
        this.lootCaption.innerHTML = (m.className == "MissionCamp") ? I18.f("MISSION_CAMP") : I18.f("MISSION_LOOT");
        var allowIllusion = m.groupMissionsCount == 1 || m.hero;
        var buffs = FBuff.getMissionBuffs(m.to.terrain, m.to.estateType, m.owner.race, m.to.race, allowIllusion, m.armyBuffs);
        var monsterRace = FBuff.hasBuff(m.armyBuffs, BuffType.MONSTER_RACE, null, this.snapshotTick >= 0 ? this.snapshotTick : Tick.now());
        var av = COM(COM(this.armyView).armyView);
        
        av.setUnits(m.owner.race, m.ownerUnitLevels, monsterRace, m, buffs);
        av.setCount(m.army, m.armyFaded);
        av.setHero(m.hero, m.owner.race);
        av.onHeroClick = $.proxy(function()
        {
            this.lastUpdateTick = Tick.now();
            heroDialog = HeroDialog.create(null, m.hero, m.army, m.owner.race, buffs, this.snapshotTick >= 0 ? this.snapshotTick : NaN, true, true);
        }, this);
    }, this));
    
    UI.show(m.isSpecial(), [this.unitView], $.proxy(function()
    {
        COM(this.unitView).utype = m.specialUnit;
        COM(this.unitView).update(1, m.unitCount);
    }, this));
    
    var showRes = !m.res.isZero();
    var race = m.to.race ? m.to.race : Race.MONSTER;
    if (m.specialUnit && m.specialUnit.name == UnitType.SETTLER.name)
    {
        race = m.from.race;
        showRes = m.state.name != MissionState.RETURNING.name;
    }
    UI.show(showRes, [this.resources], $.proxy(function()
    {
        COM(this.resView).update(m.res, race);
    }, this));
    
    this.ownerLabel.innerHTML = I18.f("OPPONENT_ARMY", FUser.getUserNameForMessages(m.owner.username));
    UI.show(m.isGroupMission, [this.initiatorLabel, this.initiatorSpacer], $.proxy(function()
    {
        this.initiatorLabel.innerHTML = I18.f("GROUP_INITIATOR", FUser.getUserNameForMessages(m.groupInitiator)
                + (m.groupInitiatorAttackIndex >= 0 ? " (" + (m.groupInitiatorAttackIndex + 1) + ")" : "" ));
    }, this));
    
    UI.show(m.calculable && !m.userCanCancel, [this.calculateButton]);
    UI.show(m.userCanCancel && this.snapshotTick < 0, [this.cancelButton], this.adjustButtonPos);
    UI.show(m.hasSpeedBuffs(), [this.missionSpeed], $.proxy(function()
    {
    	$(this.missionSpeed).html("");
    	var tip = NumberEffectTip.createSpeedTip("MISSION_DURATION", 0, m.speedupBuffs, m.speedupAncientMagic, m.isConstantDuration ? m.arriveTick - m.startTick : -1, m.owner.username);
    	$(this.missionSpeed).append(tip.DOM);
    }, this));
    
    this.mission = m;
    if (this.heroDialog && this.heroDialog.DOM.style.display != "none" && m.hero)
    {
        this.heroDialog.updateMainHero(m.hero);
    }
    var casts = m.isMilitary ? Arrays.filter(m.nonHeroCasts, $.proxy(function(c)
    {
        return c.isMilitary(true, false);
    }, this)) : [];

    UI.show(casts.length > 0, [this.castsView, this.castsLabel, this.castsSpacer], $.proxy(function()
    {
        COM(this.castsView).update(null, m.hero, casts, false);
    }, this));

    this.adjustButtonPos();
    
    this.createCityTooltip(this.fromEstateTip, m.from);
    this.createCityTooltip(this.toEstateTip, m.to);
    
    setTimeout($.proxy(this.scroll.refresh, this.scroll), 300);
}

MissionBodyView.prototype.createCityTooltip = function(parent, estate)
{
    var owner = estate.owner;
    var desc = "";
    var hideStars = false;
    if (FUser.isCurrentUser(owner.username))
    {
        desc = I18.f("YOUR_ESTATE");
    }
    else if (!owner.username || owner.username.length == 0)
    {
        desc = I18.f("ESTATE_OF_NOONE");
        hideStars = true;
    }
    else if (owner.username.charAt(0) == '$')
    {
        desc = I18.f("ESTATE_OF_CLAN", estate.location, owner.username.substr(1));
    }
    else
    {
        desc = I18.f("ESTATE_OF_PLAYER", estate.location, FUser.getUserNameForMessages(owner.username));
    }

    var view = EstateStarsClanView.create(owner, estate.getEstateIcon(), desc, estate.estateType);
    if(hideStars)
    {
        view.starsView.style.display = "none";
        view.iconImage.style.display = "none";
    }
    
    parent.innerHTML = "";
    parent.appendChild(view.DOM);
}

MissionBodyView.prototype.tick = function()
{
    var cancelTip = "";
    if (!this.mission.ownerCanCancel && this.mission.isAssist && this.mission.state.name != MissionState.RETURNING.name)
    {
        cancelTip = this.mission.to.estateType.name == EstateType.CITY.name ? I18.f("CANNOT_CANCEL_ASSISTANCE_GATE_CLOSED") : I18.f("CANNOT_CANCEL_ASSISTANCE_TUNNEL_CLOSED");
    }
    if (this.mission.isDeputy && !this.mission.userCanCancel)
    {
        cancelTip = I18.f("DEPUTY_CAN_NOT_CANCEL_THIS_MISSION");
    }
    if (this.mission.userCanCancel)
    {
        //cancelTip = I18.f("CANCEL_MISSION");
    }
    if(!this.mission.cancelable)
    {
    	cancelTip = I18.f("CANNOT_CANCEL_MISSION");
    }
    //COM(this.cancelButton).enabled = this.mission.cancelable;
    COM(this.cancelButton).reason = cancelTip;

    COM(this.cancelButton).visible = this.mission.userCanCancel && this.snapshotTick < 0;
    COM(this.calculateButton).visible = this.mission.calculable && !this.mission.userCanCancel;

    this.adjustButtonPos();
    if (this.heroDialog && this.heroDialog.DOM.style.display != "none" && this.mission.hero)
    {
        var now = Tick.now();
        this.mission.hero.tick(now - this.lastUpdateTick);
        this.lastUpdateTick = now;
        this.heroDialog.tick();
    }
}

MissionBodyView.prototype.onCancel = function()
{
	Alert.create(I18.f('CANCEL_MISSION_CONFIRMATION'), "", $.proxy(this.doCancel, this));
}

MissionBodyView.prototype.doCancel = function()
{
    if (!Quarantine.contains(this.cancelButton))
    {
        Quarantine.put(this.cancelButton);
        if (this.mission.cancelable)
        {
            if (this.mission.isAncientsAssist)
            {
                Alert.create(I18.f("CANCEL_ANCIENTS_WARNING"), "",
                $.proxy(function()
                {
                    this.cancelHandler(this.mission);
                    this.parent.turnOffBody();
                }, this),
                function()
                {
                    Quarantine.clear();
                });
            }
            else
            {
                this.cancelHandler(this.mission);
                this.parent.turnOffBody();
            }
        }
    }
}

MissionBodyView.prototype.onCalculate = function()
{
    if (!Quarantine.contains(this.calculateButton))
    {
        Quarantine.put(this.calculateButton);
        this.calculateHandler(this.mission);
    }
}

MissionBodyView.prototype.adjustButtonPos = function()
{
    /*
    if (unitView.visible && !armyBox.visible) {
        cancelButton.x = 77;
        cancelButton.y = 0;
        missionSpeed.x = 77;
        missionSpeed.y = 20;
    } else {
        cancelButton.x = 700;
        cancelButton.y = 19;
        missionSpeed.x = 700;
        missionSpeed.y = 39;
    }
    */
}

MissionBodyView.prototype.onMissionSpeedTip = function(e)
{
    //e.toolTip = NumberEffectTip.createSpeedTip("MISSION_DURATION", 0, mission.speedupBuffs, mission.speedupAncientMagic, mission.isConstantDuration ? mission.arriveTick - mission.startTick : -1, mission.owner.username);
}