function MissionDetails()
{
	callParentConstructor(MissionDetails.prototype, this);
	
	this.calculateHandler = null;
    this.cancelHandler = null;
    this.snapshotTick = -1;
    this.user = null;
    this.estateId = null;

    this.bodyView = null;
    
    this.__defineGetter__("mission", this.getMission);
}

MissionDetails.EMPTY_FIELD = "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;-&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";

extendClass(MissionDetails.prototype, ScrollableListItem.prototype);

MissionDetails.prototype.__construct = function()
{
	COM.bindExclusiveClick(this.DOM, $.proxy(this.turnOnBody, this))
}

MissionDetails.prototype.refresh = function(data)
{
    this.goalLabel.innerHTML = I18.getMissionGoal(this.mission);
    UI.show(false, [this.stateImage]);
    UI.show(true, [this.stateLabel]);
    this.stateLabel.innerHTML = I18.getMissionStateWithResult(this.mission)
    
    this.arriveLabel.innerHTML = Tick.isInfinity(this.mission.arriveTick) ? MissionDetails.EMPTY_FIELD : Tick.toTimeFutureSeconds(this.mission.arriveTick, false);
    this.returnLabel.innerHTML = Tick.isInfinity(this.mission.returnTick) ? MissionDetails.EMPTY_FIELD : Tick.toTimeFutureSeconds(this.mission.returnTick, false);
    
    this.originLabel.innerHTML = "<b>" + this.mission.origin.getName() + "</b>" + "<br/>" + (this.mission.origin.location ? this.mission.origin.location.toString() : "");
    this.originImg.src = STATIC_URL + this.mission.origin.getEstateIcon();
    this.targetLabel.innerHTML = "<b>" + this.mission.target.getName() + "</b>" + "<br/>" + (this.mission.target.location ? this.mission.target.location.toString() : "");
    this.targetImg.src = STATIC_URL + this.mission.target.getEstateIcon();
    this.DOM.style.backgroundColor = this.getHeaderColor(this.mission);
}

MissionDetails.prototype.turnOnBody = function()
{
    this.bodyView = MissionBodyView.create(this.mission, this.calculateHandler, this.cancelHandler, this.snapshotTick);
    this.bodyView.parent = this;
    MissionListDialog.missionDetailsDialog = this.bodyView;
    this.bodyView.onClose = $.proxy(function()
    {
    	MissionListDialog.missionDetailsDialog = null;
    	this.bodyView = null;
    }, this);
}


MissionDetails.prototype.turnOffBody = function()
{
	this.bodyView.close();
	this.bodyView = null;
}

MissionDetails.prototype.getMission = function()
{
    return this.data;
}

MissionDetails.prototype.tick = function()
{
    var left = Tick.isInfinity(this.mission.nextUpdateTick) ? this.mission.nextUpdateTick :
            (this.snapshotTick < 0 ? Tick.ticksTill(this.mission.nextUpdateTick) : this.mission.nextUpdateTick - this.snapshotTick);
	
    this.timeLabel.innerHTML = Tick.toTimeLeft(left);
    if (this.bodyView)
    {
        this.bodyView.tick();
    }
}

MissionDetails.prototype.getHeaderColor = function(m)
{
    var red = "#f44a4a"; //Effects.RED;
    var blue = "#4a9af4"; //Effects.BLUE;
    var green = "#4af452"; //Effects.GREEN;
    var grey = "#aaaaaa"; //Effects.DARK_GREY;
    if (m.isPhantom)
    {
        return grey;
    }

    if (m.isAttack)
    {
        if (m.state.name != MissionState.RETURNING.name)
        {
            if (FUser.isSameName(m.victimUsername, this.user.username))
            {
                if (FUser.isFakeUserName(this.user.username))
                {
                    if (m.from.id.equals(this.estateId))
                    {
                        return blue;
                    }
                }
                return red;
            }

            if (this.user.diplomacy && this.user.hasBuff(BuffType.SENSE_THREATS_TO_ALLIES))
            {
                for(var i=0; i<this.user.diplomacy.allies.length; i++)
                {
                	if (FUser.isSameName(m.victimUsername, this.user.diplomacy.allies[i].user.username))
                	{
                        return red;
                    }
                }
            }
            if (this.user.diplomacy && this.user.hasClanRole(ClanRole.WARMINISTER))
            {
                if (this.user.isClanMate(m.victimUsername))
                {
                    return red;
                }
            }
        }
        if (m.isGroupMission)
        {
            return blue;
        }
        if (m.owner.isFake())
        {
            return blue;
        }
        return green;
    }
    if (m.isAssist)
    {
        return Effects.ORANGE;
    }
    return "#fff";
}

MissionDetails.prototype.displayAsGroupMission = function(m)
{
    return (m.isAttack && m.groupId != -1 && (m.groupMultipleInvited || m.groupMissionsCount > 1));
}

MissionDetails.prototype.onGoalTooltip = function(e)
{
    /*
    Assets.waitCanShowMission(mission, function ():void {
        e.toolTip = Tip.create(MissionDetailsTip.create(mission, displayAsGroupMission(mission), snapshotTick));
    });
    */
}

MissionDetails.prototype.onCityTooltip = function(event, estate)
{
    /*
    var owner:FUserInfo = estate.owner;
    var desc:String;
    if (FUser.isCurrentUser(owner.username)) {
        desc = I18.YOUR_ESTATE.f();
    } else if (owner.username == null || owner.username.length == 0) {
        desc = I18.ESTATE_OF_NOONE.f();
    } else if (owner.username.charAt(0) == '$') {
        desc = I18.ESTATE_OF_CLAN.f(estate.location, owner.username.substr(1));
    } else {
        desc = I18.ESTATE_OF_PLAYER.f(estate.location, FUser.getUserNameForMessages(owner.username));
    }
    var view:EstateStarsClanView = EstateStarsClanView.create(owner, estate.getEstateIcon(), desc, estate.estateType);
    event.toolTip = Tip.create(view);
    */
}