function MissionDialogChild()
{
	this.parentDialog = null;
    this.missionOrder = null;
    
    this.__defineGetter__("canApply", this.getCanApply);
    this.__defineSetter__("canApply", this.setCanApply);
    this.__defineGetter__("target", this.getTarget);
    this.__defineGetter__("info", this.getInfo);
    this.__defineGetter__("estate", this.getEstate);
    this.__defineGetter__("race", this.getRace);
    this.__defineGetter__("hasDeathBuffs", this.getHasDeathBuffs);
    this.__defineGetter__("hasCapacityBuffs", this.getHasCapacityBuffs);
}

MissionDialogChild._instances = {};

MissionDialogChild.create = function(childClass, parent)
{
    var dialog = MissionDialogChild._instances[childClass];
    if (!dialog)
    {
        dialog = COM.create(childClass);
        //MissionDialogChild._instances[childClass] = dialog;
        dialog.parentDialog = parent;
        dialog.createChildren();
    }
    else
    {
        dialog.reset();
        dialog.parentDialog = parent;
        dialog.resetDialog();
    }
    parent.onResult = $.proxy(dialog.onResult, dialog);
    
    return dialog;
}

MissionDialogChild.prototype.__construct = function()
{
}

MissionDialogChild.prototype.setNotification = function(text, color)
{
	this.parentDialog.setNotification(text, color);
}

MissionDialogChild.prototype.isNotificationSet = function()
{
    return COM(this.parentDialog.apply).reason;
}

MissionDialogChild.prototype.getCanApply = function()
{
	return !COM(this.parentDialog.apply).reason;
}

MissionDialogChild.prototype.setCanApply = function(value)
{
	//COM(this.parentDialog.apply).skin = value ? "green" : "red";
	if(value)
	{
		COM(this.parentDialog.apply).reason = null;
	}
	else
	{
		if(!COM(this.parentDialog.apply).reason)
		{
			COM(this.parentDialog.apply).reason = "Ololo";
		}
	}
}

MissionDialogChild.prototype.createMissionOrder = function()
{
}

MissionDialogChild.prototype.update = function(info)
{
    COM(this.parentDialog.cost).onTip = null;
}

MissionDialogChild.prototype.tick = function()
{
}

MissionDialogChild.prototype.onApply = function()
{
}

MissionDialogChild.prototype.init = function()
{
	this.resetDialog();
}

MissionDialogChild.prototype.updateTicks = function()
{
    this.missionOrder.updateTicks();
}

MissionDialogChild.prototype.getOriginId = function(combo)
{
    return combo.selected.id;
}

MissionDialogChild.prototype.getTarget = function()
{
    return this.parentDialog.target;
}

MissionDialogChild.prototype.getInfo = function()
{
    return this.parentDialog.prepareInfo;
}

MissionDialogChild.prototype.getEstate = function()
{
    return this.info.estate;
}

MissionDialogChild.prototype.getRace = function()
{
    return FUser.CURRENT_USER.race;
}

MissionDialogChild.prototype.setupParentComponentsVisibility = function()
{
}

MissionDialogChild.prototype.setupParentComponentsProperties = function()
{
}

MissionDialogChild.prototype.resetParentComponentsProperties = function()
{
    COM(this.parentDialog.etaGridRow).resetComponentProperties();
}

MissionDialogChild.prototype.createChildren = function()
{
    this.createMissionOrder();
}

MissionDialogChild.prototype.resetDialog = function()
{
    this.resetParentComponentsProperties();
    this.setupParentComponentsProperties();
    this.setupParentComponentsVisibility();
}

MissionDialogChild.prototype.showTravelMissionsComponents = function()
{
    UI.show(true, [this.parentDialog.etaGridRow, this.parentDialog.missionSpeedBox]);
    UI.show(false, [COM(this.parentDialog.etaGridRow).boostExpeditorsIcon, COM(this.parentDialog.etaGridRow).boostMerchantsIcon]);
    COM(this.parentDialog.etaGridRow).showReturnTime = false;
}

MissionDialogChild.prototype.reset = function()
{
    this.createMissionOrder();
}

MissionDialogChild.prototype.updateFromEstate = function()
{
}

MissionDialogChild.prototype.onResult = function(mission)
{
    //console.log(mission);
    //return;
    
    var m = mission.prepareForRequest();
    //console.log(m);
    //return;
    
    Service.mission.callMethod("createMission", [m],
    	$.proxy(function(f)
    	{
            //console.log(f);
            
            Alert.create(I18.f('MISSION_CREATED'));
            
            if(f.owner)
            {
            	FUser.updateCurrentUser(f.owner);
            	//hack
            	//APP.getCityMediator().viewComponent.estate.owner.currentMissionsCount++;
            }
            
            this.parentDialog.close();
        }, this),
        $.proxy(function(error)
        {
            if (error.IS_GAME_EXCEPTION && error.type == "DURATION_CHANGED")
            {
                Alert.create(I18.f("DURATION_HAS_CHANGED"));
                return true;
            }
            if (error.__className__ == "CantStartMissionException" || error.__className__ == "NotEnoughResourcesException" || error.__className__ == "NotEnoughArmyException")
            {
				Alert.create(error.getMessage());
                return true;
            }
            this.parentDialog.close();
            Alert.create(I18.f("MISSION_WAS_NOT_SENT"));
            return false;
        }, this));
}

MissionDialogChild.prototype.getHasDeathBuffs = function()
{
    return false;
}

MissionDialogChild.prototype.getHasCapacityBuffs = function()
{
    return false;
}

MissionDialogChild.prototype.onCapacityTooltip = function(e)
{
}

MissionDialogChild.prototype.onArriveTimeTooltip = function(e)
{
}

MissionDialogChild.prototype.onReturnTimeTooltip = function(e)
{
}

MissionDialogChild.prototype.onDeathTooltip = function(e)
{
}

MissionDialogChild.prototype.createSpeedTip = function(estate, baseSpeed, outcomingBuff, incomingBuff, buildingBoosted, boostValue, mo)
{
    var speedupBuffs = [];
    var buff;
    
    if (mo.IS_MILITARY_MISSION_ORDER)
    {
        if (mo.heroId != -1)
        {
            for(var i=0; i<estate.buffs.length; i++)
            {
            	buff = estate.buffs[i];
            	if (buff.buffType.name == BuffType.HERO_MISSION_TIME.name && buff.isActiveAt(FEstate.tickNowForEstate(estate)) && buff.affectsEstateField(estate, mo.getToField()))
            	{
                    speedupBuffs.push(buff);
                }
            }
        }
    }
    for(var i=0; i<estate.buffs.length; i++)
    {
        buff = estate.buffs[i];
        if (outcomingBuff.name == buff.buffType.name && buff.isActiveAt(FEstate.tickNowForEstate(estate)) && buff.affectsEstateField(estate, mo.getToField()))
        {
            speedupBuffs.push(buff);
        }
    }
    
    var fieldBuffs = mo.getToField().buffs;
    for(var i=0; i<fieldBuffs.length; i++)
    {
        buff = fieldBuffs[i];
        if (incomingBuff.name == buff.buffType.name && buff.isActiveAt(FEstate.tickNowForEstate(estate)) && buff.affectsEstateField(estate, mo.getToField()))
        {
            speedupBuffs.push(buff);
        }
    }
    var speedupAncientMagic = 0;
    if (buildingBoosted)
    {
        if (estate.isMagicActive(buildingBoosted))
        {
            speedupAncientMagic = boostValue;
        }
    }
    
    var tip = NumberEffectTip.createSpeedTip("MISSION_DURATION", baseSpeed, speedupBuffs, speedupAncientMagic, -1, mo.getFromEstate().owner.username);
    TipDialog.create(tip);
}

MissionDialogChild.prototype.createDeathTip = function(estate, baseDeathRate, bt)
{
    var tip = Tip.create();
    tip.addLine(UI.styledText(I18.f("DEATH_RATE"), Style.BOLD), Strings.formatPercent(NumberUtils.ceil(baseDeathRate * 100)));
    var buff;
    for(i=0; i<estate.buffs.length; i++)
    {
    	buff = estate.buffs[i];
    	if (buff.buffType.name == bt.name && buff.isActiveAt(FEstate.tickNowForEstate(estate)))
    	{
            tip.addLine(I18.getBuffCause(buff), I18.percentReversedText(buff.value));
        }
    }

    TipDialog.create(tip);
}

MissionDialogChild.prototype.checkMissionCommonConstraints = function(estate)
{
    if (estate.__className__ == "FCamp" && estate.isSettingUp)
    {
        this.setNotification(I18.f("CAMP_IS_SETTING_UP"));
        return false;
    }
    return true;
}
