function MissionExpeditionDialog()
{
	callParentConstructor(MissionExpeditionDialog, this);
}

extendClass(MissionExpeditionDialog.prototype, MissionDialogChild.prototype);

MissionExpeditionDialog.prototype.setupParentComponentsVisibility = function()
{
    UI.show(true, [this.parentDialog.specialUnitImage, this.parentDialog.deathBox, this.parentDialog.etaGridRow]);
    this.showTravelMissionsComponents();
    UI.show(true, [COM(this.parentDialog.etaGridRow).boostExpeditorsIcon]);
    COM(this.parentDialog.etaGridRow).showReturnTime = true;
}

MissionExpeditionDialog.prototype.setupParentComponentsProperties = function()
{
    this.parentDialog.costLabel.innerHTML = I18.f("MISSION_SQUAD_WAGE");
    //this.parentDialog.goal.innerHTML = I18.f("MISSION_GOAL_EXPEDITION");
	this.parentDialog.setCaption(I18.f("MISSION_GOAL_EXPEDITION"));
    
    var ut = COM(this.parentDialog.specialUnit);
    ut.utype = UnitType.EXPEDITOR;
    ut.showCount = true;
    ut.showMaximum = true;
    ut.showZero = true;
    ut.grayscaleZero = false;
}

MissionExpeditionDialog.prototype.createMissionOrder = function()
{
    this.missionOrder = new MissionExpeditionOrder();
}

MissionExpeditionDialog.prototype.tick = function()
{
    superMethod(MissionExpeditionDialog.prototype, "tick", this)();
    
    var ratePercent = NumberUtils.ceil(100 * NumberUtils.clamp01(MissionSpeed.getExpeditorDeathRate(this.missionOrder.speed) + this.info.estate.sumOfBuffs(BuffType.EXPEDITOR_DEATH)));
    this.parentDialog.deathRate.innerHTML = Strings.formatPercent(ratePercent);
    
    var count = this.info.estate.countFreeSpecialUnits(UnitType.EXPEDITOR);
    COM(this.parentDialog.specialUnit).update(0, 1, count);
    
    if (count == 0)
    {
        if (this.info.estate.hasSpecialUnits(UnitType.EXPEDITOR))
        {
            this.parentDialog.specialUnitsInUnavailableSector(UnitType.EXPEDITOR);
        }
        else
        {
            this.parentDialog.noSpecialUnitsAvailable(UnitType.EXPEDITOR);
        }
        COM(this.parentDialog.specialUnit).unitCount.style.color = Effects.RED;
    }
    else
    {
        COM(this.parentDialog.specialUnit).unitCount.style.color = Effects.WHITE;
        
        if (!this.checkMissionCommonConstraints(this.estate))
        {
            //COM(this.parentDialog.apply).enabled = false;
        }
        else if (this.info.estate.hasBuff(BuffType.EXPEDITORS_SPEED_STOP))
        {
            var buff = FBuff.findFirst(this.info.estate.buffs, BuffType.EXPEDITORS_SPEED_STOP, null, Tick.now());
            if (buff && !CastCause.PERSONAL_EVENT.equals(buff.cast.cause))
            {
                this.setNotification(I18.f("EXPEDITORS_SPEED_STOP_OTHER", I18.getBuffCause(buff)));
            }
            else
            {
                this.setNotification(I18.getMegaString("EXPEDITORS_SPEED_STOP"));
            }
            //COM(this.parentDialog.apply).enabled = false;
        }
        else if (!this.parentDialog.target.getLocationXYZ().isInRadiusFrom(this.info.estate.location, FUser.CURRENT_USER.getScienceValue(Science.CARTOGRAPHY)))
        {
            this.setNotification(I18.f("MISSION_EXPEDITION_IS_TO_FAR"));
            //COM(this.parentDialog.apply).enabled = false;
        }
        else if (this.parentDialog.isMissionsLimitReached())
        {
            this.setNotification(I18.f("MISSIONS_LIMIT_REACHED"));
            //COM(this.parentDialog.apply).enabled = false;
        }
        else if (FConfig.GATE_STATE_AFFECTS_EXPEDITORS && this.info.estate.isGateClosed())
        {
            this.setNotification(I18.getMissionEntranceClosedMessage(this.info.estate.isOuterEstate));
            //COM(this.parentDialog.apply).enabled = false;
        }
    }
}

MissionExpeditionDialog.prototype.getHasDeathBuffs = function()
{
    return this.estate.hasBuff(BuffType.EXPEDITOR_DEATH);
}

MissionExpeditionDialog.prototype.onArriveTimeTooltip = function(e)
{
    this.createSpeedTip(this.estate, this.missionOrder.getBaseTravelTime(),
            this.missionOrder.outcomingSpeedBuffType, this.missionOrder.incomingSpeedBuffType, BType.EXPEDITION, ServiceHelper.getGloabalServiceDecl(BType.EXPEDITION.name, 0).getValue(), this.missionOrder);
}

MissionExpeditionDialog.prototype.onReturnTimeTooltip = function(e)
{
    this.createSpeedTip(this.estate, this.missionOrder.getBaseTravelTime() * 2,
            this.missionOrder.outcomingSpeedBuffType, this.missionOrder.incomingSpeedBuffType, BType.EXPEDITION, ServiceHelper.getGloabalServiceDecl(BType.EXPEDITION.name, 0).getValue(), this.missionOrder);
}

MissionExpeditionDialog.prototype.onDeathTooltip = function(e)
{
    this.createDeathTip(this.estate, MissionSpeed.getExpeditorDeathRate(this.missionOrder.speed), BuffType.EXPEDITOR_DEATH);
}

MissionExpeditionDialog.prototype.update = function(info)
{
    superMethod(MissionExpeditionDialog.prototype, "update", this)(info);
    COM(COM(this.parentDialog.etaGridRow).boostExpeditorsIcon).update(info.estate);
}