function MissionListDialog()
{
	callParentConstructor(MissionListDialog.prototype, this);
	
	this.user = null;
    this.ticker = null;
    this.missions = null;
    this.filterId = null;
    this.nextUpdateTick = null;
    this.report = null;
    this.estateInfo = null;
    
    this.refreshMissionsList = $.proxy(this.refreshMissionsList, this);
}

extendClass(MissionListDialog.prototype, Dialog.prototype);

MissionListDialog.ITEMS_ON_PAGE = 6;
MissionListDialog.UPDATE_MIN = 1;
MissionListDialog.UPDATE_DEFAULT = 35;

MissionListDialog.instance = null;

MissionListDialog.popup = function(parent, user, filterId, onClose, result, info)
{
    if (!MissionListDialog.instance)
    {
        MissionListDialog.instance = COM.create("MissionListDialog");
    }
    MissionListDialog.instance.report = result;
    MissionListDialog.instance.estateInfo = info;
    MissionListDialog.instance.user = user;
    MissionListDialog.instance.filterId = filterId;
    MissionListDialog.instance.onClose = onClose;
    MissionListDialog.instance.popup(parent);
}

MissionListDialog.prototype.__construct = function()
{
	this.setCaption(I18.f("MISSIONS"));
	
	$(this.showOnlyEstateMissions).on('change', this.refreshMissionsList);
	$(this.showClanMatesMissionsToo).on('change', this.refreshMissionsList);
	COM.bindExclusiveClick(this.saveReport, $.proxy(this.onSaveReport, this));
	
	UI.show(false, [this.showClanMatesMissionsToo, this.saveReport]);
	
	COM(this.missionList).setHeader([I18.f("MISSION_NEAREST"), I18.f("MISSION_GOAL"), I18.f("MISSION_STATE"), I18.f("MISSION_REACH_DEST"), I18.f("MISSION_RETURN"), I18.f("MISSION_ORIGIN_CITY"), I18.f("MISSION_TARGET")]);
	
	COM(this.missionList).createRenderer = $.proxy(this.createMissionRenderer, this);
	
	this.fullScreen();
	
	COM(this.pager).onSelect = $.proxy(this.updateList, this);
}

MissionListDialog.prototype.fitLayout = function(container)
{
	superMethod(MissionListDialog.prototype, "fitLayout", this)(container);
	
	this.windowWrapper.style.width = this.content.offsetWidth + "px";
	this.windowWrapper.style.height = this.content.offsetHeight + "px";
	
	this.contentContainer.style.width = (this.content.offsetWidth - this.pagerContainer.offsetWidth) + "px";
	this.contentContainer.style.height = h;
	
	this.pagerContainer.style.height = this.content.offsetHeight + "px";
	
	var h = (this.content.offsetHeight - this.controlsBar.offsetHeight) + "px";
	this.missionListWrapper.style.height = h;
	this.missionList.style.height = h;
}

MissionListDialog.prototype.onShow = function()
{
	setTimeout($.proxy(this.init, this), 100);
}

MissionListDialog.prototype.init = function()
{
    //this.missionList.style.display = "none";
    
    /*
    for each(var r:MissionDetails in missionList.renderers) {
        r.user = user;
        r.estateId = report != null ? report.target.id : null;
    }
    */

    COM(this.showOnlyEstateMissions).checked = this.report  ? true : false;
    COM(this.showClanMatesMissionsToo).checked = this.user.isCurrent() && FUser.alertOnClanmateAttack;
    UI.show(this.user.isCurrent() && this.user.hasClanRole(ClanRole.WARMINISTER), [this.showClanMatesMissionsToo]);
    this.refreshMissionsList();
}

MissionListDialog.prototype.refreshMissionsList = function()
{
	if (this.user.isCurrent())
    {
        FUser.alertOnClanmateAttack = COM(this.showClanMatesMissionsToo).checked;
    }
    if (!this.report)
    {
        APP.showLoader();
        Service.mission.callMethod("getMissions", [this.user.username, this.getMissionFilter()], $.proxy(this.update, this), $.proxy(this.onError, this));
    }
    else
    {
        this.update(this.report.missions);
    }
}

MissionListDialog.prototype.onError = function(e)
{
    if (e.__className__ == "AccessDeniedException")
    {
        this.close();
    }
    return false;
}

MissionListDialog.prototype.close = function()
{
    clearInterval(this.ticker);
    
    /*
    if (this.report && !this.report.saved)
    {
        Service.mail.callMethod("discardHiddenObservationReport");
    }
    this.missionList.style.display = "";
    this.missions = [];
    this.missionList.dataProvider = missions;
    this.nextUpdateTick = Tick.TICK_INFINITY;
    this.report = null;
    */
    
    MissionListDialog.instance = null;
    
    superMethod(MissionListDialog.prototype, "close", this)();
}

MissionListDialog.prototype.tick = function(e)
{
    var list = COM(this.missionList);
    var child;
    for(var i=0; i<list.items.length; i++)
    {
    	child = COM(list.items[i]);
    	child.mission.updateTransient();
        child.tick();
    }
    
    if (Tick.hasPassed(this.nextUpdateTick))
    {
        this.refreshMissionsList();
    }
}

MissionListDialog.sort = function(missions)
{
    if($.isArray(missions))
    {
    	Arrays.sort(missions);
    	return missions;
    }
    
    var ret = [];
    
    for(var i in missions)
    {
    	ret.push(missions[i]);
    }
    
    Arrays.sort(ret)
    
    return ret;
}

MissionListDialog.prototype.updateList = function()
{
	var currentPage = COM(this.pager).page;
	var start = currentPage*MissionListDialog.ITEMS_ON_PAGE;
    var ml = this.missions.slice(start, start+MissionListDialog.ITEMS_ON_PAGE);
    COM(this.missionList).dataProvider = ml;
    this.tick();
}

MissionListDialog.missionDetailsDialog = null;

MissionListDialog.prototype.update = function(missions)
{
    if(missions && missions.__isMapped__)
    {
     	delete(missions.__isMapped__);
    }
    
    var selected = this.missionList.selected;
    var selectedId = selected ? selected.mission.id : -1;
    missions = MissionListDialog.sort(missions);
    
    if(MissionListDialog.missionDetailsDialog)
    {
    	var current = null;
    	
    	for(var i=0; i<missions.length; i++)
    	{
    		if(missions[i].id == MissionListDialog.missionDetailsDialog.mission.id)
    		{
    			current = missions[i];
    			break;
    		}
    	}
    	
    	if(!current)
    	{
    		MissionListDialog.missionDetailsDialog.close();
    		MissionListDialog.missionDetailsDialog = null;
    	}
    	else
    	{
    		current.updateTransient();
			MissionListDialog.missionDetailsDialog.update(current);
    	}
    }
    
    this.missions = missions;
    
    var isReportView = this.report ? true : false;
    COM(this.saveReport).enabled = isReportView && !this.report.saved;
    this.saveReport.style.display = isReportView ? "" : "none";
    
    COM(this.showOnlyEstateMissions).disabled = isReportView;
    COM(this.showClanMatesMissionsToo).disabled = isReportView;

    //this.missionsLimit.style.display = !isReportView ? "" : "none";
    //this.missionsLimit.innerHTML = I18.f("TOTAL_MISSIONS_X_FROM_Y", [this.getOwnMissionsCount(), this.user.missionsLimit]);

    this.nextUpdateTick = !isReportView ? this.getNextUpdateTick(missions) : Tick.TICK_INFINITY;
    
    this.missionList.style.display = "";
    
    var cp = COM(this.pager).page;
    var max = Math.ceil(this.missions.length/MissionListDialog.ITEMS_ON_PAGE);
    if(cp >= max) cp = 0;
    COM(this.pager).setIndexAndCount(cp, max);
    
    this.updateList();
	
	if (!this.ticker)
    {
        this.ticker = setInterval($.proxy(this.tick, this), 1000);
    }
	
    this.tick();
    
    COM(this.missionList).refreshHeader();
    
    APP.hideLoader();
}

MissionListDialog.prototype.onCalculate = function(mission)
{
    if (mission.groupId == -1)
    {
        CombatCalculatorView.popup(this.estateInfo, [mission]);
    }
    else
    {
        CombatCalculatorView.popup(this.estateInfo, Arrays.filter(this.missions, function (m)
        {
            return m.groupId == mission.groupId;
        }));
    }
}

MissionListDialog.showObservationResult = function(result)
{
	if (result.isSuccess)
    {
        MissionListDialog.popup(null, result.targetOwner, result.target.id, null, result);
    }
}

MissionListDialog.prototype.getNextUpdateTick = function(missions)
{
    var now = Tick.now();
    var tick = now + MissionListDialog.UPDATE_DEFAULT;
    var m;
    
    for(var i=0; i<missions.length; i++)
    {
    	m = missions[i];
    	if (m.nextUpdateTick > 0)
    	{
            if (m.nextUpdateTick < tick)
            {
                tick = m.nextUpdateTick;
            }
        }
        if (m.cancelTick > now - 1)
        {
            if (m.cancelTick < tick)
            {
                tick = m.cancelTick;
            }
        }
    }
    
    return Math.max(tick, now) + MissionListDialog.UPDATE_MIN;
}

MissionListDialog.prototype.getOwnMissionsCount = function()
{
    var cnt = 0;
    for(var i in this.missions)
    {
    	if(this.missions[i].owner && this.missions[i].owner.username == FUser.CURRENT_USER.username)
    	{
    		cnt++;
    	}
    }
    return cnt;
}

MissionListDialog.prototype.createMissionRenderer = function()
{
    var renderer = COM.create("MissionDetails");
    renderer.snapshotTick = this.report ? this.report.tick : -1;
    renderer.user = this.user;
    renderer.calculateHandler = $.proxy(this.onCalculate, this);
    renderer.cancelHandler = $.proxy(this.cancelMission, this);
    
    return renderer;
}

MissionListDialog.prototype.getMissionFilter = function()
{
    var f = new MissionFilter();
    f.estateId = COM(this.showOnlyEstateMissions).checked ? this.filterId : null;
    f.filterClanViewer = !COM(this.showClanMatesMissionsToo).checked;
    return f;
}

MissionListDialog.prototype.cancelMission = function(m)
{
    Service.mission.callMethod("cancelMission", [this.user.username, m.id, this.getMissionFilter()],
            $.proxy(function (missions)
            {
                Events.onMissionCancel.fire(m);
                this.update(missions);
            }, this),
            $.proxy(function (e)
            {
                if (e.__className__ == "MissionCancelException")
                {
                    Alert.create(I18.f("CANNOT_CANCEL_MISSION"));
                    return true;
                }
                return false;
            }, this)
    );
}

MissionListDialog.prototype.onSaveReport = function()
{
    Service.mail.callMethod("saveHiddenObservationReport", [], $.proxy(function()
    {
        Alert.create(I18.f("SAVE_REPORT_SUCCESS"));
        COM(this.saveReport).enabled = false;
        this.report.saved = true;
    }, this));
}