function MissionMoveResourcesDialog()
{
	callParentConstructor(MissionMoveResourcesDialog.prototype, this);
	
	this.requiredMerchants = 0;
    this.merchantCapacity = 0;
}

extendClass(MissionMoveResourcesDialog.prototype, MissionDialogChild.prototype);

MissionMoveResourcesDialog.prototype.__construct = function()
{
	COM(this.resourceChooser).onChange = $.proxy(this.onResourcesChange, this);
}

MissionMoveResourcesDialog.prototype.setupParentComponentsVisibility = function()
{
    var mdp = this.parentDialog;
    UI.show(true, [mdp.capacityBox, mdp.capacity, mdp.specialUnitImage, mdp.missionCostBox, mdp.deathBox, mdp.moveResourcesSpacer]);
    
    mdp.capacity.innerHTML = I18.f("M_FROM_N", [0, this.requiredMerchants * this.merchantCapacity]);
    //this.parentDialog.capacity.toolTip = hasCapacityBuffs ? " " : "";
    UI.show(true, [this.resourceChooser]);
    this.showTravelMissionsComponents();
    UI.show(true, [COM(this.parentDialog.etaGridRow).boostMerchantsIcon]);
}

MissionMoveResourcesDialog.prototype.setupParentComponentsProperties = function()
{
    COM(this.resourceChooser).update(this.info.estate);
    
    this.merchantCapacity = FConfig.getMerchantCapacity(this.info.estate) * (1 + this.info.estate.sumOfBuffs(BuffType.MERCHANTS_CAPACITY));
    this.requiredMerchants = 0;

    //this.parentDialog.goal.innerHTML = I18.f("MISSION_GOAL_MOVERESOURCES");
	this.parentDialog.setCaption(I18.f("MISSION_GOAL_MOVERESOURCES"));
    this.parentDialog.costLabel.innerHTML = I18.f("MOVE_RESOURCES_TAX");

    var su = COM(this.parentDialog.specialUnit);
    su.utype = UnitType.MERCHANT;
    su.showCount = true;
    su.showMaximum = true;
    su.showZero = true;
    su.grayscaleZero = false;
}

MissionMoveResourcesDialog.prototype.reset = function()
{
    superMethod(MissionMoveResourcesDialog.prototype, "reset", this)();
    COM(this.resourceChooser).reset();
}

MissionMoveResourcesDialog.prototype.createMissionOrder = function()
{
    this.missionOrder = new MissionMoveResourcesOrder();
}

MissionMoveResourcesDialog.prototype.tick = function()
{
    superMethod(MissionMoveResourcesDialog.prototype, "tick", this)();

    var deathRate = NumberUtils.ceil(100 * NumberUtils.clamp01(FConfig.getMerchantDeathRate(this.info.estate.owner.race) + this.info.estate.sumOfBuffs(BuffType.MERCHANT_DEATH)));
    this.parentDialog.deathRate.innerHTML = Strings.formatPercent(deathRate);

    var estate = this.parentDialog.prepareInfo.estate;
    var limit = estate.bank.copy().floor();
    limit.multiply(1.0 / (1.0 + this.missionOrder.getCurrentTax()));
    limit.floor();
    COM(this.resourceChooser).tick(limit);
    var freeMerchants = estate.countFreeSpecialUnits(UnitType.MERCHANT);
    COM(this.parentDialog.specialUnit).update(0, this.requiredMerchants, freeMerchants);
    //COM(this.parentDialog.specialUnit).unitCount.setStyle("color", Effects.WHITE);
    
    this.checkMissionCommonConstraints(estate);
    this.checkConstarints(estate);
}

MissionMoveResourcesDialog.prototype.onArriveTimeTooltip = function(e)
{
    this.createSpeedTip(this.estate, this.missionOrder.getBaseTravelTime(), this.missionOrder.outcomingSpeedBuffType,
            this.missionOrder.incomingSpeedBuffType, BType.MERCHANT,
            ServiceHelper.getGloabalServiceDecl(BType.MERCHANT.name, 0).getValue(), this.missionOrder);
}

MissionMoveResourcesDialog.prototype.onReturnTimeTooltip = function(e)
{
    this.createSpeedTip(this.estate, this.missionOrder.getBaseTravelTime() * 2, this.missionOrder.outcomingSpeedBuffType, this.missionOrder.incomingSpeedBuffType, BType.MERCHANT, ServiceHelper.getGloabalServiceDecl(BType.MERCHANT.name, 0).getValue(), this.missionOrder);
}

MissionMoveResourcesDialog.prototype.update = function(info)
{
    superMethod(MissionMoveResourcesDialog.prototype, "update", this)(info);
    
    COM(this.resourceChooser).update(this.info.estate);
    COM(COM(this.parentDialog.etaGridRow).boostMerchantsIcon).update(this.info.estate);
    COM(this.parentDialog.etaGridRow).showReturnTime = false;
    this.merchantCapacity = FConfig.getMerchantCapacity(info.estate) * (1.0 + info.estate.sumOfBuffs(BuffType.MERCHANTS_CAPACITY));
    
    this.updateRequiredMerchants();
}

MissionMoveResourcesDialog.prototype.updateRequiredMerchants = function()
{
    var totalCount = this.missionOrder.res.sum();
    this.requiredMerchants = NumberUtils.ceil(totalCount / this.merchantCapacity);
    this.parentDialog.capacity.innerHTML = I18.f("M_FROM_N", [totalCount, this.requiredMerchants * this.merchantCapacity]);
    if (this.requiredMerchants != 0 && this.hasCapacityBuffs)
    {
        //Style.assignTip(parentDialog.capacity, " ");
    }
    else
    {
        //Style.removeTip(parentDialog.capacity);
	}
	
    //parentDialog.capacity.toolTip = hasCapacityBuffs ? " " : "";
    this.parentDialog.tick();
}

MissionMoveResourcesDialog.prototype.onResourcesChange = function(f)
{
    this.missionOrder.res = f.copy().floor();
    this.updateRequiredMerchants();
}

MissionMoveResourcesDialog.prototype.onDeathTooltip = function(e)
{
    this.createDeathTip(this.estate, FConfig.getMerchantDeathRate(this.estate.owner.race), BuffType.MERCHANT_DEATH);
}

MissionMoveResourcesDialog.prototype.getHasDeathBuffs = function()
{
    return this.estate.hasBuff(BuffType.MERCHANT_DEATH);
}

MissionMoveResourcesDialog.prototype.getHasCapacityBuffs = function()
{
    return this.estate.hasBuff(BuffType.MERCHANTS_CAPACITY);
}

MissionMoveResourcesDialog.prototype.onCapacityTooltip = function(e)
{
     var tip = NumberEffectTip.createTip("BASE_MERCHANTS_CAPACITY", this.estate.buffs,
                this.requiredMerchants * FConfig.getMerchantCapacity(this.info.estate), BuffType.MERCHANTS_CAPACITY,
                FEstate.tickNowForEstate(this.estate));
	TipDialog.create(tip);
}

MissionMoveResourcesDialog.prototype.onResult = function(mission)
{
    superMethod(MissionMoveResourcesDialog.prototype, "onResult", this)(mission);
    
    if (COM(this.resourceChooser).stepIndex != FUser.CURRENT_USER.settings.resourceChooserStepIndex)
    {
        Events.onResourceChooserStepIndexChanged.fire(COM(this.resourceChooser).stepIndex);
    }
}

MissionMoveResourcesDialog.prototype.checkConstarints = function(estate)
{
    var freeMerchants = this.estate.countFreeSpecialUnits(UnitType.MERCHANT);
    if (this.estate.isGateClosed())
    {
        this.setNotification(I18.getMissionEntranceClosedMessage(this.estate.isOuterEstate));
        return false;
    }
    else if (this.requiredMerchants > freeMerchants || freeMerchants <= 0)
    {
        this.setNotification(I18.f("MISSION_NOT_ENOUGH_FREE_MERCHANTS"));
        //COM(COM(this.parentDialog.specialUnit).unitCount).setStyle("color", Effects.RED);
        return false;
    }
    else if (this.parentDialog.isMissionsLimitReached())
    {
        this.setNotification(I18.f("MISSIONS_LIMIT_REACHED"));
        return false;
    }
    else if (!this.parentDialog.isEnoughCost())
    {
        this.setNotification(I18.f("MISSION_NOT_ENOUGH_RESOURCES"));
        return false;
    }
    else if (this.missionOrder.res.isZero())
    {
        this.setNotification(I18.f("MISSION_NO_RESOURCES_SELECTED"));
        return false;
    }
    return true;
}