function ResourceChooser()
{
	this.race = Race.DEMON;
    this.total = new Funds();

	this.stepIndex = -1;
	this.multipliers = [1, 100, 3000];

	this.onChange = function(){};
	this.onMove = $.proxy(this.onMove, this);
	
	this.__defineGetter__("changed", this.getChanged);
	
	this.selectAllDirect = 1;
	
	this.onSelectResource = $.proxy(this.onSelectResource, this);
}

ResourceChooser.prototype.__construct = function()
{
	this.allSliders = [this.sliderGold, this.sliderWood, this.sliderStone, this.sliderGrain, this.sliderIron, this.sliderPeople];
	
	COM(this.sliderGold).data = Resource.GOLD;
	COM(this.sliderWood).data = Resource.WOOD;
	COM(this.sliderStone).data = Resource.STONE;
	COM(this.sliderGrain).data = Resource.GRAIN;
	COM(this.sliderIron).data = Resource.IRON;
	COM(this.sliderPeople).data = Resource.PEOPLE;
	
	for(var i=0; i<this.allSliders.length; i++)
	{
		COM(this.allSliders[i]).onChange = this.onMove;
	}
	
	COM.bindExclusiveClick(this.changeMultiplierButton, $.proxy(this.onChangeMult, this));
	
	COM.bindExclusiveClick(this.selectAllButton, $.proxy(this.onSelectAllNone, this));

	COM(this.availableView).showIconBacks(true);
	COM(this.selectedView).showIconBacks(true);
	
	var onSelectResource = this.onSelectResource;
	$("iconandvalue", this.availableView).each(function(ix, el)
	{
		//COM.bindExclusiveClick(el, onSelectResource);
		$(el).on(CLICK_EVENT, onSelectResource);
	});
	$("iconandvalue", this.selectedView).each(function(ix, el)
	{
		//COM.bindExclusiveClick(el, this.onSelectResource)
		$(el).on(CLICK_EVENT, onSelectResource);
	});
	$("cell", this.arrowsRow).each(function(ix, el)
	{
		//COM.bindExclusiveClick(el, this.onSelectResource)
		$(el).on(CLICK_EVENT, onSelectResource);
	});
	
	//COM(this.selectAllButton).caption = I18.f("MOVE_ALL");
	
	this.updateSelectAllButton();
}

ResourceChooser.prototype.updateSelectAllButton = function()
{
	if(this.selectAllDirect > 0)
	{
		COM(this.selectAllButton).icon = STATIC_INTERFACE_URL + "icons/down_arrow.png";
	}
	else
	{
		COM(this.selectAllButton).icon = STATIC_INTERFACE_URL + "icons/up_arrow.png";
	}
}

ResourceChooser.prototype.onSelectAllNone = function()
{
	if(this.selectAllDirect > 0)
	{
		this.onSelectAll();
	}
	else
	{
		this.onSelectNone();
	}
	
	this.selectAllDirect *= -1;
	this.updateSelectAllButton();
}

ResourceChooser.prototype.onSelectResource = function(e)
{
	if (e && e.currentTarget)
	{
		var r = Resource[$(e.currentTarget).attr("name").toUpperCase()];
		if (r)
		{
			var to = COM(this.selectedView).funds.getValue(r), from = this.total.getValue(r) - to;
			var d = SliderDialog.create(r, from, to, $.proxy(this.onMove, this), FUser.CURRENT_USER.settings.resourceChooserStepIndex, this.multipliers, $.proxy(this.onChangeMultiplier, this));
		}
	}
}

ResourceChooser.prototype.onChangeMultiplier = function(newIndex)
{
	Events.onResourceChooserStepIndexChanged.fire(newIndex);
}

ResourceChooser.prototype.update = function(estate, styleName)
{
    if(typeof styleName == "undefined")
    {
    	styleName = "tip";
    }
    
    COM(this.availableView).styleName = styleName;
    COM(this.selectedView).styleName = styleName;
    var sumOfBuffs = (!estate ? 0 : estate.sumOfBuffs(BuffType.MERCHANTS_CAPACITY));
    
    this.multipliers[2] = Math.floor(FConfig.getMerchantCapacity(estate) * (1 + sumOfBuffs));
    if (this.stepIndex < 0)
    {
        this.reset();
    }
}

ResourceChooser.prototype.reset = function()
{
    this.total = new Funds();
    COM(this.selectedView).update(new Funds(), this.race);
    COM(this.availableView).update(new Funds(), this.race);
    this.stepIndex = FUser.CURRENT_USER.settings.resourceChooserStepIndex;
    this.setMult();
    this.race = FUser.CURRENT_USER.race;
}

ResourceChooser.prototype.tick = function(available)
{
    this.total = available.copy();
    if (!this.changed)
    {
        COM(this.selectedView).update(new Funds(), this.race);
        COM(this.availableView).update(this.total.copy(), this.race);
    }
    else
    {
        var oldSelected = COM(this.selectedView).funds.copy();
        COM(this.selectedView).update(COM(this.selectedView).funds.limitByCopy(this.total), this.race);
        COM(this.availableView).update(this.total.substractCopy(COM(this.selectedView).funds), this.race);
        if (!COM(this.selectedView).funds.equals(oldSelected))
        {
            this.onChange(COM(this.selectedView).funds);
        }
    }
    
    var r;
    for(var i=0; i<Resource.FUNDS_RESOURCES.length; i++)
    {
    	r = Resource.FUNDS_RESOURCES[i];
    	var s = this.sliderByResource(r);
        s.sliderEnabled = this.total.getValue(r) > 0;
        s.maximum = this.total.getValue(r);
        s.value = COM(this.selectedView).funds.getValue(r);
    }
}

ResourceChooser.prototype.setMult = function()
{
	this.changeMultiplierButton.innerHTML = this.multipliers[this.stepIndex];
	
	for(var i=0; i<this.allSliders.length; i++)
	{
		COM(this.allSliders[i]).multiplier = this.multipliers[this.stepIndex];
	}
}

ResourceChooser.prototype.onChangeMult = function()
{
    this.stepIndex = (this.stepIndex + 1) % this.multipliers.length;
    this.setMult();
}

ResourceChooser.prototype.sliderByResource = function(r)
{
    var c;
    for(var i=0; i<this.allSliders.length; i++)
    {
    	c = COM(this.allSliders[i]);
    	if(r.equals(c.data))
    	{
    		return c;
    	}
    }
    
    throw Error("Could not find Slider for resource: " + r.name);
}

ResourceChooser.prototype.getChanged = function()
{
    return !COM(this.selectedView).funds.isZero();
}

/*
ResourceChooser.prototype.onMove = function(slider, value)
{
    this.select(COM(this.selectedView).funds.copy().setValue(slider.data, value).limitByCopy(this.total));
}
*/

ResourceChooser.prototype.onMove = function(resource, value)
{
    this.select(COM(this.selectedView).funds.copy().setValue(resource, value).limitByCopy(this.total));
}

ResourceChooser.prototype.onSelectAll = function()
{
    this.select(this.total.copy());
}

ResourceChooser.prototype.onSelectNone = function()
{
    this.select(new Funds());
}

ResourceChooser.prototype.select = function(funds)
{
    var selected = funds.limitByCopy(this.total);
    COM(this.selectedView).update(selected, this.race);
    COM(this.availableView).update(this.total.substractCopy(selected), this.race);
    this.onChange(selected);
}