function FMission()
{
	this.id = null;
    this.owner = null;

    this.from = null;
    this.to = null;
    this.startTick = null;
    this.arriveTick = null;
    this.returnTick = null;
    this.nextUpdateTick = null;
    this.cancelTick = null;

    this.army = null;
    this.armyFaded = null;
    this.fromEstateBonuses = null;
    this.toEstateBonuses = null;
    this.ownerUnitLevels = null;
    this.victimUsername = null;

    this.res = null;
    this.specialUnit = null;
    this.unitCount = null;
    this.state = null;
    this.result = null;
    this.className = null;
    this.isOwner = null;
    this.isDeputy = null;
    this.userCanCancel = null;
    this.ownerCanCancel = null;
    this.isAttack = null;
    this.isAssist = null;
    this.isAncientsAssist = null;
    this.isMilitary = null;
    this.isPhantom = null;
    this.isConstantDuration = null;

    this.groupId = null;
    this.groupInitiator = null;
    this.groupInitiatorAttackIndex = null;
    this.groupMultipleInvited = null;
    this.groupMissionsCount = null;

    this.sortId = null;

    this.resource = null;

    this.cancelable = null;
    this.calculable = null;

    this.hero = null;
    this.nonHeroCasts = null;
    this.armyBuffs = null;
    this.speedupBuffs = null;
    this.speedupAncientMagic = null;
    
    this.__defineGetter__("origin", this.getOrigin);
    this.__defineGetter__("target", this.getTarget);
    this.__defineGetter__("isGroupMission", this.getIsGroupMission);
    this.__defineGetter__("armyInBattle", this.getArmyInBattle);
}

FMission.prototype.__CLS__ = "game.service.mission.FMission";

FMission.prototype.updateTransient = function()
{
    this.cancelable = this.isCancelable();
    this.calculable = this.isCalculable();
}

FMission.prototype.isCalculable = function()
{
    if (!this.isAttack)
    {
        return false;
    }
    if (this.state.name == MissionState.RETURNING.name)
    {
        return false;
    }
    if (FUser.isCurrentUser(this.victimUsername))
    {
        return true;
    }
    if (this.to.estateType && (this.to.estateType.name == EstateType.CASTLE.name || this.to.estateType.name == EstateType.CATAPULT.name) && FUser.CURRENT_USER.isClanMate(this.victimUsername))
    {
        return true;
    }
    return false;
}

FMission.prototype.isCancelable = function()
{
    if (!this.userCanCancel)
    {
        return false;
    }
    if (Tick.hasPassed(this.cancelTick))
    {
        return false;
    }
    return true;
}

FMission.prototype.isSpecial = function()
{
    return this.specialUnit ? true : false;
}

FMission.prototype.compareTo = function(other)
{
    if (this.nextUpdateTick != other.nextUpdateTick)
    {
        return this.nextUpdateTick - other.nextUpdateTick;
    }
    if (this.sortId != other.sortId)
    {
        return this.sortId - other.sortId;
    }
    
    return this.id - other.id;
}

FMission.prototype.getOrigin = function()
{
    return this.state.name == MissionState.RETURNING.name ? this.to : this.from;
}

FMission.prototype.getTarget = function()
{
    return this.state.name == MissionState.RETURNING.name ? this.from : this.to;
}

FMission.prototype.getIsGroupMission = function()
{
    return this.isAttack && this.groupId != -1 && (this.groupMultipleInvited || this.groupMissionsCount > 1);
}

FMission.prototype.getArmyInBattle = function()
{
    var a = new ArmyInBattle();
    a.army = this.army.toArray();
    a.levels = this.ownerUnitLevels;
    a.race = this.owner.race;
    a.betrayer = this.owner.isBetrayer();
    a.retreatLimit = 100;
    a.username = this.owner.username;
    a.titleIndex = this.owner.titleIndex;
    a.armyFaded = this.armyFaded;
    a.hero = this.hero;
    a.nonHeroCasts = this.nonHeroCasts;
    return a;
}

FMission.prototype.getBonus = function(ut)
{
    for (var i=0, im = UnitType.MILITARY.length; i < im; i++)
    {
        if (UnitType.MILITARY[i].name == ut.name)
        {
            if (this.state.name == MissionState.RETURNING.name && this.fromEstateBonuses)
            {
                return this.fromEstateBonuses[i];
            }
            else if (this.toEstateBonuses)
            {
                return this.toEstateBonuses[i];
            }
        }
    }
    
    return new AuraBonus();
}

FMission.prototype.hasSpeedBuffs = function()
{
    return this.isConstantDuration || this.speedupBuffs.length > 0 || this.speedupAncientMagic;
}

FMission.prototype.getStateIcon = function()
{
    var key = "MISSION_" + this.state.name;
    if (this.result.name != MissionResult.UNKNOWN.name)
    {
        key += "_" + result.name;
    }
    return SocNetworkAssets[key];
}