function MilitaryMissionOrder()
{
	callParentConstructor(MilitaryMissionOrder.prototype, this);
	
	this.army = new Army();
    this.heroId = -1;
    this.retreatLimit = 0;
    
    this.IS_MILITARY_MISSION_ORDER = true;
}

extendClass(MilitaryMissionOrder.prototype, MissionOrder.prototype);

MilitaryMissionOrder.prototype.__CLS__ = "game.model.mission.MilitaryMissionOrder";

MilitaryMissionOrder.prototype.calculateCost = function()
{
    return Funds.createWithGold(NumberUtils.ceil(this.getCostGold()));
}

MilitaryMissionOrder.prototype.getCostGold = function()
{
    return Math.max(0, (this.getBaseTravelCost() + this.getBaseExecutionCost()) * (1 + this.getFromEstate().sumOfBuffs(BuffType.ARMY_RATION)));
}

MilitaryMissionOrder.prototype.getConsumptionPerTick = function()
{
    return this.getFromEstate().computeConsumptionSpeedPerTick(this.army);
}

MilitaryMissionOrder.prototype.getDefaultCost = function()
{
    var consumptionPerTick = this.getConsumptionPerTick();
    var travelDuration = this.getMissionDuration() - this.getExecuteDuration();
    var travelCost = travelDuration * consumptionPerTick * MissionSpeed.getConsumption(this.speed);
    var executeCost = this.getExecuteDuration() * consumptionPerTick;

    return travelCost + executeCost;
}

MilitaryMissionOrder.prototype.getBaseTravelCost = function()
{
    return (this.getMissionDuration() - this.getExecuteDuration()) * this.getConsumptionPerTick() * MissionSpeed.getConsumption(this.speed);
}

MilitaryMissionOrder.prototype.getBaseExecutionCost = function()
{
    return this.getExecuteDuration() * this.getConsumptionPerTick();
}

MilitaryMissionOrder.prototype.getBaseCost = function()
{
    return this.getBaseTravelCost() + this.getBaseExecutionCost();
}

MilitaryMissionOrder.prototype.getTravelDurationMultiplier = function()
{
    var speedup = 1;
    if (this.getFromEstate().buffs)
    {
        if (this.heroId != -1)
        {
            speedup += FBuff.sumOfTerrainBuffs(this.getFromEstate().buffs, BuffType.HERO_MISSION_TIME, this.getFromEstate(), this.getToField());
        }
        speedup += FBuff.sumOfTerrainBuffs(this.getFromEstate().buffs, BuffType.MILITARY_MISSION_TIME, this.getFromEstate(), this.getToField());
    }
    return speedup;
}

MilitaryMissionOrder.prototype.getHasSpeedBuffs = function()
{
    if (this.heroId != -1 && this.getFromEstate().hasBuff(BuffType.HERO_MISSION_TIME))
    {
        return true;
    }
    if (this.getFromEstate().hasBuff(BuffType.MILITARY_MISSION_TIME))
    {
        return true;
    }
    return false;
}

MilitaryMissionOrder.prototype.getOutcomingSpeedBuffType = function()
{
    return BuffType.MILITARY_MISSION_TIME;
}