function MissionOrder()
{
	this.fromEstate = null;
    
    this.fromEstateXYZ = null;
    this.toFieldXYZ = null;
    
    this.toField = null;

    this.startTick = null;
    this.arriveTick = null;
    this.clientDuration = null;
    this.returnTick = null;
    this.speed = 3;
    this.allyAcceleration = null;
    
    this.IS_MILITARY_MISSION_ORDER = false;
    
    this.__defineGetter__("hasSpeedBuffs", this.getHasSpeedBuffs);
    this.__defineGetter__("incomingSpeedBuffType", this.getIncomingSpeedBuffType);
    this.__defineGetter__("outcomingSpeedBuffType", this.getOutcomingSpeedBuffType);
}

MissionOrder.prototype.__CLS__ = "game.model.mission.MissionOrder";

MissionOrder.prototype.prepareForRequest = function()
{
	var ret = {};
	for(var i in this)
	{
		/*
		if(i != "fromEstate" && i != "toField" && i != "superClass")
		{
			if($.isPlainObject(this[i]) || typeof this[i] != "function")
			{
				ret[i] = cloneRequestParam(this[i]);
			}
		}
		*/
		
		if(i != "fromEstate" && i != "toField" && i != "IS_MILITARY_MISSION_ORDER")
		{
			ret[i] = (this[i] && this[i].prepareForRequest) ? this[i].prepareForRequest() : this[i];
		}
	}
	
	return ret;
}

MissionOrder.prototype.setFromEstate = function(e)
{
	this.fromEstate = e;
    this.fromEstateXYZ = e.location;
}

MissionOrder.prototype.getFromEstate = function()
{
    return this.fromEstate;
}

MissionOrder.prototype.setToField = function(field)
{
    this.toField = field;
}

MissionOrder.prototype.getToField = function()
{
    return this.toField;
}

MissionOrder.prototype.calculateCost = function()
{
}

MissionOrder.prototype.getExecuteDuration = function()
{
    return 0;
}

MissionOrder.prototype.updateTicks = function(group)
{
    this.startTick = Tick.now();
    var oldDuration = this.clientDuration;
    this.refreshTravelDuration();
    
    if (oldDuration && this.clientDuration != oldDuration)
    {
        Alert.create(I18.f("DURATION_HAS_CHANGED"));
    }
    this.arriveTick = this.startTick + this.clientDuration;
    this.returnTick = this.arriveTick + this.clientDuration + this.getExecuteDuration();
}

MissionOrder.prototype.getMissionDuration = function()
{
    return this.returnTick - this.startTick;
}

MissionOrder.prototype.getTravelDuration = function()
{
    return MissionOrder.getTravelDuration(this.fromEstate.location, this.toField.getLocationXYZ(), this.speed, this.allyAcceleration, this.getTravelDurationMultiplier());
}

MissionOrder.prototype.refreshTravelDuration = function()
{
    this.clientDuration = this.getTravelDuration();
}

MissionOrder.prototype.getBaseTravelTime = function()
{
    return MissionOrder.getTravelDuration(this.fromEstate.location, this.toField.getLocationXYZ(), this.speed, this.allyAcceleration, 1);
}

MissionOrder.prototype.getTravelDurationMultiplier = function()
{
    return 1;
}

MissionOrder.prototype.isTravelDurationOverbusted = function()
{
    return MissionOrder.isTravelDurationOverbusted(this.fromEstate.location, this.toField.getLocationXYZ(), this.speed, this.getTravelDurationMultiplier());
}

MissionOrder.isAcceleratedByOwnedField = function(toField, currentUser)
{
    var isOwner = toField.isOwnedByUser(currentUser);
    var isAlly = false;
    var isClanmate = false;
    if (!isOwner && toField.hasEstate() && toField.estate.hasOwner())
    {
        isAlly = currentUser.isAlly(toField.estate.owner.username);
        isClanmate = currentUser.isClanMate(toField.estate.owner.username);
    }
    return isOwner || isAlly || isClanmate;
}

MissionOrder.getTravelDuration = function(fromXYZ, toXYZ, speed, isOwnEstate, speedup)
{
    var duration = FConfig.BASE_TRAVEL_TIME;
    if (fromXYZ.x == toXYZ.x && fromXYZ.y == toXYZ.y)
    {
        duration += FConfig.TRAVEL_TIME_PER_FIELD *
                    FKingdom.getFieldLocationByZ(fromXYZ.z).distanceTo(FKingdom.getFieldLocationByZ(toXYZ.z));
    }
    else
    {
        duration += FConfig.TRAVEL_TIME_PER_KINGDOM * fromXYZ.distanceTo(toXYZ);
    }
    duration = Math.max(FConfig.TRAVEL_TIME_MIN, duration * Math.max(FConfig.TRAVEL_TIME_MIN_RATE, speedup) / MissionSpeed.getRatio(speed));

    if (isOwnEstate)
    {
        duration *= (100 - FConfig.MISSION_ACCELERATION_TO_OWN_FIELD) / 100;
    }
    
    return NumberUtils.ceil(duration);
}

MissionOrder.isTravelDurationOverbusted = function(fromXYZ, toXYZ, speed, speedup)
{
    if (NumberUtils.compareTo(speedup, FConfig.TRAVEL_TIME_MIN_RATE) < 0)
    {
        return true;
    }
    var duration = FConfig.BASE_TRAVEL_TIME;
    if (fromXYZ.x == toXYZ.x && fromXYZ.y == toXYZ.y)
    {
        duration += FConfig.TRAVEL_TIME_PER_FIELD *
                    FKingdom.getFieldLocationByZ(fromXYZ.z).distanceTo(FKingdom.getFieldLocationByZ(toXYZ.z));
    }
    else
    {
        duration += FConfig.TRAVEL_TIME_PER_KINGDOM * fromXYZ.distanceTo(toXYZ);
    }
    return NumberUtils.compareTo(duration * speedup / MissionSpeed.getRatio(speed), FConfig.TRAVEL_TIME_MIN) < 0;
}

MissionOrder.prototype.getPreferedSpeed = function()
{
    return 3;
}

MissionOrder.prototype.getHasSpeedBuffs = function()
{
    return false;
}

MissionOrder.prototype.getIncomingSpeedBuffType = function()
{
    return null;
}

MissionOrder.prototype.getOutcomingSpeedBuffType = function()
{
    return null;
}