function PaymentDialog()
{
	callParentConstructor(PaymentDialog.prototype, this);
	
	this.timer = null;
	this.onPremiumExtended = null;
	this.ownerUser = null;
	
	this.tabButtons = [];
	this.tabViews = [];
}

extendClass(PaymentDialog.prototype, Dialog.prototype);

PaymentDialog.prototype.init = function()
{
    this.startTimer();
    
    COM(this.globalFeatures).init(this.ownerUser);
}

PaymentDialog.prototype.__construct = function()
{
	superMethod(PaymentDialog.prototype, "__construct", this)();
	
	var tabs = COM(this.tabs);
	
	this.tabButtons = [];
	
	this.tabButtons.push(tabs.addItem(I18.f("PREMIUM_ACCOUNT"), 0));
	this.tabButtons.push(tabs.addItem(I18.f("GLOBAL_FEATURES"), 1));
	this.tabButtons.push(tabs.addItem(I18.f("LOCAL_FEATURES"), 2));
	
	tabs.onChange = $.proxy(this.changeTab, this);
	
	this.tabViews = [COM(this.premiumAccount), COM(this.globalFeatures), COM(this.localFeatures)];
	
	this.setCaption(I18.f("CHEST"));
}

PaymentDialog.prototype.__destruct = function()
{
	superMethod(PaymentDialog.prototype, "__destruct", this)();
	
	this.stopTimer();
}

PaymentDialog.prototype.fitLayout = function(container)
{
	superMethod(PaymentDialog.prototype, "fitLayout", this)(container);
	
	var c = {width: this.content.offsetWidth, height: this.content.offsetHeight};
	
	this.windowWrapper.style.width = c.width + 'px';
	this.windowWrapper.style.height = c.height + 'px';
	
	this.fitChildren();
}

PaymentDialog.prototype.fitChildren = function()
{
	var c = {width: this.content.offsetWidth, height: this.content.offsetHeight};
	
	COM(this.premiumAccount).fitLayout(c);
	COM(this.globalFeatures).fitLayout(c);
	COM(this.localFeatures).fitLayout(c);
}

PaymentDialog.prototype.startTimer = function()
{
	this.timer = setInterval($.proxy(this.tick, this), 1000);
}

PaymentDialog.prototype.stopTimer = function()
{
	clearInterval(this.timer);
}

PaymentDialog.prototype.showRoutine = function()
{
    this.startTimer();
}

PaymentDialog.prototype.closeRoutine = function()
{
    this.stopTimer();
}

PaymentDialog.popup = function(user, estate, onExtendPremium)
{
    var pd = COM.create("PaymentDialog");
    pd.ownerUser = user;
	
    if (user.isCurrent())
    {
        COM(pd.localFeatures).init(FUser.CURRENT_USER, FUser.LINKS, null);
    }
    else
    {
        COM(pd.localFeatures).init(estate.owner, estate.links, estate);
    }
    
    pd.setTabVisible(0, Boolean(user.avatarState));
    
    if (user.isPremiumFeatureWarning())
    {
        pd.showTab(0);
    }
    else if (user.isGlobalFeatureWarning())
    {
        pd.showTab(1);
    }
    else if (user.isLocalFeatureWarning())
    {
        pd.showTab(2);
    }
    else if (!pd.premiumAccount.tabVisible)
    {
        pd.showTab(1);
    }
    else
    {
        pd.showTab(0);
    }
    pd.tick();
    
    pd.onPremiumExtended = onExtendPremium;
    
    pd.tick();
    
    pd.popup();
    
    return pd;
}

PaymentDialog.prototype.popup = function(parent)
{
	superMethod(PaymentDialog.prototype, "popup", this)(parent);
	
	this.fullScreen();
	
	this.init();
}

PaymentDialog.prototype.setTabVisible = function(ix, show)
{
	UI.show(show, [this.tabButtons[ix], this.tabViews[ix]]);
}

PaymentDialog.prototype.hideAllTabs = function()
{
	UI.show(false, this.tabViews);
}

PaymentDialog.prototype.showTab = function(ix)
{
	this.hideAllTabs();
	
	UI.show(true, [this.tabViews[ix]]);
	
	this.tabViews[ix].refresh();
	
	COM(this.tabs).value = ix;
	
	this.fitChildren();
}

PaymentDialog.prototype.changeTab = function()
{
	this.showTab(COM(this.tabs).value);
}

PaymentDialog.updateStateLabel = function(s, warning)
{
    if (warning && !s.styleName)
    {
        Style.FUNDS_FULL.applyTo(s);
    }
    else
    {
        s.styleName = null;
    }
}

PaymentDialog.hasWarning = function(user, estateId, service)
{
    var global = user.globalWarnings;
    var local = user.localWarnings;
    return !estateId && global.contains(service) ||
            estateId && local.containsKey(estateId.mapKey()) && local.getValue(estateId.mapKey()).contains(service);
}

PaymentDialog.prototype.tick = function()
{
    var user = this.ownerUser.isCurrent() ? FUser.CURRENT_USER : this.ownerUser;
    
	COM(this.premiumAccount).user = user;
    COM(this.globalFeatures).user = user;
    COM(this.localFeatures).user = user;
    
    if (this.premiumAccount && this.premiumAccount.style.display != "none")
    {
        COM(this.premiumAccount).tick();
    }
    if (this.globalFeatures && this.globalFeatures.style.display != "none")
    {
        COM(this.globalFeatures).tick();
    }
    if (this.localFeatures && this.localFeatures.style.display != "none")
    {
        COM(this.localFeatures).tick();
    }
}