function PremiumAccountTab()
{
	this.tabVisible = false;
    this.decl = null;
    this.onPremiumExtended = null;
    this.user = null;
    
    this.__defineGetter__("isFree", this.getIsFree);
    this.__defineGetter__("isPromo", this.getIsPromo);
    this.__defineGetter__("isVip", this.getIsVip);
    this.__defineGetter__("isForeverVip", this.getIsForeverVip);
    this.__defineGetter__("serviceItemIndex", this.getServiceItemIndex);
}

PremiumAccountTab.prototype.__construct = function()
{
	$(this.periodCombo).on("change", $.proxy(this.periodChanged, this));
	COM.bindExclusiveClick(this.apply, $.proxy(this.onApply, this));
	$(this.autoOrderCheckbox).on("change", $.proxy(this.doAutoOrder, this));
	COM.bindExclusiveClick(this.chooseFree, $.proxy(this.onPlayForFree, this));
	
	this.scroll = new iScroll(this.wrapper, {scrollbarColor: '#784f0b'});
}

PremiumAccountTab.prototype.fitLayout = function(container)
{
	this.wrapper.style.width = container.width+"px";
	this.wrapper.style.height = container.height+"px";
	
	this.refreshScroll();
}

PremiumAccountTab.prototype.refreshScroll = function()
{
	setTimeout($.proxy(this.scroll.refresh, this.scroll), 500);
}

PremiumAccountTab.prototype.refresh = function()
{
	this.refreshScroll();
	
	if(this.user)
	{
		this.tick();
	}
}

PremiumAccountTab.prototype.getIsFree = function()
{
    return this.user.avatarState == "FREE";
}

PremiumAccountTab.prototype.getIsPromo = function()
{
    return this.user.avatarState == "PROMO";
}

PremiumAccountTab.prototype.getIsVip = function()
{
    return this.user.avatarState == "VIP";
}

PremiumAccountTab.prototype.getIsForeverVip = function()
{
    return this.isVip && this.user.subscriptionExpirationTick == -1;
}

PremiumAccountTab.prototype.getServiceItemIndex = function()
{
    return this.decl.items.length > 1 ? COM(this.periodCombo).selectedIndex : 0;
}

PremiumAccountTab.prototype.tick = function()
{
    var service = this.isFree ? ServiceDecl.PREMIUM : ServiceDecl.SUBSCRIPTION;
    if (!this.decl || this.decl.service != service)
    {
        this.decl = ServiceHelper.getGlobalService(service);
        UI.show(!this.isForeverVip && this.decl.items.length > 1, [this.periodCombo],
                $.proxy(function()
                {
                    var pc = COM(this.periodCombo);
                    pc.dataProvider = ServiceGridRow.getComboItems(this.decl);
                    if(pc.selectedIndex < 0) pc.selectedIndex = 0;
                }, this),
                $.proxy(function()
                {
                    COM(this.price).text = ServiceGridRow.getComboItem(this.decl, 0);
                }, this)
        );
    };
    
    var title;
    var endTick;
    
    if (this.isPromo)
    {
        title = I18.f("PROMO_PERIOD");
        endTick = this.user.promoExpirationTick;
        PaymentDialog.updateStateLabel(this.status.innerHTML, FUser.isFeatureWarning(endTick));
    }
    else if (this.isFree)
    {
        title = I18.f("PREMIUM_ACCOUNT");
        endTick = this.user.premiumExpirationTick;
        PaymentDialog.updateStateLabel(this.status.innerHTML, PaymentDialog.hasWarning(this.user, null, ServiceDecl.PREMIUM));
    }
    else
    {
        title = I18.f("SUBSCRIPTION");
        endTick = this.user.subscriptionExpirationTick;
        PaymentDialog.updateStateLabel(this.status.innerHTML, PaymentDialog.hasWarning(this.user, null, ServiceDecl.SUBSCRIPTION));
    }
    
    this.titleLabel.innerHTML = title;
    this.advantages.innerHTML = this.isFree ? I18.f("PREMIUM_ADVANTAGES") : I18.f("SUBSCRIPTION_ADVANTAGES");
    this.updateDescription();
    var label = this.isFree ? I18.f("PREMIUM_ACCOUNT") : I18.f("SUBSCRIPTION");

    this.status.innerHTML = this.isForeverVip ? I18.f("ACTIVE_FOREVER") : (!Tick.hasPassed(endTick) ? I18.f("ACTIVE_TILL", Tick.toTimeFutureWithYear(endTick)) : I18.f("INACTIVE"));

    UI.show(!this.isForeverVip, [this.extendLabel, this.price, this.apply], $.proxy(function()
    {
        this.extendLabel.innerHTML = this.isFree ? I18.f("PRICE") :
                (this.user.subscriptionExpirationTick > Tick.now()) ? I18.f("EXTEND_SUBSCRIPTION") : I18.f("BUY_SUBSCRIPTION");
        COM(this.apply).caption = (endTick > Tick.now()) && !this.isPromo ? I18.f("PAYMENT_PROLONGATE") : I18.f("ORDER_SERVICE");
    }, this));
    UI.show(!this.isFree && !this.isForeverVip, [this.chooseFree]);
    
    this.periodChanged();
}

PremiumAccountTab.prototype.updateDescription = function()
{
    if (this.isFree)
    {
        this.description.innerHTML = I18.flow("PREMIUM_ACCOUNT_DESC", FConfig.PREMIUM_MESSAGE_ARCHIVE_CAPACITY, FConfig.PREMIUM_CONTACT_LIST_CAPACITY);
        UI.show(false, [this.description2, this.description3]);
    }
    else
    {
        if (FConfig.IS_SURVIVAL)
        {
            this.description.innerHTML = I18.flow("SUBSCRIPTION_SURVIVAL_DESC_1", this.user.withdrawalEnabled ? I18.f("WITHDRAWAL") : I18.f("WITHDRAWAL_TO_PORTAL"));
            UI.show(true, [this.description2, this.description3]);
            this.description2.innerHTML = I18.flow("SUBSCRIPTION_SURVIVAL_DESC_2");
            //this.description2.toolTip = I18.DONT_ACT_ON_SURVIVAL.f();
            this.description3.innerHTML = I18.flow("SUBSCRIPTION_SURVIVAL_DESC_3", FConfig.PREMIUM_MESSAGE_ARCHIVE_CAPACITY, FConfig.PREMIUM_CONTACT_LIST_CAPACITY);
        }
        else
        {
            this.description.innerHTML = I18.flow("SUBSCRIPTION_DESC", FConfig.PREMIUM_MESSAGE_ARCHIVE_CAPACITY, FConfig.PREMIUM_CONTACT_LIST_CAPACITY,
                    this.user.withdrawalEnabled ? I18.f("WITHDRAWAL") : I18.f("WITHDRAWAL_TO_PORTAL"));
            UI.show(false, [this.description2, this.description3]);
        }
    }
}

PremiumAccountTab.prototype.onPlayForFree = function()
{
    var confirmation = ChooseFreeConfirmation.popup(this, $.proxy(function()
    {
        Service.user.callMethod("chooseFree", [], $.proxy(function(u)
        {
            confirmation.close();
            FUser.updateCurrentUser(u);
            this.tick();
            Alert.create(I18.f("SWITCHED_TO_FREE_MODEL"));
        }, this));
    }, this));
}

PremiumAccountTab.prototype.onApply = function()
{
    if (!Quarantine.contains(Quarantine.PREMIUM))
    {
        Quarantine.put(Quarantine.PREMIUM);
        ServiceHelper.orderGlobalServiceSimple(this.decl.service, this.serviceItemIndex, null, null, $.proxy(this.handleExtendPremium, this));
    }
}

PremiumAccountTab.prototype.handleExtendPremium = function(e)
{
    Alert.create(!this.isFree ? I18.f("SUBSCRIPTION_EXTENDED_SUCCESSFULLY") : I18.f("PREMIUM_EXTENDED_SUCCESSFULLY"));
    FUser.updateCurrentUser(e.currentUser);
    this.tick();
    if (this.onPremiumExtended)
    {
        this.onPremiumExtended(e.currentUser);
    }
}

PremiumAccountTab.prototype.periodChanged = function()
{
    if (this.serviceItemIndex < 0)
    {
        return;
    }
    
    var item = this.decl.items.getItemAt(this.serviceItemIndex);
    
    COM(this.price).update(item.price);
    var manageable = this.user.username == FUser.CURRENT_USER.username;
    COM(this.chooseFree).enabled = manageable;

    if (this.user.premiumExpirationTick > Tick.now() + FConfig.PEARL_MAX_ADVANCE_DURATION)
    {
        //COM(this.apply).enabled = false;
        COM(this.apply).reason = I18.f("PEARL_PAYMENT_REACHED_MAX_ADVANCE");
    }
    else if (this.user.withdrawalForbiddenInSession)
    {
        //apply.enabled = false;
        COM(this.apply).reason = I18.f("PREMIUM_ACCOUNT_FORBIDDEN_COUNTRY");
    }
    else if (this.user.pearl < item.price)
    {
        //apply.enabled = false;
        COM(this.apply).reason = I18.f("PREMIUM_ACCOUNT_NOT_ENOUGH_RESOURCES");
    }
    else
    {
        COM(this.apply).enabled = manageable;
        COM(this.apply).reason = "";
    }
	
	console.log(item.autoOrderable, this.isForeverVip);
    UI.show(item.autoOrderable && !this.isForeverVip, [this.autoOrderCheckbox], $.proxy(function()
    {
		var cb = COM(this.autoOrderCheckbox);
		
		cb.checked = this.user.servicesAutoOrder.contains(this.decl.service);
        cb.enabled = (this.user.premiumExpirationTick >= Tick.now() || this.user.servicesAutoOrder.contains(this.decl.service)) && manageable;
        cb.setLabel(I18.f("AUTO_ORDER") + (cb.enabled ? "" : ". " + I18.f("NOT_ORDERED")));
    }, this));
}

PremiumAccountTab.prototype.doAutoOrder = function()
{
    ServiceHelper.setGlobalServiceAutoOrder(this.decl.service, COM(this.autoOrderCheckbox).checked, null);
}