function ServiceGridRow()
{
	this.decl = null;
    this.estate = null;
    this.selectedItem = null;
    this.onBuyService = null;
    this.onAutoOrderService = null;
}

ServiceGridRow.getRewardParams = function(decl, level)
{
    if (decl.items.length == 1)
    {
        return decl.items[0].dtoReward[0].rewardParams;
    }
    else if (level < decl.items.length && decl.items[level].dtoReward.length > 0)
    {
        return decl.items[level].customParams;
    }
    return null;
}

ServiceGridRow.getComboItems = function(decl)
{
    var comboItems = [];
    for (var i=0; i<decl.items.length; i++)
    {
        comboItems.push(ServiceGridRow.getComboItem(decl, i));
    }
    return comboItems;
}

ServiceGridRow.getComboItem = function(decl, i)
{
    var item = decl.items[i];
    var substitutedLabel = I18.getMegaString(item.label);
    return I18.getMegaString(substitutedLabel, ServiceGridRow.getRewardParams(decl, i));
}

ServiceGridRow.prototype.__construct = function()
{
	COM.bindExclusiveClick(this.buy, $.proxy(this.enterParams, this));
	COM.bindExclusiveClick(this.autoOrderCheckbox, $.proxy(this.doAutoOrder, this));
}

ServiceGridRow.prototype.init = function(user, estate, decl, onBuyService, onAutoOrderService)
{
    this.decl = decl;
    this.estate = estate;
    this.onBuyService = onBuyService;
    this.onAutoOrderService = onAutoOrderService;
    this.selectedItem = decl.items[0];

    var comboItems = ServiceGridRow.getComboItems(decl);
    
    if (comboItems.length == 1)
    {
        this.comboItem.appendChild(UI.label(comboItems[0]));
    }
    else
    {
        var combo = COM.create("ComboBox");
        combo.dataProvider = comboItems;
        combo.id = "combo";
        combo.selectedIndex = 0;
        $(combo.DOM).on("change", $.proxy(function()
        {
            this.selectedItem = decl.items.getItemAt(combo.selectedIndex);
            COM(this.price).update(this.selectedItem.price);
        }, this));
        this.comboItem.appendChild(combo.DOM);
    }

    var description = I18.getServiceDescription(decl);
    if (description)
    {
        UIh.addItem(this.descBox, description);
    }
    else
    {
        EffectsView.createDescriptiveView(this.descBox, this.selectedItem.dtoReward, false, false, false, false);
    }

    if (decl.service == ServiceDecl.VACATION)
    {
        this.descBox.style.textDecoration = "underline";
        COM.bindExclusiveClick(this.descBox, function()
        {
        	Alert.create(I18.getMegaString(I18.f("VACATION_TIP"), FConfig));
        });
    }
	
    this.tick(user, estate);
}

ServiceGridRow.prototype.getSelectedItem = function()
{
    return this.selectedItem;
}

ServiceGridRow.prototype.enterParams = function()
{
    var d = COM.create("UseTournamentServiceDialog");
    d.reward = this.selectedItem.dtoReward;
    d.onChoose = $.proxy(function(rewardData)
    {
        this.onBuyService(this.decl.service, this.decl.items.getItemIndex(this.selectedItem), rewardData,
                this.estate ? this.estate.id : GlobalSelectionCitiesCombo.selectedCityId,
                null, COM(this.buy).caption == I18.f("PAYMENT_PROLONGATE"));
    }, this);
    d.popup();
}

ServiceGridRow.prototype.doAutoOrder = function()
{
    this.onAutoOrderService(this.decl.service, COM(this.autoOrderCheckbox).checked, !this.estate ? GlobalSelectionCitiesCombo.selectedCityId : this.estate.id);
}

ServiceGridRow.prototype.updateStatus = function(text, on, warning)
{
    UI.show(true, [this.statusLabel], $.proxy(function()
    {
        this.statusLabel.innerHTML = text;
        PaymentDialog.updateStateLabel(this.statusLabel, warning);
    }, this));
}

ServiceGridRow.prototype.tick = function(user, estate)
{
    this.estate = estate;
    var servicesCooldowns;
    var servicesActiveTill;
    var servicesValues;
    var autoOrder;
    var underAttack;
    
    if (estate)
    {
        servicesCooldowns = new FMap();
        servicesActiveTill = estate.getServicesActiveTill();
        servicesValues = estate.servicesValues;
        autoOrder = estate.servicesAutoOrder;
        underAttack = estate.isEstateUnderSiege;
    }
    else
    {
        servicesCooldowns = user.servicesCooldowns;
        servicesActiveTill = user.getServicesActiveTill();
        servicesValues = user.servicesValues;
        autoOrder = user.servicesAutoOrder;
        underAttack = user.isUserUnderSiege;
    }

    COM(this.price).ownerUser = user;
    COM(this.price).update(this.selectedItem.price);
    
    var buy = COM(this.buy);
    
    var cooldown = servicesCooldowns.getValueOrNull(this.decl.service);
    if (!cooldown)
    {
        cooldown = 0;
    }
    var activeTill = servicesActiveTill.getValueOrNull(this.decl.service);
    if (!activeTill)
    {
        activeTill = 0;
    }
    var serviceValue = servicesValues.getValueOrNull(this.decl.service);
    if (!serviceValue)
    {
        serviceValue = 0;
    }

    if (activeTill >= Tick.now())
    {
        buy.caption = I18.f("PAYMENT_PROLONGATE");
        this.updateStatus(
                I18.f("PAYMENT_TILL", Tick.toTimeFuture(activeTill)),
                true,
                PaymentDialog.hasWarning(user, !estate ? null : estate.id, this.decl.service)
        );
    }
    else
    {
        buy.caption = I18.f("ORDER_SERVICE");
        this.updateStatus(
                (cooldown > Tick.now()) ? I18.mega("UNAVAILABLE_TILL", [cooldown]) :
                        (this.selectedItem.valueInfo.length != 0) ? I18.getString(this.selectedItem.valueInfo, [serviceValue]) : null,
                false
        );
    }
    
    var canBuy = false;
    
    if (Tick.now() + FConfig.PEARL_MAX_ADVANCE_DURATION < activeTill)
    {
        buy.reason = I18.f("PEARL_PAYMENT_REACHED_MAX_ADVANCE");
    }
    else if (this.selectedItem.requirePositiveValue && serviceValue <= 0)
    {
        var tips = this.selectedItem.wrongValueTips.split(",");
        var index = -serviceValue;
        if (index >= tips.length)
        {
            index = tips.length - 1;
        }
        
        buy.reason = I18.getMegaString(I18.getString(tips[index]), ServiceGridRow.getRewardParams(this.decl, 0));
    }
    else if (cooldown > Tick.now())
    {
        buy.reason = I18.f("PEARL_BOOST_ERROR_COOLDOWN");
    }
    else if (!this.selectedItem.underAttack && this.underAttack)
    {
        buy.reason = I18.getMegaString(I18.f("PEARL_BOOST_ERROR_UNDER_ATTACK"), FConfig);
    }
    else if (user.pearl < this.selectedItem.price)
    {
        buy.reason = I18.f("NOT_ENOUGH_PEARL");
    }
    else
    {
        canBuy = true;
        buy.reason = "";
    }
    
    var manageable = user.username == FUser.CURRENT_USER.username;
    buy.enabled = manageable;
    
    var cb = COM(this.autoOrderCheckbox);
    
    cb.DOM.style.display = this.selectedItem.autoOrderable ? "" : "none";
    cb.disabled = !((activeTill >= Tick.now() || autoOrder.contains(this.decl.service)) && manageable);
    //cb.toolTip = I18.AUTO_ORDER.f() + (autoOrderCheckbox.enabled ? "" : "\n" + I18.NOT_ORDERED.f());
    cb.checked = autoOrder.contains(this.decl.service);
}