function TournamentTab()
{
	this.tabVisible = true;
	this.tabToolTip = "";
	this.isFlag = false;
	this.tournamentData = null;
	
	this.__defineGetter__("services", this.getServices);
	this.__defineGetter__("rules", this.getRules);
	this.__defineGetter__("notParticipate", this.getNotParticipate);
	this.__defineGetter__("tournamentName", this.getTournamentName);
	
	this.update = $.proxy(this.update, this);
	this.buyAndUseTournamentService = $.proxy(this.buyAndUseTournamentService, this);
}

TournamentTab.prototype.__construct = function()
{
	var attr = $(this.DOM).attr();
	this.isFlag = (attr['isflag'] ? Boolean(attr['isflag']) : false);
	
	this.scroll = new iScroll(this.wrapper);
}

TournamentTab.prototype.fitLayout = function(container)
{
	this.wrapper.style.width = container.width + "px";
	this.wrapper.style.height = (container.height-16) + "px";
}

TournamentTab.prototype.getServices = function()
{
    if(this.tournamentData.isKing)
    {
        return FConfig.TOURNAMENT_KING_SERVICES;
    }
    else
    {
        if (this.isFlag)
        {
            return FConfig.TOURNAMENT_SERVICES;
        }
        else
        {
            return FConfig.TOURNAMENT_GROTTO_SERVICES;
        }
    }
}

TournamentTab.prototype.getRules = function()
{
    if (this.tournamentData.isKing)
    {
        return I18.f("TOURNAMENT_KING_DESCRIPTION");
    }
    else
    {
        if (this.isFlag)
        {
            return I18.f("TOURNAMENT_DESCRIPTION");
        }
        else
        {
            return I18.f("TOURNAMENT_GROTTO_DESCRIPTION");
        }
    }
}

TournamentTab.prototype.getNotParticipate = function()
{
    return this.isFlag ? I18.f("YOU_ARE_NOT_PARTICIPATE_IN_TOURNAMENT") : I18.f("YOU_ARE_NOT_PARTICIPATE_IN_GROTTO");
}

TournamentTab.prototype.getTournamentName = function()
{
    return this.isFlag ? I18.f("TOURNAMENT_FLAG") : I18.f("TOURNAMENT_GROTTO");
}

TournamentTab.getLabel = function(isFlag, isKing)
{
    return I18.getString("TOURNAMENT_" + (isKing ? "KING" : "NORMAL") + "_" + (isFlag ? "FLAG" : "GROTTO"));
}

TournamentTab.prototype.update = function(user)
{
    this.tournamentData = this.isFlag ? user.tournament : user.tournamentGrotto;
    
    var label = TournamentTab.getLabel(this.isFlag, this.tournamentData.isKing);
    
    if(!this.tournamentData) return;
    
    var descText;
    if (this.tournamentData.isRunning)
    {
        descText = this.tournamentData.isInTournament ? this.rules : this.notParticipate;
    }
    else
    {
        descText = I18.f("NO_TOURNAMENT_IS_RUNNING", DateUtils.formatWithDateAndSeconds(Tick.toMs(this.tournamentData.nextTournamentStartTick)));
    }
    this.desc.innerHTML = descText;

    UI.show(this.tournamentData.isRunning, [this.tournamentEndInfo, this.helpBox, this.rulesLabel]);
    this.tournamentEndInfo.innerHTML = I18.f("TOURNAMENT_ENDS", Tick.toTimeWithDate(this.tournamentData.tournamentFinishTick));
    
    this.rulesLabel.innerHTML = I18.f(this.tournamentData.isKing ? "TOURNAMENT_KING_RULES" : "TOURNAMENT_RULES", this.tournamentName);

    var hasRewards = this.tournamentData.rewards.length > 0;
    this.rewardsHolder.style.display = hasRewards ? "" : "none";
    this.noRewardsLabel.style.display = !hasRewards ? "" : "none";
    
    COM.clear(this.rewardsHolder);
    
    if (hasRewards)
    {
        for (var i = 0; i < this.tournamentData.rewards.length; i++)
        {
            var renderer = this.createRewardRenderer();
            renderer.update(this.tournamentData.rewards[i]);
            this.rewardsHolder.appendChild(renderer.DOM);
        }
    }
    this.updateAvailable();
    
    setTimeout($.proxy(this.scroll.refresh, this.scroll), 500);
}

TournamentTab.prototype.isChanged = function()
{
    var children = this.grid.children;
    if (this.services.keys.length != children.length)
    {
        return true;
    }
    
    var entries = this.getTournamentServices()._entries;
    for(var i=0; i<entries.length; i++)
    {
    	var row = COM(children[i]);
    	if(row)
    	{
    		key = entries[i].key;
    		if (key != row.service)
    		{
            	return true;
        	}
    	}
    	else return true;
    }
    
    return false;
}

TournamentTab.prototype.updateAvailable = function()
{
    var canBuy = this.tournamentData.isRunning && this.tournamentData.isInTournament && this.tournamentData.timedServicesAvailableTillTick > Tick.now();
    var buyToolTip = "";
    var disabledByTimeout = false;
    if (!this.tournamentData.isRunning)
    {
        buyToolTip = I18.f("TOURNAMENT_IS_NOT_RUNNING");
    }
    else if (!this.tournamentData.isInTournament)
    {
        buyToolTip = this.notParticipate;
    }
    else if (this.tournamentData.timedServicesAvailableTillTick <= Tick.now())
    {
        buyToolTip = I18.f("TOURNAMENT_SERVICE_BUY_TIMEOUT", NumberUtils.floor(FConfig.TOURNAMENT_SERVICES_TIMEOUT / Tick.TICKS_PER_HOUR));
        disabledByTimeout = true;
    }
    if (this.isChanged())
    {
        this.remakeRenderers();
    }
    
    var children = this.grid.children;
    for(var i=0; i<children.length; i++)
    {
        var grow = COM(children[i]);
        if (grow)
        {
            var decl = this.services.getValueOrNull(grow.service);
            if (decl && decl.hasTimer || !disabledByTimeout)
            {
                COM(grow.buy).enabled = canBuy;
                var contains = false;
                for(var n=0; n<this.tournamentData.usedServices.length; n++)
                {
                	if(this.tournamentData.usedServices[n].key == grow.service)
                	{
                		contains = true;
                		break;
                	}
                }
                if (contains)
                {
                    COM(grow.buy).reason = I18.f("TOURNAMENT_SERVICE_ALREADY_ORDERED") + "<br/>" + buyToolTip;
                }
                else
                {
                    COM(grow.buy).reason = buyToolTip;
                }
            }
            grow.update(this.tournamentData);
        }
    }
}

TournamentTab.prototype.remakeRenderers = function()
{
    var entries = this.getTournamentServices()._entries;
    var needed = entries.length;
    var actual = this.grid.children.length;
    var i = 0;
    while (i < actual - needed)
    {
        COM(this.grid.children[0]).destroy();
        i++;
    }
    
    i = 0;
    while (i < needed - actual)
    {
        this.grid.appendChild(COM.create("TournamentGridRow").DOM);
        i++;
    }

    var children = this.grid.children;
    for(var i=0; i<children.length; i++)
    {
    	var grow = COM(children[i]);
    	if(grow)
    	{
	        grow.service = entries[i].key;
	        grow.onUpdate = this.update;
	        //grow.isFlags = this.isFlag;
	        grow.decl = this.services.getValue(grow.service);
            grow.onBuy = this.buyAndUseTournamentService;
       }
    }
}

TournamentTab.prototype.buyAndUseTournamentService = function(type, params)
{
    Service.quest.callMethod("buyAndUseTournamentService", [type, params, this.isFlag],
            $.proxy(function (user)
            {
                Alert.create(I18.f("FEATURE_ACTIVATED"));
                this.update(user);
            }, this),
            function (e)
            {
                if (GameException.isDisplayable(e))
                {
                    Alert.create(e.getMessage());
                    return true;
                }
                return false;
            });
}

TournamentTab.prototype.createRewardRenderer = function()
{
    var r = COM.create("TournamentRewardRenderer");
    r.onGetReward = $.proxy(function (a)
    {
        Service.quest.callMethod("getTournamentReward", a.id,
        $.proxy(function(user)
        {
            this.update(user);
            AdvisorInfoDialog.create(I18.mega(I18.f("TOURNAMENT_GET_REWARD_SUCCESS"), a.declaration, this.tournamentName), user.race).popup();
        }, this),
        function (e)
        {
            if (e.__className__ == "TournamentException")
            {
                AdvisorInfoDialog.create(e.getMessage(), FUser.CURRENT_USER.race).popup();
                return true;
            }
            return false;
        });
    }, this);
    return r;
}

TournamentTab.prototype.getTournamentServices = function()
{
	return this.services;
    /*
    return services.entries.sort(function (e1:Array, e2:Array)
    {
        return TournamentServiceDecl(e1[1]).place - TournamentServiceDecl(e2[1]).place;
    });
    */
}