function GameServerDialog()
{
	callParentConstructor(GameServerDialog.prototype, this);
	
	this.onEnterGame = function (serverId) {};
	this.onAccountUpdated = function (acc) {};
	this.onEditMoveAvatarToServer = function (serverId){};
	
	this.showServerInfo = true;
	this.gameServer = null;
	this.avatarInfo = null;
	this.account = null;
	
	this.ticker = null;
}

extendClass(GameServerDialog.prototype, Dialog.prototype);

GameServerDialog.prototype.showServerInfo = true;
GameServerDialog.prototype.gameServer = null;
GameServerDialog.prototype.avatarInfo = null;
GameServerDialog.prototype.account = null;

GameServerDialog.create = function(server, avatar, onEnter, onAccount, onEditMove)
{
	var d = COM.create('GameServerDialog');
	d.avatarInfo = avatar || null;
	d.onEnterGame = onEnter || function (serverId) {};
	d.onAccountUpdated = onAccount || function (acc) {};
	d.onEditMoveAvatarToServer = onEditMove || function (serverId){};
	d.update(server);
	d.popup();
	return d;
}

GameServerDialog.prototype.__construct = function()
{
    this.ticker = setInterval($.proxy(this.tick, this), 1000);

    this.accountUpdateHandler = $.proxy(this.accountUpdateHandler, this);
    
    COM.bindExclusiveClick(this.serverImage.parentNode, $.proxy(function(){
    	
    	if (!this.gameServer) return false;
    	
    	var tip = document.createElement('div');
		
		var txt = this.gameServer.config.aboutServer.getValue ? this.gameServer.config.aboutServer.getValue(0) : this.gameServer.config.aboutServer;
    	txt = txt.split("\n"), title = txt.shift();
    	title = '<div class="text-normal-bold" style="text-align: center;">' + title + '</div>';
    	txt = title + '<p style="text-indent: 2em;">' + txt.join('</p><p style="text-indent: 2em;">') + '</p>';
    	tip.style.textAlign = 'justify';
    	tip.innerHTML = txt;
    	TipDialog.create(tip);
    	
    }, this));
    COM.bindExclusiveClick(this.enterButton, $.proxy(this.onEnterGameClick, this));
    COM.bindExclusiveClick(this.btnDeleteAvatar, $.proxy(this.onDeleteAvatar, this));
    COM.bindExclusiveClick(this.btnMergeAvatar, $.proxy(this.onMergeAvatarClick, this));
    COM.bindExclusiveClick(this.financeButton, $.proxy(this.transferPearlClick, this));
}

GameServerDialog.prototype.onClose = function()
{
	this.ticker = clearTimeout(this.ticker);
}

GameServerDialog.prototype.update = function(gs)
{
    this.gameServer = gs;

    this.updateServerIcon();
    
    this.totalCount.innerHTML = this.gameServer.onlineInfo.totalCount;
    this.activeCount.innerHTML = this.gameServer.onlineInfo.activeCount;
    this.onlineCount.innerHTML = this.gameServer.onlineInfo.onlineCount;
    
    var name = this.gameServer.displayName.split(': '); 
    this.serverName.innerHTML = name[1]; 
    this.serverSubName.innerHTML = name[0]; 
    
    this.updateEnterButton();
    this.updateServerState();
    
    //UI.show(this.gameServer.avatarExists, [this.detailsPanel]);
    //COM(this.toggleButton).enabled = gameServer.avatarExists;

    this.tick();
}

GameServerDialog.prototype.onTimerUpdate = function()
{
	this.getAvatarInfo();
}

GameServerDialog.prototype.tick = function()
{
    if (!this.gameServer) return;

    if (this.gameServer.saveWorldTimeout*1 >= 0) 
    {
        this.serverState.innerHTML = I18.f('TILL_SERVER_SAVE', Tick.toTimeLeft(this.gameServer.saveWorldTimeout));
        this.gameServer.saveWorldTimeout--;
        if (this.gameServer.saveWorldTimeout*1 < 0) 
        {
            this.gameServer.saveWorldTimeout = 0;
        }
    }

    UI.show(this.avatarInfo && this.gameServer.avatarExists && (this.gameServer.type.name != GameServerType.SURVIVAL.name), [this.btnDeleteAvatar]);
    
    if (this.avatarInfo)
    {
        if (this.avatarInfo.avatarDeletionTimeLeft > 0) 
        {
            this.avatarInfo.avatarDeletionTimeLeft -= 1;
        }
        if (this.avatarInfo.avatarDeletionTimeLeft >= 0) 
        {
            //btnDeleteAvatar.source = LoginAssets.UNDELETE_AVATAR;
            
            COM(this.btnDeleteAvatar).caption = I18.f('UNDELETE_AVATAR');
            COM(this.btnDeleteAvatar).subCaption = I18.f('DELETE_AVATAR_LEFT', Tick.toTimeLeft(this.avatarInfo.avatarDeletionTimeLeft));
            this.serverState.innerHTML = I18.f('OWNER_DELETION_PENDING_MESSAGE');
            //btnDeleteAvatar.enabled = true;
            
            COM(this.btnDeleteAvatar).icon = null;
            
            $(this.btnDeleteAvatar).removeClass("do_delete");
        } 
        else 
        {
        	COM(this.btnDeleteAvatar).caption = "";
        	COM(this.btnDeleteAvatar).subCaption = '';
        	
        	COM(this.btnDeleteAvatar).icon = STATIC_INTERFACE_URL + "shared/icons/close.png";
            
            //btnDeleteAvatar.source = LoginAssets.DELETE_AVATAR;
            if (!this.gameServer.avatarExists) 
            {
                COM(this.btnDeleteAvatar).reason = I18.f('NO_AVATAR');
            } 
            else if (this.gameServer.type.name == GameServerType.SURVIVAL.name) 
            {
                COM(this.btnDeleteAvatar).reason = I18.f('CANT_DELETE_AVATAR');
            } 
            else 
            {
                //COM(this.btnDeleteAvatar).caption = I18.f('DELETE_AVATAR');
            }
            
            $(this.btnDeleteAvatar).addClass("do_delete");
        }
        
	    COM(this.financeButton).subCaption = I18.f('BALANCE') + ": " 
	    	//+ FPortalConfig.formatPearl(this.avatarInfo.pearlAmount)
	    	+ I18.f("PORTAL_CURRENCY_VALUE", Math.floor(this.avatarInfo.pearlAmount));
	    ;
        
		UI.show(this.avatarInfo.avatarDeletionTimeLeft < 0, [this.enterButton]);
    }
    
    UI.show(!!this.gameServer.mergeToServerId, [this.btnMergeAvatar]);
    
    var nextStart = (this.gameServer.type.name == GameServerType.SURVIVAL.name 
    	&& this.gameServer.config 
    	&& this.gameServer.config.registrationPeriodExpired 
    	&& this.gameServer.config.nextStartDate >= Tick.now()
    ) 
		? I18.f('NEXT_GAME_SERVER_START_AT', Tick.toTimeLeft(this.gameServer.config.nextStartDate - Tick.now()))
		: ''
	;
	this.nextStartMessage.innerHTML = nextStart;
}

GameServerDialog.prototype.updateServerIcon = function()
{
    this.serverImage.src = STATIC_INTERFACE_URL + LoginAssets.getServerIcon(this.gameServer);
}

GameServerDialog.prototype.updateServerState = function()
{
    if (this.gameServer.saveWorldTimeout*1 >= 0) return; // tick
    
    var txt = '';
    if (this.gameServer.available) 
    {
        if (this.gameServer.avatarExists) 
        {
            if (this.gameServer.subscriptionActive) 
            {
                txt = I18.f('SUBSCRIBED_TILL', DateUtils.format(Tick.toMs(this.gameServer.subscribedTill), "DD MMM YYYY"));
            } 
            else if (this.gameServer.config.foreverVip) 
            {
                txt = I18.f('SUBSCRIPTION_ACTIVE');
            }
        } 
        else 
        {
            if (!this.gameServer.registrationAllowed) 
            {
                txt = I18.f('SERVER_IS_FULL');
            }
        }
    }
    this.serverState.innerHTML = txt;
}

GameServerDialog.prototype.updateEnterButton = function()
{
    COM(this.enterButton).caption = I18.f('DO_PLAY');
	COM(this.enterButton).icon = STATIC_INTERFACE_URL + LoginAssets.getServerStatusIcon(this.gameServer);
	
	var reason = null;
    if (this.gameServer.isSaveInProgress) 
    {
        reason = I18.f('GAME_SERVER_IS_SAVING');
    } 
    else if (!this.gameServer.available) 
    {
        reason = I18.f('GAME_SERVER_IS_UNAVAILABLE');
    } 
    else if (this.gameServer.avatarExists) 
    {
    } 
    else if (!this.gameServer.hasInvite) 
    {
        reason = this.gameServer.type.name == GameServerType.SURVIVAL.name 
        	? I18.f('GAME_SERVER_IS_SURVIVAL') 
        	: I18.f('GAME_SERVER_IS_HERO_TEST')
        ;
    } 
    else if (!this.gameServer.registrationAllowed) 
    {
        reason = this.gameServer.config.registrationPeriodExpired
			? I18.f('GAME_SERVER_REGISTRATION_PERIOD_EXPIRED')
			: I18.f('GAME_SERVER_REGISTRATION_IS_CLOSED')
		;
    } 
    else 
    {
        COM(this.enterButton).caption = I18.f('DO_REGISTER');
    }
    
	COM(this.enterButton).reason = reason;
}

GameServerDialog.prototype.updateAvatarDetails = function()
{
    if (!this.avatarInfo) return;

    if (!this.gameServer.avatarExists) 
    {
        //financeButton.enabled = false;
        COM(this.financeButton).reason = I18.f('NO_AVATAR');
    } 
    else if (!this.gameServer.config.moneyTransferAllowed) 
    {
        //financeButton.enabled = false;
        COM(this.financeButton).reason = I18.f('TRANSFER_NOT_ALLOWED');
    } 
    else 
    {
    	COM(this.financeButton).reason = null;
    	COM(this.financeButton).skin = this.gameServer.subscriptionActive ? 'green' : 'yellow';
        //financeButton.tip = gameServer.subscriptionActive ? I18.TRANSFER_PEARL_PAID.f() : I18.TRANSFER_PEARL_FREE.f();
    }
    
    COM(this.financeButton).subCaption = I18.f('BALANCE') + ": " 
    	//+ FPortalConfig.formatPearl(this.avatarInfo.pearlAmount)
    	+ I18.f("PORTAL_CURRENCY_VALUE", Math.floor(this.avatarInfo.pearlAmount));
    ;
}

GameServerDialog.prototype.getAvatarInfo = function()
{
    PortalService.account.callMethod('getAvatarInfo', [this.gameServer.id], $.proxy(function (fai)
    {
        this.avatarInfo = fai;
        this.updateAvatarDetails();
        this.tick();
        
    }, this));
}

GameServerDialog.prototype.accountUpdateHandler = function(acc)
{
    this.account = acc;
    this.onAccountUpdated(acc);
    this.onTimerUpdate();
}

GameServerDialog.prototype.onDeleteAvatar = function()
{
    if (!this.avatarInfo) return; 
	
	if (this.avatarInfo.avatarDeletionTimeLeft != -1) 
	{
	    Alert.create(I18.f('CANCEL_DELETE_AVATAR'), null, $.proxy(function(){
	    	PortalService.account.callMethod('cancelAvatarDeletion', [this.gameServer.id], this.accountUpdateHandler);
	    }, this));
	} 
	else 
	{
	    Alert.create(I18.f('DELETE_AVATAR_Q'), null, $.proxy(function(){
	    	PortalService.account.callMethod('pendAvatarDeletion', [this.gameServer.id], this.accountUpdateHandler);
	    }, this));
	}
}

GameServerDialog.prototype.onEnterGameClick = function()
{
    this.onEnterGame(this.gameServer);
    this.close();
}

GameServerDialog.prototype.transferPearlClick = function()
{
	if (!Quarantine.contains(this.financeButton.id))
	{
        Quarantine.put(this.financeButton.id);

        var exchangeDialog = TransferDialog.popup(null/*parent.parent*/, this.gameServer.id, $.proxy(function()
        {
            PortalService.account.callMethod("getAccount", [], this.accountUpdateHandler);
            
        }, this));
    }
}

GameServerDialog.prototype.getPossibleServers = function(serverId, race, current)
{
	var targetIds = [];
	for (var i = 0, len = FPortalConfig.MERGE_SERVER_TARGETS.length; i < len; i++) 
	{
		var targetServerId = FPortalConfig.MERGE_SERVER_TARGETS[i];
		var target = this.account.findServer(targetServerId);
		if (target && (!target.avatarExists || target.deletionPending) && (target.id == current || target.hasSpaceFor(race))) 
		{
			var occupied = false;
			for (var j = 0, len2 = this.account.servers.length; j < len2; j++) 
			{
				var gs = this.account.servers[j];
				if (serverId != gs.id && gs.mergeToServerId == targetServerId) 
				{
					occupied = true;
					break;
				}
			}
			if (!occupied) 
			{
				targetIds.push(targetServerId);
			}
		}
	}
	return targetIds;
}

GameServerDialog.prototype.onMergeAvatarClick = function()
{
	if (!this.gameServer.mergeToServerId) return;

	PortalService.account.callMethod("getAccount", [], $.proxy(function(acc){
		
		this.accountUpdateHandler(acc);

		MergeServerDialog.popup(
			this.account, 
			this.gameServer.id, 
			this.gameServer.mergeToServerId || null, 
			this.getPossibleServers(this.gameServer.id, this.gameServer.race, this.gameServer.mergeToServerId)
		);
		
	}, this));
}
    
