function ServerListItem()
{
	this.server = null;
	this.avatarInfo = null;
	this.onEnter = function(){};
}

ServerListItem.prototype.server = null;
ServerListItem.prototype.avatarInfo = null;

ServerListItem.prototype.__construct = function()
{
	this.accountUpdateHandler = $.proxy(this.accountUpdateHandler, this);
	
	UI.show(false, [this.financeButton, this.btnDeleteAvatar]);
	
	COM.bindExclusiveClick(this.DOM, $.proxy(function(){
		
		this.getAvatarInfo($.proxy(function()
		{
			GameServerDialog.create(
				this.server, this.avatarInfo,
				this.onEnter,
				$.proxy(this.onAccountUpdate, this),
				$.proxy(this.onEditMoveAvatarToServer, this)
			);
		}, this));
		
	}, this));
	
	COM.bindExclusiveClick(this.serverType, $.proxy(function(e){
		
		var descr = this.server.config.aboutServer.getValue ? this.server.config.aboutServer.getValue(0) : this.server.config.aboutServer;
		descr = descr.replace(/\r\n|\r|\n/g, "<br/>");
		Alert.create(descr, I18.f("SERVER_INFO"));
		
		e.stopImmediatePropagation();
		return false;
	}, this));
	
	/*
	COM.bindExclusiveClick(this.status, this, $.proxy(function(){
		
		this.onEnter(this.server);
		return false;
		
	}, this));
	*/
}

ServerListItem.prototype.setItem = function(data)
{
	this.server = data;
	/*
	if (!this.avatarInfo)
	{
		return this.getAvatarInfo();
	}
	*/
	this.tick();    
}

ServerListItem.prototype.tick = function()
{
	if (!this.server) return;
	
	var img = 'server_' 
		+ this.server.type.name.toLowerCase() 
		+ '_' + (this.server.config.foreverVip ? 'paid' : 'free')
	;
	
	var sname = this.server.displayName.split(":");
	this.serverName.innerHTML = sname[1];
	this.serverSubName.innerHTML = sname[0];
	
	this.server.gameServerName = sname[1];
	this.server.gameServerIcon = STATIC_INTERFACE_URL + 'login/' + img + '.png';
	this.server.gameServerOnline = this.server.onlineInfo.onlineCount;

	//sname = "<b>"+sname[1]+"</b><br>"+sname[0];
	
	this.serverType.src = this.server.gameServerIcon;
	this.total.innerHTML = this.server.onlineInfo.totalCount;
	this.active.innerHTML = this.server.onlineInfo.activeCount;
	this.online.innerHTML = this.server.onlineInfo.onlineCount;
	
	var btn = COM(this.status);
	btn.reason = null;
	
	if (this.server.config.saveWorldTimeout >= 0)
	{
		btn.image.src = STATIC_INTERFACE_URL + 'login/round_icon_busy.png';
		btn.reason = I18.f('GAME_SERVER_IS_SAVING');
	}
	else if (!this.server.available)
	{
		btn.image.src = STATIC_INTERFACE_URL + 'login/server_closed.png';
		btn.reason = I18.f('GAME_SERVER_IS_UNAVAILABLE');
	}
	else if (this.server.avatarExists)
	{
		btn.image.src = STATIC_INTERFACE_URL + 'login/can_enter_server.png';
	}
	else if (!this.server.hasInvite)
	{
		btn.image.src = STATIC_INTERFACE_URL + 'login/server_closed.png';
		btn.reason = (this.server.type.name == GameServerType.SURVIVAL.name)
			? I18.f('GAME_SERVER_IS_SURVIVAL')
			: I18.f('GAME_SERVER_IS_HERO_TEST')
		;
	}
	else if (!this.server.registrationAllowed)
	{
		btn.image.src = STATIC_INTERFACE_URL + 'login/server_closed.png';
		btn.reason = this.server.config.registrationPeriodExpired
			? I18.f('GAME_SERVER_REGISTRATION_PERIOD_EXPIRED')
			: I18.f('GAME_SERVER_REGISTRATION_IS_CLOSED')
		;
	}
	else
	{
		btn.image.src = STATIC_INTERFACE_URL + 'login/can_register_server.png';
	}
}

ServerListItem.prototype.getAvatarInfo = function(callback)
{
    PortalService.account.callMethod('getAvatarInfo', [this.server.id], 
	    $.proxy(function (fai)
	    {
	        this.avatarInfo = fai;
	        this.updateAvatarDetails();
	        this.tick();
	        
	        if(callback) callback();
	        
	    }, this)/*, $.proxy(function (err) { APP.back(); return true; }, this) */
    );
}

ServerListItem.prototype.updateAvatarDetails = function()
{
	if (!this.avatarInfo) return;
	this.server.avatarExists = this.avatarInfo.exists;
	this.tick();
}

ServerListItem.prototype.transferPearlClick = function(e)
{
	if (!Quarantine.contains(this.financeButton))
	{
        Quarantine.put(this.financeButton);

        var exchangeDialog = TransferDialog.popup(parent.parent, this.server.id, $.proxy(function()
        {
            PortalService.account.callMethod("getAccount", [], this.accountUpdateHandler);
        }, this));
    }
	
	e.stopImmediatePropagation();
	return false;
}

ServerListItem.prototype.accountUpdateHandler = function(acc)
{
    this.getAvatarInfo();
}

ServerListItem.prototype.requestError = function(response)
{
	console.log('ERROR', response);
	APP.changeGameServer(null);
}
	
ServerListItem.prototype.onDeleteAvatar = function()
{
    if (!this.avatarInfo) return; 
	
	if (this.avatarInfo.avatarDeletionTimeLeft != -1) 
	{
	    Alert.create(I18.f('CANCEL_DELETE_AVATAR'), null, $.proxy(function(){
	    	PortalService.account.callMethod('cancelAvatarDeletion', [this.server.id], this.accountUpdateHandler);
	    }, this));
	} 
	else 
	{
	    Alert.create(I18.f('DELETE_AVATAR_Q'), null, $.proxy(function(){
	    	PortalService.account.callMethod('pendAvatarDeletion', [this.server.id], this.accountUpdateHandler);
	    }, this));
	}
}

