function FinanceDialog()
{
	callParentConstructor(FinanceDialog.prototype, this);
	
	this.account = null;
	
	this.allViews = [];
}

extendClass(FinanceDialog.prototype, Dialog.prototype);

FinanceDialog.popup = function(acc)
{
    var fd = COM.create("FinanceDialog");
    
    //fd.setCaption(I18.f("FINANCE_ACCOUNT"));
    
    fd.fullScreen();
    
    if (!acc)
    {
        PortalService.account.callMethod("getAccount", [], $.proxy(fd.popupWithAccount, fd));
    }
    else
    {
        fd.popupWithAccount(acc);
    }
}

FinanceDialog.convertFinanceDate = function(str)
{
	var d = new Date();
	if(!str) return d;
	
	var tmp = str.split(" ");
	var tmp1 = tmp[0].split("-");
	var tmp2 = tmp[1].split(":");
	
	d.setYear(tmp1[0]*1);
	d.setMonth(tmp1[1]*1-1);
	d.setDate(tmp1[2]*1);
	d.setHours(tmp2[0]*1);
	d.setMinutes(tmp2[1]*1);
	d.setSeconds(tmp2[2]*1);
	
	var offset = new Date().getTimezoneOffset();
	offset += (offset - d.getTimezoneOffset());
	
	d.setHours(d.getHours() + offset/60);
	
	return d;
}

FinanceDialog.prototype.__construct = function()
{
	superMethod(FinanceDialog.prototype, "__construct", this)();
	
	var tabs = COM(this.tabs);
	
	this.tabsButtons = {};
	
	this.tabsButtons.inputButton = tabs.addItem(I18.f("FINANCE_INPUT_LABEL"), 0, true);
	this.tabsButtons.outputWebmoneyButton = tabs.addItem(I18.f("FINANCE_OUTPUT_LABEL"), 1, false);
	this.tabsButtons.outputManualButton = tabs.addItem(I18.f("FINANCE_OUTPUT_LABEL"), 2, false);
	this.tabsButtons.outputEpayButton = tabs.addItem(I18.f("FINANCE_OUTPUT_LABEL"), 3, false);
	this.tabsButtons.pendingButton = tabs.addItem(I18.f("FINANCE_PENDING_LABEL"), 4, false);
	this.tabsButtons.reportButton = tabs.addItem(I18.f("FINANCE_REPORT_LABEL"), 5, false);
	//this.tabsButtons.partnerButton = tabs.addItem(I18.f("FINANCE_PARTNER_LABEL"), 6, false);
	
	//tabs.selectedIndex = 0;
	
	//$(this.tabs).on("change", $.proxy(this.changeTab, this));
	tabs.onChange = $.proxy(this.changeTab, this);
	
	this.allViews = [this.inputTab, this.outputWebmoneyTab, this.outputManualTab, this.outputEpayTab, this.pendingTab, this.reportTab/*, this.partnerTab*/];
	
	COM(this.reportTab).dataRefresher = FinanceDialog.reportTabRefresher;
}

FinanceDialog.prototype.onShow = function()
{
	this.headerLeftBox.style.display = "none";
	this.headerCaptionBox.style.display = "none";
}

FinanceDialog.prototype.popupWithAccount = function(acc)
{
    this.update(acc);
    this.popup();
}

FinanceDialog.prototype.fitLayout = function(container)
{
	superMethod(FinanceDialog.prototype, "fitLayout", this)(container);
	
	this.windowWrapper.style.width = this.content.offsetWidth + "px";
	this.windowWrapper.style.height = this.content.offsetHeight + "px";
	
	this.fitChildLayout();
}

FinanceDialog.prototype.fitChildLayout = function()
{
	var view = COM(this.allViews[COM(this.tabs).value]);
	if(view.fitLayout) view.fitLayout({width: this.content.offsetWidth, height: this.content.offsetHeight});
}

FinanceDialog.prototype.update = function(acc)
{
    this.account = acc;
    
    COM(this.inputTab).update(acc, null);
    COM(this.reportTab).account = acc;
    
    /*
    if (acc.getActualPercent() == 0)
    {
        this.tabsButtons.partnerButton.DOM.style.display = "none";
    }
    else
    {
        COM(this.partnerTab).account = this.account;
    }
    */
    
    if (!FPortalConfig.WEBMONEY_OUTCOME_ENABLED)
    {
        this.tabsButtons.outputWebmoneyButton.DOM.style.display = "none";
    }
    else
    {
        this.updateWithdrawEnabled(this.tabsButtons.outputWebmoneyButton, acc);
        COM(this.outputWebmoneyTab).update(acc);
    }
    
    if (!FPortalConfig.MANUAL_OUTCOME_ENABLED)
    {
        this.tabsButtons.outputManualButton.DOM.style.display = "none";
    }
    else
    {
        this.updateWithdrawEnabled(this.tabsButtons.outputManualButton, acc);
        COM(this.outputManualTab).update(acc);
    }
    
    if (!FPortalConfig.EPAY_OUTCOME_ENABLED)
    {
        this.tabsButtons.outputEpayButton.DOM.style.display = "none";
    }
    else
    {
        this.updateWithdrawEnabled(this.tabsButtons.outputEpayButton, acc);
        COM(this.outputEpayTab).update(acc);
    }
    
    if (!FPortalConfig.WEBMONEY_OUTCOME_ENABLED && !FPortalConfig.MANUAL_OUTCOME_ENABLED && !FPortalConfig.EPAY_OUTCOME_ENABLED)
    {
        this.tabsButtons.pendingButton.DOM.style.display = "none";
    }
   
	this.changeTab();
}

FinanceDialog.prototype.hideAllTabs = function()
{
	for(var i=0; i<this.allViews.length; i++)
	{
		this.allViews[i].style.display = "none";
	}
}

FinanceDialog.prototype.changeTab = function()
{
	this.hideAllTabs();
	
	var view = this.allViews[COM(this.tabs).value];
	view.style.display = "";
	
	var com = COM(view);
	
	if(view == this.inputTab) com.showPaymentMethods();
	else com.refresh();
	
	this.fitChildLayout();
}

FinanceDialog.prototype.updateWithdrawEnabled = function(component, account)
{
    if (!account.withdrawalEnabled)
    {
        component.reason = I18.f("CLIENT_KNOWS_ERROR_WITHDRAWAL_DISABLED");
    }
    else
    {
        component.reason = null;
    }
}

FinanceDialog.reportTabRefresher = function(fromDate, toDate, page, onResultCallback)
{
    PortalService.account.callMethod("getFinanceInfo", [DateUtils.formatForServer(fromDate), DateUtils.formatForServer(toDate), page], onResultCallback);
}

FinanceDialog.prototype.onTabChange = function()
{
}

FinanceDialog.prototype.onClose = function()
{
    COM(this.inputTab).onClose();
}