function FinanceReportTab()
{
	callParentConstructor(FinanceReportTab.prototype, this);
	
	this.info = null;
    this.showRealColumn = false;
    this.sourceWidth = 100;
    this.destinationWidth = 100;
    this.operationWidth = 140;
    this.amountMldWidth = 90;
    this.amountRealWidth = 90;
    this.dateWidth = 100;
    this.commentWidth = 100;
    
    this.update = $.proxy(this.update, this);
}

extendClass(FinanceReportTab.prototype, FinanceTabParent.prototype);

FinanceReportTab.dateFormat = "dd.mm.yyyy";

FinanceReportTab.prototype.__construct = function()
{
	var self = this;
	
	var m = [];
	for(var i=0; i<12; i++) m.push(I18.f("FULL_MONTH_"+i));
	
	var pickerDefaults = 
	{
		format: FinanceReportTab.dateFormat,
		monthsFull: m,
		today: '',
    	clear: '',
		onSet: function()
		{
			COM(this.$node[0]).value = this.get();
			self.dateChanged();
		}
	};
	
	COM.bindExclusiveClick(this.toDatePicker, preventEvent);
	
	this.fromDatePicker = $(this.dateFrom).pickadate(pickerDefaults).pickadate('picker');
	this.toDatePicker = $(this.dateTo).pickadate(pickerDefaults).pickadate('picker');
	
	COM(this.dateFrom).disabled = true;
	COM(this.dateTo).disabled = true;
	
	$(this.dateFrom).removeClass("disabled");
	$(this.dateTo).removeClass("disabled");
	
	COM(this.dateFrom).value = new Date().format(FinanceReportTab.dateFormat);
	COM(this.dateTo).value = new Date().format(FinanceReportTab.dateFormat);
	
	COM(this.grid).setHeader([I18.f("SOURCE"),
							  I18.f("DESTINATION"),
							  I18.f("OPERATION"),
							  I18.f("TOTAL_PRICE_MLD", I18.f("PORTAL_CURRENCY")),
							  I18.f("DATE"),
							  I18.f("COMMENT")]);
}

FinanceReportTab.prototype.getDate = function(date)
{
	return date.pick;
}

FinanceReportTab.prototype.getFromDate = function()
{
	return this.getDate(this.fromDatePicker.get("select"));
}

FinanceReportTab.prototype.getToDate = function()
{
	return this.getDate(this.toDatePicker.get("select"));
}

FinanceReportTab.prototype.dataRefresher = function(_fromDate, _toDate, _page, _onResultCallback)
{
}

FinanceReportTab.prototype.fitLayout = function(container)
{
	var grid = COM(this.grid).list;
	
	grid.style.width = (container.width - 4) + "px";
	grid.style.height = (container.height - this.headerRow.offsetHeight) + "px";
}

FinanceReportTab.prototype.refresh = function()
{
    if(!this.account) return;
    
    this.dataRefresher(
            DateUtils.startOfDay(this.getFromDate()),
            DateUtils.endOfDay(this.getToDate()),
            0,
            this.update
    );
}

FinanceReportTab.parseDate = function(date)
{
	var chains = date.toString().split(" ");
	var dateChains = chains[0].split("-");
	var timeChains = chains[1].split(":");
	
	var d = new Date();
	
	d.setYear(dateChains[0]*1);
	d.setMonth(dateChains[1]*1-1);
	d.setDate(dateChains[2]*1);
	
	d.setHours(timeChains[0]*1);
	d.setMinutes(timeChains[1]*1);
	d.setSeconds(0);
	d.setMilliseconds(0);
	
	return d;
}

FinanceReportTab.prototype.update = function(newInfo)
{
    this.info = newInfo;
    
    //grid.dataProvider = info.transactions;
    this.balance.innerHTML = I18.f("BALANCE") + ": " + FPortalConfig.formatPearl(this.info.balance);
    
    var grid = COM(this.grid);
    grid.clearChildren();
    
    var row, data, od;
    
    for(var i=0; i<this.info.transactions.length; i++)
    {
    	data = this.info.transactions[i];
    	
    	row = COM.create("ScrollableListItem");
    	
    	row.data = [this.getContractorName(data.source),
    			    this.getContractorName(data.destination),
    			    FinanceTabParent.getOperationName(data),
    			    FinanceTabParent.getAmount(data),
    			    FinanceReportTab.parseDate(data.date).format("dd.mm.yy HH:MM"),
    			    data.comment ? data.comment : ""];
    	
    	grid.addItem(row);
    }
    
    //pager.setIndexAndCount(info.pageIndex, info.pageCount);
}

FinanceReportTab.prototype.pageChanged = function(index)
{
    info.pageIndex = index;
    refresh();
}

FinanceReportTab.prototype.dateChanged = function()
{
    //info.pageIndex = 0;
    this.refresh();
}

FinanceReportTab.prototype.dateLabelFunction = function(item, column)
{
    if (this.detailed)
    {
        return DateUtils.formatForPortalkaMs(item.date.getTime());
    }
    else
    {
        return superMethod(FinanceReportTab.prototype, "dateLabelFunction", this)(item, column);
    }
}