function ZoneInfo()
{
	this.__defineGetter__('key', function()
	{
		if (!ZoneInfo.zones) return null;
		var entries = ZoneInfo.zones.entries;
		for (var i = 0; i < entries.length; i++)
		{
			if (entries[i][1].equals(this)) return entries[i][0];
		}
	});
	
	this.__defineGetter__('language', function()
	{
		var key = this.key;
		return key ? key.split(/[^a-zA-Z]/).shift() : null;
	});
	
	this.__defineGetter__('country', function()
	{
		var key = this.key;
		return key ? key.split(/[^a-zA-Z]/).pop() : '_unknown';
	});

	this.flagImage = '';
	this.flagToolTip = '';
	this.flagUrl = '';
}

ZoneInfo.zones = null;

ZoneInfo.prototype.__CLS__ = "game.config.ZoneInfo";

ZoneInfo.prototype.flagImage = '';
ZoneInfo.prototype.flagToolTip = '';
ZoneInfo.prototype.flagUrl = '';

ZoneInfo.find = function(key)
{
	if (!ZoneInfo.zones || !ZoneInfo.zones.entries.length || !key) return new ZoneInfo();
	if (ZoneInfo.zones.size == 1) return ZoneInfo.zones.entries[0][1];
	
	if (typeof key == 'object')
	{
		var entries = ZoneInfo.zones.entries;
		for (var i = 0; i < entries.length; i++)
		{
			if (entries[i][1].equals(key)) return entries[i][1];
		}
		return new ZoneInfo();
	}
	
	key = key.split(/[^a-zA-Z]/);
	
	var lang = (key[0] || '').toLowerCase(), 
		country = key.length > 1 ? key[1] : false,
		entries = ZoneInfo.zones.entries
	;

	if (lang) for (var i = 0; i < entries.length; i++)
	{
		var e = entries[i][1], k = entries[i][0];
		if (country)
		{
			if (k.toLowerCase() == key.join('_').toLowerCase()) return e;
		}
		else
		{
			if (lang == e.language.toLowerCase() || lang == e.country.toLowerCase())
			{
				return e;
			}
		}
	}
	
	var langMap = {
		'ru' : ['uk', 'be', 'kz', 'az'] // TODO: get full list... move to config
	}
	for (var prop in langMap)
	{
		if (langMap[prop].indexOf(lang) > -1)
		{
			return ZoneInfo.find(prop);
		}
	}

	// default case
	var defaultZone, defaultLang = window['DEFAULT_LANGUAGE'] || 'en';
	if (lang != defaultLang) 
	{
		console.warn('Zone "' + lang + '" not found. Attempting to connect default zone: "' + defaultLang + '"');
		defaultZone = ZoneInfo.find(defaultLang);
	}
	if (!defaultZone || (defaultZone.language != defaultLang)) 
	{
		defaultZone = ZoneInfo.zones.entries[0][1];
		console.warn('Default zone "' + defaultLang + '" not found. Attempting to connect first zone in list: "' + (defaultZone ? defaultZone.language : 'unknown') + '"');
	}
	return defaultZone;
}

ZoneInfo.getCountries = function()
{
	var ret = [];
	if (ZoneInfo.zones)
	{
		var zones = ZoneInfo.zones.values;
		for (var i = 0; i < zones.length; i++) ret.push(zones[i].country);
	}
	return ret;
}

ZoneInfo.prototype.equals = function(z)
{
	return z && (this.flagImage == z.flagImage) && (this.flagToolTip == z.flagToolTip) && (this.flagUrl == z.flagUrl);
}

