function RegisterSelectRace()
{
	this.__defineGetter__('race', this.getRace);
	this.__defineSetter__('race', this.setRace);
	
	this.__defineGetter__('server', this.getServer);
	this.__defineSetter__('server', this.setServer);
}

RegisterSelectRace.prototype.races = [Race.HUMAN, Race.DEMON, Race.ELF, Race.DROW];

RegisterSelectRace.prototype.__construct = function()
{
	if (!Session.get('gameServer'))
	{
		// cleanup before registering
		PortalService.logout($.proxy(this.init, this));
	}
	else
	{
		// init race selection only
		this.init();
	}
}

RegisterSelectRace.prototype.__destruct = function()
{
	APP.registration.terms = false;
}

RegisterSelectRace.prototype.createScroll = function()
{
	if(this.scroll)
	{
		this.scroll.destroy();
		this.scroll = null;
	}
	
	this.scroll = new iScroll(this.raceSelector.parentNode, {
		hScroll: true, vScroll: false, hScrollbar: false, vScrollbar: false,
		snap: 'race', momentum: false,
		onScrollEnd: $.proxy(function(){
			this.race = this.races[this.scroll.currPageX];
		}, this),
	});
}

RegisterSelectRace.prototype.init = function()
{
	var handleEnd = false;
	
	this.createScroll();
	
	PortalService.signupInfo($.proxy(this.onSignupInfo, this));
	
	$(this.serverSelector).on('change', $.proxy(this.serverChanged, this));
	
	COM(this.ok).reason = APP.registration.terms ? I18.f('SHOULD_ACCEPT_LICENCE') : null;
	
	var refreshTerms = $.proxy(function()
	{
		COM(this.ok).reason = COM(this.agree).checked ? null : I18.f('SHOULD_ACCEPT_LICENCE');
		APP.registration.terms = COM(this.agree).checked;
	}, this)

	COM(this.agree).checked = APP.registration.terms;
	refreshTerms();
	$(this.agree).on('change', refreshTerms);
	
	$(this.terms).on(MOUSE_DOWN_EVENT, APP.showTerms);
	
	$(this.ok).on(CLICK_EVENT, $.proxy(function(){
		
		if (!COM(this.serverSelector).disabled)
		{
			APP.registration.race = this.race;
			APP.registration.server = this.server;
			return APP.route('RegisterCreateAccount');
		}
		
		var token = APP.PORTAL_AUTH_TOKEN;
		PortalService.account.callMethod("createAvatar", [COM(this.serverSelector).value.id, token, this.race], 
			$.proxy(function(response){
				
				if (typeof response != "string") return console.error('ERROR', response);
				
				var url = response.split("?")[0];
				Session.set('gameServer', null);
				Session.set('gameServerURL', url);

				GAHelper.sendEvent("portal", "enterGame", COM(this.serverSelector).value.id);

				APP.navigate("game.html");
				
			}, this),
			$.proxy(function(response){
				
				console.error('ERROR', response);
				//Session.set('gameServer', null);
				//APP.view.setContent('ServerSelect');
				
			}, this)
		);
		
	
	}, this));
	
	$(this.info).on(CLICK_EVENT, $.proxy(this.showServerTip, this));
}

RegisterSelectRace.prototype.onSignupInfo = function(data)
{
	APP.hideLoader();
	APP.registration.signupInfo = data;
	
	var dp = [];
	var images = {
	    "MILITARY": STATIC_INTERFACE_URL + "common/register/server_military_free.png",
	    "MINER": STATIC_INTERFACE_URL + "common/register/server_miner_free.png",
	    "SURVIVAL": STATIC_INTERFACE_URL + "common/register/server_survival.png"
	}
	
	var srv = Session.get('gameServer');
	if (srv)
	{
		var srvData = {
			"about": srv.config.aboutServer.values[0],
			"canRegister": {},
			"id": srv.id,
			"name": srv.displayName,
			"race": srv.race.name,
			"type": srv.type.name,
			"url": "",
		};
		
		for (var i = 0; i < srv.config.canRegister.keys.length; i++)
		{
			var r = srv.config.canRegister.keys[i].name;
			srvData.canRegister[r] = srv.config.canRegister.values[i];
		}
		
		COM(this.serverSelector).dataProvider = [{
			"label": srvData.name,
			"value": srvData,
			"class": srvData.type
		}];
		COM(this.serverSelector).selectedIndex = 0;
		this.lockServer();
	}
	else
	{
		for (var i = 0; i < data.servers.length; i++)
		{
			var s = data.servers[i];
			dp.push({
				"label": s.name, 
				"value": s,
				"class": s.type
			});
		}
		COM(this.serverSelector).dataProvider = dp;
		COM(this.serverSelector).selectedIndex = 0;
	}
	
	var items = COM(COM(this.serverSelector).list).items;
	for (var i = 0; i < items.length; i++)
	{
		$(items[i]).addClass(data.servers[i].type);
	}
	
	this.serverChanged(); 
}

RegisterSelectRace.prototype.refreshScroll = function(timeout)
{
	if (timeout) return setTimeout($.proxy(this.refreshScroll, this), timeout*1);
	if (this.scroll) this.scroll.refresh();
}

RegisterSelectRace.prototype.getRace = function()
{
	return this.races[this.scroll.currPageX];
}

RegisterSelectRace.prototype.getRaceIndex = function(val)
{
	if (typeof val == 'object') val = val.name;
	for (var i = 0; i < this.races.length; i++) if (this.races[i].name == val) return i;
	return -1;
}

RegisterSelectRace.prototype.setRace = function(val)
{
	if (typeof val == 'object') val = val.name;
	
	APP.registration.race = Race[val];

	var idx = this.getRaceIndex(val);
	if (this.scroll.currPageX != idx) 
	{
		this.scroll.scrollToElement(this.raceSelector.children[idx], 0);
		this.scroll.currPageX = idx;
	}

	$('dot', this.dots).removeClass('active');
	$(this.dots.children[idx]).addClass('active');
	
	//this.raceName.innerHTML = I18.f('RACE_' + val + '_DESC');

	this.raceName.innerHTML = I18.f('RACE_' + val + '_NAME');
	this.raceDescription.innerHTML = I18.f('RACE_' + val + '_DESC');
}

RegisterSelectRace.prototype.getServer = function()
{
	return COM(this.serverSelector).value;
}

RegisterSelectRace.prototype.setServer = function(val)
{
	if (typeof val == 'object') val = val.id;
	var dp = COM(this.serverSelector).dataProvider;
	
	var old = COM(this.serverSelector).selectedIndex*1, idx = 0;
	
	for (var i = 0; i < dp.length; i++)
	{
		if (dp[i].id == val)
		{
			idx = i;
			break;
		}
	}
	
	if (idx != old)
	{
		COM(this.serverSelector).selectedIndex = idx;
		this.serverChanged();
	}
}

RegisterSelectRace.prototype.lockServer = function(flag)
{
	if (typeof flag == 'undefined') flag = true;
	
	COM(this.serverSelector).disabled = !!flag;
	COM(this.agree).checked = !!flag;
	//COM(this.agree).disabled = !!flag;
	UI.show(!!flag, [this.termsBox]);
	COM(this.ok).reason = null;
}

RegisterSelectRace.prototype.setRaces = function(races, race)
{
	this.races = races || this.races;
	this.raceSelector.style.visibility = 'hidden';
	var len = this.races.length, cnt = this.raceSelector.children.length;
	while (cnt > len) 
	{
		this.raceSelector.removeChild(this.raceSelector.children[0]);
		this.dots.removeChild(this.dots.children[0]);
		cnt--;
	}
	while (cnt < len) 
	{
		this.raceSelector.appendChild(document.createElement('race'));
		this.dots.appendChild(document.createElement('dot'));
		cnt++;
	}
	for (var i = 0; i < len; i++)
	{
		this.raceSelector.children[i].setAttribute('race', this.races[i].name);
	}
	
	this.createScroll();
	this.refreshScroll(300);
	
	setTimeout($.proxy(function() { 
		this.raceSelector.style.visibility = '';
		this.race = race; 
	}, this), 400);
}

RegisterSelectRace.prototype.serverChanged = function()
{
	var race = this.race, server = COM(this.serverSelector).value;
	APP.registration.server = server;
	
	var races = [];
	for (var r in server.canRegister)
	{
		if (server.canRegister[r])
		{
			races.push(Race[r]);
		}
	}
	
	var race = Race[server.race];
	if (!race || (race.name == 'MONSTER')) race = races[0];
	this.setRaces(races, race);
}

RegisterSelectRace.prototype.showServerTip = function()
{
	var server = COM(this.serverSelector).value;
	if (!server) return false;
	TipDialog.create(server.about, server.name);
}

