var COM = function(param)
{
	return COM.get(param);
}

COM.NOT_FOUND = []; // COLLECT UNDEFINED COMPONENTS HERE...
COM.INSTANCES = {}; // ID-INSTANCE
COM.DATA = {}; // CLASS-DATA

COM.bindProperties = function(instance, recursive, cls, root)
{
	var __bindingDebug = (instance.DOM.nodeName == 'BUILDQUEUEVIEW' && false);
	
	if (!root) root = instance.DOM;
	var len = root.childNodes.length;
	for (var i = 0; i < len; i++)
	{
		var node = root.childNodes.item(i);
		if (node.nodeType != 1) continue;
		
		if (__bindingDebug) console.log(i, node.getAttribute('name'), node, root);
		if (node.hasAttribute('name') && !node.hasAttribute('skip-binding') && (!cls || !node.hasAttribute('bind') || (node.getAttribute('bind') == cls)))
		{
			var prop = node.getAttribute('name');
			if (instance[prop] && (instance[prop] instanceof HTMLElement))
			{
				instance[prop] = [instance[prop]];
			}
			if ($.isArray(instance[prop]))
			{
				instance[prop].push(node);
			}
			else
			{
				instance[prop] = node;
			}
		}
		
		if (!recursive)
		{
			if (node.nodeName == 'COM' || node.getAttribute('role') == 'component') continue;
		}
		COM.bindProperties(instance, recursive, cls, node);
	}
	
	instance.specialSound = instance['specialSound'] || instance.DOM.getAttribute('sound') || null;
	instance.DOM.addEventListener(CLICK_EVENT, $.proxy(function(){
		if (!APP['specialSound'] && this.specialSound) APP.specialSound = this.specialSound;
	}, instance));
}

COM.setData = function(cls, key, data)
{
	if (key === null)
	{
		delete(COM.DATA[cls]);
		return;
	}
	
	if (typeof COM.DATA[cls] == 'undefined') 
	{
		COM.DATA[cls] = {};
	}
	
	if (typeof key == 'object')
	{
		$.extend(COM.DATA[cls], key)
	}
	else
	{
		COM.DATA[cls][key] = data;
	}
}

COM.getData = function(cls, key)
{
	var data = COM.DATA[cls] || null;
	if (typeof key != 'undefined' && data)
	{
		data = data[key] || null;
	}
	return data;
}

COM.loadQueue = function(queue, basePath, d)
{
	var rnd = function(){ return ('ontouchstart' in window) ? '' : '?rnd' + Math.round(Math.random()*10000) + '=' + Math.random() + '_' + (new Date().getTime()); }
	
	if (!d) 
	{
		d = $.Deferred();
	}
	if (queue.length == 0)
	{
		return d.resolve();
	}
	
	var cls = queue.shift();
	
	var qualifiedName = (cls.replace('.', '/')).split('/');
	cls = qualifiedName.pop();
	
	var data = COM.getData(cls);
	if (data)
	{
		return COM.loadQueue(queue, basePath, d);
	}
	
	basePath = basePath || COM.SETTINGS.basePath;
	
	var clsPackage = (qualifiedName.length ? qualifiedName.join('/') + '/' : '');
	var path = (clsPackage.substring(0, 1) == '/' ? '' : basePath + '/') + clsPackage;
	
	var url = path + cls + '.html' + rnd();
	
	var req = $.get(url, null, null, 'text');
	req.done(function(data, status)
	{
		// parse static assignments in text data 
		data = COM.parseStaticData(data);
		
		// create HTML fragment
		data = $(data);
		
		// binding properties
		binding = 
		{
			recursive: false,
			strict: false
		};
		if (data[0].hasAttribute('binding'))
		{
			if (data.attr('binding') == 'no')
			{
				binding = false;
			}
			else
			{
				var bindParams = data.attr('binding').split(' ');
				binding = {
					recursive: ($.inArray('recursive', bindParams) >= 0),
					strict: (($.inArray('strict', bindParams) >= 0) ? cls : false)
				};
			}
		}
		COM.setData(cls, 'binding', binding);
		
		if (data.attr('template'))
		{
			COM.setData(cls, 'templateClass', data.attr('template'));
			data.attr('template', null);
		}
		
		// collect includes
		function includesReady()
		{
			var scripts = data.find('script').toArray();
			while (scripts.length)
			{
				var src = scripts.shift(), script = document.createElement('script');
				script.innerHTML = src.innerHTML;
				document.head.appendChild(script);
				document.head.removeChild(script);
				
				var fn = (typeof window[cls] == 'function')
					? $.extend(window[cls], COM.prototype)
					: function() { console.warn('COM constructor not found "%s"', cls); }
				;
				COM.addComponentBehavior(fn, cls);
				COM.setData(cls, 'fn', fn);
			}
			data.find('script').remove();
			
			var fn = (typeof window[cls] == 'function')
				? $.extend(window[cls], COM.prototype)
				: function() { console.warn('COM constructor not found "%s"', cls); }
			;
			COM.addComponentBehavior(fn, cls);
			COM.setData(cls, 'fn', fn);
			
			// template
			COM.setData(cls, 'template', data.html());
			
			d.notify(cls);
			return COM.loadQueue(queue, basePath, d);
		}
		
		var includes = data.find('include *').toArray();
		data.find('include').remove();
		
		if (includes.length == 0)
		{
			var src = data.attr('src');
			data.attr('src', null);
			
			var script = document.createElement('script');
			script.type = 'text/javascript';
			script.src = path + (src || cls+'.js') + rnd();
		}
		
		var head = document.getElementsByTagName('head')[0];

		var incCounter = includes.length;
		if (incCounter > 0)
		{ 
			while (includes.length)
			{
				var inc = includes.shift();
				var script = document.createElement(inc.nodeName);
				switch (inc.nodeName.toUpperCase())
				{
					case 'LINK':
						var src = inc.getAttribute('href');
						if (src.substr(0, 2) == './')
						{
							src = path + src.substr(2);
						}
						src += rnd();
						inc.setAttribute('href', src);
						break;
						
					case 'SCRIPT':
						
						var src = inc.getAttribute('src');
						if (src.substr(0, 2) == './')
						{
							src = path + src.substr(2);
						}
						src += rnd();
						inc.setAttribute('src', src);
						break;
						
					default:
						break;
				}
				
				var attrs = $(inc).attr();
				for (var attr in attrs)
				{
					script.setAttribute(attr, (attr == 'src') ? src : attrs[attr]);
				}
				
				script.onload = function()
				{
					head.removeChild(this);
					if (--incCounter == 0) includesReady();
				}
				script.onerror = function()
				{
					incCounter--;
					if (this.hasAttribute('required'))
					{
						throw new Error('Required file not loaded for "' + cls + '"');
						return; 
					}
					console.warn('Included file not loaded for "%s"', cls);
					head.removeChild(this);
					if (--incCounter == 0) includesReady();
				}
				head.appendChild(script);
			}
		}
		else
		{
			return includesReady();
		}
		return d;
	})
	req.fail(function(err, status)
	{
		COM.setData(cls, null);
		console.error('Error loading component "%s" (%s)', cls, path + cls);
	});
	
	return d;
}

COM.load = function(cls, basePath)
{
	if (!$.isArray(cls))
	{
		cls = [cls]
	}
	return COM.loadQueue(cls, basePath);
}

COM.defaultMethods = 
{
	'__init': function(){},
	'__construct': function(){},
	'__destruct': function(){},
	'__getTemplate': function()
	{
		var comCls = this.TYPE + '';
		var tplCls = this.__templateClass + '';
		while ((comCls != tplCls) && tplCls)
		{
			comCls = COM.getData(comCls, 'templateClass') || comCls;
			tplCls = COM.getData(comCls, 'templateClass') || tplCls;
		}
		
		return COM.getData(tplCls, 'template');
	},
	'destroy': function()
	{
		if (typeof this['__destruct'] == 'function')
		{
			this.__destruct();
		}

		COM.destroy(this.DOM);
		this.DOM = null;
	}
}

COM.destroy = function(node)
{
	if(!node || !node.children)
	{
		return;
	}
	
	var cm, child;
	while(node.children.length)
	{
		child = node.children[0];
		cm = COM(child);
		if(cm) cm.destroy();
		else COM.destroy(child);
	}
	
	cm = COM(node);
	if(cm) delete(COM.INSTANCES[cm.ID]);
	$(node).remove();
}

COM.clear = function(node)
{
	if(!node || !node.children)
	{
		return;
	}
	
	while(node.children.length)
	{
		COM.destroy(node.children[0]);
	}
}

COM.addComponentBehavior = function(fn, cls)
{
	//fn.prototype.ID = null;
	//fn.prototype.DOM = null;
	fn.prototype.TYPE = cls;
	
	for (var method in COM.defaultMethods)
	{
		if (typeof fn.prototype[method] == 'function' && fn.prototype[method].toLocaleString() != COM.defaultMethods[method].toLocaleString()) 
		{
			continue;
		}
		fn.prototype[method] = COM.defaultMethods[method];
	}
}

COM.createDOMElement = function(cls, id)
{
	var el = document.createElement(cls);
	
	el.id = id;
	el.className = 'COM-' + cls;
	el.setAttribute('role', 'component');
	
	return el;
}

COM.parseDOMElement = function(el)
{
	// DOMElement only
	if (el.nodeType != 1) return;
	
	// element is unparsed component
	if (el.nodeName == 'COM')
	{
		var cls = el.getAttribute('type');
		el.removeAttribute('type'); // remove it to avoid double parsing
		return COM.create(cls, el);
	}
	
	// else parse all children
	var parentClass = (el.getAttribute('role') == 'component') ? el.nodeName : false;
	var len = el.childNodes.length;
	for (var i = 0; i < len; i++)
	{
		childClass = el.childNodes[i].nodeName == 'COM' ? el.childNodes[i].getAttribute('type') : false;
		if (parentClass && childClass && (parentClass.toUpperCase() == childClass.toUpperCase()))
		{
			console.error('Component self recursion detected in ' + childClass, el);
			throw new Error('Component self recursion detected in ' + childClass);
		}
		COM.parseDOMElement(el.childNodes[i]);
	}
}

COM.create = function(cls, obj, params)
{
	var name = null;
	cls = cls.split('#');
	if (cls.length > 1)
	{
		name = cls.pop();
	}
	cls = cls.shift();
	
	var data = COM.getData(cls);
	// no data - no component
	if (!data)
	{
		//console.warn('COMPONENT NOT FOUND: %s', cls);
		if ($.inArray(cls, COM.NOT_FOUND) < 0)
		{
			COM.NOT_FOUND.push(cls);
		}
		data = {fn:null, template:'', binding:{resursive:false, strict:false}};
		COM.setData(cls, data);
	}
	
	if (typeof data.fn != 'function')
	{
		window[cls] = function(){};
		COM.addComponentBehavior(window[cls], cls);
		data.fn = window[cls];
	}
	
	var com = new data.fn(params); 
	com.ID = UID();
	
	if(COM.INSTANCES[com.ID])
	{
		console.log(COM.INSTANCES[com.ID], com, com.ID);
		throw Error("COM IDS error");
	}
	
	com.DOM = COM.createDOMElement(cls, com.ID);
	
	// initial attributes
	var attributes = {}, cssClasses = [];
	if (typeof obj == 'object')
	{
		if (obj instanceof HTMLElement)
		{
			attributes = $(obj).attr();
			cssClasses = obj.className.split(' ');
		}
		else
		{
			attributes = $.extend({}, obj);
			if (typeof attributes['class'] != 'undefined')
			{
				cssClasses = attributes['class'].split(' ');
				delete(attributes['class']);
			}
			obj = null;
		}
	}
	
	attributes['name'] = name || (obj && obj.getAttribute('name')) || com.ID;
	for (var attr in attributes)
	{
		com.DOM.setAttribute(attr, attributes[attr]);
	}
	while (cssClasses.length > 0)
	{
		$(com.DOM).addClass(cssClasses.shift());
	}
	
	com.__templateClass = COM.getData(cls, 'templateClass') || cls;
	
	com.__inlineData = (obj ? obj.innerHTML : '');
	
	com.DOM.innerHTML = com.DOM.hasAttribute('inline') 
		? com.__inlineData 
		: com.__getTemplate()
	;
	
	COM.parseDOMElement(com.DOM);
	
	if (data.binding)
	{
		COM.bindProperties(com, data.binding.recursive, data.binding.strict);
	}
	
	com.__init();
	if (obj)
	{
		$(obj).replaceWith(com.DOM);
	}
	
	if (typeof com['__construct'] == 'function')
	{
		params = Array.prototype.slice.call(arguments, 2);
		com.__construct.apply(com, params);
	}
	
	COM.INSTANCES[com.ID] = com;
	
	if(window.onActiveInputBlur) $("input, textarea", com.DOM).on("blur", onActiveInputBlur);
	if(ANDROID_MODE && window.alignScroll) $("input, textarea", com.DOM).on("focus", alignScroll);
	
	return com;
}

COM.parseStaticData = function(html)
{
	if (typeof html == 'object' && html instanceof HTMLElement)
	{
		html.innerHTML = COM.parseStaticData(html.innerHTML);
		return; 
	}
	
	var parseCallback = function()
	{
		var type = (arguments[1] || ''), 
			arg = (arguments[2] || ''), 
			mod = (arguments[3] || '')
		;
		if (!type)
		{
			return typeof window[arg] == 'undefined' ? '['+arg+']' : window[arg];
		}
		
		if (type == 'I18')
		{
			return I18.f(arg) || '{' + arg + '}';
		}
		
		if (type == 'FConfig')
		{
			return FConfig[arg];
		}
		
		if (type.substr(-6) == 'Assets')
		{
			mod = mod.toLowerCase();
			switch (mod)
			{
				case 'interface': 
					mod = STATIC_INTERFACE_URL; break;
				default:
					mod = STATIC_URL; break;
			}
			return mod + window[type][arg];
		}
		
		return arguments[0];
	}
	
	// strip scripts to prevent parsing JS
	// XXX: fail if script contains string "</script>"
	var r, scripts = [];
	r = new RegExp('(<script[^]+</script>)', 'gmi');
	while (r.test(html))
	{
		html = html.replace(r, function(script){
			scripts.push(script);
			return '[SCRIPT' + scripts.length + ']';
		});
	}
	
	r = new RegExp('\\{(' + [
		
		'I18',
		'FConfig',
		'\\w*Assets',
		'INTERFACE_IMAGES_PACK',
		'GAME_IMAGES_PACK',
		'STATIC_URL_BASE',
		'STATIC_URL',
		'STATIC_INTERFACE_URL_BASE',
		'STATIC_INTERFACE_URL',
		
	].join('|') + ')?\\.?(\\w+):?(\\w+)?\\}', 'g');
	
	while (r.test(html))
	{
		html = html.replace(r, parseCallback);
	}
	
	// return scripts
	while (scripts.length)
	{
		html = html.replace('[SCRIPT' + scripts.length + ']', scripts.pop());
	}

	return html;
}

COM.get = function(uid)
{
	if (!uid) return null;
	if (typeof uid['ID'] != 'undefined')
	{
		uid = uid.ID;
	}
	if (!COM.INSTANCES[uid])
	{
		if (typeof uid == 'object')
		{
			uid = $(uid).attr('id');
		}
		else
		{
			uid = $('*[role=component][name='+uid+']').not('*[role=component] *').attr('id');
		}
	}
	return uid ? COM.INSTANCES[uid] : null;
}

COM.nodeZoom = function(node, zoom)
{
	if (typeof zoom == 'undefined') zoom = {x:1, y:1};
	if (!node || !node.style) return zoom;

	var m = new WebKitCSSMatrix(window.getComputedStyle(node).webkitTransform);
	zoom.x *= m.a; 
	zoom.y *= m.d;
	return COM.nodeZoom(node.parentNode, zoom);
}

COM.AABB = function(uid)
{
	var com = COM.get(uid);
	if (!com) return null;
	 
	var $com = $(com.DOM), 
		offset = $com.offset(),
		zoom = COM.nodeZoom(com.DOM)
	;
	return {
		left: offset.left, 
		top: offset.top, 
		width: $com.outerWidth() * zoom.x, 
		height: $com.outerHeight() * zoom.y
	};
}

COM.hitTestPoint = function(uid, x, y, cachedAABB)
{
	var aabb = cachedAABB || COM.AABB(uid);
	return aabb ? (
		x >= aabb.left && x <= aabb.left + aabb.width && 
		y >= aabb.top && y <= aabb.top + aabb.height
	) : false;
}

COM.atPoint = function(x, y)
{
    var $list = $('body *').filter(function(){
        return COM.hitTestPoint(this, x, y);
    });
    return $list.map(function(){
        return COM(this);
    }).get();
}

COM.lastExclusiveClick = 0;
COM.bindExclusiveClick = function(el, fn)
{
	var fn2 = function(e)
	{
		var t1 = e.originalEvent ? (e.originalEvent['touches'] || []) : [];
		var t2 = e.originalEvent ? (e.originalEvent['changedTouches'] || []) : [];
		var len = t1.length + t2.length;
		var now = (new Date()).getTime();
		if (len > 1 || ((now - COM.lastExclusiveClick) < 300) || isKeyboardVisible())
		{
			e.preventDefault();
			e.stopImmediatePropagation();
			return false;
		}
		COM.lastExclusiveClick = now;
        
		return fn.apply(this, arguments);;
	}
	var node = el ? (el['DOM'] || el) : undefined;
	if (typeof fn != 'function') return $(node);
	return $(node).on(CLICK_EVENT, fn2);
}

COM.SETTINGS = 
{
	basePath: ''
}

COM.init = function(params, preload, callback)
{
	if (params) 
	{
		$.extend(COM.SETTINGS, params);
	}
	if (typeof callback != 'function')
	{
		callback = function(){};
	}
	if (preload)
	{
		COM.load(preload).done(callback);
	}
	else
	{
		callback();
	}
}

COM.GC = function()
{
	var trash = [];
	for (var id in COM.INSTANCES)
	{
		if (!COM.INSTANCES[id].DOM && !document.getElementById(id))
		{
			trash.push(id);
		}
	}
	
	if (trash.length) 
	{
		console.debug("COM.GC %s lost components garbaged", trash.length);
	}
	
	while (trash.length)
	{
		var id = trash.pop();
		COM.INSTANCES[id].DOM = null;
		COM.INSTANCES[id] = null;
		delete(COM.INSTANCES[id]);
	}
	
};

setTimeout(COM.GC, 5000);
//------------------------------------------------------------------------------------------------------------------------------
