LONGTAP_DELAY = 500; // delay to wait
LONGTAP_THRESHOLD = 6; // px to move before break

function LongTap(target, delay, threshold)
{
	this.target = target;
	this.listeners = [];
	
	this.delay = delay*1 || LONGTAP_DELAY;
	this.threshold = threshold*1 || LONGTAP_THRESHOLD;
	this.timer = null;
	
	this.start = null;
	
	LongTap.instances.push(this);
	
	this.startHandler = $.proxy(this.startHandler, this);
	this.target.addEventListener(MOUSE_DOWN_EVENT, this.startHandler, false);
}

LongTap.instances = [];
LongTap.findTarget = function(target)
{
	for (var i = 0, len = LongTap.instances.length; i < len; i++)
	{
		if (LongTap.instances[i].target.isSameNode(target))
		{
			return i;
		}
	}
	return -1;
}

LongTap.prototype.busy = false;
LongTap.prototype.startHandler = function(startEvent)
{
	if (this.busy) return;
	if (typeof startEvent['button'] != 'undefined' && startEvent['button'] != 0) return;

	this.busy = true;
	this.start = startEvent['touches'] ? startEvent.touches[0] : startEvent;
	this.start = {x: this.start.pageX*1, y: this.start.pageY*1};
	
	var runDelay = $.proxy(function()
	{
		this.timer = setTimeout(dispatchLongTap, this.delay);
		document.addEventListener(MOUSE_MOVE_EVENT, checkMoveBreak, false);
		document.addEventListener(MOUSE_UP_EVENT, breakLongTap, true);
	}, this);
	
	var breakDelay = $.proxy(function()
	{
		clearTimeout(this.timer);
		this.timer = null;
	}, this);
	
	var checkMoveBreak = $.proxy(function(moveEvent)
	{
		if (!this.timer) 
		{
			return breakLongTap();
		}
		
		var data = moveEvent['touches'] ? moveEvent.touches[0] : moveEvent,
			dx = Math.abs(data.pageX - this.start.x),
			dy = Math.abs(data.pageY - this.start.y)
		;
		
		if (this.threshold < dx || this.threshold < dy)
		{
			return breakLongTap();
		}
	}, this);
	
	var breakLongTap = $.proxy(function()
	{
		breakDelay();
		document.removeEventListener(MOUSE_MOVE_EVENT, checkMoveBreak, false);
		document.removeEventListener(MOUSE_UP_EVENT, breakLongTap, true);
		this.busy = false;
		
	}, this);
	
	var dispatchLongTap = $.proxy(function()
	{
		breakLongTap();
		
		var evt = document.createEvent("MouseEvents");
		evt.initEvent("longtap", true, true);
		this.start = null;
		this.target.dispatchEvent(evt);
	}, this);
	
	runDelay();
}

LongTap.destroyHandler = function(e)
{
	LongTap.remove(e.target);
}

LongTap.prototype.findListener = function(listener)
{
	for (var i = 0, len = this.listeners.length; i < len; i++)
	{
		if (this.listeners[i] === listener)
		{
			return i;
		}
	}
	return -1;
}

LongTap.prototype.addListener = function(listener)
{
	if (listener && (this.findListener(listener) < 0))
	{
		this.listeners.push(listener);
		this.target.addEventListener('longtap', listener, false);
	}
	return this.listeners.length;
}

LongTap.prototype.removeListener = function(listener)
{
	if (listener)
	{
		this.target.removeEventListener('longtap', listener, false);
		var idx = this.findListener(listener);
		if (idx >= 0)
		{
			this.listeners.splice(idx, 1);
		}
	}
	else
	{
		while (this.listeners.length)
		{
			listener = this.listeners.shift();
			this.target.removeEventListener('longtap', listener, false);
		}
	}
	return this.listeners.length;
}

LongTap.add = function(target, listener)
{
	var idx = LongTap.findTarget(target), 
		instance = idx < 0 ? new LongTap(target) : LongTap.instances[idx];
	;
	instance.addListener(listener);
}

LongTap.remove = function(target, listener)
{
	var idx = LongTap.findTarget(target);
	if (idx < 0) return false;

	var instance = LongTap.instances[idx];
	if (instance.removeListener(listener) == 0)
	{
		instance.target.removeEventListener(MOUSE_DOWN_EVENT, instance.startHandler, false);
		LongTap.instances.splice(idx, 1);
		instance = null;
	}
}
