cordova.define("org.apache.cordova.inappbrowser.InAppBrowser", function(require, exports, module) {/*
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
*/

var exec = require('cordova/exec');
var channel = require('cordova/channel');
var modulemapper = require('cordova/modulemapper');

function InAppBrowser() {
	this.channels = {
			'loadstart': channel.create('loadstart'),
			'loadstop' : channel.create('loadstop'),
			'loaderror' : channel.create('loaderror'),
			'exit' : channel.create('exit')
	};
}

InAppBrowser.prototype = {
    
	_eventHandler: function (event) {
		if (event.type in this.channels) 
		{
			this.channels[event.type].fire(event);
		}
	},
	
	_globalEventHandler: function (event) {
        if (typeof IABGlobal != 'object') return;
    	var handlers = {
    		'loadstart': 'onLoadStart',
    		'loadstop' : 'onLoadStop',
    		'loaderror' : 'onLoadError',
    		'exit' : 'onExit'
    	};
		var fn = event.type in handlers ? IABGlobal[handlers[event.type]] : null;
		if (typeof fn == 'function') 
        {
        	fn(event);
        }
    },
    
    close: function (eventname) {
        exec(null, null, "InAppBrowser", "close", []);
    },
    clearCookies: function (eventname) {
        exec(null, null, "InAppBrowser", "clearCookies", []);
    },
    show: function (eventname) {
      exec(null, null, "InAppBrowser", "show", []);
    },
    addEventListener: function (eventname,f) {
        if (eventname in this.channels) {
            this.channels[eventname].subscribe(f);
        }
    },
    removeEventListener: function(eventname, f) {
        if (eventname in this.channels) {
            this.channels[eventname].unsubscribe(f);
        }
    },

    executeScript: function(injectDetails, cb) {
        if (injectDetails.code) {
            exec(cb, null, "InAppBrowser", "injectScriptCode", [injectDetails.code, !!cb]);
        } else if (injectDetails.file) {
            exec(cb, null, "InAppBrowser", "injectScriptFile", [injectDetails.file, !!cb]);
        } else {
            throw new Error('executeScript requires exactly one of code or file to be specified');
        }
    },

    insertCSS: function(injectDetails, cb) {
        if (injectDetails.code) {
            exec(cb, null, "InAppBrowser", "injectStyleCode", [injectDetails.code, !!cb]);
        } else if (injectDetails.file) {
            exec(cb, null, "InAppBrowser", "injectStyleFile", [injectDetails.file, !!cb]);
        } else {
            throw new Error('insertCSS requires exactly one of code or file to be specified');
        }
    }
};

module.exports = function(strUrl, strWindowName, strWindowFeatures, clearCookies)
{
    strWindowFeatures = strWindowFeatures || '';
    
	var iab = new InAppBrowser();
    if (clearCookies) iab.clearCookies();
    
    var cb = function(eventname)
    {
    	eventname.target = iab;
    	iab._eventHandler(eventname);
    	iab._globalEventHandler(eventname);
    };

    // Don't catch calls that write to existing frames (e.g. named iframes).
    if (window.frames && window.frames[strWindowName]) {
        var origOpenFunc = modulemapper.getOriginalSymbol(window, 'open');
        return origOpenFunc.apply(window, arguments);
    }

    // apply default window features defined in InAppBrowser.global
    if ((typeof IABGlobal == 'object') && (typeof IABGlobal['windowFeatures'] == 'object'))
    {
	    var features = {};
	    strWindowFeatures = strWindowFeatures ? strWindowFeatures.split(',') : [];
	    while (strWindowFeatures.length > 0)
	    {
	    	var f = strWindowFeatures.shift(), idx = f.indexOf('='), v = f.substring(idx + 1);
	    	f = f.substring(0, idx),
	    	features[f] = v;
	    }
	    for (var f in features)
	    {
	    	strWindowFeatures.push(f + '=' + features[f]);
	    }
	    for (var f in IABGlobal.windowFeatures)
	    {
	    	if (!(f in features)) strWindowFeatures.push(f + '=' + IABGlobal.windowFeatures[f]);
	    }
	    strWindowFeatures = strWindowFeatures.join(',');
    }
    
    exec(cb, cb, "InAppBrowser", "open", [strUrl, strWindowName, strWindowFeatures]);
    return iab;
};

module.exports.clearCookies = function()
{
	var iab = new InAppBrowser();
	iab.clearCookies();
}

});
