var TOUCH_SUPPORT = ('ontouchstart' in window);
var MOUSE_DOWN_EVENT = TOUCH_SUPPORT ? 'touchstart' : 'mousedown';
var MOUSE_UP_EVENT = TOUCH_SUPPORT ? 'touchend' : 'mouseup';
var MOUSE_MOVE_EVENT = TOUCH_SUPPORT ? 'touchmove' : 'mousemove';
var CLICK_EVENT = TOUCH_SUPPORT ? 'touchend' : 'mouseup';

var INTERFACE_IMAGES_PACK = 3;
var GAME_IMAGES_PACK = 1;

var STATIC_URL_BASE = "static/images/game/";
var STATIC_URL = STATIC_URL_BASE + GAME_IMAGES_PACK + '/';
var STATIC_INTERFACE_URL_BASE = "static/images/interface/";
var STATIC_INTERFACE_URL = STATIC_INTERFACE_URL_BASE + INTERFACE_IMAGES_PACK + '/';

var PHONE_MODE = false;
var ANDROID_MODE = false;

if((navigator.userAgent.match(/iPhone/i)) || (navigator.userAgent.match(/iPod/i))) PHONE_MODE = true;
if(navigator.userAgent.match(/Android/i)) ANDROID_MODE = true;

var HEXES_SCALE = 1;
var KINGDOM_SCALE = 1;

function onDeviceReady()
{
    GAHelper.setTrackingId(GA_TRACKING_ID);
    APP.onDeviceReady();
}

if (window.cordova) 
{
	document.addEventListener('deviceready', onDeviceReady, false);
}
else {
    $(onDeviceReady);
}

function preventScroll(force)
{
	if(window['APP'] && APP.ticksIsPaused) return;
	
	if(!force && isInputActive())
	{
		if(document.body.scrollLeft != 0)
		{
			window.scrollTo(0, document.body.scrollTop);
		}
		return;
	}
	
	if(document.body.scrollLeft == 0 && document.body.scrollTop == 1)
	{
		return;
	}
	
	window.scrollTo(0, 1);
}

function onActiveInputBlur()
{
    setTimeout(function()
    {
    	if(isInputActive())
        {
            return;
        }
    	
    	if(ANDROID_MODE)
    	{
    		document.body.style.height = "auto";
    		document.body.style.bottom = "0px";
    	}
    	
    	preventScroll();
    }, 10);
}

function alignScroll()
{
	if(!ANDROID_MODE) return;
    
    setTimeout(alignScrollLeft, 10);
}

function alignScrollLeft()
{
	if(!ANDROID_MODE) return;
    
    var y = document.body.scrollTop;
	if("ontouchstart" in window && isInputActive())
	{
		y = getObjectY(document.activeElement);
		y -= window.innerHeight/6;
	}
	
	document.body.style.height = "2048px";
	document.body.style.bottom = "auto";
	
	setTimeout(function()
	{
		window.scrollTo(0, y);
	}, 500);
}

function isInputActive()
{
	if (!document.activeElement) return false;
	if (['INPUT','TEXTAREA'].indexOf(document.activeElement.nodeName.toUpperCase()) < 0) 
	{
		return false;
	} 
	
	if (document.activeElement.nodeName.toUpperCase() != 'TEXTAREA')
	{
		if (['text','password','number','email', 'range'].indexOf((document.activeElement.getAttribute('type')).toLowerCase()) < 0) 
		{
			return false;
		}
	}
	return true;
}

function getTopDialog()
{
	if (!window['COM'] || !window['Dialog'] || isKeyboardVisible()) return null;
	if (document.querySelector('#keyboard') && document.querySelector('#keyboard').children.length > 0) return null;
	
	var container = document.querySelector('#popup'), dialog = null;
	if (container && container.children.length)
	{
		var dialog = COM(container.children[container.children.length-1]);
	}
	return dialog || null;
}

function closeTopDialog(ignoreCanClose)
{
	var dialog = getTopDialog();
	if (dialog && (ignoreCanClose || dialog.userCanClose))
	{
		dialog.close();
		return true;
	}
	return false;
}

function isKeyboardVisible()
{
	return (window['Keyboard'] && Keyboard.isVisible);
}

var BOOTSTRAP_FUNC = function(deviceInfo)
 {
    if(ANDROID_MODE && window.StatusBar) {
        console.log("HIDE STATUS BAR");
        StatusBar.hide();
    }

    var scale = 1/(window.devicePixelRatio ? window.devicePixelRatio : 1);

    var meta = document.createElement('meta');
    meta.name = "viewport";
    var width = Math.max(window.innerWidth, window.innerHeight);

    var dpi = "device-dpi";
    var additional = "";

    if(ANDROID_MODE)
    {
        var w = Math.floor(Math.max(screen.width, screen.height)/scale);
        var h = Math.floor(Math.min(screen.width, screen.height)/scale);

        //additional = ", height=device-height";

        if(w > 1024)
        {
            if(window.devicePixelRatio >= 2)
            {
                HEXES_SCALE = 1.65;
                KINGDOM_SCALE = 1.3;

                PHONE_MODE = true;
                scale = 1;

                //alert(window.device.screen.dpi);
                dpi = "320dpi";
            }
            else
            {
                var s = w/1024;

                HEXES_SCALE = s;
                KINGDOM_SCALE = 1;
            }
        }

        if(navigator.userAgent.match(/Chrome/i))
        {
            if(document.getElementById("popup"))
            {
                document.getElementById("popup").style.webkitTransform = "translate3d(0px, 0px, 0px)";
            }

            dpi = (deviceInfo && deviceInfo.screen ? deviceInfo.screen.dpi : 320);
            scale = 1 / (window.devicePixelRatio > 2 ? 2 : window.devicePixelRatio);
            //scale = 320 / dpi;

            dpi += "dpi";
        }
        else scale = 1;
        //alert([w, h, scale, dpi].join(' '));

    }
    else
    {
        if(window.innerWidth/scale > 1024 && !PHONE_MODE) scale = 1;
        if(PHONE_MODE)
        {
            HEXES_SCALE = 1.65;
            KINGDOM_SCALE = 1.3;
        }
    }

    meta.content = "width=device-width, target-densitydpi="+dpi+", user-scalable=no, initial-scale="+scale+", maximum-scale="+scale+", minimum-scale="+scale+additional;

    //alert(meta.content);

    document.getElementsByTagName('head')[0].appendChild(meta);

    preventScroll();

     document.body.addEventListener("touchstart", function(e)
    {
        var tagCurrent = e.currentTarget.tagName.toLowerCase();
         var tagTarget = e.target.tagName.toLowerCase();

        if (isInputActive())
         {
             if(!(tagTarget == "input" || tagTarget == "textarea"))
             {
                 document.activeElement.blur();
             }
         }
        else if (ANDROID_MODE)
        {
            if(!(tagTarget == "input" || tagTarget == "textarea"))
            {
                e.preventDefault();
                return false;
            }
        }
    });

    document.body.addEventListener("touchmove", function(e)
    {
        if (e.target.nodeName.toLowerCase() == "input") return true;

        e.preventDefault();
        return false;
    }, false);

    document.getElementById("popup").addEventListener("touchmove", function(e)
     {
        if(e.target.nodeName.toLowerCase() == "input") return true;

        e.preventDefault();
        return false;
    }, false);

     /*
    if("ontouchstart" in window) document.body.style.height = "2048px";
    else document.body.style.overflow = "hidden";
     */

    //document.body.style.overflow = "hidden";

    document.addEventListener('keydown', function(e)
    {
        if (isInputActive()) return;

        if (e.keyCode == 8) // BACKSPACE
        {
            e.preventDefault();
            return false;
        }

        else if (e.keyCode == 27) // ESC
        {
            closeTopDialog();
            e.preventDefault();
            return false;
        }

        //else console.log(e.keyCode);
    });

    // Cordova specific events
    document.addEventListener("backbutton", function(){

        if (isKeyboardVisible()) return false;

        var dialog = getTopDialog();
        if (dialog)
        {
            if (dialog.userCanClose) dialog.close();
        }
        else if (window['APP'] && APP['back'])
        {
            APP.back();
        }
        return false;
    }, false);

    document.addEventListener("pause", function(){
        if (window['APP'] && APP['stopBackgroundMusic']) APP.stopBackgroundMusic();
    }, false);

    document.addEventListener("resume", function(){
        if (window['APP'] && APP['refreshBackgroundMusic']) APP.refreshBackgroundMusic();
    }, false);

    setTimeout("document.body.style.visibility = '';", 500);
 };

if(window.cordova) {
    console.log("BOOTSTRAP CORDOVA");
    document.addEventListener('deviceready', function() {
        console.log("ANDROID MODE " + ANDROID_MODE);
        console.log("PLUGIN " + window.device);

        if(ANDROID_MODE && window.device) {
            console.log("GET DEVICE INFO");
            window.device.getInfo(BOOTSTRAP_FUNC);
        }
        else BOOTSTRAP_FUNC();
    }, false);
}
else {
    console.log("BOOTSTRAP BROWSER");
    $(BOOTSTRAP_FUNC);
}

function onKeyboardShow()
{
	//alignScrollLeft();
}

function onKeyboardHide()
{
	//preventScroll(true);
	if(document.activeElement) document.activeElement.blur();
}

var MAT_REF_ID = null;
function onMATResponse(data)
{
    if(data && data.site_event_type)
    {
        MAT_REF_ID = data.advertiser_ref_id;
    }
}

//-----------------------------------------------------------------------------
// CONSOLE OVERRIDES
//-----------------------------------------------------------------------------
var emptyFunc = function(){};
if (!console) console = {};
if (!console.assert) console.assert = emptyFunc;
if (!console.clear) console.clear = emptyFunc;
if (!console.count) console.count = emptyFunc;
if (!console.debug) console.debug = emptyFunc;
if (!console.dir) console.dir = emptyFunc;
if (!console.dirxml) console.dirxml = emptyFunc;
if (!console.error) console.error = emptyFunc;
if (!console.group) console.group = emptyFunc;
if (!console.groupCollapsed) console.groupCollapsed = emptyFunc;
if (!console.groupEnd) console.groupEnd = emptyFunc;
if (!console.info) console.info = emptyFunc;
if (!console.log) console.log = emptyFunc;
if (!console.profile) console.profile = emptyFunc;
if (!console.profileEnd) console.profileEnd = emptyFunc;
if (!console.time) console.time = emptyFunc;
if (!console.timeEnd) console.timeEnd = emptyFunc;
if (!console.timeStamp) console.timeStamp = emptyFunc;
if (!console.trace) console.trace = emptyFunc;
if (!console.warn) console.warn = emptyFunc;
